//
// Rep++ Framework for Windows Application
// Copyright (C) 2005 Consyst SQL Inc.
// All Rights Reserved
//
// This source code is only intended as a supplement to the
// Rep++ Studio products.
//
using System;
using System.Collections;
using System.Windows.Forms;
using RepPP;
using RepPP.Toolkit;

namespace RepPP.Framework.Window {
    /// <summary>
    /// The ISelectionBuffer is the interface called to refresh the selection
    /// buffer user interface.
    /// </summary>
    public interface ISelectionBuffer {
        /// <summary>
        /// Add an item to the selection buffer
        /// </summary>
        /// <param name="rowset">       Rowset of the selection buffer</param>
        /// <param name="bDuplicate">   true if duplicate the item</param>
        void        AddItemToSelectionBuffer(Rowset rowset, bool bDuplicate);
        /// <summary>
        /// Refresh the item in the selection buffer
        /// </summary>
        /// <param name="rowset">Rowset where to take the updated value</param>
        void        RefreshItemInSelectionBuffer(Rowset rowset);
        /// <summary>
        /// Delete the item from the selection buffer
        /// </summary>
        /// <param name="bReselect">true to reselect a new item after deletion</param>
        void        DeleteItemFromSelectionBuffer(bool bReselect);
        /// <summary>
        /// Get the number of items if the current kind
        /// </summary>
        /// <returns>Number of items</returns>
        int         ItemCount { get; }
        /// <summary>
        /// Get the current item index for the current kind
        /// </summary>
        /// <returns>Item index (0..n)</returns>
        int         CurrentItemIndex { get; }
        /// <summary>
        /// Select an item in the selection buffer
        /// </summary>
        /// <param name="iPos">Position of the item to select (0..n)</param>
        /// <returns>true if succeed, false if not</returns>
        bool        SelectItem(int iPos);
        /// <summary>
        /// Returns the selection buffer rowset associated with the current item
        /// </summary>
        /// <returns>Current rowset</returns>
        Rowset   CurrentRowset { get; }
        /// <summary>
        /// Gets the currently selected item primary key
        /// </summary>
        /// <returns>PK values</returns>
        string[]    CurrentItemPK { get; }
        /// <summary>
        /// Gets the currently selected item
        /// </summary>
        object      CurrentItemObject { get; }
        /// <summary>
        /// Gets the type of the currently selected item
        /// </summary>
        int         CurrentItemType { get; }
        /// <summary>
        /// Current filter
        /// </summary>
        ISBFilter   CurrentFilter { get; }
        /// <summary>
        /// Current search methods
        /// </summary>
        ISearchMethod[] SearchMethods { get; }
        /// <summary>
        /// Index of the search method to be executed
        /// </summary>
        int NextSearchMethodIndex { get; set;}
        /// <summary>
        /// Index of the search method that was used to load the selection buffer
        /// </summary>
        int ActiveSearchMethodIndex { get; set;}
        /// <summary>
        /// Reloads the search methods
        /// </summary>
        void ReloadSearchMethods();
        /// <summary>
        /// Fill the selection buffer
        /// </summary>
        /// <param name="bAddNewIfEmpty">Indicates if a new item should be added if the selection buffer will be empty </param>
        void Fill(bool bAddNewIfEmpty);
        /// <summary>
        /// Selection buffer rowset tree
        /// </summary>
        RowsetTree RowsetTree { get; }
    }
}
