//
// Rep++ Framework for Windows Application
// Copyright (C) 2005 Consyst SQL Inc.
// All Rights Reserved
//
// This source code is only intended as a supplement to the
// Rep++ Studio products.
//
using System;
using System.Collections;
using RepPP;
using RepPP.Toolkit.Window;
namespace RepPP.Framework.Window {
    /// <summary>Type of conversion</summary>
    public enum DataConversionTypeE {
        /// <summary>Encode to put in the database</summary>
        DC_Encode   = 1,
        /// <summary>Decode the value coming from database</summary>
        DC_Decode   = 2
    };

    /// <summary>Allow operation</summary>
    public enum sdAllow {
        /// <summary>Let the framework decide on the context</summary>
        sdAllowAuto = 0,
        /// <summary>Allow the operation</summary>
        sdAllowYes  = 1,
        /// <summary>Forbid the operation</summary>
        sdAllowNo   = 2
    };

    /// <summary>Commands supported by the cardgroup and card handler</summary>
    public enum sdHandlerCmd {
        /// <summary>Null operation</summary>
        DoNothing = 0,         
        /// <summary>Save the current item (line)</summary>
        ItemSave = 1,          
        /// <summary>Save all dirty items (lines)</summary>
        ItemSaveAll = 2,       
        /// <summary>Open a new set</summary>
        RowsetTreeSearch = 3,           
        /// <summary>Undo the last item operation</summary>
        ItemUndo = 4,              
        /// <summary>Cut</summary>
        FieldCut = 5,               
        /// <summary>Copy</summary>
        FieldCopy = 6,              
        /// <summary>Paste</summary>
        FieldPaste = 7,             
        /// <summary>Popup choice list</summary>
        FieldChoice = 8,            
        /// <summary>Display field help</summary>
        FieldHelp = 9,              
        /// <summary>Erase the current field</summary>
        FieldErase = 10,       
        /// <summary>Set the current field value to its default value</summary>
        FieldDefault = 11,     
        /// <summary>Erase all field values</summary>
        FieldEraseAll = 12,   
        /// <summary>Set the current field values to their default value</summary>
        FieldDefaultAll = 13, 
        /// <summary>Select the first line of the active card group handler</summary>
        LineFirst = 14,        
        /// <summary>Select the previous line of the active card group handler</summary>
        LinePrev = 15,     
        /// <summary>Select the next line of the active card group handler</summary>
        LineNext = 16,         
        /// <summary>Select the last line of the active card group handler</summary>
        LineLast = 17,         
        /// <summary>Create a new line in the active card group handler</summary>
        LineNew = 18,          
        /// <summary>Delete a line from the active card group handler</summary>
        LineDelete = 19,       
        /// <summary>Duplicate a line from the active card group handler</summary>
        LineDuplicate = 20,    
        /// <summary>Select the first item (line)</summary>
        ItemFirst = 100,           
        /// <summary>Select the previous item (line)</summary>
        ItemPrev = 101,        
        /// <summary>Select the next item (line)</summary>
        ItemNext = 102,            
        /// <summary>Select the last item (line)</summary>
        ItemLast = 103,            
        /// <summary>Add a new item (line)</summary>
        ItemNew = 104,             
        /// <summary>Deletes the current item (line)</summary>
        ItemDel = 105,
        /// <summary>Duplicates the current item (line)</summary>
        ItemDup = 106
    };

    /// <summary>Actions supported by the cardgroup and card handler</summary>
    public enum sdAction {
        /// <summary>Add a new item</summary>
        sdActionAdd     = 1,    
        /// <summary>Duplicate an item</summary>
        sdActionDup     = 2,    
        /// <summary>Update an item</summary>
        sdActionUpd     = 3,    
        /// <summary>Delete an item</summary>
        sdActionDel     = 4,    
        /// <summary>Select previous or first item</summary>
        sdActionFirst   = 5,    
        /// <summary>Select next or last item</summary>
        sdActionLast    = 6,    
        /// <summary>Save the item</summary>
        sdActionSave    = 7,    
        /// <summary>Save all items</summary>
        sdActionSaveAll = 8,    
        /// <summary>Undo the last operation</summary>
        sdActionUndo    = 9
    };

    /// <summary>
    /// SCCardGroupHandler:    Handle a card group in a hiearchy
    /// </summary>
	public class RowsetTreeDefNodeHandler : IRowsetTreeDefNodeHandler, IDisposable {

	    /// <summary>Event sent each time a new line is selected for this card group</summary>
	    public event LineSelectedEventHandler               LineSelected;
        /// <summary>Father line changed. Select rowset</summary>
        public event System.EventHandler                    FatherLineSelected;
	    /// <summary>Event sent before a line is validated</summary>
	    public event LineValidationEventHandler             BeforeLineValidated;
	    /// <summary>Event sent before a line is validated</summary>
	    public event LineValidationEventHandler             AfterLineValidated;
        /// <summary>State must be adjust</summary>
        public event System.EventHandler                    AdjustState;
        /// <summary>Called before a command is executed</summary>
        public event ExecuteCmdEventHandler                 BeforeExecuteCmd;
        /// <summary>Called after a command is executed</summary>
        public event ExecuteCmdEventHandler                 AfterExecuteCmd;
        /// <summary>An error Occurred</summary>
        public event ErrorOccurredEventHandler              ErrorOccurred;
        /// <summary>Called when data conversion must be done</summary>
        public event DataConversionEventHandler             DataConversion;
	    /// <summary>Father card group handler</summary>
	    private RowsetTreeDefNodeHandler                    m_nodeHandlerFather;
	    /// <summary>RowsetTreeDef group handle by this class</summary>
	    private RowsetTreeDefNode                           m_node;
	    /// <summary>true to allow insertion for this card group</summary>
	    private sdAllow                                     m_eAllowInsert;
	    /// <summary>true to allow deletion for this card group</summary>
	    private sdAllow                                     m_eAllowDelete;
	    /// <summary>true to allow update for this card group</summary>
	    private sdAllow                                     m_eAllowUpdate;
        /// <summary>Item state</summary>
        private LineState                                   m_eState;
	    /// <summary>true to see deleted line</summary>
	    private bool                                        m_bSeeDeletedLine;
	    /// <summary>Attached rowset</summary>
	    private Rowset                                      m_rowset;
	    /// <summary>Array of child node handler</summary>
	    private ArrayList                                   m_arrChildNodeHandler;
	    /// <summary>FieldDef selectable state</summary>
	    private RepPP.Toolkit.Util.FldSelectableState       m_fldSelectableState;
	    /// <summary>Object handling the list of lines</summary>
	    private ILineListHandler                            m_lineHandler;
	    /// <summary>Default object handling the list of lines</summary>
	    private ILineListHandler                            m_lineHandlerDefault;
        /// <summary>Attached command holder</summary>
        private CmdHolder                                   m_cmdHolder;

        //*********************************************************     
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="nodeHandlerFather">    Father node handler</param>
        /// <param name="node">                 Attached rowset tree definition node</param>
        /// <param name="bSeeDeletedLine">      true to see deleted line</param>
        //  
        //*********************************************************     
		public RowsetTreeDefNodeHandler(RowsetTreeDefNodeHandler nodeHandlerFather, RowsetTreeDefNode node, bool bSeeDeletedLine) {
		    m_nodeHandlerFather     = nodeHandlerFather;
		    m_node                  = node;
		    m_fldSelectableState    = new RepPP.Toolkit.Util.FldSelectableState(node.RowsetDef);
            m_eState                = LineState.sdLineOld;
            m_eAllowInsert          = (nodeHandlerFather == null) ? sdAllow.sdAllowYes : sdAllow.sdAllowAuto;
            m_eAllowUpdate          = (nodeHandlerFather == null) ? sdAllow.sdAllowYes : sdAllow.sdAllowAuto;
            m_eAllowDelete          = (nodeHandlerFather == null) ? sdAllow.sdAllowYes : sdAllow.sdAllowAuto;
		    m_bSeeDeletedLine       = bSeeDeletedLine;
		    m_arrChildNodeHandler   = new ArrayList();
	        m_lineHandler           = null;
	        m_lineHandlerDefault    = new RowsetLineHandler(this, bSeeDeletedLine);
            m_cmdHolder             = null;
		    if (nodeHandlerFather != null) {
		        nodeHandlerFather.m_arrChildNodeHandler.Add(this);
		    }
            node.Application.FldChange += new RepPP.FldChangeEventHandler(FldChgSink);
            m_lineHandlerDefault.LineSelectedInList += new LineSelectedInListEventHandler(OnLineListHandler_LineSelectedInList);
		}

        //*********************************************************     
        //
        /// <summary>
        /// Dispose:                        Dispose the class
        /// </summary>
        //  
        //*********************************************************     
		public virtual void Dispose() {
		    if (m_nodeHandlerFather != null && m_nodeHandlerFather.m_arrChildNodeHandler != null) {
		        m_nodeHandlerFather.m_arrChildNodeHandler.Remove(this);
		    }
		    if (m_node != null) {
                m_node.Application.FldChange -= new RepPP.FldChangeEventHandler(FldChgSink);
            }
            if (m_lineHandler != null) {
                m_lineHandler.LineSelectedInList -= new LineSelectedInListEventHandler(OnLineListHandler_LineSelectedInList);
            }
            m_lineHandlerDefault.LineSelectedInList -= new LineSelectedInListEventHandler(OnLineListHandler_LineSelectedInList);
		    m_nodeHandlerFather                      = null;
		    m_node                                   = null;
		    m_arrChildNodeHandler                    = null;
		    m_cmdHolder                              = null;
		    if (m_fldSelectableState != null) {
		        m_fldSelectableState.Dispose();
		    }
		}

        //*********************************************************     
        //
        /// <summary>
        /// Handler:                    Return a pointer on the node handler
        /// </summary>
        //  
        //*********************************************************     
        public virtual RowsetTreeDefNodeHandler Handler {
            get {
                return(this);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RowsetTreeDefNode:          Rowset Tree Definition node handle by this class
        /// </summary>
        //  
        //*********************************************************     
        public virtual RowsetTreeDefNode RowsetTreeDefNode {
            get {
                return(m_node);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// FatherRowsetTreeDefNodeHandler: Father node handler
        /// </summary>
        //  
        //*********************************************************     
        public virtual RowsetTreeDefNodeHandler FatherRowsetTreeDefNodeHandler {
            get {
                return(m_nodeHandlerFather);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RootRowsetTreeDefNodeHandler:           Root node handler
        /// </summary>
        //  
        //*********************************************************     
        public virtual RowsetTreeDefNodeHandlerRootBase RootRowsetTreeDefNodeHandler {
            get {
                RowsetTreeDefNodeHandler    nodeHandler;
                
                nodeHandler = this;
                while (nodeHandler != null && !(nodeHandler is RowsetTreeDefNodeHandlerRootBase)) {
                    nodeHandler = nodeHandler.FatherRowsetTreeDefNodeHandler;
                }
                return(nodeHandler as RowsetTreeDefNodeHandlerRootBase);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Rowset:          Attached rowset
        /// </summary>
        //  
        //*********************************************************     
        public virtual Rowset Rowset {
            get {
                Rowset   rowsetRetVal;
                
                rowsetRetVal = (m_rowset == null || m_rowset.IsDetached) ? null : m_rowset;
                return(rowsetRetVal);
            }
            set {
                if (m_rowset != value) {
                    m_rowset = value;
                    if (m_rowset != null) {
                        m_rowset.Select(false, false);
                    }
                    LineListHandler.AssociateRowset(value);
                    RowsetTreeDefNode.Refresh(false);
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// LineListHandler:                Handler for the lines
        /// </summary>
        //  
        //*********************************************************     
		public virtual ILineListHandler LineListHandler {
		    get {
		        return(m_lineHandler != null ? m_lineHandler : m_lineHandlerDefault);
		    }
		    set {
                if (m_lineHandler != null) {
                    m_lineHandler.LineSelectedInList -= new LineSelectedInListEventHandler(OnLineListHandler_LineSelectedInList);
                }
		        m_lineHandler = value;
                if (m_lineHandler != null) {
                    m_lineHandler.AssociateRowset(Rowset);
                    m_lineHandler.LineSelectedInList += new LineSelectedInListEventHandler(OnLineListHandler_LineSelectedInList);
                }
		    }
		}

        //*********************************************************     
        //
        /// <summary>
        /// CommandHolder:              Command holder
        /// </summary>
        //  
        //*********************************************************     
        public virtual CmdHolder CommandHolder {
            get {
                return(m_cmdHolder);
            }
            set {
                if (value != m_cmdHolder) {
                    if (m_cmdHolder != null) {
                        m_cmdHolder.RemoveCmd(sdHandlerCmd.ItemNext.ToString());
                        m_cmdHolder.RemoveCmd(sdHandlerCmd.ItemPrev.ToString());
                        m_cmdHolder.RemoveCmd(sdHandlerCmd.ItemFirst.ToString());
                        m_cmdHolder.RemoveCmd(sdHandlerCmd.ItemLast.ToString());
                        m_cmdHolder.RemoveCmd(sdHandlerCmd.ItemNew.ToString());
                        m_cmdHolder.RemoveCmd(sdHandlerCmd.ItemDup.ToString());
                        m_cmdHolder.RemoveCmd(sdHandlerCmd.ItemDel.ToString());
                    }
                    m_cmdHolder = value;
                    if (m_cmdHolder != null) {
                        m_cmdHolder.AddCmd(sdHandlerCmd.ItemNext.ToString(),        new CmdHolder.OnCommand(CmdItemNext), CmdHolder.CmdStateE.Disabled);
                        m_cmdHolder.AddCmd(sdHandlerCmd.ItemPrev.ToString(),        new CmdHolder.OnCommand(CmdItemPrev), CmdHolder.CmdStateE.Disabled);
                        m_cmdHolder.AddCmd(sdHandlerCmd.ItemFirst.ToString(),       new CmdHolder.OnCommand(CmdItemFirst), CmdHolder.CmdStateE.Disabled);
                        m_cmdHolder.AddCmd(sdHandlerCmd.ItemLast.ToString(),        new CmdHolder.OnCommand(CmdItemLast), CmdHolder.CmdStateE.Disabled);
                        m_cmdHolder.AddCmd(sdHandlerCmd.ItemNew.ToString(),         new CmdHolder.OnCommand(CmdItemNew), CmdHolder.CmdStateE.Disabled);
                        m_cmdHolder.AddCmd(sdHandlerCmd.ItemDup.ToString(),         new CmdHolder.OnCommand(CmdItemDup), CmdHolder.CmdStateE.Disabled);
                        m_cmdHolder.AddCmd(sdHandlerCmd.ItemDel.ToString(),         new CmdHolder.OnCommand(CmdItemDel), CmdHolder.CmdStateE.Disabled);
                    }
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdItemNext:            Command 'Next line'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdItemNext() {
            HandleCommand(sdHandlerCmd.ItemNext, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdItemPrev:            Command 'Previous line'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdItemPrev() {
            HandleCommand(sdHandlerCmd.ItemPrev, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdItemFirst:           Command 'First line'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdItemFirst() {
            HandleCommand(sdHandlerCmd.ItemFirst, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdItemLast:            Command 'Last line'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdItemLast() {
            HandleCommand(sdHandlerCmd.ItemLast, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdItemNew:             Command 'New line'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdItemNew() {
            HandleCommand(sdHandlerCmd.ItemNew, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdItemDup:             Command 'Duplicate line'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdItemDup() {
            HandleCommand(sdHandlerCmd.ItemDup, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdItemDel:             Command 'Delete line'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdItemDel() {
            HandleCommand(sdHandlerCmd.ItemDel, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// IsLineDeleted:      Returns true if the line or a father line is deleted
        /// </summary>
        //  
        //*********************************************************     
        public virtual bool IsLineDeleted {
            get {
                bool                        bRetVal;
                Rowset                      rowset;
                RowsetTreeDefNodeHandler    nodeHandlerFather;
                
                rowset = Rowset;
                if (rowset == null) {
                    bRetVal = false;
                } else {
                    if (rowset.LineState == LineState.sdLineDeleted) {
                        bRetVal = true;
                    } else {
                        nodeHandlerFather = FatherRowsetTreeDefNodeHandler;
                        bRetVal = (nodeHandlerFather == null) ? false : nodeHandlerFather.IsLineDeleted;
                    }
                }
                return(bRetVal);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// AllowInsert:            Insertion allowed if true
        /// </summary>
        //  
        //*********************************************************     
		public virtual sdAllow AllowInsert {
		    get {
		        return(m_eAllowInsert);
		    }
		    set {
		        if (m_eAllowInsert != value) {
		            m_eAllowInsert = value;
                    RaiseAdjustState();
                }
		    }
		}

        //*********************************************************     
        //
        /// <summary>
        /// AllowUpdate:            Update allowed if true
        /// </summary>
        //  
        //*********************************************************     
        public virtual sdAllow AllowUpdate {
            get {
                return(m_eAllowUpdate);
            }
            set {
                if (m_eAllowUpdate != value) {
                    m_eAllowUpdate = value;
                    RaiseAdjustState();
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// AllowDelete:        Delete allowed or not for this card group
        /// </summary>
        //  
        //*********************************************************     
        public virtual sdAllow AllowDelete {
            get {
                return(m_eAllowDelete);
            }
            set {
                if (m_eAllowDelete != value) {
                    m_eAllowDelete = value;
                    RaiseAdjustState();
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// EffectiveAllowInsert:       Return true if a new line can be inserted
        /// </summary>
        //  
        //*********************************************************     
        public virtual bool EffectiveAllowInsert {
            get {
                bool                        bRetVal;
                RowsetTreeDefNodeHandler    nodeHandlerFather;
                
                switch(AllowInsert) {
                case sdAllow.sdAllowYes:
                    bRetVal = true;
                    break;
                case sdAllow.sdAllowNo:
                    bRetVal = false;
                    break;
                default:
                    nodeHandlerFather = FatherRowsetTreeDefNodeHandler;
                    if (nodeHandlerFather == null) {
                        bRetVal = true;
                    } else {
                        if (nodeHandlerFather.EffectiveAllowUpdate) {
                            bRetVal = true;
                        } else {
                            bRetVal = (nodeHandlerFather.SBLineCount != 0 && 
                                       nodeHandlerFather.GetItemState() == RepPP.LineState.sdLineNew) ? true : false;
                        }
                    }
                    break;
                }                
                return(bRetVal);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// EffectiveAllowUpdate:       Update allowed if true
        /// </summary>
        //  
        //*********************************************************     
        public virtual bool EffectiveAllowUpdate {
            get {
                bool                        bRetVal;
                RowsetTreeDefNodeHandler    nodeHandlerFather;
                
                switch(AllowUpdate) {
                case sdAllow.sdAllowYes:
                    bRetVal = true;
                    break;
                case sdAllow.sdAllowNo:
                    bRetVal = false;
                    break;
                default:
                    nodeHandlerFather = FatherRowsetTreeDefNodeHandler;
                    if (nodeHandlerFather == null) {
                        bRetVal = true;
                    } else {
                        if (nodeHandlerFather.EffectiveAllowUpdate) {
                            bRetVal = true;
                        } else {
                            bRetVal = (nodeHandlerFather.SBLineCount != 0 && 
                                       nodeHandlerFather.GetItemState() == RepPP.LineState.sdLineNew) ? true : false;
                        }
                    }
                    break;
                }                
                return(bRetVal);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// EffectiveAllowDelete:       Delete allowed if true
        /// </summary>
        //  
        //*********************************************************     
        public virtual bool EffectiveAllowDelete {
            get {
                bool                        bRetVal;
                RowsetTreeDefNodeHandler    nodeHandlerFather;
                
                switch(AllowDelete) {
                case sdAllow.sdAllowYes:
                    bRetVal = true;
                    break;
                case sdAllow.sdAllowNo:
                    bRetVal = false;
                    break;
                default:
                    nodeHandlerFather = FatherRowsetTreeDefNodeHandler;
                    if (nodeHandlerFather == null) {
                        bRetVal = true;
                    } else {
                        if (nodeHandlerFather.EffectiveAllowUpdate) {
                            bRetVal = true;
                        } else {
                            bRetVal = (nodeHandlerFather.SBLineCount != 0 && 
                                       nodeHandlerFather.GetItemState() == LineState.sdLineNew) ? true : false;
                            if (SBLineCount != 0 && GetItemState() == LineState.sdLineNew) {
                                bRetVal = true;
                            }
                        }
                    }
                    break;
                }                
                return(bRetVal);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// FieldProtection:    Actual protection level of the fields
        /// </summary>
        //  
        //*********************************************************     
        public virtual RepPP.Toolkit.Util.FldSelectableState.SelStateE FieldProtection {
            get {
                return(m_fldSelectableState.State);
            }
            set {
                m_fldSelectableState.State = value;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// UpdateOriginalSelectableState:    Updates the original selectable state
        /// </summary>
        //  
        //*********************************************************     
        public virtual void UpdateOriginalSelectableState(params FieldDef[] flds) {
            m_fldSelectableState.UpdateOriginalSelectableState(flds);
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetFieldProtection:     Set the field protection according to the context
        /// </summary>
        //  
        //*********************************************************     
        public virtual void SetFieldProtection() {
            RepPP.LineState eState;
            
            if (SBLineCount == 0) {
                FieldProtection = RepPP.Toolkit.Util.FldSelectableState.SelStateE.SS_None;
            } else {
                eState = GetItemState();
                switch(eState) {
                case RepPP.LineState.sdLineNew:
                    FieldProtection = RepPP.Toolkit.Util.FldSelectableState.SelStateE.SS_All;
                    break;
                case RepPP.LineState.sdLineDeleted:
                    FieldProtection = RepPP.Toolkit.Util.FldSelectableState.SelStateE.SS_None;
                    break;
                default:
                    FieldProtection = (EffectiveAllowUpdate) ? RepPP.Toolkit.Util.FldSelectableState.SelStateE.SS_NoPK : RepPP.Toolkit.Util.FldSelectableState.SelStateE.SS_None;
                    break;
                }
            }
        }    

        //*********************************************************     
        //
        /// <summary>
        /// FindRowsetTreeDefNodeHandler:    Find a node handler using the attached rowset definition
        /// </summary>
        /// 
        /// <param name="rowsetDef">         Rowset definition</param>
        /// <returns> 
        /// Rowset Tree Definition node handler or null if not found.
        /// </returns>
        //  
        //*********************************************************     
        public virtual RowsetTreeDefNodeHandler FindRowsetTreeDefNodeHandler(RowsetDef rowsetDef) {
            RowsetTreeDefNodeHandler    nodeHandlerRetVal;
            
            nodeHandlerRetVal = (rowsetDef == RowsetTreeDefNode.RowsetDef) ? this : null;
            foreach(RowsetTreeDefNodeHandler nodeHandlerChild in m_arrChildNodeHandler) {
                if (nodeHandlerRetVal == null) {
                    nodeHandlerRetVal = nodeHandlerChild.FindRowsetTreeDefNodeHandler(rowsetDef);
                }
            }
            return(nodeHandlerRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// FindRowsetTreeDefNodeHandler:   Find a handler using the attached rowset tree definition node
        /// </summary>
        /// 
        /// <param name="node">             Rowset tree definition node</param>
        /// <returns> 
        /// Rowset tree definition node handler or null if not found.
        /// </returns>
        //  
        //*********************************************************     
        public virtual RowsetTreeDefNodeHandler FindRowsetTreeDefNodeHandler(RowsetTreeDefNode node) {
            RowsetTreeDefNodeHandler    nodeHandlerRetVal;
            
            nodeHandlerRetVal = (node == RowsetTreeDefNode) ? this : null;
            foreach(RowsetTreeDefNodeHandler nodeHandlerChild in m_arrChildNodeHandler) {
                if (nodeHandlerRetVal == null) {
                    nodeHandlerRetVal = nodeHandlerChild.FindRowsetTreeDefNodeHandler(node);
                }
            }
            return(nodeHandlerRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CurrentLine:            Gets the current line position
        /// </summary>
        //  
        //*********************************************************     
        public virtual int CurrentLine {
            get {
                Rowset   rowset;
                
                rowset = Rowset;
                return((rowset == null) ? -1 : rowset.CurrentLine);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// LineCount:              Gets the current line count
        /// </summary>
        //
        //*********************************************************     
        public virtual int LineCount {
            get {
                Rowset   rowset;
                
                rowset = Rowset;
                return((rowset == null) ? 0 : rowset.LineCount);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SBCurrentLine:          Gets the current line in the selection buffer
        /// </summary>
        //  
        //*********************************************************     
        public virtual int SBCurrentLine {
            get {
                int             iRetVal;
                Rowset   rowset;
                
                rowset = Rowset;
                if (rowset == null) {
                    iRetVal = -1;
                } else if (rowset.Type == RowsetType.sdVertical) {
                    iRetVal = 0;
                } else if (m_bSeeDeletedLine) {
                    iRetVal = rowset.CurrentLine;
                } else {
                    iRetVal = rowset.CurrentUndelLine;
                }
                return(iRetVal);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SBLineCount:            Gets the current line count in the selection buffer
        /// </summary>
        //
        //*********************************************************     
        public virtual int SBLineCount {
            get {
                int             iRetVal;
                Rowset   rowset;
                
                rowset = Rowset;
                if (rowset == null) {
                    iRetVal = 0;
                } else if (rowset.Type == RowsetType.sdVertical) {
                    iRetVal = 1;
                } else if (m_bSeeDeletedLine) {
                    iRetVal = rowset.LineCount;
                } else {
                    iRetVal = rowset.UndelLineCount;
                }
                return(iRetVal);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// TriggerResetFieldError: Reset the last error message from field validation
        /// </summary>
        //  
        //*********************************************************     
        public virtual void TriggerResetFieldError() {
            RowsetTreeDefNodeHandler    nodeHandlerFather;
            
            nodeHandlerFather = FatherRowsetTreeDefNodeHandler;
            if (nodeHandlerFather != null) {
                nodeHandlerFather.TriggerResetFieldError();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnLineSelected:     Reset the last error message from field validation
        /// </summary>
        //  
        //*********************************************************     
		public virtual void OnLineSelected(LineSelectedEventArgs e) {
		    if (LineSelected != null) {
		        LineSelected(this, e);
		    }
		    RaiseAdjustState();
		    TriggerResetFieldError();
		    foreach (RowsetTreeDefNodeHandler nodeHandlerChild in m_arrChildNodeHandler) {
		        nodeHandlerChild.RaiseFatherLineSelected();
		    }
		}

        //*********************************************************     
        //
        /// <summary>
        /// RaiseFatherLineSelected:Trigger the FatherLineSelected event for this control
        /// </summary>
        //  
        //*********************************************************     
        private void RaiseFatherLineSelected() {
            RowsetTreeDefNodeHandler    nodeHandlerFather;
            Rowset                      rowsetFather;
            Rowset                      rowset;
            
            nodeHandlerFather = FatherRowsetTreeDefNodeHandler;
            if (nodeHandlerFather != null) {
                rowsetFather   = nodeHandlerFather.Rowset;
                rowset         = (rowsetFather != null && rowsetFather.ChildRowsets.Count > 0) ? rowsetFather.ChildRowsets[RowsetTreeDefNode.ChildId] : null;
                Rowset         = rowset;
                if (rowsetFather != null && rowsetFather.LineCount != 0) {
                    OnFatherLineSelected(EventArgs.Empty);
                }
                OnAdjustState(EventArgs.Empty);
                foreach (RowsetTreeDefNodeHandler nodeHandlerChild in m_arrChildNodeHandler) {
                    nodeHandlerChild.RaiseFatherLineSelected();
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnFatherLineSelected:   Trigger the FatherLineSelected event for this control
        /// </summary>
        /// <param name="e">        Event arguments</param>
        //  
        //*********************************************************     
        public virtual void OnFatherLineSelected(System.EventArgs e) {
            if (FatherLineSelected != null) {
                FatherLineSelected(this, e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnBeforeLineValidated:  Trigger the BeforeLineValidated event for this control
        /// </summary>
        /// <param name="e">        Line validated event arguments</param>
        //  
        //*********************************************************     
        public virtual void OnBeforeLineValidated(LineValidationEventArgs e) {
            if (BeforeLineValidated != null) {
                BeforeLineValidated(this, e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RaiseBeforeLineValidated:   Trigger the BeforeLineValidated event for this control
        ///                             and all child
        /// </summary>
        /// <param name="e">            Line validated event arguments</param>
        //  
        //*********************************************************     
        private void RaiseBeforeLineValidated(LineValidationEventArgs e) {
            Rowset   rowset;
            
            rowset = Rowset;
            if (rowset != null && rowset.UndelLineCount != 0 && rowset.LineState != LineState.sdLineDeleted) {
                foreach (RepPP.FieldDef fldDef in m_node.RowsetDef.FieldDefs) {
                    if (!fldDef.IsValid) {
                        OnDataConversion(new DataConversionEventArgs(this, DataConversionTypeE.DC_Encode));
                        break;
                    }
                }
                OnBeforeLineValidated(e);
                foreach (RowsetTreeDefNodeHandler crdHandlerChild in m_arrChildNodeHandler) {
                    crdHandlerChild.RaiseBeforeLineValidated(e);
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnAfterLineValidated:   Trigger the AfterLineValidated event for this control
        ///                         and all child.
        /// </summary>
        /// <param name="e">        Line validated event argument</param>
        //  
        //*********************************************************     
        public virtual void OnAfterLineValidated(LineValidationEventArgs e) {
            if (AfterLineValidated != null) {
                AfterLineValidated(this, e);
            }
            foreach (RowsetTreeDefNodeHandler nodeChild in m_arrChildNodeHandler) {
                nodeChild.OnAfterLineValidated(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetCommandHolderState:  Set the state of the command holder
        /// </summary>
        //  
        //*********************************************************     
        public virtual void SetCommandHolderState() {
            CmdHolder.CmdStateE     eFirst;
            CmdHolder.CmdStateE     eLast;
            CmdHolder.CmdStateE     eNew;
            CmdHolder.CmdStateE     eDup;
            CmdHolder.CmdStateE     eDel;
            CmdHolder               cmdHolder;
            
            cmdHolder = CommandHolder;
            if (cmdHolder != null) {
                eFirst = GetActionState(sdAction.sdActionFirst) ? 
                         CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled;
                eLast  = GetActionState(sdAction.sdActionLast) ? 
                         CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled;
                eNew   = GetActionState(sdAction.sdActionAdd) ? 
                         CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled;
                eDup   = GetActionState(sdAction.sdActionDup) ? 
                         CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled;
                eDel   = GetActionState(sdAction.sdActionDel) ? 
                         CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled;
                cmdHolder.SetCmdState(sdHandlerCmd.ItemFirst.ToString(), eFirst);
                cmdHolder.SetCmdState(sdHandlerCmd.ItemPrev.ToString(), eFirst);
                cmdHolder.SetCmdState(sdHandlerCmd.ItemLast.ToString(), eLast);
                cmdHolder.SetCmdState(sdHandlerCmd.ItemNext.ToString(), eLast);
                cmdHolder.SetCmdState(sdHandlerCmd.ItemNew.ToString(), eNew);
                cmdHolder.SetCmdState(sdHandlerCmd.ItemDup.ToString(), eDup);
                cmdHolder.SetCmdState(sdHandlerCmd.ItemDel.ToString(), eDel);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnAdjustState:      Raise the AdjustState event
        /// </summary>
        //  
        //*********************************************************     
        public virtual void OnAdjustState(EventArgs e) {
            SetFieldProtection();
            SetCommandHolderState();
            if (AdjustState != null) {
                AdjustState(this, e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnBeforeExecuteCmd:     Informs that a command is about to be executed
        /// </summary>
        /// <param name="e">        Line validated event</param>
        //  
        //*********************************************************     
        public virtual void OnBeforeExecuteCmd(ExecuteCmdEventArgs e) {
            if (BeforeExecuteCmd != null) {
                BeforeExecuteCmd(this, e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnAfterExecuteCmd:      Informs that a command has been executed
        /// </summary>
        /// <param name="e">        Line validated event</param>
        //  
        //*********************************************************     
        public virtual void OnAfterExecuteCmd(ExecuteCmdEventArgs e) {
            if (AfterExecuteCmd != null) {
                AfterExecuteCmd(this, e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnErrorOccurred:    Trigger the ErrorOccurred event for this control
        ///                     and all child
        /// </summary>
        /// <param name="e">    Error Occurred event arguments</param>
        //  
        //*********************************************************     
        public virtual void OnErrorOccurred(ErrorOccurredEventArgs e) {
            if (ErrorOccurred != null) {
                ErrorOccurred(this, e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// TriggerErrors:      Trigger the ErrorOccurred event 
        /// </summary>
        //  
        //*********************************************************     
        public virtual void TriggerErrors() {
            int         iIndex;
            Application app;

            app = RowsetTreeDefNode.Application;
            for (iIndex = 0; iIndex < app.Errors.Count; iIndex ++) {
                OnErrorOccurred(new ErrorOccurredEventArgs(app.Errors[iIndex]));
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnDataConversion:   Trigger the DataConversion event for this control
        /// </summary>
        /// <param name="e">    Error Occurred event arguments</param>
        //  
        //*********************************************************     
        public virtual void OnDataConversion(DataConversionEventArgs e) {
            if (DataConversion != null) {
                DataConversion(this, e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RaiseAdjustState:       Raise the adjust state for this control and father.
        /// </summary>
        //  
        //*********************************************************     
        public virtual void RaiseAdjustState() {
            RowsetTreeDefNodeHandler    nodeHandlerFather;
            
            OnAdjustState(EventArgs.Empty);
            nodeHandlerFather = FatherRowsetTreeDefNodeHandler;
            if (nodeHandlerFather != null) {
                nodeHandlerFather.RaiseAdjustState();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// CommitEdit:             Commit the edit before validating
        /// </summary>
        //  
        //*********************************************************     
        public virtual void CommitEdit() {
            LineListHandler.CommitEdit();
        }

        //*********************************************************     
        //
        /// <summary>
        /// CommitAllEdit:              Commit the edit of all nodes
        /// </summary>
        /// <param name="nodeHandler">  Node handler</param>
        //  
        //*********************************************************     
        public virtual void CommitAllEdit(RowsetTreeDefNodeHandler nodeHandler) {
            nodeHandler.CommitEdit();
            foreach (RowsetTreeDefNodeHandler nodeHandlerChild in nodeHandler.m_arrChildNodeHandler) {
                CommitAllEdit(nodeHandlerChild);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// ValidateCurLine:        Validates the current line
        /// </summary>
        /// <returns>
        ///     false:              Line not valid
        ///     true:               Line valid
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool ValidateCurLine() {
            bool                        bRetVal = true;
            LineValidationEventArgs     e;
            FieldDef                    fldInError;
            Rowset                      rowset;

            rowset = Rowset;
            if (rowset != null && rowset.UndelLineCount != 0 && rowset.LineState != LineState.sdLineDeleted) {
                CommitAllEdit(this);
                e = new LineValidationEventArgs();
                RaiseBeforeLineValidated(e);
                if (e.Cancel) {
                    bRetVal = false;
                } else {
                    if (m_rowset.ValidActiveLine(true, out fldInError) == 0) {
                        OnAfterLineValidated(e);
                        if (e.Cancel) {
                            bRetVal = false;
                        }
                    } else {
                        bRetVal = false;
                    }
                }
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetCurrentLine:             Sets the current logical line
        /// </summary>
        /// <param name="iLine">        Logical or physical line</param>
        /// <returns>
        /// true                        New line selected
        /// false                       Line not selected
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool SetCurrentLine(int iLine) {
            bool    bRetVal = false;
            int     iOldLine;
            Rowset  rowset;

            rowset   = Rowset;
            if (rowset != null) {
                iOldLine = SBCurrentLine;
                if (iLine >= 0 && iLine != iOldLine && iLine < SBLineCount) {
                    if (ValidateCurLine()) {
                        bRetVal = LineListHandler.SelectLine(iLine);
                    }
                }
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectFirstLine:        Select the first line
        /// </summary>
        /// <returns>
        ///     true:               The first line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool SelectFirstLine() {
            return(SetCurrentLine(0));
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectPrevLine:         Select the previous line
        /// </summary>
        /// <returns>
        ///     true:               The previous line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool SelectPrevLine() {
            return(SetCurrentLine(SBCurrentLine - 1));
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectNextLine:         Select the next line
        /// </summary>
        /// <returns>
        ///     true:               The next line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool SelectNextLine() {
            return(SetCurrentLine(SBCurrentLine + 1));
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectLastLine:         Select the last line
        /// </summary>
        /// <returns>
        ///     true:               The last line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool SelectLastLine() {
            return(SetCurrentLine(SBLineCount - 1));
        }

		//*********************************************************     
		//
		/// <summary>
		/// InsertNewLine:          Insert a new line
		/// </summary>
		/// <param name="bAtEnd">   true to insert at the end</param>
		/// <returns>
		/// true:                   A new line has been inserted
		/// false:                  No line inserted
		/// </returns>
		//  
		//*********************************************************     
		public virtual bool InsertNewLine(bool bAtEnd) {
			bool                bRetVal = false;
			Rowset       rowset;
			ILineListHandler    lineHandler;

			rowset = Rowset;
			if (rowset != null) {
    			lineHandler = LineListHandler;
                if (CurrentLine == -1 || ValidateCurLine()) {
					bRetVal = lineHandler.InsertLine(bAtEnd);
					if (bRetVal) {
					    OnDataConversion(new DataConversionEventArgs(this, DataConversionTypeE.DC_Decode));
					}
				}
			}
			return(bRetVal);
		}

        //*********************************************************     
        //
        /// <summary>
        /// DuplicateLine:              Duplicate the current line
        /// </summary>
        /// <returns>
        /// true:                       The line has been duplicated
        /// false:                      No line inserted
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool DuplicateLine() {
            bool                bRetVal = false;
			Rowset       rowset;
			ILineListHandler    lineHandler;

			rowset  = Rowset;
			if (rowset != null) {
    			lineHandler = LineListHandler;
                if (CurrentLine != -1 && ValidateCurLine()) {
					bRetVal = lineHandler.DuplicateLine();
					if (bRetVal) {
					    OnDataConversion(new DataConversionEventArgs(this, DataConversionTypeE.DC_Decode));
					    //OnLineSelected(new LineSelectedEventArgs(iOldLine, CurrentLine));
					}
				}
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// DeleteCurrentLine:      Deletes the current line
        /// </summary>
        /// <returns>
        ///     true:               The line has been deleted
        ///     false:              No line deleted
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool DeleteCurrentLine() {
            bool                bRetVal = false;
			Rowset              rowset;
			ILineListHandler    lineHandler;

			rowset  = Rowset;
			if (rowset != null) {
    			lineHandler = LineListHandler;
				bRetVal     = lineHandler.DeleteCurrentLine();
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// HandleCommand:          Handle a command coming from a menu or a toolbar
        /// </summary>
        /// <param name="eCmd">     Command Id</param>
        /// <param name="bEvent">   true to trigger event</param>
        //  
        //*********************************************************     
        public virtual void HandleCommand(sdHandlerCmd eCmd, bool bEvent) {
            ExecuteCmdEventArgs     e;
            Boolean                 bSuccess = true;

            e = new ExecuteCmdEventArgs(eCmd);
            if (bEvent) {
                OnBeforeExecuteCmd(e);
            }
            if (!e.Cancel) {
                switch(eCmd) {
                case sdHandlerCmd.DoNothing:
                    break;
                case sdHandlerCmd.ItemFirst:
                    bSuccess = SelectFirstLine();
                    break;
                case sdHandlerCmd.ItemPrev:
                    bSuccess = SelectPrevLine();
                    break;
                case sdHandlerCmd.ItemNext:
                    bSuccess = SelectNextLine();
                    break;
                case sdHandlerCmd.ItemLast:
                    bSuccess = SelectLastLine();
                    break;
                case sdHandlerCmd.ItemNew:
                    bSuccess = InsertNewLine(true);
                    break;
                case sdHandlerCmd.ItemDel:
                    bSuccess = DeleteCurrentLine();
                    break;
                case sdHandlerCmd.ItemDup:
                    bSuccess = DuplicateLine();
                    break;
                }
                if (bEvent && bSuccess) {
                    OnAfterExecuteCmd(e);
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// GetActionState:             Get the specified action state
        /// </summary>
        /// <param name="eAction">      Action</param>
        /// <returns>
        /// false:                      Disabled
        /// true:                       Enabled
        /// </returns>returns>
        //  
        //*********************************************************     
        public virtual bool GetActionState(sdAction eAction) {
            bool    bRetVal;
            Rowset  rowset;
            int     iLinePos;
            int     iLineCount;
            
            rowset = Rowset;
            if (rowset == null) {
                bRetVal = false;
            } else {
                switch(eAction) {
                case sdAction.sdActionAdd:
                    bRetVal = (EffectiveAllowInsert) ? true : false;
                    break;
                case sdAction.sdActionDup:
                    iLineCount = SBLineCount;
                    bRetVal    = (iLineCount != 0) ? ((EffectiveAllowInsert) ? true : false) : false;
                    break;
                case sdAction.sdActionDel:
                    iLineCount = SBLineCount;
                    if (iLineCount != 0 && rowset.LineState != RepPP.LineState.sdLineDeleted) {
                        bRetVal = (rowset.LineState == RepPP.LineState.sdLineNew) ? true : EffectiveAllowDelete;
                    } else {
                        bRetVal = false;
                    }
                    break;
                case sdAction.sdActionFirst:
                    iLinePos   = SBCurrentLine;
                    iLineCount = SBLineCount;
                    bRetVal    = (iLineCount != 0 && iLinePos != 0) ? true : false;
                    break;
                case sdAction.sdActionLast:
                    iLinePos   = SBCurrentLine;
                    iLineCount = SBLineCount;
                    bRetVal    = (iLineCount != 0 && iLinePos < iLineCount - 1) ? true : false;
                    break;
                default:
                    bRetVal = false;
                    break;
                }
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// IsDirty:        Check if the card is dirty
        /// </summary>
        /// <returns>
        ///  true:          Yes
        ///  false:         No
        /// </returns>
        //
        //*********************************************************     
        public virtual bool IsDirty() {
            bool    bRetVal;
            Rowset  rowset;
            
            rowset = Rowset;
            if (rowset == null || SBLineCount == 0) {
                bRetVal = false;
            } else {
                bRetVal = rowset.IsLineModified();
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// GetItemState:   Get the state of the currently selected item
        /// </summary>
        /// <returns>
        /// LineState
        /// </returns>
        //  
        //*********************************************************     
        public virtual RepPP.LineState GetItemState() {
            RepPP.LineState     eRetVal;
            RepPP.Rowset rowset;
            RepPP.Rowset rowsetFather;
            bool                bDeleted;

            rowset = Rowset;
            if (rowset != null && SBLineCount != 0) {
                m_eState = rowset.LineState;
                if (m_eState == RepPP.LineState.sdLineOld && IsDirty()) {
                    m_eState = RepPP.LineState.sdLineChanged;
                }
            } else {
                m_eState = RepPP.LineState.sdLineOld;
            }
            bDeleted    = false;
            rowsetFather = rowset;
            while (rowsetFather != null && !bDeleted) {
                if (rowsetFather.LineCount != 0 && rowsetFather.LineState == RepPP.LineState.sdLineDeleted) {
                    bDeleted = true;
                } else {
                    rowsetFather = rowsetFather.FatherRowset;
                }
            }
            eRetVal = (bDeleted) ? RepPP.LineState.sdLineDeleted : m_eState;
            return(eRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// FldChgSink:                 Sink for FldChange event
        /// </summary>
        /// 
        /// <param name="sender">       Sender object</param>
        /// <param name="e">            Argument</param>
        //  
        //*********************************************************     
        private void FldChgSink(object sender, RepPP.FieldGenEventArgs e) {
            Rowset   rowset;
            
            rowset = Rowset;
            if (rowset != null && m_eState == RepPP.LineState.sdLineOld && rowset.LineState != m_eState) {
                if (e.FieldDef.RowsetDef == RowsetTreeDefNode.RowsetDef) {
                    RaiseAdjustState();
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnLineListHandler_LineSelectedInList:   Sink for LineSelectedInList event
        ///                                         generated by the default LineListHandler
        /// </summary>
        /// 
        /// <param name="sender">                   Sender object</param>
        /// <param name="e">                        Argument</param>
        //  
        //*********************************************************     
        private void OnLineListHandler_LineSelectedInList(object sender, 
                                                          RepPP.Toolkit.Window.LineSelectedInListEventArgs e){
            OnLineSelected(new LineSelectedEventArgs(e.OldLine, e.NewLine));
        }

        //*********************************************************
        //
        /// <summary>
        /// LineValidationEventArgs:    Arguments of the event when a line is validated
        /// </summary>
        //
        //*********************************************************
        public class LineValidationEventArgs : System.EventArgs {
            /// <summary>true to refuse the validation</summary>
            public  bool        Cancel;
            /// <summary>Defines the argument of the event</summary>
            public              LineValidationEventArgs() : base() { Cancel = false; }
        };
        /// <summary>Defines the delegate for the event</summary>
        public delegate void LineValidationEventHandler(object sender, LineValidationEventArgs e);

        //*********************************************************
        //
        /// <summary>
        /// RowsetLineHandler:  Implements the default line handler. This handler handle directly the rowset.
        /// </summary>
        //
        //*********************************************************
        private class RowsetLineHandler : ILineListHandler {
            /// <summary>Father node handler</summary>
            private RowsetTreeDefNodeHandler                                    m_nodeHandlerFather;
            /// <summary>true to see deleted line</summary>
            private bool                                                        m_bSeeDeletedLine;
            /// <summary> Called when a line is selected in the LineListHandler</summary>
            public event RepPP.Toolkit.Window.LineSelectedInListEventHandler    LineSelectedInList;

            //*********************************************************
            //
            /// <summary>
            /// ctor:                               Class constructor
            /// </summary>
            /// <param name="nodeHandlerFather">    Father card group handler</param>
            /// <param name="bSeeDeletedLine">      true to see deleted line</param>
            //
            //*********************************************************
            public RowsetLineHandler(RowsetTreeDefNodeHandler nodeHandlerFather, bool bSeeDeletedLine) {
                m_nodeHandlerFather = nodeHandlerFather;
                m_bSeeDeletedLine   = bSeeDeletedLine;
            }

            //*********************************************************
            //
            /// <summary>
            /// SelectLine:                         Select a line
            /// </summary>
            /// <returns>
            /// true if succeed, false if failed
            /// </returns>
            //
            //*********************************************************
            public bool SelectLine(int iPos) {
                Rowset  rowset;
                bool    bRetVal;
                int     iOldLine = -1;
                
                rowset = m_nodeHandlerFather.Rowset;
                if (rowset == null) {
                    bRetVal = false;
                } else {
                    iOldLine = rowset.CurrentLine;
                    if (m_bSeeDeletedLine) {
                        rowset.SelectLine(iPos, true, true);
                    } else {
                        rowset.SelectUndelLine(iPos, true, true);
                    }
                    OnLineSelectedInList(new LineSelectedInListEventArgs(iOldLine, rowset.CurrentLine));
                    bRetVal = true;
                }
                return(bRetVal);
            }

            //*********************************************************
            //
            /// <summary>
            /// InsertLine:                         Insert a new line
            /// </summary>
            /// <param name="bAtEnd">               true to insert at end</param>
            /// <returns>
            /// true if succeed, false if failed
            /// </returns>
            //
            //*********************************************************
            public bool InsertLine(bool bAtEnd) {
                bool    bRetVal;
                Rowset  rowset;
                int     iOldLine;
                
                rowset = m_nodeHandlerFather.Rowset;
                if (rowset == null) {
                    bRetVal = false;
                } else {
                    iOldLine = rowset.CurrentLine;
                    rowset.AddBlankLine(bAtEnd);
                    OnLineSelectedInList(new LineSelectedInListEventArgs(iOldLine, rowset.CurrentLine));
                    bRetVal = true;
                }
                return(bRetVal);
            }

            //*********************************************************
            //
            /// <summary>
            /// DuplicateLine:                      Duplicate the current line
            /// </summary>
            /// <returns>
            /// true if succeed, false if failed
            /// </returns>
            //
            //*********************************************************
            public bool DuplicateLine() {
                bool    bRetVal;
                Rowset  rowset;
                int     iOldLine;
                
                rowset = m_nodeHandlerFather.Rowset;
                if (rowset == null) {
                    bRetVal = false;
                } else {
                    iOldLine = rowset.CurrentLine;
                    rowset.DuplicateLine();
                    OnLineSelectedInList(new LineSelectedInListEventArgs(iOldLine, rowset.CurrentLine));
                    bRetVal = true;
                }
                return(bRetVal);
            }

            //*********************************************************
            //
            /// <summary>
            /// DeleteCurrentLine:                  Deletes the current line
            /// </summary>
            /// <returns>
            /// Index of the currently selected item
            /// </returns>
            //
            //*********************************************************
            public bool DeleteCurrentLine() {
                bool    bRetVal;
                Rowset  rowset;
                int     iOldLine;
                
                rowset = m_nodeHandlerFather.Rowset;
                if (rowset == null) {
                    bRetVal = false;
                } else {
                    iOldLine = rowset.CurrentLine;
                    rowset.DeleteLine();
                    OnLineSelectedInList(new LineSelectedInListEventArgs(iOldLine, rowset.CurrentLine));
                    bRetVal = true;
                }
                return(bRetVal);
            }

            //*********************************************************
            //
            /// <summary>
            /// CommitEdit:                         Commit edit before validation
            /// </summary>
            //
            //*********************************************************
            public void CommitEdit() {
            }

            //*********************************************************
            //
            /// <summary>
            /// Refresh:                            Refresh the list from the rowset
            /// </summary>
            //
            //*********************************************************
            public void Refresh() {
            }

            //*********************************************************
            //
            /// <summary>
            /// AssociateRowset:             Associate a rowset
            /// </summary>
            /// <param name="rowset">                RowsetDef instance to associate</param>
            //
            //*********************************************************
            public void AssociateRowset(Rowset rowset) {
                int     iCurLine;

                iCurLine = (rowset == null) ? -1 : rowset.CurrentLine;
                OnLineSelectedInList(new LineSelectedInListEventArgs(-1, iCurLine));
            }

            //********************************************************************************
            //
            /// <summary>
            /// OnLineSelectedInList:           Triggers the LineSelectedInList event
            /// </summary>
            /// <param name="e">                Event Arguments</param>
            //
            //********************************************************************************
            public void OnLineSelectedInList(LineSelectedInListEventArgs e) {
                if (LineSelectedInList != null){
                    LineSelectedInList(this, e);
                }
            }

        } // Class RowsetLineHandler

        //*********************************************************
        //
        /// <summary>
        /// LineSelectedEventArgs:  Arguments of the event when a new line is selected
        /// </summary>
        //
        //*********************************************************
        public class LineSelectedEventArgs : System.EventArgs {
            /// <summary>Line which was selected before the event</summary>
            public  int         OldLine;
            /// <summary>New selected line</summary>
            public  int         NewLine;

            //*********************************************************
            //
            /// <summary>
            /// ctor:                   Class constructor
            /// </summary>
            /// <param name="OldLine">  Line previously selected</param>
            /// <param name="NewLine">  Line newly selected</param>
            //
            //*********************************************************
            public LineSelectedEventArgs(int OldLine, int NewLine) : base() { this.OldLine = OldLine; this.NewLine = NewLine; }
        }; // Class LineSelectedEventArgs
        /// <summary>Delegate for the event</summary>
        public delegate void LineSelectedEventHandler(object sender, LineSelectedEventArgs e);

        //*********************************************************
        //
        /// <summary>
        /// ExecuteCmdEventArgs:    Arguments of the event when a command is executed
        /// </summary>
        //
        //*********************************************************
        public class ExecuteCmdEventArgs : System.EventArgs {
            /// <summary>true to cancel the command</summary>
            public  bool            Cancel;
            /// <summary>Command being executed</summary>
            public  sdHandlerCmd    Cmd;
            /// <summary>Defines the argument of the event</summary>
            public                  ExecuteCmdEventArgs(sdHandlerCmd eCmd) : base() { Cancel = false; Cmd = eCmd; }
        };
        /// <summary>Defines the delegate for the event</summary>
        public delegate void ExecuteCmdEventHandler(object sender, ExecuteCmdEventArgs e);

        //*********************************************************
        //
        /// <summary>
        /// ErrorArgs:  Arguments of the event when a new line is selected
        /// </summary>
        //
        //*********************************************************
        public class ErrorOccurredEventArgs : System.EventArgs {
            /// <summary>Error being report</summary>
            public  RepPP.Error     Error;
            /// <summary>Defines the argument of the event</summary>
            public                  ErrorOccurredEventArgs(RepPP.Error scError) : base() { this.Error = scError; }
        };
        /// <summary>Defines the delegate for the event</summary>
        public delegate void ErrorOccurredEventHandler(object sender, ErrorOccurredEventArgs e);

        //*********************************************************
        //
        /// <summary>
        /// DataConversionEventArgs:    Arguments of the event when data conversion is need
        /// </summary>
        //
        //*********************************************************
        public class DataConversionEventArgs : System.EventArgs {
            /// <summary>Type of dataconversion</summary>
            public DataConversionTypeE      Type;
            /// <summary>Rowset tree definition node handler</summary>
            public RowsetTreeDefNodeHandler NodeHandler;
            /// <summary>Defines the argument of the event</summary>
            public                          DataConversionEventArgs(RowsetTreeDefNodeHandler node, DataConversionTypeE eType) : base() { NodeHandler = node; Type = eType; }
        };
        /// <summary>Defines the delegate for the event</summary>
        public delegate void DataConversionEventHandler(object sender, DataConversionEventArgs e);
	} // Class RowsetTreeDefNodeHandler
} // Namespace
