//
// Rep++ Framework for Windows Application
// Copyright (C) 2005 Consyst SQL Inc.
// All Rights Reserved
//
// This source code is only intended as a supplement to the
// Rep++ Studio products.
//
using System;
using System.Collections;
using RepPP;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;
using System.Windows.Forms;
namespace RepPP.Framework.Window {
    /// <summary>
    /// SCCardGroupHandler:    Handle a card group in a hiearchy
    /// </summary>
	public class RowsetTreeDefNodeHandlerRoot : RowsetTreeDefNodeHandlerRootBase, IRowsetTreeDefNodeHandlerRoot {
        /// <summary>true if set is openned</summary>
        private bool                                m_bSetOpen;

        //*********************************************************     
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        /// <param name="rowsetTreeDef">    Attached rowset tree definition</param>
        /// <param name="bSeeDeletedLine">  true to see deleted line, false not to see it</param>
        /// <param name="bUndo">            true to support UNDO mode</param>
        //  
        //*********************************************************     
		public RowsetTreeDefNodeHandlerRoot(RowsetTreeDef rowsetTreeDef, bool bSeeDeletedLine, bool bUndo) : base(rowsetTreeDef, bSeeDeletedLine, bUndo) {
            m_bSetOpen = false;
		}

        //*********************************************************     
        //
        /// <summary>
        /// Dispose:                Clean the class
        /// </summary>
        //  
        //*********************************************************     
        public override void Dispose() {
            m_bSetOpen = false;
            base.Dispose();
        }

        //*********************************************************     
        //
        /// <summary>
        /// CommandHolder:              Command holder
        /// </summary>
        //  
        //*********************************************************     
        public override CmdHolder CommandHolder {
            get {
                return(base.CommandHolder);
            }
            set {
                CmdHolder   cmdHolder;
                
                cmdHolder = CommandHolder;
                if (cmdHolder != value) {
                    if (cmdHolder != null) {
                        cmdHolder.RemoveCmd(sdHandlerCmd.ItemSaveAll.ToString());
                        cmdHolder.RemoveCmd(sdHandlerCmd.RowsetTreeSearch.ToString());
                    }
                    base.CommandHolder = value;
                    if (value != null) {
                        value.AddCmd(sdHandlerCmd.ItemSaveAll.ToString(), new CmdHolder.OnCommand(CmdItemSaveAll), CmdHolder.CmdStateE.Disabled);
                        value.AddCmd(sdHandlerCmd.RowsetTreeSearch.ToString(), new CmdHolder.OnCommand(CmdRowsetTreeSearch), CmdHolder.CmdStateE.Enabled);
                    }
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetOpen:                        Determine if the set of rows is opened or closed.
        /// </summary>
        //  
        //*********************************************************     
        public virtual bool SetOpen {
            get {
                return(m_bSetOpen);
            }
            set {
                m_bSetOpen = value;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RowsetTree:           Attached rowset tree
        /// </summary>
        //  
        //*********************************************************     
        public override RowsetTree RowsetTree {
            get {
                return(base.RowsetTree);
            }
            set {
                base.RowsetTree = value;
                SetOpen = false;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdItemSaveAll:             Command 'Save All'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdItemSaveAll() {
            HandleCommand(sdHandlerCmd.ItemSaveAll, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdRowsetTreeSearch:              Command 'Search'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdRowsetTreeSearch() {
            HandleCommand(sdHandlerCmd.RowsetTreeSearch, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetCommandHolderState:  Set the state of the command holder
        /// </summary>
        //  
        //*********************************************************     
        public override void SetCommandHolderState() {
            CmdHolder.CmdStateE     eSaveAll;
            CmdHolder               cmdHolder;

            base.SetCommandHolderState();
            cmdHolder = CommandHolder;
            if (cmdHolder != null) {
                eSaveAll = GetActionState(sdAction.sdActionSaveAll) ?
                           CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled;
                cmdHolder.SetCmdState(sdHandlerCmd.ItemSaveAll.ToString(), eSaveAll);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// HandleCommand:          Handle a command coming from a menu or a toolbar
        /// </summary>
        /// <param name="eCmd">     Command Id</param>
        /// <param name="bEvent">   true to trigger event</param>
        //  
        //*********************************************************     
        public override void HandleCommand(sdHandlerCmd eCmd, bool bEvent) {
            ExecuteCmdEventArgs e;
            Boolean             bSuccess = true;

            if (m_app != null) {
                m_app.Errors.Clear();
            }
            e = new RowsetTreeDefNodeHandler.ExecuteCmdEventArgs(eCmd);
            if (bEvent) {
                OnBeforeExecuteCmd(e);
            }
            if (!e.Cancel) {
                base.HandleCommand(eCmd, false);
                switch(eCmd) {
                case sdHandlerCmd.ItemSaveAll:
                    bSuccess = SaveAllLines();
                    break;
                case sdHandlerCmd.RowsetTreeSearch:
                    bSuccess = OpenSet(CurrentFieldDef);
                    break;
                }
                if (bEvent && bSuccess) {
                    OnAfterExecuteCmd(e);
                }
            }
            TriggerErrors();
        }

        //*********************************************************     
        //
        /// <summary>
        /// InsertNewLine:          Insert a new line
        /// </summary>
        /// 
        /// <param name="bAtEnd">   true to insert at the end</param>
        /// <returns>
        ///     true:               A new line has been inserted
        ///     false:              No line inserted
        /// </returns>
        //  
        //*********************************************************     
        public override bool InsertNewLine(bool bAtEnd) {
            bool            bRetVal;
            RowsetTree      rowsetTree;
            RowsetTree      rowsetTreeUndo;
            Rowset          rowset;
            Rowset          rowsetUndo;

            rowsetTreeUndo  = UndoRowsetTree;
            bRetVal         = base.InsertNewLine(bAtEnd);
            if (bRetVal && rowsetTreeUndo != null) {
                rowsetTree = RowsetTree;
                rowset     = rowsetTree.RootRowset;
                rowsetUndo = rowsetTreeUndo.RootRowset;
                if (!bAtEnd && rowsetUndo.LineCount != 0) {
                    rowsetUndo.SelectLine(rowset.CurrentLine, false, false);
                }
                rowsetUndo.AddBlankLine(bAtEnd);
                rowset.Select(true);
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// UpdateLineToDb:             Update the line to DB
        /// </summary>
        /// <returns>
        ///     Zero                    No error
        ///     Non-Zero                Error
        /// </returns>
        //  
        //*********************************************************     
        protected virtual int UpdateLineToDb() {
            int                     iRetVal;
            RepPP.RowsetState       eReason;
            string                  strError;
            ITransactionDataAccess  dataAccess;
            RowsetTree              rowsetTree;

            dataAccess  = DataAccess;
            rowsetTree  = RowsetTree;
            if (dataAccess != null) {
                iRetVal = dataAccess.SaveTransactionData(rowsetTree, out strError);
            } else {
                iRetVal = rowsetTree.UpdateLineToDb(true);
            }
            if (iRetVal == 0) {
                iRetVal = AfterSave() ? 0: (int)RepPP.ErrorCode.sdRetPrgRefused;
            }
            if (iRetVal == 0) {
                eReason = rowsetTree.RootRowset.State;
                if (AutoCommit) {
                    m_app.DataConnection.Commit();
                    rowsetTree.RootRowset.ResetLine();
                }
                AfterCommit(eReason);
            } else {
                m_app.DataConnection.Rollback();
            }
            return(iRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// SaveLine:               Save the current line to DB
        /// </summary>
        /// <returns>
        ///     false:              Not saved
        ///     true:               Saved
        /// </returns>
        //  
        //*********************************************************     
        public override bool SaveLine() {
            bool        bRetVal = false;
            LineState   eLineState;
            int         iLinePos;
            Rowset      rowset;
            Rowset      rowsetUndo;
            
            //TODO: Improve error reporting...
            //      If error from database, full error report please.
            rowset   = Rowset;
            if (rowset != null && SBCurrentLine != -1) {                                
                if (rowset.LineState == LineState.sdLineDeleted || 
                    ValidateCurLine()) {
                    rowsetUndo  = (UndoRowsetTree == null) ? null : UndoRowsetTree.RootRowset;
                    eLineState = rowset.LineState;
                    iLinePos   = rowset.CurrentLine;
                    if (UpdateLineToDb() == 0) {
                        bRetVal = true;
                        if (rowsetUndo != null) {
                            rowsetUndo.SelectLine(iLinePos, true, false);
                            // If line was deleted, remote it from undo
                            if (eLineState == LineState.sdLineDeleted) {
                                rowsetUndo.LineState = LineState.sdLineNew;
                                rowsetUndo.DeleteLine();
                            } else {
                                rowsetUndo.CopyLineFrom(rowset, CopyFieldOpt.sdResetAnsChg | CopyFieldOpt.sdSetValid, true);
                                rowsetUndo.LineState = LineState.sdLineOld;
                            }
                            rowset.Select(true);
                        }
                    } else {
                        bRetVal = false;
                    }
                    OnLineSelected(new LineSelectedEventArgs(iLinePos, CurrentLine));
                }
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CloseSet:                Close a card set
        /// </summary>
        /// <returns>
        ///  true:                   Set closed
        ///  false:                  Set not closed
        /// </returns>
        //
        //*********************************************************     
        public virtual bool CloseSet() {
            bool        bRetVal;
            RowsetTree  rowsetTree;

            rowsetTree = RowsetTree;
            if (rowsetTree == null) {
                bRetVal = true;
            } else {
                if (SetOpen) {
                    // Trigger ErrorOccurred event if some errors are added 
                    // to rep++ error collection and clear this collection
                    // because HandleCommand call CloseSetb and check for
                    // errors.
                    m_app.Errors.Clear();
                    bRetVal = AskToSave();
                    if (bRetVal) {
                        SetOpen = false;
                    }
                    TriggerErrors();
                    m_app.Errors.Clear();
                } else {
                    bRetVal = true;
                }
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// OpenSet:                 Open a card set using the current rowset
        /// </summary>
        /// <returns>
        ///  true:                   Set openned
        ///  false:                  Set not openned
        /// </returns>
        //
        //*********************************************************     
        public virtual bool OpenSet() {
            bool          bRetVal;
            RowsetTree    rowsetTree;

            rowsetTree = RowsetTree;
            if (rowsetTree == null) {
                bRetVal = false;
            } else {            
                bRetVal = CloseSet();
                if (bRetVal) {
                    rowsetTree.RootRowset.Select(true, false);
                    SetOpen = true;
                    OnLineSelected(new LineSelectedEventArgs(-1, CurrentLine));
                }
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// OpenSet:                        Open a card set using the specified WHERE clause
        /// </summary>
        ///
        /// <param name="strWhereClause">   Where clause</param>
        /// <returns>
        ///  true:                          Set openned
        ///  false:                         Set not openned
        /// </returns>
        //
        //*********************************************************     
        public virtual bool OpenSet(string strWhereClause) {
            bool            bRetVal;
            RowsetTree      rowsetTree;

            rowsetTree = RowsetTree;
            if (rowsetTree == null) {
                bRetVal = false;
            } else {            
                bRetVal = CloseSet();
                if (bRetVal) {
                    if (ReadFromDb(strWhereClause, null) != 0) {
                        bRetVal = false;
                    } else {
                        if (rowsetTree.RootRowset.LineCount == 0) {
                            rowsetTree.RootRowset.AddBlankLine(true);
                        }
                        SetOpen = true;
                        OnLineSelected(new LineSelectedEventArgs(-1, CurrentLine));
                    }
                }
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// OpenSet:                Open a card set using the specified FieldDef value
        /// </summary>
        ///
        /// <param name="fldDef">   Field definition used for selection</param>
        /// <returns>
        ///  true:                  Set openned
        ///  false:                 Set not openned
        /// </returns>
        //
        //*********************************************************     
        public virtual bool OpenSet(FieldDef fldDef) {
            string strWhereClause = string.Empty;
            string strOperator;
            bool   bRetVal;

            if (fldDef != null && fldDef.IsDatabaseLinked && fldDef.Value != string.Empty) {
                if (fldDef.Value.IndexOf("%") != -1){
                    strOperator = "LIKE";
                } else {
                    strOperator = "=";
                }
                strWhereClause = String.Format(" WHERE {0} {1} :{2}.{3}",
                                               fldDef.ColName,
                                               strOperator,
                                               fldDef.RowsetDef.Name,
                                               fldDef.Name);
            }
            bRetVal = OpenSet(strWhereClause);
            return (bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// GetActionState:         Get the specified action state
        /// </summary>
        ///
        /// <param name="eAction">  Action</param>
        /// <returns>
        ///      false:             Disabled
        ///      true:              Enabled
        /// </returns>
        //  
        //*********************************************************     
        public override bool GetActionState(sdAction eAction) {
            bool        bRetVal;
            RowsetTree  rowsetTree;

            switch(eAction) {
            case sdAction.sdActionSaveAll:
                rowsetTree   = RowsetTree;
                bRetVal = (rowsetTree != null && rowsetTree.RootRowset.IsModified()) ? true : false;
                break;
            default:
                bRetVal = base.GetActionState(eAction);
                break;
            }
            return(bRetVal);
        }
	} // Class RootRowsetTreeDefNodeHandler
} // Namespace
