//
// Rep++ Framework for Windows Application
// Copyright (C) 2005 Consyst SQL Inc.
// All Rights Reserved
//
// This source code is only intended as a supplement to the
// Rep++ Studio products.
//
using System;
using System.Collections;
using RepPP;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;
using System.Windows.Forms;

namespace RepPP.Framework.Window {
    /// <summary>
    /// SCCardGroupHandler:    Handle a card group in a hiearchy
    /// </summary>
	public abstract class RowsetTreeDefNodeHandlerRootBase : RowsetTreeDefNodeHandler {
        /// <summary>Reset field error</summary>
        public event System.EventHandler            ResetFieldError;
        /// <summary>Adjust the field state</summary>
        public event System.EventHandler            AdjustFieldState;
        
        /// <summary>Application object</summary>
        protected RepPP.Application                 m_app;
        /// <summary>RowsetTreeDef attached to this wrapper</summary>
        private RowsetTreeDef                       m_rowsetTreeDef;
        /// <summary>Current rowset tree</summary>
        private RowsetTree                          m_rowsetTree;
        /// <summary>Undo rowset tree</summary>
        private RowsetTree                          m_rowsetTreeUndo;
        /// <summary>Current field</summary>
        private   FieldDef                          m_fldDefCurrent;
        /// <summary>true if undo is supported</summary>
        private   bool                              m_bUndo;
        /// <summary>true for auto-commit</summary>
        private   bool                              m_bAutoCommit;
        /// <summary>Array of user validation object</summary>
        private   ArrayList                         m_arrValidationObj;
        /// <summary>data access object</summary>
        private ITransactionDataAccess              m_dataAccess;

        //*********************************************************     
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        /// <param name="rowsetTreeDef">    Attached rowset tree definition</param>
        /// <param name="bSeeDeletedLine">  true to see deleted line, false not to see it</param>
        /// <param name="bUndo">            true to support UNDO mode</param>
        //  
        //*********************************************************     
		protected RowsetTreeDefNodeHandlerRootBase(RowsetTreeDef rowsetTreeDef, bool bSeeDeletedLine, bool bUndo) : base(null, rowsetTreeDef.Nodes[0], bSeeDeletedLine) {
            m_app                     = rowsetTreeDef.Application;
            m_rowsetTree              = null;
            m_rowsetTreeUndo          = null;
            m_rowsetTreeDef           = rowsetTreeDef;
            m_fldDefCurrent           = null;
            m_bUndo                   = bUndo;
            m_bAutoCommit             = true;
            m_dataAccess              = null;
            IsRowsetTreeValid(rowsetTreeDef);
            m_app.FldCtlLoseFocus    += new FldCtlLoseFocusEventHandler(FldCtlLoseFocusSink);
            m_app.FldCtlReceiveFocus += new FldCtlReceiveFocusEventHandler(FldCtlRecFocusSink);
            m_app.FldEnter           += new FldEnterEventHandler(FldEnterSink);
            m_app.DbAfterReadLine    += new DbAfterReadLineEventHandler(DbAfterReadLineSink);
            LoadUserValidationCode();
		}

        //*********************************************************     
        //
        /// <summary>
        /// Dispose:                Clean the class
        /// </summary>
        //  
        //*********************************************************     
        public override void Dispose() {
            IUserValidationCode uvc;
            
            if (m_arrValidationObj != null) {
                foreach(object obj in m_arrValidationObj) {
                    uvc = (IUserValidationCode)obj;
                    if (uvc != null) {
                        uvc.Dispose();
                    }        
                }
            }
            m_app.FldCtlLoseFocus      -= new FldCtlLoseFocusEventHandler(FldCtlLoseFocusSink);
            m_app.FldCtlReceiveFocus   -= new FldCtlReceiveFocusEventHandler(FldCtlRecFocusSink);
            m_app.FldEnter             -= new FldEnterEventHandler(FldEnterSink);
            m_app.DbAfterReadLine      -= new DbAfterReadLineEventHandler(DbAfterReadLineSink);
            m_fldDefCurrent                = null;
            if (m_rowsetTreeUndo != null) {
                m_rowsetTreeUndo.Delete();
                m_rowsetTreeUndo = null;
            }
            m_rowsetTree = null;
            m_rowsetTreeDef   = null;
            m_app   = null;
            base.Dispose();
        }

        //*********************************************************     
        //
        /// <summary>
        /// AutoCommit:             true if auto-commit is on
        /// </summary>
        //  
        //*********************************************************     
        public virtual bool AutoCommit {
            get {
                return(m_bAutoCommit);
            }
            set {
                m_bAutoCommit = value;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// CommandHolder:              Command holder
        /// </summary>
        //  
        //*********************************************************     
        public override CmdHolder CommandHolder {
            get {
                return(base.CommandHolder);
            }
            set {
                CmdHolder   cmdHolder;
                
                cmdHolder = CommandHolder;
                if (cmdHolder != value) {
                    if (cmdHolder != null) {
                        cmdHolder.RemoveCmd(sdHandlerCmd.ItemSave.ToString());
                        cmdHolder.RemoveCmd(sdHandlerCmd.ItemUndo.ToString());
                        cmdHolder.RemoveCmd(sdHandlerCmd.FieldCut.ToString());
                        cmdHolder.RemoveCmd(sdHandlerCmd.FieldCopy.ToString());
                        cmdHolder.RemoveCmd(sdHandlerCmd.FieldPaste.ToString());
                        cmdHolder.RemoveCmd(sdHandlerCmd.FieldChoice.ToString());
                        cmdHolder.RemoveCmd(sdHandlerCmd.FieldHelp.ToString());
                        cmdHolder.RemoveCmd(sdHandlerCmd.FieldErase.ToString());
                        cmdHolder.RemoveCmd(sdHandlerCmd.FieldDefault.ToString());
                        cmdHolder.RemoveCmd(sdHandlerCmd.FieldEraseAll.ToString());
                        cmdHolder.RemoveCmd(sdHandlerCmd.FieldDefaultAll.ToString());
                        cmdHolder.RemoveCmd(sdHandlerCmd.LineNext.ToString());
                        cmdHolder.RemoveCmd(sdHandlerCmd.LinePrev.ToString());
                        cmdHolder.RemoveCmd(sdHandlerCmd.LineFirst.ToString());
                        cmdHolder.RemoveCmd(sdHandlerCmd.LineLast.ToString());
                        cmdHolder.RemoveCmd(sdHandlerCmd.LineNew.ToString());
                        cmdHolder.RemoveCmd(sdHandlerCmd.LineDuplicate.ToString());
                        cmdHolder.RemoveCmd(sdHandlerCmd.LineDelete.ToString());
                    }
                    base.CommandHolder = value;
                    if (value != null) {
                        value.AddCmd(sdHandlerCmd.ItemSave.ToString(),          new CmdHolder.OnCommand(CmdItemSave), CmdHolder.CmdStateE.Disabled);
                        value.AddCmd(sdHandlerCmd.ItemUndo.ToString(),          new CmdHolder.OnCommand(CmdItemUndo), CmdHolder.CmdStateE.Disabled);
                        value.AddCmd(sdHandlerCmd.FieldCut.ToString(),          new CmdHolder.OnCommand(CmdFieldCut), CmdHolder.CmdStateE.Hidden);
                        value.AddCmd(sdHandlerCmd.FieldCopy.ToString(),         new CmdHolder.OnCommand(CmdFieldCopy), CmdHolder.CmdStateE.Hidden);
                        value.AddCmd(sdHandlerCmd.FieldPaste.ToString(),        new CmdHolder.OnCommand(CmdFieldPaste), CmdHolder.CmdStateE.Hidden);
                        value.AddCmd(sdHandlerCmd.FieldChoice.ToString(),       new CmdHolder.OnCommand(CmdFieldChoice), CmdHolder.CmdStateE.Disabled);
                        value.AddCmd(sdHandlerCmd.FieldHelp.ToString(),         new CmdHolder.OnCommand(CmdFieldHelp), CmdHolder.CmdStateE.Disabled);
                        value.AddCmd(sdHandlerCmd.FieldErase.ToString(),        new CmdHolder.OnCommand(CmdFieldErase), CmdHolder.CmdStateE.Disabled);
                        value.AddCmd(sdHandlerCmd.FieldDefault.ToString(),      new CmdHolder.OnCommand(CmdFieldDefault), CmdHolder.CmdStateE.Disabled);
                        value.AddCmd(sdHandlerCmd.FieldEraseAll.ToString(),     new CmdHolder.OnCommand(CmdFieldEraseAll), CmdHolder.CmdStateE.Disabled);
                        value.AddCmd(sdHandlerCmd.FieldDefaultAll.ToString(),   new CmdHolder.OnCommand(CmdFieldDefaultAll), CmdHolder.CmdStateE.Disabled);
                        value.AddCmd(sdHandlerCmd.LineNext.ToString(),          new CmdHolder.OnCommand(CmdLineNext), CmdHolder.CmdStateE.Disabled);
                        value.AddCmd(sdHandlerCmd.LinePrev.ToString(),          new CmdHolder.OnCommand(CmdLinePrev), CmdHolder.CmdStateE.Disabled);
                        value.AddCmd(sdHandlerCmd.LineFirst.ToString(),         new CmdHolder.OnCommand(CmdLineFirst), CmdHolder.CmdStateE.Disabled);
                        value.AddCmd(sdHandlerCmd.LineLast.ToString(),          new CmdHolder.OnCommand(CmdLineLast), CmdHolder.CmdStateE.Disabled);
                        value.AddCmd(sdHandlerCmd.LineNew.ToString(),           new CmdHolder.OnCommand(CmdLineNew), CmdHolder.CmdStateE.Disabled);
                        value.AddCmd(sdHandlerCmd.LineDuplicate.ToString(),     new CmdHolder.OnCommand(CmdLineDuplicate), CmdHolder.CmdStateE.Disabled);
                        value.AddCmd(sdHandlerCmd.LineDelete.ToString(),        new CmdHolder.OnCommand(CmdLineDelete), CmdHolder.CmdStateE.Disabled);
                    }
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// DataAccess:              Data access object
        /// </summary>
        //  
        //*********************************************************     
        public virtual ITransactionDataAccess DataAccess {
            get {
                return(m_dataAccess);
            }
            set {
                m_dataAccess = value;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdItemSave:                Command 'Save'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdItemSave() {
            HandleCommand(sdHandlerCmd.ItemSave, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdItemUndo:                Command 'Undo'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdItemUndo() {
            HandleCommand(sdHandlerCmd.ItemUndo, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdFieldCut:                Command 'FieldDef Cut'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdFieldCut() {
            HandleCommand(sdHandlerCmd.FieldCut, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdFieldCopy:               Command 'FieldDef Copy'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdFieldCopy() {
            HandleCommand(sdHandlerCmd.FieldCopy, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdFieldPaste:              Command 'FieldDef Paste'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdFieldPaste() {
            HandleCommand(sdHandlerCmd.FieldPaste, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdFieldChoice:             Command 'FieldDef Choice'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdFieldChoice() {
            HandleCommand(sdHandlerCmd.FieldChoice, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdFieldHelp:               Command 'FieldDef Help'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdFieldHelp() {
            HandleCommand(sdHandlerCmd.FieldHelp, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdFieldErase:              Command 'Erase FieldDef'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdFieldErase() {
            HandleCommand(sdHandlerCmd.FieldErase, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdFieldDefault:            Command 'Default FieldDef'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdFieldDefault() {
            HandleCommand(sdHandlerCmd.FieldDefault, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdFieldEraseAll:           Command 'Erase all FieldDefs'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdFieldEraseAll() {
            HandleCommand(sdHandlerCmd.FieldEraseAll, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdFieldDefaultAll:         Command 'Default all FieldDefs'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdFieldDefaultAll() {
            HandleCommand(sdHandlerCmd.FieldDefaultAll, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdLineFirst:               Command 'First Line'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdLineFirst() {
            HandleCommand(sdHandlerCmd.LineFirst, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdLineLast:                Command 'Last Line'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdLineLast() {
            HandleCommand(sdHandlerCmd.LineLast, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdLinePrev:                Command 'Previous Line'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdLinePrev() {
            HandleCommand(sdHandlerCmd.LinePrev, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdLineNext:                Command 'Next Line'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdLineNext() {
            HandleCommand(sdHandlerCmd.LineNext, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdLineNew:                 Command 'New Line'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdLineNew() {
            HandleCommand(sdHandlerCmd.LineNew, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdLineDuplicate:           Command 'Duplicate Line'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdLineDuplicate() {
            HandleCommand(sdHandlerCmd.LineDuplicate, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdLineDelete:              Command 'Delete Line'
        /// </summary>
        //  
        //*********************************************************     
        private void CmdLineDelete() {
            HandleCommand(sdHandlerCmd.LineDelete, true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetCommandHolderState:  Set the state of the command holder
        /// </summary>
        //  
        //*********************************************************     
        public override void SetCommandHolderState() {
            RepPP.FieldDef              fldCurrent;
            RowsetTreeDefNodeHandler    nodeHandler;
            RowsetTreeDefNodeHandler    nodeHandlerActive;
            CmdHolder.CmdStateE         eSave;
            CmdHolder.CmdStateE         eUndo;
            CmdHolder.CmdStateE         eIsCurrentField;
            CmdHolder.CmdStateE         eIsSelectable;
            CmdHolder.CmdStateE         eIsAnySelectable;
            CmdHolder.CmdStateE         eIsChoiceAvailable;
            CmdHolder                   cmdHolder;
            bool                        bUpdateAllowed;

            base.SetCommandHolderState();
            cmdHolder = CommandHolder;
            if (cmdHolder != null) {
                fldCurrent  = CurrentFieldDef;
                nodeHandler = (fldCurrent == null) ? null : FindRowsetTreeDefNodeHandler(fldCurrent.RowsetDef);
                if (nodeHandler == null && fldCurrent != null) {
                    nodeHandler = this;
                }
                bUpdateAllowed      = (nodeHandler != null && 
                                       nodeHandler.Rowset != null && 
                                       nodeHandler.Rowset.IsDetached == false && 
                                       nodeHandler.Rowset.UndelLineCount > 0 && 
                                       nodeHandler.AllowUpdate != sdAllow.sdAllowNo);
                eSave               = GetActionState(sdAction.sdActionSave) ?
                                      CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled;
                eUndo               = GetActionState(sdAction.sdActionUndo) ?
                                      CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled;
                eIsCurrentField     = (bUpdateAllowed && fldCurrent != null) ?
                                      CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled;
                eIsSelectable       = (bUpdateAllowed && fldCurrent != null && fldCurrent.IsSelectable) ?
                                      CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled;
                eIsAnySelectable    = (bUpdateAllowed && nodeHandler != null && nodeHandler.FieldProtection != RepPP.Toolkit.Util.FldSelectableState.SelStateE.SS_None) ?
                                      CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled;
                eIsChoiceAvailable  = (bUpdateAllowed && fldCurrent != null && fldCurrent.IsChoiceAvailable) ?
                                      CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled;
                cmdHolder.SetCmdState(sdHandlerCmd.ItemSave.ToString(),         eSave);
                cmdHolder.SetCmdState(sdHandlerCmd.ItemUndo.ToString(),         eUndo);
                if (cmdHolder.GetCmdState(sdHandlerCmd.FieldCut.ToString()) != CmdHolder.CmdStateE.Hidden) {
                    cmdHolder.SetCmdState(sdHandlerCmd.FieldCut.ToString(), eIsSelectable);
                }
                if (cmdHolder.GetCmdState(sdHandlerCmd.FieldCopy.ToString()) != CmdHolder.CmdStateE.Hidden) {
                    cmdHolder.SetCmdState(sdHandlerCmd.FieldCopy.ToString(), eIsCurrentField);
                }
                if (cmdHolder.GetCmdState(sdHandlerCmd.FieldPaste.ToString()) != CmdHolder.CmdStateE.Hidden) {
                    cmdHolder.SetCmdState(sdHandlerCmd.FieldPaste.ToString(), eIsSelectable);
                }
                if (cmdHolder.GetCmdState(sdHandlerCmd.FieldChoice.ToString()) != CmdHolder.CmdStateE.Hidden) {
                    cmdHolder.SetCmdState(sdHandlerCmd.FieldChoice.ToString(), eIsChoiceAvailable);
                }
                if (cmdHolder.GetCmdState(sdHandlerCmd.FieldHelp.ToString()) != CmdHolder.CmdStateE.Hidden) {
                    cmdHolder.SetCmdState(sdHandlerCmd.FieldHelp.ToString(), fldCurrent != null && !string.IsNullOrEmpty(fldCurrent.HelpText)  ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled);
                }
                if (cmdHolder.GetCmdState(sdHandlerCmd.FieldErase.ToString()) != CmdHolder.CmdStateE.Hidden) {
                    cmdHolder.SetCmdState(sdHandlerCmd.FieldErase.ToString(), eIsSelectable);
                }
                if (cmdHolder.GetCmdState(sdHandlerCmd.FieldDefault.ToString()) != CmdHolder.CmdStateE.Hidden) {
                    cmdHolder.SetCmdState(sdHandlerCmd.FieldDefault.ToString(), eIsSelectable);
                }
                if (cmdHolder.GetCmdState(sdHandlerCmd.FieldEraseAll.ToString()) != CmdHolder.CmdStateE.Hidden) {
                    cmdHolder.SetCmdState(sdHandlerCmd.FieldEraseAll.ToString(),    eIsAnySelectable);
                }
                if (cmdHolder.GetCmdState(sdHandlerCmd.FieldDefaultAll.ToString()) != CmdHolder.CmdStateE.Hidden) {
                    cmdHolder.SetCmdState(sdHandlerCmd.FieldDefaultAll.ToString(),  eIsAnySelectable);
                }
                nodeHandlerActive = ActiveNodeHandler;
                cmdHolder.SetCmdState(sdHandlerCmd.LineFirst.ToString(),   
                                      (nodeHandlerActive == null || !nodeHandlerActive.GetActionState(sdAction.sdActionFirst)) ? 
                                      CmdHolder.CmdStateE.Disabled : CmdHolder.CmdStateE.Enabled);
                cmdHolder.SetCmdState(sdHandlerCmd.LineLast.ToString(),
                                      (nodeHandlerActive == null || !nodeHandlerActive.GetActionState(sdAction.sdActionLast)) ? 
                                      CmdHolder.CmdStateE.Disabled : CmdHolder.CmdStateE.Enabled);
                cmdHolder.SetCmdState(sdHandlerCmd.LineNext.ToString(),
                                      (nodeHandlerActive == null || !nodeHandlerActive.GetActionState(sdAction.sdActionLast)) ? 
                                      CmdHolder.CmdStateE.Disabled : CmdHolder.CmdStateE.Enabled);
                cmdHolder.SetCmdState(sdHandlerCmd.LinePrev.ToString(),
                                      (nodeHandlerActive == null || !nodeHandlerActive.GetActionState(sdAction.sdActionFirst)) ? 
                                      CmdHolder.CmdStateE.Disabled : CmdHolder.CmdStateE.Enabled);
                cmdHolder.SetCmdState(sdHandlerCmd.LineNew.ToString(),
                                      (nodeHandlerActive == null) ?
                                      CmdHolder.CmdStateE.Disabled : CmdHolder.CmdStateE.Enabled);
                cmdHolder.SetCmdState(sdHandlerCmd.LineDelete.ToString(), 
                                      (nodeHandlerActive == null || !nodeHandlerActive.GetActionState(sdAction.sdActionDel)) ?
                                      CmdHolder.CmdStateE.Disabled : CmdHolder.CmdStateE.Enabled);
                cmdHolder.SetCmdState(sdHandlerCmd.LineDuplicate.ToString(),
                                      (nodeHandlerActive == null || !nodeHandlerActive.GetActionState(sdAction.sdActionDup)) ? 
                                      CmdHolder.CmdStateE.Disabled : CmdHolder.CmdStateE.Enabled);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// LoadUserValidationCode: Load the user validation module
        /// </summary>
        //  
        //*********************************************************     
        private void LoadUserValidationCode() {
            System.Reflection.Assembly          assembly;
            System.Type[]                       types;
            System.Type                         typeInterface;
            System.Reflection.ConstructorInfo   ctorInfo;
            IUserValidationCode                 uvc;
            string                              strFileName;

            m_arrValidationObj = new ArrayList();            
            foreach(RepPP.Module mod in m_app.Modules) {
                strFileName = mod.System + "_" + mod.Name + ".dll"; 
                strFileName = strFileName.Replace("$", "").ToLower();
                try {
                    if (System.IO.File.Exists(strFileName)) {
                        assembly = System.Reflection.Assembly.LoadFrom(strFileName);
                        if (assembly != null) {
                            types = assembly.GetExportedTypes();
                            foreach(Type type in types) {
                                try {
                                    typeInterface = type.GetInterface("IUserValidationCode");
                                    if (typeInterface != null) {
                                        ctorInfo = type.GetConstructor(new System.Type[0]);
                                        if (ctorInfo != null) {
                                            uvc = (IUserValidationCode)ctorInfo.Invoke(new object[0]);
                                            if (uvc != null) {
                                                uvc.InitializeV((IParentValidation)this);
                                                m_arrValidationObj.Add(uvc);
                                            }
                                        }
                                    }
                                } catch(Exception) {
                                }
                            }
                        }
                    }
                } catch(System.IO.FileNotFoundException) {
                }
            }        
        }

        //*********************************************************     
        //
        /// <summary>
        /// IsNodeValid:                Are node and its child valid ?
        /// </summary>
        ///
        /// <param name="node">         Rowset tree definition node to validate</param>
        /// <param name="strErr">       Returned error message</param>
        /// <remarks>
        ///  The following check are done:
        ///      All nodes must have the following attribue
        ///         Support delta information
        ///         Support field database buffer
        /// </remarks>
        //  
        //*********************************************************     
        protected void IsNodeValid(RepPP.RowsetTreeDefNode node, ref string strErr) {
            string      strInfo;

            strInfo = "The rowset tree definition node  '" + node.RowsetTreeDef.Name + "." + node.RowsetDef.Name + "' ";
            if ((node.RowsetFlag & RowsetFlag.sdDeltaInfo) == 0) {
                strErr += strInfo + "must support the delta info option\r\n";
            }
            if ((node.RowsetFlag & RowsetFlag.sdFieldDbBuf) == 0) {
                strErr += strInfo + "must support the field database buffer option\r\n";
            }
            foreach(RowsetTreeDefNode nodeChild in node.Nodes) {
                IsNodeValid(nodeChild, ref strErr);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// IsRowsetTreeValid:          Check if the rowset tree definition is valid
        /// </summary>
        /// 
        /// <param name="rowsetTreeDef">Rowset tree definition to validate</param>
        //  
        //*********************************************************     
        protected void IsRowsetTreeValid(RowsetTreeDef rowsetTreeDef) {
            string  strErr = "";
            
            if (rowsetTreeDef.Nodes.Count == 0) {
                strErr = "The rowset tree definition '" + rowsetTreeDef.Name + "' must have a root node.";
            } else {
                IsNodeValid(rowsetTreeDef.Nodes[0], ref strErr);
            }
            if (strErr.Length != 0) {
                throw new SystemException(strErr);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RowsetTreeDef:              Attached rowset tree definition node
        /// </summary>
        //  
        //*********************************************************     
        public virtual RowsetTreeDef RowsetTreeDef {
            get {
                return(m_rowsetTreeDef);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RowsetTree:           Attached rowset tree
        /// </summary>
        //  
        //*********************************************************     
        public virtual RowsetTree RowsetTree {
            get {
                return(m_rowsetTree);
            }
            set {
                if (m_rowsetTree != value) {
                    if (m_rowsetTree != null && m_rowsetTree.RowsetTreeDef != m_rowsetTreeDef) {
                        throw new SystemException("The rowset tree specified is not from '" + m_rowsetTreeDef.Name + "'");
                    }
                    m_rowsetTree = value;
                    if (m_rowsetTree == null) {
                        Rowset = null;
                        if (m_rowsetTreeUndo != null) {
                            m_rowsetTreeUndo.Delete();
                            m_rowsetTreeUndo   = null;
                        }
                    } else {
                        m_rowsetTree.AutoReset = false;
                        if (m_bUndo) {
                            m_rowsetTreeUndo   = m_rowsetTreeDef.RowsetTrees.Add();
                        }
                        m_rowsetTree.RootRowset.Select(true);
                        Rowset = m_rowsetTree.RootRowset;
                    }
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// CurrentFieldDef:           FieldDef currently selected
        /// </summary>
        //  
        //*********************************************************     
        public virtual FieldDef CurrentFieldDef {
            get {
                FieldDef fldDefRetVal;

                fldDefRetVal = m_fldDefCurrent;
                if (fldDefRetVal != null && fldDefRetVal.IsDetached){
                    fldDefRetVal = null;
                }
                return(fldDefRetVal);
            }
            set {
                if (m_fldDefCurrent != value) {
                    m_fldDefCurrent = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// UndoRowsetTree:           RowsetTree use to hold undo value
        /// </summary>
        //  
        //*********************************************************     
        public virtual RowsetTree UndoRowsetTree {
            get {
                return(m_rowsetTreeUndo);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// TakeSnapshot:           Takes a snapshot
        /// </summary>
        //  
        //*********************************************************     
        protected void TakeSnapshot() {
            RowsetTree    rowsetTree;
            RowsetTree    rowsetTreeUndo;
            string[]      arrEventNames = new string[1] { "FldChg" };
            
            rowsetTree       = RowsetTree;
            rowsetTreeUndo   = UndoRowsetTree;
            if (rowsetTree != null && rowsetTreeUndo != null) {
                try {
                    m_app.SetEventState(EventMode.sdModeNormal, false, arrEventNames);
                    rowsetTreeUndo.RootRowset.CopyFrom(rowsetTree.RootRowset, true, false);
                }
                finally {
                    m_app.SetEventState(EventMode.sdModeNormal, true, arrEventNames);
                }
                rowsetTree.RootRowset.Select(true, false);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// HandleCommand:          Handle a command coming from a menu or a toolbar
        /// </summary>
        /// <param name="eCmd">     Command Id</param>
        /// <param name="bEvent">   true to trigger event</param>
        //  
        //*********************************************************     
        public override void HandleCommand(sdHandlerCmd eCmd, bool bEvent) {
            ExecuteCmdEventArgs e;
            Boolean             bSuccess = true;

            if (m_app != null) {
                m_app.Errors.Clear();
            }
            e = new RowsetTreeDefNodeHandler.ExecuteCmdEventArgs(eCmd);
            if (bEvent) {
                OnBeforeExecuteCmd(e);
            }
            if (!e.Cancel) {
                base.HandleCommand(eCmd, false);
                switch(eCmd) {
                case sdHandlerCmd.DoNothing:
                    break;
                case sdHandlerCmd.ItemSave:
                    bSuccess = SaveLine();
                    break;
                case sdHandlerCmd.ItemUndo:
                    Undo();
                    break;
                case sdHandlerCmd.FieldChoice:
                    bSuccess = DisplayFieldChoices();
                    break;
                case sdHandlerCmd.FieldHelp:
                    bSuccess = DisplayFieldHelp();
                    break;
                case sdHandlerCmd.FieldErase:
                    bSuccess = EraseFieldValue();
                    break;
                case sdHandlerCmd.FieldDefault:
                    bSuccess = DefaultFieldValue();
                    break;
                case sdHandlerCmd.FieldEraseAll:
                    EraseAllFieldsValue();
                    break;
                case sdHandlerCmd.FieldDefaultAll:
                    DefaultAllFieldsValue();
                    break;
                case sdHandlerCmd.LineFirst:
                    ExecCmdOnActiveNodeHandler(sdHandlerCmd.ItemFirst);
                    break;
                case sdHandlerCmd.LineLast:
                    ExecCmdOnActiveNodeHandler(sdHandlerCmd.ItemLast);
                    break;
                case sdHandlerCmd.LineNext:
                    ExecCmdOnActiveNodeHandler(sdHandlerCmd.ItemNext);
                    break;
                case sdHandlerCmd.LinePrev:
                    ExecCmdOnActiveNodeHandler(sdHandlerCmd.ItemPrev);
                    break;
                case sdHandlerCmd.LineNew:
                    ExecCmdOnActiveNodeHandler(sdHandlerCmd.ItemNew);
                    break;
                case sdHandlerCmd.LineDuplicate:
                    ExecCmdOnActiveNodeHandler(sdHandlerCmd.ItemDup);
                    break;
                case sdHandlerCmd.LineDelete:
                    ExecCmdOnActiveNodeHandler(sdHandlerCmd.ItemDel);
                    break;
                }
                if (bEvent && bSuccess) {
                    OnAfterExecuteCmd(e);
                }
            }
            TriggerErrors();
        }

        //*********************************************************     
        //
        /// <summary>
        /// Refresh:            Refresh the control
        /// </summary>
        //  
        //*********************************************************     
        public virtual void Refresh() {
            LineListHandler.Refresh();
            OnLineSelected(new LineSelectedEventArgs(-1, CurrentLine));
        }

        //*********************************************************     
        //
        /// <summary>
        /// ReadFromDb:                     Read from database
        /// </summary>
        /// <param name="strWhereClause">   Where clause</param>
        /// <param name="arrKeyValues">     Array of key values</param>
        /// <returns>
        ///  Error code
        /// </returns>
        //
        //*********************************************************     
        protected int ReadFromDb(string strWhereClause, string[] arrKeyValues) {
            int                     iRetVal;
            RowsetTreeDefNode       node;
            RowsetTree              rowsetTree;
            Rowset                  rowsetRoot;
            string                  strOldWhereClause;
            int                     iIndex;
            ITransactionDataAccess  dataAccess;
            
            node        = RowsetTreeDefNode;
            rowsetTree  = RowsetTree;
            dataAccess  = DataAccess;
            if (node.SelectSqlCommand == null || node.SelectSqlCommand.Length == 0) {
                node.RowsetTreeDef.BuildSqlCommand(true);
            }
            strOldWhereClause = node.GetSelectSqlWhere();
            node.SetSelectSqlWhere(strWhereClause);
            try {
                if (dataAccess != null) {
                    iRetVal = dataAccess.LoadTransactionData(rowsetTree, arrKeyValues);
                } else {
                    if (arrKeyValues != null && arrKeyValues.Length > 0) {
                        rowsetRoot = rowsetTree.RootRowset;
                        rowsetRoot.AddBlankLine();
                        iIndex = 0;
                        foreach (FieldDef fldDef in rowsetRoot.RowsetDef.FieldDefs) {
                            if (fldDef.IsPrimaryKey) {
                                fldDef.Value = arrKeyValues[iIndex++];
                            }
                        }
                    }
                    iRetVal = rowsetTree.ReadFromDb(false);
                }
            } finally {
                node.SetSelectSqlWhere(strOldWhereClause);
            }
            if (iRetVal == 0) {
                TakeSnapshot();
                Refresh();
            }
            return(iRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// IsDirty:                Check if the card is dirty
        /// </summary>
        /// <returns>
        ///  true:                  Yes
        ///  false:                 No
        /// </returns>
        //
        //*********************************************************     
        public override bool IsDirty() {
            bool              bRetVal;
            RepPP.RowsetTree  rowsetTree;
            
            rowsetTree = RowsetTree;
            if (rowsetTree == null || rowsetTree.RootRowset.LineCount == 0) {
                bRetVal = false;
            } else {
                bRetVal = rowsetTree.IsModified;
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// SaveLine:                           Save changes for the current line
        /// </summary>
        /// <returns>
        ///  true:                              Changed saved or no changes
        ///  false:                             Not saved. User refused or error
        /// </returns>
        //
        //*********************************************************     
        public abstract bool SaveLine();

        //*********************************************************     
        //
        /// <summary>
        /// AskToSave:                          Save changes if any after asking user
        /// </summary>
        /// <param name="bEnsureFatherExist">   If an operation on a child of this item is done and this item is new, we must not
        ///                                     accept to not save it.</param>
        /// <returns>
        ///  true:                              Changed saved or no changes
        ///  false:                             Not saved. User refused or error
        /// </returns>
        //
        //*********************************************************     
        public virtual bool AskToSave(bool bEnsureFatherExist) {
            bool            bRetVal;
            DialogResult    eVal;
            string          strMsg;
            Rowset          rowset;

            if (IsDirty()) {
                rowset  = Rowset;
                strMsg = m_app.GetAtomText("$FCH2", RepPP.AtomType.sdAtomMsg);
                eVal   = MessageBox.Show(strMsg,
                                         m_app.ProgramName,
                                         (bEnsureFatherExist && rowset.LineState == LineState.sdLineNew) ?
                                         MessageBoxButtons.OKCancel : MessageBoxButtons.YesNoCancel,
                                         MessageBoxIcon.Question);
                switch(eVal) {
                case DialogResult.Cancel:
                    bRetVal = false;
                    break;
                case DialogResult.Yes:
                case DialogResult.OK:
                    bRetVal = SaveAllLines();
                    break;
                default:
                    if (Rowset.LineState == RepPP.LineState.sdLineNew) {
                        bRetVal = DeleteCurrentLine();
                    } else {
                        bRetVal = true;
                    }
                    break;
                }
            } else {
                bRetVal = true;
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// AskToSave:                          Save changes if any after asking user
        /// </summary>
        /// <returns>
        ///  true:                              Changed saved or no changes
        ///  false:                             Not saved. User refused or error
        /// </returns>
        //
        //*********************************************************     
        public virtual bool AskToSave() {
            return(AskToSave(false));
        }

        //*********************************************************     
        //
        /// <summary>
        /// UpdateAllLinesToDb:     Update all lines to DB
        /// </summary>
        /// <returns>
        ///     Zero                No error
        ///     Non-Zero            Error
        /// </returns>
        //  
        //*********************************************************     
        protected virtual int UpdateAllLinesToDb() {
            int                     iRetVal;
            RowsetState             eReason;
            RowsetTree              rowsetTree;
            string                  strError;
            ITransactionDataAccess  dataAccess;

            rowsetTree  = RowsetTree;
            dataAccess  = DataAccess;
            if (dataAccess != null) {
                iRetVal = dataAccess.SaveTransactionData(rowsetTree, out strError);
            } else {
                iRetVal = rowsetTree.UpdateToDb(true);
            }
            if (iRetVal == 0) {
                iRetVal = AfterSave() ? 0: (int)RepPP.ErrorCode.sdRetPrgRefused;
            }
            if (iRetVal == 0) {
                eReason = rowsetTree.RootRowset.State;
                if (AutoCommit) {
                    m_app.DataConnection.Commit();
                    rowsetTree.ResetState();
                }
                AfterCommit(eReason);
            } else {
                m_app.DataConnection.Rollback();
            }
            return(iRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// BuildPKWhereClause:             Build the WHERE clause for the PK
        /// </summary>
        /// <param name="rowset">           Rowset containing the values</param>
        /// <returns>
        /// WHERE clause
        /// </returns>
        //
        //******************************************************************
	    private string BuildPKWhereClause(RepPP.Rowset rowset) {
	        string  strRetVal;
	        
	        strRetVal = "";
	        if (rowset.FatherRowset != null) {
	            strRetVal = BuildPKWhereClause(rowset.FatherRowset);
	        }
	        foreach (RepPP.FieldDef fldDef in rowset.RowsetDef.FieldDefs) {
	            if (fldDef.IsPrimaryKey) {
	                if (strRetVal == String.Empty) {
	                    strRetVal = " WHERE ";
	                } else {
	                    strRetVal += " AND ";
	                }
	                strRetVal += fldDef.ColName + "=:" + fldDef.RowsetDef.Name + "." + fldDef.Name;
	            }
	        }
	        return(strRetVal);
	    }

        //******************************************************************
        //
        /// <summary>
        /// CheckIfDataChangedSinceRead:    Checks if the data has changed in the DB since it has been read
        /// </summary>
        /// <param name="rowset">           Rowset containing the values</param>
        /// <param name="fldDefStamp">      Field def containing the date stamp</param>
        /// <param name="bDisplayMsg">      Flag to display a message box</param>
        /// <returns>
        /// true if changed, false if not
        /// </returns>
        //
        //******************************************************************
	    public virtual bool CheckIfDataChangedSinceRead(Rowset rowset, FieldDef fldDefStamp, bool bDisplayMsg) {
	        bool                bRetVal;
	        Connection          conn;
	        string              strSqlCmd;
	        string              strStamping;
	        string              strMsg;
	        SqlCursor           sqlCursor;
	        
	        if (rowset.RowsetTreeDefNode != null) {
	            conn = rowset.RowsetTreeDefNode.Connection;
	        } else {
	            conn = rowset.Application.DataConnection;
	        }
	        strSqlCmd = "SELECT " + fldDefStamp.ColName + " FROM " + fldDefStamp.TableName + BuildPKWhereClause(rowset) + " FOR UPDATE";
	        if (conn.Execute(strSqlCmd, RepPP.SqlOptimization.sdOptCursor, out sqlCursor) == 0) {
	            using(sqlCursor) {
	                if (sqlCursor.Next() == 0) {
	                    strStamping = sqlCursor.GetColumnValue(0).Replace('-','/');
	                    bRetVal     = (strStamping != fldDefStamp.Value);
	                } else {
	                    bRetVal     = true;
	                }
	            }
	        } else {
	            throw new System.ApplicationException(conn.ErrorMessage);
	        }
	        if (bRetVal && bDisplayMsg) {
	            if (rowset.Application.IsAtomExist("$dbdatachg", RepPP.AtomType.sdAtomErr)) {
	                strMsg = rowset.Application.GetAtomText("$dbdatachg", RepPP.AtomType.sdAtomErr);
	            } else {
	                strMsg = "The values in the database were modified by another user since they were fetched. The update cannot be done. Please fetch the data again and reapply your modifications.";
	            }
	            MessageBox.Show(strMsg, "", MessageBoxButtons.OK, MessageBoxIcon.Error);
	        }
	        return(bRetVal);
	    }

        //*********************************************************     
        //
        /// <summary>
        /// AfterSave:              Called after a save is done.
        /// </summary>
        /// <returns>
        ///     false:              Cancel
        ///     true:               Continue
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool AfterSave() {
            return(true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// AfterCommit:            Called after a database operation is done.
        /// </summary>
        /// <param name="eReason">  Reason of the commit</param>
        //  
        //*********************************************************     
        public virtual void AfterCommit(RepPP.RowsetState eReason) {
        }

        //*********************************************************     
        //
        /// <summary>
        /// SaveAllLines:           Save all dirty lines to DB
        /// </summary>
        /// <returns>
        ///     false:              Not saved
        ///     true:               Saved
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool SaveAllLines() {
            bool    bRetVal = false;
            int     iLinePos;

            //TODO: Improve error reporting...
            //      If error from database, full error report please.
            if (Rowset != null && SBCurrentLine != -1) {                                
                if (Rowset.LineState == LineState.sdLineDeleted || 
                    ValidateCurLine()) {
                    iLinePos = CurrentLine;
                    if (UpdateAllLinesToDb() == 0) {
                        bRetVal = true;
                        TakeSnapshot();
                    } else {
                        bRetVal = false;
                    }
                    OnLineSelected(new LineSelectedEventArgs(iLinePos, CurrentLine));
                }
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Undo:                   Undo a logical operation
        /// </summary>
        //  
        //*********************************************************     
        public virtual void Undo() {
            RowsetTree  rowsetTree;
            RowsetTree  rowsetTreeUndo;
            Rowset      rowset;
            Rowset      rowsetUndo;
            int         iLinePos;

            rowsetTree       = RowsetTree;
            rowsetTreeUndo   = UndoRowsetTree;
            if (rowsetTree != null && rowsetTreeUndo != null) {
                // GPC4806: End edit the datagrid before the undo is done
                CommitAllEdit(this);
                rowset     = rowsetTree.RootRowset;
                rowsetUndo = rowsetTreeUndo.RootRowset;
                iLinePos  = CurrentLine;
                if (iLinePos != -1) {
                    if (rowset.LineState == LineState.sdLineDeleted) {
                        if (rowset.RowsetDef.IsAnsChg()) {
                            rowset.LineState = LineState.sdLineChanged;
                        } else {                            
                            rowset.LineState = (rowset.IsLineChildModified()) ? LineState.sdLineChanged : LineState.sdLineOld;
                        }
                        OnLineSelected(new LineSelectedEventArgs(iLinePos, CurrentLine));
                    } else if (rowset.IsLineModified()) {
                        rowsetUndo.SelectLine(iLinePos, true, false);
                        rowset.CopyLineFrom(rowsetUndo, CopyFieldOpt.sdResetAnsChg | CopyFieldOpt.sdSetValid, true);
                        rowset.LineState = LineState.sdLineOld;
                        rowset.Select(true);
                        rowsetTree.RowsetTreeDef.Refresh();
                        OnLineSelected(new LineSelectedEventArgs(iLinePos, CurrentLine));
                    }
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// DisplayFieldHelp        Display the help text of a field
        /// </summary>
        /// <param name="fldDef">   Field definition</param>
        /// <returns>
        ///      True for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public virtual bool DisplayFieldHelp(FieldDef fldDef) {
            bool    bRetVal;
            string  strTitle;
            string  strHelp;

            bRetVal = false;
            if (fldDef != null) {
                strTitle = fldDef.Prompt;
                strHelp  = fldDef.HelpText;
                if (strHelp.Length != 0) {
                    bRetVal = true;
                    MessageBox.Show(strHelp, strTitle);
                }
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// DisplayFieldHelp:    Display the help text of the current field
        /// </summary>
        /// <returns>
        ///      true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public virtual bool DisplayFieldHelp() {
            return(DisplayFieldHelp(CurrentFieldDef));
        }

        //*********************************************************
        //
        /// <summary>
        /// AdjustFieldNodeState:   Touch the field node state
        /// </summary>
        /// <param name="fldDef">   Field definition</param>
        //
        //*********************************************************     
        private void AdjustFieldNodeState(FieldDef fldDef) {
            RowsetTreeDefNodeHandler    nodeHandler;

            if (fldDef != null) {            
                nodeHandler = FindRowsetTreeDefNodeHandler(fldDef.RowsetDef);
                if (nodeHandler != null) {
                    nodeHandler.RaiseAdjustState();
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// DisplayFieldChoices:    Display the list of choices of a field
        /// </summary>
        /// <param name="fldDef">   Field definition</param>
        /// <returns>
        ///      false:          No choice list proposed
        ///      true:           Choice list proposed
        /// </returns>
        //
        //*********************************************************     
        public virtual bool DisplayFieldChoices(FieldDef fldDef) {
            bool    bRetVal = false;
            string  strTitle;

            if (fldDef != null && fldDef.IsSelectable) {
                strTitle = fldDef.Prompt;
                bRetVal  = true;
                fldDef.PopupChoiceList(strTitle);
                AdjustFieldNodeState(fldDef);
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        ///  DisplayFieldChoices:   Display the list of choices of the current field
        /// </summary>
        /// <returns>
        ///      false:             No choice list proposed
        ///      true:              Choice list proposed
        /// </returns>
        //
        //*********************************************************     
        public virtual bool DisplayFieldChoices() {
            return(DisplayFieldChoices(CurrentFieldDef));
        }

        //*********************************************************     
        //
        /// <summary>
        /// EraseFieldValue:        Erase value of the field if permitted.
        /// </summary>
        ///  <param name="fldDef">  Field to erase</param>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public virtual bool EraseFieldValue(FieldDef fldDef) {
            bool    bRetVal = false;
            
            if (fldDef != null && fldDef.IsSelectable) {
                bRetVal = true;
                fldDef.SetValue("");
                AdjustFieldNodeState(fldDef);
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// EraseFieldValue:     Erase value of the current field
        /// </summary>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public virtual bool EraseFieldValue() {
            return(EraseFieldValue(CurrentFieldDef));
        }

        //*********************************************************     
        //
        /// <summary>
        /// DefaultFieldValue:      Set the value of the field to its default value if permitted.
        /// </summary>
        ///  <param name="fldDef">  Field definition</param>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public virtual bool DefaultFieldValue(FieldDef fldDef) {
            bool    bRetVal = false;
            
            if (fldDef != null && fldDef.IsSelectable) {
                bRetVal = true;
                fldDef.SetValue(fldDef.DefaultValue);
                AdjustFieldNodeState(fldDef);
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// DefaultFieldValue:   Set the value of the current field to its default value if permitted.
        /// </summary>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public virtual bool DefaultFieldValue() {
            return(DefaultFieldValue(CurrentFieldDef));
        }

        //*********************************************************     
        //
        /// <summary>
        ///  EraseAllFieldsValue:    Erase the value of the all fields of the current rowset.
        /// </summary>
        //
        //*********************************************************     
        public virtual void EraseAllFieldsValue() {
            FieldDef   fldCurrent;
            
            fldCurrent = CurrentFieldDef;
            if (fldCurrent != null) {
                foreach(FieldDef fldDef in fldCurrent.RowsetDef.FieldDefs) {
                    EraseFieldValue(fldDef);
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        ///  DefaultAllFieldsValue:    Set the value of the all fields of the current rowset to their default value.
        /// </summary>
        //
        //*********************************************************     
        public virtual void DefaultAllFieldsValue() {
            RowsetTreeDefNodeHandler    nodeHandler;
            FieldDef                    fldCurrent;
            
            fldCurrent = CurrentFieldDef;
            if (fldCurrent != null) {
                foreach(FieldDef fldDef in fldCurrent.RowsetDef.FieldDefs) {
                    DefaultFieldValue(fldDef);
                }
                nodeHandler = FindRowsetTreeDefNodeHandler(fldCurrent.RowsetDef);
                if (nodeHandler != null) {
                    nodeHandler.OnDataConversion(new RowsetTreeDefNodeHandler.DataConversionEventArgs(nodeHandler, DataConversionTypeE.DC_Decode));
                }
            }
        }        

        //*********************************************************     
        //
        /// <summary>
        /// ActiveNodeHandler:     Gets the active node handler
        /// </summary>
        //
        //*********************************************************     
        public virtual RowsetTreeDefNodeHandler ActiveNodeHandler {
            get {
                RowsetTreeDefNodeHandler    nodeHandler;
                RepPP.Rowset                rowsetActive;
                
                rowsetActive = m_app.ActiveRowset;
                nodeHandler = (rowsetActive != null) ? FindRowsetTreeDefNodeHandler(rowsetActive.RowsetTreeDefNode) : null;
                return(nodeHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// ExecCmdOnActiveNodeHandler: Execute the specified command on the active node handler if any
        /// </summary>
        /// <param name="eCmd">         Command to execute on the active node handler</param>
        //
        //*********************************************************     
        public virtual void ExecCmdOnActiveNodeHandler(sdHandlerCmd eCmd) {
            RowsetTreeDefNodeHandler    nodeActive;
            
            nodeActive = ActiveNodeHandler;
            if (nodeActive != null) {
                nodeActive.HandleCommand(eCmd, true);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// GetActionState:         Get the specified action state
        /// </summary>
        /// <param name="eAction">  Action</param>
        /// <returns>
        ///      false:          Disabled
        ///      true:           Enabled
        /// </returns>
        //  
        //*********************************************************     
        public override bool GetActionState(sdAction eAction) {
            bool        bRetVal;
            RowsetTree  rowsetTree;

            rowsetTree = RowsetTree;
            switch(eAction) {
            case sdAction.sdActionSave:
                bRetVal = IsDirty();
                break;                    
            case sdAction.sdActionUndo:
                if (IsDirty()) {
                    bRetVal = (rowsetTree.RootRowset.LineState != RepPP.LineState.sdLineNew);
                } else {
                    bRetVal = false;
                }
                break;
            default:
                bRetVal = base.GetActionState(eAction);
                break;
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// TriggerResetFieldError: Reset the last error message from field validation
        /// </summary>
        //  
        //*********************************************************     
        public override void TriggerResetFieldError() {
            OnResetFieldError(EventArgs.Empty);
        }

        //*********************************************************     
        //
        /// <summary>
        ///  GetApplication:    Get the rep++ application object
        /// </summary>
        //
        //*********************************************************     
        public virtual IApplication GetApplication() {
            return(m_app);
        }

        //*********************************************************     
        //
        /// <summary>
        ///  OnResetFieldError:  Trigger the ResetFieldError event
        /// </summary>
        //  
        //*********************************************************     
        public virtual void OnResetFieldError(System.EventArgs e) {
            if (ResetFieldError != null) {
                ResetFieldError(this, e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        ///  OnAdjustFieldState: Trigger the AdjustFieldState event
        /// </summary>
        //  
        //*********************************************************     
        public virtual void OnAdjustFieldState(System.EventArgs e) {
            if (AdjustFieldState != null) {
                AdjustFieldState(this, e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        ///  FldEnterSink:       Called when a field is selected
        /// </summary>
        //
        //*********************************************************     
        private void FldEnterSink(object sender, FieldEventArgs e) {
            OnResetFieldError(System.EventArgs.Empty);
        }            

        //*********************************************************     
        //
        /// <summary>
        ///  FldCtlLoseFocusSink:    Called when a field control loose focus
        /// </summary>
        //
        //*********************************************************     
        private void FldCtlLoseFocusSink(object sender, FieldEventArgs e) {
            CurrentFieldDef = e.FieldDef;
            OnAdjustFieldState(System.EventArgs.Empty);
        }            

        //*********************************************************     
        //
        /// <summary>
        ///  FldCtlRecFocusSink:     Called when a field control receive focus
        /// </summary>
        //
        //*********************************************************     
        private void FldCtlRecFocusSink(object sender, FieldEventArgs e) {
            CurrentFieldDef = e.FieldDef;
            SetCommandHolderState();
            OnAdjustFieldState(System.EventArgs.Empty);
        }

        //*********************************************************     
        //
        /// <summary>
        ///  DbAfterReadLineSink:    Called when a line has been read
        /// </summary>
        //
        //*********************************************************     
        private void DbAfterReadLineSink(object sender, RowsetEventArgs e) {
            RowsetTreeDefNodeHandler    nodeHandler;
            
            if (e.Rowset != null) {
                if (e.Rowset.RowsetTree == RowsetTree) {
                    nodeHandler = FindRowsetTreeDefNodeHandler(e.Rowset.RowsetDef);
                    if (nodeHandler != null) {
                        nodeHandler.OnDataConversion(new RowsetTreeDefNodeHandler.DataConversionEventArgs(nodeHandler, DataConversionTypeE.DC_Decode));
                    }
                }
            }
        }
	} // Class RootRowsetTreeDefNodeHandler
} // Namespace
