using System;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {

    //******************************************************************
    //
    /// <summary>
    /// SCSingleChildEditorFactory: Default implementation of the class factory for top editor
    /// having only one child. If you want to pass more info to the child constructor or if the top
    /// editor has more than one child editor, you must define your own class factory implementing
    /// the ISBChildEditorFactory interface. This implementation use reflection to create the
    /// specified child editor object.
    /// </summary>
    //
    //******************************************************************
    public class SingleChildEditorFactory  : ISBChildEditorFactory {
        /// <summary>Father editor</summary>
        private frmSBEditorBase         m_fatherEditor;
        /// <summary>Name of the rowset tree indentifying the transaction</summary>
        private string                  m_strRowsetTreeName;
        /// <summary>Type of the child editor the factory must creates</summary>
        private Type                    m_typeChildEditor;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                           Build a new instance of the class factory
        /// </summary>
        /// <param name="fatherEditor">     Father editor</param>
        /// <param name="strRowsetTreeName">Name of the rowset tree defining the transaction</param>
        /// <param name="typeChildEditor">  Type of the child editor to create</param>
        //
        //******************************************************************
        public SingleChildEditorFactory(frmSBEditorBase     fatherEditor,
                                        string              strRowsetTreeName,
                                        Type                typeChildEditor) {
            m_fatherEditor      = fatherEditor;
            m_strRowsetTreeName = strRowsetTreeName;
            m_typeChildEditor   = typeChildEditor;
        }

        //******************************************************************
        //
        /// <summary>
        /// CreateChildEditor           Creates a Selection Buffer Child Editor
        ///                             of the specified type
        /// </summary>
        /// <param name="iEditorType">  Type of the Editor</param>
        /// <returns>
        /// A reference to the selection buffer child editor
        /// </returns>
        //
        //******************************************************************
        public frmSBChildEditor CreateChildEditor(int iEditorType) {
            System.Reflection.ConstructorInfo   ctorInfo;
            
            ctorInfo = m_typeChildEditor.GetConstructor(new Type[] { typeof(frmSBEditorBase), 
                                                                     typeof(string) });
            if (ctorInfo == null) {
                throw new System.ApplicationException("The type " + m_typeChildEditor.Name + " must contains a constructor with the proper signature!");
            }
            return(ctorInfo.Invoke(new object[] { m_fatherEditor,
                                                  m_strRowsetTreeName }) as frmSBChildEditor);
        }
    }
}
