using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
    /// <summary>
    /// Default toolbar to handle the RowsetTreeDefNodeHandler command
    /// </summary>
    public class ToolbarRowset : ToolStrip {
        private enum sdStateImageNumber {
            /// <summary>Index of the image for the OLD state</summary>
            sdImageOld = 7,
            /// <summary>Index of the image for the NEW state</summary>
            sdImageNew = 9,
            /// <summary>Index of the image for the CHANGED state</summary>
            sdImageChanged = 8,
            /// <summary>Index of the image for the DELETED state</summary>
            sdImageDeleted = 10
        };
        /// <summary>The available ToolBar button sizes</summary>
        public enum ToolBarSize {
            /// <summary>Use small buttons in the toolbar</summary>
            Small       = 0,
            /// <summary>Use large buttons in the toolbar</summary>
            Big         = 2
        };
        
        /// <summary>Attached node handler</summary>
        private  RowsetTreeDefNodeHandler                   m_nodeHandler;
        /// <summary>Size of the buttons used in the toolbar</summary>
        private  ToolBarSize                                m_eButtonSize;
        /// <summary>String to display the position and the row count to the user</summary>
        private string                                      m_strPos;
        /// <summary>Button First Item</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butFirst;
        /// <summary>Button Previous Item</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butPrevious;
        /// <summary>Pseudo-button use to display the position</summary>
        public  System.Windows.Forms.ToolStripLabel         butPos;
        /// <summary>Button used to display the item state</summary>
        public System.Windows.Forms.ToolStripLabel          butState;
        /// <summary>Button Next Item</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butNext;
        /// <summary>Button Last Item</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butLast;
        /// <summary>Separator</summary>
        public  System.Windows.Forms.ToolStripSeparator     butSep1;
        /// <summary>Button New Item</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butNew;
        /// <summary>Button Duplicate Item</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butDuplicate;
        /// <summary>Button Delete Item</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butDelete;
        private System.Windows.Forms.ImageList              imageListSmall;
        private System.Windows.Forms.ImageList              imageListBig;
        private System.ComponentModel.IContainer            components;

        //*********************************************************	    
        //
        /// <summary>
        /// ctor:           Class constructor
        /// </summary>
        //
        //*********************************************************	    
        public ToolbarRowset() {
            m_strPos         = "";
            InitializeComponent();
            m_eButtonSize    = ToolBarSize.Small;
            m_nodeHandler    = null;
        }

        //*********************************************************     
        //
        /// <summary>
        /// Dispose:                    Cleanup the object
        /// </summary>
        /// <param name="bDisposing">   true if dispose, false if GC</param>
        //  
        //*********************************************************     
        protected override void Dispose(bool bDisposing) {
            if (bDisposing) {
                if (m_nodeHandler != null) {
                    UnregisterTrigger();
                    m_nodeHandler.AdjustState -= new System.EventHandler(AdjustStateSink);
                }
                if (components != null) {
                    components.Dispose();
                }
            }
            base.Dispose(bDisposing);
        }

        //*********************************************************	    
        //
        /// <summary>
        ///	UnregisterTrigger:          Unregister the button to the command holder
        /// </summary>
        //	
        //*********************************************************	    
        private void UnregisterTrigger() {
            CmdHolder   cmdHolder;
            
            if (m_nodeHandler != null) {
                cmdHolder = m_nodeHandler.CommandHolder;
                if (cmdHolder != null) {
                    cmdHolder.RemoveTrigger(sdHandlerCmd.ItemFirst.ToString(),      new object[] { butFirst });
                    cmdHolder.RemoveTrigger(sdHandlerCmd.ItemPrev.ToString(),       new Object[] { butPrevious });
                    cmdHolder.RemoveTrigger(sdHandlerCmd.ItemNext.ToString(),       new Object[] { butNext });
                    cmdHolder.RemoveTrigger(sdHandlerCmd.ItemLast.ToString(),       new Object[] { butLast });
                    cmdHolder.RemoveTrigger(sdHandlerCmd.ItemNew.ToString(),        new Object[] { butNew });
                    cmdHolder.RemoveTrigger(sdHandlerCmd.ItemDup.ToString(),        new Object[] { butDuplicate });
                    cmdHolder.RemoveTrigger(sdHandlerCmd.ItemDel.ToString(),        new Object[] { butDelete });
                }
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        ///	RegisterTrigger:        Register the button to the command holder
        /// </summary>
        //	
        //*********************************************************	    
        private void RegisterTrigger() {
            CmdHolder cmdHolder;
            
            cmdHolder = m_nodeHandler.CommandHolder;
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemFirst.ToString(), new object[] { butFirst });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemPrev.ToString(), new Object[] { butPrevious });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemNext.ToString(), new Object[] { butNext });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemLast.ToString(), new Object[] { butLast });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemNew.ToString(), new Object[] { butNew });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemDup.ToString(), new Object[] { butDuplicate });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemDel.ToString(), new Object[] { butDelete });
        }

        //*********************************************************	    
        //
        /// <summary>
        ///	RowsetTreeDefNodeHandler:       Gets the SCCardGroup object
        /// </summary>
        //	
        //*********************************************************	    
        public virtual RowsetTreeDefNodeHandler NodeHandler {
            get {
                return(m_nodeHandler);
            }
            set {
                if (m_nodeHandler != value) {
                    if (m_nodeHandler != null) {
                        m_nodeHandler.AdjustState -= new System.EventHandler(AdjustStateSink);
                        UnregisterTrigger();
                    }
                    m_nodeHandler = value;
                    if (m_nodeHandler != null) {
                        m_nodeHandler.AdjustState += new System.EventHandler(AdjustStateSink);
                        if (m_nodeHandler.CommandHolder == null) {
                            m_nodeHandler.CommandHolder = new CmdHolder();
                        }
                        RegisterTrigger();
                        UpdateToolbarState();
                    }
                }
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        ///	ToolBarButtonSize:      Button size
        /// </summary>
        //	
        //*********************************************************	    
        public ToolBarSize ToolBarButtonSize {
            get {
                return(m_eButtonSize);
            }
            set {
                if (m_eButtonSize != value) {
                    switch(value) {
                    case ToolBarSize.Small:
                        m_eButtonSize = value;
                        ImageList     = imageListSmall;
                        butPos.Text   = "                        .";
                        break;
                    case ToolBarSize.Big:
                        m_eButtonSize = value;
                        ImageList     = imageListBig;
                        butPos.Text   = "                                   .";
                        break;
                    }
                    Refresh();
                }
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        ///	UpdateToolbarState: Update the toolbar state
        /// </summary>
        //	
        //*********************************************************	    
        private void UpdateToolbarState() {
            int                 iLinePos;
            int                 iLineCount;
            string              strAtomName = String.Empty;
            RepPP.Application   app;

            iLinePos   = m_nodeHandler.SBCurrentLine;
            iLineCount = m_nodeHandler.SBLineCount;
            switch (m_nodeHandler.GetItemState()) {
            case RepPP.LineState.sdLineOld:
                butState.ImageIndex  = (int)sdStateImageNumber.sdImageOld;
                strAtomName          = "UNCHANGED_STATE";
                break;
            case RepPP.LineState.sdLineDeleted:
                butState.ImageIndex  = (int)sdStateImageNumber.sdImageDeleted;
                strAtomName          = "DELETED_STATE";
                break;
            case RepPP.LineState.sdLineNew:
                butState.ImageIndex  = (int)sdStateImageNumber.sdImageNew;
                strAtomName          = "NEW_STATE";
                break;
            case RepPP.LineState.sdLineChanged:
                butState.ImageIndex  = (int)sdStateImageNumber.sdImageChanged;
                strAtomName          = "MODIFIED_STATE";
                break;
            }
            m_strPos = "[" + (iLinePos + 1) + " / " + iLineCount + "]";
            if (butPos.Text !=  m_strPos && m_strPos != "") {
                butPos.Text = m_strPos;
            }
            app = RepPP.Application.GetApplication();
            if (app != null && app.IsAtomExist(strAtomName, RepPP.AtomType.sdAtomMsg)) {
                butState.ToolTipText = app.GetAtomText(strAtomName, RepPP.AtomType.sdAtomMsg);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        ///	AdjustStateSink:        Adjust the state
        /// </summary>
        ///
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Argument</param>
        //	
        //*********************************************************	    
        protected virtual void AdjustStateSink(object sender, System.EventArgs e) {
            UpdateToolbarState();
        }

        //*********************************************************	    
        //
        /// <summary>
        /// DrawPos:                Draw the position
        /// </summary>
        //	
        //*********************************************************	    
        private void DrawPos() {
            System.Drawing.Font font;

            butPos.Text = m_strPos;
            font        = (m_eButtonSize == ToolBarSize.Big) ? font = new Font("Arial", 13) :
                                                               font = new Font("Arial", 9);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// WndProc:            Override the window proc to redraw the disable button correctly.
        /// </summary>
        /// <param name="msg">  Message</param>
        //	
        //*********************************************************	 
        protected override void WndProc(ref System.Windows.Forms.Message msg) {
            base.WndProc(ref msg);
            if (msg.Msg == 15) {
                if (Items[4].Visible) {
                    DrawPos();
                }
            }
        }

        #region Component Designer generated code
        /// <summary> 
        /// Required method for Designer support - do not modify 
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent() {
            this.components = new System.ComponentModel.Container();
            System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(ToolbarRowset));
            this.butFirst = new System.Windows.Forms.ToolStripMenuItem();
            this.butPrevious = new System.Windows.Forms.ToolStripMenuItem();
            this.butPos = new System.Windows.Forms.ToolStripLabel();
            this.butState = new System.Windows.Forms.ToolStripLabel();
            this.butNext = new System.Windows.Forms.ToolStripMenuItem();
            this.butLast = new System.Windows.Forms.ToolStripMenuItem();
            this.butSep1 = new System.Windows.Forms.ToolStripSeparator();
            this.butNew = new System.Windows.Forms.ToolStripMenuItem();
            this.butDuplicate = new System.Windows.Forms.ToolStripMenuItem();
            this.butDelete = new System.Windows.Forms.ToolStripMenuItem();
            this.imageListSmall = new System.Windows.Forms.ImageList(this.components);
            this.imageListBig = new System.Windows.Forms.ImageList(this.components);

            // 
            // butFirst
            // 
            this.butFirst.AutoSize = false;
            this.butFirst.Size = new Size(22, 20);
            this.butFirst.ImageIndex = 0;


            this.butFirst.Tag = "100";
            this.butFirst.ToolTipText = "First";
            // 
            // butPrevious
            // 
            this.butPrevious.AutoSize = false;
            this.butPrevious.Size = new Size(22, 20);
            this.butPrevious.ImageIndex = 1;


            this.butPrevious.Tag = "101";
            this.butPrevious.ToolTipText = "Previous";
            // 
            // butPos
            // 
            this.butPos.Text = "                        .";
            // 
            // butState
            // 
            this.butState.Text = "";
            this.butState.AutoSize = false;
            this.butState.Size = new Size(22, 20);
            this.butState.ImageIndex = 7;


            // 
            // butNext
            // 
            this.butNext.AutoSize = false;
            this.butNext.Size = new Size(22, 20);
            this.butNext.ImageIndex = 2;


            this.butNext.Tag = "102";
            this.butNext.ToolTipText = "Next";
            // 
            // butLast
            // 
            this.butLast.AutoSize = false;
            this.butLast.Size = new Size(22, 20);
            this.butLast.ImageIndex = 3;


            this.butLast.Tag = "103";
            this.butLast.ToolTipText = "Last";
            // 
            // butNew
            // 
            this.butNew.AutoSize = false;
            this.butNew.Size = new Size(22, 20);
            this.butNew.ImageIndex = 4;


            this.butNew.Tag = "104";
            this.butNew.ToolTipText = "New";
            // 
            // butDuplicate
            // 
            this.butDuplicate.AutoSize = false;
            this.butDuplicate.Size = new Size(22, 20);
            this.butDuplicate.ImageIndex = 5;


            this.butDuplicate.Tag = "106";
            this.butDuplicate.ToolTipText = "Duplicate";
            // 
            // butDelete
            // 
            this.butDelete.AutoSize = false;
            this.butDelete.Size = new Size(22, 20);
            this.butDelete.ImageIndex = 6;


            this.butDelete.Tag = "105";
            this.butDelete.ToolTipText = "Delete";
            // 
            // imageListSmall
            // 
            this.imageListSmall.ColorDepth = System.Windows.Forms.ColorDepth.Depth24Bit;
            this.imageListSmall.ImageSize = new System.Drawing.Size(16, 16);
            this.imageListSmall.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageListSmall.ImageStream")));
            this.imageListSmall.TransparentColor = System.Drawing.Color.Magenta;
            // 
            // imageListBig
            // 
            this.imageListBig.ColorDepth = System.Windows.Forms.ColorDepth.Depth24Bit;
            this.imageListBig.ImageSize = new System.Drawing.Size(32, 32);
            this.imageListBig.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageListBig.ImageStream")));
            this.imageListBig.TransparentColor = System.Drawing.Color.Magenta;
            // 
            // ToolbarRowset
            // 
            this.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                                                                               this.butFirst,
                                                                               this.butPrevious,
                                                                               this.butNext,
                                                                               this.butLast,
                                                                               this.butPos,
                                                                               this.butState,
                                                                               this.butSep1,
                                                                               this.butNew,
                                                                               this.butDuplicate,
                                                                               this.butDelete});
            this.ImageList = this.imageListSmall;

        }
        #endregion

    }
}
