using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using RepPP;
using RepPP.Toolkit.Window;
using RepPP.Toolkit;

namespace RepPP.Framework.Window {
    /// <summary>
    /// Default toolbar for the rowset tree handler
    /// </summary>
    public class ToolbarRowsetTree : ToolStrip {
        /// <summary>
        /// Enumeration for the available ToolBar button sizes
        /// </summary>
        public enum ToolBarSize {
            /// <summary>Use small buttons in the toolbar</summary>
            Small       = 0,
            /// <summary>Use large buttons in the toolbar</summary>
            Big         = 2
        };
        
        /// <summary>RowsetTreeDef handler attached to the toolbar</summary>
        private  RowsetTreeDefNodeHandlerRootBase           m_nodeHandler;
        /// <summary>Size of the buttons used in the toolbar</summary>
        private  ToolBarSize                                m_eButtonSize;
        private  ISupportDirectSearch                       m_dsHandler;
        /// <summary>The button to save the current item</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butSave;
        /// <summary>The button to save all modified items</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butSaveAll;
        /// <summary>The button to searc for a list of rows</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butSearch;
        /// <summary>The button to undo the changed on the current item</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butUndo;
        /// <summary>Separator</summary>
        public  System.Windows.Forms.ToolStripSeparator     butSep1;
        /// <summary>The button to cut the value of the current field</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butCut;
        /// <summary>The button to copy the value of the current field</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butCopy;
        /// <summary>The button to paste the value into the current field</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butPaste;
        /// <summary>Separator</summary>
        public  System.Windows.Forms.ToolStripSeparator     butSep2;
        /// <summary>The button to popup a list of choice for the current field</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butChoice;
        /// <summary>The button to show help on the current field</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butHelp;
        /// <summary>Separator</summary>
        public  System.Windows.Forms.ToolStripSeparator     butSep3;
        /// <summary>The button to erase the value of the current field</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butEraseField;
        /// <summary>The button to set the default value of the current field</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butDefaultField;
        /// <summary>Separator</summary>
        public  System.Windows.Forms.ToolStripSeparator     butSep4;
        /// <summary>The button to erase the value of all fields</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butEraseAllFields;
        /// <summary>The button to set the default value of all fields</summary>
        public  System.Windows.Forms.ToolStripMenuItem      butDefaultAllFields;
        /// <summary>Direct search field label</summary>
        public System.Windows.Forms.ToolStripLabel          mnuLblField;
        /// <summary>Direct search fields combo box</summary>
        public System.Windows.Forms.ToolStripComboBox       mnuCbxFields;
        /// <summary>Direct search field value</summary>
        public System.Windows.Forms.ToolStripTextBox        mnuTxtFieldValue;
        /// <summary>Direct search button</summary>
        public System.Windows.Forms.ToolStripButton         mnuButSearch;
        /// <summary>Direct search separator</summary>
        public System.Windows.Forms.ToolStripSeparator      toolStripSeparatorDS;
        private System.Windows.Forms.ImageList              imageListBig;
        private System.Windows.Forms.ImageList              imageListSmall;
        private System.ComponentModel.IContainer            components;

        //*********************************************************	    
        //
        /// <summary>
        ///	ctor:           Class constructor
        /// </summary>
        //	
        //*********************************************************	    
        public ToolbarRowsetTree() {
            InitializeComponent();
            m_eButtonSize = ToolBarSize.Small;
            m_dsHandler   = null;
        }

        //*********************************************************	    
        //
        /// <summary>
        ///	Dispose:                    Dispose
        /// </summary>
        ///
        /// <param name="bDisposing">   true if disposing, false if GC</param>
        //	
        //*********************************************************	    
        protected override void Dispose(bool bDisposing) {
            if (bDisposing) {
                m_nodeHandler = null;
                m_dsHandler     = null;
                if (components != null) {
                    components.Dispose();
                }
            }
            base.Dispose(bDisposing);
        }

        //*********************************************************	    
        //
        /// <summary>
        ///	UnregisterTrigger:          Unregister the button to the command holder
        /// </summary>
        //	
        //*********************************************************	    
        private void UnregisterTrigger() {
            CmdHolder    cmdHolder;
            
            cmdHolder = m_nodeHandler.CommandHolder;
            if (cmdHolder != null) {
                cmdHolder.RemoveTrigger(sdHandlerCmd.ItemSave.ToString(),        new object[] { butSave });
                cmdHolder.RemoveTrigger(sdHandlerCmd.ItemSaveAll.ToString(),     new object[] { butSaveAll });
                cmdHolder.RemoveTrigger(sdHandlerCmd.ItemUndo.ToString(),        new object[] { butUndo });
                cmdHolder.RemoveTrigger(sdHandlerCmd.FieldCut.ToString(),        new object[] { butCut });
                cmdHolder.RemoveTrigger(sdHandlerCmd.FieldCopy.ToString(),       new object[] { butCopy });
                cmdHolder.RemoveTrigger(sdHandlerCmd.FieldPaste.ToString(),      new object[] { butPaste });
                cmdHolder.RemoveTrigger(sdHandlerCmd.FieldChoice.ToString(),     new object[] { butChoice });
                cmdHolder.RemoveTrigger(sdHandlerCmd.FieldHelp.ToString(),       new object[] { butHelp });
                cmdHolder.RemoveTrigger(sdHandlerCmd.FieldErase.ToString(),      new object[] { butEraseField });
                cmdHolder.RemoveTrigger(sdHandlerCmd.FieldDefault.ToString(),    new object[] { butDefaultField });
                cmdHolder.RemoveTrigger(sdHandlerCmd.FieldEraseAll.ToString(),   new object[] { butEraseAllFields });
                cmdHolder.RemoveTrigger(sdHandlerCmd.FieldDefaultAll.ToString(), new object[] { butDefaultAllFields });
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        ///	RegisterTrigger:        Register the button to the command holder
        /// </summary>
        //	
        //*********************************************************	    
        private void RegisterTrigger() {
            CmdHolder cmdHolder;
            
            cmdHolder = m_nodeHandler.CommandHolder;
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemSave.ToString(), new object[] { butSave });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemSaveAll.ToString(), new object[] { butSaveAll });
            cmdHolder.AddTrigger(null, sdHandlerCmd.RowsetTreeSearch.ToString(), new object[] { butSearch });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemUndo.ToString(), new object[] { butUndo });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldCut.ToString(), new object[] { butCut });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldCopy.ToString(), new object[] { butCopy });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldPaste.ToString(), new object[] { butPaste });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldChoice.ToString(), new object[] { butChoice });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldHelp.ToString(), new object[] { butHelp });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldErase.ToString(), new object[] { butEraseField });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldDefault.ToString(), new object[] { butDefaultField });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldEraseAll.ToString(), new object[] { butEraseAllFields });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldDefaultAll.ToString(), new object[] { butDefaultAllFields });
        }

        //*********************************************************	    
        //
        /// <summary>
        ///	OnDirectSearch:        Handles the click event of the 
        ///                        Search button
        /// </summary>
        /// <param name="sender">  The direct search button</param>
        /// <param name="e">       The event argument</param>
        //	
        //*********************************************************	    
        private void OnDirectSearch(object sender, EventArgs e) {
            Application app;
            FieldDef       fldDef     = null;
            string      strText = string.Empty;
            string[]    arrVals = null;
            if (mnuCbxFields.SelectedItem != null) {
                arrVals = ((FldComboBox.ListItem)mnuCbxFields.SelectedItem).Value.Split('.');
                strText = mnuTxtFieldValue.Text;
            }
            if (arrVals != null && arrVals.Length == 2) {
                app = RootNodeHandler.RowsetTreeDef.Application;
                fldDef = app.FieldDef(arrVals[0], arrVals[1]);
                DirectSearchHandler.ExecuteDirectSearch(fldDef, strText);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        ///	OnSearchFieldChanged:  Handles the SelectedIndexChanged
        ///                        event of the direct search fields
        ///                        combo box
        /// </summary>
        /// <param name="sender">  The direct search fields combo box</param>
        /// <param name="e">       The event argument</param>
        //	
        //*********************************************************	    
        private void OnSearchFieldChanged(object sender, EventArgs e) {
            mnuButSearch.Enabled = (mnuCbxFields.SelectedItem != null);
        }

        //*********************************************************	    
        //
        /// <summary>
        ///	DirectSearchHandler:    Gets/Sets the direct search handler
        /// </summary>
        //	
        //*********************************************************	    
        public virtual ISupportDirectSearch DirectSearchHandler {
            get {
                return(m_dsHandler);
            }
            set {
                if (m_dsHandler == null && value != null) {
                    InitializeDirectSearchToolBar();
                }
                m_dsHandler = value;
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        ///	RootNodeHandler:    Gets the root node handler
        /// </summary>
        //	
        //*********************************************************	    
        public virtual RowsetTreeDefNodeHandlerRootBase RootNodeHandler {
            get {
                return(m_nodeHandler);
            }
            set {
                if (m_nodeHandler != value) {
                    if (m_nodeHandler != null) {
                        UnregisterTrigger();
                    }
                    m_nodeHandler = value;
                    if (m_nodeHandler != null) {
                        if (m_nodeHandler.CommandHolder == null) {
                            m_nodeHandler.CommandHolder = new CmdHolder();
                        }
                        RegisterTrigger();
                        if (DirectSearchHandler != null) {
                            LoadDirectSearchFields();
                        }
                    }
                }
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        ///	ToolBarButtonSize:      Button size
        /// </summary>
        //	
        //*********************************************************	    
        public ToolBarSize ToolBarButtonSize {
            get {
                return(m_eButtonSize);
            }
            set {
                if (m_eButtonSize != value) {
                    switch(value) {
                    case ToolBarSize.Small:
                        m_eButtonSize = value;
                        ImageList     = imageListSmall;
                        break;
                    case ToolBarSize.Big:
                        m_eButtonSize = value;
                        ImageList     = imageListBig;
                        break;
                    }
                    Refresh();
                }
            }
        }


        //*********************************************************	    
        //
        /// <summary>
        ///	InitializeDirectSearchToolBar:  Creates the direct search
        ///                                 controls and adds them to
        ///                                 the toolbar
        /// </summary>
        //	
        //*********************************************************	    
        protected virtual void InitializeDirectSearchToolBar() {
            this.mnuLblField          = new System.Windows.Forms.ToolStripLabel();
            this.mnuCbxFields         = new System.Windows.Forms.ToolStripComboBox();
            this.mnuTxtFieldValue     = new System.Windows.Forms.ToolStripTextBox();
            this.mnuButSearch         = new System.Windows.Forms.ToolStripButton();
            this.toolStripSeparatorDS = new System.Windows.Forms.ToolStripSeparator();
            this.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {this.toolStripSeparatorDS,
                                                                          this.mnuLblField,
                                                                          this.mnuCbxFields,
                                                                          this.mnuTxtFieldValue,
                                                                          this.mnuButSearch});
            // 
            // mnuLblField
            // 
            this.mnuLblField.Name = "mnuLblField";
            this.mnuLblField.Size = new System.Drawing.Size(36, 22);
            this.mnuLblField.Text = "Field: ";
            // 
            // mnuCbxFields
            // 
            this.mnuCbxFields.DropDownStyle         = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.mnuCbxFields.DropDownWidth         = 200;
            this.mnuCbxFields.DropDownHeight        = 600;
            this.mnuCbxFields.Name                  = "mnuCbxFields";
            this.mnuCbxFields.Size                  = new System.Drawing.Size(121, 25);
            this.mnuCbxFields.SelectedIndexChanged += new EventHandler(OnSearchFieldChanged);
            // 
            // mnuTxtFieldValue
            // 
            this.mnuTxtFieldValue.Name = "mnuTxtFieldValue";
            this.mnuTxtFieldValue.Size = new System.Drawing.Size(100, 25);
            // 
            // mnuButSearch
            // 
            this.mnuButSearch.DisplayStyle  = System.Windows.Forms.ToolStripItemDisplayStyle.Text;
            this.mnuButSearch.Enabled       = false;
            this.mnuButSearch.Name          = "mnuButSearch";
            this.mnuButSearch.Size          = new System.Drawing.Size(23, 22);
            this.mnuButSearch.Text          = "...";
            this.mnuButSearch.ToolTipText   = "Perform a direct search using the specified field and value.";
            this.mnuButSearch.Click        += new System.EventHandler(this.OnDirectSearch);
            // 
            // toolStripSeparatorDS
            // 
            this.toolStripSeparatorDS.Name  = "toolStripSeparatorDS";
            this.toolStripSeparatorDS.Size  = new System.Drawing.Size(6, 25);
        }

        //*********************************************************	    
        //
        /// <summary>
        ///	LoadDirectSearchFields:     Loads the list of fields that
        ///                             can be searched directly using
        ///                             the transaction card
        /// </summary>
        //	
        //*********************************************************
        protected virtual void LoadDirectSearchFields() {
            mnuCbxFields.Items.Clear();            
            LoadDirectSearchFields(RootNodeHandler.RowsetTreeDef.Nodes);
        }

        //*********************************************************	    
        //
        /// <summary>
        ///	LoadDirectSearchFields:     Loads the list of fields that
        ///                             can be searched directly in a 
        ///                             specific card group
        /// </summary>
	    /// <param name="cardGroups">   RowsetTreeDef RowsetDef</param>
        //	
        //*********************************************************
        private void LoadDirectSearchFields(RowsetTreeDefNodes cardGroups) {
            RowsetDef  rowsetDef;

            foreach(RowsetTreeDefNode node in cardGroups) {
                rowsetDef = node.RowsetDef;
                foreach(FieldDef fldDef in rowsetDef.FieldDefs) {
                    if (DirectSearchHandler.IsDirectSearchField(fldDef)) {
                        AddField(fldDef, null);
                    }
                }
                LoadDirectSearchFields(node.Nodes);
            }
        }

        //*********************************************************	    
        //
	    /// <summary>
	    /// AddField                    Adds a field name to the list
        ///                             of fields that can be searched
	    /// </summary>
	    /// <param name="fldDef">          FieldDef</param>
	    /// <param name="strFldDesc">   FieldDef Prompt</param>
        //	
        //*********************************************************
        protected virtual void AddField(FieldDef fldDef, string strFldDesc) {
            FldComboBox.ListItem    item;
            FieldDef                fldDefAlt;
            string                  strRowsetName;
            string                  strFldName;
            string                  strFldPrompt;
            string                  strFldFullName;
            bool                    bExists = false;

            fldDefAlt = SBFilterBase.GetAlternateFilterField(fldDef);
            if (fldDefAlt != null) {
                fldDef = fldDefAlt;
            }
            strRowsetName   = fldDef.RowsetDef.Name;
            strFldName      = fldDef.Name;
            strFldFullName  = strRowsetName + "." + strFldName;
            foreach (FldComboBox.ListItem li in mnuCbxFields.Items) {
                if (li.Value == strFldFullName){
                    bExists = true;
                }
            }
            if (!bExists) {
                if (strFldDesc != null && strFldDesc != string.Empty) {
                    strFldPrompt = strFldDesc;
                } else if (fldDef.Prompt != string.Empty) {
                    strFldPrompt = Util.GetDirectSearchFieldPrompt(fldDef);
                } else {
                    strFldPrompt = "[" + strFldFullName + "]"; 
                }
                item = new FldComboBox.ListItem(strFldFullName, strFldPrompt);
                mnuCbxFields.Items.Add(item);
            }
        }


        #region Component Designer generated code
        /// <summary> 
        /// Required method for Designer support - do not modify 
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent() {
            this.components = new System.ComponentModel.Container();
            System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(ToolbarRowsetTree));
            this.butSave = new System.Windows.Forms.ToolStripMenuItem();
            this.butSaveAll = new System.Windows.Forms.ToolStripMenuItem();
            this.butSearch = new System.Windows.Forms.ToolStripMenuItem();
            this.butUndo = new System.Windows.Forms.ToolStripMenuItem();
            this.butSep1 = new System.Windows.Forms.ToolStripSeparator();
            this.butCut = new System.Windows.Forms.ToolStripMenuItem();
            this.butCopy = new System.Windows.Forms.ToolStripMenuItem();
            this.butPaste = new System.Windows.Forms.ToolStripMenuItem();
            this.butSep2 = new System.Windows.Forms.ToolStripSeparator();
            this.butChoice = new System.Windows.Forms.ToolStripMenuItem();
            this.butHelp = new System.Windows.Forms.ToolStripMenuItem();
            this.butSep3 = new System.Windows.Forms.ToolStripSeparator();
            this.butEraseField = new System.Windows.Forms.ToolStripMenuItem();
            this.butDefaultField = new System.Windows.Forms.ToolStripMenuItem();
            this.butSep4 = new System.Windows.Forms.ToolStripSeparator();
            this.butEraseAllFields = new System.Windows.Forms.ToolStripMenuItem();
            this.butDefaultAllFields = new System.Windows.Forms.ToolStripMenuItem();
            this.imageListBig = new System.Windows.Forms.ImageList(this.components);
            this.imageListSmall = new System.Windows.Forms.ImageList(this.components);
            // 
            // butSave
            // 
            this.butSave.AutoSize = false;
            this.butSave.Size = new Size(22, 20);
            this.butSave.ImageIndex = 0;
            this.butSave.Tag = "1";
            this.butSave.ToolTipText = "Save";
            // 
            // butSaveAll
            // 
            this.butSaveAll.AutoSize = false;
            this.butSaveAll.Size = new Size(22, 20);
            this.butSaveAll.ImageIndex = 1;
            this.butSaveAll.Tag = "2";
            this.butSaveAll.ToolTipText = "Save All";
            // 
            // butSearch
            // 
            this.butSearch.AutoSize = false;
            this.butSearch.Size = new Size(22, 20);
            this.butSearch.ImageIndex = 2;
            this.butSearch.Tag = "3";
            this.butSearch.ToolTipText = "Search";
            // 
            // butUndo
            // 
            this.butUndo.AutoSize = false;
            this.butUndo.Size = new Size(22, 20);
            this.butUndo.ImageIndex = 3;
            this.butUndo.Tag = "4";
            this.butUndo.ToolTipText = "Undo";
            // 
            // butCut
            // 
            this.butCut.AutoSize = false;
            this.butCut.Size = new Size(22, 20);
            this.butCut.Tag = "5";
            this.butCut.ToolTipText = "Cut";
            this.butCut.Visible = false;
            // 
            // butCopy
            // 
            this.butCopy.AutoSize = false;
            this.butCopy.Size = new Size(22, 20);
            this.butCopy.Tag = "6";
            this.butCopy.ToolTipText = "Copy";
            this.butCopy.Visible = false;
            // 
            // butPaste
            // 
            this.butPaste.AutoSize = false;
            this.butPaste.Size = new Size(22, 20);
            this.butPaste.Tag = "7";
            this.butPaste.ToolTipText = "Paste";
            this.butPaste.Visible = false;
            // 
            // butSep2
            // 
            this.butSep2.Visible = false;
            // 
            // butChoice
            // 
            this.butChoice.AutoSize = false;
            this.butChoice.Size = new Size(22, 20);
            this.butChoice.ImageIndex = 4;
            this.butChoice.Tag = "8";
            this.butChoice.ToolTipText = "List of Choice";
            // 
            // butHelp
            // 
            this.butHelp.AutoSize = false;
            this.butHelp.Size = new Size(22, 20);
            this.butHelp.ImageIndex = 5;
            this.butHelp.Tag = "9";
            this.butHelp.ToolTipText = "Help";
            // 
            // butEraseField
            // 
            this.butEraseField.AutoSize = false;
            this.butEraseField.Size = new Size(22, 20);
            this.butEraseField.ImageIndex = 6;
            this.butEraseField.Tag = "10";
            this.butEraseField.ToolTipText = "Erase Field";
            // 
            // butDefaultField
            // 
            this.butDefaultField.AutoSize = false;
            this.butDefaultField.Size = new Size(22, 20);
            this.butDefaultField.ImageIndex = 7;
            this.butDefaultField.Tag = "11";
            this.butDefaultField.ToolTipText = "Field Default Answer";
            // 
            // butEraseAllFields
            // 
            this.butEraseAllFields.AutoSize = false;
            this.butEraseAllFields.Size = new Size(22, 20);
            this.butEraseAllFields.ImageIndex = 8;
            this.butEraseAllFields.Tag = "12";
            this.butEraseAllFields.ToolTipText = "Erase All Fields";
            // 
            // butDefaultAllFields
            // 
            this.butDefaultAllFields.AutoSize = false;
            this.butDefaultAllFields.Size = new Size(22, 20);
            this.butDefaultAllFields.ImageIndex = 9;
            this.butDefaultAllFields.Tag = "13";
            this.butDefaultAllFields.ToolTipText = "All Fields Default Answer";
            // 
            // imageListBig
            // 
            this.imageListBig.ColorDepth = System.Windows.Forms.ColorDepth.Depth24Bit;
            this.imageListBig.ImageSize = new System.Drawing.Size(32, 32);
            this.imageListBig.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageListBig.ImageStream")));
            this.imageListBig.TransparentColor = System.Drawing.Color.Magenta;
            // 
            // imageListSmall
            // 
            this.imageListSmall.ColorDepth = System.Windows.Forms.ColorDepth.Depth24Bit;
            this.imageListSmall.ImageSize = new System.Drawing.Size(16, 16);
            this.imageListSmall.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageListSmall.ImageStream")));
            this.imageListSmall.TransparentColor = System.Drawing.Color.Magenta;
            // 
            // ToolbarRowsetTree
            // 
            this.ImageList = this.imageListSmall;
            this.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                                                                               this.butSave,
                                                                               this.butSaveAll,
                                                                               this.butSearch,
                                                                               this.butUndo,
                                                                               this.butSep1,
                                                                               this.butCut,
                                                                               this.butCopy,
                                                                               this.butPaste,
                                                                               this.butSep2,
                                                                               this.butChoice,
                                                                               this.butHelp,
                                                                               this.butSep3,
                                                                               this.butEraseField,
                                                                               this.butDefaultField,
                                                                               this.butSep4,
                                                                               this.butEraseAllFields,
                                                                               this.butDefaultAllFields});
            this.ImageScalingSize = new Size(16, 16);
            this.Size = new System.Drawing.Size(100, 24);
        }
        #endregion
    }
}
