//
// Rep++ Framework for Windows Application
// Copyright (C) 2005 Consyst SQL Inc.
// All Rights Reserved
//
// This source code is only intended as a supplement to the
// Rep++ Studio products.
//
using System;
using System.Collections;
using RepPP;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
	/// <summary>
	/// Summary description for IRowsetTreeDefNodeHandler.
	/// </summary>
	public interface IRowsetTreeDefNodeHandler {	    
	    /// <summary>Event sent each time a new line is selected for this node</summary>
	    event RowsetTreeDefNodeHandler.LineSelectedEventHandler     LineSelected;
        /// <summary>Father line changed. Select rowset</summary>
        event System.EventHandler                           FatherLineSelected;
	    /// <summary>Event sent before a line is validated</summary>
	    event RowsetTreeDefNodeHandler.LineValidationEventHandler   BeforeLineValidated;
	    /// <summary>Event sent before a line is validated</summary>
	    event RowsetTreeDefNodeHandler.LineValidationEventHandler   AfterLineValidated;
        /// <summary>State must be adjust</summary>
        event System.EventHandler                           AdjustState;
        /// <summary>Called before a command is executed</summary>
        event RowsetTreeDefNodeHandler.ExecuteCmdEventHandler       BeforeExecuteCmd;
        /// <summary>Called after a command is executed</summary>
        event RowsetTreeDefNodeHandler.ExecuteCmdEventHandler       AfterExecuteCmd;
        /// <summary>An error Occurred</summary>
        event RowsetTreeDefNodeHandler.ErrorOccurredEventHandler    ErrorOccurred;
        /// <summary>Called when data conversion must be done</summary>
        event RowsetTreeDefNodeHandler.DataConversionEventHandler   DataConversion;

        //*********************************************************     
        //
        /// <summary>
        /// Handler:            Return a pointer on the card group handler
        /// </summary>
        //  
        //*********************************************************     
        RowsetTreeDefNodeHandler Handler { get; }

        //*********************************************************     
        //
        /// <summary>
        /// RowsetTreeDefNode:                      RowsetTreeDef group handle by this class
        /// </summary>
        //  
        //*********************************************************     
        RowsetTreeDefNode RowsetTreeDefNode { get; }

        //*********************************************************     
        //
        /// <summary>
        /// FatherRowsetTreeDefNodeHandler:         Father card group handler
        /// </summary>
        //  
        //*********************************************************     
        RowsetTreeDefNodeHandler FatherRowsetTreeDefNodeHandler { get; }

        //*********************************************************     
        //
        /// <summary>
        /// RootRowsetTreeDefNodeHandler:           Root card group handler
        /// </summary>
        //  
        //*********************************************************     
        RowsetTreeDefNodeHandlerRootBase RootRowsetTreeDefNodeHandler { get; }

        //*********************************************************     
        //
        /// <summary>
        /// Rowset:          Attached rowset
        /// </summary>
        //  
        //*********************************************************     
        Rowset Rowset { get; set; }

        //*********************************************************     
        //
        /// <summary>
        /// LineListHandler:                Handler for the lines
        /// </summary>
        //  
        //*********************************************************     
		ILineListHandler LineListHandler { get; set; }

        //*********************************************************     
        //
        /// <summary>
        /// CommandHolder:              Command holder
        /// </summary>
        //  
        //*********************************************************     
        CmdHolder CommandHolder { get; set; }

        //*********************************************************     
        //
        /// <summary>
        /// IsLineDeleted:      Returns true if the line or a father line is deleted
        /// </summary>
        //  
        //*********************************************************     
        bool IsLineDeleted { get; }

        //*********************************************************     
        //
        /// <summary>
        /// AllowInsert:            Insertion allowed if true
        /// </summary>
        //  
        //*********************************************************     
		sdAllow AllowInsert { get; set; }

        //*********************************************************     
        //
        /// <summary>
        /// AllowUpdate:            Update allowed if true
        /// </summary>
        //  
        //*********************************************************     
        sdAllow AllowUpdate { get; set; }

        //*********************************************************     
        //
        /// <summary>
        /// AllowDelete:        Delete allowed or not for this card group
        /// </summary>
        //  
        //*********************************************************     
        sdAllow AllowDelete { get; set; }

        //*********************************************************     
        //
        /// <summary>
        /// EffectiveAllowInsert:       Return true if a new line can be inserted
        /// </summary>
        //  
        //*********************************************************     
        bool EffectiveAllowInsert { get; }

        //*********************************************************     
        //
        /// <summary>
        /// EffectiveAllowUpdate:       Update allowed if true
        /// </summary>
        //  
        //*********************************************************     
        bool EffectiveAllowUpdate { get; }

        //*********************************************************     
        //
        /// <summary>
        /// EffectiveAllowDelete:       Delete allowed if true
        /// </summary>
        //  
        //*********************************************************     
        bool EffectiveAllowDelete { get; }

        //*********************************************************     
        //
        /// <summary>
        /// FieldProtection:    Actual protection level of the fields
        /// </summary>
        //  
        //*********************************************************     
        RepPP.Toolkit.Util.FldSelectableState.SelStateE FieldProtection { get; set; }

        //*********************************************************     
        //
        /// <summary>
        /// SetFieldProtection:     Set the field protection according to the context
        /// </summary>
        //  
        //*********************************************************     
        void SetFieldProtection();

        //*********************************************************     
        //
        /// <summary>
        /// FindRowsetTreeDefNodeHandler:       Find a card group handler using the attached group
        /// </summary>
        /// 
        /// <param name="rowsetDef">          RowsetDef</param>
        /// <returns> 
        /// RowsetTreeDef group handler or null if not found.
        /// </returns>
        //  
        //*********************************************************     
        RowsetTreeDefNodeHandler FindRowsetTreeDefNodeHandler(RowsetDef rowsetDef);

        //*********************************************************     
        //
        /// <summary>
        /// FindRowsetTreeDefNodeHandler:       Find a card group handler using the attached card group
        /// </summary>
        /// 
        /// <param name="node">         RowsetTreeDef group</param>
        /// <returns> 
        /// RowsetTreeDef group handler or null if not found.
        /// </returns>
        //  
        //*********************************************************     
        RowsetTreeDefNodeHandler FindRowsetTreeDefNodeHandler(RowsetTreeDefNode node);

        //*********************************************************     
        //
        /// <summary>
        /// CurrentLine:            Gets the current line position
        /// </summary>
        //  
        //*********************************************************     
        int CurrentLine { get; }

        //*********************************************************     
        //
        /// <summary>
        /// LineCount:              Gets the current line count
        /// </summary>
        //
        //*********************************************************     
        int LineCount { get; }

        //*********************************************************     
        //
        /// <summary>
        /// SBCurrentLine:          Gets the current line in the selection buffer
        /// </summary>
        //  
        //*********************************************************     
        int SBCurrentLine { get; }

        //*********************************************************     
        //
        /// <summary>
        /// SBLineCount:            Gets the current line count in the selection buffer
        /// </summary>
        //
        //*********************************************************     
        int SBLineCount { get; }

        //*********************************************************     
        //
        /// <summary>
        /// TriggerResetFieldError: Reset the last error message from field validation
        /// </summary>
        //  
        //*********************************************************     
        void TriggerResetFieldError();

        //*********************************************************     
        //
        /// <summary>
        /// OnLineSelected:     Reset the last error message from field validation
        /// </summary>
        //  
        //*********************************************************     
		void OnLineSelected(RowsetTreeDefNodeHandler.LineSelectedEventArgs e);

        //*********************************************************     
        //
        /// <summary>
        /// OnFatherLineSelected:   Trigger the FatherLineSelected event for this control
        /// </summary>
        /// <param name="e">        Event arguments</param>
        //  
        //*********************************************************     
        void OnFatherLineSelected(System.EventArgs e);

        //*********************************************************     
        //
        /// <summary>
        /// OnBeforeLineValidated:  Trigger the BeforeLineValidated event for this control
        ///                         and all child
        /// </summary>
        /// <param name="e">        Line validated event arguments</param>
        //  
        //*********************************************************     
        void OnBeforeLineValidated(RowsetTreeDefNodeHandler.LineValidationEventArgs e);

        //*********************************************************     
        //
        /// <summary>
        /// OnAfterLineValidated:   Trigger the AfterLineValidated event for this control
        ///                         and all child.
        /// </summary>
        /// <param name="e">        Line validated event argument</param>
        //  
        //*********************************************************     
        void OnAfterLineValidated(RowsetTreeDefNodeHandler.LineValidationEventArgs e);

        //*********************************************************     
        //
        /// <summary>
        /// SetCommandHolderState:  Set the state of the command holder
        /// </summary>
        //  
        //*********************************************************     
        void SetCommandHolderState();

        //*********************************************************     
        //
        /// <summary>
        /// OnAdjustState:      Raise the AdjustState event
        /// </summary>
        //  
        //*********************************************************     
        void OnAdjustState(EventArgs e);

        //*********************************************************     
        //
        /// <summary>
        /// OnBeforeExecuteCmd:     Informs that a command is about to be executed
        /// </summary>
        /// <param name="e">        Line validated event</param>
        //  
        //*********************************************************     
        void OnBeforeExecuteCmd(RowsetTreeDefNodeHandler.ExecuteCmdEventArgs e);

        //*********************************************************     
        //
        /// <summary>
        /// OnAfterExecuteCmd:      Informs that a command has been executed
        /// </summary>
        /// <param name="e">        Line validated event</param>
        //  
        //*********************************************************     
        void OnAfterExecuteCmd(RowsetTreeDefNodeHandler.ExecuteCmdEventArgs e);

        //*********************************************************     
        //
        /// <summary>
        /// OnErrorOccurred:    Trigger the ErrorOccurred event for this control
        ///                     and all child
        /// </summary>
        /// <param name="e">    Error Occurred event arguments</param>
        //  
        //*********************************************************     
        void OnErrorOccurred(RowsetTreeDefNodeHandler.ErrorOccurredEventArgs e);

        //*********************************************************     
        //
        /// <summary>
        /// TriggerErrors:      Trigger the ErrorOccurred event 
        /// </summary>
        //  
        //*********************************************************     
        void TriggerErrors();

        //*********************************************************     
        //
        /// <summary>
        /// OnDataConversion:   Trigger the DataConversion event for this control
        /// </summary>
        /// <param name="e">    Error Occurred event arguments</param>
        //  
        //*********************************************************     
        void OnDataConversion(RowsetTreeDefNodeHandler.DataConversionEventArgs e);

        //*********************************************************     
        //
        /// <summary>
        /// RaiseAdjustState:       Raise the adjust state for this control and father.
        /// </summary>
        //  
        //*********************************************************     
        void RaiseAdjustState();

        //*********************************************************     
        //
        /// <summary>
        /// CommitEdit:             Commit the edit before validating
        /// </summary>
        //  
        //*********************************************************     
        void CommitEdit();

        //*********************************************************     
        //
        /// <summary>
        /// CommitAllEdit:              Commit the edit of all card group
        /// </summary>
        /// <param name="nodeHandler">  RowsetTreeDef group handler</param>
        //  
        //*********************************************************     
        void CommitAllEdit(RowsetTreeDefNodeHandler nodeHandler);

        //*********************************************************     
        //
        /// <summary>
        /// ValidateCurLine:        Validates the current line
        /// </summary>
        /// <returns>
        ///     false:              Line not valid
        ///     true:               Line valid
        /// </returns>
        //  
        //*********************************************************     
        bool ValidateCurLine();

        //*********************************************************     
        //
        /// <summary>
        /// SetCurrentLine:             Sets the current logical line
        /// </summary>
        /// <param name="iLine">        Logical or physical line</param>
        /// <returns>
        /// true                        New line selected
        /// false                       Line not selected
        /// </returns>
        //  
        //*********************************************************     
        bool SetCurrentLine(int iLine);

        //*********************************************************     
        //
        /// <summary>
        /// SelectFirstLine:        Select the first line
        /// </summary>
        /// <returns>
        ///     true:               The first line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        bool SelectFirstLine();

        //*********************************************************     
        //
        /// <summary>
        /// SelectPrevLine:         Select the previous line
        /// </summary>
        /// <returns>
        ///     true:               The previous line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        bool SelectPrevLine();

        //*********************************************************     
        //
        /// <summary>
        /// SelectNextLine:         Select the next line
        /// </summary>
        /// <returns>
        ///     true:               The next line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        bool SelectNextLine();

        //*********************************************************     
        //
        /// <summary>
        /// SelectLastLine:         Select the last line
        /// </summary>
        /// <returns>
        ///     true:               The last line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        bool SelectLastLine();

		//*********************************************************     
		//
		/// <summary>
		/// InsertNewLine:          Insert a new line
		/// </summary>
		/// <param name="bAtEnd">   true to insert at the end</param>
		/// <returns>
		/// true:                   A new line has been inserted
		/// false:                  No line inserted
		/// </returns>
		//  
		//*********************************************************     
		bool InsertNewLine(bool bAtEnd);

        //*********************************************************     
        //
        /// <summary>
        /// DuplicateLine:              Duplicate the current line
        /// </summary>
        /// <returns>
        /// true:                       The line has been duplicated
        /// false:                      No line inserted
        /// </returns>
        //  
        //*********************************************************     
        bool DuplicateLine();

        //*********************************************************     
        //
        /// <summary>
        /// DeleteCurrentLine:      Deletes the current line
        /// </summary>
        /// <returns>
        ///     true:               The line has been deleted
        ///     false:              No line deleted
        /// </returns>
        //  
        //*********************************************************     
        bool DeleteCurrentLine();

        //*********************************************************     
        //
        /// <summary>
        /// HandleCommand:          Handle a command coming from a menu or a toolbar
        /// </summary>
        /// <param name="eCmd">     Command Id</param>
        /// <param name="bEvent">   true to trigger event</param>
        //  
        //*********************************************************     
        void HandleCommand(sdHandlerCmd eCmd, bool bEvent);

        //*********************************************************     
        //
        /// <summary>
        /// GetActionState:             Get the specified action state
        /// </summary>
        /// <param name="eAction">      Action</param>
        /// <returns>
        /// false:                      Disabled
        /// true:                       Enabled
        /// </returns>returns>
        //  
        //*********************************************************     
        bool GetActionState(sdAction eAction);

        //*********************************************************     
        //
        /// <summary>
        /// IsDirty:                Check if the card is dirty
        /// </summary>
        /// <returns>
        ///  true:                   Yes
        ///  false:                  No
        /// </returns>
        //
        //*********************************************************     
        bool IsDirty();

        //*********************************************************     
        //
        /// <summary>
        /// GetItemState:           Get the state of the currently selected item
        /// </summary>
        /// <returns>
        /// LineState
        /// </returns>
        //  
        //*********************************************************     
        RepPP.LineState GetItemState();
    }
}
