//
// Rep++ Framework for Windows Application
// Copyright (C) 2005 Consyst SQL Inc.
// All Rights Reserved
//
// This source code is only intended as a supplement to the
// Rep++ Studio products.
//
using System;
using System.Collections;
using RepPP;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
	/// <summary>
	/// Summary description for IRootCardGroupHandlerForSB.
	/// </summary>
	public interface IRowsetTreeDefNodeHandlerRootForSB : IRowsetTreeDefNodeHandler {
	    /// <summary>Event sent each time a new line is selected for this card group</summary>
        /// <summary>Reset field error</summary>
        event System.EventHandler                           ResetFieldError;
        /// <summary>Adjust the field state</summary>
        event System.EventHandler                           AdjustFieldState;

        //*********************************************************     
        //
        /// <summary>
        /// AutoCommit:             true if auto-commit is on
        /// </summary>
        //  
        //*********************************************************     
        bool AutoCommit { get; set; }

        //*********************************************************     
        //
        /// <summary>
        /// DataAccess:              Data access object
        /// </summary>
        //  
        //*********************************************************     
        RepPP.Toolkit.ITransactionDataAccess DataAccess { get; set; }

        //*********************************************************     
        //
        /// <summary>
        /// RowsetTreeDef:                       Attached card
        /// </summary>
        //  
        //*********************************************************     
        RowsetTreeDef RowsetTreeDef { get; }

        //*********************************************************     
        //
        /// <summary>
        /// RowsetTree:             Attached rowset tree
        /// </summary>
        //  
        //*********************************************************     
        RowsetTree RowsetTree { get; set; }

        //*********************************************************     
        //
        /// <summary>
        /// CurrentFieldDef:           FieldDef currently selected
        /// </summary>
        //  
        //*********************************************************     
        FieldDef CurrentFieldDef { get; set; }

        //*********************************************************     
        //
        /// <summary>
        /// UndoRowsetTree:           RowsetTree use to hold undo value
        /// </summary>
        //  
        //*********************************************************     
        RowsetTree UndoRowsetTree { get; }

        //*********************************************************     
        //
        /// <summary>
        /// Refresh:                    Refresh the control
        /// </summary>
        //  
        //*********************************************************     
        void Refresh();

        //*********************************************************     
        //
        /// <summary>
        /// SaveLine:                           Save changes for the current line
        /// </summary>
        /// <returns>
        ///  true:                              Changed saved or no changes
        ///  false:                             Not saved. User refused or error
        /// </returns>
        //
        //*********************************************************     
        bool SaveLine();

        //*********************************************************     
        //
        /// <summary>
        /// AskToSave:                          Save changes if any after asking user
        /// </summary>
        /// <param name="bEnsureFatherExist">   If an operation on a child of this item is done and this item is new, we must not
        ///                                     accept to not save it.</param>
        /// <returns>
        ///  true:                              Changed saved or no changes
        ///  false:                             Not saved. User refused or error
        /// </returns>
        //
        //*********************************************************     
        bool AskToSave(bool bEnsureFatherExist);

        //*********************************************************     
        //
        /// <summary>
        /// AskToSave:                          Save changes if any after asking user
        /// </summary>
        /// <returns>
        ///  true:                              Changed saved or no changes
        ///  false:                             Not saved. User refused or error
        /// </returns>
        //
        //*********************************************************     
        bool AskToSave();

        //*********************************************************     
        //
        /// <summary>
        /// AfterSave:              Called after a save is done.
        /// </summary>
        /// <returns>
        ///     false:              Cancel
        ///     true:               Continue
        /// </returns>
        //  
        //*********************************************************     
        bool AfterSave();

        //*********************************************************     
        //
        /// <summary>
        /// AfterCommit:            Called after a database operation is done.
        /// </summary>
        /// <param name="eReason">  Reason of the commit</param>
        //  
        //*********************************************************     
        void AfterCommit(RepPP.RowsetState eReason);

        //*********************************************************     
        //
        /// <summary>
        /// SaveAllLines:           Save all dirty lines to DB
        /// </summary>
        /// <returns>
        ///     false:              Not saved
        ///     true:               Saved
        /// </returns>
        //  
        //*********************************************************     
        bool SaveAllLines();

        //******************************************************************
        //
        /// <summary>
        /// CheckIfDataChangedSinceRead:    Checks if the data has changed in the DB since it has been read
        /// </summary>
        /// <param name="rowset">            RowsetDef instance containing the values</param>
        /// <param name="fldStamp">         FieldDef containing the date stamp</param>
        /// <param name="bDisplayMsg">      Flag indicating if a message box shoulb be displayed</param>
        /// <returns>
        /// true if changed, false if not
        /// </returns>
        //
        //******************************************************************
	    bool CheckIfDataChangedSinceRead(Rowset rowset, FieldDef fldStamp, bool bDisplayMsg);

        //*********************************************************     
        //
        /// <summary>
        /// Undo:                   Undo a logical operation
        /// </summary>
        //  
        //*********************************************************     
        void Undo();

        //*********************************************************     
        //
        /// <summary>
        /// DisplayFieldHelp        Display the help text of a field
        /// </summary>
        /// <param name="fldDef">      FieldDef object</param>
        /// <returns>
        ///      True for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        bool DisplayFieldHelp(FieldDef fldDef);

        //*********************************************************     
        //
        /// <summary>
        /// DisplayFieldHelp:    Display the help text of the current field
        /// </summary>
        /// <returns>
        ///      true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        bool DisplayFieldHelp();

        //*********************************************************     
        //
        /// <summary>
        /// DisplayFieldChoices: Display the list of choices of a field
        /// </summary>
        /// <param name="fldDef">   FieldDef object</param>
        /// <returns>
        ///      false:          No choice list proposed
        ///      true:           Choice list proposed
        /// </returns>
        //
        //*********************************************************     
        bool DisplayFieldChoices(FieldDef fldDef);

        //*********************************************************     
        //
        /// <summary>
        ///  DisplayFieldChoices:Display the list of choices of the current field
        /// </summary>
        /// <returns>
        ///      false:          No choice list proposed
        ///      true:           Choice list proposed
        /// </returns>
        //
        //*********************************************************     
        bool DisplayFieldChoices();

        //*********************************************************     
        //
        /// <summary>
        /// EraseFieldValue:     Erase value of 'fldDef' if permitted.
        /// </summary>
        ///  <param name="fldDef">  FieldDef to erase</param>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        bool EraseFieldValue(FieldDef fldDef);

        //*********************************************************     
        //
        /// <summary>
        /// EraseFieldValue:     Erase value of the current field
        /// </summary>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        bool EraseFieldValue();

        //*********************************************************     
        //
        /// <summary>
        /// DefaultFieldValue:   Set the value of 'fldDef' to its default value if permitted.
        /// </summary>
        ///  <param name="fldDef">  FieldDef object</param>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        bool DefaultFieldValue(FieldDef fldDef);

        //*********************************************************     
        //
        /// <summary>
        /// DefaultFieldValue:   Set the value of the current field to its default value if permitted.
        /// </summary>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        bool DefaultFieldValue();

        //*********************************************************     
        //
        /// <summary>
        ///  EraseAllFieldsValue:    Erase the value of the all fields of the current group.
        /// </summary>
        //
        //*********************************************************     
        void EraseAllFieldsValue();

        //*********************************************************     
        //
        /// <summary>
        ///  DefaultAllFieldsValue:    Set the value of the all fields of the current group to their default value.
        /// </summary>
        //
        //*********************************************************     
        void DefaultAllFieldsValue();

        //*********************************************************     
        //
        /// <summary>
        /// ActiveNodeHandler:     Gets the active card group handler
        /// </summary>
        //
        //*********************************************************     
        RowsetTreeDefNodeHandler ActiveNodeHandler { get; }

        //*********************************************************     
        //
        /// <summary>
        /// ExecCmdOnActiveNodeHandler: Execute the specified command on the active node handler if any
        /// </summary>
        /// <param name="eCmd">         Command to execute on the active node handler</param>
        //
        //*********************************************************     
        void ExecCmdOnActiveNodeHandler(sdHandlerCmd eCmd);

        //*********************************************************     
        //
        /// <summary>
        ///  GetApplication:    Get the rep++ application object
        /// </summary>
        //
        //*********************************************************     
        IApplication GetApplication();

        //*********************************************************     
        //
        /// <summary>
        ///  OnResetFieldError:  Trigger the ResetFieldError event
        /// </summary>
        //  
        //*********************************************************     
        void OnResetFieldError(System.EventArgs e);

        //*********************************************************     
        //
        /// <summary>
        ///  OnAdjustFieldState: Trigger the AdjustFieldState event
        /// </summary>
        //  
        //*********************************************************     
        void OnAdjustFieldState(System.EventArgs e);

        //*********************************************************     
        //
        /// <summary>
        /// SelectionBuffer:                Selection buffer
        /// </summary>
        //  
        //*********************************************************     
		ISelectionBuffer SelectionBuffer { get; set; }

        //*********************************************************     
        //
        /// <summary>
        /// Empty:                  Empty the content of the rowset tree
        /// </summary>
        //  
        //*********************************************************     
        void Empty();

        //*********************************************************     
        //
        /// <summary>
        /// SelectItem:                         Use by selection buffer to select an item using its primary key
        /// </summary>
        /// <param name="arrPK">                Primary key values</param>
        /// <returns>
        ///  true:                              Set openned
        ///  false:                             Set not openned
        /// </returns>
        //
        //*********************************************************     
        bool SelectItem(string[] arrPK);

        //*********************************************************     
        //
        /// <summary>
        /// SetCurrentLine:             Sets the current line
        /// </summary>
        /// <param name="iLine">        Line position</param>
        /// <param name="bAsk">         Ask to save unsaved data</param>
        /// <returns>
        ///     true                    New line selected
        ///     false                   Line not selected
        /// </returns>
        //  
        //*********************************************************     
        bool SetCurrentLine(int iLine, bool bAsk);

        //*********************************************************     
        //
        /// <summary>
        /// AskForDeleteB:          Ask before deleting a row
        /// </summary>
        /// <returns>
        ///  true:                  Accept
        ///  false:                 Refused
        /// </returns>
        //
        //*********************************************************     
        bool AskForDelete();

        //*********************************************************     
        //
        /// <summary>
        /// SetDefValue:                        Set the default value of the specified field
        /// </summary>
        /// <param name="arrFldDef">               Array of fields</param>
        /// <param name="arrValue">             Array of values</param>
        //
        //*********************************************************     
        void SetDefValue(FieldDef[] arrFldDef, string[] arrValue);

        //*********************************************************     
        //
        /// <summary>
        /// InsertNewLine:                      Insert a new root line
        /// </summary>
        /// <param name="bAtEnd">               true to insert at the end of the list</param>
        /// <param name="arrFldDef">               FieldDefs</param>
        /// <param name="arrValue">             FieldDef values</param>
        /// <returns>
        ///  true:                              Succeed
        ///  false:                             Failed
        /// </returns>
        //
        //*********************************************************     
        bool InsertNewLine(bool bAtEnd, FieldDef[] arrFldDef, string[] arrValue);

        //*********************************************************     
        //
        /// <summary>
        /// InsertNewLine:                      Insert a new root line
        /// </summary>
        /// <param name="rowsetSB">              Selection buffer rowset</param>
        /// <param name="bAtEnd">               true to insert at the end of the list</param>
        /// <param name="arrFldDef">               Array of fields</param>
        /// <param name="arrValue">             Array of values</param>
        /// <returns>
        ///  true:                              Succeed
        ///  false:                             Failed
        /// </returns>
        //
        //*********************************************************     
        bool InsertNewLine(Rowset rowsetSB, bool bAtEnd, FieldDef[] arrFldDef, string[] arrValue);

        //*********************************************************     
        //
        /// <summary>
        /// DeleteCurrentLine:          Deletes the current line
        /// </summary>
        /// <param name="bReselect">    true to select another row</param>
        /// <returns>
        ///     true:                   The line has been deleted
        ///     false:                  No line deleted
        /// </returns>
        //  
        //*********************************************************     
        bool DeleteCurrentLine(bool bReselect);
    }
}
