//
// Rep++ Framework for Windows Application
// Copyright (C) 2005 Consyst SQL Inc.
// All Rights Reserved
//
// This source code is only intended as a supplement to the
// Rep++ Studio products.
//
using System;
using System.Collections;
using RepPP;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
	/// <summary>
	/// Implements a delegator to RootRowsetTreeDefNodeHandler. The delegator is used to emulate
	/// multiple inheritance.
	/// </summary>
	public class RowsetTreeDefNodeHandlerDel : RowsetTreeDefNodeHandler {
	    /// <summary>Outer object to delegate to</summary>
	    IRowsetTreeDefNodeHandler               m_outer = null;

        //*********************************************************     
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="outer">                Object to which we delegate</param>
        /// <param name="nodeHandlerFather">    Father card group handler</param>
        /// <param name="node">                 RowsetTreeDef group attached with the card group handler</param>
        /// <param name="bSeeDeletedLine">      true to see deleted line
        ///                                     false not to see it</param>
        //  
        //*********************************************************     
        public RowsetTreeDefNodeHandlerDel(IRowsetTreeDefNodeHandler    outer,
                                   RowsetTreeDefNodeHandler     nodeHandlerFather,
                                   RowsetTreeDefNode            node,
                                   bool                 bSeeDeletedLine) : base(nodeHandlerFather, node, bSeeDeletedLine) {
            m_outer = outer;
        }
#line hidden

        //*********************************************************     
        //
        /// <summary>
        /// Handler:               Return a pointer on the card group handler
        /// </summary>
        //  
        //*********************************************************     
        public override RowsetTreeDefNodeHandler Handler {
            get {
                return ((m_outer == null) ? null : m_outer.Handler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RowsetTreeDefNode:                      RowsetTreeDef group handle by this class
        /// </summary>
        //  
        //*********************************************************     
        public override RowsetTreeDefNode RowsetTreeDefNode { 
            get {
                return ((m_outer == null) ? null : m_outer.RowsetTreeDefNode);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// FatherRowsetTreeDefNodeHandler:         Father card group handler
        /// </summary>
        //  
        //*********************************************************     
        public override RowsetTreeDefNodeHandler FatherRowsetTreeDefNodeHandler { 
            get {
                return ((m_outer == null) ? null : m_outer.FatherRowsetTreeDefNodeHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RootRowsetTreeDefNodeHandler:           Root card group handler
        /// </summary>
        //  
        //*********************************************************     
        public override RowsetTreeDefNodeHandlerRootBase RootRowsetTreeDefNodeHandler { 
            get {
                return ((m_outer == null) ? null : m_outer.RootRowsetTreeDefNodeHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Rowset:          Attached rowset
        /// </summary>
        //  
        //*********************************************************     
        public override Rowset Rowset {
            get {
                return ((m_outer == null) ? null : m_outer.Rowset);
            }
            set {
                if (m_outer != null) {
                    m_outer.Rowset = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// LineListHandler:                Handler for the lines
        /// </summary>
        //  
        //*********************************************************     
		public override ILineListHandler LineListHandler {
		    get {
                return ((m_outer == null) ? null : m_outer.LineListHandler);
		    }
		    set {
                if (m_outer != null) {
                    m_outer.LineListHandler = value;
                }
		    }
		}

        //*********************************************************     
        //
        /// <summary>
        /// CommandHolder:              Command holder
        /// </summary>
        //  
        //*********************************************************     
        public override CmdHolder CommandHolder {
            get {
                return ((m_outer == null) ? null : m_outer.CommandHolder);
            }
            set {
                if (m_outer != null) {
                    m_outer.CommandHolder = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// IsLineDeleted:      Returns true if the line or a father line is deleted
        /// </summary>
        //  
        //*********************************************************     
        public override bool IsLineDeleted {
            get {
                return ((m_outer == null) ? false : m_outer.IsLineDeleted);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// AllowInsert:            Insertion allowed if true
        /// </summary>
        //  
        //*********************************************************     
		public override sdAllow AllowInsert {
		    get {
                return ((m_outer == null) ? sdAllow.sdAllowAuto : m_outer.AllowInsert);
		    }
		    set {
                if (m_outer != null) {
                    m_outer.AllowInsert = value;
                }
		    }
		}

        //*********************************************************     
        //
        /// <summary>
        /// AllowUpdate:            Update allowed if true
        /// </summary>
        //  
        //*********************************************************     
        public override sdAllow AllowUpdate {
            get {
                return ((m_outer == null) ? sdAllow.sdAllowAuto : m_outer.AllowUpdate);
            }
            set {
                if (m_outer != null) {
                    m_outer.AllowUpdate = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// AllowDelete:        Delete allowed or not for this card group
        /// </summary>
        //  
        //*********************************************************     
        public override sdAllow AllowDelete {
            get {
                return ((m_outer == null) ? sdAllow.sdAllowAuto : m_outer.AllowDelete);
            }
            set {
                if (m_outer != null) {
                    m_outer.AllowDelete = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// EffectiveAllowInsert:       Return true if a new line can be inserted
        /// </summary>
        //  
        //*********************************************************     
        public override bool EffectiveAllowInsert {
            get {
                return ((m_outer == null) ? true : m_outer.EffectiveAllowInsert);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// EffectiveAllowUpdate:       Update allowed if true
        /// </summary>
        //  
        //*********************************************************     
        public override bool EffectiveAllowUpdate {
            get {
                return ((m_outer == null) ? true : m_outer.EffectiveAllowUpdate);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// EffectiveAllowDelete:       Delete allowed if true
        /// </summary>
        //  
        //*********************************************************     
        public override bool EffectiveAllowDelete {
            get {
                return ((m_outer == null) ? true : m_outer.EffectiveAllowDelete);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// FieldProtection:    Actual protection level of the fields
        /// </summary>
        //  
        //*********************************************************     
        public override RepPP.Toolkit.Util.FldSelectableState.SelStateE FieldProtection {
            get {
                return ((m_outer == null) ? RepPP.Toolkit.Util.FldSelectableState.SelStateE.SS_All : m_outer.FieldProtection);
            }
            set {
                if (m_outer != null) {
                    m_outer.FieldProtection = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetFieldProtection:     Set the field protection according to the context
        /// </summary>
        //  
        //*********************************************************     
        public override void SetFieldProtection() {
            if (m_outer != null) {
                m_outer.SetFieldProtection();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// FindRowsetTreeDefNodeHandler:       Find a card group handler using the attached group
        /// </summary>
        /// 
        /// <param name="rowsetDef">          RowsetDef</param>
        /// <returns> 
        /// RowsetTreeDef group handler or null if not found.
        /// </returns>
        //  
        //*********************************************************     
        public override RowsetTreeDefNodeHandler FindRowsetTreeDefNodeHandler(RowsetDef rowsetDef) {
            return ((m_outer == null) ? null : m_outer.FindRowsetTreeDefNodeHandler(rowsetDef));
        }

        //*********************************************************     
        //
        /// <summary>
        /// FindRowsetTreeDefNodeHandler:       Find a card group handler using the attached card group
        /// </summary>
        /// 
        /// <param name="node">         RowsetTreeDef group</param>
        /// <returns> 
        /// RowsetTreeDef group handler or null if not found.
        /// </returns>
        //  
        //*********************************************************     
        public override RowsetTreeDefNodeHandler FindRowsetTreeDefNodeHandler(RowsetTreeDefNode node) {
            return ((m_outer == null) ? null : m_outer.FindRowsetTreeDefNodeHandler(node));
        }

        //*********************************************************     
        //
        /// <summary>
        /// CurrentLine:            Gets the current line position
        /// </summary>
        //  
        //*********************************************************     
        public override int CurrentLine {
            get {
                return ((m_outer == null) ? -1 : m_outer.CurrentLine);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// LineCount:              Gets the current line count
        /// </summary>
        //
        //*********************************************************     
        public override int LineCount {
            get {
                return ((m_outer == null) ? 0 : m_outer.LineCount);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SBCurrentLine:          Gets the current line in the selection buffer
        /// </summary>
        //  
        //*********************************************************     
        public override int SBCurrentLine {
            get {
                return ((m_outer == null) ? -1 : m_outer.SBCurrentLine);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SBLineCount:            Gets the current line count in the selection buffer
        /// </summary>
        //
        //*********************************************************     
        public override int SBLineCount {
            get {
                return ((m_outer == null) ? 0 : m_outer.SBLineCount);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// TriggerResetFieldError: Reset the last error message from field validation
        /// </summary>
        //  
        //*********************************************************     
        public override void TriggerResetFieldError() {
            if (m_outer != null) {
                m_outer.TriggerResetFieldError();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnLineSelected:     Reset the last error message from field validation
        /// </summary>
        //  
        //*********************************************************     
		public override void OnLineSelected(RowsetTreeDefNodeHandler.LineSelectedEventArgs e) {
            if (m_outer != null) {
		        m_outer.OnLineSelected(e);
            }
		}

        //*********************************************************     
        //
        /// <summary>
        /// OnFatherLineSelected:   Trigger the FatherLineSelected event for this control
        /// </summary>
        /// <param name="e">        Event arguments</param>
        //  
        //*********************************************************     
        public override void OnFatherLineSelected(System.EventArgs e) {
            if (m_outer != null) {
                m_outer.OnFatherLineSelected(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnBeforeLineValidated:  Trigger the BeforeLineValidated event for this control
        ///                         and all child
        /// </summary>
        /// <param name="e">        Line validated event arguments</param>
        //  
        //*********************************************************     
        public override void OnBeforeLineValidated(RowsetTreeDefNodeHandler.LineValidationEventArgs e) {
            if (m_outer != null) {
                m_outer.OnBeforeLineValidated(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnAfterLineValidated:   Trigger the AfterLineValidated event for this control
        ///                         and all child.
        /// </summary>
        /// <param name="e">        Line validated event argument</param>
        //  
        //*********************************************************     
        public override void OnAfterLineValidated(RowsetTreeDefNodeHandler.LineValidationEventArgs e) {
            if (m_outer != null) {
                m_outer.OnAfterLineValidated(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetCommandHolderState:  Set the state of the command holder
        /// </summary>
        //  
        //*********************************************************     
        public override void SetCommandHolderState() {
            if (m_outer != null) {
                m_outer.SetCommandHolderState();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnAdjustState:      Raise the AdjustState event
        /// </summary>
        //  
        //*********************************************************     
        public override void OnAdjustState(EventArgs e) {
            if (m_outer != null) {
                m_outer.OnAdjustState(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnBeforeExecuteCmd:     Informs that a command is about to be executed
        /// </summary>
        /// <param name="e">        Line validated event</param>
        //  
        //*********************************************************     
        public override void OnBeforeExecuteCmd(RowsetTreeDefNodeHandler.ExecuteCmdEventArgs e) {
            if (m_outer != null) {
                m_outer.OnBeforeExecuteCmd(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnAfterExecuteCmd:      Informs that a command has been executed
        /// </summary>
        /// <param name="e">        Line validated event</param>
        //  
        //*********************************************************     
        public override void OnAfterExecuteCmd(RowsetTreeDefNodeHandler.ExecuteCmdEventArgs e) {
            if (m_outer != null) {
                m_outer.OnAfterExecuteCmd(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnErrorOccurred:    Trigger the ErrorOccurred event for this control
        ///                     and all child
        /// </summary>
        /// <param name="e">    Error Occurred event arguments</param>
        //  
        //*********************************************************     
        public override void OnErrorOccurred(RowsetTreeDefNodeHandler.ErrorOccurredEventArgs e) {
            if (m_outer != null) {
                m_outer.OnErrorOccurred(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// TriggerErrors:      Trigger the ErrorOccurred event 
        /// </summary>
        //  
        //*********************************************************     
        public override void TriggerErrors() {
            if (m_outer != null) {
                m_outer.TriggerErrors();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnDataConversion:   Trigger the DataConversion event for this control
        /// </summary>
        /// <param name="e">    Error Occurred event arguments</param>
        //  
        //*********************************************************     
        public override void OnDataConversion(RowsetTreeDefNodeHandler.DataConversionEventArgs e) {
            if (m_outer != null) {
                m_outer.OnDataConversion(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RaiseAdjustState:       Raise the adjust state for this control and father.
        /// </summary>
        //  
        //*********************************************************     
        public override void RaiseAdjustState() {
            if (m_outer != null) {
                m_outer.RaiseAdjustState();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseHandler:        Return a pointer on the card group handler
        /// </summary>
        //  
        //*********************************************************     
        public virtual RowsetTreeDefNodeHandler baseHandler {
            get {
                return(base.Handler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseCardGroup:              RowsetTreeDef group handle by this class
        /// </summary>
        //  
        //*********************************************************     
        public RowsetTreeDefNode baseCardGroup { 
            get {
                return(base.RowsetTreeDefNode);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseFatherCardGroupHandler: Father card group handler
        /// </summary>
        //  
        //*********************************************************     
        public RowsetTreeDefNodeHandler baseFatherCardGroupHandler { 
            get {
                return(base.FatherRowsetTreeDefNodeHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseRootCardGroupHandler:   Root card group handler
        /// </summary>
        //  
        //*********************************************************     
        public RowsetTreeDefNodeHandlerRootBase baseRootCardGroupHandler { 
            get {
                return(base.RootRowsetTreeDefNodeHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseGroupInstance:          Attached rowset
        /// </summary>
        //  
        //*********************************************************     
        public Rowset baseGroupInstance {
            get {
                return(base.Rowset);
            }
            set {
                base.Rowset = value;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseLineListHandler:                Handler for the lines
        /// </summary>
        //  
        //*********************************************************     
		public ILineListHandler baseLineListHandler {
		    get {
		        return(base.LineListHandler);
		    }
		    set {
		        base.LineListHandler = value;
		    }
		}

        //*********************************************************     
        //
        /// <summary>
        /// baseCommandHolder:              Command holder
        /// </summary>
        //  
        //*********************************************************     
        public CmdHolder baseCommandHolder {
            get {
                return(base.CommandHolder);
            }
            set {
                base.CommandHolder = value;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseIsLineDeleted:      Returns true if the line or a father line is deleted
        /// </summary>
        //  
        //*********************************************************     
        public bool baseIsLineDeleted {
            get {
                return(base.IsLineDeleted);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseAllowInsert:            Insertion allowed if true
        /// </summary>
        //  
        //*********************************************************     
		public sdAllow baseAllowInsert {
		    get {
		        return(base.AllowInsert);
		    }
		    set {
		        base.AllowInsert = value;
		    }
		}

        //*********************************************************     
        //
        /// <summary>
        /// baseAllowUpdate:            Update allowed if true
        /// </summary>
        //  
        //*********************************************************     
        public sdAllow baseAllowUpdate {
            get {
                return(base.AllowUpdate);
            }
            set {
                base.AllowUpdate = value;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseAllowDelete:        Delete allowed or not for this card group
        /// </summary>
        //  
        //*********************************************************     
        public sdAllow baseAllowDelete {
            get {
                return(base.AllowDelete);
            }
            set {
                base.AllowDelete = value;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseEffectiveAllowInsert:       Return true if a new line can be inserted
        /// </summary>
        //  
        //*********************************************************     
        public bool baseEffectiveAllowInsert {
            get {
                return(base.EffectiveAllowInsert);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseEffectiveAllowUpdate:       Update allowed if true
        /// </summary>
        //  
        //*********************************************************     
        public bool baseEffectiveAllowUpdate {
            get {
                return(base.EffectiveAllowUpdate);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseEffectiveAllowDelete:       Delete allowed if true
        /// </summary>
        //  
        //*********************************************************     
        public bool baseEffectiveAllowDelete {
            get {
                return(base.EffectiveAllowDelete);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseFieldProtection:    Actual protection level of the fields
        /// </summary>
        //  
        //*********************************************************     
        public RepPP.Toolkit.Util.FldSelectableState.SelStateE baseFieldProtection {
            get {
                return(base.FieldProtection);
            }
            set {
                base.FieldProtection = value;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSetFieldProtection:     Set the field protection according to the context
        /// </summary>
        //  
        //*********************************************************     
        public void baseSetFieldProtection() {
            base.SetFieldProtection();
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseFindCardGroupHandler:       Find a card group handler using the attached group
        /// </summary>
        /// 
        /// <param name="rowsetDef">          RowsetDef</param>
        /// <returns> 
        /// RowsetTreeDef group handler or null if not found.
        /// </returns>
        //  
        //*********************************************************     
        public RowsetTreeDefNodeHandler baseFindCardGroupHandler(RowsetDef rowsetDef) {
            return(base.FindRowsetTreeDefNodeHandler(rowsetDef));
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseFindCardGroupHandler:       Find a card group handler using the attached card group
        /// </summary>
        /// 
        /// <param name="node">         RowsetTreeDef group</param>
        /// <returns> 
        /// RowsetTreeDef group handler or null if not found.
        /// </returns>
        //  
        //*********************************************************     
        public RowsetTreeDefNodeHandler baseFindCardGroupHandler(RowsetTreeDefNode node) {
            return(base.FindRowsetTreeDefNodeHandler(node));
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseCurrentLine:            Gets the current line position
        /// </summary>
        //  
        //*********************************************************     
        public int baseCurrentLine {
            get {
                return(base.CurrentLine);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseLineCount:              Gets the current line count
        /// </summary>
        //
        //*********************************************************     
        public int baseLineCount {
            get {
                return(base.LineCount);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSBCurrentLine:          Gets the current line in the selection buffer
        /// </summary>
        //  
        //*********************************************************     
        public int baseSBCurrentLine {
            get {
                return(base.SBCurrentLine);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSBLineCount:            Gets the current line count in the selection buffer
        /// </summary>
        //
        //*********************************************************     
        public int baseSBLineCount {
            get {
                return(base.SBLineCount);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseTriggerResetFieldError: Reset the last error message from field validation
        /// </summary>
        //  
        //*********************************************************     
        public void baseTriggerResetFieldError() {
            base.TriggerResetFieldError();
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseOnLineSelected:     Reset the last error message from field validation
        /// </summary>
        //  
        //*********************************************************     
		public void baseOnLineSelected(RowsetTreeDefNodeHandler.LineSelectedEventArgs e) {
		    base.OnLineSelected(e);
		}

        //*********************************************************     
        //
        /// <summary>
        /// baseOnFatherLineSelected:   Trigger the FatherLineSelected event for this control
        /// </summary>
        /// <param name="e">        Event arguments</param>
        //  
        //*********************************************************     
        public void baseOnFatherLineSelected(System.EventArgs e) {
            base.OnFatherLineSelected(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseOnBeforeLineValidated:  Trigger the BeforeLineValidated event for this control
        ///                             and all child
        /// </summary>
        /// <param name="e">        Line validated event arguments</param>
        //  
        //*********************************************************     
        public void baseOnBeforeLineValidated(RowsetTreeDefNodeHandler.LineValidationEventArgs e) {
            base.OnBeforeLineValidated(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseOnAfterLineValidated:   Trigger the AfterLineValidated event for this control
        ///                             and all child.
        /// </summary>
        /// <param name="e">            Line validated event argument</param>
        //  
        //*********************************************************     
        public void baseOnAfterLineValidated(RowsetTreeDefNodeHandler.LineValidationEventArgs e) {
            base.OnAfterLineValidated(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSetCommandHolderState:  Set the state of the command holder
        /// </summary>
        //  
        //*********************************************************     
        public void baseSetCommandHolderState() {
            base.SetCommandHolderState();
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseOnAdjustState:      Raise the AdjustState event
        /// </summary>
        //  
        //*********************************************************     
        public void baseOnAdjustState(EventArgs e) {
            base.OnAdjustState(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseOnBeforeExecuteCmd:     Informs that a command is about to be executed
        /// </summary>
        /// <param name="e">            Line validated event</param>
        //  
        //*********************************************************     
        public void baseOnBeforeExecuteCmd(RowsetTreeDefNodeHandler.ExecuteCmdEventArgs e) {
            base.OnBeforeExecuteCmd(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseOnAfterExecuteCmd:      Informs that a command has been executed
        /// </summary>
        /// <param name="e">            Line validated event</param>
        //  
        //*********************************************************     
        public void baseOnAfterExecuteCmd(RowsetTreeDefNodeHandler.ExecuteCmdEventArgs e) {
            base.OnAfterExecuteCmd(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseOnErrorOccurred:    Trigger the ErrorOccurred event for this control
        ///                         and all child
        /// </summary>
        /// <param name="e">        Error Occurred event arguments</param>
        //  
        //*********************************************************     
        public void baseOnErrorOccurred(RowsetTreeDefNodeHandler.ErrorOccurredEventArgs e) {
            base.OnErrorOccurred(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseTriggerErrors:      Trigger the ErrorOccurred event 
        /// </summary>
        //  
        //*********************************************************     
        public void baseTriggerErrors() {
            base.TriggerErrors();
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseOnDataConversion:   Trigger the DataConversion event for this control
        /// </summary>
        /// <param name="e">        Error Occurred event arguments</param>
        //  
        //*********************************************************     
        public void baseOnDataConversion(RowsetTreeDefNodeHandler.DataConversionEventArgs e) {
            base.OnDataConversion(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseRaiseAdjustState:   Raise the adjust state for this control and father.
        /// </summary>
        //  
        //*********************************************************     
        public void baseRaiseAdjustState() {
            base.RaiseAdjustState();
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseCommitEdit:         Commit the edit before validating
        /// </summary>
        //  
        //*********************************************************     
        public void baseCommitEdit() {
            base.CommitEdit();
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseCommitAllEdit:      Commit the edit of all card group
        /// </summary>
        /// <param name="nodeHandler">  RowsetTreeDef group handler</param>
        //  
        //*********************************************************     
        public void baseCommitAllEdit(RowsetTreeDefNodeHandler nodeHandler) {
            base.CommitAllEdit(nodeHandler);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseValidateCurLine:        Validates the current line
        /// </summary>
        /// <returns>
        ///     false:                  Line not valid
        ///     true:                   Line valid
        /// </returns>
        //  
        //*********************************************************     
        public bool baseValidateCurLine() {
            return(base.ValidateCurLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSetCurrentLine:         Sets the current logical line
        /// </summary>
        /// <param name="iLine">        Logical or physical line</param>
        /// <returns>
        /// true                        New line selected
        /// false                       Line not selected
        /// </returns>
        //  
        //*********************************************************     
        public bool baseSetCurrentLine(int iLine) {
            return(base.SetCurrentLine(iLine));
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSelectFirstLine:    Select the first line
        /// </summary>
        /// <returns>
        ///     true:               The first line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public bool baseSelectFirstLine() {
            return(base.SelectFirstLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSelectPrevLine:     Select the previous line
        /// </summary>
        /// <returns>
        ///     true:               The previous line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public bool baseSelectPrevLine() {
            return(base.SelectPrevLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSelectNextLine:     Select the next line
        /// </summary>
        /// <returns>
        ///     true:               The next line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public bool baseSelectNextLine() {
            return(base.SelectNextLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSelectLastLine:     Select the last line
        /// </summary>
        /// <returns>
        ///     true:               The last line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public bool baseSelectLastLine() {
            return(base.SelectLastLine());
        }

		//*********************************************************     
		//
		/// <summary>
		/// baseInsertNewLine:      Insert a new line
		/// </summary>
		/// <param name="bAtEnd">   true to insert at the end</param>
		/// <returns>
		/// true:                   A new line has been inserted
		/// false:                  No line inserted
		/// </returns>
		//  
		//*********************************************************     
		public bool baseInsertNewLine(bool bAtEnd) {
		    return(base.InsertNewLine(bAtEnd));
		}

        //*********************************************************     
        //
        /// <summary>
        /// baseDuplicateLine:          Duplicate the current line
        /// </summary>
        /// <returns>
        /// true:                       The line has been duplicated
        /// false:                      No line inserted
        /// </returns>
        //  
        //*********************************************************     
        public bool baseDuplicateLine() {
            return(base.DuplicateLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseDeleteCurrentLine:      Deletes the current line
        /// </summary>
        /// <returns>
        ///     true:                   The line has been deleted
        ///     false:                  No line deleted
        /// </returns>
        //  
        //*********************************************************     
        public bool baseDeleteCurrentLine() {
            return(base.DeleteCurrentLine());            
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseHandleCommand:      Handle a command coming from a menu or a toolbar
        /// </summary>
        /// <param name="eCmd">     Command Id</param>
        /// <param name="bEvent">   true to trigger event</param>
        //  
        //*********************************************************     
        public void baseHandleCommand(sdHandlerCmd eCmd, bool bEvent) {
            base.HandleCommand(eCmd, bEvent);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseGetActionState:         Get the specified action state
        /// </summary>
        /// <param name="eAction">      Action</param>
        /// <returns>
        /// false:                      Disabled
        /// true:                       Enabled
        /// </returns>returns>
        //  
        //*********************************************************     
        public bool baseGetActionState(sdAction eAction) {
            return(base.GetActionState(eAction));
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseIsDirty:            Check if the card is dirty
        /// </summary>
        /// <returns>
        ///  true:                   Yes
        ///  false:                  No
        /// </returns>
        //
        //*********************************************************     
        public bool baseIsDirty() {
            return(base.IsDirty());
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseGetItemState:           Get the state of the currently selected item
        /// </summary>
        /// <returns>
        /// LineState
        /// </returns>
        //  
        //*********************************************************     
        public LineState baseGetItemState() {
            return(base.GetItemState());
        }
#line default
    }
}
