//
// Rep++ Framework for Windows Application
// Copyright (C) 2005 Consyst SQL Inc.
// All Rights Reserved
//
// This source code is only intended as a supplement to the
// Rep++ Studio products.
//
using System;
using System.Collections;
using RepPP;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
	/// <summary>
	/// Implements a delegator to RootCardGroupHandlerForSB. The delegator is used to emulate
	/// multiple inheritance.
	/// </summary>
	public class RowsetTreeNodeHandleRootForSBDel : RowsetTreeDefNodeHandlerRootForSB {
	    /// <summary>Outer object to delegate to</summary>
	    IRowsetTreeDefNodeHandlerRootForSB      m_outer = null;

        //*********************************************************     
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        ///
        /// <param name="outer">                outer object to delegate to</param>
        /// <param name="rowsetTreeDef">        Rowset tree definition to attach</param>
        /// <param name="bSeeDelLine">          true to see deleted line
        ///                                     false not to see it</param>
        /// <param name="bUndo">                true to support undo operation
        ///                                     false to not support undo operation</param>
        /// <param name="askForPrimaryKey">     Interface to ask for a primary key</param>
        /// <param name="selectionBuffer">      Interface to update selection buffer</param>
        //  
        //*********************************************************     
        public RowsetTreeNodeHandleRootForSBDel(IRowsetTreeDefNodeHandlerRootForSB  outer,
                                                RowsetTreeDef                       rowsetTreeDef,
                                                bool                                bSeeDelLine,
                                                bool                                bUndo,
                                                IAskForPrimaryKey                   askForPrimaryKey,
                                                ISelectionBuffer                    selectionBuffer) : base(rowsetTreeDef, bSeeDelLine, bUndo, askForPrimaryKey, selectionBuffer) {
            m_outer = outer;
        }
#line hidden

        //*********************************************************     
        //
        /// <summary>
        /// Handler:               Return a pointer on the card group handler
        /// </summary>
        //  
        //*********************************************************     
        public override RowsetTreeDefNodeHandler Handler {
            get {
                return ((m_outer == null) ? null : m_outer.Handler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RowsetTreeDefNode:                      RowsetTreeDef group handle by this class
        /// </summary>
        //  
        //*********************************************************     
        public override RowsetTreeDefNode RowsetTreeDefNode { 
            get {
                return ((m_outer == null) ? null : m_outer.RowsetTreeDefNode);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// FatherRowsetTreeDefNodeHandler:         Father card group handler
        /// </summary>
        //  
        //*********************************************************     
        public override RowsetTreeDefNodeHandler FatherRowsetTreeDefNodeHandler { 
            get {
                return ((m_outer == null) ? null : m_outer.FatherRowsetTreeDefNodeHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RootRowsetTreeDefNodeHandler:           Root card group handler
        /// </summary>
        //  
        //*********************************************************     
        public override RowsetTreeDefNodeHandlerRootBase RootRowsetTreeDefNodeHandler { 
            get {
                return ((m_outer == null) ? null : m_outer.RootRowsetTreeDefNodeHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Rowset:          Attached rowset
        /// </summary>
        //  
        //*********************************************************     
        public override Rowset Rowset {
            get {
                return ((m_outer == null) ? null : m_outer.Rowset);
            }
            set {
                if (m_outer != null) {
                    m_outer.Rowset = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// LineListHandler:                Handler for the lines
        /// </summary>
        //  
        //*********************************************************     
		public override ILineListHandler LineListHandler {
		    get {
                return ((m_outer == null) ? null : m_outer.LineListHandler);
		    }
		    set {
                if (m_outer != null) {
                    m_outer.LineListHandler = value;
                }
		    }
		}

        //*********************************************************     
        //
        /// <summary>
        /// CommandHolder:              Command holder
        /// </summary>
        //  
        //*********************************************************     
        public override CmdHolder CommandHolder {
            get {
                return ((m_outer == null) ? null : m_outer.CommandHolder);
            }
            set {
                if (m_outer != null) {
                    m_outer.CommandHolder = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// IsLineDeleted:      Returns true if the line or a father line is deleted
        /// </summary>
        //  
        //*********************************************************     
        public override bool IsLineDeleted {
            get {
                return ((m_outer == null) ? false : m_outer.IsLineDeleted);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// AllowInsert:            Insertion allowed if true
        /// </summary>
        //  
        //*********************************************************     
		public override sdAllow AllowInsert {
		    get {
                return ((m_outer == null) ? sdAllow.sdAllowAuto : m_outer.AllowInsert);
		    }
		    set {
                if (m_outer != null) {
                    m_outer.AllowInsert = value;
                }
		    }
		}

        //*********************************************************     
        //
        /// <summary>
        /// AllowUpdate:            Update allowed if true
        /// </summary>
        //  
        //*********************************************************     
        public override sdAllow AllowUpdate {
            get {
                return ((m_outer == null) ? sdAllow.sdAllowAuto : m_outer.AllowUpdate);
            }
            set {
                if (m_outer != null) {
                    m_outer.AllowUpdate = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// AllowDelete:        Delete allowed or not for this card group
        /// </summary>
        //  
        //*********************************************************     
        public override sdAllow AllowDelete {
            get {
                return ((m_outer == null) ? sdAllow.sdAllowAuto : m_outer.AllowDelete);
            }
            set {
                if (m_outer != null) {
                    m_outer.AllowDelete = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// EffectiveAllowInsert:       Return true if a new line can be inserted
        /// </summary>
        //  
        //*********************************************************     
        public override bool EffectiveAllowInsert {
            get {
                return ((m_outer == null) ? true : m_outer.EffectiveAllowInsert);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// EffectiveAllowUpdate:       Update allowed if true
        /// </summary>
        //  
        //*********************************************************     
        public override bool EffectiveAllowUpdate {
            get {
                return ((m_outer == null) ? true : m_outer.EffectiveAllowUpdate);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// EffectiveAllowDelete:       Delete allowed if true
        /// </summary>
        //  
        //*********************************************************     
        public override bool EffectiveAllowDelete {
            get {
                return ((m_outer == null) ? true : m_outer.EffectiveAllowDelete);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// FieldProtection:    Actual protection level of the fields
        /// </summary>
        //  
        //*********************************************************     
        public override RepPP.Toolkit.Util.FldSelectableState.SelStateE FieldProtection {
            get {
                return ((m_outer == null) ? RepPP.Toolkit.Util.FldSelectableState.SelStateE.SS_All : m_outer.FieldProtection);
            }
            set {
                if (m_outer != null) {
                    m_outer.FieldProtection = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetFieldProtection:     Set the field protection according to the context
        /// </summary>
        //  
        //*********************************************************     
        public override void SetFieldProtection() {
            if (m_outer != null) {
                m_outer.SetFieldProtection();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// FindRowsetTreeDefNodeHandler:       Find a card group handler using the attached group
        /// </summary>
        /// 
        /// <param name="rowsetDef">          RowsetDef</param>
        /// <returns> 
        /// RowsetTreeDef group handler or null if not found.
        /// </returns>
        //  
        //*********************************************************     
        public override RowsetTreeDefNodeHandler FindRowsetTreeDefNodeHandler(RowsetDef rowsetDef) {
            return ((m_outer == null) ? null : m_outer.FindRowsetTreeDefNodeHandler(rowsetDef));
        }

        //*********************************************************     
        //
        /// <summary>
        /// FindRowsetTreeDefNodeHandler:       Find a card group handler using the attached card group
        /// </summary>
        /// 
        /// <param name="node">         RowsetTreeDef group</param>
        /// <returns> 
        /// RowsetTreeDef group handler or null if not found.
        /// </returns>
        //  
        //*********************************************************     
        public override RowsetTreeDefNodeHandler FindRowsetTreeDefNodeHandler(RowsetTreeDefNode node) {
            return ((m_outer == null) ? null : m_outer.FindRowsetTreeDefNodeHandler(node));
        }

        //*********************************************************     
        //
        /// <summary>
        /// CurrentLine:            Gets the current line position
        /// </summary>
        //  
        //*********************************************************     
        public override int CurrentLine {
            get {
                return ((m_outer == null) ? -1 : m_outer.CurrentLine);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// LineCount:              Gets the current line count
        /// </summary>
        //
        //*********************************************************     
        public override int LineCount {
            get {
                return ((m_outer == null) ? 0 : m_outer.LineCount);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SBCurrentLine:          Gets the current line in the selection buffer
        /// </summary>
        //  
        //*********************************************************     
        public override int SBCurrentLine {
            get {
                return ((m_outer == null) ? -1 : m_outer.SBCurrentLine);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SBLineCount:            Gets the current line count in the selection buffer
        /// </summary>
        //
        //*********************************************************     
        public override int SBLineCount {
            get {
                return ((m_outer == null) ? 0 : m_outer.SBLineCount);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// TriggerResetFieldError: Reset the last error message from field validation
        /// </summary>
        //  
        //*********************************************************     
        public override void TriggerResetFieldError() {
            if (m_outer != null) {
                m_outer.TriggerResetFieldError();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnLineSelected:     Reset the last error message from field validation
        /// </summary>
        //  
        //*********************************************************     
		public override void OnLineSelected(RowsetTreeDefNodeHandler.LineSelectedEventArgs e) {
            if (m_outer != null) {
                m_outer.OnLineSelected(e);
            }
		}

        //*********************************************************     
        //
        /// <summary>
        /// OnFatherLineSelected:   Trigger the FatherLineSelected event for this control
        /// </summary>
        /// <param name="e">        Event arguments</param>
        //  
        //*********************************************************     
        public override void OnFatherLineSelected(System.EventArgs e) {
            if (m_outer != null) {
                m_outer.OnFatherLineSelected(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnBeforeLineValidated:  Trigger the BeforeLineValidated event for this control
        ///                         and all child
        /// </summary>
        /// <param name="e">        Line validated event arguments</param>
        //  
        //*********************************************************     
        public override void OnBeforeLineValidated(RowsetTreeDefNodeHandler.LineValidationEventArgs e) {
            if (m_outer != null) {
                m_outer.OnBeforeLineValidated(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnAfterLineValidated:   Trigger the AfterLineValidated event for this control
        ///                         and all child.
        /// </summary>
        /// <param name="e">        Line validated event argument</param>
        //  
        //*********************************************************     
        public override void OnAfterLineValidated(RowsetTreeDefNodeHandler.LineValidationEventArgs e) {
            if (m_outer != null) {
                m_outer.OnAfterLineValidated(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetCommandHolderState:  Set the state of the command holder
        /// </summary>
        //  
        //*********************************************************     
        public override void SetCommandHolderState() {
            if (m_outer != null) {
                m_outer.SetCommandHolderState();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnAdjustState:      Raise the AdjustState event
        /// </summary>
        //  
        //*********************************************************     
        public override void OnAdjustState(EventArgs e) {
            if (m_outer != null) {
                m_outer.OnAdjustState(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnBeforeExecuteCmd:     Informs that a command is about to be executed
        /// </summary>
        /// <param name="e">        Line validated event</param>
        //  
        //*********************************************************     
        public override void OnBeforeExecuteCmd(RowsetTreeDefNodeHandler.ExecuteCmdEventArgs e) {
            if (m_outer != null) {
                m_outer.OnBeforeExecuteCmd(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnAfterExecuteCmd:      Informs that a command has been executed
        /// </summary>
        /// <param name="e">        Line validated event</param>
        //  
        //*********************************************************     
        public override void OnAfterExecuteCmd(RowsetTreeDefNodeHandler.ExecuteCmdEventArgs e) {
            if (m_outer != null) {
                m_outer.OnAfterExecuteCmd(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnErrorOccurred:    Trigger the ErrorOccurred event for this control
        ///                     and all child
        /// </summary>
        /// <param name="e">    Error Occurred event arguments</param>
        //  
        //*********************************************************     
        public override void OnErrorOccurred(RowsetTreeDefNodeHandler.ErrorOccurredEventArgs e) {
            if (m_outer != null) {
                m_outer.OnErrorOccurred(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// TriggerErrors:      Trigger the ErrorOccurred event 
        /// </summary>
        //  
        //*********************************************************     
        public override void TriggerErrors() {
            if (m_outer != null) {
                m_outer.TriggerErrors();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnDataConversion:   Trigger the DataConversion event for this control
        /// </summary>
        /// <param name="e">    Error Occurred event arguments</param>
        //  
        //*********************************************************     
        public override void OnDataConversion(RowsetTreeDefNodeHandler.DataConversionEventArgs e) {
            if (m_outer != null) {
                m_outer.OnDataConversion(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RaiseAdjustState:       Raise the adjust state for this control and father.
        /// </summary>
        //  
        //*********************************************************     
        public override void RaiseAdjustState() {
            if (m_outer != null) {
                m_outer.RaiseAdjustState();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// CommitEdit:             Commit the edit before validating
        /// </summary>
        //  
        //*********************************************************     
        public override void CommitEdit() {
            if (m_outer != null) {
                m_outer.CommitEdit();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// CommitAllEdit:              Commit the edit of all card group
        /// </summary>
        /// <param name="nodeHandler">  RowsetTreeDef group handler</param>
        //  
        //*********************************************************     
        public override void CommitAllEdit(RowsetTreeDefNodeHandler nodeHandler) {
            if (m_outer != null) {
                m_outer.CommitAllEdit(nodeHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// ValidateCurLine:        Validates the current line
        /// </summary>
        /// <returns>
        ///     false:              Line not valid
        ///     true:               Line valid
        /// </returns>
        //  
        //*********************************************************     
        public override bool ValidateCurLine() {
            return ((m_outer == null) ? true : m_outer.ValidateCurLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetCurrentLine:             Sets the current logical line
        /// </summary>
        /// <param name="iLine">        Logical or physical line</param>
        /// <returns>
        /// true                        New line selected
        /// false                       Line not selected
        /// </returns>
        //  
        //*********************************************************     
        public override bool SetCurrentLine(int iLine) {
            return ((m_outer == null) ? true : m_outer.SetCurrentLine(iLine));
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectFirstLine:        Select the first line
        /// </summary>
        /// <returns>
        ///     true:               The first line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public override bool SelectFirstLine() {
            return ((m_outer == null) ? true : m_outer.SelectFirstLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectPrevLine:         Select the previous line
        /// </summary>
        /// <returns>
        ///     true:               The previous line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public override bool SelectPrevLine() {
            return ((m_outer == null) ? true : m_outer.SelectPrevLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectNextLine:         Select the next line
        /// </summary>
        /// <returns>
        ///     true:               The next line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public override bool SelectNextLine() {
            return ((m_outer == null) ? true : m_outer.SelectNextLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectLastLine:         Select the last line
        /// </summary>
        /// <returns>
        ///     true:               The last line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public override bool SelectLastLine() {
            return ((m_outer == null) ? true : m_outer.SelectLastLine());
        }

		//*********************************************************     
		//
		/// <summary>
		/// InsertNewLine:          Insert a new line
		/// </summary>
		/// <param name="bAtEnd">   true to insert at the end</param>
		/// <returns>
		/// true:                   A new line has been inserted
		/// false:                  No line inserted
		/// </returns>
		//  
		//*********************************************************     
		public override bool InsertNewLine(bool bAtEnd) {
            return ((m_outer == null) ? true : m_outer.InsertNewLine(bAtEnd));
		}

        //*********************************************************     
        //
        /// <summary>
        /// DuplicateLine:              Duplicate the current line
        /// </summary>
        /// <returns>
        /// true:                       The line has been duplicated
        /// false:                      No line inserted
        /// </returns>
        //  
        //*********************************************************     
        public override bool DuplicateLine() {
            return ((m_outer == null) ? true : m_outer.DuplicateLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// DeleteCurrentLine:      Deletes the current line
        /// </summary>
        /// <returns>
        ///     true:               The line has been deleted
        ///     false:              No line deleted
        /// </returns>
        //  
        //*********************************************************     
        public override bool DeleteCurrentLine() {
            return ((m_outer == null) ? true : m_outer.DeleteCurrentLine());            
        }

        //*********************************************************     
        //
        /// <summary>
        /// HandleCommand:          Handle a command coming from a menu or a toolbar
        /// </summary>
        /// <param name="eCmd">     Command Id</param>
        /// <param name="bEvent">   true to trigger event</param>
        //  
        //*********************************************************     
        public override void HandleCommand(sdHandlerCmd eCmd, bool bEvent) {
            if (m_outer != null) {
                m_outer.HandleCommand(eCmd, bEvent);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// GetActionState:             Get the specified action state
        /// </summary>
        /// <param name="eAction">      Action</param>
        /// <returns>
        /// false:                      Disabled
        /// true:                       Enabled
        /// </returns>returns>
        //  
        //*********************************************************     
        public override bool GetActionState(sdAction eAction) {
            return ((m_outer == null) ? true : m_outer.GetActionState(eAction));
        }

        //*********************************************************     
        //
        /// <summary>
        /// IsDirty:                Check if the card is dirty
        /// </summary>
        /// <returns>
        ///  true:                   Yes
        ///  false:                  No
        /// </returns>
        //
        //*********************************************************     
        public override bool IsDirty() {
            return ((m_outer == null) ? false : m_outer.IsDirty());
        }

        //*********************************************************     
        //
        /// <summary>
        /// GetItemState:           Get the state of the currently selected item
        /// </summary>
        /// <returns>
        /// LineState
        /// </returns>
        //  
        //*********************************************************     
        public override LineState GetItemState() {
            return ((m_outer == null) ? LineState.sdLineOld : m_outer.GetItemState());
        }

        //*********************************************************     
        //
        /// <summary>
        /// AutoCommit:             true if auto-commit is on
        /// </summary>
        //  
        //*********************************************************     
        public override bool AutoCommit {
            get {
                return ((m_outer == null) ? true : m_outer.AutoCommit);
            }
            set {
                if (m_outer != null) {
                    m_outer.AutoCommit = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// DataAccess:              Data access object
        /// </summary>
        //  
        //*********************************************************     
        public override RepPP.Toolkit.ITransactionDataAccess DataAccess {
            get {
                return ((m_outer == null) ? null : m_outer.DataAccess);
            }
            set {
                if (m_outer != null) {
                    m_outer.DataAccess = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RowsetTreeDef:                       Attached card
        /// </summary>
        //  
        //*********************************************************     
        public override RowsetTreeDef RowsetTreeDef {
            get {
                return ((m_outer == null) ? null : m_outer.RowsetTreeDef);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RowsetTree:           Attached rowset tree
        /// </summary>
        //  
        //*********************************************************     
        public override RowsetTree RowsetTree {
            get {
                return ((m_outer == null) ? null : m_outer.RowsetTree);
            }
            set {
                if (m_outer != null) {
                    m_outer.RowsetTree = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// CurrentFieldDef:           FieldDef currently selected
        /// </summary>
        //  
        //*********************************************************     
        public override FieldDef CurrentFieldDef {
            get {
                return ((m_outer == null) ? null : m_outer.CurrentFieldDef);
            }
            set {
                if (m_outer != null) {
                    m_outer.CurrentFieldDef = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// UndoRowsetTree:           RowsetTreeDef instance use to hold undo value
        /// </summary>
        //  
        //*********************************************************     
        public override RowsetTree UndoRowsetTree {
            get {
                return ((m_outer == null) ? null : m_outer.UndoRowsetTree);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Refresh:            Refresh the control
        /// </summary>
        //  
        //*********************************************************     
        public override void Refresh() {
            if (m_outer != null) {
                m_outer.Refresh();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SaveLine:                           Save changes for the current line
        /// </summary>
        /// <returns>
        ///  true:                              Changed saved or no changes
        ///  false:                             Not saved. User refused or error
        /// </returns>
        //
        //*********************************************************     
        public override bool SaveLine() {
            return ((m_outer == null) ? true : m_outer.SaveLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// AskToSave:                          Save changes if any after asking user
        /// </summary>
        /// <param name="bEnsureFatherExist">   If an operation on a child of this item is done and this item is new, we must not
        ///                                     accept to not save it.</param>
        /// <returns>
        ///  true:                              Changed saved or no changes
        ///  false:                             Not saved. User refused or error
        /// </returns>
        //
        //*********************************************************     
        public override bool AskToSave(bool bEnsureFatherExist) {
            return ((m_outer == null) ? true : m_outer.AskToSave(bEnsureFatherExist));
        }

        //*********************************************************     
        //
        /// <summary>
        /// AskToSave:                          Save changes if any after asking user
        /// </summary>
        /// <returns>
        ///  true:                              Changed saved or no changes
        ///  false:                             Not saved. User refused or error
        /// </returns>
        //
        //*********************************************************     
        public override bool AskToSave() {
            return ((m_outer == null) ? true : m_outer.AskToSave());
        }

        //*********************************************************     
        //
        /// <summary>
        /// AfterSave:              Called after a save is done.
        /// </summary>
        /// <returns>
        ///     false:              Cancel
        ///     true:               Continue
        /// </returns>
        //  
        //*********************************************************     
        public override bool AfterSave() {
            return ((m_outer == null) ? true : m_outer.AfterSave());
        }

        //*********************************************************     
        //
        /// <summary>
        /// AfterCommit:            Called after a database operation is done.
        /// </summary>
        /// <param name="eReason">  Reason of the commit</param>
        //  
        //*********************************************************     
        public override void AfterCommit(RepPP.RowsetState eReason) {
            if (m_outer != null) {
                m_outer.AfterCommit(eReason);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SaveAllLines:           Save all dirty lines to DB
        /// </summary>
        /// <returns>
        ///     false:              Not saved
        ///     true:               Saved
        /// </returns>
        //  
        //*********************************************************     
        public override bool SaveAllLines() {
            return ((m_outer == null) ? true : m_outer.SaveAllLines());
        }

        //******************************************************************
        //
        /// <summary>
        /// CheckIfDataChangedSinceRead:    Checks if the data has changed in the DB since it has been read
        /// </summary>
        /// <param name="rowset">            RowsetDef instance containing the values</param>
        /// <param name="fldStamp">         FieldDef containing the date stamp</param>
        /// <param name="bDisplayMsg">      Flag indicating if a message box shoulb be displayed</param>
        /// <returns>
        /// true if changed, false if not
        /// </returns>
        //
        //******************************************************************
	    public override bool CheckIfDataChangedSinceRead(Rowset rowset, FieldDef fldStamp, bool bDisplayMsg) {
            return ((m_outer == null) ? true : m_outer.CheckIfDataChangedSinceRead(rowset, fldStamp, bDisplayMsg));
	    }

        //*********************************************************     
        //
        /// <summary>
        /// Undo:                   Undo a logical operation
        /// </summary>
        //  
        //*********************************************************     
        public override void Undo() {
            if (m_outer != null) {
                m_outer.Undo();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// DisplayFieldHelp        Display the help text of a field
        /// </summary>
        /// <param name="fldDef">      FieldDef object</param>
        /// <returns>
        ///      True for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public override bool DisplayFieldHelp(FieldDef fldDef) {
            return ((m_outer == null) ? true : m_outer.DisplayFieldHelp(fldDef));
        }

        //*********************************************************     
        //
        /// <summary>
        /// DisplayFieldHelp:    Display the help text of the current field
        /// </summary>
        /// <returns>
        ///      true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public override bool DisplayFieldHelp() {
            return ((m_outer == null) ? true : m_outer.DisplayFieldHelp());
        }            

        //*********************************************************     
        //
        /// <summary>
        /// DisplayFieldChoices: Display the list of choices of a field
        /// </summary>
        /// <param name="fldDef">   FieldDef object</param>
        /// <returns>
        ///      false:          No choice list proposed
        ///      true:           Choice list proposed
        /// </returns>
        //
        //*********************************************************     
        public override bool DisplayFieldChoices(FieldDef fldDef) {
            return ((m_outer == null) ? true : m_outer.DisplayFieldChoices(fldDef));
        }

        //*********************************************************     
        //
        /// <summary>
        ///  DisplayFieldChoices:Display the list of choices of the current field
        /// </summary>
        /// <returns>
        ///      false:          No choice list proposed
        ///      true:           Choice list proposed
        /// </returns>
        //
        //*********************************************************     
        public override bool DisplayFieldChoices() {
            return ((m_outer == null) ? true : m_outer.DisplayFieldChoices());
        }

        //*********************************************************     
        //
        /// <summary>
        /// EraseFieldValue:     Erase value of 'fldDef' if permitted.
        /// </summary>
        ///  <param name="fldDef">  FieldDef to erase</param>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public override bool EraseFieldValue(FieldDef fldDef) {
            return ((m_outer == null) ? true : m_outer.EraseFieldValue(fldDef));
        }

        //*********************************************************     
        //
        /// <summary>
        /// EraseFieldValue:     Erase value of the current field
        /// </summary>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public override bool EraseFieldValue() {
            return ((m_outer == null) ? true : m_outer.EraseFieldValue());
        }

        //*********************************************************     
        //
        /// <summary>
        /// DefaultFieldValue:   Set the value of 'fldDef' to its default value if permitted.
        /// </summary>
        ///  <param name="fldDef">  FieldDef object</param>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public override bool DefaultFieldValue(FieldDef fldDef) {
            return ((m_outer == null) ? true : m_outer.DefaultFieldValue(fldDef));
        }

        //*********************************************************     
        //
        /// <summary>
        /// DefaultFieldValue:   Set the value of the current field to its default value if permitted.
        /// </summary>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public override bool DefaultFieldValue() {
            return ((m_outer == null) ? true : m_outer.DefaultFieldValue());
        }

        //*********************************************************     
        //
        /// <summary>
        ///  EraseAllFieldsValue:    Erase the value of the all fields of the current group.
        /// </summary>
        //
        //*********************************************************     
        public override void EraseAllFieldsValue() {
            if (m_outer != null) {
                m_outer.EraseAllFieldsValue();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        ///  DefaultAllFieldsValue:    Set the value of the all fields of the current group to their default value.
        /// </summary>
        //
        //*********************************************************     
        public override void DefaultAllFieldsValue() {
            if (m_outer != null) {
                m_outer.DefaultAllFieldsValue();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// ActiveNodeHandler:     Gets the active card group handler
        /// </summary>
        //
        //*********************************************************     
        public override RowsetTreeDefNodeHandler ActiveNodeHandler {
            get {
                return ((m_outer == null) ? null : m_outer.ActiveNodeHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// ExecCmdOnActiveNodeHandler:   Execute the specified command on the active card group if any
        /// </summary>
        /// <param name="eCmd">         Command to execute on the active card group</param>
        //
        //*********************************************************     
        public override void ExecCmdOnActiveNodeHandler(sdHandlerCmd eCmd) {
            if (m_outer != null) {
                m_outer.ExecCmdOnActiveNodeHandler(eCmd);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        ///  GetApplication:    Get the rep++ application object
        /// </summary>
        //
        //*********************************************************     
        public override IApplication GetApplication() {
            return ((m_outer == null) ? null : m_outer.GetApplication());
        }

        //*********************************************************     
        //
        /// <summary>
        ///  OnResetFieldError:  Trigger the ResetFieldError event
        /// </summary>
        //  
        //*********************************************************     
        public override void OnResetFieldError(System.EventArgs e) {
            if (m_outer != null) {
                m_outer.OnResetFieldError(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        ///  OnAdjustFieldState: Trigger the AdjustFieldState event
        /// </summary>
        //  
        //*********************************************************     
        public override void OnAdjustFieldState(System.EventArgs e) {
            if (m_outer != null) {
                m_outer.OnAdjustFieldState(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectionBuffer:                Selection buffer
        /// </summary>
        //  
        //*********************************************************     
		public override ISelectionBuffer SelectionBuffer {
		    get {
                return ((m_outer == null) ? null : m_outer.SelectionBuffer);
		    }
		    set {
                if (m_outer != null) {
                    m_outer.SelectionBuffer = value;
                }
		    }
		}

        //*********************************************************     
        //
        /// <summary>
        /// Empty:                  Empty the content of the rowset tree
        /// </summary>
        //  
        //*********************************************************     
        public override void Empty() {
            if (m_outer != null) {
                m_outer.Empty();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectItem:                         Use by selection buffer to select an item using its primary key
        /// </summary>
        /// <param name="arrPK">                Primary key values</param>
        /// <returns>
        ///  true:                              Set openned
        ///  false:                             Set not openned
        /// </returns>
        //
        //*********************************************************     
        public override bool SelectItem(string[] arrPK) {
            return ((m_outer == null) ? true : m_outer.SelectItem(arrPK));
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetCurrentLine:             Sets the current line
        /// </summary>
        /// <param name="iLine">        Line position</param>
        /// <param name="bAsk">         Ask to save unsaved data</param>
        /// <returns>
        ///     true                    New line selected
        ///     false                   Line not selected
        /// </returns>
        //  
        //*********************************************************     
        public override bool SetCurrentLine(int iLine, bool bAsk) {
            return ((m_outer == null) ? true : m_outer.SetCurrentLine(iLine, bAsk));
        }

        //*********************************************************     
        //
        /// <summary>
        /// AskForDeleteB:          Ask before deleting a row
        /// </summary>
        /// <returns>
        ///  true:                  Accept
        ///  false:                 Refused
        /// </returns>
        //
        //*********************************************************     
        public override bool AskForDelete() {
            return ((m_outer == null) ? true : m_outer.AskForDelete());
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetDefValue:                        Set the default value of the specified field
        /// </summary>
        /// <param name="arrFldDef">               Array of fields</param>
        /// <param name="arrValue">             Array of values</param>
        //
        //*********************************************************     
        public override void SetDefValue(FieldDef[] arrFldDef, string[] arrValue) {
            if (m_outer != null) {
                m_outer.SetDefValue(arrFldDef, arrValue);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// InsertNewLine:                      Insert a new line
        /// </summary>
        /// <param name="rowsetSB">              Selection buffer rowset</param>
        /// <param name="bAtEnd">               true to insert at the end of the list</param>
        /// <param name="arrFldDef">               Array of fields</param>
        /// <param name="arrValue">             Array of values</param>
        /// <returns>
        ///  true:                              Succeed
        ///  false:                             Failed
        /// </returns>
        //
        //*********************************************************     
        public override bool InsertNewLine(Rowset rowsetSB, bool bAtEnd, FieldDef[] arrFldDef, string[] arrValue) {
            return ((m_outer == null) ? true : m_outer.InsertNewLine(rowsetSB, bAtEnd, arrFldDef, arrValue));
        }

        //*********************************************************     
        //
        /// <summary>
        /// InsertNewLine:                      Insert a new root line
        /// </summary>
        /// <param name="bAtEnd">               true to insert at the end of the list</param>
        /// <param name="arrFldDef">               FieldDefs</param>
        /// <param name="arrValue">             FieldDef values</param>
        /// <returns>
        ///  true:                              Succeed
        ///  false:                             Failed
        /// </returns>
        //
        //*********************************************************     
        public override bool InsertNewLine(bool bAtEnd, FieldDef[] arrFldDef, string[] arrValue) {
            return ((m_outer == null) ? true : m_outer.InsertNewLine(bAtEnd, arrFldDef, arrValue));
        }

        //*********************************************************     
        //
        /// <summary>
        /// DeleteCurrentLine:          Deletes the current line
        /// </summary>
        /// <param name="bReselect">    true to select another row</param>
        /// <returns>
        ///     true:                   The line has been deleted
        ///     false:                  No line deleted
        /// </returns>
        //  
        //*********************************************************     
        public override bool DeleteCurrentLine(bool bReselect) {
            return ((m_outer == null) ? true : m_outer.DeleteCurrentLine(bReselect));
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseHandler:        Return a pointer on the card group handler
        /// </summary>
        //  
        //*********************************************************     
        public virtual RowsetTreeDefNodeHandler baseHandler {
            get {
                return(base.Handler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseCardGroup:              RowsetTreeDef group handle by this class
        /// </summary>
        //  
        //*********************************************************     
        public RowsetTreeDefNode baseCardGroup { 
            get {
                return(base.RowsetTreeDefNode);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseFatherCardGroupHandler: Father card group handler
        /// </summary>
        //  
        //*********************************************************     
        public RowsetTreeDefNodeHandler baseFatherCardGroupHandler { 
            get {
                return(base.FatherRowsetTreeDefNodeHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseRootCardGroupHandler:   Root card group handler
        /// </summary>
        //  
        //*********************************************************     
        public RowsetTreeDefNodeHandlerRootBase baseRootCardGroupHandler { 
            get {
                return(base.RootRowsetTreeDefNodeHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseGroupInstance:          Attached rowset
        /// </summary>
        //  
        //*********************************************************     
        public Rowset baseGroupInstance {
            get {
                return(base.Rowset);
            }
            set {
                base.Rowset = value;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseLineListHandler:                Handler for the lines
        /// </summary>
        //  
        //*********************************************************     
		public ILineListHandler baseLineListHandler {
		    get {
		        return(base.LineListHandler);
		    }
		    set {
		        base.LineListHandler = value;
		    }
		}

        //*********************************************************     
        //
        /// <summary>
        /// baseCommandHolder:              Command holder
        /// </summary>
        //  
        //*********************************************************     
        public CmdHolder baseCommandHolder {
            get {
                return(base.CommandHolder);
            }
            set {
                base.CommandHolder = value;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseIsLineDeleted:      Returns true if the line or a father line is deleted
        /// </summary>
        //  
        //*********************************************************     
        public bool baseIsLineDeleted {
            get {
                return(base.IsLineDeleted);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseAllowInsert:            Insertion allowed if true
        /// </summary>
        //  
        //*********************************************************     
		public sdAllow baseAllowInsert {
		    get {
		        return(base.AllowInsert);
		    }
		    set {
		        base.AllowInsert = value;
		    }
		}

        //*********************************************************     
        //
        /// <summary>
        /// baseAllowUpdate:            Update allowed if true
        /// </summary>
        //  
        //*********************************************************     
        public sdAllow baseAllowUpdate {
            get {
                return(base.AllowUpdate);
            }
            set {
                base.AllowUpdate = value;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseAllowDelete:        Delete allowed or not for this card group
        /// </summary>
        //  
        //*********************************************************     
        public sdAllow baseAllowDelete {
            get {
                return(base.AllowDelete);
            }
            set {
                base.AllowDelete = value;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseEffectiveAllowInsert:       Return true if a new line can be inserted
        /// </summary>
        //  
        //*********************************************************     
        public bool baseEffectiveAllowInsert {
            get {
                return(base.EffectiveAllowInsert);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseEffectiveAllowUpdate:       Update allowed if true
        /// </summary>
        //  
        //*********************************************************     
        public bool baseEffectiveAllowUpdate {
            get {
                return(base.EffectiveAllowUpdate);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseEffectiveAllowDelete:       Delete allowed if true
        /// </summary>
        //  
        //*********************************************************     
        public bool baseEffectiveAllowDelete {
            get {
                return(base.EffectiveAllowDelete);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseFieldProtection:    Actual protection level of the fields
        /// </summary>
        //  
        //*********************************************************     
        public RepPP.Toolkit.Util.FldSelectableState.SelStateE baseFieldProtection {
            get {
                return(base.FieldProtection);
            }
            set {
                base.FieldProtection = value;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSetFieldProtection:     Set the field protection according to the context
        /// </summary>
        //  
        //*********************************************************     
        public void baseSetFieldProtection() {
            base.SetFieldProtection();
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseFindCardGroupHandler:       Find a card group handler using the attached group
        /// </summary>
        /// 
        /// <param name="rowsetDef">          RowsetDef</param>
        /// <returns> 
        /// RowsetTreeDef group handler or null if not found.
        /// </returns>
        //  
        //*********************************************************     
        public RowsetTreeDefNodeHandler baseFindCardGroupHandler(RowsetDef rowsetDef) {
            return(base.FindRowsetTreeDefNodeHandler(rowsetDef));
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseFindCardGroupHandler:       Find a card group handler using the attached card group
        /// </summary>
        /// 
        /// <param name="node">         RowsetTreeDef group</param>
        /// <returns> 
        /// RowsetTreeDef group handler or null if not found.
        /// </returns>
        //  
        //*********************************************************     
        public RowsetTreeDefNodeHandler baseFindCardGroupHandler(RowsetTreeDefNode node) {
            return(base.FindRowsetTreeDefNodeHandler(node));
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseCurrentLine:            Gets the current line position
        /// </summary>
        //  
        //*********************************************************     
        public int baseCurrentLine {
            get {
                return(base.CurrentLine);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseLineCount:              Gets the current line count
        /// </summary>
        //
        //*********************************************************     
        public int baseLineCount {
            get {
                return(base.LineCount);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSBCurrentLine:          Gets the current line in the selection buffer
        /// </summary>
        //  
        //*********************************************************     
        public int baseSBCurrentLine {
            get {
                return(base.SBCurrentLine);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSBLineCount:            Gets the current line count in the selection buffer
        /// </summary>
        //
        //*********************************************************     
        public int baseSBLineCount {
            get {
                return(base.SBLineCount);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseTriggerResetFieldError: Reset the last error message from field validation
        /// </summary>
        //  
        //*********************************************************     
        public void baseTriggerResetFieldError() {
            base.TriggerResetFieldError();
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseOnLineSelected:     Reset the last error message from field validation
        /// </summary>
        //  
        //*********************************************************     
		public void baseOnLineSelected(RowsetTreeDefNodeHandler.LineSelectedEventArgs e) {
		    base.OnLineSelected(e);
		}

        //*********************************************************     
        //
        /// <summary>
        /// baseOnFatherLineSelected:   Trigger the FatherLineSelected event for this control
        /// </summary>
        /// <param name="e">        Event arguments</param>
        //  
        //*********************************************************     
        public void baseOnFatherLineSelected(System.EventArgs e) {
            base.OnFatherLineSelected(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseOnBeforeLineValidated:  Trigger the BeforeLineValidated event for this control
        ///                             and all child
        /// </summary>
        /// <param name="e">        Line validated event arguments</param>
        //  
        //*********************************************************     
        public void baseOnBeforeLineValidated(RowsetTreeDefNodeHandler.LineValidationEventArgs e) {
            base.OnBeforeLineValidated(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseOnAfterLineValidated:   Trigger the AfterLineValidated event for this control
        ///                             and all child.
        /// </summary>
        /// <param name="e">            Line validated event argument</param>
        //  
        //*********************************************************     
        public void baseOnAfterLineValidated(RowsetTreeDefNodeHandler.LineValidationEventArgs e) {
            base.OnAfterLineValidated(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSetCommandHolderState:  Set the state of the command holder
        /// </summary>
        //  
        //*********************************************************     
        public void baseSetCommandHolderState() {
            base.SetCommandHolderState();
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseOnAdjustState:      Raise the AdjustState event
        /// </summary>
        //  
        //*********************************************************     
        public void baseOnAdjustState(EventArgs e) {
            base.OnAdjustState(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseOnBeforeExecuteCmd:     Informs that a command is about to be executed
        /// </summary>
        /// <param name="e">            Line validated event</param>
        //  
        //*********************************************************     
        public void baseOnBeforeExecuteCmd(RowsetTreeDefNodeHandler.ExecuteCmdEventArgs e) {
            base.OnBeforeExecuteCmd(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseOnAfterExecuteCmd:      Informs that a command has been executed
        /// </summary>
        /// <param name="e">            Line validated event</param>
        //  
        //*********************************************************     
        public void baseOnAfterExecuteCmd(RowsetTreeDefNodeHandler.ExecuteCmdEventArgs e) {
            base.OnAfterExecuteCmd(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseOnErrorOccurred:    Trigger the ErrorOccurred event for this control
        ///                         and all child
        /// </summary>
        /// <param name="e">        Error Occurred event arguments</param>
        //  
        //*********************************************************     
        public void baseOnErrorOccurred(RowsetTreeDefNodeHandler.ErrorOccurredEventArgs e) {
            base.OnErrorOccurred(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseTriggerErrors:      Trigger the ErrorOccurred event 
        /// </summary>
        //  
        //*********************************************************     
        public void baseTriggerErrors() {
            base.TriggerErrors();
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseOnDataConversion:   Trigger the DataConversion event for this control
        /// </summary>
        /// <param name="e">        Error Occurred event arguments</param>
        //  
        //*********************************************************     
        public void baseOnDataConversion(RowsetTreeDefNodeHandler.DataConversionEventArgs e) {
            base.OnDataConversion(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseRaiseAdjustState:   Raise the adjust state for this control and father.
        /// </summary>
        //  
        //*********************************************************     
        public void baseRaiseAdjustState() {
            base.RaiseAdjustState();
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseCommitEdit:         Commit the edit before validating
        /// </summary>
        //  
        //*********************************************************     
        public void baseCommitEdit() {
            base.CommitEdit();
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseCommitAllEdit:      Commit the edit of all card group
        /// </summary>
        /// <param name="nodeHandler">  RowsetTreeDef group handler</param>
        //  
        //*********************************************************     
        public void baseCommitAllEdit(RowsetTreeDefNodeHandler nodeHandler) {
            base.CommitAllEdit(nodeHandler);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseValidateCurLine:        Validates the current line
        /// </summary>
        /// <returns>
        ///     false:                  Line not valid
        ///     true:                   Line valid
        /// </returns>
        //  
        //*********************************************************     
        public bool baseValidateCurLine() {
            return(base.ValidateCurLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSetCurrentLine:         Sets the current logical line
        /// </summary>
        /// <param name="iLine">        Logical or physical line</param>
        /// <returns>
        /// true                        New line selected
        /// false                       Line not selected
        /// </returns>
        //  
        //*********************************************************     
        public bool baseSetCurrentLine(int iLine) {
            return(base.SetCurrentLine(iLine));
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSelectFirstLine:    Select the first line
        /// </summary>
        /// <returns>
        ///     true:               The first line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public bool baseSelectFirstLine() {
            return(base.SelectFirstLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSelectPrevLine:     Select the previous line
        /// </summary>
        /// <returns>
        ///     true:               The previous line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public bool baseSelectPrevLine() {
            return(base.SelectPrevLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSelectNextLine:     Select the next line
        /// </summary>
        /// <returns>
        ///     true:               The next line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public bool baseSelectNextLine() {
            return(base.SelectNextLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSelectLastLine:     Select the last line
        /// </summary>
        /// <returns>
        ///     true:               The last line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public bool baseSelectLastLine() {
            return(base.SelectLastLine());
        }

		//*********************************************************     
		//
		/// <summary>
		/// baseInsertNewLine:      Insert a new line
		/// </summary>
		/// <param name="bAtEnd">   true to insert at the end</param>
		/// <returns>
		/// true:                   A new line has been inserted
		/// false:                  No line inserted
		/// </returns>
		//  
		//*********************************************************     
		public bool baseInsertNewLine(bool bAtEnd) {
		    return(base.InsertNewLine(bAtEnd));
		}

        //*********************************************************     
        //
        /// <summary>
        /// baseDuplicateLine:          Duplicate the current line
        /// </summary>
        /// <returns>
        /// true:                       The line has been duplicated
        /// false:                      No line inserted
        /// </returns>
        //  
        //*********************************************************     
        public bool baseDuplicateLine() {
            return(base.DuplicateLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseDeleteCurrentLine:      Deletes the current line
        /// </summary>
        /// <returns>
        ///     true:                   The line has been deleted
        ///     false:                  No line deleted
        /// </returns>
        //  
        //*********************************************************     
        public bool baseDeleteCurrentLine() {
            return(base.DeleteCurrentLine());            
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseHandleCommand:      Handle a command coming from a menu or a toolbar
        /// </summary>
        /// <param name="eCmd">     Command Id</param>
        /// <param name="bEvent">   true to trigger event</param>
        //  
        //*********************************************************     
        public void baseHandleCommand(sdHandlerCmd eCmd, bool bEvent) {
            base.HandleCommand(eCmd, bEvent);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseGetActionState:         Get the specified action state
        /// </summary>
        /// <param name="eAction">      Action</param>
        /// <returns>
        /// false:                      Disabled
        /// true:                       Enabled
        /// </returns>returns>
        //  
        //*********************************************************     
        public bool baseGetActionState(sdAction eAction) {
            return(base.GetActionState(eAction));
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseIsDirty:            Check if the card is dirty
        /// </summary>
        /// <returns>
        ///  true:                   Yes
        ///  false:                  No
        /// </returns>
        //
        //*********************************************************     
        public bool baseIsDirty() {
            return(base.IsDirty());
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseGetItemState:           Get the state of the currently selected item
        /// </summary>
        /// <returns>
        /// LineState
        /// </returns>
        //  
        //*********************************************************     
        public LineState baseGetItemState() {
            return(base.GetItemState());
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseAutoCommit:             true if auto-commit is on
        /// </summary>
        //  
        //*********************************************************     
        public bool baseAutoCommit {
            get {
                return(base.AutoCommit);
            }
            set {
                base.AutoCommit = value;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseDataAccess:              Data access object
        /// </summary>
        //  
        //*********************************************************     
        public RepPP.Toolkit.ITransactionDataAccess baseDataAccess {
            get {
                return(base.DataAccess);
            }
            set {
                base.DataAccess = value;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseCard:                   Attached card
        /// </summary>
        //  
        //*********************************************************     
        public RowsetTreeDef baseCard {
            get {
                return(base.RowsetTreeDef);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseCardInstance:           Attached rowset tree
        /// </summary>
        //  
        //*********************************************************     
        public RowsetTree baseCardInstance {
            get {
                return(base.RowsetTree);
            }
            set {
                base.RowsetTree = value;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseCurrentField:           FieldDef currently selected
        /// </summary>
        //  
        //*********************************************************     
        public FieldDef baseCurrentField {
            get {
                return(base.CurrentFieldDef);
            }
            set {
                base.CurrentFieldDef = value;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseUndoCardInstance:       RowsetTreeDef instance use to hold undo value
        /// </summary>
        //  
        //*********************************************************     
        public RowsetTree baseUndoCardInstance {
            get {
                return(base.UndoRowsetTree);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseRefresh:                Refresh the control
        /// </summary>
        //  
        //*********************************************************     
        public void baseRefresh() {
            base.Refresh();
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSaveLine:                       Save changes for the current line
        /// </summary>
        /// <returns>
        ///  true:                              Changed saved or no changes
        ///  false:                             Not saved. User refused or error
        /// </returns>
        //
        //*********************************************************     
        public bool baseSaveLine() {
            return(base.SaveLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseAskToSave:                      Save changes if any after asking user
        /// </summary>
        /// <param name="bEnsureFatherExist">   If an operation on a child of this item is done and this item is new, we must not
        ///                                     accept to not save it.</param>
        /// <returns>
        ///  true:                              Changed saved or no changes
        ///  false:                             Not saved. User refused or error
        /// </returns>
        //
        //*********************************************************     
        public bool baseAskToSave(bool bEnsureFatherExist) {
            return(base.AskToSave(bEnsureFatherExist));
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseAskToSave:                      Save changes if any after asking user
        /// </summary>
        /// <returns>
        ///  true:                              Changed saved or no changes
        ///  false:                             Not saved. User refused or error
        /// </returns>
        //
        //*********************************************************     
        public bool baseAskToSave() {
            return(base.AskToSave());
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseAfterSave:          Called after a save is done.
        /// </summary>
        /// <returns>
        ///     false:              Cancel
        ///     true:               Continue
        /// </returns>
        //  
        //*********************************************************     
        public bool baseAfterSave() {
            return(base.AfterSave());
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseAfterCommit:        Called after a database operation is done.
        /// </summary>
        /// <param name="eReason">  Reason of the commit</param>
        //  
        //*********************************************************     
        public void baseAfterCommit(RepPP.RowsetState eReason) {
            base.AfterCommit(eReason);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSaveAllLines:       Save all dirty lines to DB
        /// </summary>
        /// <returns>
        ///     false:              Not saved
        ///     true:               Saved
        /// </returns>
        //  
        //*********************************************************     
        public bool baseSaveAllLines() {
            return(base.SaveAllLines());
        }

        //******************************************************************
        //
        /// <summary>
        /// baseCheckIfDataChangedSinceRead:    Checks if the data has changed in the DB since it has been read
        /// </summary>
        /// <param name="rowset">                RowsetDef instance containing the values</param>
        /// <param name="fldStamp">             FieldDef containing the date stamp</param>
        /// <param name="bDisplayMsg">          Flag indicating if a message box shoulb be displayed</param>
        /// <returns>
        /// true if changed, false if not
        /// </returns>
        //
        //******************************************************************
	    bool baseCheckIfDataChangedSinceRead(Rowset rowset, FieldDef fldStamp, bool bDisplayMsg) {
	        return(base.CheckIfDataChangedSinceRead(rowset, fldStamp, bDisplayMsg));
	    }

        //*********************************************************     
        //
        /// <summary>
        /// baseUndo:               Undo a logical operation
        /// </summary>
        //  
        //*********************************************************     
        public void baseUndo() {
            base.Undo();
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseDisplayFieldHelp    Display the help text of a field
        /// </summary>
        /// <param name="fldDef">      FieldDef object</param>
        /// <returns>
        ///      True for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public bool baseDisplayFieldHelp(FieldDef fldDef) {
            return(base.DisplayFieldHelp(fldDef));
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseDisplayFieldHelp:   Display the help text of the current field
        /// </summary>
        /// <returns>
        ///      true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public bool baseDisplayFieldHelp() {
            return(base.DisplayFieldHelp());
        }            

        //*********************************************************     
        //
        /// <summary>
        /// baseDisplayFieldChoices: Display the list of choices of a field
        /// </summary>
        /// <param name="fldDef">      FieldDef object</param>
        /// <returns>
        ///      false:             No choice list proposed
        ///      true:              Choice list proposed
        /// </returns>
        //
        //*********************************************************     
        public bool baseDisplayFieldChoices(FieldDef fldDef) {
            return(base.DisplayFieldChoices(fldDef));
        }

        //*********************************************************     
        //
        /// <summary>
        ///  baseDisplayFieldChoices:Display the list of choices of the current field
        /// </summary>
        /// <returns>
        ///      false:             No choice list proposed
        ///      true:              Choice list proposed
        /// </returns>
        //
        //*********************************************************     
        public bool baseDisplayFieldChoices() {
            return(base.DisplayFieldChoices());
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseEraseFieldValue:    Erase value of 'fldDef' if permitted.
        /// </summary>
        ///  <param name="fldDef">     FieldDef to erase</param>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public bool baseEraseFieldValue(FieldDef fldDef) {
            return(base.EraseFieldValue(fldDef));
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseEraseFieldValue:    Erase value of the current field
        /// </summary>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public bool baseEraseFieldValue() {
            return(base.EraseFieldValue());
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseDefaultFieldValue:  Set the value of 'fldDef' to its default value if permitted.
        /// </summary>
        ///  <param name="fldDef">     FieldDef object</param>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public bool baseDefaultFieldValue(FieldDef fldDef) {
            return(base.DefaultFieldValue(fldDef));
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseDefaultFieldValue:   Set the value of the current field to its default value if permitted.
        /// </summary>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public bool baseDefaultFieldValue() {
            return(base.DefaultFieldValue());
        }

        //*********************************************************     
        //
        /// <summary>
        ///  baseEraseAllFieldsValue:   Erase the value of the all fields of the current group.
        /// </summary>
        //
        //*********************************************************     
        public void baseEraseAllFieldsValue() {
            base.EraseAllFieldsValue();
        }

        //*********************************************************     
        //
        /// <summary>
        ///  baseDefaultAllFieldsValue: Set the value of the all fields of the current group to their default value.
        /// </summary>
        //
        //*********************************************************     
        public void baseDefaultAllFieldsValue() {
            base.DefaultAllFieldsValue();
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseActiveCardGroupHandler:     Gets the active card group handler
        /// </summary>
        //
        //*********************************************************     
        public RowsetTreeDefNodeHandler baseActiveCardGroupHandler {
            get {
                return(base.ActiveNodeHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseExecCmdOnActiveCardGroup:   Execute the specified command on the active card group if any
        /// </summary>
        /// <param name="eCmd">             Command to execute on the active card group</param>
        //
        //*********************************************************     
        public void baseExecCmdOnActiveCardGroup(sdHandlerCmd eCmd) {
            base.ExecCmdOnActiveNodeHandler(eCmd);
        }

        //*********************************************************     
        //
        /// <summary>
        ///  baseGetApplication:    Get the rep++ application object
        /// </summary>
        //
        //*********************************************************     
        public IApplication baseGetApplication() {
            return(base.GetApplication());
        }

        //*********************************************************     
        //
        /// <summary>
        ///  baseOnResetFieldError: Trigger the ResetFieldError event
        /// </summary>
        //  
        //*********************************************************     
        public void baseOnResetFieldError(System.EventArgs e) {
            base.OnResetFieldError(e);
        }

        //*********************************************************     
        //
        /// <summary>
        ///  baseOnAdjustFieldState: Trigger the AdjustFieldState event
        /// </summary>
        //  
        //*********************************************************     
        public void baseOnAdjustFieldState(System.EventArgs e) {
            base.OnAdjustFieldState(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSelectionBuffer:                Selection buffer
        /// </summary>
        //  
        //*********************************************************     
		public ISelectionBuffer baseSelectionBuffer {
		    get {
		        return(base.SelectionBuffer);
		    }
		    set {
		        base.SelectionBuffer = value;
		    }
		}

        //*********************************************************     
        //
        /// <summary>
        /// baseEmpty:                  Empty the content of the rowset tree
        /// </summary>
        //  
        //*********************************************************     
        public void baseEmpty() {
            base.Empty();
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSelectItem:                     Use by selection buffer to select an item using its primary key
        /// </summary>
        /// <param name="arrPK">                Primary key values</param>
        /// <returns>
        ///  true:                              Set openned
        ///  false:                             Set not openned
        /// </returns>
        //
        //*********************************************************     
        public bool baseSelectItem(string[] arrPK) {
            return(base.SelectItem(arrPK));
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSetCurrentLine:         Sets the current line
        /// </summary>
        /// <param name="iLine">        Line position</param>
        /// <param name="bAsk">         Ask to save unsaved data</param>
        /// <returns>
        ///     true                    New line selected
        ///     false                   Line not selected
        /// </returns>
        //  
        //*********************************************************     
        public bool baseSetCurrentLine(int iLine, bool bAsk) {
            return(base.SetCurrentLine(iLine, bAsk));
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseAskForDeleteB:          Ask before deleting a row
        /// </summary>
        /// <returns>
        ///  true:                      Accept
        ///  false:                     Refused
        /// </returns>
        //
        //*********************************************************     
        public bool baseAskForDelete() {
            return(base.AskForDelete());
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseSetDefValue:                    Set the default value of the specified field
        /// </summary>
        /// <param name="arrFldDef">               Array of fields</param>
        /// <param name="arrValue">             Array of values</param>
        //
        //*********************************************************     
        public virtual void baseSetDefValue(FieldDef[] arrFldDef, string[] arrValue) {
            base.SetDefValue(arrFldDef, arrValue);
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseInsertNewLine:                  Insert a new line
        /// </summary>
        /// <param name="rowsetSB">              Selection buffer rowset</param>
        /// <param name="bAtEnd">               true to insert at the end of the list</param>
        /// <param name="arrFldDef">               Array of fields</param>
        /// <param name="arrValue">             Array of values</param>
        /// <returns>
        ///  true:                              Succeed
        ///  false:                             Failed
        /// </returns>
        //
        //*********************************************************     
        public bool baseInsertNewLine(Rowset rowsetSB, bool bAtEnd, FieldDef[] arrFldDef, string[] arrValue) {
            return(base.InsertNewLine(rowsetSB, bAtEnd, arrFldDef, arrValue));
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseInsertNewLine:                  Insert a new root line
        /// </summary>
        /// <param name="bAtEnd">               true to insert at the end of the list</param>
        /// <param name="arrFldDef">               FieldDefs</param>
        /// <param name="arrValue">             FieldDef values</param>
        /// <returns>
        ///  true:                              Succeed
        ///  false:                             Failed
        /// </returns>
        //
        //*********************************************************     
        public bool baseInsertNewLine(bool bAtEnd, FieldDef[] arrFldDef, string[] arrValue) {
            return(base.InsertNewLine(bAtEnd, arrFldDef, arrValue));
        }

        //*********************************************************     
        //
        /// <summary>
        /// baseDeleteCurrentLine:      Deletes the current line
        /// </summary>
        /// <param name="bReselect">    true to select another row</param>
        /// <returns>
        ///     true:                   The line has been deleted
        ///     false:                  No line deleted
        /// </returns>
        //  
        //*********************************************************     
        public bool baseDeleteCurrentLine(bool bReselect) {
            return(base.DeleteCurrentLine(bReselect));
        }
#line default
    }
}
