//
// Rep++ Framework for Windows Application
// Copyright (C) 2005 Consyst SQL Inc.
// All Rights Reserved
//
// This source code is only intended as a supplement to the
// Rep++ Studio products.
//
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using RepPP;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
	/// <summary>
	/// Form using a delegate class to also inherit from RowsetTreeDefNodeHandler class
	/// </summary>
	public class frmRowsetTreeDefNodeHandler : System.Windows.Forms.Form, IRowsetTreeDefNodeHandler {
	    /// <summary>Event sent each time a new line is selected for this card group</summary>
	    public event RowsetTreeDefNodeHandler.LineSelectedEventHandler      LineSelected;
        /// <summary>Father line changed. Select rowset</summary>
        public event System.EventHandler                                    FatherLineSelected;
	    /// <summary>Event sent before a line is validated</summary>
	    public event RowsetTreeDefNodeHandler.LineValidationEventHandler    BeforeLineValidated;
	    /// <summary>Event sent before a line is validated</summary>
	    public event RowsetTreeDefNodeHandler.LineValidationEventHandler    AfterLineValidated;
        /// <summary>State must be adjust</summary>
        public event System.EventHandler                                    AdjustState;
        /// <summary>Called before a command is executed</summary>
        public event RowsetTreeDefNodeHandler.ExecuteCmdEventHandler        BeforeExecuteCmd;
        /// <summary>Called after a command is executed</summary>
        public event RowsetTreeDefNodeHandler.ExecuteCmdEventHandler        AfterExecuteCmd;
        /// <summary>An error Occurred</summary>
        public event RowsetTreeDefNodeHandler.ErrorOccurredEventHandler     ErrorOccurred;
        /// <summary>Called when data conversion must be done</summary>
        public event RowsetTreeDefNodeHandler.DataConversionEventHandler    DataConversion;
		/// <summary>Delegator to the RootCardGroupHandleForSB (simulate multiple inheritance)</summary>
		private RowsetTreeDefNodeHandlerDel                                 m_del = null;
		/// <summary>Required designer variable.</summary>
		private System.ComponentModel.Container                             components = null;

        //*********************************************************     
        //
        /// <summary>
        /// ctor:                               Class constructor. Use in design mode
        /// </summary>
        //  
        //*********************************************************     
		public frmRowsetTreeDefNodeHandler() {
			InitializeComponent();
		}

        //*********************************************************     
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="nodeHandlerFather">    Father card group handler</param>
        /// <param name="selBuf">               Selection Buffer</param>
        /// <param name="node">                 RowsetTreeDef group attached with the card group handler</param>
        /// <param name="bSeeDeletedLine">      true to see deleted line</param>
        //  
        //*********************************************************     
        public frmRowsetTreeDefNodeHandler(RowsetTreeDefNodeHandler     nodeHandlerFather,
                                           ISelectionBuffer             selBuf,
                                           RowsetTreeDefNode            node,
                                           bool                         bSeeDeletedLine) : this() {
            m_del = new RowsetTreeDefNodeHandlerDel(this, nodeHandlerFather, node, bSeeDeletedLine);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Dispose:                            Dispose the form
        /// </summary>
        /// <param name="bDisposing">           true if dispose, false if GC</param>
        //  
        //*********************************************************     
		protected override void Dispose(bool bDisposing) {
			base.Dispose(bDisposing);
            if (bDisposing) {
				if (m_del != null) {
				    m_del.Dispose();
				    m_del = null;
				}
				if (components != null) {
					components.Dispose();
				}
			}
		}
#line hidden

        //*********************************************************     
        //
        /// <summary>
        /// Handler:                    Return a pointer on the card group handler
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RowsetTreeDefNodeHandler Handler {
            get {
                return((m_del == null) ? null : m_del.baseHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RowsetTreeDefNode:                      Rowset Tree definition node handle by this class
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RowsetTreeDefNode RowsetTreeDefNode { 
            get {
                return((m_del == null) ? null : m_del.baseCardGroup);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// FatherRowsetTreeDefNodeHandler:         Rowset tree definition node handler
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RowsetTreeDefNodeHandler FatherRowsetTreeDefNodeHandler { 
            get {
                return((m_del == null) ? null : m_del.baseFatherCardGroupHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RootRowsetTreeDefNodeHandler:           Root rowset tree definition node handler
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RowsetTreeDefNodeHandlerRootBase RootRowsetTreeDefNodeHandler { 
            get {
                return((m_del == null) ? null : m_del.baseRootCardGroupHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Rowset:          Attached rowset
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual Rowset Rowset {
            get {
                return((m_del == null) ? null : m_del.baseGroupInstance);
            }
            set {
                if (m_del != null) {
                    m_del.baseGroupInstance = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// LineListHandler:                Handler for the lines
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public virtual ILineListHandler LineListHandler {
            get {
		        return((m_del == null) ? null : m_del.baseLineListHandler);
		    }
		    set {
                if (m_del != null) {
                    m_del.baseLineListHandler = value;
                }
		    }
		}

        //*********************************************************     
        //
        /// <summary>
        /// CommandHolder:              Command holder
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual CmdHolder CommandHolder {
            get {
                return((m_del == null) ? null : m_del.baseCommandHolder);
            }
            set {
                if (m_del != null) {
                    m_del.baseCommandHolder = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// IsLineDeleted:          Returns true if the line or a father line is deleted
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual bool IsLineDeleted {
            get {
                return((m_del == null) ? false : m_del.baseIsLineDeleted);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// AllowInsert:            Insertion allowed if true
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual sdAllow AllowInsert {
		    get {
		        return((m_del == null) ? sdAllow.sdAllowAuto : m_del.baseAllowInsert);
		    }
		    set {
                if (m_del != null) {
                    m_del.baseAllowInsert = value;
                }
		    }
		}

        //*********************************************************     
        //
        /// <summary>
        /// AllowUpdate:            Update allowed if true
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual sdAllow AllowUpdate {
            get {
                return ((m_del == null) ? sdAllow.sdAllowAuto : m_del.baseAllowUpdate);
            }
            set {
                if (m_del != null) {
                    m_del.baseAllowUpdate = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// AllowDelete:        Delete allowed or not for this node
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual sdAllow AllowDelete {
            get {
                return((m_del == null) ? sdAllow.sdAllowAuto : m_del.baseAllowDelete);
            }
            set {
                if (m_del != null) {
                    m_del.baseAllowDelete = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// EffectiveAllowInsert:       Return true if a new line can be inserted
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual bool EffectiveAllowInsert {
            get {
                return((m_del == null) ? true : m_del.baseEffectiveAllowInsert);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// EffectiveAllowUpdate:       Update allowed if true
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual bool EffectiveAllowUpdate {
            get {
                return((m_del == null) ? true : m_del.baseEffectiveAllowUpdate);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// EffectiveAllowDelete:       Delete allowed if true
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual bool EffectiveAllowDelete {
            get {
                return((m_del == null) ? true : m_del.baseEffectiveAllowDelete);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// FieldProtection:    Actual protection level of the fields
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RepPP.Toolkit.Util.FldSelectableState.SelStateE FieldProtection {
            get {
                return((m_del == null) ? RepPP.Toolkit.Util.FldSelectableState.SelStateE.SS_All : m_del.baseFieldProtection);
            }
            set {
                if (m_del != null) {
                    m_del.baseFieldProtection = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetFieldProtection:     Set the field protection according to the context
        /// </summary>
        //  
        //*********************************************************     
        public virtual void SetFieldProtection() {
            if (m_del != null) {
                m_del.baseSetFieldProtection();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// FindRowsetTreeDefNodeHandler:       Find a rowset tree definition node handler using the attached rowset
        /// </summary>
        /// 
        /// <param name="rowsetDef">            Rowset definition</param>
        /// <returns> 
        /// RowsetTree definition handler or null if not found.
        /// </returns>
        //  
        //*********************************************************     
        public virtual RowsetTreeDefNodeHandler FindRowsetTreeDefNodeHandler(RowsetDef rowsetDef) {
            return((m_del == null) ? null : m_del.baseFindCardGroupHandler(rowsetDef));
        }

        //*********************************************************     
        //
        /// <summary>
        /// FindRowsetTreeDefNodeHandler:   Find a rowset tree definition node handler using the attached rowset node
        /// </summary>
        /// 
        /// <param name="node">             Rowset tree definition node</param>
        /// <returns> 
        /// RowsetTreeDef group handler or null if not found.
        /// </returns>
        //  
        //*********************************************************     
        public virtual RowsetTreeDefNodeHandler FindRowsetTreeDefNodeHandler(RowsetTreeDefNode node) {
            return ((m_del == null) ? null : m_del.baseFindCardGroupHandler(node));
        }

        //*********************************************************     
        //
        /// <summary>
        /// CurrentLine:            Gets the current line position
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual int CurrentLine {
            get {
                return((m_del == null) ? -1 : m_del.baseCurrentLine);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// LineCount:              Gets the current line count
        /// </summary>
        //
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual int LineCount {
            get {
                return((m_del == null) ? 0 : m_del.baseLineCount);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SBCurrentLine:          Gets the current line in the selection buffer
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual int SBCurrentLine {
            get {
                return((m_del == null) ? -1 : m_del.baseSBCurrentLine);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SBLineCount:            Gets the current line count in the selection buffer
        /// </summary>
        //
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual int SBLineCount {
            get {
                return ((m_del == null) ? 0 : m_del.baseSBLineCount);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// TriggerResetFieldError: Reset the last error message from field validation
        /// </summary>
        //  
        //*********************************************************     
        public virtual void TriggerResetFieldError() {
            if (m_del != null) {
                m_del.baseTriggerResetFieldError();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetCommandHolderState:  Set the state of the command holder
        /// </summary>
        //  
        //*********************************************************     
        public virtual void SetCommandHolderState() {            
            if (m_del != null) {
                m_del.baseSetCommandHolderState();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// TriggerErrors:      Trigger the ErrorOccurred event 
        /// </summary>
        //  
        //*********************************************************     
        public virtual void TriggerErrors() {
            if (m_del != null) {
                m_del.baseTriggerErrors();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RaiseAdjustState:       Raise the adjust state for this control and father.
        /// </summary>
        //  
        //*********************************************************     
        public virtual void RaiseAdjustState() {
            if (m_del != null) {
                m_del.baseRaiseAdjustState();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// CommitEdit:             Commit the edit before validating
        /// </summary>
        //  
        //*********************************************************     
        public virtual void CommitEdit() {
            if (m_del != null) {
                m_del.baseCommitEdit();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// CommitAllEdit:              Commit the edit of all rowset tree definition node
        /// </summary>
        /// <param name="nodeHandler">  RowsetTreeDef node handler</param>
        //  
        //*********************************************************     
        public virtual void CommitAllEdit(RowsetTreeDefNodeHandler nodeHandler) {
            if (m_del != null) {
                m_del.baseCommitAllEdit(nodeHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// ValidateCurLine:        Validates the current line
        /// </summary>
        /// <returns>
        ///     false:              Line not valid
        ///     true:               Line valid
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool ValidateCurLine() {
            return ((m_del == null) ? true : m_del.baseValidateCurLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetCurrentLine:             Sets the current logical line
        /// </summary>
        /// <param name="iLine">        Logical or physical line</param>
        /// <returns>
        /// true                        New line selected
        /// false                       Line not selected
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool SetCurrentLine(int iLine) {
            return ((m_del == null) ? true : m_del.baseSetCurrentLine(iLine));
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectFirstLine:        Select the first line
        /// </summary>
        /// <returns>
        ///     true:               The first line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool SelectFirstLine() {
            return ((m_del == null) ? true : m_del.baseSelectFirstLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectPrevLine:         Select the previous line
        /// </summary>
        /// <returns>
        ///     true:               The previous line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool SelectPrevLine() {
            return ((m_del == null) ? true : m_del.baseSelectPrevLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectNextLine:         Select the next line
        /// </summary>
        /// <returns>
        ///     true:               The next line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool SelectNextLine() {
            return ((m_del == null) ? true : m_del.baseSelectNextLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectLastLine:         Select the last line
        /// </summary>
        /// <returns>
        ///     true:               The last line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool SelectLastLine() {
            return ((m_del == null) ? true : m_del.baseSelectLastLine());
        }

		//*********************************************************     
		//
		/// <summary>
		/// InsertNewLine:          Insert a new line
		/// </summary>
		/// <param name="bAtEnd">   true to insert at the end</param>
		/// <returns>
		/// true:                   A new line has been inserted
		/// false:                  No line inserted
		/// </returns>
		//  
		//*********************************************************     
		public virtual bool InsertNewLine(bool bAtEnd) {
            return ((m_del == null) ? true : m_del.baseInsertNewLine(bAtEnd));
		}

        //*********************************************************     
        //
        /// <summary>
        /// DuplicateLine:              Duplicate the current line
        /// </summary>
        /// <returns>
        /// true:                       The line has been duplicated
        /// false:                      No line inserted
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool DuplicateLine() {
            return ((m_del == null) ? true : m_del.baseDuplicateLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// DeleteCurrentLine:      Deletes the current line
        /// </summary>
        /// <returns>
        ///     true:               The line has been deleted
        ///     false:              No line deleted
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool DeleteCurrentLine() {
            return ((m_del == null) ? true : m_del.baseDeleteCurrentLine());            
        }

        //*********************************************************     
        //
        /// <summary>
        /// HandleCommand:          Handle a command coming from a menu or a toolbar
        /// </summary>
        /// <param name="eCmd">     Command Id</param>
        /// <param name="bEvent">   true to trigger event</param>
        //  
        //*********************************************************     
        public virtual void HandleCommand(sdHandlerCmd eCmd, bool bEvent) {
            if (m_del != null) {
                m_del.baseHandleCommand(eCmd, bEvent);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// GetActionState:             Get the specified action state
        /// </summary>
        /// <param name="eAction">      Action</param>
        /// <returns>
        /// false:                      Disabled
        /// true:                       Enabled
        /// </returns>returns>
        //  
        //*********************************************************     
        public virtual bool GetActionState(sdAction eAction) {
            return ((m_del == null) ? true : m_del.baseGetActionState(eAction));
        }

        //*********************************************************     
        //
        /// <summary>
        /// IsDirty:                Checks if the rowset tree definition node handler is dirty
        /// </summary>
        /// <returns>
        ///  true:                  Yes
        ///  false:                 No
        /// </returns>
        //
        //*********************************************************     
        public virtual bool IsDirty() {
            return ((m_del == null) ? true : m_del.baseIsDirty());
        }

        //*********************************************************     
        //
        /// <summary>
        /// GetItemState:           Get the state of the currently selected item
        /// </summary>
        /// <returns>
        /// LineState
        /// </returns>
        //  
        //*********************************************************     
        public virtual LineState GetItemState() {
            return ((m_del == null) ? LineState.sdLineOld : m_del.baseGetItemState());
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnLineSelected:     Reset the last error message from field validation
        /// </summary>
        //  
        //*********************************************************     
		public virtual void OnLineSelected(RowsetTreeDefNodeHandler.LineSelectedEventArgs e) {
		    if (LineSelected != null) {
		        LineSelected(this, e);
		    }
		    if (m_del != null) {
                m_del.baseOnLineSelected(e);
            }
		}

        //*********************************************************     
        //
        /// <summary>
        /// OnFatherLineSelected:   Trigger the FatherLineSelected event for this control
        /// </summary>
        /// <param name="e">        Event arguments</param>
        //  
        //*********************************************************     
        public virtual void OnFatherLineSelected(System.EventArgs e) {
            if (FatherLineSelected != null) {
                FatherLineSelected(this, e);
            }
            if (m_del != null) {
                m_del.baseOnFatherLineSelected(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnBeforeLineValidated:  Trigger the BeforeLineValidated event for this control
        ///                         and all child
        /// </summary>
        /// <param name="e">        Line validated event arguments</param>
        //  
        //*********************************************************     
        public virtual void OnBeforeLineValidated(RowsetTreeDefNodeHandler.LineValidationEventArgs e) {
            if (BeforeLineValidated != null) {
                BeforeLineValidated(this, e);
            }
            if (m_del != null) {
                m_del.baseOnBeforeLineValidated(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnAfterLineValidated:   Trigger the AfterLineValidated event for this control
        ///                         and all child.
        /// </summary>
        /// <param name="e">        Line validated event argument</param>
        //  
        //*********************************************************     
        public virtual void OnAfterLineValidated(RowsetTreeDefNodeHandler.LineValidationEventArgs e) {
            if (AfterLineValidated != null) {
                AfterLineValidated(this, e);
            }
            if (m_del != null) {
                m_del.baseOnAfterLineValidated(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnAdjustState:      Raise the AdjustState event
        /// </summary>
        //  
        //*********************************************************     
        public virtual void OnAdjustState(EventArgs e) {
            if (m_del != null) {
                m_del.baseOnAdjustState(e);
            }
            if (AdjustState != null) {
                AdjustState(this, e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnBeforeExecuteCmd:     Informs that a command is about to be executed
        /// </summary>
        /// <param name="e">        Line validated event</param>
        //  
        //*********************************************************     
        public virtual void OnBeforeExecuteCmd(RowsetTreeDefNodeHandler.ExecuteCmdEventArgs e) {
            if (BeforeExecuteCmd != null) {
                BeforeExecuteCmd(this, e);
            }
            if (m_del != null) {
                m_del.baseOnBeforeExecuteCmd(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnAfterExecuteCmd:      Informs that a command has been executed
        /// </summary>
        /// <param name="e">        Line validated event</param>
        //  
        //*********************************************************     
        public virtual void OnAfterExecuteCmd(RowsetTreeDefNodeHandler.ExecuteCmdEventArgs e) {
            if (AfterExecuteCmd != null) {
                AfterExecuteCmd(this, e);
            }
            if (m_del != null) {
                m_del.baseOnAfterExecuteCmd(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnErrorOccurred:    Trigger the ErrorOccurred event for this control
        ///                     and all child
        /// </summary>
        /// <param name="e">    Error Occurred event arguments</param>
        //  
        //*********************************************************     
        public virtual void OnErrorOccurred(RowsetTreeDefNodeHandler.ErrorOccurredEventArgs e) {
            if (ErrorOccurred != null) {
                ErrorOccurred(this, e);
            }
            if (m_del != null) {
                m_del.baseOnErrorOccurred(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnDataConversion:   Trigger the DataConversion event for this control
        /// </summary>
        /// <param name="e">    Error Occurred event arguments</param>
        //  
        //*********************************************************     
        public virtual void OnDataConversion(RowsetTreeDefNodeHandler.DataConversionEventArgs e) {
            if (DataConversion != null) {
                DataConversion(this, e);
            }
            if (m_del != null) {
                m_del.baseOnDataConversion(e);
            }
        }
#line default
        #region Windows Form Designer generated code
        /// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            // 
            // frmRowsetTreeDefNodeHandler
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(856, 542);
            this.Name = "frmCardGroupHandler";
            this.Text = "frmCardGroupHandler";

        }
		#endregion
	}
}
