//
// Rep++ Framework for Windows Application
// Copyright (C) 2005 Consyst SQL Inc.
// All Rights Reserved
//
// This source code is only intended as a supplement to the
// Rep++ Studio products.
//
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using RepPP;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
	/// <summary>
	/// Form using a delegate class to also inherit from RowsetTreeDefNodeHandlerRoot class
	/// </summary>
	public class frmRowsetTreeDefNodeHandlerRoot : System.Windows.Forms.Form, IRowsetTreeDefNodeHandlerRoot {
	    /// <summary>Event sent each time a new line is selected for this rowset tree def node</summary>
	    public event RowsetTreeDefNodeHandler.LineSelectedEventHandler      LineSelected;
        /// <summary>Father line changed. Select rowset</summary>
        public event System.EventHandler                                    FatherLineSelected;
	    /// <summary>Event sent before a line is validated</summary>
	    public event RowsetTreeDefNodeHandler.LineValidationEventHandler    BeforeLineValidated;
	    /// <summary>Event sent before a line is validated</summary>
	    public event RowsetTreeDefNodeHandler.LineValidationEventHandler    AfterLineValidated;
        /// <summary>State must be adjust</summary>
        public event System.EventHandler                                    AdjustState;
        /// <summary>Called before a command is executed</summary>
        public event RowsetTreeDefNodeHandler.ExecuteCmdEventHandler        BeforeExecuteCmd;
        /// <summary>Called after a command is executed</summary>
        public event RowsetTreeDefNodeHandler.ExecuteCmdEventHandler        AfterExecuteCmd;
        /// <summary>An error Occurred</summary>
        public event RowsetTreeDefNodeHandler.ErrorOccurredEventHandler     ErrorOccurred;
        /// <summary>Called when data conversion must be done</summary>
        public event RowsetTreeDefNodeHandler.DataConversionEventHandler    DataConversion;
        /// <summary>Reset field error</summary>
        public event System.EventHandler                                    ResetFieldError;
        /// <summary>Adjust the field state</summary>
        public event System.EventHandler                                    AdjustFieldState;
		/// <summary>Required designer variable.</summary>
		private System.ComponentModel.Container                             components = null;
		/// <summary>Delegator to the RowsetTreeDefNodeHandlerRoot (simulate multiple inheritance)</summary>
		private RowsetTreeDefNodeHandlerRootDel                                     m_del = null;
	    /// <summary>Rep++ Application object</summary>
	    protected RepPP.IApplication                                        m_app;

        //*********************************************************     
        //
        /// <summary>
        /// ctor:                               Class constructor. Use in design mode
        /// </summary>
        //  
        //*********************************************************     
		public frmRowsetTreeDefNodeHandlerRoot () {
			InitializeComponent();
		}

        //*********************************************************     
        //
        /// <summary>
        /// Dispose:                            Dispose the form
        /// </summary>
        /// <param name="bDisposing">           true if dispose, false if GC</param>
        //  
        //*********************************************************     
		protected override void Dispose(bool bDisposing) {
			base.Dispose(bDisposing);
			if (bDisposing) {
				if (m_del != null) {
				    m_del.Dispose();
				}
				if (components != null) {
					components.Dispose();
				}
			}
		}

        //*********************************************************     
        //
        /// <summary>
        /// InitDel:                            Initialize the delegator class
        /// </summary>
        /// <param name="app">                  Rep++ Application object</param>
        /// <param name="rowsetTreeDef">        RowsetTreeDef attached to the child editor</param>
        //  
        //*********************************************************     
		internal void InitDel(RepPP.Application     app,
                              RowsetTreeDef         rowsetTreeDef) {
            m_app   = app;
            m_del   = new RowsetTreeDefNodeHandlerRootDel(this,                // Delegate to us
                                                  rowsetTreeDef,
                                                  true,                // See deleted line
                                                  true);               // Undo
		}
#line hidden

        //*********************************************************     
        //
        /// <summary>
        /// Handler:               Return a pointer on the card group handler
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RowsetTreeDefNodeHandler Handler {
            get {
                return ((m_del == null) ? null : m_del.baseHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RowsetTreeDefNode:                      RowsetTreeDef group handle by this class
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RowsetTreeDefNode RowsetTreeDefNode { 
            get {
                return ((m_del == null) ? null : m_del.baseCardGroup);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// FatherRowsetTreeDefNodeHandler:         Father card group handler
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RowsetTreeDefNodeHandler FatherRowsetTreeDefNodeHandler { 
            get {
                return ((m_del == null) ? null : m_del.baseFatherCardGroupHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RootRowsetTreeDefNodeHandler:           Root card group handler
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RowsetTreeDefNodeHandlerRootBase RootRowsetTreeDefNodeHandler { 
            get {
                return ((m_del == null) ? null : m_del.baseRootCardGroupHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Rowset:          Attached rowset
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual Rowset Rowset {
            get {
                return ((m_del == null) ? null : m_del.baseGroupInstance);
            }
            set {
                if (m_del != null) {
                    m_del.baseGroupInstance = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// LineListHandler:                Handler for the lines
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual ILineListHandler LineListHandler {
		    get {
                return ((m_del == null) ? null : m_del.baseLineListHandler);
		    }
		    set {
                if (m_del != null) {
                    m_del.baseLineListHandler = value;
                }
		    }
		}

        //*********************************************************     
        //
        /// <summary>
        /// CommandHolder:              Command holder
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual CmdHolder CommandHolder {
            get {
                return ((m_del == null) ? null : m_del.baseCommandHolder);
            }
            set {
                if (m_del != null) {
                    m_del.baseCommandHolder = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// IsLineDeleted:      Returns true if the line or a father line is deleted
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual bool IsLineDeleted {
            get {
                return ((m_del == null) ? false : m_del.baseIsLineDeleted);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// AllowInsert:            Insertion allowed if true
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual sdAllow AllowInsert {
		    get {
                return ((m_del == null) ? sdAllow.sdAllowAuto : m_del.baseAllowInsert);
		    }
		    set {
                if (m_del != null) {
                    m_del.baseAllowInsert = value;
                }
		    }
		}

        //*********************************************************     
        //
        /// <summary>
        /// AllowUpdate:            Update allowed if true
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual sdAllow AllowUpdate {
            get {
                return ((m_del == null) ? sdAllow.sdAllowAuto : m_del.baseAllowUpdate);
            }
            set {
                if (m_del != null) {
                    m_del.baseAllowUpdate = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// AllowDelete:        Delete allowed or not for this card group
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual sdAllow AllowDelete {
            get {
                return ((m_del == null) ? sdAllow.sdAllowAuto : m_del.baseAllowDelete);
            }
            set {
                if (m_del != null) {
                    m_del.baseAllowDelete = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// EffectiveAllowInsert:       Return true if a new line can be inserted
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual bool EffectiveAllowInsert {
            get {
                return ((m_del == null) ? true : m_del.baseEffectiveAllowInsert);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// EffectiveAllowUpdate:       Update allowed if true
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual bool EffectiveAllowUpdate {
            get {
                return ((m_del == null) ? true : m_del.baseEffectiveAllowUpdate);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// EffectiveAllowDelete:       Delete allowed if true
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual bool EffectiveAllowDelete {
            get {
                return ((m_del == null) ? true : m_del.baseEffectiveAllowDelete);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// FieldProtection:    Actual protection level of the fields
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RepPP.Toolkit.Util.FldSelectableState.SelStateE FieldProtection {
            get {
                return ((m_del == null) ? RepPP.Toolkit.Util.FldSelectableState.SelStateE.SS_All: m_del.baseFieldProtection);
            }
            set {
                if (m_del != null) {
                    m_del.baseFieldProtection = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetFieldProtection:     Set the field protection according to the context
        /// </summary>
        //  
        //*********************************************************     
        public virtual void SetFieldProtection() {
            if (m_del != null) {
                m_del.baseSetFieldProtection();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// FindRowsetTreeDefNodeHandler:       Find a card group handler using the attached group
        /// </summary>
        /// 
        /// <param name="rowsetDef">          RowsetDef</param>
        /// <returns> 
        /// RowsetTreeDef group handler or null if not found.
        /// </returns>
        //  
        //*********************************************************     
        public virtual RowsetTreeDefNodeHandler FindRowsetTreeDefNodeHandler(RowsetDef rowsetDef) {
            return ((m_del == null) ? null : m_del.baseFindCardGroupHandler(rowsetDef));
        }

        //*********************************************************     
        //
        /// <summary>
        /// FindRowsetTreeDefNodeHandler:       Find a card group handler using the attached card group
        /// </summary>
        /// 
        /// <param name="node">         RowsetTreeDef group</param>
        /// <returns> 
        /// RowsetTreeDef group handler or null if not found.
        /// </returns>
        //  
        //*********************************************************     
        public virtual RowsetTreeDefNodeHandler FindRowsetTreeDefNodeHandler(RowsetTreeDefNode node) {
            return ((m_del == null) ? null : m_del.baseFindCardGroupHandler(node));
        }

        //*********************************************************     
        //
        /// <summary>
        /// CurrentLine:            Gets the current line position
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual int CurrentLine {
            get {
                return ((m_del == null) ? -1 : m_del.baseCurrentLine);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// LineCount:              Gets the current line count
        /// </summary>
        //
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual int LineCount {
            get {
                return ((m_del == null) ? 0 : m_del.baseLineCount);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SBCurrentLine:          Gets the current line in the selection buffer
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual int SBCurrentLine {
            get {
                return ((m_del == null) ? -1 : m_del.baseSBCurrentLine);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SBLineCount:            Gets the current line count in the selection buffer
        /// </summary>
        //
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual int SBLineCount {
            get {
                return ((m_del == null) ? 0 : m_del.baseSBLineCount);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// TriggerResetFieldError: Reset the last error message from field validation
        /// </summary>
        //  
        //*********************************************************     
        public virtual void TriggerResetFieldError() {
            m_del.baseTriggerResetFieldError();
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetCommandHolderState:  Set the state of the command holder
        /// </summary>
        //  
        //*********************************************************     
        public virtual void SetCommandHolderState() {            
            if (m_del != null) {
                m_del.baseSetCommandHolderState();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// TriggerErrors:      Trigger the ErrorOccurred event 
        /// </summary>
        //  
        //*********************************************************     
        public virtual void TriggerErrors() {
            m_del.baseTriggerErrors();
        }

        //*********************************************************     
        //
        /// <summary>
        /// RaiseAdjustState:       Raise the adjust state for this control and father.
        /// </summary>
        //  
        //*********************************************************     
        public virtual void RaiseAdjustState() {
            if (m_del != null) {
                m_del.baseRaiseAdjustState();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// CommitEdit:             Commit the edit before validating
        /// </summary>
        //  
        //*********************************************************     
        public virtual void CommitEdit() {
            if (m_del != null) {
                m_del.baseCommitEdit();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// CommitAllEdit:              Commit the edit of all card group
        /// </summary>
        /// <param name="nodeHandler">  RowsetTreeDef group handler</param>
        //  
        //*********************************************************     
        public virtual void CommitAllEdit(RowsetTreeDefNodeHandler nodeHandler) {
            if (m_del != null) {
                m_del.baseCommitAllEdit(nodeHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// ValidateCurLine:        Validates the current line
        /// </summary>
        /// <returns>
        ///     false:              Line not valid
        ///     true:               Line valid
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool ValidateCurLine() {
            return ((m_del == null) ? true : m_del.baseValidateCurLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetCurrentLine:             Sets the current logical line
        /// </summary>
        /// <param name="iLine">        Logical or physical line</param>
        /// <returns>
        /// true                        New line selected
        /// false                       Line not selected
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool SetCurrentLine(int iLine) {
            return ((m_del == null) ? true : m_del.baseSetCurrentLine(iLine));
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectFirstLine:        Select the first line
        /// </summary>
        /// <returns>
        ///     true:               The first line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool SelectFirstLine() {
            return ((m_del == null) ? true : m_del.baseSelectFirstLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectPrevLine:         Select the previous line
        /// </summary>
        /// <returns>
        ///     true:               The previous line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool SelectPrevLine() {
            return((m_del == null) ? true : m_del.baseSelectPrevLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectNextLine:         Select the next line
        /// </summary>
        /// <returns>
        ///     true:               The next line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool SelectNextLine() {
            return ((m_del == null) ? true : m_del.baseSelectNextLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectLastLine:         Select the last line
        /// </summary>
        /// <returns>
        ///     true:               The last line has been selected
        ///     false:              No line selected
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool SelectLastLine() {
            return ((m_del == null) ? true : m_del.baseSelectLastLine());
        }

		//*********************************************************     
		//
		/// <summary>
		/// InsertNewLine:          Insert a new line
		/// </summary>
		/// <param name="bAtEnd">   true to insert at the end</param>
		/// <returns>
		/// true:                   A new line has been inserted
		/// false:                  No line inserted
		/// </returns>
		//  
		//*********************************************************     
		public virtual bool InsertNewLine(bool bAtEnd) {
            return ((m_del == null) ? true : m_del.baseInsertNewLine(bAtEnd));
		}

        //*********************************************************     
        //
        /// <summary>
        /// DuplicateLine:              Duplicate the current line
        /// </summary>
        /// <returns>
        /// true:                       The line has been duplicated
        /// false:                      No line inserted
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool DuplicateLine() {
            return ((m_del == null) ? true : m_del.baseDuplicateLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// DeleteCurrentLine:      Deletes the current line
        /// </summary>
        /// <returns>
        ///     true:               The line has been deleted
        ///     false:              No line deleted
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool DeleteCurrentLine() {
            return ((m_del == null) ? true : m_del.baseDeleteCurrentLine());            
        }

        //*********************************************************     
        //
        /// <summary>
        /// HandleCommand:          Handle a command coming from a menu or a toolbar
        /// </summary>
        /// <param name="eCmd">     Command Id</param>
        /// <param name="bEvent">   true to trigger event</param>
        //  
        //*********************************************************     
        public virtual void HandleCommand(sdHandlerCmd eCmd, bool bEvent) {
            if (m_del != null) {
                m_del.baseHandleCommand(eCmd, bEvent);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// GetActionState:             Get the specified action state
        /// </summary>
        /// <param name="eAction">      Action</param>
        /// <returns>
        /// false:                      Disabled
        /// true:                       Enabled
        /// </returns>returns>
        //  
        //*********************************************************     
        public virtual bool GetActionState(sdAction eAction) {
            return ((m_del == null) ? true : m_del.baseGetActionState(eAction));
        }

        //*********************************************************     
        //
        /// <summary>
        /// IsDirty:                Check if the card is dirty
        /// </summary>
        /// <returns>
        ///  true:                   Yes
        ///  false:                  No
        /// </returns>
        //
        //*********************************************************     
        public virtual bool IsDirty() {
            return ((m_del == null) ? true : m_del.baseIsDirty());
        }

        //*********************************************************     
        //
        /// <summary>
        /// GetItemState:           Get the state of the currently selected item
        /// </summary>
        /// <returns>
        /// LineState
        /// </returns>
        //  
        //*********************************************************     
        public virtual LineState GetItemState() {
            return ((m_del == null) ? LineState.sdLineOld : m_del.baseGetItemState());
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnLineSelected:     Reset the last error message from field validation
        /// </summary>
        //  
        //*********************************************************     
		public virtual void OnLineSelected(RowsetTreeDefNodeHandler.LineSelectedEventArgs e) {
		    if (LineSelected != null) {
		        LineSelected(this, e);
		    }
		    if (m_del != null) {
                m_del.baseOnLineSelected(e);
            }
		}

        //*********************************************************     
        //
        /// <summary>
        /// OnFatherLineSelected:   Trigger the FatherLineSelected event for this control
        /// </summary>
        /// <param name="e">        Event arguments</param>
        //  
        //*********************************************************     
        public virtual void OnFatherLineSelected(System.EventArgs e) {
            if (FatherLineSelected != null) {
                FatherLineSelected(this, e);
            }
            if (m_del != null) {
                m_del.baseOnFatherLineSelected(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnBeforeLineValidated:  Trigger the BeforeLineValidated event for this control
        ///                         and all child
        /// </summary>
        /// <param name="e">        Line validated event arguments</param>
        //  
        //*********************************************************     
        public virtual void OnBeforeLineValidated(RowsetTreeDefNodeHandler.LineValidationEventArgs e) {
            if (BeforeLineValidated != null) {
                BeforeLineValidated(this, e);
            }
            if (m_del != null) {
                m_del.baseOnBeforeLineValidated(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnAfterLineValidated:   Trigger the AfterLineValidated event for this control
        ///                         and all child.
        /// </summary>
        /// <param name="e">        Line validated event argument</param>
        //  
        //*********************************************************     
        public virtual void OnAfterLineValidated(RowsetTreeDefNodeHandler.LineValidationEventArgs e) {
            if (AfterLineValidated != null) {
                AfterLineValidated(this, e);
            }
            if (m_del != null) {
                m_del.baseOnAfterLineValidated(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnAdjustState:      Raise the AdjustState event
        /// </summary>
        //  
        //*********************************************************     
        public virtual void OnAdjustState(EventArgs e) {
            if (m_del != null) {
                m_del.baseOnAdjustState(e);
            }
            if (AdjustState != null) {
                AdjustState(this, e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnBeforeExecuteCmd:     Informs that a command is about to be executed
        /// </summary>
        /// <param name="e">        Line validated event</param>
        //  
        //*********************************************************     
        public virtual void OnBeforeExecuteCmd(RowsetTreeDefNodeHandler.ExecuteCmdEventArgs e) {
            if (BeforeExecuteCmd != null) {
                BeforeExecuteCmd(this, e);
            }
            if (m_del != null) {
                m_del.baseOnBeforeExecuteCmd(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnAfterExecuteCmd:      Informs that a command has been executed
        /// </summary>
        /// <param name="e">        Line validated event</param>
        //  
        //*********************************************************     
        public virtual void OnAfterExecuteCmd(RowsetTreeDefNodeHandler.ExecuteCmdEventArgs e) {
            if (AfterExecuteCmd != null) {
                AfterExecuteCmd(this, e);
            }
            if (m_del != null) {
                m_del.baseOnAfterExecuteCmd(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnErrorOccurred:    Trigger the ErrorOccurred event for this control
        ///                     and all child
        /// </summary>
        /// <param name="e">    Error Occurred event arguments</param>
        //  
        //*********************************************************     
        public virtual void OnErrorOccurred(RowsetTreeDefNodeHandler.ErrorOccurredEventArgs e) {
            if (ErrorOccurred != null) {
                ErrorOccurred(this, e);
            }
            if (m_del != null) {
                m_del.baseOnErrorOccurred(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnDataConversion:   Trigger the DataConversion event for this control
        /// </summary>
        /// <param name="e">    Error Occurred event arguments</param>
        //  
        //*********************************************************     
        public virtual void OnDataConversion(RowsetTreeDefNodeHandler.DataConversionEventArgs e) {
            if (DataConversion != null) {
                DataConversion(this, e);
            }
            if (m_del != null) {
                m_del.baseOnDataConversion(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// AutoCommit:             true if auto-commit is on
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual bool AutoCommit {
            get {
                return ((m_del == null) ? true : m_del.baseAutoCommit);
            }
            set {
                if (m_del != null) {
                    m_del.baseAutoCommit = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// DataAccess:              Data access object
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RepPP.Toolkit.ITransactionDataAccess DataAccess {
            get {
                return ((m_del == null) ? null : m_del.baseDataAccess);
            }
            set {
                if (m_del != null) {
                    m_del.baseDataAccess = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RowsetTreeDef:                       Attached card
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RowsetTreeDef RowsetTreeDef {
            get {
                return ((m_del == null) ? null : m_del.baseCard);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// RowsetTree:           Attached rowset tree
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RowsetTree RowsetTree {
            get {
                return ((m_del == null) ? null : m_del.baseCardInstance);
            }
            set {
                if (m_del != null) {
                    m_del.baseCardInstance = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// CurrentFieldDef:           FieldDef currently selected
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual FieldDef CurrentFieldDef {
            get {
                return ((m_del == null) ? null : m_del.baseCurrentField);
            }
            set {
                if (m_del != null) {
                    m_del.baseCurrentField = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// UndoRowsetTree:           RowsetTreeDef instance use to hold undo value
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RowsetTree UndoRowsetTree {
            get {
                return ((m_del == null) ? null : m_del.baseUndoCardInstance);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Refresh:            Refresh the control
        /// </summary>
        //  
        //*********************************************************     
        public override void Refresh() {
            if (m_del != null) {
                m_del.baseRefresh();
            }
            base.Refresh();
        }

        //*********************************************************     
        //
        /// <summary>
        /// SaveLine:                           Save changes for the current line
        /// </summary>
        /// <returns>
        ///  true:                              Changed saved or no changes
        ///  false:                             Not saved. User refused or error
        /// </returns>
        //
        //*********************************************************     
        public virtual bool SaveLine() {
            return ((m_del == null) ? true : m_del.baseSaveLine());
        }

        //*********************************************************     
        //
        /// <summary>
        /// AskToSave:                          Save changes if any after asking user
        /// </summary>
        /// <param name="bEnsureFatherExist">   If an operation on a child of this item is done and this item is new, we must not
        ///                                     accept to not save it.</param>
        /// <returns>
        ///  true:                              Changed saved or no changes
        ///  false:                             Not saved. User refused or error
        /// </returns>
        //
        //*********************************************************     
        public virtual bool AskToSave(bool bEnsureFatherExist) {
            return ((m_del == null) ? true : m_del.baseAskToSave(bEnsureFatherExist));
        }

        //*********************************************************     
        //
        /// <summary>
        /// AskToSave:                          Save changes if any after asking user
        /// </summary>
        /// <returns>
        ///  true:                              Changed saved or no changes
        ///  false:                             Not saved. User refused or error
        /// </returns>
        //
        //*********************************************************     
        public virtual bool AskToSave() {
            return ((m_del == null) ? true : m_del.baseAskToSave());
        }

        //*********************************************************     
        //
        /// <summary>
        /// AfterSave:              Called after a save is done.
        /// </summary>
        /// <returns>
        ///     false:              Cancel
        ///     true:               Continue
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool AfterSave() {
            return((m_del == null) ? true : m_del.baseAfterSave());
        }

        //*********************************************************     
        //
        /// <summary>
        /// AfterCommit:            Called after a database operation is done.
        /// </summary>
        /// <param name="eReason">  Reason of the commit</param>
        //  
        //*********************************************************     
        public virtual void AfterCommit(RepPP.RowsetState eReason) {
            if (m_del != null) {
                m_del.baseAfterCommit(eReason);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SaveAllLines:           Save all dirty lines to DB
        /// </summary>
        /// <returns>
        ///     false:              Not saved
        ///     true:               Saved
        /// </returns>
        //  
        //*********************************************************     
        public virtual bool SaveAllLines() {
            return ((m_del == null) ? true : m_del.baseSaveAllLines());
        }

        //******************************************************************
        //
        /// <summary>
        /// CheckIfDataChangedSinceRead:    Checks if the data has changed in the DB since it has been read
        /// </summary>
        /// <param name="rowset">            RowsetDef instance containing the values</param>
        /// <param name="fldStamp">         FieldDef containing the date stamp</param>
        /// <param name="bDisplayMsg">      Flag indicating if a message box shoulb be displayed</param>
        /// <returns>
        /// true if changed, false if not
        /// </returns>
        //
        //******************************************************************
	    public virtual bool CheckIfDataChangedSinceRead(Rowset rowset, FieldDef fldStamp, bool bDisplayMsg) {
            return ((m_del == null) ? true : m_del.CheckIfDataChangedSinceRead(rowset, fldStamp, bDisplayMsg));
	    }

        //*********************************************************     
        //
        /// <summary>
        /// Undo:                   Undo a logical operation
        /// </summary>
        //  
        //*********************************************************     
        public virtual void Undo() {
            if (m_del != null) {
                m_del.baseUndo();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// DisplayFieldHelp        Display the help text of a field
        /// </summary>
        /// <param name="fldDef">      FieldDef object</param>
        /// <returns>
        ///      True for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public virtual bool DisplayFieldHelp(FieldDef fldDef) {
            return ((m_del == null) ? true : m_del.baseDisplayFieldHelp(fldDef));
        }

        //*********************************************************     
        //
        /// <summary>
        /// DisplayFieldHelp:    Display the help text of the current field
        /// </summary>
        /// <returns>
        ///      true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public virtual bool DisplayFieldHelp() {
            return ((m_del == null) ? true : m_del.baseDisplayFieldHelp());
        }            

        //*********************************************************     
        //
        /// <summary>
        /// DisplayFieldChoices: Display the list of choices of a field
        /// </summary>
        /// <param name="fldDef">   FieldDef object</param>
        /// <returns>
        ///      false:          No choice list proposed
        ///      true:           Choice list proposed
        /// </returns>
        //
        //*********************************************************     
        public virtual bool DisplayFieldChoices(FieldDef fldDef) {
            return ((m_del == null) ? true : m_del.baseDisplayFieldChoices(fldDef));
        }

        //*********************************************************     
        //
        /// <summary>
        ///  DisplayFieldChoices:Display the list of choices of the current field
        /// </summary>
        /// <returns>
        ///      false:          No choice list proposed
        ///      true:           Choice list proposed
        /// </returns>
        //
        //*********************************************************     
        public virtual bool DisplayFieldChoices() {
            return ((m_del == null) ? true : m_del.baseDisplayFieldChoices());
        }

        //*********************************************************     
        //
        /// <summary>
        /// EraseFieldValue:     Erase value of 'fldDef' if permitted.
        /// </summary>
        ///  <param name="fldDef">  FieldDef to erase</param>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public virtual bool EraseFieldValue(FieldDef fldDef) {
            return ((m_del == null) ? true : m_del.baseEraseFieldValue(fldDef));
        }

        //*********************************************************     
        //
        /// <summary>
        /// EraseFieldValue:     Erase value of the current field
        /// </summary>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public virtual bool EraseFieldValue() {
            return ((m_del == null) ? true : m_del.baseEraseFieldValue());
        }

        //*********************************************************     
        //
        /// <summary>
        /// DefaultFieldValue:   Set the value of 'fldDef' to its default value if permitted.
        /// </summary>
        ///  <param name="fldDef">  FieldDef object</param>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public virtual bool DefaultFieldValue(FieldDef fldDef) {
            return ((m_del == null) ? true : m_del.baseDefaultFieldValue(fldDef));
        }

        //*********************************************************     
        //
        /// <summary>
        /// DefaultFieldValue:   Set the value of the current field to its default value if permitted.
        /// </summary>
        /// <returns>
        ///  true for success, else otherwize
        /// </returns>
        //
        //*********************************************************     
        public virtual bool DefaultFieldValue() {
            return ((m_del == null) ? true : m_del.baseDefaultFieldValue());
        }

        //*********************************************************     
        //
        /// <summary>
        ///  EraseAllFieldsValue:    Erase the value of the all fields of the current group.
        /// </summary>
        //
        //*********************************************************     
        public virtual void EraseAllFieldsValue() {
            if (m_del != null) {
                m_del.baseEraseAllFieldsValue();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        ///  DefaultAllFieldsValue:    Set the value of the all fields of the current group to their default value.
        /// </summary>
        //
        //*********************************************************     
        public virtual void DefaultAllFieldsValue() {
            if (m_del != null) {
                m_del.baseDefaultAllFieldsValue();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// ActiveNodeHandler:     Gets the active card group handler
        /// </summary>
        //
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual RowsetTreeDefNodeHandler ActiveNodeHandler {
            get {
                return ((m_del == null) ? null : m_del.baseActiveCardGroupHandler);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// ExecCmdOnActiveNodeHandler:   Execute the specified command on the active card group if any
        /// </summary>
        /// <param name="eCmd">         Command to execute on the active card group</param>
        //
        //*********************************************************     
        public virtual void ExecCmdOnActiveNodeHandler(sdHandlerCmd eCmd) {
            if (m_del != null) {
                m_del.baseExecCmdOnActiveCardGroup(eCmd);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        ///  GetApplication:    Get the rep++ application object
        /// </summary>
        //
        //*********************************************************     
        public virtual IApplication GetApplication() {
            return ((m_del == null) ? null : m_del.baseGetApplication());
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetOpen:            Determine if the set of rows is opened or closed.
        /// </summary>
        //  
        //*********************************************************     
        public virtual bool SetOpen {
            get {
                return ((m_del == null) ? false : m_del.baseSetOpen);
            }
            set {
                if (m_del != null) {
                    m_del.baseSetOpen = value;
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// CloseSet:           Close a card set
        /// </summary>
        /// <returns>
        ///  true:              Set closed
        ///  false:             Set not closed
        /// </returns>
        //
        //*********************************************************     
        public virtual bool CloseSet() {
            return ((m_del == null) ? false : m_del.baseCloseSet());
        }

        //*********************************************************     
        //
        /// <summary>
        /// OpenSet:            Open a card set using the current rowset
        /// </summary>
        /// <returns>
        ///  true:              Set openned
        ///  false:             Set not openned
        /// </returns>
        //
        //*********************************************************     
        public virtual bool OpenSet() {
            return ((m_del == null) ? false : m_del.baseOpenSet());
        }

        //*********************************************************     
        //
        /// <summary>
        /// OpenSet:            Open a card set using the specified WHERE clause
        /// </summary>
        ///
        /// <param name="strWhereClause">  Where clause</param>
        /// <returns>
        ///  true:              Set openned
        ///  false:             Set not openned
        /// </returns>
        //
        //*********************************************************     
        public virtual bool OpenSet(string strWhereClause) {
            return ((m_del == null) ? false : m_del.baseOpenSet(strWhereClause));
        }

        //*********************************************************     
        //
        /// <summary>
        ///  OnResetFieldError: Trigger the ResetFieldError event
        /// </summary>
        //  
        //*********************************************************     
        public virtual void OnResetFieldError(System.EventArgs e) {
            if (ResetFieldError != null) {
                ResetFieldError(this, e);
            }
            if (m_del != null) {
                m_del.baseOnResetFieldError(e);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        ///  OnAdjustFieldState: Trigger the AdjustFieldState event
        /// </summary>
        //  
        //*********************************************************     
        public virtual void OnAdjustFieldState(System.EventArgs e) {
            if (AdjustFieldState != null) {
                AdjustFieldState(this, e);
            }
            if (m_del != null) {
                m_del.baseOnAdjustFieldState(e);
            }
        }
#line default
        #region Windows Form Designer generated code
        /// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            // 
            // frmSBChildEditor
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(928, 616);
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
            this.Name = "frmCardGroupHandlerRootForSB";
            this.Text = "frmCardGroupHandlerRootForSB";

        }
		#endregion
    }
}
