//
// Rep++ Framework for Windows Application
// Copyright (C) 2005 Consyst SQL Inc.
// All Rights Reserved
//
// This source code is only intended as a supplement to the
// Rep++ Studio products.
//
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using RepPP;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
	/// <summary>
    /// Child form which is included in top editor that does not use a selection buffer
	/// </summary>
	public class frmChildEditor : frmRowsetTreeDefNodeHandlerRoot {
		/// <summary>Required designer variable.</summary>
		private System.ComponentModel.Container                     components = null;
        /// <summary>RowsetTreeDef instance created for the transaction</summary>
        private RowsetTree                                        m_rowsetTree;
        /// <summary>Father editor</summary>
        private frmEditorStandalone                                 m_fatherEditor;
        /// <summary>The last where clause used to refresh the editor</summary>
        private string                                              m_strLastWhereClause = string.Empty;

        //*********************************************************     
        //
        /// <summary>
        /// ctor:                               Class constructor. Use in design mode
        /// </summary>
        //  
        //*********************************************************     
		public frmChildEditor() {
			InitializeComponent();
		}

        //*********************************************************     
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="fatherEditor">         Father editor</param>
        /// <param name="strRowsetTreeName">    Name of the rowset tree attached to the child editor</param>
        //  
        //*********************************************************     
        public frmChildEditor(frmEditorStandalone fatherEditor,
                              string              strRowsetTreeName) : this() {
            RepPP.Application   app;
            RowsetTreeDef       rowsetTreeDef;
            
            app             = fatherEditor.Application;
            m_fatherEditor  = fatherEditor;
            rowsetTreeDef   = app.RowsetTreeDefs[strRowsetTreeName];
            InitDel(app, rowsetTreeDef);
            SetTopLevel(false);
            m_rowsetTree    = rowsetTreeDef.RowsetTrees.Add();
            CommandHolder   = null;
            AutoCommit      = true;
            RowsetTree    = m_rowsetTree;                                                               
            fatherEditor.ConfigurationHelper.RegisterForm(this);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Dispose:                            Dispose the form
        /// </summary>
        /// <param name="bDisposing">           true if dispose, false if GC</param>
        //  
        //*********************************************************     
		protected override void Dispose(bool bDisposing) {
			if (bDisposing) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose(bDisposing);
			if (bDisposing) {
				if (m_rowsetTree != null) {
				    m_rowsetTree.Delete();
				    m_rowsetTree = null;
				}
			}
		}

        //*********************************************************     
        //
        /// <summary>
        /// OnLoad:                             Called when the form is loaded
        /// </summary>
        /// <param name="e">                    Event argument</param>
        //  
        //*********************************************************     
        protected override void OnLoad(EventArgs e) {
            base.OnLoad (e);
            if (FatherEditor != null) {
                FatherEditor.ControlTextTranslator.TranslateControl(this);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OpenSet:                        Open a card set using the specified WHERE clause
        /// </summary>
        ///
        /// <param name="strWhereClause">   Where clause</param>
        /// <returns>
        ///  true:                          Set openned
        ///  false:                         Set not openned
        /// </returns>
        //
        //*********************************************************     
        public override bool OpenSet(string strWhereClause) {
            bool bRetVal;

            bRetVal = base.OpenSet(strWhereClause);
            if (bRetVal) {
                m_strLastWhereClause = strWhereClause;
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// FormRefresh:                Refreshes the content of the editor
        /// </summary>
        /// <param name="bForceClose">  True to ignore any changes</param>
        /// <returns>
        ///  true:    Set openned
        ///  false:   Set not openned
        /// </returns>
        //
        //*********************************************************     
        public virtual bool FormRefresh(bool bForceClose) {
            bool bRetVal;

            if (bForceClose) {
                bRetVal = true;
            } else {
                bRetVal = CloseSet();
            }
            if (bRetVal) {
                bRetVal = base.OpenSet(m_strLastWhereClause);
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// FatherEditor:                   Father editor
        /// </summary>
        //
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public frmEditorStandalone FatherEditor {
            get {
                return(m_fatherEditor);
            }
        }

        //*********************************************************     
        //
		/// <summary>
		/// AttachChildEditor:              Attach this child editor to the father editor
		/// </summary>
		/// <param name="ctlFather">        Container control</param>
		//
        //*********************************************************     
		public virtual void AttachChildEditor(Control ctlFather) {
		    ctlFather.Controls.Add(this);
		    Dock            = DockStyle.Fill;
		    CommandHolder   = FatherEditor.CommandHolder;
            Show();
		}

        //*********************************************************     
        //
        /// <summary>
        /// DetachChildEditor:              Detach this child editor to the father editor
        /// </summary>
        /// <param name="bForceClose">      True to force close. Don't ask for saving</param>
        //
        //*********************************************************     
        public virtual bool DetachChildEditor(bool bForceClose) {
            bool bRetVal;

            if (bForceClose){
                bRetVal = true;
            } else {
                bRetVal = CloseSet();
            }
            if (bRetVal) {
                Hide();
                CommandHolder   = null;
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// ProcessTabKey:                      Process the TAB key. Override to permit navigation
        ///                                     out of the contained form.
        /// </summary>
        /// <param name="bFoward">              true if foward, false if reverse</param>
        /// <returns>
        /// true if a new control has been selected, false if not.
        /// </returns>
        //  
        //*********************************************************     
        protected override bool ProcessTabKey(bool bFoward) {
            bool    bRetVal;

            bRetVal = SelectNextControl(ActiveControl, bFoward, true, true, false);
            if (!bRetVal) {
                bRetVal = TopLevelControl.SelectNextControl(this, bFoward, true, true, true);
            }
            return(bRetVal);
        }
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            // 
            // frmSBChildEditor
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(928, 616);
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
            this.Name = "frmSBChildEditor";
            this.Text = "frmSBChildEditor";

        }
		#endregion
	}
}
