using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
    //******************************************************************        
    //
    /// <summary>
	/// Class implementing a form that allows for the creation of selection 
    /// macros from any application that uses this framework
	/// </summary>
    //
    //******************************************************************        
    public class frmEditMacro : Form {

        //******************************************************************
        //
        /// <summary>
	    /// Class implementing a macro parameter
	    /// </summary>
        //
        //******************************************************************        
        private class Parameter {
            /// <summary>Parameter name</summary>
            public string       Name;
            /// <summary>Parameter FieldDef Name</summary>
            public string       FieldName;
            /// <summary>Parameter prompt</summary>
            public string       Prompt;
            /// <summary>Parameter Default Value</summary>
            public string       DefaultValue;
            /// <summary>Parameter FieldDef Options</summary>
            public FieldOptions FieldOptions;

            //******************************************************************
            //
            /// <summary>
	        /// Class constructor
	        /// </summary>
            /// <param name="strName">  Parameter name</param>
            //
            //******************************************************************        
            public Parameter(string strName) {
                Name            = strName;
                FieldName       = string.Empty;
                Prompt          = string.Empty;
                DefaultValue    = string.Empty;
                FieldOptions    = (FieldOptions)0;
            }
        } // Class Parameter

        /// <summary> Number of pages in the Create macro wizard</summary>
        const   int                             m_ciNumberOfSteps           = 5;
        /// <summary> The current page index</summary>
        private int                             m_iCurrentStep              = 0;
        /// <summary> The existing search macro to be edited</summary>
        private SearchMacro                     m_searchMacro               = null;
        /// <summary> Rep++ application object</summary>
        private RepPP.Application               m_app                       = null;
        /// <summary> Selection Buffer RowsetTreeDef Name</summary>
        private string                          m_strRowsetTreeName         = null;
        /// <summary> Dictionary holding the list of parameters</summary>
        private System.Collections.Hashtable    m_hParams                   = new System.Collections.Hashtable(16);
        /// <summary> Control text translator </summary>
        private ControlTextTranslator           m_translator                = null;

        //******************************************************************
        //
        /// <summary>
	    /// Class constructor
	    /// </summary>
        /// <param name="app">              Rep++ Application object</param>
        /// <param name="strRowsetTreeName">SB Rowset tree definition Name</param>
        /// <param name="translator">       Control Text Translator</param>
        /// <param name="searchMacro">      Search Macro</param>
        /// <param name="iStartStep">       Index of the start step</param>
        //
        //******************************************************************        
        private frmEditMacro(RepPP.Application app, string strRowsetTreeName, ControlTextTranslator translator, SearchMacro searchMacro, int iStartStep) {
            string strSectWhere;
            string strSystName;
            string strSectFullPath;

            InitializeComponent();
            m_app               = app;
            m_strRowsetTreeName = strRowsetTreeName;
            m_iCurrentStep      = iStartStep;
            m_searchMacro       = searchMacro;
            m_translator        = new ControlTextTranslator(m_app, new string[]{(searchMacro == null) ? "txtFrmCreateMacro" : "txtFrmEditMacro"}, translator);
            Translate();
            if (!DesignMode) {
                strSystName     = app.SystemName;
                strSectFullPath = app.Modules[strSystName, app.ProgramName].Section;
                strSectWhere    = GetSectionWhereClause(strSystName, strSectFullPath);
                qeWhereClause.Initialize(app, strSectWhere, "WHERE ");
                qeWhereClause.ShowTemplates = false;
                qeWhereClause.QueryLabel   += ": SELECT * FROM " + app.RowsetTreeDefs[strRowsetTreeName].Nodes[0].RowsetDef.TableName;
                cbxFields.Items.AddRange(GetFldNames(strSystName, strSectFullPath));
                SetState();
                txtUniqueName.KeyPress += new KeyPressEventHandler(OnUniqueNameKeyPress);
            }
        }

        //******************************************************************
        //
        /// <summary>
	    /// Class constructor
	    /// </summary>
        /// <param name="app">              Rep++ Application object</param>
        /// <param name="strRowsetTreeName">SB Rowset tree definition name</param>
        /// <param name="translator">   Control Text Translator</param>
        //
        //******************************************************************        
        public frmEditMacro(RepPP.Application app, string strRowsetTreeName, ControlTextTranslator translator): this(app, strRowsetTreeName, translator, null, 0) {
        }

        //******************************************************************
        //
        /// <summary>
	    /// Class constructor
	    /// </summary>
        /// <param name="searchMacro">  Search Macro</param>
        /// <param name="translator">   Control Text Translator</param>
        //
        //******************************************************************        
        public frmEditMacro(SearchMacro searchMacro, ControlTextTranslator translator): this(searchMacro.Macro.Application, 
                                                                                             searchMacro.Macro.RowsetTreeDef.Name, 
                                                                                             translator,
                                                                                             searchMacro,
                                                                                             1) {
            LoadMacro();
        }

        //******************************************************************
        //
        /// <summary>
	    /// Loads the macro data into the controls
	    /// </summary>
        //
        //******************************************************************        
        private void LoadMacro() {
            RepPP.Macro macro;

            macro               = m_searchMacro.Macro;
            txtUniqueName.Text  = macro.UniqueName;
            txtMenuLabel.Text   = macro.Name;
            qeWhereClause.Query = macro.SqlCommand;
            txtDescription.Text = macro.Description;
        }

        //*********************************************************
        //
        /// <summary>
        /// Translates the form
        /// </summary>
        //  
        //*********************************************************
        private void Translate() {
            m_translator.TranslateControl(this);
            lblIntro.Text         = lblIntro.Text.Replace("\n", "\r\n");
            lblSubTitleStep2.Text = string.Format(lblSubTitleStep2.Text, m_app.RowsetTreeDefs[m_strRowsetTreeName].Nodes[0].RowsetDef.TableName).Replace("\n", "\r\n");
            colParamName.Text     = m_translator.GetItemText("colParamName", "Text");
            gbxParameter.Text     = m_translator.GetItemText("gbxParameter", "Text");
        }

        //******************************************************************
        //
        /// <summary>
        ///  Gets the WHERE clause to load the section's components
        /// </summary>
        /// <param name="strAppSystName">   System name</param>
        /// <param name="strSectFullPath">  Section full path</param>
        /// <returns>
        /// The WHERE Clause
        /// </returns>
        //
        //******************************************************************
        private string GetSectionWhereClause(string strAppSystName, string strSectFullPath) {
            string   strRetVal;
            string[] arrSections;
            string   strSections;
            
            arrSections  = strSectFullPath.Split('.');
            for (int i = 1; i < arrSections.Length; i++) {
                arrSections[i] = arrSections[i-1] + "." + arrSections[i];
            }
            strSections = string.Join(".' OR SECTFULLPATH = '", arrSections, 0, arrSections.Length - 1);
            strRetVal   = string.Format("WHERE APPSYSTNAME='{0}' AND (SECTFULLPATH='{1}.')", strAppSystName, strSections);
            return (strRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        ///  Called when a key is pressed in the unique name textbox to
        ///  filter only valid charachters
        /// </summary>
        /// <param name="sender">   Textbox</param>
        /// <param name="e">        Event argument</param>
        //
        //******************************************************************
        void OnUniqueNameKeyPress(object sender, KeyPressEventArgs e) {
            if (Char.IsLetterOrDigit(e.KeyChar) || 
                Char.IsControl(e.KeyChar)       || 
                Char.IsLetter(e.KeyChar)        || 
                e.KeyChar == '_') {
                e.Handled = false;
            } else {
                e.Handled = true;
            }
        }

        //******************************************************************
        //
        /// <summary>
        ///  Called when the next button is clicked
        /// </summary>
        /// <param name="sender">   Next button</param>
        /// <param name="e">        Event argument</param>
        //
        //******************************************************************
        private void butNext_Click(object sender, EventArgs e) {
            MoveToStep(+1);
        }

        //******************************************************************
        //
        /// <summary>
        /// Called when the Back button is clicked
        /// </summary>
        /// <param name="sender">   Back Button</param>
        /// <param name="e">        Event argument</param>
        //
        //******************************************************************
        private void butBack_Click(object sender, EventArgs e) {
            MoveToStep(-1);
        }

        //******************************************************************
        //
        /// <summary>
        /// Moves to a step using an offset
        /// </summary>
        /// <param name="iOffset">  Positive/Negative offset to navigate Forward/Backward</param>
        //
        //******************************************************************
        private void MoveToStep(int iOffset) {
            bool bContinue;

            switch(m_iCurrentStep) {
            case 0:
                bContinue = OnBeforeLeaveWelcomeStep(ref iOffset);
                break;
            case 1:
                bContinue = OnBeforeLeavePropertiesStep(ref iOffset);
                break;
            case 2:
                bContinue = OnBeforeLeaveSqlStep(ref iOffset);
                break;
            case 3:
                bContinue = OnBeforeLeaveParametersStep(ref iOffset);
                break;
            case 4:
                bContinue = OnBeforeLeaveSummaryStep(ref iOffset);
                break;
            default:
                throw new Exception("Unkown step!");
            }
            if (bContinue) {
                m_iCurrentStep += iOffset;
                SetState();
                switch(m_iCurrentStep) {
                case 0:
                    OnBeforeSelectWelcomeStep();
                    break;
                case 1:
                    OnBeforeSelectPropertiesStep();
                    break;
                case 2:
                    OnBeforeSelectSqlStep();
                    break;
                case 3:
                    OnBeforeSelectParametersStep();
                    break;
                case 4:
                    OnBeforeSelectSummaryStep();
                    break;
                default:
                    throw new Exception("Unkown step!");
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// Called before leaving the Welcome page
        /// </summary>
        /// <param name="iOffset">  Navigation offset</param>
        /// <returns>
        /// False to cancel the navigation, True otherwise
        /// </returns>
        //
        //******************************************************************
        private bool OnBeforeLeaveWelcomeStep(ref int iOffset) {
            return(true);
        }

        //*********************************************************************
        //
        /// <summary>
        /// Checks if a string is alpha numeric
        /// </summary>
        /// <param name="str">  String to be verified</param>
        /// <returns>
        /// True if the string is alphanumeric. False otherwise
        /// </returns>
        //
        //**********************************************************************
        public bool IsAlphaNumeric(string str) {
            Regex regexAlphaNum = new Regex("[^a-zA-Z0-9_]");
            return !regexAlphaNum.IsMatch(str);    
        }

        //******************************************************************
        //
        /// <summary>
        /// Called before leaving the properties step
        /// </summary>
        /// <param name="iOffset">  Navigation offset</param>
        /// <returns>
        /// False to cancel the navigation, True otherwise
        /// </returns>
        //
        //******************************************************************
        private bool OnBeforeLeavePropertiesStep(ref int iOffset) {
            bool bInError = false;

            if (iOffset > 0) {
                if (txtUniqueName.Text == string.Empty) {
                    epErrors.SetError(txtUniqueName, m_translator.GetItemText("errUniqueRequired", "Text"));
                    bInError = true;
                } else if (!IsAlphaNumeric(txtUniqueName.Text)) {
                    epErrors.SetError(txtUniqueName, m_translator.GetItemText("errUniqueInvalid", "Text"));
                    bInError = true;
                } else if (!EnsureUniqueName()) {
                    epErrors.SetError(txtUniqueName, m_translator.GetItemText("errNotUnique", "Text"));
                    bInError = true;
                } else {
                    epErrors.SetError(txtUniqueName,"");
                }
                if (txtMenuLabel.Text == string.Empty) {
                    epErrors.SetError(txtMenuLabel, m_translator.GetItemText("errMissingLabel", "Text"));
                    bInError = true;
                } else {
                    epErrors.SetError(txtMenuLabel,"");
                }
            }
            return(!bInError);
        }

        //******************************************************************
        //
        /// <summary>
        /// Ensures that the macro name is unique
        /// </summary>
        /// <returns>
        /// True if the macro name is unique
        /// </returns>
        //
        //******************************************************************
        private bool EnsureUniqueName() {
            bool        bRetVal = true;
            SqlCommand  sqlCmd;
            SqlCursor   cursor;
            string      strSqlCmd;

            if (m_searchMacro != null && string.Compare(m_searchMacro.Macro.UniqueName, txtUniqueName.Text, true) == 0) {
                bRetVal = true;
            } else {
                strSqlCmd = m_app.GetAtomText("IsMacroNameUnique", AtomType.sdAtomSql);
                using (sqlCmd = m_app.RepositoryConnection.SqlCommands.Open(strSqlCmd)) {
                    sqlCmd.SetParameterValue("APPSYSTNAME",     m_app.SystemName);
                    sqlCmd.SetParameterValue("APPNAME",         m_app.ProgramName);
                    sqlCmd.SetParameterValue("LANGUE",          ((int)m_app.LanguageCode).ToString());
                    sqlCmd.SetParameterValue("MACROUNIQUENAME", txtUniqueName.Text);
                    if(sqlCmd.Execute(out cursor) != 0) {
                        throw new Exception("Cannot verify uniqueness of macro name");
                    } else {
                        using(cursor) {
                            if (cursor.Next() == 0) {
                                bRetVal = false;
                            }
                        }
                    }
                }
            }
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// Called before leaving the sql command step
        /// </summary>
        /// <param name="iOffset">  Navigation offset</param>
        /// <returns>
        /// False to cancel the navigation, True otherwise
        /// </returns>
        //
        //******************************************************************
        private bool OnBeforeLeaveSqlStep(ref int iOffset) {
            bool bRetVal = true;

            butValidCmd.Visible = false;
            if (iOffset > 0) {
                AdjustParameters();
                if (m_hParams.Count <= 0){
                    iOffset++;
                }
            }
            return bRetVal;
        }

        //******************************************************************
        //
        /// <summary>
        /// Refreshes the list of parameters based on the sql command
        /// </summary>
        //
        //******************************************************************
        private void AdjustParameters() {
            string[]        arrParameters;
            ArrayList		lstToDelete = new ArrayList();
            ListViewItem    lviParam;

            arrParameters = GetParameterNames();
            foreach(string strParam in m_hParams.Keys) {
                if (Array.IndexOf(arrParameters, strParam) == -1){
                    lstToDelete.Add(strParam);
                }
            }
            foreach(string strParam in lstToDelete) {
                m_hParams.Remove(strParam);
				for(int i=0; i < lvParams.Items.Count; i++) {
					if (lvParams.Items[i].Text == strParam) {
						lvParams.Items.RemoveAt(i);
						break;
					}
				}
            }
            foreach(string strParam in arrParameters) {
                if (!m_hParams.ContainsKey(strParam)) {
                    m_hParams.Add(strParam, CreateParameter(strParam));
                    lviParam     = lvParams.Items.Add(strParam, 0);
                    lviParam.Tag = m_hParams[strParam];
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// Creates a new parameter. If the parameter is an existing macro argument,
        /// it sets its properties
        /// </summary>
        /// <param name="strParam"> Parameter name</param>
        /// <returns>
        /// The newly created parameter
        /// </returns>
        //
        //******************************************************************
        private Parameter CreateParameter(string strParam) {
            Parameter   paramRetVal = new Parameter(strParam);
            MacroArg    arg;
            FieldDef    fldDef;

            if (m_searchMacro != null) {
                arg = m_searchMacro.Macro.Arguments[strParam];
                if (arg != null){
                    fldDef                   = arg.FieldDef;
                    paramRetVal.FieldName    = fldDef.Name; 
                    paramRetVal.Prompt       = fldDef.Prompt; 
                    paramRetVal.DefaultValue = fldDef.DefaultValue;
                    paramRetVal.FieldOptions = fldDef.Options;
                }
            }
            return(paramRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// Called before leaving the parameters step
        /// </summary>
        /// <param name="iOffset">  Navigation offset</param>
        /// <returns>
        /// False to cancel the navigation, True otherwise
        /// </returns>
        //
        //******************************************************************
        private bool OnBeforeLeaveParametersStep(ref int iOffset) {
            bool bRetVal = true;

            if (iOffset > 0) {
                bRetVal = ValidateParameters();
            }
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// Validates the list of parameters 
        /// </summary>
        /// <returns>
        /// True if all parameters are valid, false otherwise
        /// </returns>
        //
        //******************************************************************
        private bool ValidateParameters() {
            bool      bRetVal   = true;
            Hashtable hUsedFlds = new Hashtable();
            Parameter param;

            foreach(ListViewItem lvi in lvParams.Items) {
                param = lvi.Tag as Parameter;
                if (param.FieldName == string.Empty) {
                    epErrors.SetError(cbxFields, m_translator.GetItemText("errFldName", "Text"));
                    bRetVal = false;
                } else {
                    if (hUsedFlds.ContainsKey(param.FieldName)) {
                        epErrors.SetError(cbxFields, m_translator.GetItemText("errFldUsed", "Text") + " " + hUsedFlds[param.FieldName]);
                        bRetVal = false;
                    } else {
                        hUsedFlds.Add(param.FieldName, param.Name);
                    }
                }
                if (!bRetVal) {
                    lvi.Selected = true;
                    break;
                } else {
                    epErrors.SetError(cbxFields, "");
                }
            }
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// Called before leaving the summary page
        /// </summary>
        /// <param name="iOffset">  Navigation offset</param>
        /// <returns>
        /// False to cancel the navigation, True otherwise
        /// </returns>
        //
        //******************************************************************
        private bool OnBeforeLeaveSummaryStep(ref int iOffset) {
            bool bRetVal = true;

            if (iOffset < 0 && m_hParams.Count <= 0){
                iOffset--;
            }
            return bRetVal;
        }

        //******************************************************************
        //
        /// <summary>
        /// Called before the welcome page is displayed
        /// </summary>
        //
        //******************************************************************
        private void OnBeforeSelectWelcomeStep() {
        }

        //******************************************************************
        //
        /// <summary>
        /// Called before the properties page is displayed
        /// </summary>
        //
        //******************************************************************
        private void OnBeforeSelectPropertiesStep() {
            ActiveControl = txtUniqueName;
        }

        //******************************************************************
        //
        /// <summary>
        /// Called before the sql comand page is displayed
        /// </summary>
        //
        //******************************************************************
        private void OnBeforeSelectSqlStep() {
            ActiveControl = qeWhereClause;
            butValidCmd.Visible = true;
        }

        //******************************************************************
        //
        /// <summary>
        ///  Called before the parameters page is displayed
        /// </summary>
        //
        //******************************************************************
        private void OnBeforeSelectParametersStep() {
            ActiveControl = lvParams;
        }

        //******************************************************************
        //
        /// <summary>
        /// Called Before displaying the summary page
        /// </summary>
        //
        //******************************************************************
        private void OnBeforeSelectSummaryStep() {
            string      strMacroSummary = m_translator.GetItemText("lblMacroSummary", "Text").Replace("\n", "\r\n");
            string      strHaving       = m_translator.GetItemText("lblWithParam", "Text").Replace("\n", "\r\n");
            string      strParamSummary = m_translator.GetItemText("lblParamSummary", "Text").Replace("\n", "\r\n");
            string      strNoEcho       = m_translator.GetItemText("lblNoEcho", "Text");
            string      strRequired     = m_translator.GetItemText("lblRequired", "Text");
            string      strSummary;
            string      strName;
            Parameter   param;
            bool        bNoEcho;
            bool        bRequired;

            strSummary = string.Format(strMacroSummary, 
                                       txtUniqueName.Text, 
                                       txtMenuLabel.Text, 
                                       qeWhereClause.Query,
                                       txtDescription.Text);
            if (lvParams.Items.Count > 0) {
                strSummary += strHaving;
                foreach(ListViewItem lvi in lvParams.Items) {
                    param       = lvi.Tag as Parameter;
                    strName     = param.Name;
                    bNoEcho     = ((param.FieldOptions & FieldOptions.sdFieldNoEcho) != 0);
                    bRequired   = ((param.FieldOptions & FieldOptions.sdFieldRequired) != 0);
                    if (bNoEcho || bRequired) {
                        strName += " (";
                        if (bNoEcho) {
                            strName += strNoEcho;
                            if (bRequired) {
                                strName += ", ";
                            }
                        }
                        if (bRequired) {
                            strName += strRequired;
                        }
                        strName += ")";
                    }
                    strSummary += string.Format(strParamSummary, 
                                                strName, 
                                                param.FieldName, 
                                                param.Prompt, 
                                                param.DefaultValue); 
                }
            }
            txtSummary.Text = strSummary;
        }

        //******************************************************************
        //
        /// <summary>
        /// Sets the state of the panels and controls depending on the current step
        /// </summary>
        //
        //******************************************************************
        private void SetState() {
            pnlWelcome.Visible  = (m_iCurrentStep == 0);
            pnlStep1.Visible    = (m_iCurrentStep == 1);
            pnlStep2.Visible    = (m_iCurrentStep == 2);
            pnlStep3.Visible    = (m_iCurrentStep == 3);
            pnlSummary.Visible  = (m_iCurrentStep == 4);
            butBack.Enabled     = (m_searchMacro == null) ? (m_iCurrentStep > 0) : (m_iCurrentStep > 1);
            butNext.Enabled     = (m_iCurrentStep < m_ciNumberOfSteps - 1);
            butNext.Visible     =  butNext.Enabled;
            butFinish.Visible   = !butNext.Enabled;
            AcceptButton        = butFinish.Visible ? butFinish : null;
        }

        //******************************************************************
        //
        /// <summary>
        /// Validates the sql command
        /// </summary>
        /// <param name="strSqlCmd">        Command to be validated</param>
        //
        //******************************************************************
        public void ValidateSqlCommand(String strSqlCmd) {
            String      strMsg;
            SqlCommand  sqlCommand       = null;
            bool        bDelayedPrepare;

            if (strSqlCmd != null && strSqlCmd != string.Empty) {
                if (strSqlCmd.Length >= 0) {
                    bDelayedPrepare = m_app.DataConnection.DelayedPrepare;
                    m_app.DataConnection.DelayedPrepare = false;
                    try {
                        sqlCommand = m_app.DataConnection.SqlCommands.Open(strSqlCmd);
                        sqlCommand.Close();
                        strMsg = m_translator.GetItemText("lblCmdIsCorrect", "Text");
                        MessageBox.Show(strMsg);
                    } catch {
                    } finally {
                        m_app.DataConnection.DelayedPrepare = bDelayedPrepare;
                    }
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// Extracts the list of parameter names from the sql command
        /// </summary>
        /// <returns>
        /// List of parameter names
        /// </returns>
        //
        //******************************************************************
        private string[] GetParameterNames() {
            ArrayList		lstRetVal   = new ArrayList(16);
            string          strSQL;
            int             iStart      = 0;
            int             iEnd        = 0;

            strSQL = qeWhereClause.Query;
            do {
                StringUtility.StrFindKeyword(strSQL,
                                             iEnd,
                                             out iStart,
                                             ":",
                                             StringUtility.SearchTypeE.SK_NOT_IN_DOUBLE_QUOTE |
                                             StringUtility.SearchTypeE.SK_NOT_IN_SIMPLE_QUOTE);
                if (iStart != -1) {
                    iEnd   = strSQL.IndexOfAny(new char[] {'.', ' ', '\t', '\n', '\f', '\r', '=', '>', '<', '/', '+', '-', '*', ')', ','}, iStart);
                    if (iEnd  != iStart + 1) {
                        if(iEnd == -1) {
                            lstRetVal.Add(strSQL.Substring(iStart + 1));
                            iStart = -1;
                        } else if (strSQL[iEnd] != '.'){
                            lstRetVal.Add(strSQL.Substring(iStart + 1, iEnd - iStart - 1));
                        }
                    }
                }
            } while (iStart != -1);
            return ((string[])lstRetVal.ToArray(typeof(string)));
        }

        //******************************************************************
        //
        /// <summary>
        /// Called to validate the SQL Command
        /// </summary>
        /// <param name="sender">   Validate Button</param>
        /// <param name="e">        Event Argument</param>
        //
        //******************************************************************
        private void butValidCmd_Click(object sender, EventArgs e) {
            RowsetTreeDef rowsetTreeDef;

            rowsetTreeDef = m_app.RowsetTreeDefs[m_strRowsetTreeName];
            if (rowsetTreeDef == null) {
                throw new Exception("Cannot find card: " + m_strRowsetTreeName);
            }
            ValidateSqlCommand("SELECT * FROM " + rowsetTreeDef.Nodes[0].RowsetDef.TableName + " " + qeWhereClause.Query);
        }

        //*********************************************************************
        //
        /// <summary>
        /// GetFldNames:                    Gets the names of all accessible fields
        /// </summary>
        /// <param name="strAppSystName">   System Name</param>
        /// <param name="strSectFullPath">  Section Full Path</param>
        //      
        //*********************************************************************
        private string[] GetFldNames(string strAppSystName, string strSectFullPath) {
            ArrayList		lstRetVal = new ArrayList(16);
            RepPP.SqlCursor cursor;
            string          strSqlCmd;

            strSqlCmd = string.Format(m_app.GetAtomText("GetFldNames", AtomType.sdAtomSql), 
                                      GetSectionWhereClause(strAppSystName, strSectFullPath));
            if (m_app.RepositoryConnection.Execute(strSqlCmd, out cursor) != 0) {
                throw new Exception("Cannot verify uniqueness of macro name");
            } else {
                using(cursor) {
                    while (cursor.Next() == 0) {
                        lstRetVal.Add(cursor.GetColumnValue(0));    
                    }
                }
            }
            return((string[])lstRetVal.ToArray(typeof(string)));
        }

        //******************************************************************
        //
        /// <summary>
        /// Called when the selected parameter changes
        /// </summary>
        /// <param name="sender">   ListView</param>
        /// <param name="e">        Event Argument</param>
        //
        //******************************************************************
        private void lvParams_SelectedIndexChanged(object sender, EventArgs e) {
            bool            bIsEnabled = false;
            Parameter       param;
            string          strText    = m_translator.GetItemText("gbxParameter", "Text");
            string          strField   = string.Empty;
            string          strLabel   = string.Empty;
            string          strDefault = string.Empty;
            FieldOptions    fldOpts    = (FieldOptions)0;
            
            if (lvParams.SelectedIndices.Count > 0) {
                bIsEnabled = true;
                strText    = lvParams.SelectedItems[0].Text;
                param      = lvParams.SelectedItems[0].Tag as Parameter;
                strField   = param.FieldName; 
                strLabel   = param.Prompt; 
                strDefault = param.DefaultValue;
                fldOpts    = param.FieldOptions;
            }

            cbxFields.Enabled       = bIsEnabled;            
            txtPrompt.Enabled       = bIsEnabled;
            txtDefaultValue.Enabled = bIsEnabled;
            chkNoEcho.Enabled       = bIsEnabled;
            chkRequired.Enabled     = bIsEnabled;
            butDwn.Enabled          = bIsEnabled && (lvParams.SelectedIndices[0] < lvParams.Items.Count - 1);
            butUp.Enabled           = bIsEnabled && (lvParams.SelectedIndices[0] > 0);
            if (strField == string.Empty) {
                cbxFields.SelectedIndex = -1;
            } else {
                cbxFields.Text = strField;
            }
            txtPrompt.Text          = strLabel;
            txtDefaultValue.Text    = strDefault;
            gbxParameter.Text       = strText;
            chkNoEcho.Checked       = ((fldOpts & FieldOptions.sdFieldNoEcho) != 0);
            chkRequired.Checked     = ((fldOpts & FieldOptions.sdFieldRequired) != 0);
        }

        //******************************************************************
        //
        /// <summary>
        /// Called when the field of a parameter changes 
        /// </summary>
        /// <param name="sender">   Combobox</param>
        /// <param name="e">        Event Argument</param>
        //
        //******************************************************************
        private void cbxFields_SelectedIndexChanged(object sender, EventArgs e) {
            Parameter param;

            if (lvParams.SelectedIndices.Count > 0) {
                param           = lvParams.SelectedItems[0].Tag as Parameter;
                param.FieldName = cbxFields.Text;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// Called when the prompt of a parameter changes
        /// </summary>
        /// <param name="sender">   Textbox</param>
        /// <param name="e">        Event argument</param>
        //
        //******************************************************************
        private void txtPrompt_TextChanged(object sender, EventArgs e) {
            Parameter param;

            if (lvParams.SelectedIndices.Count > 0) {
                param           = lvParams.SelectedItems[0].Tag as Parameter;
                param.Prompt    = txtPrompt.Text;
            }
        }

        //******************************************************************
        //
        /// <summary>
        ///  Called when the default value of a parameter changes
        /// </summary>
        /// <param name="sender">   Textbox</param>
        /// <param name="e">        Event Argument</param>
        //
        //******************************************************************
        private void txtDefaultValue_TextChanged(object sender, EventArgs e) {
            Parameter param;

            if (lvParams.SelectedIndices.Count > 0) {
                param              = lvParams.SelectedItems[0].Tag as Parameter;
                param.DefaultValue = txtDefaultValue.Text;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// Changes the position of a parameter
        /// </summary>
        /// <param name="sender">   Button</param>
        /// <param name="e">        Event Argument</param>
        //
        //******************************************************************
        private void OnChangeParamPos(object sender, EventArgs e) {
            int          iIndex;
            int          iNewIndex;
            ListViewItem lvi;
            string       strText;
            object       oTag;

            iIndex  = lvParams.SelectedIndices[0];
            if (sender == butUp) {
                iNewIndex = iIndex - 1;
            } else {
                iNewIndex = iIndex + 1;
            }
            if (iNewIndex >= 0 && iNewIndex < lvParams.Items.Count) {
                lvi     = lvParams.Items[iIndex];
                strText = lvi.Text;
                oTag    = lvi.Tag;
                lvParams.Items.RemoveAt(iIndex);
                lvi          = lvParams.Items.Insert(iNewIndex, strText, 0);
                lvi.Tag      = oTag;
                lvi.Selected = true;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// Rename                          Helper to rename a parameter within
        ///                                 the sql command
        /// </summary>
        /// <param name="strOrg">           Original parameter name</param>
        /// <param name="strNew">           New paramter name</param>
        //
        //******************************************************************
        private void Rename(string strOrg, string strNew) {
            string  strSql          = qeWhereClause.Query;
            string  strNewSql       = string.Empty;
            int     iSearchStart    = 0;
            int     iKeywordStart;
            int     iKeywordEnd;

            iKeywordEnd  = StringUtility.StrFindKeyword(strSql,
                                                        iSearchStart,
                                                        out iKeywordStart,
                                                        strOrg,
                                                        StringUtility.SearchTypeE.SK_NON_ALNUM_AFTER    |
                                                        StringUtility.SearchTypeE.SK_NON_ALNUM_BEFORE   |
                                                        StringUtility.SearchTypeE.SK_IGNORE_CASE        |
                                                        StringUtility.SearchTypeE.SK_NOT_IN_DOUBLE_QUOTE|
                                                        StringUtility.SearchTypeE.SK_NOT_IN_SIMPLE_QUOTE);
            while (iKeywordEnd != -1) {
                strNewSql   += strSql.Substring(iSearchStart, iKeywordStart - iSearchStart);
                strNewSql   += strNew;
                iSearchStart = iKeywordEnd;
                iKeywordEnd  = StringUtility.StrFindKeyword(strSql,
                                                            iSearchStart,
                                                            out iKeywordStart,
                                                            strOrg,
                                                            StringUtility.SearchTypeE.SK_NON_ALNUM_AFTER    |
                                                            StringUtility.SearchTypeE.SK_NON_ALNUM_BEFORE   |
                                                            StringUtility.SearchTypeE.SK_IGNORE_CASE        |
                                                            StringUtility.SearchTypeE.SK_NOT_IN_DOUBLE_QUOTE|
                                                            StringUtility.SearchTypeE.SK_NOT_IN_SIMPLE_QUOTE);
            }
            if (strNewSql != string.Empty) {
                strNewSql          += strSql.Substring(iSearchStart);
                qeWhereClause.Query = strNewSql;    
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// Called after a listview label has been edited
        /// </summary>
        /// <param name="sender">   ListView</param>
        /// <param name="e">        Event argument</param>
        //
        //******************************************************************
        private void lvParams_AfterLabelEdit(object sender, LabelEditEventArgs e) {
            ListViewItem lvi;
            string       strOldKey = lvParams.Items[e.Item].Text;
            string       strNewKey = e.Label;

            if (strNewKey == null || !IsAlphaNumeric(strNewKey) || m_hParams.ContainsKey(strNewKey)) {
                e.CancelEdit = true;
            } else {
                ((Parameter)m_hParams[strOldKey]).Name = strNewKey; 
                m_hParams.Add(strNewKey, m_hParams[strOldKey]);
                m_hParams.Remove(strOldKey);
                lvParams.Items.RemoveAt(e.Item);
                lvi          = lvParams.Items.Insert(e.Item, strNewKey, 0);
                lvi.Tag      = m_hParams[strNewKey];
                lvi.Selected = true;
                Rename(":" + strOldKey, ":" + strNewKey);
            }
        }

        //******************************************************************
        //
        /// <summary>
        ///  Called when the Finish button is clicked
        /// </summary>
        /// <param name="sender">   Finish button</param>
        /// <param name="e">        Event argument</param>
        //
        //******************************************************************
        private void butFinish_Click(object sender, EventArgs e) {
            RepPP.Macro macro;
            Parameter   param;
            int         i      = 0;

            if (m_searchMacro != null) {
                m_searchMacro.Macro.Delete();
                m_searchMacro = null;
            }
            macro = m_app.Macros.Add(txtUniqueName.Text, 
                                     txtMenuLabel.Text, 
                                     MacroType.sdSelection,
                                     AccessType.sdSystem,
                                     m_app.ProgramName, 
                                     qeWhereClause.Query, 
                                     txtDescription.Text, 
                                     0, 
                                     m_app.RowsetTreeDefs[m_strRowsetTreeName]);
            foreach(ListViewItem lvi in lvParams.Items) {            
                param = lvi.Tag as Parameter;    
                macro.Arguments.Add(param.Name,
                                    param.FieldName,
                                    i++,
                                    param.Prompt,
                                    param.DefaultValue,
                                    param.FieldOptions);
            }
            if (macro.Update() != 0) {
                MessageBox.Show(m_translator.GetItemText("lblCreateFail", "Text"));
            } else {
                this.DialogResult = DialogResult.OK;
                Close();
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// Called when one of the checkboxes is (un)checked
        /// </summary>
        /// <param name="sender">   Checkbox</param>
        /// <param name="e">        Event argument</param>
        //
        //******************************************************************
        private void OnCheckedChanged(object sender, EventArgs e) {
            Parameter param;

            if (lvParams.SelectedIndices.Count > 0) {
                param = lvParams.SelectedItems[0].Tag as Parameter;
                if (((CheckBox)sender).Checked) {
                    param.FieldOptions |= (sender == chkNoEcho) ? FieldOptions.sdFieldNoEcho : FieldOptions.sdFieldRequired;
                } else {
                    param.FieldOptions &= ~((sender == chkNoEcho) ? FieldOptions.sdFieldNoEcho : FieldOptions.sdFieldRequired);
                }
            }
        }

        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.IContainer components = null;

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
        protected override void Dispose(bool disposing) {
            if (disposing && (components != null)) {
                components.Dispose();
            }
            base.Dispose(disposing);
        }

        #region Windows Form Designer generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent() {
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(frmEditMacro));
            this.pnlAll = new System.Windows.Forms.Panel();
            this.butFinish = new System.Windows.Forms.Button();
            this.butValidCmd = new System.Windows.Forms.Button();
            this.butBack = new System.Windows.Forms.Button();
            this.butNext = new System.Windows.Forms.Button();
            this.butCancel = new System.Windows.Forms.Button();
            this.lblBorder = new System.Windows.Forms.Label();
            this.pnlStep3 = new System.Windows.Forms.Panel();
            this.groupBox1 = new System.Windows.Forms.GroupBox();
            this.gbxParameter = new System.Windows.Forms.GroupBox();
            this.chkRequired = new System.Windows.Forms.CheckBox();
            this.chkNoEcho = new System.Windows.Forms.CheckBox();
            this.cbxFields = new System.Windows.Forms.ComboBox();
            this.lblField = new System.Windows.Forms.Label();
            this.txtDefaultValue = new System.Windows.Forms.TextBox();
            this.lblPrompt = new System.Windows.Forms.Label();
            this.txtPrompt = new System.Windows.Forms.TextBox();
            this.lblDefaultValue = new System.Windows.Forms.Label();
            this.lvParams = new System.Windows.Forms.ListView();
            this.colParamName = new System.Windows.Forms.ColumnHeader();
            this.imageList1 = new System.Windows.Forms.ImageList(this.components);
            this.butDwn = new System.Windows.Forms.Button();
            this.butUp = new System.Windows.Forms.Button();
            this.label4 = new System.Windows.Forms.Label();
            this.pnlHeaderStep3 = new System.Windows.Forms.Panel();
            this.lblSubTitleStep3 = new System.Windows.Forms.Label();
            this.lblTitleStep3 = new System.Windows.Forms.Label();
            this.pnlStep1 = new System.Windows.Forms.Panel();
            this.gbxStep1 = new System.Windows.Forms.GroupBox();
            this.txtDescription = new System.Windows.Forms.TextBox();
            this.lblDescription = new System.Windows.Forms.Label();
            this.txtMenuLabel = new System.Windows.Forms.TextBox();
            this.lblMenuLabel = new System.Windows.Forms.Label();
            this.txtUniqueName = new System.Windows.Forms.TextBox();
            this.lblUniqueName = new System.Windows.Forms.Label();
            this.lblBorderStep1 = new System.Windows.Forms.Label();
            this.pnlHeaderStep1 = new System.Windows.Forms.Panel();
            this.lblSubTitleStep1 = new System.Windows.Forms.Label();
            this.lblTitleStep1 = new System.Windows.Forms.Label();
            this.pnlStep2 = new System.Windows.Forms.Panel();
            this.qeWhereClause = new RepPP.Miscellaneous.QueryEditor();
            this.lblBorderStep2 = new System.Windows.Forms.Label();
            this.pnlHeaderStep2 = new System.Windows.Forms.Panel();
            this.lblSubTitleStep2 = new System.Windows.Forms.Label();
            this.lblTitleStep2 = new System.Windows.Forms.Label();
            this.pnlSummary = new System.Windows.Forms.Panel();
            this.txtSummary = new System.Windows.Forms.TextBox();
            this.pbSummary = new System.Windows.Forms.PictureBox();
            this.lblSummary = new System.Windows.Forms.Label();
            this.pnlWelcome = new System.Windows.Forms.Panel();
            this.pictureBox1 = new System.Windows.Forms.PictureBox();
            this.lblIntro = new System.Windows.Forms.Label();
            this.lblWelcome = new System.Windows.Forms.Label();
			this.butCancel.UseVisualStyleBackColor = true;
			this.epErrors = new System.Windows.Forms.ErrorProvider(this.components);
            this.pnlAll.SuspendLayout();
            this.pnlStep3.SuspendLayout();
            this.groupBox1.SuspendLayout();
            this.gbxParameter.SuspendLayout();
            this.pnlHeaderStep3.SuspendLayout();
            this.pnlStep1.SuspendLayout();
            this.gbxStep1.SuspendLayout();
            this.pnlHeaderStep1.SuspendLayout();
            this.pnlStep2.SuspendLayout();
            this.pnlHeaderStep2.SuspendLayout();
            this.pnlSummary.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.pbSummary)).BeginInit();
            this.pnlWelcome.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.pictureBox1)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.epErrors)).BeginInit();
            this.SuspendLayout();
            // 
            // pnlAll
            // 
            this.pnlAll.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
            this.pnlAll.Controls.Add(this.butFinish);
            this.pnlAll.Controls.Add(this.butValidCmd);
            this.pnlAll.Controls.Add(this.butBack);
            this.pnlAll.Controls.Add(this.butNext);
            this.pnlAll.Controls.Add(this.butCancel);
            this.pnlAll.Controls.Add(this.lblBorder);
            this.pnlAll.Controls.Add(this.pnlStep1);
            this.pnlAll.Controls.Add(this.pnlStep2);
            this.pnlAll.Controls.Add(this.pnlSummary);
            this.pnlAll.Controls.Add(this.pnlWelcome);
            this.pnlAll.Controls.Add(this.pnlStep3);
            this.pnlAll.Dock = System.Windows.Forms.DockStyle.Fill;
            this.pnlAll.Location = new System.Drawing.Point(7, 7);
            this.pnlAll.Name = "pnlAll";
            this.pnlAll.Size = new System.Drawing.Size(862, 428);
            this.pnlAll.TabIndex = 0;
            // 
            // butFinish
            // 
            this.butFinish.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.butFinish.Location = new System.Drawing.Point(686, 393);
            this.butFinish.Name = "butFinish";
            this.butFinish.Size = new System.Drawing.Size(75, 23);
            this.butFinish.TabIndex = 4;
            this.butFinish.Text = "&Finish";
            this.butFinish.UseVisualStyleBackColor = true;
            this.butFinish.Visible = false;
            this.butFinish.Click += new System.EventHandler(this.butFinish_Click);
            // 
            // butValidCmd
            // 
            this.butValidCmd.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
            this.butValidCmd.Image = ((System.Drawing.Image)(resources.GetObject("butValidCmd.Image")));
            this.butValidCmd.ImageAlign = System.Drawing.ContentAlignment.MiddleLeft;
            this.butValidCmd.Location = new System.Drawing.Point(8, 392);
            this.butValidCmd.Name = "butValidCmd";
            this.butValidCmd.Size = new System.Drawing.Size(88, 24);
            this.butValidCmd.TabIndex = 1;
            this.butValidCmd.Text = "Valider";
            this.butValidCmd.Visible = false;
            this.butValidCmd.Click += new System.EventHandler(this.butValidCmd_Click);
            // 
            // butBack
            // 
            this.butBack.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.butBack.Location = new System.Drawing.Point(609, 393);
            this.butBack.Name = "butBack";
            this.butBack.Size = new System.Drawing.Size(75, 23);
			this.butBack.TabIndex = 2;
            this.butBack.Text = "< &Back";
			this.butBack.UseVisualStyleBackColor = true;
            this.butBack.Click += new System.EventHandler(this.butBack_Click);
            // 
            // butNext
            // 
            this.butNext.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.butNext.Location = new System.Drawing.Point(686, 393);
            this.butNext.Name = "butNext";
            this.butNext.Size = new System.Drawing.Size(75, 23);
            this.butNext.TabIndex = 3;
            this.butNext.Text = "&Next >";
			this.butNext.UseVisualStyleBackColor = true;
            this.butNext.Click += new System.EventHandler(this.butNext_Click);
            // 
            // butCancel
            // 
            this.butCancel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.butCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            this.butCancel.Location = new System.Drawing.Point(776, 393);
            this.butCancel.Name = "butCancel";
            this.butCancel.Size = new System.Drawing.Size(75, 23);
            this.butCancel.TabIndex = 5;
            this.butCancel.Text = "Cancel";
			this.butNext.UseVisualStyleBackColor = true;
            // 
            // lblBorder
            // 
            this.lblBorder.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.lblBorder.BackColor = System.Drawing.SystemColors.Control;
            this.lblBorder.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.lblBorder.Location = new System.Drawing.Point(-1, 381);
            this.lblBorder.Name = "lblBorder";
            this.lblBorder.Size = new System.Drawing.Size(862, 1);
            this.lblBorder.TabIndex = 1;
            // 
            // pnlStep3
            // 
            this.pnlStep3.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.pnlStep3.BackColor = System.Drawing.SystemColors.Control;
			this.butCancel.UseVisualStyleBackColor = true;
			this.pnlStep3.BackgroundImageLayout = System.Windows.Forms.ImageLayout.None;
            this.pnlStep3.Controls.Add(this.groupBox1);
            this.pnlStep3.Controls.Add(this.label4);
            this.pnlStep3.Controls.Add(this.pnlHeaderStep3);
            this.pnlStep3.Location = new System.Drawing.Point(0, 0);
            this.pnlStep3.Name = "pnlStep3";
            this.pnlStep3.Size = new System.Drawing.Size(861, 381);
            this.pnlStep3.TabIndex = 0;
            // 
            // groupBox1
            // 
            this.groupBox1.Controls.Add(this.gbxParameter);
            this.groupBox1.Controls.Add(this.lvParams);
            this.groupBox1.Controls.Add(this.butDwn);
            this.groupBox1.Controls.Add(this.butUp);
            this.groupBox1.Location = new System.Drawing.Point(47, 97);
            this.groupBox1.Name = "groupBox1";
            this.groupBox1.Size = new System.Drawing.Size(766, 243);
            this.groupBox1.TabIndex = 1;
            this.groupBox1.TabStop = false;
            // 
            // gbxParameter
            // 
            this.gbxParameter.Controls.Add(this.chkRequired);
            this.gbxParameter.Controls.Add(this.chkNoEcho);
            this.gbxParameter.Controls.Add(this.cbxFields);
            this.gbxParameter.Controls.Add(this.lblField);
            this.gbxParameter.Controls.Add(this.txtDefaultValue);
            this.gbxParameter.Controls.Add(this.lblPrompt);
            this.gbxParameter.Controls.Add(this.txtPrompt);
            this.gbxParameter.Controls.Add(this.lblDefaultValue);
            this.gbxParameter.Location = new System.Drawing.Point(285, 22);
            this.gbxParameter.Name = "gbxParameter";
            this.gbxParameter.Size = new System.Drawing.Size(453, 205);
            this.gbxParameter.TabIndex = 3;
            this.gbxParameter.TabStop = false;
            this.gbxParameter.Text = "Select a parameter to view its properties: ";
            // 
            // chkRequired
            // 
			this.chkRequired.AutoSize = true;
            this.chkRequired.Enabled = false;
            this.chkRequired.Location = new System.Drawing.Point(125, 154);
            this.chkRequired.Name = "chkRequired";
            this.chkRequired.Size = new System.Drawing.Size(107, 17);
            this.chkRequired.TabIndex = 7;
            this.chkRequired.Text = "Answer Required";
			this.chkRequired.UseVisualStyleBackColor = true;
            this.chkRequired.CheckedChanged += new System.EventHandler(this.OnCheckedChanged);
            // 
            // chkNoEcho
            // 
			this.chkNoEcho.AutoSize = true;
            this.chkNoEcho.Enabled = false;
            this.chkNoEcho.Location = new System.Drawing.Point(125, 131);
            this.chkNoEcho.Name = "chkNoEcho";
            this.chkNoEcho.Size = new System.Drawing.Size(68, 17);
            this.chkNoEcho.TabIndex = 6;
            this.chkNoEcho.Text = "No Echo";
			this.chkNoEcho.UseVisualStyleBackColor = true;
            this.chkNoEcho.CheckedChanged += new System.EventHandler(this.OnCheckedChanged);
            // 
            // cbxFields
            // 
            this.cbxFields.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.cbxFields.Enabled = false;
			this.cbxFields.FormattingEnabled = true;
            this.cbxFields.Location = new System.Drawing.Point(125, 52);
            this.cbxFields.Name = "cbxFields";
            this.cbxFields.Size = new System.Drawing.Size(287, 21);
            this.cbxFields.TabIndex = 1;
            this.cbxFields.SelectedIndexChanged += new System.EventHandler(this.cbxFields_SelectedIndexChanged);
            // 
            // lblField
            // 
            this.lblField.AutoSize = true;
            this.lblField.Location = new System.Drawing.Point(21, 55);
            this.lblField.Name = "lblField";
            this.lblField.Size = new System.Drawing.Size(29, 13);
            this.lblField.TabIndex = 0;
            this.lblField.Text = "Field";
            // 
            // txtDefaultValue
            // 
            this.txtDefaultValue.Enabled = false;
            this.txtDefaultValue.Location = new System.Drawing.Point(125, 105);
            this.txtDefaultValue.MaxLength = 32;
            this.txtDefaultValue.Name = "txtDefaultValue";
            this.txtDefaultValue.Size = new System.Drawing.Size(287, 20);
            this.txtDefaultValue.TabIndex = 5;
            this.txtDefaultValue.TextChanged += new System.EventHandler(this.txtDefaultValue_TextChanged);
            // 
            // lblPrompt
            // 
            this.lblPrompt.AutoSize = true;
            this.lblPrompt.Location = new System.Drawing.Point(21, 82);
            this.lblPrompt.Name = "lblPrompt";
            this.lblPrompt.Size = new System.Drawing.Size(33, 13);
            this.lblPrompt.TabIndex = 2;
            this.lblPrompt.Text = "Label";
            // 
            // txtPrompt
            // 
            this.txtPrompt.Enabled = false;
            this.txtPrompt.Location = new System.Drawing.Point(125, 79);
            this.txtPrompt.MaxLength = 32;
            this.txtPrompt.Name = "txtPrompt";
            this.txtPrompt.Size = new System.Drawing.Size(287, 20);
            this.txtPrompt.TabIndex = 3;
            this.txtPrompt.TextChanged += new System.EventHandler(this.txtPrompt_TextChanged);
            // 
            // lblDefaultValue
            // 
            this.lblDefaultValue.AutoSize = true;
            this.lblDefaultValue.Location = new System.Drawing.Point(21, 108);
            this.lblDefaultValue.Name = "lblDefaultValue";
            this.lblDefaultValue.Size = new System.Drawing.Size(71, 13);
            this.lblDefaultValue.TabIndex = 4;
            this.lblDefaultValue.Text = "Default Value";
            // 
            // lvParams
            // 
            this.lvParams.AutoArrange = false;
            this.lvParams.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
            this.colParamName});
            this.lvParams.FullRowSelect = true;
            this.lvParams.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
            this.lvParams.HideSelection = false;
            this.lvParams.LabelEdit = true;
            this.lvParams.Location = new System.Drawing.Point(28, 22);
            this.lvParams.MultiSelect = false;
            this.lvParams.Name = "lvParams";
            this.lvParams.Size = new System.Drawing.Size(215, 205);
            this.lvParams.SmallImageList = this.imageList1;
            this.lvParams.TabIndex = 0;
			this.lvParams.UseCompatibleStateImageBehavior = false;
            this.lvParams.View = System.Windows.Forms.View.Details;
            this.lvParams.AfterLabelEdit += new System.Windows.Forms.LabelEditEventHandler(this.lvParams_AfterLabelEdit);
            this.lvParams.SelectedIndexChanged += new System.EventHandler(this.lvParams_SelectedIndexChanged);
            // 
            // colParamName
            // 
            this.colParamName.Text = "Parameter Name";
            this.colParamName.Width = 190;
            // 
            // imageList1
            // 
            this.imageList1.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList1.ImageStream")));
            this.imageList1.TransparentColor = System.Drawing.Color.Transparent;
			this.imageList1.Images.SetKeyName(0, "package_find.png");
            // 
            // butDwn
            // 
            this.butDwn.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.butDwn.Enabled = false;
            this.butDwn.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.butDwn.Image = ((System.Drawing.Image)(resources.GetObject("butDwn.Image")));
            this.butDwn.Location = new System.Drawing.Point(249, 124);
            this.butDwn.Name = "butDwn";
            this.butDwn.Size = new System.Drawing.Size(30, 30);
            this.butDwn.TabIndex = 2;
            this.butDwn.Click += new System.EventHandler(this.OnChangeParamPos);
            // 
            // butUp
            // 
            this.butUp.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.butUp.Enabled = false;
            this.butUp.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.butUp.Image = ((System.Drawing.Image)(resources.GetObject("butUp.Image")));
            this.butUp.Location = new System.Drawing.Point(249, 87);
            this.butUp.Name = "butUp";
            this.butUp.Size = new System.Drawing.Size(30, 30);
            this.butUp.TabIndex = 1;
            this.butUp.Click += new System.EventHandler(this.OnChangeParamPos);
            // 
            // label4
            // 
            this.label4.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.label4.BackColor = System.Drawing.SystemColors.Control;
            this.label4.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.label4.Location = new System.Drawing.Point(-1, 59);
            this.label4.Name = "label4";
            this.label4.Size = new System.Drawing.Size(862, 1);
            this.label4.TabIndex = 2;
            // 
            // pnlHeaderStep3
            // 
            this.pnlHeaderStep3.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.pnlHeaderStep3.BackColor = System.Drawing.Color.White;
            this.pnlHeaderStep3.Controls.Add(this.lblSubTitleStep3);
            this.pnlHeaderStep3.Controls.Add(this.lblTitleStep3);
            this.pnlHeaderStep3.Location = new System.Drawing.Point(0, 0);
            this.pnlHeaderStep3.Name = "pnlHeaderStep3";
            this.pnlHeaderStep3.Size = new System.Drawing.Size(861, 60);
            this.pnlHeaderStep3.TabIndex = 0;
            // 
            // lblSubTitleStep3
            // 
            this.lblSubTitleStep3.AutoSize = true;
            this.lblSubTitleStep3.Font = new System.Drawing.Font("Tahoma", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.lblSubTitleStep3.Location = new System.Drawing.Point(50, 35);
            this.lblSubTitleStep3.Name = "lblSubTitleStep3";
            this.lblSubTitleStep3.Size = new System.Drawing.Size(538, 14);
            this.lblSubTitleStep3.TabIndex = 1;
            this.lblSubTitleStep3.Text = "In this step you will set the properties of the parameters that you specified in " +
                "the previous step:";
            // 
            // lblTitleStep3
            // 
            this.lblTitleStep3.AutoSize = true;
            this.lblTitleStep3.Font = new System.Drawing.Font("Tahoma", 9.75F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.lblTitleStep3.Location = new System.Drawing.Point(10, 7);
            this.lblTitleStep3.Name = "lblTitleStep3";
            this.lblTitleStep3.Size = new System.Drawing.Size(128, 16);
            this.lblTitleStep3.TabIndex = 0;
            this.lblTitleStep3.Text = "Macro parameters";
            // 
            // pnlStep1
            // 
            this.pnlStep1.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.pnlStep1.BackColor = System.Drawing.SystemColors.Control;
			this.pnlStep1.BackgroundImageLayout = System.Windows.Forms.ImageLayout.None;
            this.pnlStep1.Controls.Add(this.gbxStep1);
            this.pnlStep1.Controls.Add(this.lblBorderStep1);
            this.pnlStep1.Controls.Add(this.pnlHeaderStep1);
            this.pnlStep1.Location = new System.Drawing.Point(0, 0);
            this.pnlStep1.Name = "pnlStep1";
            this.pnlStep1.Size = new System.Drawing.Size(861, 381);
            this.pnlStep1.TabIndex = 0;
            // 
            // gbxStep1
            // 
            this.gbxStep1.Controls.Add(this.txtDescription);
            this.gbxStep1.Controls.Add(this.lblDescription);
            this.gbxStep1.Controls.Add(this.txtMenuLabel);
            this.gbxStep1.Controls.Add(this.lblMenuLabel);
            this.gbxStep1.Controls.Add(this.txtUniqueName);
            this.gbxStep1.Controls.Add(this.lblUniqueName);
            this.gbxStep1.Location = new System.Drawing.Point(158, 113);
            this.gbxStep1.Name = "gbxStep1";
            this.gbxStep1.Size = new System.Drawing.Size(544, 208);
            this.gbxStep1.TabIndex = 0;
            this.gbxStep1.TabStop = false;
            // 
            // txtDescription
            // 
            this.txtDescription.Location = new System.Drawing.Point(122, 71);
            this.txtDescription.Multiline = true;
            this.txtDescription.Name = "txtDescription";
            this.txtDescription.Size = new System.Drawing.Size(401, 125);
            this.txtDescription.TabIndex = 5;
            // 
            // lblDescription
            // 
            this.lblDescription.AutoSize = true;
            this.lblDescription.Location = new System.Drawing.Point(26, 74);
            this.lblDescription.Name = "lblDescription";
            this.lblDescription.Size = new System.Drawing.Size(60, 13);
            this.lblDescription.TabIndex = 4;
            this.lblDescription.Text = "Description";
            // 
            // txtMenuLabel
            // 
            this.txtMenuLabel.Location = new System.Drawing.Point(122, 45);
            this.txtMenuLabel.MaxLength = 32;
            this.txtMenuLabel.Name = "txtMenuLabel";
            this.txtMenuLabel.Size = new System.Drawing.Size(307, 20);
            this.txtMenuLabel.TabIndex = 3;
            // 
            // lblMenuLabel
            // 
            this.lblMenuLabel.AutoSize = true;
            this.lblMenuLabel.Location = new System.Drawing.Point(26, 48);
            this.lblMenuLabel.Name = "lblMenuLabel";
            this.lblMenuLabel.Size = new System.Drawing.Size(59, 13);
            this.lblMenuLabel.TabIndex = 2;
            this.lblMenuLabel.Text = "Menu label";
            // 
            // txtUniqueName
            // 
            this.txtUniqueName.Location = new System.Drawing.Point(122, 19);
            this.txtUniqueName.MaxLength = 32;
            this.txtUniqueName.Name = "txtUniqueName";
            this.txtUniqueName.Size = new System.Drawing.Size(307, 20);
            this.txtUniqueName.TabIndex = 1;
            // 
            // lblUniqueName
            // 
            this.lblUniqueName.AutoSize = true;
            this.lblUniqueName.Location = new System.Drawing.Point(26, 22);
            this.lblUniqueName.Name = "lblUniqueName";
            this.lblUniqueName.Size = new System.Drawing.Size(72, 13);
            this.lblUniqueName.TabIndex = 0;
            this.lblUniqueName.Text = "Unique Name";
            // 
            // lblBorderStep1
            // 
            this.lblBorderStep1.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.lblBorderStep1.BackColor = System.Drawing.SystemColors.Control;
            this.lblBorderStep1.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.lblBorderStep1.Location = new System.Drawing.Point(-1, 59);
            this.lblBorderStep1.Name = "lblBorderStep1";
            this.lblBorderStep1.Size = new System.Drawing.Size(862, 1);
            this.lblBorderStep1.TabIndex = 2;
            // 
            // pnlHeaderStep1
            // 
            this.pnlHeaderStep1.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.pnlHeaderStep1.BackColor = System.Drawing.Color.White;
            this.pnlHeaderStep1.Controls.Add(this.lblSubTitleStep1);
            this.pnlHeaderStep1.Controls.Add(this.lblTitleStep1);
            this.pnlHeaderStep1.Location = new System.Drawing.Point(0, 0);
            this.pnlHeaderStep1.Name = "pnlHeaderStep1";
            this.pnlHeaderStep1.Size = new System.Drawing.Size(861, 60);
            this.pnlHeaderStep1.TabIndex = 0;
            // 
            // lblSubTitleStep1
            // 
            this.lblSubTitleStep1.AutoSize = true;
            this.lblSubTitleStep1.Font = new System.Drawing.Font("Tahoma", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.lblSubTitleStep1.Location = new System.Drawing.Point(50, 35);
            this.lblSubTitleStep1.Name = "lblSubTitleStep1";
            this.lblSubTitleStep1.Size = new System.Drawing.Size(525, 14);
            this.lblSubTitleStep1.TabIndex = 1;
            this.lblSubTitleStep1.Text = "In this step you will be asked to provide  the name, menu label and description o" +
                "f your macro.";
            // 
            // lblTitleStep1
            // 
            this.lblTitleStep1.AutoSize = true;
            this.lblTitleStep1.Font = new System.Drawing.Font("Tahoma", 9.75F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.lblTitleStep1.Location = new System.Drawing.Point(10, 7);
            this.lblTitleStep1.Name = "lblTitleStep1";
            this.lblTitleStep1.Size = new System.Drawing.Size(120, 16);
            this.lblTitleStep1.TabIndex = 0;
            this.lblTitleStep1.Text = "Macro properties";
            // 
            // pnlStep2
            // 
            this.pnlStep2.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.pnlStep2.BackColor = System.Drawing.SystemColors.Control;
			this.pnlStep2.BackgroundImageLayout = System.Windows.Forms.ImageLayout.None;
            this.pnlStep2.Controls.Add(this.qeWhereClause);
            this.pnlStep2.Controls.Add(this.lblBorderStep2);
            this.pnlStep2.Controls.Add(this.pnlHeaderStep2);
            this.pnlStep2.Location = new System.Drawing.Point(0, 0);
            this.pnlStep2.Name = "pnlStep2";
            this.pnlStep2.Size = new System.Drawing.Size(861, 381);
            this.pnlStep2.TabIndex = 0;
            // 
            // qeWhereClause
            // 
            this.qeWhereClause.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.qeWhereClause.Location = new System.Drawing.Point(3, 63);
            this.qeWhereClause.Name = "qeWhereClause";
            this.qeWhereClause.Query = "";
            this.qeWhereClause.Size = new System.Drawing.Size(854, 315);
            this.qeWhereClause.TabIndex = 0;
            // 
            // lblBorderStep2
            // 
            this.lblBorderStep2.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.lblBorderStep2.BackColor = System.Drawing.SystemColors.Control;
            this.lblBorderStep2.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.lblBorderStep2.Location = new System.Drawing.Point(-1, 59);
            this.lblBorderStep2.Name = "lblBorderStep2";
            this.lblBorderStep2.Size = new System.Drawing.Size(862, 1);
            this.lblBorderStep2.TabIndex = 2;
            // 
            // pnlHeaderStep2
            // 
            this.pnlHeaderStep2.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.pnlHeaderStep2.BackColor = System.Drawing.Color.White;
            this.pnlHeaderStep2.Controls.Add(this.lblSubTitleStep2);
            this.pnlHeaderStep2.Controls.Add(this.lblTitleStep2);
            this.pnlHeaderStep2.Location = new System.Drawing.Point(0, 0);
            this.pnlHeaderStep2.Name = "pnlHeaderStep2";
			this.pnlHeaderStep2.Padding = new System.Windows.Forms.Padding(7);
            this.pnlHeaderStep2.Size = new System.Drawing.Size(861, 60);
            this.pnlHeaderStep2.TabIndex = 0;
            // 
            // lblSubTitleStep2
            // 
            this.lblSubTitleStep2.AutoSize = true;
            this.lblSubTitleStep2.Font = new System.Drawing.Font("Tahoma", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.lblSubTitleStep2.Location = new System.Drawing.Point(50, 28);
            this.lblSubTitleStep2.Name = "lblSubTitleStep2";
            this.lblSubTitleStep2.Size = new System.Drawing.Size(628, 28);
            this.lblSubTitleStep2.TabIndex = 1;
            this.lblSubTitleStep2.Text = "Provide the WHERE clause that will be used to load the selection buffer data.\r\nNo" +
                "te: Your WHERE clause will be appended to the auto generated command \"SELECT * F" +
                "ROM [TABLE_NAME]\".";
            // 
            // lblTitleStep2
            // 
            this.lblTitleStep2.AutoSize = true;
            this.lblTitleStep2.Font = new System.Drawing.Font("Tahoma", 9.75F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.lblTitleStep2.Location = new System.Drawing.Point(10, 7);
            this.lblTitleStep2.Name = "lblTitleStep2";
            this.lblTitleStep2.Size = new System.Drawing.Size(142, 16);
            this.lblTitleStep2.TabIndex = 0;
            this.lblTitleStep2.Text = "Macro SQL Command";
            // 
            // pnlSummary
            // 
            this.pnlSummary.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.pnlSummary.BackColor = System.Drawing.Color.White;
			this.pnlSummary.BackgroundImageLayout = System.Windows.Forms.ImageLayout.None;
            this.pnlSummary.Controls.Add(this.txtSummary);
            this.pnlSummary.Controls.Add(this.pbSummary);
            this.pnlSummary.Controls.Add(this.lblSummary);
            this.pnlSummary.Location = new System.Drawing.Point(0, 0);
            this.pnlSummary.Name = "pnlSummary";
            this.pnlSummary.Size = new System.Drawing.Size(861, 381);
            this.pnlSummary.TabIndex = 0;
            // 
            // txtSummary
            // 
            this.txtSummary.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.txtSummary.BackColor = System.Drawing.Color.White;
            this.txtSummary.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
            this.txtSummary.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.txtSummary.Location = new System.Drawing.Point(185, 53);
            this.txtSummary.Multiline = true;
            this.txtSummary.Name = "txtSummary";
            this.txtSummary.ReadOnly = true;
            this.txtSummary.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
            this.txtSummary.Size = new System.Drawing.Size(653, 311);
            this.txtSummary.TabIndex = 1;
            this.txtSummary.TabStop = false;
            this.txtSummary.Text = "Press finish to create the following  Macro:";
            // 
            // pbSummary
            // 
            this.pbSummary.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)));
            this.pbSummary.Image = ((System.Drawing.Image)(resources.GetObject("pbSummary.Image")));
            this.pbSummary.Location = new System.Drawing.Point(0, 0);
            this.pbSummary.Name = "pbSummary";
            this.pbSummary.Size = new System.Drawing.Size(163, 381);
            this.pbSummary.SizeMode = System.Windows.Forms.PictureBoxSizeMode.StretchImage;
            this.pbSummary.TabIndex = 2;
            this.pbSummary.TabStop = false;
            // 
            // lblSummary
            // 
            this.lblSummary.AutoSize = true;
            this.lblSummary.Font = new System.Drawing.Font("Tahoma", 18F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.lblSummary.Location = new System.Drawing.Point(179, 20);
            this.lblSummary.Name = "lblSummary";
            this.lblSummary.Size = new System.Drawing.Size(127, 29);
            this.lblSummary.TabIndex = 0;
            this.lblSummary.Text = "Summary";
            // 
            // pnlWelcome
            // 
            this.pnlWelcome.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.pnlWelcome.BackColor = System.Drawing.Color.White;
			this.pnlWelcome.BackgroundImageLayout = System.Windows.Forms.ImageLayout.None;
            this.pnlWelcome.Controls.Add(this.pictureBox1);
            this.pnlWelcome.Controls.Add(this.lblIntro);
            this.pnlWelcome.Controls.Add(this.lblWelcome);
            this.pnlWelcome.Location = new System.Drawing.Point(0, 0);
            this.pnlWelcome.Name = "pnlWelcome";
            this.pnlWelcome.Size = new System.Drawing.Size(861, 381);
            this.pnlWelcome.TabIndex = 0;
            // 
            // pictureBox1
            // 
            this.pictureBox1.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)));
            this.pictureBox1.Image = ((System.Drawing.Image)(resources.GetObject("pictureBox1.Image")));
            this.pictureBox1.Location = new System.Drawing.Point(0, 0);
            this.pictureBox1.Name = "pictureBox1";
            this.pictureBox1.Size = new System.Drawing.Size(163, 381);
            this.pictureBox1.SizeMode = System.Windows.Forms.PictureBoxSizeMode.StretchImage;
            this.pictureBox1.TabIndex = 2;
            this.pictureBox1.TabStop = false;
            // 
            // lblIntro
            // 
            this.lblIntro.Font = new System.Drawing.Font("Tahoma", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(177)));
            this.lblIntro.Location = new System.Drawing.Point(182, 79);
            this.lblIntro.Name = "lblIntro";
            this.lblIntro.Size = new System.Drawing.Size(598, 285);
            this.lblIntro.TabIndex = 1;
            this.lblIntro.Text = resources.GetString("lblIntro.Text");
            // 
            // lblWelcome
            // 
            this.lblWelcome.AutoSize = true;
            this.lblWelcome.Font = new System.Drawing.Font("Tahoma", 18F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.lblWelcome.Location = new System.Drawing.Point(179, 20);
            this.lblWelcome.Name = "lblWelcome";
            this.lblWelcome.Size = new System.Drawing.Size(466, 29);
            this.lblWelcome.TabIndex = 0;
            this.lblWelcome.Text = "Welcome to the Create Macro Wizard!";
            // 
            // epErrors
            // 
            this.epErrors.ContainerControl = this;
            // 
            // frmEditMacro
            // 
            this.AcceptButton = this.butFinish;
			this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
            this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
            this.CancelButton = this.butCancel;
            this.ClientSize = new System.Drawing.Size(876, 442);
            this.Controls.Add(this.pnlAll);
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.Sizable;
            this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.MaximizeBox = false;
            this.MinimizeBox = false;
            this.MinimumSize = new System.Drawing.Size(876, 442); 
            this.Name = "frmEditMacro";
			this.Padding = new System.Windows.Forms.Padding(7);
            this.ShowInTaskbar = false;
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
            this.Text = "Create Macro Wizard";
            this.pnlAll.ResumeLayout(false);
            this.pnlStep3.ResumeLayout(false);
            this.groupBox1.ResumeLayout(false);
            this.gbxParameter.ResumeLayout(false);
            this.gbxParameter.PerformLayout();
            this.pnlHeaderStep3.ResumeLayout(false);
            this.pnlHeaderStep3.PerformLayout();
            this.pnlStep1.ResumeLayout(false);
            this.gbxStep1.ResumeLayout(false);
            this.gbxStep1.PerformLayout();
            this.pnlHeaderStep1.ResumeLayout(false);
            this.pnlHeaderStep1.PerformLayout();
            this.pnlStep2.ResumeLayout(false);
            this.pnlHeaderStep2.ResumeLayout(false);
            this.pnlHeaderStep2.PerformLayout();
            this.pnlSummary.ResumeLayout(false);
            this.pnlSummary.PerformLayout();
            ((System.ComponentModel.ISupportInitialize)(this.pbSummary)).EndInit();
            this.pnlWelcome.ResumeLayout(false);
            this.pnlWelcome.PerformLayout();
            ((System.ComponentModel.ISupportInitialize)(this.pictureBox1)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.epErrors)).EndInit();
            this.ResumeLayout(false);

        }

        #endregion

        private System.Windows.Forms.Panel pnlAll;
        private System.Windows.Forms.Panel pnlWelcome;
        private System.Windows.Forms.Label lblBorder;
        private System.Windows.Forms.Button butBack;
        private System.Windows.Forms.Button butNext;
        private System.Windows.Forms.Button butCancel;
        private System.Windows.Forms.Label lblWelcome;
        private System.Windows.Forms.PictureBox pictureBox1;
        private System.Windows.Forms.Panel pnlStep1;
        private System.Windows.Forms.Panel pnlHeaderStep1;
        private System.Windows.Forms.Label lblBorderStep1;
        private System.Windows.Forms.Panel pnlStep2;
        private System.Windows.Forms.Label lblBorderStep2;
        private System.Windows.Forms.Panel pnlHeaderStep2;
        private System.Windows.Forms.GroupBox gbxStep1;
        private System.Windows.Forms.TextBox txtUniqueName;
        private System.Windows.Forms.Label lblUniqueName;
        private System.Windows.Forms.TextBox txtDescription;
        private System.Windows.Forms.Label lblDescription;
        private System.Windows.Forms.TextBox txtMenuLabel;
        private System.Windows.Forms.Label lblMenuLabel;
        private System.Windows.Forms.Label lblSubTitleStep1;
        private System.Windows.Forms.Label lblTitleStep1;
        private RepPP.Miscellaneous.QueryEditor qeWhereClause;
        private System.Windows.Forms.Label lblSubTitleStep2;
        private System.Windows.Forms.Label lblTitleStep2;
        private System.Windows.Forms.Panel pnlStep3;
        private System.Windows.Forms.GroupBox groupBox1;
        private System.Windows.Forms.Label lblDefaultValue;
        private System.Windows.Forms.TextBox txtPrompt;
        private System.Windows.Forms.Label lblPrompt;
        private System.Windows.Forms.Label lblField;
        private System.Windows.Forms.Label label4;
        private System.Windows.Forms.Panel pnlHeaderStep3;
        private System.Windows.Forms.Label lblSubTitleStep3;
        private System.Windows.Forms.Label lblTitleStep3;
        private System.Windows.Forms.Button butValidCmd;
        private System.Windows.Forms.Button butDwn;
        private System.Windows.Forms.Button butUp;
        private System.Windows.Forms.GroupBox gbxParameter;
        private System.Windows.Forms.TextBox txtDefaultValue;
        private System.Windows.Forms.ListView lvParams;
        private System.Windows.Forms.ComboBox cbxFields;
        private System.Windows.Forms.ColumnHeader colParamName;
        private System.Windows.Forms.ImageList imageList1;
        private System.Windows.Forms.Button butFinish;
        private System.Windows.Forms.Label lblIntro;
        private System.Windows.Forms.Panel pnlSummary;
        private System.Windows.Forms.TextBox txtSummary;
        private System.Windows.Forms.PictureBox pbSummary;
        private System.Windows.Forms.Label lblSummary;
        private System.Windows.Forms.ErrorProvider epErrors;
        private System.Windows.Forms.CheckBox chkRequired;
        private System.Windows.Forms.CheckBox chkNoEcho;
    } // Class
} // Namespace