using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
    //*********************************************************     
    //
    /// <summary>
    /// Class implementing a Main Editor that does not use a 
    /// selection buffer
    /// </summary>
    //
    //*********************************************************     
    public class frmEditorStandalone : frmEditor {
        /// <summary>The current child editor form</summary>
        private frmChildEditor                   m_frmChildEditor = null;
        /// <summary>Required designer variable. </summary>
        private System.ComponentModel.IContainer components       = null;

        //*********************************************************     
        //
        /// <summary>
        /// ctor:                               Class constructor.
        /// </summary>
        //  
        //*********************************************************     
        public frmEditorStandalone() {
            InitializeComponent();
        }

        //*********************************************************     
        /// <summary>
        /// Dispose                     Clean up any resources being used.
        /// </summary>
        /// <param name="disposing">    true if managed resources should be disposed; otherwise, false.</param>
        //*********************************************************     
        protected override void Dispose(bool disposing) {
            if (disposing && (components != null)) {
                components.Dispose();
            }
            base.Dispose(disposing);
        }

        //******************************************************************
        //
        /// <summary>
        /// CurrentChildEditor:         Current child editor
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public virtual frmChildEditor CurrentChildEditor {
            get {
                return(m_frmChildEditor);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// InitEditor:                                 Initializing the editor
        /// </summary>
        /// <param name="app">                          Rep++ application object</param>
        /// <param name="configHelper">                 Object keeping the UI configuration</param>
        /// <param name="strTopTranslatorAtomName">     Name of the atom use by the top translator</param>
        /// <param name="strBaseTranslatorAtomName">    Name of the atom use by the base translator. Null for default one.</param>
        //
        //******************************************************************
        protected override void InitEditor(RepPP.Application app, UIConfigurationHelper configHelper, string strTopTranslatorAtomName, string strBaseTranslatorAtomName) {
            base.InitEditor(app,
                            configHelper,
                            strTopTranslatorAtomName,
                            (strBaseTranslatorAtomName != null) ? strBaseTranslatorAtomName : "txtfrmTopBaseEditor");
        }

        //******************************************************************
        //
        /// <summary>
        /// LoadChildEditor:                 Loades the specified child editor
        ///                                  in the specified child container.
        /// </summary>
        /// <param name="ctlChildContainer"> Control that will contain the child editor</param>
        /// <param name="frmChildEditor">    Child editor object</param>
        //
        //******************************************************************
        protected virtual bool LoadChildEditor(Control         ctlChildContainer,
                                               frmChildEditor  frmChildEditor) {
            return(LoadChildEditor(ctlChildContainer, frmChildEditor, string.Empty))  ;
        }

        //******************************************************************
        //
        /// <summary>
        /// LoadChildEditor:                 Loades the specified child editor
        ///                                  in the specified child container.
        /// </summary>
        /// <param name="ctlChildContainer"> Control that will contain the child editor</param>
        /// <param name="frmChildEditor">    Child editor object</param>
        /// <param name="strWhereClause">    Where clause</param>
        //
        //******************************************************************
        protected virtual bool LoadChildEditor(Control         ctlChildContainer,
                                               frmChildEditor  frmChildEditor,
                                               string          strWhereClause) {
            bool bRetVal = true;

            if (m_frmChildEditor != null) {
                bRetVal = m_frmChildEditor.DetachChildEditor(false);
            }
            if (bRetVal) {
                ChildContainer   = ctlChildContainer;
                m_frmChildEditor = frmChildEditor;
                if (frmChildEditor == null || ctlChildContainer == null) {
                    bRetVal = false;
                } else {
                    frmChildEditor.AttachChildEditor(ctlChildContainer);
                    RegisterTriggerToChildEditor(frmChildEditor.Handler as RowsetTreeDefNodeHandlerRootBase);
                    frmChildEditor.OpenSet(strWhereClause);
                }
            }
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// RegisterTriggerToChildEditor:   Register the trigger for the child editor commands
        /// </summary>
        /// <param name="nodeHandler">    Framework card manager</param>
        //
        //******************************************************************
        public virtual void RegisterTriggerToChildEditor(RowsetTreeDefNodeHandlerRootBase nodeHandler) {
        }

        #region Windows Form Designer generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent() {
            this.components = new System.ComponentModel.Container();
            this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
            this.Text = "frmEditorStandalone";
        }

        #endregion
    } // Class frmEditorStandalone
} // Namespace