using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
    //********************************************************************************
    //
    /// <summary>
    /// Implements the base interface of an editor without a selection buffer
    /// </summary>
    //
    //********************************************************************************    
    public class frmEditorStandaloneUI : frmEditorStandalone {
        /// <summary>Main Menu</summary>
        protected MenuStrip             mainMenu1;
        /// <summary>File menu item</summary>
        protected ToolStripMenuItem     mnuFile;
        /// <summary>New menu item</summary>
        protected ToolStripMenuItem     mnuItemNew;
        /// <summary>Duplicate menu item</summary>
        protected ToolStripMenuItem     mnuItemDup;
        /// <summary>Save menu item</summary>
        protected ToolStripMenuItem     mnuItemSave;
        /// <summary>Delete menu item</summary>
        protected ToolStripMenuItem     mnuItemDel;
        /// <summary>Undo menu item</summary>
        protected ToolStripMenuItem     mnuItemUndo;
        /// <summary>Exit menu item</summary>
        protected ToolStripMenuItem     mnuExit;
        /// <summary>Edit menu item</summary>
        protected ToolStripMenuItem     mnuEdit;
        /// <summary>FieldDef Choice menu item</summary>
        protected ToolStripMenuItem     mnuFieldChoice;
        /// <summary>FieldDef Erase menu item</summary>
        protected ToolStripMenuItem     mnuFieldErase;
        /// <summary>FieldDef Default menu item</summary>
        protected ToolStripMenuItem     mnuFieldDefault;
        /// <summary>FieldDef Erase All menu item</summary>
        protected ToolStripMenuItem     mnuFieldEraseAll;
        /// <summary>FieldDef Default All menu item</summary>
        protected ToolStripMenuItem     mnuFieldDefaultAll;
        /// <summary>Refresh menu item</summary>
        protected ToolStripMenuItem     mnuRefresh;
        /// <summary>Selection menu item</summary>
        protected ToolStripMenuItem     mnuSelection;
        /// <summary>Previous menu item</summary>
        protected ToolStripMenuItem     mnuItemPrev;
        /// <summary>Next menu item</summary>
        protected ToolStripMenuItem     mnuItemNext;
        /// <summary>First menu item</summary>
        protected ToolStripMenuItem     mnuItemFirst;
        /// <summary>Last menu item</summary>
        protected ToolStripMenuItem     mnuItemLast;
        /// <summary>Help menu item</summary>
        protected ToolStripMenuItem     mnuHelp;
        /// <summary>FieldDef Help menu item</summary>
        protected ToolStripMenuItem     mnuFieldHelp;
        /// <summary>Content menu item</summary>
        protected ToolStripMenuItem     mnuHelpContent;
        /// <summary>About menu item</summary>
        protected ToolStripMenuItem     mnuHelpAbout;
        /// <summary>StatusBar</summary>
        protected StatusStrip           mainStatusBar;
        /// <summary>StatusBar Panel</summary>
        protected ToolStripStatusLabel  statusBarPanel1;
        /// <summary>RowsetTreeDef ToolBar</summary>
        protected ToolbarRowsetTree                             toolBarRowsetTree;
        private System.ComponentModel.IContainer                components = null;
        /// <summary>Panel that will host the Child Editors</summary>
        protected Panel                                         pnlChildEditorContainer;
        /// <summary>Control displaying error icon on the side of a field</summary>
        private FldErrorProvider                                m_errorProvider;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                                       Class constructor
        /// </summary>
        //
        //******************************************************************
        public frmEditorStandaloneUI() {
            InitializeComponent();
        }

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the form
        /// </summary>
        /// <param name="bDisposing">   true if disposing, false if finalizing</param>
        //
        //******************************************************************
        protected override void Dispose(bool bDisposing) {
            RepPP.Application   app;

            if (bDisposing) {
                app = Application;
                if (app != null) {
                    app.FldOnError -= new RepPP.FldOnErrorEventHandler(OnFldOnError);
                }
                if (components != null) {
                    components.Dispose();
                }
            }
            base.Dispose(bDisposing);
        }

        //******************************************************************
        //
        /// <summary>
        /// InitEditor:                                 Initializing the editor
        /// </summary>
        /// <param name="app">                          Rep++ application object</param>
        /// <param name="configHelper">                 Object keeping the UI configuration</param>
        /// <param name="strTopTranslatorAtomName">     Name of the atom use by the top translator</param>
        /// <param name="strBaseTranslatorAtomName">    Name of the atom use by the base translator. Null for default one</param>
        //
        //******************************************************************
        protected override void InitEditor(RepPP.Application app, UIConfigurationHelper configHelper, string strTopTranslatorAtomName, string strBaseTranslatorAtomName) {
            base.InitEditor(app, configHelper, strTopTranslatorAtomName, strBaseTranslatorAtomName);
            if (app != null) {
                m_errorProvider                 = new FldErrorProvider(ErrorIconAlignment.MiddleLeft, 2);
                Application.FldOnError         += new RepPP.FldOnErrorEventHandler(OnFldOnError);
                Application.FldEnter           += new FldEnterEventHandler(OnFldEnter);
                RegisterEditorCmd();
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// LoadChildEditor:                 Loades the specified child editor
        ///                                  in the child container.
        /// </summary>
        /// <param name="frmChildEditor">    Child editor object</param>
        //
        //******************************************************************
        protected virtual bool LoadChildEditor(frmChildEditor  frmChildEditor) {
            return(LoadChildEditor(pnlChildEditorContainer, frmChildEditor, string.Empty));
        }

        //******************************************************************
        //
        /// <summary>
        /// LoadChildEditor:                 Loades the specified child editor
        ///                                  in the child container.
        /// </summary>
        /// <param name="frmChildEditor">    Child editor object</param>
        /// <param name="strWhereClause">    Where clause</param>
        //
        //******************************************************************
        protected virtual bool LoadChildEditor(frmChildEditor  frmChildEditor,
                                               string          strWhereClause) {
            return(LoadChildEditor(pnlChildEditorContainer, frmChildEditor, strWhereClause));
        }

        //******************************************************************
        //
        /// <summary>
        /// ErrorProvider:          The error provider object responsible to display
        ///                         the error icon in the side of the field controls
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public FldErrorProvider ErrorProvider {
            get {
                return(m_errorProvider);
            }
            set {
                m_errorProvider = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// RegisterEditorCmd:              Register the editor command
        /// </summary>
        //
        //******************************************************************
        protected virtual void RegisterEditorCmd() {
            CmdHolder   cmdHolder;
            
            cmdHolder = CommandHolder;
            cmdHolder.AddCmd("Exit",            new CmdHolder.OnCommand(CmdExit), CmdHolder.CmdStateE.Enabled);
            cmdHolder.AddCmd("FormRefresh",     new CmdHolder.OnCommand(FormRefresh), CmdHolder.CmdStateE.Enabled);
            cmdHolder.AddTrigger(null, "Exit",          new object[] { mnuExit} );
            cmdHolder.AddTrigger(null, "FormRefresh",   new object[] { mnuRefresh} );
        }

        //******************************************************************
        //
        /// <summary>
        /// RegisterTriggerToChildEditor:   Register the trigger for the child editor commands
        /// </summary>
        /// <param name="nodeHandler">    Framework card manager</param>
        //
        //******************************************************************
        public override void RegisterTriggerToChildEditor(RowsetTreeDefNodeHandlerRootBase nodeHandler) {            
            CmdHolder   cmdHolder;
            
            cmdHolder = CommandHolder;
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemSave.ToString(),        new object[] { mnuItemSave });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemUndo.ToString(),        new object[] { mnuItemUndo });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldChoice.ToString(),     new object[] { mnuFieldChoice });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldHelp.ToString(),       new object[] { mnuFieldHelp });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldErase.ToString(),      new object[] { mnuFieldErase });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldDefault.ToString(),    new object[] { mnuFieldDefault });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldEraseAll.ToString(),   new object[] { mnuFieldEraseAll });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldDefaultAll.ToString(), new object[] { mnuFieldDefaultAll });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemFirst.ToString(),       new object[] { mnuItemFirst });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemPrev.ToString(),        new Object[] { mnuItemPrev });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemNext.ToString(),        new Object[] { mnuItemNext });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemLast.ToString(),        new Object[] { mnuItemLast });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemNew.ToString(),         new Object[] { mnuItemNew });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemDup.ToString(),         new Object[] { mnuItemDup });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemDel.ToString(),         new Object[] { mnuItemDel });
            if (nodeHandler != null) {
                if (toolBarRowsetTree.RootNodeHandler != null) {
                    toolBarRowsetTree.RootNodeHandler.ResetFieldError -= new EventHandler(this.OnResetFieldError);
                }
                nodeHandler.ResetFieldError       += new EventHandler(this.OnResetFieldError);
                toolBarRowsetTree.RootNodeHandler     = nodeHandler;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// CmdExit:                    Exit the program
        /// </summary>
        //  
        //*********************************************************
        protected virtual void CmdExit() {
            Close();
        }

        //*********************************************************
        //
        /// <summary>
        /// FormRefresh:                Refresh the form
        /// </summary>
        /// <param name="bForceClose">  True to force close. Don't ask for saving</param>
        //  
        //*********************************************************
        public virtual void FormRefresh(bool bForceClose) {
            CurrentChildEditor.FormRefresh(bForceClose);          
        }

        //*********************************************************
        //
        /// <summary>
        /// FormRefresh:                Refresh the form
        /// </summary>
        //  
        //*********************************************************
        public virtual void FormRefresh() {
            FormRefresh(false);
        }

        //*********************************************************
        //
        /// <summary>
        /// OnClosing:                  Check before closing the form
        /// </summary>
        //  
        //*********************************************************
        protected override void OnClosing(CancelEventArgs e) {
            base.OnClosing(e);
            if (!e.Cancel && CurrentChildEditor != null && !CurrentChildEditor.AskToSave(false)) {
                e.Cancel = true;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnResetFieldError:      Handle the framework reset field error event
        /// </summary>
        /// <param name="sender">   Event sender</param>
        /// <param name="e">        Event arguments</param>
        // 
        //******************************************************************
        protected virtual void OnResetFieldError(Object sender, EventArgs e) {
            statusBarPanel1.Text = "";
            ErrorProvider.Reset();
        }

		//******************************************************************
		//
		/// <summary>
		/// OnFldOnError:           Handle the rep++ field error event
		/// </summary>
		/// <param name="sender">   Event sender</param>
		/// <param name="e">        Event arguments</param>
		// 
		//******************************************************************
		protected virtual void OnFldOnError(Object sender, RepPP.FieldGenEventArgs e) {
            FldErrorProvider errorProvider;

            errorProvider = ErrorProvider;
			errorProvider.Reset();
			errorProvider.AddError(this, e.FieldDef, e.ErrorText);
            // TODO: Meaningful name for statusBarPanel1
            statusBarPanel1.Text = e.FieldDef.Prompt + ": " + e.ErrorText;
		}

        //******************************************************************
        //
        /// <summary>
        /// OnFldEnter:             Handle the rep++ field enter event to
        ///                         enable\disable search button
        /// </summary>
        /// <param name="sender">   Event sender</param>
        /// <param name="e">        Event arguments</param>
        // 
        //******************************************************************
        void OnFldEnter(object sender, FieldEventArgs e) {
            string strRootGroupName = string.Empty;

            if (CurrentChildEditor != null &&
                CurrentChildEditor.Handler != null &&
                CurrentChildEditor.Handler.Rowset != null) {
                strRootGroupName = CurrentChildEditor.Handler.Rowset.RowsetDef.Name;
            }
            if (e.FieldDef.RowsetDef.Name == strRootGroupName) {
                CommandHolder.SetCmdState(sdHandlerCmd.RowsetTreeSearch.ToString(), CmdHolder.CmdStateE.Enabled);
            } else {
                CommandHolder.SetCmdState(sdHandlerCmd.RowsetTreeSearch.ToString(), CmdHolder.CmdStateE.Disabled);
            }
        }

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent() {
            this.components = new System.ComponentModel.Container();
            this.mainMenu1 = new System.Windows.Forms.MenuStrip();
            this.mnuFile = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemNew = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemDup = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemSave = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemDel = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemUndo = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuExit = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuEdit = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuFieldChoice = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuFieldErase = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuFieldDefault = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuFieldEraseAll = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuFieldDefaultAll = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuRefresh = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuSelection = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemPrev = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemNext = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemFirst = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemLast = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuHelp = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuFieldHelp = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuHelpContent = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuHelpAbout = new System.Windows.Forms.ToolStripMenuItem();
            this.mainStatusBar = new System.Windows.Forms.StatusStrip();
            this.statusBarPanel1 = new System.Windows.Forms.ToolStripStatusLabel();
            this.mainMenu1.SuspendLayout();
            this.mainStatusBar.SuspendLayout();
            this.pnlChildEditorContainer = new System.Windows.Forms.Panel();
            this.toolBarRowsetTree = new RepPP.Framework.Window.ToolbarRowsetTree();

            this.SuspendLayout();
           
            // 
            // mainMenu1
            // 
            this.mainMenu1.BackColor = System.Drawing.SystemColors.Control;
            this.mainMenu1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.mnuFile,
            this.mnuEdit,
            this.mnuSelection,
            this.mnuHelp});
            this.mainMenu1.Location = new System.Drawing.Point(0, 0);
            this.mainMenu1.Name = "mainMenu1";
            this.mainMenu1.Size = new System.Drawing.Size(336, 24);
            this.mainMenu1.TabIndex = 16;
            this.mainMenu1.Text = "mainMenu1";
            // 
            // mnuFile
            // 
            this.mnuFile.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.mnuItemNew,
            this.mnuItemDup,
            this.mnuItemSave,
            this.mnuItemDel,
            this.mnuItemUndo,
            this.mnuExit});
            this.mnuFile.Name = "mnuFile";
            this.mnuFile.Size = new System.Drawing.Size(35, 20);
            this.mnuFile.Text = "&File";
            // 
            // mnuItemNew
            // 
            this.mnuItemNew.Name = "mnuItemNew";
            this.mnuItemNew.Size = new System.Drawing.Size(118, 22);
            this.mnuItemNew.Text = "&New";
            // 
            // mnuItemDup
            // 
            this.mnuItemDup.Name = "mnuItemDup";
            this.mnuItemDup.Size = new System.Drawing.Size(118, 22);
            this.mnuItemDup.Text = "D&uplicate";
            // 
            // mnuItemSave
            // 
            this.mnuItemSave.Name = "mnuItemSave";
            this.mnuItemSave.Size = new System.Drawing.Size(118, 22);
            this.mnuItemSave.Text = "&Save";
            // 
            // mnuItemDel
            // 
            this.mnuItemDel.Name = "mnuItemDel";
            this.mnuItemDel.Size = new System.Drawing.Size(118, 22);
            this.mnuItemDel.Text = "&Delete";
            // 
            // mnuItemUndo
            // 
            this.mnuItemUndo.Name = "mnuItemUndo";
            this.mnuItemUndo.Size = new System.Drawing.Size(118, 22);
            this.mnuItemUndo.Text = "&Undo";
            // 
            // mnuExit
            // 
            this.mnuExit.Name = "mnuExit";
            this.mnuExit.Size = new System.Drawing.Size(118, 22);
            this.mnuExit.Text = "&Exit";
            // 
            // mnuEdit
            // 
            this.mnuEdit.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.mnuFieldChoice,
            this.mnuFieldErase,
            this.mnuFieldDefault,
            this.mnuFieldEraseAll,
            this.mnuFieldDefaultAll,
            this.mnuRefresh});
            this.mnuEdit.Name = "mnuEdit";
            this.mnuEdit.Size = new System.Drawing.Size(37, 20);
            this.mnuEdit.Text = "&Edit";
            // 
            // mnuFieldChoice
            // 
            this.mnuFieldChoice.Name = "mnuFieldChoice";
            this.mnuFieldChoice.Size = new System.Drawing.Size(185, 22);
            this.mnuFieldChoice.Text = "Field &Choices...";
            // 
            // mnuFieldErase
            // 
            this.mnuFieldErase.Name = "mnuFieldErase";
            this.mnuFieldErase.Size = new System.Drawing.Size(185, 22);
            this.mnuFieldErase.Text = "&Erase Current Field";
            // 
            // mnuFieldDefault
            // 
            this.mnuFieldDefault.Name = "mnuFieldDefault";
            this.mnuFieldDefault.Size = new System.Drawing.Size(185, 22);
            this.mnuFieldDefault.Text = "&Set Field to Default";
            // 
            // mnuFieldEraseAll
            // 
            this.mnuFieldEraseAll.Name = "mnuFieldEraseAll";
            this.mnuFieldEraseAll.Size = new System.Drawing.Size(185, 22);
            this.mnuFieldEraseAll.Text = "Erase &All Fields";
            // 
            // mnuFieldDefaultAll
            // 
            this.mnuFieldDefaultAll.Name = "mnuFieldDefaultAll";
            this.mnuFieldDefaultAll.Size = new System.Drawing.Size(185, 22);
            this.mnuFieldDefaultAll.Text = "Set All Fields to &Default";
            // 
            // mnuRefresh
            // 
            this.mnuRefresh.Name = "mnuRefresh";
            this.mnuRefresh.Size = new System.Drawing.Size(185, 22);
            this.mnuRefresh.Text = "&Refresh";
            // 
            // mnuSelection
            // 
            this.mnuSelection.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.mnuItemPrev,
            this.mnuItemNext,
            this.mnuItemFirst,
            this.mnuItemLast});
            this.mnuSelection.Name = "mnuSelection";
            this.mnuSelection.Size = new System.Drawing.Size(62, 20);
            this.mnuSelection.Text = "&Selection";
            // 
            // mnuItemPrev
            // 
            this.mnuItemPrev.Name = "mnuItemPrev";
            this.mnuItemPrev.Size = new System.Drawing.Size(152, 22);
            this.mnuItemPrev.Text = "&Previous";
            // 
            // mnuItemNext
            // 
            this.mnuItemNext.Name = "mnuItemNext";
            this.mnuItemNext.Size = new System.Drawing.Size(152, 22);
            this.mnuItemNext.Text = "&Next";
            // 
            // mnuItemFirst
            // 
            this.mnuItemFirst.Name = "mnuItemFirst";
            this.mnuItemFirst.Size = new System.Drawing.Size(152, 22);
            this.mnuItemFirst.Text = "&First";
            // 
            // mnuItemLast
            // 
            this.mnuItemLast.Name = "mnuItemLast";
            this.mnuItemLast.Size = new System.Drawing.Size(152, 22);
            this.mnuItemLast.Text = "&Last";
            // 
            // mnuHelp
            // 
            this.mnuHelp.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.mnuFieldHelp,
            this.mnuHelpContent,
            this.mnuHelpAbout});
            this.mnuHelp.Name = "mnuHelp";
            this.mnuHelp.Size = new System.Drawing.Size(40, 20);
            this.mnuHelp.Text = "&Help";
            // 
            // mnuFieldHelp
            // 
            this.mnuFieldHelp.Name = "mnuFieldHelp";
            this.mnuFieldHelp.Size = new System.Drawing.Size(125, 22);
            this.mnuFieldHelp.Text = "&Field Help";
            // 
            // mnuHelpContent
            // 
            this.mnuHelpContent.Name = "mnuHelpContent";
            this.mnuHelpContent.Size = new System.Drawing.Size(125, 22);
            this.mnuHelpContent.Text = "&Content...";
            // 
            // mnuHelpAbout
            // 
            this.mnuHelpAbout.Name = "mnuHelpAbout";
            this.mnuHelpAbout.Size = new System.Drawing.Size(125, 22);
            this.mnuHelpAbout.Text = "&About";
            // 
            // mainStatusBar
            // 
            this.mainStatusBar.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.statusBarPanel1});
            this.mainStatusBar.Location = new System.Drawing.Point(0, 257);
            this.mainStatusBar.Name = "mainStatusBar";
            this.mainStatusBar.Size = new System.Drawing.Size(336, 22);
            this.mainStatusBar.TabIndex = 2;
            // 
            // statusBarPanel1
            // 
            this.statusBarPanel1.Name = "statusBarPanel1";
            this.statusBarPanel1.Size = new System.Drawing.Size(0, 17);
            // 
            // toolBarRowsetTree
            // 
            this.toolBarRowsetTree.Location = new System.Drawing.Point(0, 24);
            this.toolBarRowsetTree.Name = "toolBarRowsetTree";
            this.toolBarRowsetTree.RootNodeHandler = null;
            this.toolBarRowsetTree.Size = new System.Drawing.Size(336, 25);
            this.toolBarRowsetTree.TabIndex = 15;
            this.toolBarRowsetTree.ToolBarButtonSize = RepPP.Framework.Window.ToolbarRowsetTree.ToolBarSize.Small;
            // 
            // pnlChildEditorContainer
            // 
            this.pnlChildEditorContainer.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.pnlChildEditorContainer.Location = new System.Drawing.Point(0, 52);
            this.pnlChildEditorContainer.Name = "pnlChildEditorContainer";
            this.pnlChildEditorContainer.Size = new System.Drawing.Size(336, 202);
            this.pnlChildEditorContainer.TabIndex = 17;
            // 
            // frmEditorStandaloneUI
            // 
            this.ClientSize = new System.Drawing.Size(336, 279);
            this.Controls.Add(this.pnlChildEditorContainer);
            this.Controls.Add(this.toolBarRowsetTree);
            this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
            this.Controls.Add(this.mainMenu1);
            this.Controls.Add(this.mainStatusBar);
            this.MainMenuStrip = this.mainMenu1;
            this.mainMenu1.ResumeLayout(false);
            this.mainMenu1.PerformLayout();
            this.mainStatusBar.ResumeLayout(false);
            this.mainStatusBar.PerformLayout();
            this.Name = "frmEditorStandaloneUI";
            this.Text = "frmSBEditor";
            this.ResumeLayout(false);
            this.PerformLayout();

        }
        #endregion
    }
}
