using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using RepPP;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
    /// <summary>
    /// Child form which is not displayed within the Main Editor.
    /// </summary>
    public class frmPopupChildEditor : RepPP.Framework.Window.frmRowsetTreeDefNodeHandler {
		/// <summary>Need by the component designer</summary>
		private System.ComponentModel.IContainer    components = null;
		/// <summary>Father editor</summary>
		private frmSBEditorBase                     m_fatherEditor;

        //*********************************************************     
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        //  
        //*********************************************************     
        public frmPopupChildEditor() {
            InitializeComponent();
        }

        //*********************************************************     
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="fatherEditor">         Father editor</param>
        /// <param name="nodeHandlerFather">    Father node handler</param>
        /// <param name="node">                 Rowset tree definition node for childeditor popup</param>
        /// <param name="bSeeDeletedLine">      true if see deleted line</param>
        //  
        //*********************************************************     
        protected frmPopupChildEditor(frmSBEditorBase           fatherEditor,
                                      RowsetTreeDefNodeHandler  nodeHandlerFather,
                                      RepPP.RowsetTreeDefNode   node,
                                      bool                      bSeeDeletedLine) : base(nodeHandlerFather, null, node, bSeeDeletedLine) {
            CommandHolder   = new CmdHolder();
            m_fatherEditor  = fatherEditor;
            m_fatherEditor.ConfigurationHelper.RegisterForm(this);
            InitializeComponent();
		}

        //*********************************************************     
        //
        /// <summary>
        /// Dispose:                            Dispose the form
        /// </summary>
        /// <param name="bDisposing">           true if dispose, false if GC</param>
        //  
        //*********************************************************     
		protected override void Dispose(bool bDisposing) {			
			if (bDisposing) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose(bDisposing);
		}

        //*********************************************************     
        //
        /// <summary>
        /// OnLoad:                             Called when the form is loaded
        /// </summary>
        /// <param name="e">                    Event argument</param>
        //  
        //*********************************************************     
        protected override void OnLoad(EventArgs e) {
            base.OnLoad (e);
            if (FatherEditor != null) {
                FatherEditor.ControlTextTranslator.TranslateControl(this);
                Refresh();
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// FatherEditor:                   Father editor
        /// </summary>
        //
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public frmSBEditorBase FatherEditor {
            get {
                return(m_fatherEditor);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Refresh:                            Refresh the list from the rowset
        /// </summary>
        //
        //*********************************************************		
        public override void Refresh() {
            if (FatherRowsetTreeDefNodeHandler != null) {
                Rowset = FatherRowsetTreeDefNodeHandler.Rowset.ChildRowsets[RowsetTreeDefNode.Name];
                OnLineSelected(new RowsetTreeDefNodeHandler.LineSelectedEventArgs(-1, CurrentLine));
            }
            base.Refresh();
        }

		#region Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion
	}
}

