using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using RepPP;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
    /// <summary>
    /// Class providing a default user interface for a Child form which is 
    /// not displayed within the Main Editor.
    /// </summary>
    public class frmPopupChildEditorUI : RepPP.Framework.Window.frmPopupChildEditor {
        /// <summary>Rep++ Application object</summary>
        protected RepPP.IApplication                            m_app;
        /// <summary>Main Menu</summary>
        protected MenuStrip                                     mainMenu1;
        /// <summary>File menu item</summary>
        protected ToolStripMenuItem                             mnuFile;
        /// <summary>New menu item</summary>
        protected ToolStripMenuItem                             mnuItemNew;
        /// <summary>Duplicate menu item</summary>
        protected ToolStripMenuItem                             mnuItemDup;
        /// <summary>Save menu item</summary>
        protected ToolStripMenuItem                             mnuItemSave;
        /// <summary>Delete menu item</summary>
        protected ToolStripMenuItem                             mnuItemDel;
        /// <summary>Undo menu item</summary>
        protected ToolStripMenuItem                             mnuItemUndo;
        /// <summary>Exit menu item</summary>
        protected ToolStripMenuItem                             mnuExit;
        /// <summary>Edit menu item</summary>
        protected ToolStripMenuItem                             mnuEdit;
        /// <summary>FieldDef Choice menu item</summary>
        protected ToolStripMenuItem                             mnuFieldChoice;
        /// <summary>FieldDef Erase menu item</summary>
        protected ToolStripMenuItem                             mnuFieldErase;
        /// <summary>FieldDef Default menu item</summary>
        protected ToolStripMenuItem                             mnuFieldDefault;
        /// <summary>FieldDef Erase All menu item</summary>
        protected ToolStripMenuItem                             mnuFieldEraseAll;
        /// <summary>FieldDef Default All menu item</summary>
        protected ToolStripMenuItem                             mnuFieldDefaultAll;
        /// <summary>Refresh menu item</summary>
        protected ToolStripMenuItem                             mnuRefresh;
        /// <summary>Selection menu item</summary>
        protected ToolStripMenuItem                             mnuSelection;
        /// <summary>Previous menu item</summary>
        protected ToolStripMenuItem                             mnuItemPrev;
        /// <summary>Next menu item</summary>
        protected ToolStripMenuItem                             mnuItemNext;
        /// <summary>First menu item</summary>
        protected ToolStripMenuItem                             mnuItemFirst;
        /// <summary>Last menu item</summary>
        protected ToolStripMenuItem                             mnuItemLast;
        /// <summary>Help menu item</summary>
        protected ToolStripMenuItem                             mnuHelp;
        /// <summary>FieldDef Help menu item</summary>
        protected ToolStripMenuItem                             mnuFieldHelp;
        /// <summary>Content menu item</summary>
        protected ToolStripMenuItem                             mnuHelpContent;
        /// <summary>About menu item</summary>
        protected ToolStripMenuItem                             mnuHelpAbout;
        /// <summary>StatusBar</summary>
        protected StatusStrip                                   statusStrip1;
        /// <summary>StatusBar Panel</summary>
        protected ToolStripStatusLabel                          statusBarPanel1;
        private System.ComponentModel.IContainer                components = null;
        /// <summary>Control displaying error icon on the side of a field</summary>
        private FldErrorProvider                                m_errorProvider;

        //*********************************************************     
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="fatherEditor">         Father editor</param>
        /// <param name="nodeHandlerFather">    Father node handler</param>
        /// <param name="node">                 Rowset tree definition node</param>
        /// <param name="bSeeDeletedLine">      true if see deleted line</param>
        //  
        //*********************************************************     
        protected frmPopupChildEditorUI(frmSBEditorBase          fatherEditor,
                                        RowsetTreeDefNodeHandler nodeHandlerFather,
                                        RepPP.RowsetTreeDefNode  node,
                                        bool                     bSeeDeletedLine) : base(fatherEditor, nodeHandlerFather, node, bSeeDeletedLine) {
            m_app           = fatherEditor.Application;
            m_errorProvider = new FldErrorProvider(ErrorIconAlignment.MiddleLeft, 2);
            InitializeComponent();
            RegisterEditorCmd();
            RootRowsetTreeDefNodeHandler.ResetFieldError += new EventHandler(this.OnResetFieldError);
   			m_app.FldOnError                             += new RepPP.FldOnErrorEventHandler(this.OnFldOnError);
        }

        //*********************************************************     
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        //  
        //*********************************************************     
        public frmPopupChildEditorUI() {
			InitializeComponent();
		}

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the form
        /// </summary>
        /// <param name="bDisposing">   true if disposing, false if finalizing</param>
        //
        //******************************************************************
        protected override void Dispose(bool bDisposing) {
            if (bDisposing) {
                if (RootRowsetTreeDefNodeHandler != null) {
                    RootRowsetTreeDefNodeHandler.ResetFieldError -= new EventHandler(this.OnResetFieldError);
                }
		        if (m_app != null) {
			        m_app.FldOnError -= new RepPP.FldOnErrorEventHandler(this.OnFldOnError);
			    }
                UnregisterEditorCmd();
                if (components != null) {
                    components.Dispose();
                }
            }
            base.Dispose(bDisposing);
        }

        //******************************************************************
        //
        /// <summary>
        /// ErrorProvider:          The error provider object responsible to display
        ///                         the error icon in the side of the field controls
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public FldErrorProvider ErrorProvider {
            get {
                return(m_errorProvider);
            }
            set {
                m_errorProvider = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// RegisterEditorCmd:              Register the editor command
        /// </summary>
        //
        //******************************************************************
        protected virtual void RegisterEditorCmd() {
            CmdHolder   cmdHolder;
            CmdHolder   cmdHolderFather;
            
            cmdHolder = CommandHolder;
            cmdHolder.AddCmd("Exit",            new CmdHolder.OnCommand(CmdExit),             CmdHolder.CmdStateE.Enabled);
            cmdHolder.AddCmd("FormRefresh",     new CmdHolder.OnCommand(FormRefresh),         CmdHolder.CmdStateE.Enabled);

            cmdHolder.AddTrigger(null, "Exit",              new object[] {mnuExit} );
            cmdHolder.AddTrigger(null, "FormRefresh",       new object[] {mnuRefresh} );
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemFirst.ToString(), new object[] { mnuItemFirst });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemPrev.ToString(), new Object[] { mnuItemPrev });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemNext.ToString(), new Object[] { mnuItemNext });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemLast.ToString(), new Object[] { mnuItemLast });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemNew.ToString(), new Object[] { mnuItemNew });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemDup.ToString(), new Object[] { mnuItemDup });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemDel.ToString(), new Object[] { mnuItemDel });

            cmdHolderFather = FatherEditor.CommandHolder;
            cmdHolderFather.AddTrigger(null, sdHandlerCmd.FieldChoice.ToString(),       new object[] { mnuFieldChoice });
            cmdHolderFather.AddTrigger(null, sdHandlerCmd.FieldHelp.ToString(),         new object[] { mnuFieldHelp });
            cmdHolderFather.AddTrigger(null, sdHandlerCmd.FieldErase.ToString(),        new object[] { mnuFieldErase });
            cmdHolderFather.AddTrigger(null, sdHandlerCmd.FieldDefault.ToString(),      new object[] { mnuFieldDefault });
            cmdHolderFather.AddTrigger(null, sdHandlerCmd.FieldEraseAll.ToString(),     new object[] { mnuFieldEraseAll });
            cmdHolderFather.AddTrigger(null, sdHandlerCmd.FieldDefaultAll.ToString(),   new object[] { mnuFieldDefaultAll });
        }

        //******************************************************************
        //
        /// <summary>
        /// UnregisterEditorCmd:              Unreegister the editor command
        /// </summary>
        //
        //******************************************************************
        protected virtual void UnregisterEditorCmd() {
            CmdHolder   cmdHolder;
            CmdHolder   cmdHolderFather;
            
            cmdHolder = CommandHolder;
            if (cmdHolder != null) {
                cmdHolder.RemoveCmd("Exit");
                cmdHolder.RemoveCmd("FormRefresh");

                cmdHolderFather = FatherEditor.CommandHolder;
                cmdHolderFather.RemoveTrigger(sdHandlerCmd.FieldChoice.ToString(),      new object[] { mnuFieldChoice });
                cmdHolderFather.RemoveTrigger(sdHandlerCmd.FieldHelp.ToString(),        new object[] { mnuFieldHelp });
                cmdHolderFather.RemoveTrigger(sdHandlerCmd.FieldErase.ToString(),       new object[] { mnuFieldErase });
                cmdHolderFather.RemoveTrigger(sdHandlerCmd.FieldDefault.ToString(),     new object[] { mnuFieldDefault });
                cmdHolderFather.RemoveTrigger(sdHandlerCmd.FieldEraseAll.ToString(),    new object[] { mnuFieldEraseAll });
                cmdHolderFather.RemoveTrigger(sdHandlerCmd.FieldDefaultAll.ToString(),  new object[] { mnuFieldDefaultAll });
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// CmdExit:                    Exit the program
        /// </summary>
        //  
        //*********************************************************
        protected virtual void CmdExit() {
            Close();
        }

        //*********************************************************
        //
        /// <summary>
        /// FormRefresh:                Refresh the form
        /// </summary>
        /// <param name="bForceClose">  True to force close. Don't ask for saving</param>
        //  
        //*********************************************************
        public virtual void FormRefresh(bool bForceClose) {
            
        }

        //*********************************************************
        //
        /// <summary>
        /// FormRefresh:                Refresh the form
        /// </summary>
        //  
        //*********************************************************
        public virtual void FormRefresh() {
            FormRefresh(false);
        }

        //******************************************************************
        //
        /// <summary>
        /// OnResetFieldError:      Handle the framework reset field error event
        /// </summary>
        /// <param name="sender">   Event sender</param>
        /// <param name="e">        Event arguments</param>
        // 
        //******************************************************************
        protected virtual void OnResetFieldError(Object sender, EventArgs e) {
            FldErrorProvider    errorProvider;
            
            errorProvider = ErrorProvider;
            statusBarPanel1.Text = "";
            errorProvider.Reset();
        }

		//******************************************************************
		//
		/// <summary>
		/// OnFldOnError:          Handle the rep++ field error event
		/// </summary>
		/// <param name="sender">  Event sender</param>
		/// <param name="e">       Event arguments</param>
		// 
		//******************************************************************
		protected virtual void OnFldOnError(Object sender, RepPP.FieldGenEventArgs e) {
			statusBarPanel1.Text = e.FieldDef.Prompt + ": " + e.ErrorText;
		}

		#region Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.mainMenu1 = new System.Windows.Forms.MenuStrip();
            this.mnuFile = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemNew = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemDup = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemSave = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemDel = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemUndo = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuExit = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuEdit = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuFieldChoice = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuFieldErase = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuFieldDefault = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuFieldEraseAll = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuFieldDefaultAll = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuRefresh = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuSelection = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemPrev = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemNext = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemFirst = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemLast = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuHelp = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuFieldHelp = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuHelpContent = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuHelpAbout = new System.Windows.Forms.ToolStripMenuItem();
            this.statusStrip1 = new System.Windows.Forms.StatusStrip();
            this.statusBarPanel1 = new System.Windows.Forms.ToolStripStatusLabel();
            this.mainMenu1.SuspendLayout();
            this.statusStrip1.SuspendLayout();
            this.SuspendLayout();
            // 
            // mainMenu1
            // 
            this.mainMenu1.BackColor = System.Drawing.SystemColors.Control;
            this.mainMenu1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.mnuFile,
            this.mnuEdit,
            this.mnuSelection,
            this.mnuHelp});
            this.mainMenu1.Location = new System.Drawing.Point(0, 0);
            this.mainMenu1.Name = "mainMenu1";
            this.mainMenu1.Size = new System.Drawing.Size(856, 24);
            this.mainMenu1.TabIndex = 16;
            this.mainMenu1.Text = "mainMenu1";
            // 
            // mnuFile1ToolStripMenuItem
            // 
            this.mnuFile.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.mnuItemNew,
            this.mnuItemDup,
            this.mnuItemSave,
            this.mnuItemDel,
            this.mnuItemUndo,
            this.mnuExit});
            this.mnuFile.Name = "mnuFile";
            this.mnuFile.Size = new System.Drawing.Size(35, 20);
            this.mnuFile.Text = "&File";
            // 
            // mnuItemNew
            // 
            this.mnuItemNew.Name = "mnuItemNew";
            this.mnuItemNew.Size = new System.Drawing.Size(152, 22);
            this.mnuItemNew.Text = "&New";
            // 
            // mnuItemDup
            // 
            this.mnuItemDup.Name = "mnuItemDup";
            this.mnuItemDup.Size = new System.Drawing.Size(152, 22);
            this.mnuItemDup.Text = "D&uplicate";
            // 
            // mnuItemSave
            // 
            this.mnuItemSave.Name = "mnuItemSave";
            this.mnuItemSave.Size = new System.Drawing.Size(152, 22);
            this.mnuItemSave.Text = "&Save";
            // 
            // mnuItemDel
            // 
            this.mnuItemDel.Name = "mnuItemDel";
            this.mnuItemDel.Size = new System.Drawing.Size(152, 22);
            this.mnuItemDel.Text = "&Delete";
            // 
            // mnuItemUndo
            // 
            this.mnuItemUndo.Name = "mnuItemUndo";
            this.mnuItemUndo.Size = new System.Drawing.Size(152, 22);
            this.mnuItemUndo.Text = "&Undo";
            // 
            // mnuExit
            // 
            this.mnuExit.Name = "mnuExit";
            this.mnuExit.Size = new System.Drawing.Size(152, 22);
            this.mnuExit.Text = "&Exit";
            // 
            // mnuEdit
            // 
            this.mnuEdit.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.mnuFieldChoice,
            this.mnuFieldErase,
            this.mnuFieldDefault,
            this.mnuFieldEraseAll,
            this.mnuFieldDefaultAll,
            this.mnuRefresh});
            this.mnuEdit.Name = "mnuEdit";
            this.mnuEdit.Size = new System.Drawing.Size(37, 20);
            this.mnuEdit.Text = "&Edit";
            // 
            // mnuFieldChoice
            // 
            this.mnuFieldChoice.Name = "mnuFieldChoice";
            this.mnuFieldChoice.Size = new System.Drawing.Size(196, 22);
            this.mnuFieldChoice.Text = "Field &Choices...";
            // 
            // mnuFieldErase
            // 
            this.mnuFieldErase.Name = "mnuFieldErase";
            this.mnuFieldErase.Size = new System.Drawing.Size(196, 22);
            this.mnuFieldErase.Text = "&Erase Current Field";
            // 
            // mnuFieldDefault
            // 
            this.mnuFieldDefault.Name = "mnuFieldDefault";
            this.mnuFieldDefault.Size = new System.Drawing.Size(196, 22);
            this.mnuFieldDefault.Text = "&Set Field to Default";
            // 
            // mnuFieldEraseAll
            // 
            this.mnuFieldEraseAll.Name = "mnuFieldEraseAll";
            this.mnuFieldEraseAll.Size = new System.Drawing.Size(196, 22);
            this.mnuFieldEraseAll.Text = "Erase &All Fields";
            // 
            // mnuFieldDefaultAll
            // 
            this.mnuFieldDefaultAll.Name = "mnuFieldDefaultAll";
            this.mnuFieldDefaultAll.Size = new System.Drawing.Size(196, 22);
            this.mnuFieldDefaultAll.Text = "Set All Fields to &Default";
            // 
            // mnuRefresh
            // 
            this.mnuRefresh.Name = "mnuRefresh";
            this.mnuRefresh.Size = new System.Drawing.Size(196, 22);
            this.mnuRefresh.Text = "&Refresh";
            // 
            // mnuSelection
            // 
            this.mnuSelection.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.mnuItemPrev,
            this.mnuItemNext,
            this.mnuItemFirst,
            this.mnuItemLast});
            this.mnuSelection.Name = "mnuSelection";
            this.mnuSelection.Size = new System.Drawing.Size(62, 20);
            this.mnuSelection.Text = "&Selection";
            // 
            // mnuItemPrev
            // 
            this.mnuItemPrev.Name = "mnuItemPrev";
            this.mnuItemPrev.Size = new System.Drawing.Size(152, 22);
            this.mnuItemPrev.Text = "&Previous";
            // 
            // mnuItemNext
            // 
            this.mnuItemNext.Name = "mnuItemNext";
            this.mnuItemNext.Size = new System.Drawing.Size(152, 22);
            this.mnuItemNext.Text = "&Next";
            // 
            // mnuItemFirst
            // 
            this.mnuItemFirst.Name = "mnuItemFirst";
            this.mnuItemFirst.Size = new System.Drawing.Size(152, 22);
            this.mnuItemFirst.Text = "&First";
            // 
            // mnuItemLast
            // 
            this.mnuItemLast.Name = "mnuItemLast";
            this.mnuItemLast.Size = new System.Drawing.Size(152, 22);
            this.mnuItemLast.Text = "&Last";
            // 
            // mnuHelp
            // 
            this.mnuHelp.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.mnuFieldHelp,
            this.mnuHelpContent,
            this.mnuHelpAbout});
            this.mnuHelp.Name = "mnuHelp";
            this.mnuHelp.Size = new System.Drawing.Size(40, 20);
            this.mnuHelp.Text = "&Help";
            // 
            // mnuFieldHelp
            // 
            this.mnuFieldHelp.Name = "mnuFieldHelp";
            this.mnuFieldHelp.Size = new System.Drawing.Size(152, 22);
            this.mnuFieldHelp.Text = "&Field Help";
            // 
            // mnuHelpContent
            // 
            this.mnuHelpContent.Name = "mnuHelpContent";
            this.mnuHelpContent.Size = new System.Drawing.Size(152, 22);
            this.mnuHelpContent.Text = "&Content...";
            // 
            // mnuHelpAbout
            // 
            this.mnuHelpAbout.Name = "mnuHelpAbout";
            this.mnuHelpAbout.Size = new System.Drawing.Size(152, 22);
            this.mnuHelpAbout.Text = "&About";
            // 
            // statusStrip1
            // 
            this.statusStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.statusBarPanel1});
            this.statusStrip1.Location = new System.Drawing.Point(0, 244);
            this.statusStrip1.Name = "statusStrip1";
            this.statusStrip1.Size = new System.Drawing.Size(292, 22);
            this.statusStrip1.TabIndex = 2;
            this.statusStrip1.Text = "";
            // 
            // statusBarPanel1
            // 
            this.statusBarPanel1.Name = "statusBarPanel1";
            this.statusBarPanel1.Size = new System.Drawing.Size(109, 17);
            this.statusBarPanel1.Text = "";
            // 
            // frmPopupChildEditorUI
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(856, 542);
            this.Controls.Add(this.mainMenu1);
            this.Controls.Add(this.statusStrip1);
            this.MainMenuStrip = this.mainMenu1;
            this.mainMenu1.ResumeLayout(false);
            this.mainMenu1.PerformLayout();
            this.statusStrip1.ResumeLayout(false);
            this.statusStrip1.PerformLayout();
            this.Name = "frmPopupChildEditorUI";
            this.ResumeLayout(false);
        }
		#endregion
	}
}

