//
// Rep++ Framework for Windows Application
// Copyright (C) 2005 Consyst SQL Inc.
// All Rights Reserved
//
// This source code is only intended as a supplement to the
// Rep++ Studio products.
//
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using RepPP;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
	/// <summary>
	/// Child form which is include in a selection buffer top editor
	/// </summary>
	public class frmSBChildEditor : frmRowsetTreeDefNodeHandlerRootForSB, IAskForPrimaryKey {
		/// <summary>Required designer variable.</summary>
		private System.ComponentModel.Container                     components = null;
	    /// <summary>Editor type</summary>
	    private int                                                 m_iType;
        /// <summary>Selection buffer interface</summary>
        private ISelectionBuffer                                    m_selBuf;
        /// <summary>Rowset tree created for the transaction</summary>
        private RowsetTree                                          m_rowsetTree;
        /// <summary>Father editor</summary>
        private frmSBEditorBase                                     m_fatherEditor;

        //*********************************************************     
        //
        /// <summary>
        /// ctor:                               Class constructor. Use in design mode
        /// </summary>
        //  
        //*********************************************************     
		public frmSBChildEditor() {
			InitializeComponent();
		}

        //*********************************************************     
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="fatherEditor">         Father editor</param>
        /// <param name="strRowsetTreeName">    Name of the rowset tree attached to the child editor</param>
        /// <param name="iType">                Unique type of this editor</param>
        //  
        //*********************************************************     
        public frmSBChildEditor(frmSBEditorBase fatherEditor,
                                string          strRowsetTreeName,
                                int             iType) : this() {
            RepPP.Application   app;
            RowsetTreeDef                rowsetTreeDef;
            
            app             = fatherEditor.Application;
            m_fatherEditor  = fatherEditor;
            rowsetTreeDef   = app.RowsetTreeDefs[strRowsetTreeName];
            InitDel(app,
                    m_fatherEditor.SelectionBuffer,
                    rowsetTreeDef,
                    this);
            SetTopLevel(false);
            m_iType         = iType;
            m_selBuf        = m_fatherEditor.SelectionBuffer;
            m_rowsetTree    = rowsetTreeDef.RowsetTrees.Add();
            CommandHolder   = null;
            AutoCommit      = true;
            RowsetTree    = m_rowsetTree;                                                               
            fatherEditor.ConfigurationHelper.RegisterForm(this);
        }

        //*********************************************************     
        //
        /// <summary>
        /// Dispose:                            Dispose the form
        /// </summary>
        /// <param name="bDisposing">           true if dispose, false if GC</param>
        //  
        //*********************************************************     
		protected override void Dispose(bool bDisposing) {
			if (bDisposing) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose(bDisposing);
			if (bDisposing) {
				if (m_rowsetTree != null) {
				    m_rowsetTree.Delete();
				    m_rowsetTree = null;
				}
			}
		}

        //*********************************************************     
        //
        /// <summary>
        /// OnLoad:                             Called when the form is loaded
        /// </summary>
        /// <param name="e">                    Event argument</param>
        //  
        //*********************************************************     
        protected override void OnLoad(EventArgs e) {
            base.OnLoad (e);
            if (FatherEditor != null) {
                FatherEditor.ControlTextTranslator.TranslateControl(this);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// FatherEditor:                   Father editor
        /// </summary>
        //
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public frmSBEditorBase FatherEditor {
            get {
                return(m_fatherEditor);
            }
        }

        //*********************************************************     
        //
		/// <summary>
		/// AttachChildEditor:              Attach this child editor to the father editor
		/// </summary>
		/// <param name="ctlFather">        Container control</param>
		//
        //*********************************************************     
		public virtual void AttachChildEditor(Control ctlFather) {
		    ctlFather.Controls.Add(this);
		    Dock            = DockStyle.Fill;
		    CommandHolder   = FatherEditor.CommandHolder;
		    Show();
		}

        //*********************************************************     
        //
		/// <summary>
		/// InitChildEditor:                Initialize the child editor
		/// </summary>
		//
        //*********************************************************     
		public virtual void InitChildEditor() {
		}

        //*********************************************************     
        //
        /// <summary>
        /// DetachChildEditor:              Detach this child editor to the father editor
        /// </summary>
        //
        //*********************************************************     
        public virtual void DetachChildEditor() {
            Hide();
            CommandHolder   = null;
        }

        //*********************************************************     
        //
        /// <summary>
        /// SelectionBufferRowset:   RowsetDef instance of the selection buffer
        /// </summary>
        //  
        //*********************************************************     
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public RepPP.Rowset SelectionBufferRowset {
            get {
                return((m_selBuf == null) ? null : m_selBuf.CurrentRowset);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// EditorType:                 Editor type
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public int EditorType {
            get {
                return(m_iType);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// AskForPrimaryKey:       Ask for the primary key
        /// </summary>
        /// <param name="rowsetSB">  RowsetDef instance where to set the key</param>
        /// <returns>
        /// false:                  Failed
        /// true:                   Succeed
        /// </returns>
        //
        //******************************************************************
        public virtual bool AskForPrimaryKey(RepPP.Rowset rowsetSB) {
            return(true);
        }

        //******************************************************************
        //
        /// <summary>
        /// AskForPrimaryKey:       Ask for the primary key
        /// </summary>
        /// <param name="arrFldDef">Array of field definition containing default value</param>
        /// <param name="arrNewPK"> Array of values for the fields</param>
        /// <returns>
        /// false:                  Failed
        /// true:                   Succeed
        /// </returns>
        //
        //******************************************************************
        public virtual bool AskForPrimaryKey(out FieldDef[] arrFldDef, out string[] arrNewPK) {
            bool            bRetVal;
            Rowset          rowsetSB;
            ArrayList       arrListFld;
            ArrayList       arrListValues;
            RepPP.FieldDefs fldsSB;
            RepPP.FieldDefs fldsEditor;
            RepPP.FieldDef  fldEditor;
            
            arrFldDef   = null;
            arrNewPK    = null;
            rowsetSB    = SelectionBufferRowset;
            bRetVal     = AskForPrimaryKey(rowsetSB);
            if (bRetVal) {
                arrListFld      = new ArrayList();
                arrListValues   = new ArrayList();
                fldsSB          = rowsetSB.RowsetDef.FieldDefs;
                fldsEditor      = RowsetTreeDefNode.RowsetDef.FieldDefs;
                foreach (RepPP.FieldDef fldDef in fldsSB) {
                    fldEditor = fldsEditor[fldDef.Name];
                    if (fldEditor != null) {
                        arrListFld.Add(fldEditor);
                        arrListValues.Add(fldDef.Value);
                    }
                }
                arrFldDef = new FieldDef[arrListFld.Count];
                arrListFld.CopyTo(arrFldDef);
                arrNewPK = new string[arrListValues.Count];
                arrListValues.CopyTo(arrNewPK);
            }
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// ProcessTabKey:                      Process the TAB key. Override to permit navigation
        ///                                     out of the contained form.
        /// </summary>
        /// <param name="bFoward">              true if foward, false if reverse</param>
        /// <returns>
        /// true if a new control has been selected, false if not.
        /// </returns>
        //  
        //*********************************************************     
        protected override bool ProcessTabKey(bool bFoward) {
            bool    bRetVal;

            bRetVal = SelectNextControl(ActiveControl, bFoward, true, true, false);
            if (!bRetVal) {
                bRetVal = TopLevelControl.SelectNextControl(this, bFoward, true, true, true);
            }
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// BuildWhereClause:               Build the WHERE clause for the PK
        /// </summary>
        /// <param name="rowset">            RowsetDef instance containing the values</param>
        /// <returns>
        /// WHERE clause
        /// </returns>
        //
        //******************************************************************
        public string BuildWhereClause(RepPP.Rowset rowset) {
            string  strRetVal;
	        
            strRetVal = "";
            if (rowset.FatherRowset != null) {
                strRetVal = BuildWhereClause(rowset.FatherRowset);
            }
            foreach (RepPP.FieldDef fldDef in rowset.RowsetDef.FieldDefs) {
                if (fldDef.IsPrimaryKey) {
                    if (strRetVal == String.Empty) {
                        strRetVal = " WHERE ";
                    } else {
                        strRetVal += " AND ";
                    }
                    strRetVal += fldDef.ColName + "=:" + fldDef.RowsetDef.Name + "." + fldDef.Name;
                }
            }
            return(strRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// CheckIfDataChangedSinceReadB:   Checks if the data has changed in the DB since it has been read
        /// </summary>
        /// <param name="rowset">           Rowset containing the values</param>
        /// <param name="fldDefStamp">      Field definition containing the date stamp</param>
        /// <param name="bDisplayMsg">      Flag to display a message box</param>
        /// <returns>
        /// true if changed, false if not
        /// </returns>
        //
        //******************************************************************
        public virtual bool CheckIfDataChangedSinceReadB(RepPP.Rowset rowset, RepPP.FieldDef fldDefStamp, bool bDisplayMsg) {
            bool                bRetVal;
            RepPP.Connection    conn;
            string              strSqlCmd;
            string              strStamping;
            string              strMsg;
            RepPP.SqlCursor     sqlCursor;
	        
            if (rowset.RowsetTreeDefNode != null) {
                conn = rowset.RowsetTreeDefNode.Connection;
            } else {
                conn = rowset.Application.DataConnection;
            }
            strSqlCmd = "SELECT " + fldDefStamp.ColName + " FROM " + fldDefStamp.TableName + BuildWhereClause(rowset) + " FOR UPDATE";
            if (conn.Execute(strSqlCmd, RepPP.SqlOptimization.sdOptCursor, out sqlCursor) == 0) {
                using(sqlCursor) {
                    if (sqlCursor.Next() == 0) {
                        strStamping = sqlCursor.GetColumnValue(0).Replace('-','/');
                        bRetVal     = (strStamping != fldDefStamp.Value);
                    } else {
                        bRetVal     = true;
                    }
                }
            } else {
                throw new System.ApplicationException(conn.ErrorMessage);
            }
            if (bRetVal && bDisplayMsg) {
                if (rowset.Application.IsAtomExist("$dbdatachg", RepPP.AtomType.sdAtomErr)) {
                    strMsg = rowset.Application.GetAtomText("$dbdatachg", RepPP.AtomType.sdAtomErr);
                } else {
                    strMsg = "The values in the database were modified by another user since they were fetched. The update cannot be done. Please fetch the data again and reapply your modifications.";
                }
                MessageBox.Show(strMsg, "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return(bRetVal);
        }

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            // 
            // frmSBChildEditor
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(928, 616);
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
            this.Name = "frmSBChildEditor";
            this.Text = "frmSBChildEditor";

        }
		#endregion
	}
}
