using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
	/// <summary>
	/// Summary description for frmSBDirectSearch.
	/// </summary>
    public class frmSBDirectSearch : frmSerializabledForm {
        /// <summary>OK Button</summary>
        protected System.Windows.Forms.Button           butFilterOk;
        /// <summary>Cancel Button</summary>
        protected System.Windows.Forms.Button           butFilterCancel;
        /// <summary>And</summary>
        protected System.Windows.Forms.RadioButton      radioAnd;
        /// <summary>Or</summary>
        protected System.Windows.Forms.RadioButton      radioOr;
        /// <summary>All fields collapsible GroupBox</summary>
        private CollapsibleGroupBox                     gbOtherFields;
        /// <summary>Current search FieldDef GroupBox</summary>
        private GroupBox                                gbSearchField;
        /// <summary>All FieldDefs container</summary>
        private Panel                                   panelOtherFields;
        /// <summary>Search FieldDef container</summary>
        private Panel                                   panelSearchField;
		/// <summary>Required designer variable.</summary>
		private System.ComponentModel.Container         components = null;
		/// <summary>Filter object. This is the object doing the real job</summary>
        protected SBFilter                              m_filter = null;
		/// <summary>The direct search current FieldDef</summary>
        protected FieldDef                              m_fldDefCurrent = null;
		/// <summary>The direct search current FieldDef value</summary>
        protected string                                m_strCurrentValue = null;
		/// <summary>The direct search current RowsetTreeDefNode</summary>
        protected RowsetTreeDefNode                     m_nodeCurrent = null;
		/// <summary>true to Auto Size the form so the field are all visible if possible</summary>
        protected bool                                  m_bAutoSizeForm = true;
        /// <summary>Error provider for the form</summary>
        protected FldErrorProvider                      m_errorProvider;
        /// <summary>true if filter must be added, false if it replace it</summary>
        protected bool                                  m_bAddFilter;
        /// <summary>The where clause that will be used to execute the search</summary>
        protected string                                m_strWhereClause = string.Empty;
        /// <summary>The Join clause that will be used to execute the search</summary>
        protected string                                m_strJoinClause = string.Empty;

        //******************************************************************
	    //
		/// <summary>
		/// ctor:                       Form default constructor. Use by designer
		/// </summary>
	    //
	    //******************************************************************
		public frmSBDirectSearch() {
			InitializeComponent();
			m_filter        = null;
            m_fldDefCurrent = null;
            m_nodeCurrent   = null;
		}

	    //******************************************************************
	    //
		/// <summary>
		/// ctor:                           Form constructor.
		/// </summary>
        /// <param name="nodeCurrent">      Current RowsetTreeDefNode</param>
        /// <param name="fldDefCurrent">    Search Field Definition</param>
        /// <param name="strCurrentValue">  Search value</param>
        /// <param name="ctlTranslator">    Control text translator</param>
        /// <param name="configHelper">     ui configuration helper</param>
        /// <param name="bAddFilter">       true to add the filter, false to replace</param>
	    //
	    //******************************************************************
        public frmSBDirectSearch(RowsetTreeDefNode nodeCurrent, FieldDef fldDefCurrent, string strCurrentValue, ControlTextTranslator ctlTranslator, UIConfigurationHelper configHelper, bool bAddFilter) : this() {
            RepPP.Rowset rowset;

            m_configHelper    = configHelper;
            m_filter          = new SBFilter(fldDefCurrent.RowsetDef, true);
            m_fldDefCurrent   = fldDefCurrent;
            m_strCurrentValue = strCurrentValue;
            m_nodeCurrent     = nodeCurrent;
            m_bAddFilter      = bAddFilter;
            rowset            = m_filter.ParamRowset;
            rowset.RowsetDef.SaveActiveRowset();
            rowset.Select();
            radioAnd.Checked  = true;
            m_errorProvider   = new FldErrorProvider(ErrorIconAlignment.MiddleLeft, 2);
            if (ctlTranslator != null) {
                ctlTranslator.TranslateControl(this);
            }
            rowset.Application.FldOnError += new RepPP.FldOnErrorEventHandler(Application_FldOnError);
        }

	    //******************************************************************
	    //
		/// <summary>
		/// Show:                           Displays the direct search form
		/// </summary>
        /// <param name="nodeCurrent">      Current RowsetTreeDefNode</param>
        /// <param name="fldDefCurrent">    Search Field Definition</param>
        /// <param name="strCurrentValue">  Search value</param>
        /// <param name="ctlTranslator">    Control text translator</param>
        /// <param name="strJoinClause">    Inner Join Clause</param>
        /// <param name="strWhereClause">   Where Clause</param>
        /// <returns>
        /// True if the form was displayed and the user hit OK, false otherwise.
        /// </returns>
	    //
	    //******************************************************************
        public static bool Show(RowsetTreeDefNode       nodeCurrent, 
                                FieldDef                fldDefCurrent, 
                                string                  strCurrentValue, 
                                ControlTextTranslator   ctlTranslator, 
                                out string              strJoinClause, 
                                out string              strWhereClause) {
            bool              bRetVal = false;
            frmSBDirectSearch frmDirectSearch;

            using(frmDirectSearch = new frmSBDirectSearch(nodeCurrent, fldDefCurrent, strCurrentValue, ctlTranslator, null, false)) {
                if (frmDirectSearch.ShowDialog() == DialogResult.OK) {
                    strJoinClause  = frmDirectSearch.JoinClause;
                    strWhereClause = frmDirectSearch.WhereClause;
                    bRetVal = true;
                } else {
                    strJoinClause  = string.Empty;
                    strWhereClause = string.Empty;
                }
            }
            return(bRetVal);
        }

	    //******************************************************************
	    //
		/// <summary>
		/// Dispose:                    Clean up any resources being used.
		/// </summary>
		/// <param name="bDisposing">   true if dispose, false if GC</param>
	    //
	    //******************************************************************
		protected override void Dispose(bool bDisposing) {
			if (bDisposing)	{
			    if (m_filter != null) {
			        m_filter.DisposeFormResource();
                    m_filter.ParamRowset.Application.FldOnError -= new RepPP.FldOnErrorEventHandler(Application_FldOnError);
                    m_filter.ParamRowset.RowsetDef.RestoreActiveRowset(false);
    		        m_filter.Dispose();
                    m_filter = null;
			    }
				if (components != null)	{
					components.Dispose();
				}
			}
			base.Dispose(bDisposing);
		}

	    //******************************************************************
	    //
		/// <summary>
		/// ResizeForm:                 Resize the form to show as much field as possible
		/// </summary>
		//
	    //******************************************************************
        protected virtual void ResizeForm() {
            int iOldHeight;
            int iDelta;

            if (m_filter != null) {
                iOldHeight = Height;
                m_filter.ResizeForm(this, panelOtherFields);
                iDelta                   = Height - iOldHeight;
                panelOtherFields.Height += iDelta ;
                gbOtherFields.Height    += iDelta ;
            }
        }

	    //******************************************************************
	    //
		/// <summary>
		/// OnCollapseBoxClicked:           Handles the Expand/Collapse event
        ///                                 to adjust the form size
        /// <param name="sender">           The group box</param>
		/// </summary>
		//
	    //******************************************************************
        private void OnCollapseBoxClicked(object sender) {
            Height = gbSearchField.Height + gbOtherFields.Height + butFilterOk.Height + 70;
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnLoad:                         Load the form state when form is loading
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        protected override void OnLoad(EventArgs e) {
            if (m_filter != null && m_fldDefCurrent != null) {
                m_filter.FillPanel(panelSearchField,
                                   panelOtherFields,
                                   m_fldDefCurrent,
                                   m_strCurrentValue,
                                   this.Font,
                                   40,
                                   40);
            }
            base.OnLoad(e);
            if (AutoSizeForm) {
                ResizeForm();
            }
            gbOtherFields.ToggleCollapsed();
        }

	    //******************************************************************
	    //
        /// <summary>
        /// butOK_Click:                Called when the OK button is clicked
        /// </summary>
        /// <param name="sender">       Sender object</param>
        /// <param name="e">            Event argument</param>
        //
	    //******************************************************************
        private void butOK_Click(object sender, System.EventArgs e) {
            m_errorProvider.Reset();
            if (m_filter.IsValid()) {
                if (m_bAddFilter) {
                    m_filter.AddFilter((radioAnd.Checked) ? SBFilter.AndOrE.AND : SBFilter.AndOrE.OR);
                } else {
                    m_filter.SetFilter((radioAnd.Checked) ? SBFilter.AndOrE.AND : SBFilter.AndOrE.OR);
                }
                SetSBWhereClause();
                DialogResult = DialogResult.OK;
                Close();
            }
        }

	    //******************************************************************
	    //
        /// <summary>
        /// SetWhereClause:             Sets the Where clause based on the 
        ///                             value of the direct search filter.
        /// </summary>
        //
	    //******************************************************************
        private void SetSBWhereClause() {
            WhereClause = m_filter.WhereClause;
            if (WhereClause != string.Empty) {
                JoinClause = m_filter.BuildDSInnerClause(m_nodeCurrent);
            } else {
                JoinClause = string.Empty;
            }
        }

	    //******************************************************************
	    //
        /// <summary>
        /// WhereClause:                The where clause
        /// </summary>
        //
	    //******************************************************************
        public virtual string WhereClause {
            get {
                return(m_strWhereClause);
            }
            set {
                m_strWhereClause = value;
            }
        }

	    //******************************************************************
	    //
        /// <summary>
        /// JoinClause:                The join clause
        /// </summary>
        //
	    //******************************************************************
        public virtual string JoinClause {
            get {
                return(m_strJoinClause);
            }
            set {
                m_strJoinClause = value;
            }
        }

	    //******************************************************************
	    //
        /// <summary>
        /// AutoSizeForm:                   true to auto size the form to fit as much field as possible
        /// </summary>
        //
	    //******************************************************************
        public virtual bool AutoSizeForm {
            get {
                return(m_bAutoSizeForm);
            }
            set {
                m_bAutoSizeForm = value;
            }
        }

	    //******************************************************************
	    //
        /// <summary>
        /// Application_FldOnError:     Called when a field is in error
        /// </summary>
        /// <param name="sender">       Sender object</param>
        /// <param name="e">            Event argument</param>
        //
	    //******************************************************************
        protected virtual void Application_FldOnError(object sender, RepPP.FieldGenEventArgs e) {
            m_errorProvider.AddError(this, e.FieldDef, e.ErrorText);
        }

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.butFilterOk = new System.Windows.Forms.Button();
            this.butFilterCancel = new System.Windows.Forms.Button();
            this.radioAnd = new System.Windows.Forms.RadioButton();
            this.radioOr = new System.Windows.Forms.RadioButton();
            this.gbOtherFields = new RepPP.Toolkit.Window.CollapsibleGroupBox();
            this.panelOtherFields = new System.Windows.Forms.Panel();
            this.gbSearchField = new System.Windows.Forms.GroupBox();
            this.panelSearchField = new System.Windows.Forms.Panel();
            this.gbOtherFields.SuspendLayout();
            this.gbSearchField.SuspendLayout();
            this.SuspendLayout();
            // 
            // butFilterOk
            // 
            this.butFilterOk.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.butFilterOk.Location = new System.Drawing.Point(451, 600);
            this.butFilterOk.Name = "butFilterOk";
            this.butFilterOk.Size = new System.Drawing.Size(120, 24);
            this.butFilterOk.TabIndex = 1;
            this.butFilterOk.Text = "Ok";
            this.butFilterOk.Click += new System.EventHandler(this.butOK_Click);
            // 
            // butFilterCancel
            // 
            this.butFilterCancel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.butFilterCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            this.butFilterCancel.Location = new System.Drawing.Point(577, 600);
            this.butFilterCancel.Name = "butFilterCancel";
            this.butFilterCancel.Size = new System.Drawing.Size(120, 24);
            this.butFilterCancel.TabIndex = 2;
            this.butFilterCancel.Text = "Cancel";
            // 
            // radioAnd
            // 
            this.radioAnd.Location = new System.Drawing.Point(259, 19);
            this.radioAnd.Name = "radioAnd";
            this.radioAnd.Size = new System.Drawing.Size(64, 24);
            this.radioAnd.TabIndex = 3;
            this.radioAnd.Text = "And";
            // 
            // radioOr
            // 
            this.radioOr.Location = new System.Drawing.Point(339, 19);
            this.radioOr.Name = "radioOr";
            this.radioOr.Size = new System.Drawing.Size(64, 24);
            this.radioOr.TabIndex = 4;
            this.radioOr.Text = "Or";
            // 
            // gbOtherFields
            // 
            this.gbOtherFields.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.gbOtherFields.Controls.Add(this.radioAnd);
            this.gbOtherFields.Controls.Add(this.radioOr);
            this.gbOtherFields.Controls.Add(this.panelOtherFields);
            this.gbOtherFields.ForeColor = System.Drawing.Color.FromArgb(((int)(((byte)(0)))), ((int)(((byte)(70)))), ((int)(((byte)(213)))));
            this.gbOtherFields.Location = new System.Drawing.Point(12, 82);
            this.gbOtherFields.Name = "gbOtherFields";
            this.gbOtherFields.Size = new System.Drawing.Size(686, 504);
            this.gbOtherFields.TabIndex = 6;
            this.gbOtherFields.TabStop = false;
            this.gbOtherFields.Text = "All Fields";
            this.gbOtherFields.CollapseBoxClickedEvent += new RepPP.Toolkit.Window.CollapsibleGroupBox.CollapseBoxClickedEventHandler(this.OnCollapseBoxClicked);
            // 
            // panelOtherFields
            // 
            this.panelOtherFields.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.panelOtherFields.Location = new System.Drawing.Point(7, 49);
            this.panelOtherFields.Name = "panelOtherFields";
            this.panelOtherFields.Size = new System.Drawing.Size(673, 449);
            this.panelOtherFields.TabIndex = 1;
            // 
            // gbSearchField
            // 
            this.gbSearchField.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.gbSearchField.Controls.Add(this.panelSearchField);
            this.gbSearchField.ForeColor = System.Drawing.Color.FromArgb(((int)(((byte)(0)))), ((int)(((byte)(70)))), ((int)(((byte)(213)))));
            this.gbSearchField.Location = new System.Drawing.Point(12, 12);
            this.gbSearchField.Name = "gbSearchField";
            this.gbSearchField.Size = new System.Drawing.Size(686, 64);
            this.gbSearchField.TabIndex = 7;
            this.gbSearchField.TabStop = false;
            this.gbSearchField.Text = "Current Field";
            // 
            // panelSearchField
            // 
            this.panelSearchField.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.panelSearchField.Location = new System.Drawing.Point(7, 20);
            this.panelSearchField.Name = "panelSearchField";
            this.panelSearchField.Size = new System.Drawing.Size(673, 38);
            this.panelSearchField.TabIndex = 0;
            // 
            // frmSBDirectSearch
            // 
            this.AcceptButton = this.butFilterOk;
            this.CancelButton = this.butFilterCancel;
            this.ClientSize = new System.Drawing.Size(709, 636);
            this.Controls.Add(this.gbSearchField);
            this.Controls.Add(this.gbOtherFields);
            this.Controls.Add(this.butFilterCancel);
            this.Controls.Add(this.butFilterOk);
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.SizableToolWindow;
            this.MaximizeBox = false;
            this.MinimizeBox = false;
            this.Name = "frmSBDirectSearch";
            this.ShowInTaskbar = false;
            this.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Hide;
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            this.Text = "Direct Search";
            this.gbOtherFields.ResumeLayout(false);
            this.gbSearchField.ResumeLayout(false);
            this.ResumeLayout(false);

        }
        #endregion
    }
}
