using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
    //********************************************************************************
    //
    /// <summary>
    /// Top editor using a ListView as a selection buffer
    /// </summary>
    //
    //********************************************************************************    
    public class frmSBEditorListView : frmSBEditorUI {
        /// <summary>Need by the component designer</summary>
        private System.ComponentModel.IContainer                components = null;
        /// <summary>Listview attached to a rowset</summary>
        protected ListViewRowset                                listView1;
        /// <summary>Splitter window</summary>
        protected System.Windows.Forms.Splitter                 splitter1;
        /// <summary>Panel containing the child editors</summary>
        protected System.Windows.Forms.Panel                    panelChild;
        /// <summary>Adapt a listview to act as a selection buffer</summary>
        private SBHolderListView                                m_SBHolderListView;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                                       Class constructor
        /// </summary>
        //
        //******************************************************************
        public frmSBEditorListView() {
            InitializeComponent();
        }

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the form
        /// </summary>
        /// <param name="bDisposing">   true if disposing, false if finalizing</param>
        //
        //******************************************************************
        protected override void Dispose(bool bDisposing) {
			if (bDisposing) {
			    if (m_SBHolderListView != null) {
			        m_SBHolderListView.Dispose();
			        m_SBHolderListView = null;
			    }
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose(bDisposing);
		}

        //******************************************************************
        //
        /// <summary>
        /// InitEditor:                                 Initializing the editor
        /// </summary>
        /// <param name="app">                          Rep++ application object</param>
        /// <param name="configHelper">                 Object keeping the UI configuration</param>
        /// <param name="strTopTranslatorAtomName">     Name of the atom use by the top translator</param>
        /// <param name="strBaseTranslatorAtomName">    Name of the atom use by the base translator. Null for default one.</param>
        //
        //******************************************************************
        protected override void InitEditor(RepPP.Application app, UIConfigurationHelper configHelper, string strTopTranslatorAtomName, string strBaseTranslatorAtomName) {
            base.InitEditor(app, configHelper, strTopTranslatorAtomName, strBaseTranslatorAtomName);
            if (app != null) {
                ChildContainer      = panelChild;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// InitSelectionBuffer:                        Initialize the Selection buffer
        /// </summary>
        /// <param name="strSBRowsetTreeName">                Name of the card representing the selection buffer</param>
        /// <param name="childEditorFactory">           Factory class use to create the child editors</param>
        //
        //******************************************************************
        protected override void InitSelectionBuffer(string strSBRowsetTreeName, ISBChildEditorFactory childEditorFactory) {
            base.InitSelectionBuffer(strSBRowsetTreeName, childEditorFactory);
            if (Application != null) {
                m_SBHolderListView  = new SBHolderListView(ListView, this, strSBRowsetTreeName);
                SelectionBuffer     = m_SBHolderListView;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ListView:                   Selection buffer listview
        /// </summary>
        //
        //******************************************************************
        public ListViewRowset ListView {
            get {
                return(listView1);
            }
            set {
                if (value != null) {
                    Controls.Remove(listView1);
                    listView1.Dispose();
                    listView1                       = value;
                    listView1.Dock                  = DockStyle.Bottom;
                    listView1.View                  = System.Windows.Forms.View.Details;
                    listView1.AutoMoveAllowed       = false;
                    listView1.AutoRenumField        = null;
                    listView1.AutoRenumIncrement    = 10;
                    listView1.AutoSort              = false;
                    listView1.Dock                  = System.Windows.Forms.DockStyle.Bottom;
                    listView1.FullRowSelect         = true;
                    listView1.HideSelection         = false;
                    listView1.IsUserFlagFieldShown  = true;
                    listView1.KeySearch             = true;
                    listView1.Location              = new System.Drawing.Point(0, 424);
                    listView1.MaxColumnSize         = 40;
                    listView1.MinColumnSize         = 5;
                    listView1.MultiSelect           = false;
                    listView1.SelectedItem          = null;
                    listView1.SizeMethod            = RepPP.Toolkit.Window.ListViewBase.SizeMethodE.Auto;
                    Controls.Add(listView1);
                    Controls.SetChildIndex(this.listView1, 0);
                    Controls.SetChildIndex(this.splitter1, 0);
                    Controls.SetChildIndex(this.panelChild, 0);
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// RegisterTriggerToChildEditor:   Register the trigger for the child editor commands
        /// </summary>
        /// <param name="nodeHandler">    Framework card manager</param>
        //
        //******************************************************************
        public override void RegisterTriggerToChildEditor(RowsetTreeDefNodeHandlerRootBase nodeHandler) {
            base.RegisterTriggerToChildEditor(nodeHandler);
            listView1.ContextMenuStrip = CreateSBContextMenu();
        }
		#region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent() {
            this.listView1 = new RepPP.Toolkit.Window.ListViewRowset();
            this.splitter1 = new System.Windows.Forms.Splitter();
            this.panelChild = new System.Windows.Forms.Panel();
            this.SuspendLayout();
            // 
            // listView1
            // 
            this.listView1.AutoMoveAllowed = false;
            this.listView1.AutoRenumField = null;
            this.listView1.AutoRenumIncrement = 10;
            this.listView1.AutoSort = false;
            this.listView1.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.listView1.DownButton = null;
            this.listView1.FullRowSelect = true;
            this.listView1.RowsetDef = null;
            this.listView1.Rowset = null;
            this.listView1.HideSelection = false;
            this.listView1.IsUserFlagFieldShown = true;
            this.listView1.KeySearch = true;
            this.listView1.Location = new System.Drawing.Point(0, 424);
            this.listView1.MaxColumnSize = 40;
            this.listView1.MinColumnSize = 5;
            this.listView1.MultiSelect = false;
            this.listView1.Name = "listView1";
            this.listView1.SelectedItem = null;
            this.listView1.ShadeColor = System.Drawing.Color.WhiteSmoke;
            this.listView1.Size = new System.Drawing.Size(856, 134);
            this.listView1.SizeMethod = RepPP.Toolkit.Window.ListViewBase.SizeMethodE.Auto;
            this.listView1.TabIndex = 18;
            this.listView1.UpButton = null;
            this.listView1.View = System.Windows.Forms.View.Details;
            // 
            // splitter1
            // 
            this.splitter1.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.splitter1.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.splitter1.Location = new System.Drawing.Point(0, 421);
            this.splitter1.Name = "splitter1";
            this.splitter1.Size = new System.Drawing.Size(856, 3);
            this.splitter1.TabIndex = 19;
            this.splitter1.TabStop = false;
            // 
            // panelChild
            // 
            this.panelChild.AutoScroll = true;
            this.panelChild.Dock = System.Windows.Forms.DockStyle.Fill;
            this.panelChild.Location = new System.Drawing.Point(0, 28);
            this.panelChild.Name = "panelChild";
            this.panelChild.Size = new System.Drawing.Size(856, 393);
            this.panelChild.TabIndex = 20;
            // 
            // frmSBEditorListView
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(856, 582);
            this.Controls.Add(this.panelChild);
            this.Controls.Add(this.splitter1);
            this.Controls.Add(this.listView1);
            this.Name = "frmSBEditorListView";
            this.Text = "frmSBEditorListView";
            this.Controls.SetChildIndex(this.listView1, 0);
            this.Controls.SetChildIndex(this.splitter1, 0);
            this.Controls.SetChildIndex(this.panelChild, 0);
            this.ResumeLayout(false);

        }
		#endregion

    }
}
