using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
    //********************************************************************************
    //
    /// <summary>
    /// Editor using a treeview control as a selection buffer.
    /// A selection buffer editor is a container of a selection buffer and
    /// a child  editor controled by the selection buffer.
    /// </summary>
    //
    //********************************************************************************    
    public class frmSBEditorTreeView : frmSBEditorUI {
        /// <summary>Panel containing the treeview</summary>
        protected System.Windows.Forms.Panel                    pnlTreeView;
        /// <summary>Treeview attached to the selection buffer</summary>
        protected TreeViewRowsetTree                            treeView1;
        /// <summary>Splitter window</summary>
        protected System.Windows.Forms.Splitter                 splitter1;
        /// <summary>Panel containing the child editors</summary>
        protected System.Windows.Forms.Panel                    panelChild;
        /// <summary>Adapt a listview to act as a selection buffer</summary>
        protected SBHolderTreeView                              m_SBHolderTreeView;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                            Class constructor
        /// </summary>
        //
        //******************************************************************
        public frmSBEditorTreeView() {
            InitializeComponent();
		}

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the form
        /// </summary>
        /// <param name="bDisposing">   true if disposing, false if finalizing</param>
        //
        //******************************************************************
        protected override void Dispose(bool bDisposing) {
            if (bDisposing) {
                if (components != null)	{
                    components.Dispose();
                }
                if (m_SBHolderTreeView != null) {
                    m_SBHolderTreeView.Dispose();
                    m_SBHolderTreeView = null;
                }
            }
            base.Dispose(bDisposing);
        }

        //******************************************************************
        //
        /// <summary>
        /// InitEditor:                                 Initializing the editor
        /// </summary>
        /// <param name="app">                          Rep++ application object</param>
        /// <param name="configHelper">                 Object keeping the UI configuration</param>
        /// <param name="strTopTranslatorAtomName">     Name of the atom use by the top translator</param>
        /// <param name="strBaseTranslatorAtomName">    Name of the atom use by the base translator Null for default one.</param>
        //
        //******************************************************************
        protected override void InitEditor(RepPP.Application app, UIConfigurationHelper configHelper, string strTopTranslatorAtomName, string strBaseTranslatorAtomName) {
            base.InitEditor(app, configHelper, strTopTranslatorAtomName, strBaseTranslatorAtomName);
            if (app != null) {
                ChildContainer = panelChild;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// InitSelectionBuffer:                        Initialize the Selection buffer
        /// </summary>
        /// <param name="strSBRowsetTreeName">                Name of the card representing the selection buffer</param>
        /// <param name="childEditorFactory">           Factory class use to create the child editors</param>
        //
        //******************************************************************
        protected override void InitSelectionBuffer(string strSBRowsetTreeName, ISBChildEditorFactory childEditorFactory) {
            base.InitSelectionBuffer(strSBRowsetTreeName, childEditorFactory);
            if (Application != null) {
                m_SBHolderTreeView  = new SBHolderTreeView(TreeView, this, strSBRowsetTreeName);
                SelectionBuffer     = m_SBHolderTreeView;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// TreeView:                   Selection buffer treeview
        /// </summary>
        //
        //******************************************************************
        public TreeViewRowsetTree TreeView {
            get {
                return(treeView1);
            }
            set {
                if (value != null) {
                    pnlTreeView.Controls.Remove(treeView1);
                    treeView1.Dispose();
                    treeView1               = value;
                    treeView1.Dock          = System.Windows.Forms.DockStyle.Fill;
                    treeView1.HideSelection = false;                    
                    pnlTreeView.Controls.Add(treeView1);
                }
            }
        }

//        //******************************************************************
//        //
//        /// <summary>
//        /// LoadEarlySetting        Load the UI configuration state which cannot wait for form load
//        /// </summary>
//        /// <param name="helper">   Helper object</param>
//        /// <param name="node">     Node containing the serialization</param>
//        //
//        //******************************************************************
//        protected override void LoadEarlySetting(RepPP.Toolkit.Window.UIConfigurationHelper configHelper, System.Xml.XmlNode node) {
//            base.LoadEarlySetting(configHelper, node);
//            treeView1.LoadState(configHelper, node);
//        }

        //******************************************************************
        //
        /// <summary>
        /// RegisterTriggerToChildEditor:   Register the trigger for the child editor commands
        /// </summary>
        /// <param name="nodeHandler">       Framework card manager</param>
        //
        //******************************************************************
        public override void RegisterTriggerToChildEditor(RowsetTreeDefNodeHandlerRootBase nodeHandler) {
            base.RegisterTriggerToChildEditor(nodeHandler);
            treeView1.ContextMenuStrip = CreateSBContextMenu();   
        }

		#region Designer generated code
        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.Container components = null;
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.pnlTreeView = new System.Windows.Forms.Panel();
            this.treeView1 = new RepPP.Toolkit.Window.TreeViewRowsetTree();
            this.splitter1 = new System.Windows.Forms.Splitter();
            this.panelChild = new System.Windows.Forms.Panel();
            this.pnlTreeView.SuspendLayout();
            this.SuspendLayout();
            // 
            // mainStatusBar
            // 
            this.mainStatusBar.Name = "mainStatusBar";
            // 
            // toolBarRowsetTree
            // 
            this.toolBarRowsetTree.Name = "toolBarRowsetTree";
            // 
            // pnlTreeView
            // 
            this.pnlTreeView.Controls.Add(this.treeView1);
            this.pnlTreeView.Dock = System.Windows.Forms.DockStyle.Left;
            this.pnlTreeView.Location = new System.Drawing.Point(0, 28);
            this.pnlTreeView.Name = "pnlTreeView";
            this.pnlTreeView.Size = new System.Drawing.Size(216, 530);
            this.pnlTreeView.TabIndex = 18;
            // 
            // treeView1
            // 
            this.treeView1.ConfigurationHelper = null;
            this.treeView1.Dock = System.Windows.Forms.DockStyle.Fill;
            this.treeView1.HideSelection = false;
            this.treeView1.ImageIndex = -1;
            this.treeView1.Location = new System.Drawing.Point(0, 0);
            this.treeView1.Name = "treeView1";
            this.treeView1.SelectedImageIndex = -1;
            this.treeView1.Size = new System.Drawing.Size(216, 530);
            this.treeView1.TabIndex = 18;
            this.treeView1.ToolTip = null;
            // 
            // splitter1
            // 
            this.splitter1.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.splitter1.Cursor = System.Windows.Forms.Cursors.VSplit;
            this.splitter1.Location = new System.Drawing.Point(216, 28);
            this.splitter1.Name = "splitter1";
            this.splitter1.Size = new System.Drawing.Size(6, 530);
            this.splitter1.TabIndex = 19;
            this.splitter1.TabStop = false;
            // 
            // panelChild
            // 
            this.panelChild.AutoScroll = true;
            this.panelChild.Dock = System.Windows.Forms.DockStyle.Fill;
            this.panelChild.Location = new System.Drawing.Point(222, 28);
            this.panelChild.Name = "panelChild";
            this.panelChild.Size = new System.Drawing.Size(634, 530);
            this.panelChild.TabIndex = 20;
            // 
            // frmSBEditorTreeView
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(856, 582);
            this.Controls.Add(this.panelChild);
            this.Controls.Add(this.splitter1);
            this.Controls.Add(this.pnlTreeView);
            this.Name = "frmSBEditorTreeView";
            this.Text = "frmTopTreeViewSBEditor";
            this.Controls.SetChildIndex(this.mainStatusBar, 0);
            this.Controls.SetChildIndex(this.toolBarRowsetTree, 0);
            this.Controls.SetChildIndex(this.pnlTreeView, 0);
            this.Controls.SetChildIndex(this.splitter1, 0);
            this.Controls.SetChildIndex(this.panelChild, 0);
            this.pnlTreeView.ResumeLayout(false);
            this.ResumeLayout(false);
        }
		#endregion
	} // frmSBEditorTreeView
} // namespace
