using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
    //********************************************************************************
    //
    /// <summary>
    /// Editor using a treeview control as a selection buffer and a listview control 
    /// for text node.
    /// A selection buffer editor is a container of a selection buffer and
    /// a child  editor controled by the selection buffer.
    /// </summary>
    //
    //********************************************************************************    
	public class frmSBEditorTreeViewListView : frmSBEditorTreeView {
        /// <summary>Panel containing the listview</summary>
        protected System.Windows.Forms.Panel                pnlListView;
        /// <summary>Listview containing the secondary selection buffer</summary>
        protected ListViewRowset                            listView1;
        /// <summary>Splitter window</summary>
        protected System.Windows.Forms.Splitter             splitterListView;
		private System.ComponentModel.Container             components = null;
        /// <summary>Adapt a treeview and listview to act as a selection buffer</summary>
        private SBHolderTreeViewListView                    m_SBHolderTreeViewListView;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                            Class constructor
        /// </summary>
        //
        //******************************************************************
        public frmSBEditorTreeViewListView() {
            InitializeComponent();
        }

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the form
        /// </summary>
        /// <param name="bDisposing">   true if disposing, false if finalizing</param>
        //
        //******************************************************************
        protected override void Dispose(bool bDisposing) {
            if (bDisposing) {
                if (components != null)	{
                    components.Dispose();
                }
                if (m_SBHolderTreeViewListView != null) {
                    m_SBHolderTreeViewListView.Dispose();
                    m_SBHolderTreeViewListView = null;
                }
            }
            base.Dispose(bDisposing);
        }

        //******************************************************************
        //
        /// <summary>
        /// InitEditor:                                 Initializing the editor
        /// </summary>
        /// <param name="app">                          Rep++ application object</param>
        /// <param name="configHelper">                 Object keeping the UI configuration</param>
        /// <param name="strTopTranslatorAtomName">     Name of the atom use by the top translator</param>
        /// <param name="strBaseTranslatorAtomName">    Name of the atom use by the base translator Null for default one.</param>
        //
        //******************************************************************
        protected override void InitEditor(RepPP.Application app, UIConfigurationHelper configHelper, string strTopTranslatorAtomName, string strBaseTranslatorAtomName) {
            base.InitEditor(app, configHelper, strTopTranslatorAtomName, strBaseTranslatorAtomName);
            if (app != null) {
                ChildContainer = panelChild;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// InitSelectionBuffer:                        Initialize the Selection buffer
        /// </summary>
        /// <param name="strSBRowsetTreeName">                Name of the card representing the selection buffer</param>
        /// <param name="childEditorFactory">           Factory class use to create the child editors</param>
        //
        //******************************************************************
        protected override void InitSelectionBuffer(string strSBRowsetTreeName, ISBChildEditorFactory childEditorFactory) {
            base.InitSelectionBuffer(strSBRowsetTreeName, childEditorFactory);
            if (Application != null) {
                // Dispose the selection buffer holder created by the base class
                if (m_SBHolderTreeView != null) {
                    m_SBHolderTreeView.Dispose();
                    m_SBHolderTreeView = null;
                }
                m_SBHolderTreeViewListView  = new SBHolderTreeViewListView(TreeView, ListView, this, strSBRowsetTreeName);
                SelectionBuffer             = m_SBHolderTreeViewListView;
                treeView1.AfterSelect      += new TreeViewEventHandler(AfterNodeSelected);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// AfterNodeSelected:          Called after a node has been selected
        /// </summary>
        /// <param name="sender">       Sender object   </param>
        /// <param name="e">            Event arguments </param>
        //
        //******************************************************************
        protected virtual void AfterNodeSelected(object sender, TreeViewEventArgs e) {            
            pnlListView.Visible      = (e.Node is SBHolderTreeViewListView.TreeNodeListView);
            splitterListView.Visible = pnlListView.Visible;
        }

        //******************************************************************
        //
        /// <summary>
        /// ListView:                   Selection buffer listview
        /// </summary>
        //
        //******************************************************************
        public ListViewRowset ListView {
            get {
                return(listView1);
            }
            set {
                if (value != null) {
                    pnlListView.Controls.Remove(listView1);
                    listView1.Dispose();
                    listView1                       = value;
                    listView1.Dock                  = DockStyle.Bottom;
                    listView1.View                  = System.Windows.Forms.View.Details;
                    listView1.AutoMoveAllowed       = false;
                    listView1.AutoRenumField        = null;
                    listView1.AutoRenumIncrement    = 10;
                    listView1.AutoSort              = false;
                    listView1.Dock                  = System.Windows.Forms.DockStyle.Fill;
                    listView1.FullRowSelect         = true;
                    listView1.HideSelection         = false;
                    listView1.IsUserFlagFieldShown  = true;
                    listView1.KeySearch             = true;
                    listView1.MaxColumnSize         = 40;
                    listView1.MinColumnSize         = 5;
                    listView1.MultiSelect           = false;
                    listView1.SelectedItem          = null;
                    listView1.SizeMethod            = RepPP.Toolkit.Window.ListViewBase.SizeMethodE.Auto;
                    pnlListView.Controls.Add(listView1);
                }
            }
        }

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.pnlListView = new System.Windows.Forms.Panel();
            this.listView1 = new ListViewRowset();
            this.splitterListView = new System.Windows.Forms.Splitter();
            this.pnlListView.SuspendLayout();
            this.SuspendLayout();
            // 
            // pnlListView
            // 
            this.pnlListView.Controls.Add(this.listView1);
            this.pnlListView.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.pnlListView.Location = new System.Drawing.Point(222, 424);
            this.pnlListView.Name = "pnlListView";
            this.pnlListView.Size = new System.Drawing.Size(634, 134);
            this.pnlListView.TabIndex = 18;
            // 
            // listView1
            // 
            this.listView1.AutoMoveAllowed = false;
            this.listView1.AutoRenumField = null;
            this.listView1.AutoRenumIncrement = 10;
            this.listView1.AutoSort = false;
            this.listView1.Dock = System.Windows.Forms.DockStyle.Fill;
            this.listView1.DownButton = null;
            this.listView1.FullRowSelect = true;
            this.listView1.RowsetDef = null;
            this.listView1.Rowset = null;
            this.listView1.HideSelection = false;
            this.listView1.IsUserFlagFieldShown = true;
            this.listView1.Location = new System.Drawing.Point(0, 0);
            this.listView1.MaxColumnSize = 40;
            this.listView1.MinColumnSize = 5;
            this.listView1.MultiSelect = false;
            this.listView1.Name = "listView1";
            this.listView1.SelectedItem = null;
            this.listView1.ShadeColor = System.Drawing.Color.WhiteSmoke;
            this.listView1.Size = new System.Drawing.Size(634, 134);
            this.listView1.SizeMethod = ListViewBase.SizeMethodE.Auto;
            this.listView1.TabIndex = 18;
            this.listView1.UpButton = null;
            this.listView1.View = System.Windows.Forms.View.Details;
            // 
            // splitterListView
            // 
            this.splitterListView.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.splitterListView.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.splitterListView.Location = new System.Drawing.Point(222, 418);
            this.splitterListView.Name = "splitterListView";
            this.splitterListView.Size = new System.Drawing.Size(634, 6);
            this.splitterListView.TabIndex = 19;
            this.splitterListView.TabStop = false;
            // 
            // frmSBEditorTreeViewListView
            // 
            this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
            this.ClientSize = new System.Drawing.Size(856, 582);
            this.Controls.Add(this.splitterListView);
            this.Controls.Add(this.pnlListView);
            this.Name = "frmSBEditorTreeViewListView";
            this.Text = "frmSBEditorTreeViewListView";
            this.Controls.SetChildIndex(this.mainStatusBar, 0);
            this.Controls.SetChildIndex(this.toolBarRowsetTree, 0);
            this.Controls.SetChildIndex(this.pnlTreeView, 0);
            this.Controls.SetChildIndex(this.splitter1, 0);
            this.Controls.SetChildIndex(this.pnlListView, 0);
            this.Controls.SetChildIndex(this.splitterListView, 0);
            this.Controls.SetChildIndex(this.panelChild, 0);
            this.pnlListView.ResumeLayout(false);
            this.ResumeLayout(false);

        }
		#endregion
	}
}
