using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
    //********************************************************************************
    //
    /// <summary>
    /// Implements the base interface of an editor holding a selection buffer
    /// </summary>
    //
    //********************************************************************************    
    public class frmSBEditorUI : frmSBEditorBase {
        /// <summary>Main menu</summary>
        protected MenuStrip             mainMenu1;
        /// <summary>File menu item</summary>
        protected ToolStripMenuItem     mnuFile;
        /// <summary>Item New menu item</summary>
        protected ToolStripMenuItem     mnuItemNew;
        /// <summary>Item Duplicate menu item</summary>
        protected ToolStripMenuItem     mnuItemDup;
        /// <summary>Item Save menu item</summary>
        protected ToolStripMenuItem     mnuItemSave;
        /// <summary>Item Delete menu item</summary>
        protected ToolStripMenuItem     mnuItemDel;
        /// <summary>Item Undo menu item</summary>
        protected ToolStripMenuItem     mnuItemUndo;
        /// <summary>Exit menu item</summary>
        protected ToolStripMenuItem     mnuExit;
        /// <summary>Edit menu item</summary>
        protected ToolStripMenuItem     mnuEdit;
        /// <summary>FieldDef choice menu item</summary>
        protected ToolStripMenuItem     mnuFieldChoice;
        /// <summary>FieldDef erase menu item</summary>
        protected ToolStripMenuItem     mnuFieldErase;
        /// <summary>FieldDef default menu item</summary>
        protected ToolStripMenuItem     mnuFieldDefault;
        /// <summary>FieldDef erase all menu item</summary>
        protected ToolStripMenuItem     mnuFieldEraseAll;
        /// <summary>FieldDef default all menu item</summary>
        protected ToolStripMenuItem     mnuFieldDefaultAll;
        /// <summary>FieldDef direct search menu item</summary>
        protected ToolStripMenuItem     mnuDirectSearch;
#if !(SD60 || SD70)
        /// <summary>Create macro menu item</summary>
        protected ToolStripMenuItem     mnuCreateMacro;
        /// <summary>Edit macro menu item</summary>
        protected ToolStripMenuItem     mnuEditMacro;
        /// <summary>Delete macro menu item</summary>
        protected ToolStripMenuItem     mnuDeleteMacro;
        /// <summary>Separator</summary>
        protected ToolStripSeparator    mnuSep3;
#endif
        /// <summary>Refresh menu item</summary>
        protected ToolStripMenuItem     mnuRefresh;
        /// <summary>Selection menu item</summary>
        protected ToolStripMenuItem     mnuSelection;
        /// <summary>Item Previous menu item</summary>
        protected ToolStripMenuItem     mnuItemPrev;
        /// <summary>Item Next menu item</summary>
        protected ToolStripMenuItem     mnuItemNext;
        /// <summary>Item First menu item</summary>
        protected ToolStripMenuItem     mnuItemFirst;
        /// <summary>Item Last menu item</summary>
        protected ToolStripMenuItem     mnuItemLast;
        /// <summary>Separator</summary>
        protected ToolStripSeparator    mnuSep1;
        /// <summary>Separator</summary>
        protected ToolStripSeparator    mnuSep2;
        /// <summary>Set Filter menu item</summary>
        protected ToolStripMenuItem     mnuSetFilter;
        /// <summary>Reset Filter menu item</summary>
        protected ToolStripMenuItem     mnuResetFilter;
        /// <summary>Help menu item</summary>
        protected ToolStripMenuItem     mnuHelp;
        /// <summary>FieldDef Help menu item</summary>
        protected ToolStripMenuItem     mnuFieldHelp;
        /// <summary>Content menu item</summary>
        protected ToolStripMenuItem     mnuHelpContent;
        /// <summary>About menu item</summary>
        protected ToolStripMenuItem     mnuHelpAbout;
        /// <summary>StatusBar</summary>
        protected StatusStrip           mainStatusBar;
        /// <summary>StatusBar Panel</summary>
        protected ToolStripStatusLabel  statusBarPanel1;
        /// <summary>RowsetTreeDef ToolBar</summary>
        protected ToolbarRowsetTree                             toolBarRowsetTree;
        private System.ComponentModel.IContainer                components = null;
        /// <summary>Control displaying error icon on the side of a field</summary>
        private FldErrorProvider                                m_errorProvider;
        private ArrayList                                       m_arrSearchMethodMenuItems;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                                       Class constructor
        /// </summary>
        //
        //******************************************************************
        public frmSBEditorUI() {
            InitializeComponent();
            toolBarRowsetTree.DirectSearchHandler = this;
            m_arrSearchMethodMenuItems            = new ArrayList();
        }

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the form
        /// </summary>
        /// <param name="bDisposing">   true if disposing, false if finalizing</param>
        //
        //******************************************************************
        protected override void Dispose(bool bDisposing) {
            if (bDisposing) {
                if (Application != null) {
                    Application.FldOnError -= new RepPP.FldOnErrorEventHandler(OnFldOnError);
                }
                if (components != null) {
                    components.Dispose();
                }
            }
            base.Dispose(bDisposing);
        }

        //******************************************************************
        //
        /// <summary>
        /// InitEditor:                                 Initializing the editor
        /// </summary>
        /// <param name="app">                          Rep++ application object</param>
        /// <param name="configHelper">                 Object keeping the UI configuration</param>
        /// <param name="strTopTranslatorAtomName">     Name of the atom use by the top translator</param>
        /// <param name="strBaseTranslatorAtomName">    Name of the atom use by the base translator. Null for default one</param>
        //
        //******************************************************************
        protected override void InitEditor(RepPP.Application app, UIConfigurationHelper configHelper, string strTopTranslatorAtomName, string strBaseTranslatorAtomName) {
            base.InitEditor(app, configHelper, strTopTranslatorAtomName, strBaseTranslatorAtomName);
            if (app != null) {
                m_errorProvider          = new FldErrorProvider(ErrorIconAlignment.MiddleLeft, 2);
                Application.FldOnError  += new RepPP.FldOnErrorEventHandler(OnFldOnError);
                RegisterEditorCmd();
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ErrorProvider:          The error provider object responsible to display
        ///                         the error icon in the side of the field controls
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public FldErrorProvider ErrorProvider {
            get {
                return(m_errorProvider);
            }
            set {
                m_errorProvider = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// RegisterEditorCmd:              Register the editor command
        /// </summary>
        //
        //******************************************************************
        protected virtual void RegisterEditorCmd() {
            CmdHolder   cmdHolder;
            
            cmdHolder = CommandHolder;
            cmdHolder.AddCmd(sdHandlerCmd.ItemSaveAll.ToString(), null, CmdHolder.CmdStateE.Hidden);
            cmdHolder.AddCmd(sdHandlerCmd.RowsetTreeSearch.ToString(), null, CmdHolder.CmdStateE.Hidden);
            cmdHolder.AddCmd("Exit", new CmdHolder.OnCommand(CmdExit), CmdHolder.CmdStateE.Enabled);
            cmdHolder.AddCmd("FormRefresh", new CmdHolder.OnCommand(FormRefresh), CmdHolder.CmdStateE.Enabled);
            cmdHolder.AddTrigger(null, "Exit",          new object[] { mnuExit} );
            cmdHolder.AddTrigger(null, "FormRefresh",   new object[] { mnuRefresh} );
            cmdHolder.AddCmd("SetFilter", new CmdHolder.OnCommand(SetFilter), CmdHolder.CmdStateE.Enabled);
            cmdHolder.AddCmd("ResetFilter", new CmdHolder.OnCommand(ResetFilter), CmdHolder.CmdStateE.Enabled);
            cmdHolder.AddCmd("DirectSearch", new CmdHolder.OnCommand(DirectSearch), CmdHolder.CmdStateE.Disabled);
#if !(SD60 || SD70)
            bool bMacroManipulationAllowed = IsMacroManipulationAllowed();
            cmdHolder.AddCmd("CreateMacro", new CmdHolder.OnCommand(CreateMacro), bMacroManipulationAllowed ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Hidden);
            cmdHolder.AddCmd("EditMacro",   new CmdHolder.OnCommand(EditMacro),   bMacroManipulationAllowed ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Hidden);
            cmdHolder.AddCmd("DeleteMacro", new CmdHolder.OnCommand(DeleteMacro), bMacroManipulationAllowed ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Hidden);
            mnuSep3.Visible = bMacroManipulationAllowed;
#endif
        }

#if !(SD60 || SD70)

        //******************************************************************
        //
        /// <summary>
        /// ReloadSearchMethods:             Reloads the list of search methods
        /// </summary>
        //
        //******************************************************************
        public override void ReloadSearchMethods() {
            RemoveSearchMethodMenuItems();
            base.ReloadSearchMethods();
            AddSearchMethodMenuItems();
        }

        //******************************************************************
        //
        /// <summary>
        /// SelectionBuffer:                    Selection buffer
        /// </summary>
        //
        //******************************************************************
        public override ISelectionBuffer SelectionBuffer {
            get {
                return base.SelectionBuffer;
            }
            set {
                if (SelectionBuffer != value){
                    RemoveSearchMethodMenuItems();
                    base.SelectionBuffer = value;
                    AddSearchMethodMenuItems();
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// AddSearchMethodMenuItems:   Adds the Search Method menu items to
        ///                             the selection menu
        /// </summary>
        //
        //******************************************************************
        protected virtual void AddSearchMethodMenuItems() {
			ToolStripSeparator  sep; 
            ToolStripMenuItem   item;
            ISearchMethod       searchMethod;
            string              strMnuText;
            int                 iCount;

            // Do not add the first search method,
            // it is used for direct search
            iCount = SearchMethods.Length;
            if (iCount > 1) {
				sep = new ToolStripSeparator();
                mnuSelection.DropDown.Items.Add(sep);
                m_arrSearchMethodMenuItems.Add(sep);
                for(int iIndex = 1; iIndex < iCount; iIndex++) {
                    searchMethod = SearchMethods[iIndex];
                    strMnuText   = searchMethod.Name;
                    if (searchMethod.HasArguments) {
                        strMnuText += "...";
                    }
					item     = new ToolStripMenuItem(strMnuText, null, OnExecuteSearch);
                    item.Tag = iIndex;
                    if (searchMethod.AccelerationKey != 0) 
					{
						try {
                            item.ShortcutKeys = (Keys)searchMethod.AccelerationKey;
                        } 
                        catch {
                            item.ShortcutKeys = Keys.None;
                        }
                    }
					mnuSelection.DropDown.Items.Add(item);
                    m_arrSearchMethodMenuItems.Add(item);
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// RemoveSearchMethodMenuItems:    Removes the SearchMethod menu items
        /// </summary>
        //
        //******************************************************************
        protected virtual void RemoveSearchMethodMenuItems() {
			ToolStripItem item;
            int           iCount;

            iCount = m_arrSearchMethodMenuItems.Count;
            if (iCount > 0) {
                for (int iIndex = 0; iIndex < iCount; iIndex++) {
					item = m_arrSearchMethodMenuItems[iIndex] as ToolStripItem;
                    mnuSelection.DropDown.Items.Remove(item);
                    item.Dispose();
                }
                m_arrSearchMethodMenuItems.Clear();
            }
        }
#endif

        //******************************************************************
        //
        /// <summary>
        /// RegisterTriggerToChildEditor:   Register the trigger for the child editor commands
        /// </summary>
        /// <param name="nodeHandler">    Framework card manager</param>
        //
        //******************************************************************
        public override void RegisterTriggerToChildEditor(RowsetTreeDefNodeHandlerRootBase nodeHandler) {            
            CmdHolder   cmdHolder;
            
            cmdHolder = CommandHolder;
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemSave.ToString(),            new object[] { mnuItemSave });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemUndo.ToString(),            new object[] { mnuItemUndo });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldChoice.ToString(),         new object[] { mnuFieldChoice });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldHelp.ToString(),           new object[] { mnuFieldHelp });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldErase.ToString(),          new object[] { mnuFieldErase });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldDefault.ToString(),        new object[] { mnuFieldDefault });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldEraseAll.ToString(),       new object[] { mnuFieldEraseAll });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldDefaultAll.ToString(),     new object[] { mnuFieldDefaultAll });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemFirst.ToString(),           new object[] { mnuItemFirst });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemPrev.ToString(),            new Object[] { mnuItemPrev });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemNext.ToString(),            new Object[] { mnuItemNext });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemLast.ToString(),            new Object[] { mnuItemLast });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemNew.ToString(),             new Object[] { mnuItemNew });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemDup.ToString(),             new Object[] { mnuItemDup });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemDel.ToString(),             new Object[] { mnuItemDel });
            cmdHolder.AddTrigger(null, "SetFilter",                                 new Object[] { mnuSetFilter });
            cmdHolder.AddTrigger(null, "ResetFilter",                               new Object[] { mnuResetFilter });
            cmdHolder.AddTrigger(null, "DirectSearch",                              new Object[] { mnuDirectSearch });
#if !(SD60 || SD70)
            cmdHolder.AddTrigger(null, "CreateMacro",                               new Object[] { mnuCreateMacro });
            cmdHolder.AddTrigger(null, "EditMacro",                                 new Object[] { mnuEditMacro });
            cmdHolder.AddTrigger(null, "DeleteMacro",                               new Object[] { mnuDeleteMacro });
#endif
            if (nodeHandler != null) {
                if (toolBarRowsetTree.RootNodeHandler != null) {
                    toolBarRowsetTree.RootNodeHandler.ResetFieldError -= new EventHandler(this.OnResetFieldError);
                }
                nodeHandler.AdjustFieldState      += new EventHandler(this.OnAdjustFieldState);
                nodeHandler.ResetFieldError       += new EventHandler(this.OnResetFieldError);
                toolBarRowsetTree.RootNodeHandler     = nodeHandler;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CreateSBContextMenu:            Creates a context menu
        /// </summary>
        //
        //******************************************************************
        public override ContextMenuStrip CreateSBContextMenu() {
            ContextMenuStrip        ctxMenu;
            ToolStripMenuItem       miSave;
            ToolStripMenuItem       miNew;
            ToolStripMenuItem       miDup;
            ToolStripMenuItem       miDel;
            ToolStripMenuItem       miUndo;
            ToolStripMenuItem       miSetFilter;
            ToolStripMenuItem       miResetFilter;
            ToolStripSeparator      miSep1;
            ToolStripSeparator      miSep2;
            ToolStripSeparator      miSep3;
            ToolStripSeparator      miSep4;
            CmdHolder               cmdHolder;
            ControlTextTranslator   trans;
            
            cmdHolder     = CommandHolder;
            trans         = ControlTextTranslator;
            ctxMenu       = new ContextMenuStrip();
            miSave        = new ToolStripMenuItem(trans.GetItemText(sdHandlerCmd.ItemSave.ToString(),       "Text"));
            miNew         = new ToolStripMenuItem(trans.GetItemText(sdHandlerCmd.ItemNew.ToString(),        "Text"));
            miDup         = new ToolStripMenuItem(trans.GetItemText(sdHandlerCmd.ItemDup.ToString(),        "Text"));
            miDel         = new ToolStripMenuItem(trans.GetItemText(sdHandlerCmd.ItemDel.ToString(),        "Text"));
            miUndo        = new ToolStripMenuItem(trans.GetItemText(sdHandlerCmd.ItemUndo.ToString(),       "Text"));
            miSetFilter   = new ToolStripMenuItem(trans.GetItemText("miSetFilter",                          "Text"));
            miResetFilter = new ToolStripMenuItem(trans.GetItemText("miResetFilter",                        "Text"));
            miSep1        = new ToolStripSeparator();
            miSep2        = new ToolStripSeparator();
            miSep3        = new ToolStripSeparator();
            miSep4        = new ToolStripSeparator();
            ctxMenu.Items.AddRange(new ToolStripItem[] { miNew, miSep1, miDup, miUndo, miSep2, miSave, miSep3, miDel, miSep4, miSetFilter, miResetFilter });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemSave.ToString(),        new object[] { miSave });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemNew.ToString(),         new object[] { miNew });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemDup.ToString(),         new object[] { miDup });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemDel.ToString(),         new object[] { miDel });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemUndo.ToString(),        new object[] { miUndo });
            cmdHolder.AddTrigger(null, "SetFilter",                             new object[] {miSetFilter} );
            cmdHolder.AddTrigger(null, "ResetFilter",                           new object[] {miResetFilter} );
            return(ctxMenu);
        }

        //*********************************************************
        //
        /// <summary>
        /// CmdExit:                    Exit the program
        /// </summary>
        //  
        //*********************************************************
        protected virtual void CmdExit() {
            Close();
        }

        //*********************************************************
        //
        /// <summary>
        /// FormRefresh:                Refresh the form
        /// </summary>
        //  
        //*********************************************************
        public virtual void FormRefresh() {
            Refresh(false);
        }

        //*********************************************************
        //
        /// <summary>
        /// OnClosing:                  Check before closing the form
        /// </summary>
        //  
        //*********************************************************
        protected override void OnClosing(CancelEventArgs e) {
            base.OnClosing(e);
            if (!e.Cancel && !AskToSave(false)) {
                e.Cancel = true;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnAdjustFieldState:     Handle the framework adjust field state event
        /// </summary>
        /// <param name="sender">   Event sender</param>
        /// <param name="e">        Event arguments</param>
        // 
        //******************************************************************
        protected virtual void OnAdjustFieldState(Object sender, EventArgs e) {
            bool    bEnable = false;

            bEnable = IsDirectSearchField(CurrentChildEditor.CurrentFieldDef);
            CommandHolder.SetCmdState("DirectSearch", bEnable ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled);
        }

        //******************************************************************
        //
        /// <summary>
        /// OnResetFieldError:      Handle the framework reset field error event
        /// </summary>
        /// <param name="sender">   Event sender</param>
        /// <param name="e">        Event arguments</param>
        // 
        //******************************************************************
        protected virtual void OnResetFieldError(Object sender, EventArgs e) {
            FldErrorProvider    errorProvider;
            
            errorProvider = ErrorProvider;
            statusBarPanel1.Text = "";
            errorProvider.Reset();
        }

		//******************************************************************
		//
		/// <summary>
		/// OnFldOnError:           Handle the rep++ field error event
		/// </summary>
		/// <param name="sender">   Event sender</param>
		/// <param name="e">        Event arguments</param>
		// 
		//******************************************************************
		protected virtual void OnFldOnError(Object sender, RepPP.FieldGenEventArgs e) {
			m_errorProvider.Reset();
			m_errorProvider.AddError(this, e.FieldDef, e.ErrorText);
			statusBarPanel1.Text = e.FieldDef.Prompt + ": " + e.ErrorText;
		}

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent() {
            this.components = new System.ComponentModel.Container();
            this.mainMenu1 = new System.Windows.Forms.MenuStrip();
            this.mnuFile = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemNew = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemDup = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemSave = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemDel = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemUndo = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuExit = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuEdit = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuFieldChoice = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuFieldErase = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuFieldDefault = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuFieldEraseAll = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuFieldDefaultAll = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuRefresh = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuSelection = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemPrev = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemNext = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemFirst = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuItemLast = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuSep1 = new System.Windows.Forms.ToolStripSeparator();
            this.mnuSetFilter = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuResetFilter = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuDirectSearch = new System.Windows.Forms.ToolStripMenuItem();
#if !(SD60 || SD70)
            this.mnuCreateMacro = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuEditMacro = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuDeleteMacro = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuSep3 = new System.Windows.Forms.ToolStripSeparator();
#endif
            this.mnuHelp = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuFieldHelp = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuHelpContent = new System.Windows.Forms.ToolStripMenuItem();
            this.mnuHelpAbout = new System.Windows.Forms.ToolStripMenuItem();
            this.mainStatusBar = new System.Windows.Forms.StatusStrip();
            this.statusBarPanel1 = new System.Windows.Forms.ToolStripStatusLabel();
            this.mnuSep2 = new System.Windows.Forms.ToolStripSeparator();
            this.mainMenu1.SuspendLayout();
            this.mainStatusBar.SuspendLayout();
            this.toolBarRowsetTree = new RepPP.Framework.Window.ToolbarRowsetTree();
            this.SuspendLayout();
            // 
            // mainMenu1
            // 
            this.mainMenu1.BackColor = System.Drawing.SystemColors.Control;
            this.mainMenu1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.mnuFile,
            this.mnuEdit,
            this.mnuSelection,
            this.mnuHelp});
            this.mainMenu1.Location = new System.Drawing.Point(0, 0);
            this.mainMenu1.Name = "mainMenu1";
            this.mainMenu1.Size = new System.Drawing.Size(856, 24);
            this.mainMenu1.TabIndex = 16;
            this.mainMenu1.Text = "mainMenu1";
            // 
            // mnuFile
            // 
            this.mnuFile.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.mnuItemNew,
            this.mnuItemDup,
            this.mnuItemSave,
            this.mnuItemDel,
            this.mnuItemUndo,
            this.mnuExit});
            this.mnuFile.Name = "mnuFile";
            this.mnuFile.Size = new System.Drawing.Size(35, 20);
            this.mnuFile.Text = "&File";
            // 
            // mnuItemNew
            // 
            this.mnuItemNew.Name = "mnuItemNew";
            this.mnuItemNew.Size = new System.Drawing.Size(118, 22);
            this.mnuItemNew.Text = "&New";
            // 
            // mnuItemDup
            // 
            this.mnuItemDup.Name = "mnuItemDup";
            this.mnuItemDup.Size = new System.Drawing.Size(118, 22);
            this.mnuItemDup.Text = "D&uplicate";
            // 
            // mnuItemSave
            // 
            this.mnuItemSave.Name = "mnuItemSave";
            this.mnuItemSave.Size = new System.Drawing.Size(118, 22);
            this.mnuItemSave.Text = "&Save";
            // 
            // mnuItemDel
            // 
            this.mnuItemDel.Name = "mnuItemDel";
            this.mnuItemDel.Size = new System.Drawing.Size(118, 22);
            this.mnuItemDel.Text = "&Delete";
            // 
            // mnuItemUndo
            // 
            this.mnuItemUndo.Name = "mnuItemUndo";
            this.mnuItemUndo.Size = new System.Drawing.Size(118, 22);
            this.mnuItemUndo.Text = "&Undo";
            // 
            // mnuExit
            // 
            this.mnuExit.Name = "mnuExit";
            this.mnuExit.Size = new System.Drawing.Size(118, 22);
            this.mnuExit.Text = "&Exit";
            // 
            // mnuEdit
            // 
            this.mnuEdit.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.mnuFieldChoice,
            this.mnuFieldErase,
            this.mnuFieldDefault,
            this.mnuFieldEraseAll,
            this.mnuFieldDefaultAll,
            this.mnuRefresh});
            this.mnuEdit.Name = "mnuEdit";
            this.mnuEdit.Size = new System.Drawing.Size(37, 20);
            this.mnuEdit.Text = "&Edit";
            // 
            // mnuFieldChoice
            // 
            this.mnuFieldChoice.Name = "mnuFieldChoice";
            this.mnuFieldChoice.Size = new System.Drawing.Size(185, 22);
            this.mnuFieldChoice.Text = "Field &Choices...";
            // 
            // mnuFieldErase
            // 
            this.mnuFieldErase.Name = "mnuFieldErase";
            this.mnuFieldErase.Size = new System.Drawing.Size(185, 22);
            this.mnuFieldErase.Text = "&Erase Current Field";
            // 
            // mnuFieldDefault
            // 
            this.mnuFieldDefault.Name = "mnuFieldDefault";
            this.mnuFieldDefault.Size = new System.Drawing.Size(185, 22);
            this.mnuFieldDefault.Text = "&Set Field to Default";
            // 
            // mnuFieldEraseAll
            // 
            this.mnuFieldEraseAll.Name = "mnuFieldEraseAll";
            this.mnuFieldEraseAll.Size = new System.Drawing.Size(185, 22);
            this.mnuFieldEraseAll.Text = "Erase &All Fields";
            // 
            // mnuFieldDefaultAll
            // 
            this.mnuFieldDefaultAll.Name = "mnuFieldDefaultAll";
            this.mnuFieldDefaultAll.Size = new System.Drawing.Size(185, 22);
            this.mnuFieldDefaultAll.Text = "Set All Fields to &Default";
            // 
            // mnuRefresh
            // 
            this.mnuRefresh.Name = "mnuRefresh";
            this.mnuRefresh.Size = new System.Drawing.Size(185, 22);
            this.mnuRefresh.Text = "&Refresh";
            // 
            // mnuSelection
            // 
            this.mnuSelection.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.mnuItemPrev,
            this.mnuItemNext,
            this.mnuItemFirst,
            this.mnuItemLast,
            this.mnuSep1,
            this.mnuDirectSearch,
            this.mnuSep2,
#if !(SD60 || SD70)
            this.mnuCreateMacro,
            this.mnuEditMacro,
            this.mnuDeleteMacro,
            this.mnuSep3,
#endif
            this.mnuSetFilter,
            this.mnuResetFilter});
            this.mnuSelection.Name = "mnuSelection";
            this.mnuSelection.Size = new System.Drawing.Size(62, 20);
            this.mnuSelection.Text = "&Selection";
            // 
            // mnuItemPrev
            // 
            this.mnuItemPrev.Name = "mnuItemPrev";
            this.mnuItemPrev.Size = new System.Drawing.Size(188, 22);
            this.mnuItemPrev.Text = "&Previous";
            // 
            // mnuItemNext
            // 
            this.mnuItemNext.Name = "mnuItemNext";
            this.mnuItemNext.Size = new System.Drawing.Size(188, 22);
            this.mnuItemNext.Text = "&Next";
            // 
            // mnuItemFirst
            // 
            this.mnuItemFirst.Name = "mnuItemFirst";
            this.mnuItemFirst.Size = new System.Drawing.Size(188, 22);
            this.mnuItemFirst.Text = "&First";
            // 
            // mnuItemLast
            // 
            this.mnuItemLast.Name = "mnuItemLast";
            this.mnuItemLast.Size = new System.Drawing.Size(188, 22);
            this.mnuItemLast.Text = "&Last";
            // 
            // mnuSep1
            // 
            this.mnuSep1.Name = "mnuSep1";
            this.mnuSep1.Size = new System.Drawing.Size(185, 6);
            // 
            // mnuSetFilter
            // 
            this.mnuSetFilter.Name = "mnuSetFilter";
            this.mnuSetFilter.Size = new System.Drawing.Size(188, 22);
            this.mnuSetFilter.Text = "&Set Filter...";
            // 
            // mnuResetFilter
            // 
            this.mnuResetFilter.Name = "mnuResetFilter";
            this.mnuResetFilter.Size = new System.Drawing.Size(188, 22);
            this.mnuResetFilter.Text = "&Reset Filter";
            // 
            // mnuDirectSearch
            // 
            this.mnuDirectSearch.Name = "mnuDirectSearch";
            this.mnuDirectSearch.ShortcutKeys = ((System.Windows.Forms.Keys)((System.Windows.Forms.Keys.Control | System.Windows.Forms.Keys.F)));
            this.mnuDirectSearch.Size = new System.Drawing.Size(188, 22);
            this.mnuDirectSearch.Text = "Direct Search...";
#if !(SD60 || SD70)
            // 
            // mnuCreateMacro
            // 
            this.mnuCreateMacro.Name = "mnuCreateMacro";
            this.mnuCreateMacro.ShortcutKeys = ((System.Windows.Forms.Keys)((System.Windows.Forms.Keys.Control | System.Windows.Forms.Keys.M)));
            this.mnuCreateMacro.Size = new System.Drawing.Size(188, 22);
            this.mnuCreateMacro.Text = "Create Selection Macro...";
            // 
            // mnuEditMacro
            // 
            this.mnuEditMacro.Name = "mnuEditMacro";
            this.mnuEditMacro.ShortcutKeys = ((System.Windows.Forms.Keys)((System.Windows.Forms.Keys.Control | System.Windows.Forms.Keys.Shift | System.Windows.Forms.Keys.E)));
            this.mnuEditMacro.Size = new System.Drawing.Size(188, 22);
            this.mnuEditMacro.Text = "Edit Selection Macro...";
            // 
            // mnuDeleteMacro
            // 
            this.mnuDeleteMacro.Name = "mnuDeleteMacro";
            this.mnuDeleteMacro.ShortcutKeys = ((System.Windows.Forms.Keys)((System.Windows.Forms.Keys.Control | System.Windows.Forms.Keys.Shift | System.Windows.Forms.Keys.M)));
            this.mnuDeleteMacro.Size = new System.Drawing.Size(188, 22);
            this.mnuDeleteMacro.Text = "Delete Selection Macro...";
            // 
            // mnuSep3
            // 
            this.mnuSep3.Name = "mnuSep3";
            this.mnuSep3.Size = new System.Drawing.Size(185, 6);
#endif
            // 
            // mnuSep2
            // 
            this.mnuSep2.Name = "mnuSep2";
            this.mnuSep2.Size = new System.Drawing.Size(185, 6);
            // 
            // mnuHelp
            // 
            this.mnuHelp.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.mnuFieldHelp,
            this.mnuHelpContent,
            this.mnuHelpAbout});
            this.mnuHelp.Name = "mnuHelp";
            this.mnuHelp.Size = new System.Drawing.Size(40, 20);
            this.mnuHelp.Text = "&Help";
            // 
            // mnuFieldHelp
            // 
            this.mnuFieldHelp.Name = "mnuFieldHelp";
            this.mnuFieldHelp.Size = new System.Drawing.Size(125, 22);
            this.mnuFieldHelp.Text = "&Field Help";
            // 
            // mnuHelpContent
            // 
            this.mnuHelpContent.Name = "mnuHelpContent";
            this.mnuHelpContent.Size = new System.Drawing.Size(125, 22);
            this.mnuHelpContent.Text = "&Content...";
            // 
            // mnuHelpAbout
            // 
            this.mnuHelpAbout.Name = "mnuHelpAbout";
            this.mnuHelpAbout.Size = new System.Drawing.Size(125, 22);
            this.mnuHelpAbout.Text = "&About";
            // 
            // mainStatusBar
            // 
            this.mainStatusBar.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.statusBarPanel1});
            this.mainStatusBar.Location = new System.Drawing.Point(0, 560);
            this.mainStatusBar.Name = "mainStatusBar";
            this.mainStatusBar.Size = new System.Drawing.Size(856, 22);
            this.mainStatusBar.TabIndex = 2;
            // 
            // statusBarPanel1
            // 
            this.statusBarPanel1.Name = "statusBarPanel1";
            this.statusBarPanel1.Size = new System.Drawing.Size(0, 17);
            // 
            // toolBarRowsetTree
            // 
            this.toolBarRowsetTree.Location = new System.Drawing.Point(0, 24);
            this.toolBarRowsetTree.Name = "toolBarRowsetTree";
            this.toolBarRowsetTree.RootNodeHandler = null;
            this.toolBarRowsetTree.Size = new System.Drawing.Size(856, 25);
            this.toolBarRowsetTree.TabIndex = 15;
            this.toolBarRowsetTree.ToolBarButtonSize = RepPP.Framework.Window.ToolbarRowsetTree.ToolBarSize.Small;
            // 
            // frmSBEditorUI
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(856, 582);
            this.Controls.Add(this.toolBarRowsetTree);
            this.Controls.Add(this.mainMenu1);
            this.Controls.Add(this.mainStatusBar);
            this.MainMenuStrip = this.mainMenu1;
            this.mainMenu1.ResumeLayout(false);
            this.mainMenu1.PerformLayout();
            this.mainStatusBar.ResumeLayout(false);
            this.mainStatusBar.PerformLayout();
            this.Name = "frmSBEditorUI";
            this.Text = "frmSBEditor";
            this.ResumeLayout(false);
            this.PerformLayout();
        }
        #endregion
    }
}
