using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
	/// <summary>
	/// Summary description for frmSBFilter.
	/// </summary>
	public class frmSBFilter : frmSerializabledForm {
        /// <summary>Panel containing the fields used as filter</summary>
        protected System.Windows.Forms.Panel            panelMain;
        /// <summary>OK Button</summary>
        protected System.Windows.Forms.Button           butFilterOk;
        /// <summary>Cancel Button</summary>
        protected System.Windows.Forms.Button           butFilterCancel;
        /// <summary>And</summary>
        protected System.Windows.Forms.RadioButton      radioAnd;
        /// <summary>Or</summary>
        protected System.Windows.Forms.RadioButton      radioOr;
		
		/// <summary>Required designer variable.</summary>
		private System.ComponentModel.Container         components = null;
		/// <summary>Filter object. This is the object doing the real job</summary>
        protected SBFilter                              m_filter = null;
		/// <summary>true to Auto Size the form so the field are all visible if possible</summary>
        protected bool                                  m_bAutoSizeForm = true;
        /// <summary>Error provider for the form</summary>
        protected FldErrorProvider                      m_errorProvider;
        /// <summary>true if filter must be added, false if it replace it</summary>
        protected bool                                  m_bAddFilter;

        //******************************************************************
	    //
		/// <summary>
		/// ctor:                       Form default constructor. Use by designer
		/// </summary>
	    //
	    //******************************************************************
		public frmSBFilter() {
			InitializeComponent();
			m_filter = null;
		}

	    //******************************************************************
	    //
		/// <summary>
		/// ctor:                           Form constructor.
		/// </summary>
        /// <param name="filter">           Filter</param>
        /// <param name="ctlTranslator">    Control text translator</param>
        /// <param name="configHelper">     ui configuration helper</param>
        /// <param name="bAddFilter">       true to add the filter, false to replace</param>
	    //
	    //******************************************************************
        public frmSBFilter(SBFilter filter, ControlTextTranslator ctlTranslator, UIConfigurationHelper configHelper, bool bAddFilter) : this() {
            RepPP.Rowset rowset;

            m_configHelper  = configHelper;
            m_filter        = filter;
            m_bAddFilter    = bAddFilter;
            rowset          = m_filter.ParamRowset;
            rowset.RowsetDef.SaveActiveRowset();
            rowset.Select();
            radioAnd.Checked  = true;
            m_errorProvider   = new FldErrorProvider(ErrorIconAlignment.MiddleLeft, 2);
            if (ctlTranslator != null) {
                ctlTranslator.TranslateControl(this);
            }
            rowset.Application.FldOnError += new RepPP.FldOnErrorEventHandler(Application_FldOnError);
        }

	    //******************************************************************
	    //
		/// <summary>
		/// Dispose:                    Clean up any resources being used.
		/// </summary>
		/// <param name="bDisposing">   true if dispose, false if GC</param>
	    //
	    //******************************************************************
		protected override void Dispose(bool bDisposing) {
			if (bDisposing)	{
			    if (m_filter != null) {
			        m_filter.DisposeFormResource();
                    m_filter.ParamRowset.Application.FldOnError -= new RepPP.FldOnErrorEventHandler(Application_FldOnError);
                    m_filter.ParamRowset.RowsetDef.RestoreActiveRowset(false);
    		        m_filter = null;
			    }
				if (components != null)	{
					components.Dispose();
				}
			}
			base.Dispose(bDisposing);
		}

	    //******************************************************************
	    //
		/// <summary>
		/// ResizeForm:                 Resize the form to show as much field as possible
		/// </summary>
		//
	    //******************************************************************
        protected virtual void ResizeForm() {
            if (m_filter != null) {
                m_filter.ResizeForm(this, panelMain);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnLoad:                         Load the form state when form is loading
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        protected override void OnLoad(EventArgs e) {
            if (m_filter != null) {
                m_filter.FillPanel(panelMain,
                                   this.Font,
                                   40,
                                   40);
            }
            base.OnLoad(e);
            if (AutoSizeForm) {
                ResizeForm();
            }
        }

	    //******************************************************************
	    //
        /// <summary>
        /// butOK_Click:                Called when the OK button is clicked
        /// </summary>
        /// <param name="sender">       Sender object</param>
        /// <param name="e">            Event argument</param>
        //
	    //******************************************************************
        private void butOK_Click(object sender, System.EventArgs e) {
            m_errorProvider.Reset();
            if (m_filter.IsValid()) {
                if (m_bAddFilter) {
                    m_filter.AddFilter((radioAnd.Checked) ? SBFilter.AndOrE.AND : SBFilter.AndOrE.OR);
                } else {
                    m_filter.SetFilter((radioAnd.Checked) ? SBFilter.AndOrE.AND : SBFilter.AndOrE.OR);
                }
                m_filter.JoinClause = string.Empty;
                DialogResult = DialogResult.OK;
                Close();
            }
        }

	    //******************************************************************
	    //
        /// <summary>
        /// AutoSizeForm:                   true to auto size the form to fit as much field as possible
        /// </summary>
        //
	    //******************************************************************
        public virtual bool AutoSizeForm {
            get {
                return(m_bAutoSizeForm);
            }
            set {
                m_bAutoSizeForm = value;
            }
        }

	    //******************************************************************
	    //
        /// <summary>
        /// Application_FldOnError:     Called when a field is in error
        /// </summary>
        /// <param name="sender">       Sender object</param>
        /// <param name="e">            Event argument</param>
        //
	    //******************************************************************
        protected virtual void Application_FldOnError(object sender, RepPP.FieldGenEventArgs e) {
            m_errorProvider.AddError(this, e.FieldDef, e.ErrorText);
        }

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.panelMain = new System.Windows.Forms.Panel();
            this.butFilterOk = new System.Windows.Forms.Button();
            this.butFilterCancel = new System.Windows.Forms.Button();
            this.radioAnd = new System.Windows.Forms.RadioButton();
            this.radioOr = new System.Windows.Forms.RadioButton();
            this.SuspendLayout();
            // 
            // panelMain
            // 
            this.panelMain.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.panelMain.AutoScroll = true;
            this.panelMain.Location = new System.Drawing.Point(8, 40);
            this.panelMain.Name = "panelMain";
            this.panelMain.Size = new System.Drawing.Size(696, 432);
            this.panelMain.TabIndex = 0;
            // 
            // butFilterOk
            // 
            this.butFilterOk.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.butFilterOk.Location = new System.Drawing.Point(458, 482);
            this.butFilterOk.Name = "butFilterOk";
            this.butFilterOk.Size = new System.Drawing.Size(120, 24);
            this.butFilterOk.TabIndex = 1;
            this.butFilterOk.Text = "Ok";
            this.butFilterOk.Click += new System.EventHandler(this.butOK_Click);
            // 
            // butFilterCancel
            // 
            this.butFilterCancel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.butFilterCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            this.butFilterCancel.Location = new System.Drawing.Point(584, 482);
            this.butFilterCancel.Name = "butFilterCancel";
            this.butFilterCancel.Size = new System.Drawing.Size(120, 24);
            this.butFilterCancel.TabIndex = 2;
            this.butFilterCancel.Text = "Cancel";
            // 
            // radioAnd
            // 
            this.radioAnd.Location = new System.Drawing.Point(8, 12);
            this.radioAnd.Name = "radioAnd";
            this.radioAnd.Size = new System.Drawing.Size(64, 24);
            this.radioAnd.TabIndex = 3;
            this.radioAnd.Text = "And";
            // 
            // radioOr
            // 
            this.radioOr.Location = new System.Drawing.Point(88, 12);
            this.radioOr.Name = "radioOr";
            this.radioOr.Size = new System.Drawing.Size(64, 24);
            this.radioOr.TabIndex = 4;
            this.radioOr.Text = "Or";
            // 
            // frmSBFilter
            // 
            this.AcceptButton = this.butFilterOk;
            this.CancelButton = this.butFilterCancel;
            this.ClientSize = new System.Drawing.Size(712, 518);
            this.Controls.Add(this.radioOr);
            this.Controls.Add(this.radioAnd);
            this.Controls.Add(this.butFilterCancel);
            this.Controls.Add(this.butFilterOk);
            this.Controls.Add(this.panelMain);
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.SizableToolWindow;
            this.MaximizeBox = false;
            this.MinimizeBox = false;
            this.Name = "frmSBFilter";
            this.ShowInTaskbar = false;
            this.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Hide;
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            this.Text = "Set Filter";
            this.ResumeLayout(false);

        }
        #endregion

    }
}
