using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;

namespace RepPP.Framework.Window {
    //******************************************************************        
    //
    /// <summary>
	/// Class implementing a form that allows for the selection and deletion
    /// of macros form any application that uses this framework
	/// </summary>
    //
    //******************************************************************        
    public class frmSelectMacro : Form {
        /// <summary> Object that implements the ISelectionBuffer interface</summary>
        private ISelectionBuffer        m_sb                = null;
        /// <summary> Control text translator </summary>
        private ControlTextTranslator   m_translator        = null;
        /// <summary> True to delete the selected item, False to edit</summary>
        private bool                    m_bSelectForDelete  = false;

        //*********************************************************
        //
        /// <summary>
        /// Class constructor
        /// </summary>
        //  
        //*********************************************************
        public frmSelectMacro() {
            InitializeComponent();
        }

        //*********************************************************
        //
        /// <summary>
        /// Class constructor
        /// </summary>
        /// <param name="sb">               Selection buffer object</param>
        /// <param name="translator">       Control Text Translator</param>
        /// <param name="bSelectForDelete"> True to delete the selected item, False to edit</param>
        //  
        //*********************************************************
        public frmSelectMacro(ISelectionBuffer sb, ControlTextTranslator translator, bool bSelectForDelete):this() {
            ListViewItem lvi;

            m_sb                = sb;
            m_bSelectForDelete  = bSelectForDelete;
            m_translator        = new ControlTextTranslator(m_sb.RowsetTree.Application, 
                                                            new string[]{bSelectForDelete ? "txtFrmSelectMacroForDelete" : "txtFrmSelectMacroForEdit"}, 
                                                            translator);
            Translate();
            foreach(ISearchMethod method in m_sb.SearchMethods) {
                if (method is SearchMacro) {
                    lvi      = new ListViewItem(new string[]{method.Name, method.Description});
                    lvi.Tag  = method;
                    lvMacros.Items.Add(lvi);
                }
            }
            if (!m_bSelectForDelete) {
                lvMacros.MultiSelect  = false;
                lvMacros.CheckBoxes   = false;
                lvMacros.DoubleClick += butOk_Click;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Translates the form
        /// </summary>
        //  
        //*********************************************************
        private void Translate() {
            m_translator.TranslateControl(this);
            colDescription.Text = m_translator.GetItemText("colDescription", "Text");
            colUniqueName.Text  = m_translator.GetItemText("colUniqueName", "Text");
        }

        //*********************************************************
        //
        /// <summary>
        /// Called when the ok button is clicked to delete the selected
        /// macros
        /// </summary>
        /// <param name="sender">   OK Button</param>
        /// <param name="e">        Event argument</param>
        //  
        //*********************************************************
        private void butOk_Click(object sender, EventArgs e) {
            SearchMacro     searchMacro;
            frmEditMacro    frmEdit;

            if (m_bSelectForDelete) {
                foreach(int iIndex in lvMacros.CheckedIndices) {
                    searchMacro = lvMacros.Items[iIndex].Tag as SearchMacro;
                    if (searchMacro != null) {
                        searchMacro.Macro.Delete();
                    }
                }
            } else {
                if (lvMacros.SelectedIndices.Count != 0) {
                    searchMacro = lvMacros.SelectedItems[0].Tag as SearchMacro;
                    if (searchMacro != null) {
                        frmEdit = new frmEditMacro(searchMacro, m_translator);
                        using (frmEdit) {
                            frmEdit.ShowDialog();
                        }
                    }
                    
                }
            }
            DialogResult = DialogResult.OK;
            Close();
        }

        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.IContainer components = null;

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
        protected override void Dispose(bool disposing) {
            if (disposing && (components != null)) {
                components.Dispose();
            }
            base.Dispose(disposing);
        }

		#region Windows Form Designer generated code

		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() 
		{
			System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(frmSelectMacro));
			this.lvMacros = new System.Windows.Forms.ListView();
			this.colUniqueName = new System.Windows.Forms.ColumnHeader();
			this.colDescription = new System.Windows.Forms.ColumnHeader();
			this.butCancel = new System.Windows.Forms.Button();
			this.butOk = new System.Windows.Forms.Button();
			this.lblSelectMacros = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// lvMacros
			// 
			this.lvMacros.CheckBoxes = true;
			this.lvMacros.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																					   this.colUniqueName,
																					   this.colDescription});
			this.lvMacros.FullRowSelect = true;
			this.lvMacros.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.lvMacros.HideSelection = false;
			this.lvMacros.Location = new System.Drawing.Point(12, 29);
			this.lvMacros.Name = "lvMacros";
			this.lvMacros.Size = new System.Drawing.Size(521, 186);
			this.lvMacros.TabIndex = 1;
			this.lvMacros.UseCompatibleStateImageBehavior = false;
			this.lvMacros.View = System.Windows.Forms.View.Details;
			// 
			// colUniqueName
			// 
			this.colUniqueName.Text = "Name";
			this.colUniqueName.Width = 160;
			// 
			// colDescription
			// 
			this.colDescription.Text = "Description";
			this.colDescription.Width = 340;
			// 
			// butCancel
			// 
			this.butCancel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.butCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.butCancel.Location = new System.Drawing.Point(453, 221);
			this.butCancel.Name = "butCancel";
			this.butCancel.Size = new System.Drawing.Size(80, 24);
			this.butCancel.TabIndex = 3;
			this.butCancel.Text = "Cancel";
			// 
			// butOk
			// 
			this.butOk.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.butOk.Location = new System.Drawing.Point(367, 221);
			this.butOk.Name = "butOk";
			this.butOk.Size = new System.Drawing.Size(80, 24);
			this.butOk.TabIndex = 2;
			this.butOk.Text = "Ok";
			this.butOk.Click += new System.EventHandler(this.butOk_Click);
			// 
			// lblSelectMacros
			// 
			this.lblSelectMacros.AutoSize = true;
			this.lblSelectMacros.Location = new System.Drawing.Point(13, 12);
			this.lblSelectMacros.Name = "lblSelectMacros";
			this.lblSelectMacros.Size = new System.Drawing.Size(166, 13);
			this.lblSelectMacros.TabIndex = 0;
			this.lblSelectMacros.Text = "Select the macro(s) to be deleted:";
			// 
			// frmSelectMacro
			// 
			this.AcceptButton = this.butOk;
			this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
			this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
			this.CancelButton = this.butCancel;
			this.ClientSize = new System.Drawing.Size(545, 257);
			this.Controls.Add(this.lblSelectMacros);
			this.Controls.Add(this.butCancel);
			this.Controls.Add(this.butOk);
			this.Controls.Add(this.lvMacros);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "frmSelectMacro";
			this.ShowInTaskbar = false;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "Delete Selection Macro";
			this.ResumeLayout(false);
			this.PerformLayout();

		}

		#endregion

        private System.Windows.Forms.ListView lvMacros;
        private System.Windows.Forms.ColumnHeader colUniqueName;
        private System.Windows.Forms.ColumnHeader colDescription;
        private System.Windows.Forms.Label lblSelectMacros;
        private System.Windows.Forms.Button butCancel;
        private System.Windows.Forms.Button butOk;

    } // Class
} // Namespace