using DevComponents.DotNetBar;
using RepPP.Framework.Window;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace RepPP.Framework.Window.Ext {

    //********************************************************************************
    //
    /// <summary>
    /// Extended Editor (using DotNetBar) using a listview control as a selection buffer.
    /// A selection buffer editor is a container of a selection buffer and
    /// a child  editor controled by the selection buffer.
    /// </summary>
    //
    //********************************************************************************   
    public class frmSBEditorListViewExt : frmSBEditorUIExt {

        /// <summary>Adapt a listview to act as a selection buffer</summary>
        private SBHolderListView                                m_SBHolderListView;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                                       Class constructor
        /// </summary>
        //
        //******************************************************************
        public frmSBEditorListViewExt() {
            InitializeComponent();
        }

        //******************************************************************
        //
        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
        //
        //******************************************************************
        protected override void Dispose(bool disposing) {
            if (disposing) {
                if (m_SBHolderListView != null) {
                    m_SBHolderListView.Dispose();
                    m_SBHolderListView = null;
                }
                if (components != null) {
                    components.Dispose();
                }
            }
            base.Dispose(disposing);
        }

        //******************************************************************
        //
        /// <summary>
        /// ListView:                   Selection buffer listview
        /// </summary>
        //
        //******************************************************************
        public ListViewRowset ListView {
            get {
                return(listView1);
            }
            set {
                if (value != null) {
                    Controls.Remove(listView1);
                    listView1.Dispose();
                    listView1                       = value;
                    listView1.View                  = System.Windows.Forms.View.Details;
                    listView1.AutoMoveAllowed       = false;
                    listView1.AutoRenumField        = null;
                    listView1.AutoRenumIncrement    = 10;
                    listView1.AutoSort              = false;
                    listView1.Dock                  = System.Windows.Forms.DockStyle.Fill;
                    listView1.FullRowSelect         = true;
                    listView1.HideSelection         = false;
                    listView1.IsUserFlagFieldShown  = true;
                    listView1.KeySearch             = true;
                    listView1.Location              = new System.Drawing.Point(0, 424);
                    listView1.MaxColumnSize         = 40;
                    listView1.MinColumnSize         = 5;
                    listView1.MultiSelect           = false;
                    listView1.SelectedItem          = null;
                    listView1.SizeMethod            = RepPP.Toolkit.Window.ListViewBase.SizeMethodE.Auto;
                    Controls.Add(listView1);
                    Controls.SetChildIndex(listView1, 0);
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// InitEditor:                                 Initializing the editor
        /// </summary>
        /// <param name="app">                          Rep++ application object</param>
        /// <param name="configHelper">                 Object keeping the UI configuration</param>
        /// <param name="strTopTranslatorAtomName">     Name of the atom use by the top translator</param>
        /// <param name="strBaseTranslatorAtomName">    Name of the atom use by the base translator. Null for default one.</param>
        //
        //******************************************************************
        protected override void InitEditor(RepPP.Application app, UIConfigurationHelper configHelper, string strTopTranslatorAtomName, string strBaseTranslatorAtomName) {
            base.InitEditor(app, configHelper, strTopTranslatorAtomName, strBaseTranslatorAtomName);
            if (app != null) {
                ChildContainer = panelChild;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// InitSelectionBuffer:                        Initialize the Selection buffer
        /// </summary>
        /// <param name="strSBRowsetTreeName">          Name of the RowsetTree representing the selection buffer</param>
        /// <param name="childEditorFactory">           Factory class use to create the child editors</param>
        //
        //******************************************************************
        protected override void InitSelectionBuffer(string strSBRowsetTreeName, ISBChildEditorFactory childEditorFactory) {
            base.InitSelectionBuffer(strSBRowsetTreeName, childEditorFactory);
            if (Application != null) {
                m_SBHolderListView  = new SBHolderListView(ListView, this, strSBRowsetTreeName);
                SelectionBuffer     = m_SBHolderListView;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnContainerLoad:            Occurs when dotnetbar controls
        ///                             are loaded 
        /// </summary>
        //  
        //*********************************************************
        protected override void OnContainerLoad(object sender, EventArgs e) {
            BaseItem                item;
            Bar                     bar;

            item = (BaseItem)sender;
            if (item.Name == "dockListView") {
                ((DockContainerItem)item).Control = ListView;
                item.Text                         = (Application.IsAtomExist("txtListViewTitle", RepPP.AtomType.sdAtomMsg)) ?
                                                     Application.GetAtomText("txtListViewTitle", RepPP.AtomType.sdAtomMsg) : 
                                                      String.Empty;
                item.Description                  = item.Text;
                if (item.ContainerControl is Bar) {
                    bar      = item.ContainerControl as Bar;
                    bar.Text = item.Text;
                }
            } else if (item.Name == "dockTreeView") {
                bar = item.ContainerControl as Bar;
                if (bar != null) {
                    bar.Hide();
                }
            }
            base.OnContainerLoad(sender, e);
        }

        //******************************************************************
        //
        /// <summary>
        /// RegisterTriggerToChildEditor:   Register the trigger for the child editor commands
        /// </summary>
        /// <param name="nodeHandler">      Framework RowsetTreeDefNode manager</param>
        //
        //******************************************************************
        public override void RegisterTriggerToChildEditor(RowsetTreeDefNodeHandlerRootBase nodeHandler) {
            base.RegisterTriggerToChildEditor(nodeHandler);
            ListView.ContextMenuStrip = CreateSBContextMenu();
        }
        
        #region Windows Form Designer generated code
        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.IContainer components = null;
        /// <summary>Listview containing the selection buffer</summary>
        protected RepPP.Toolkit.Window.ListViewRowset listView1;
        /// <summary>Panel containing the child editor</summary>
        protected System.Windows.Forms.Panel panelChild;

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent() {
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(frmSBEditorListViewExt));
            this.panelChild = new System.Windows.Forms.Panel();
            this.listView1 = new RepPP.Toolkit.Window.ListViewRowset();
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanel1)).BeginInit();
            this.SuspendLayout();
            // 
            // imageList16x16
            // 
            this.imageList16x16.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList16x16.ImageStream")));
            // 
            // imageList32X32
            // 
            this.imageList32X32.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList32X32.ImageStream")));
            // 
            // panelChild
            // 
            this.panelChild.Dock = System.Windows.Forms.DockStyle.Fill;
            this.panelChild.Location = new System.Drawing.Point(202, 48);
            this.panelChild.Name = "panelChild";
            this.panelChild.Size = new System.Drawing.Size(570, 184);
            this.panelChild.TabIndex = 13;
            // 
            // listView1
            // 
            this.listView1.AutoMoveAllowed = false;
            this.listView1.AutoRenumField = null;
            this.listView1.AutoRenumIncrement = 10;
            this.listView1.AutoSort = false;
            this.listView1.DownButton = null;
            this.listView1.FullRowSelect = true;
            this.listView1.Rowset = null;
            this.listView1.RowsetDef = null;
            this.listView1.HideSelection = false;
            this.listView1.IsUserFlagFieldShown = true;
            this.listView1.KeySearch = true;
            this.listView1.Location = new System.Drawing.Point(202, 249);
            this.listView1.MaxColumnSize = 40;
            this.listView1.MinColumnSize = 5;
            this.listView1.MultiSelect = false;
            this.listView1.Name = "listView1";
            this.listView1.SelectedItem = null;
            this.listView1.ShadeColor = System.Drawing.Color.WhiteSmoke;
            this.listView1.Size = new System.Drawing.Size(570, 151);
            this.listView1.SizeMethod = RepPP.Toolkit.Window.ListViewBase.SizeMethodE.Auto;
            this.listView1.TabIndex = 1;
            this.listView1.UpButton = null;
            this.listView1.UseCompatibleStateImageBehavior = false;
            this.listView1.View = System.Windows.Forms.View.Details;
            // 
            // frmSBEditorListView
            // 
            this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
            this.ClientSize = new System.Drawing.Size(772, 424);
            this.Controls.Add(this.panelChild);
            this.Name = "frmSBEditorListView";
            this.Text = "frmSBEditorListView";
            this.Controls.SetChildIndex(this.mainStatusBar, 0);
            this.Controls.SetChildIndex(this.barTopDockSite, 0);
            this.Controls.SetChildIndex(this.barRightDockSite, 0);
            this.Controls.SetChildIndex(this.barLeftDockSite, 0);
            this.Controls.SetChildIndex(this.barBottomDockSite, 0);
            this.Controls.SetChildIndex(this.panelChild, 0);
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanel1)).EndInit();
            this.ResumeLayout(false);

        }

        #endregion

    } // End frmSBEditorListViewExt class
} // End namespace

