using DevComponents.DotNetBar;
using RepPP.Framework.Window;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace RepPP.Framework.Window.Ext {

    //********************************************************************************
    //
    /// <summary>
    /// Extended Editor (using DotNetBar) using a treeview control as a selection buffer.
    /// A selection buffer editor is a container of a selection buffer and
    /// a child  editor controled by the selection buffer.
    /// </summary>
    //
    //********************************************************************************  
    public class frmSBEditorTreeViewExt : frmSBEditorUIExt {

        /// <summary>Adapt a treeview to act as a selection buffer</summary>
        protected SBHolderTreeView                              m_SBHolderTreeView;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                                       Class constructor
        /// </summary>
        //
        //******************************************************************
        public frmSBEditorTreeViewExt() {
            InitializeComponent();
        }

        //******************************************************************
        //
        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="bDisposing">true if managed resources should be disposed; otherwise, false.</param>
        //
        //******************************************************************
        protected override void Dispose(bool bDisposing) {
            if (bDisposing) {
                if (m_SBHolderTreeView != null) {
                    m_SBHolderTreeView.Dispose();
                    m_SBHolderTreeView = null;
                }
                if (components != null) {
                    components.Dispose();
                }
            }
            base.Dispose(bDisposing);
        }

        //******************************************************************
        //
        /// <summary>
        /// InitEditor:                                 Initializing the editor
        /// </summary>
        /// <param name="app">                          Rep++ application object</param>
        /// <param name="configHelper">                 Object keeping the UI configuration</param>
        /// <param name="strTopTranslatorAtomName">     Name of the atom use by the top translator</param>
        /// <param name="strBaseTranslatorAtomName">    Name of the atom use by the base translator Null for default one.</param>
        //
        //******************************************************************
        protected override void InitEditor(RepPP.Application app, UIConfigurationHelper configHelper, string strTopTranslatorAtomName, string strBaseTranslatorAtomName) {
            base.InitEditor(app, configHelper, strTopTranslatorAtomName, strBaseTranslatorAtomName);
            if (app != null) {
                ChildContainer = panelChild;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// TreeView:                   Selection buffer treeview
        /// </summary>
        //
        //******************************************************************
        public TreeViewRowsetTree TreeView {
            get {
                return(treeView1);
            }
            set {
                if (value != null) {
                    Controls.Remove(treeView1);
                    treeView1.Dispose();
                    treeView1               = value;
                    treeView1.Dock          = System.Windows.Forms.DockStyle.Fill;
                    treeView1.HideSelection = false;                    
                    Controls.Add(treeView1);
                    Controls.SetChildIndex(treeView1, 0);
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// InitSelectionBuffer:                        Initialize the Selection buffer
        /// </summary>
        /// <param name="strSBRowsetTreeName">          Name of the RowsetTree representing the selection buffer</param>
        /// <param name="childEditorFactory">           Factory class use to create the child editors</param>
        //
        //******************************************************************
        protected override void InitSelectionBuffer(string strSBRowsetTreeName, ISBChildEditorFactory childEditorFactory) {
            base.InitSelectionBuffer(strSBRowsetTreeName, childEditorFactory);
            if (Application != null) {
                m_SBHolderTreeView  = new SBHolderTreeView(TreeView, this, strSBRowsetTreeName);
                SelectionBuffer     = m_SBHolderTreeView;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnContainerLoad:            Occurs when dotnetbar controls
        ///                             are loaded 
        /// </summary>
        //  
        //*********************************************************
        protected override void OnContainerLoad(object sender, EventArgs e) {
            BaseItem    item;
            Bar         bar;

			item = (BaseItem)sender;
            if (item.Name == "dockTreeView") {
                ((DockContainerItem)item).Control = TreeView;
                item.Text                         = (Application.IsAtomExist("txtTreeViewTitle", RepPP.AtomType.sdAtomMsg)) ? 
                                                      Application.GetAtomText("txtTreeViewTitle", RepPP.AtomType.sdAtomMsg) :
                                                      item.Text = String.Empty;
                item.Description                  = item.Text;
                if (item.ContainerControl is Bar) {
                    bar      = item.ContainerControl as Bar;
                    bar.Text = item.Text;
                }
            } else if (item.Name == "dockListView") {
                item.Description = item.Text;
                if (item.ContainerControl is Bar) {
                    bar         = item.ContainerControl as Bar;
                    bar.Visible = false;
                }
            }
            base.OnContainerLoad(sender, e);
        }

        //******************************************************************
        //
        /// <summary>
        /// RegisterTriggerToChildEditor:   Register the trigger for the child editor commands
        /// </summary>
        /// <param name="nodeHandler">       Framework RowsetTreeDefNode manager</param>
        //
        //******************************************************************
        public override void RegisterTriggerToChildEditor(RowsetTreeDefNodeHandlerRootBase nodeHandler) {
            base.RegisterTriggerToChildEditor(nodeHandler);
            treeView1.ContextMenuStrip = CreateSBContextMenu();
        }

        #region Windows Form Designer generated code
        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.IContainer components = null;
        /// <summary>Treeview containing the selection buffer</summary>
        protected RepPP.Toolkit.Window.TreeViewRowsetTree treeView1;
        /// <summary>Panel containing the child editor</summary>
        protected System.Windows.Forms.Panel panelChild;

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent() {
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(frmSBEditorTreeViewExt));
            this.panelChild = new System.Windows.Forms.Panel();
            this.treeView1 = new RepPP.Toolkit.Window.TreeViewRowsetTree();
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanel1)).BeginInit();
            this.SuspendLayout();
            // 
            // imageList16x16
            // 
            this.imageList16x16.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList16x16.ImageStream")));
            // 
            // imageList32X32
            // 
            this.imageList32X32.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList32X32.ImageStream")));
            // 
            // panelChild
            // 
            this.panelChild.Dock = System.Windows.Forms.DockStyle.Fill;
            this.panelChild.Location = new System.Drawing.Point(202, 48);
            this.panelChild.Name = "panelChild";
            this.panelChild.Size = new System.Drawing.Size(570, 184);
            this.panelChild.TabIndex = 13;
            // 
            // treeView1
            // 
            this.treeView1.ConfigurationHelper = null;
            this.treeView1.HideSelection = false;
            this.treeView1.LineColor = System.Drawing.Color.Empty;
            this.treeView1.Location = new System.Drawing.Point(0, 64);
            this.treeView1.Name = "treeView1";
            this.treeView1.Size = new System.Drawing.Size(202, 336);
            this.treeView1.TabIndex = 21;
            this.treeView1.ToolTip = null;
            // 
            // frmSBEditorTreeViewExt
            // 
            this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
            this.ClientSize = new System.Drawing.Size(772, 424);
            this.Controls.Add(this.panelChild);
            this.Name = "frmSBEditorTreeViewExt";
            this.Text = "frmSBEditorTreeView";
            this.Controls.SetChildIndex(this.mainStatusBar, 0);
            this.Controls.SetChildIndex(this.barTopDockSite, 0);
            this.Controls.SetChildIndex(this.barRightDockSite, 0);
            this.Controls.SetChildIndex(this.barLeftDockSite, 0);
            this.Controls.SetChildIndex(this.barBottomDockSite, 0);
            this.Controls.SetChildIndex(this.panelChild, 0);
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanel1)).EndInit();
            this.ResumeLayout(false);

        }

        #endregion

    } // End frmSBEditorTreeViewExt class
} // End namespace

