using DevComponents.DotNetBar;
using RepPP.Framework.Window;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace RepPP.Framework.Window.Ext {

    //********************************************************************************
    //
    /// <summary>
    /// Extended Editor (using DotNetBar) using a treeview control as a selection buffer and a listview control 
    /// for text node.
    /// A selection buffer editor is a container of a selection buffer and
    /// a child  editor controled by the selection buffer.
    /// </summary>
    //
    //********************************************************************************    
    public class frmSBEditorTreeViewListViewExt : frmSBEditorTreeViewExt {

        /// <summary>Adapt a treeview and listview to act as a selection buffer</summary>
        private SBHolderTreeViewListView m_SBHolderTreeViewListView;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                                       Class constructor
        /// </summary>
        //
        //******************************************************************
        public frmSBEditorTreeViewListViewExt() {
            InitializeComponent();
        }

        //******************************************************************
        //
        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="bDisposing">   true if managed resources should be disposed; otherwise, false.</param>
        //
        //******************************************************************
        protected override void Dispose(bool bDisposing) {
            if (bDisposing) {
                if (m_SBHolderTreeViewListView != null) {
                    m_SBHolderTreeViewListView.Dispose();
                    m_SBHolderTreeViewListView = null;
                }
                if (components != null) {
                    components.Dispose();
                }
            }
            base.Dispose(bDisposing);
        }

        //******************************************************************
        //
        /// <summary>
        /// InitEditor:                                 Initializing the editor
        /// </summary>
        /// <param name="app">                          Rep++ application object</param>
        /// <param name="configHelper">                 Object keeping the UI configuration</param>
        /// <param name="strTopTranslatorAtomName">     Name of the atom use by the top translator</param>
        /// <param name="strBaseTranslatorAtomName">    Name of the atom use by the base translator Null for default one.</param>
        //
        //******************************************************************
        protected override void InitEditor(RepPP.Application app, UIConfigurationHelper configHelper, string strTopTranslatorAtomName, string strBaseTranslatorAtomName) {
            base.InitEditor(app, configHelper, strTopTranslatorAtomName, strBaseTranslatorAtomName);
            if (app != null) {
                ChildContainer = panelChild;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// InitSelectionBuffer:                        Initialize the Selection buffer
        /// </summary>
        /// <param name="strSBRowsetTreeName">          Name of the RowsetTree representing the selection buffer</param>
        /// <param name="childEditorFactory">           Factory class use to create the child editors</param>
        //
        //******************************************************************
        protected override void InitSelectionBuffer(string strSBRowsetTreeName, ISBChildEditorFactory childEditorFactory) {
            base.InitSelectionBuffer(strSBRowsetTreeName, childEditorFactory);
            if (Application != null) {
                // Dispose the selection buffer holder created by the base class
                if (m_SBHolderTreeView != null) {
                    m_SBHolderTreeView.Dispose();
                    m_SBHolderTreeView = null;
                }
                m_SBHolderTreeViewListView  = new SBHolderTreeViewListView(TreeView, ListView, this, strSBRowsetTreeName);
                SelectionBuffer             = m_SBHolderTreeViewListView;
                TreeView.AfterSelect       += new TreeViewEventHandler(AfterNodeSelected);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// AfterNodeSelected:          Called after a node has been selected
        /// </summary>
        /// <param name="sender">       Sender object   </param>
        /// <param name="e">            Event arguments </param>
        //
        //******************************************************************
        protected virtual void AfterNodeSelected(object sender, TreeViewEventArgs e) {
            Bar             barListView;

            if (dotNetBarManagerMain.IsDefinitionLoaded) {
                barListView         = dotNetBarManagerMain.Bars["barListView"];
                barListView.Visible = (e.Node is SBHolderTreeViewListView.TreeNodeListView);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ListView:                   Selection buffer listview
        /// </summary>
        //
        //******************************************************************
        public ListViewRowset ListView {
            get {
                return(listView1);
            }
            set {
                if (value != null) {
                    Controls.Remove(listView1);
                    listView1.Dispose();
                    listView1                       = value;
                    listView1.View                  = System.Windows.Forms.View.Details;
                    listView1.AutoMoveAllowed       = false;
                    listView1.AutoRenumField        = null;
                    listView1.AutoRenumIncrement    = 10;
                    listView1.AutoSort              = false;
                    listView1.Dock                  = System.Windows.Forms.DockStyle.Fill;
                    listView1.FullRowSelect         = true;
                    listView1.HideSelection         = false;
                    listView1.IsUserFlagFieldShown  = true;
                    listView1.KeySearch             = true;
                    listView1.MaxColumnSize         = 40;
                    listView1.MinColumnSize         = 5;
                    listView1.MultiSelect           = false;
                    listView1.SelectedItem          = null;
                    listView1.SizeMethod            = RepPP.Toolkit.Window.ListViewBase.SizeMethodE.Auto;
                    Controls.Add(listView1);
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnContainerLoad:            Occurs when dotnetbar controls
        ///                             are loaded 
        /// </summary>
        //  
        //*********************************************************
         protected override void OnContainerLoad(object sender, System.EventArgs e) {
            BaseItem    item;
            Bar         bar;

	        base.OnContainerLoad(sender, e);
            item = (BaseItem)sender;
            if (item != null) {
                if (item.Name == "dockListView") {
                    ((DockContainerItem)item).Control = ListView;
                    item.Text                         = (Application.IsAtomExist("txtListViewTitle", RepPP.AtomType.sdAtomMsg)) ? 
                                                           Application.GetAtomText("txtListViewTitle", RepPP.AtomType.sdAtomMsg) :
                                                           String.Empty;
                    item.Description                  = item.Text;
                    if (item.ContainerControl is Bar) {
                        bar      = item.ContainerControl as Bar;
                        bar.Text = item.Text;
                    }
                }
            }
        }

        #region Windows Form Designer generated code
        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.IContainer components = null;
        /// <summary>Listview containing the selection buffer</summary>
        protected RepPP.Toolkit.Window.ListViewRowset listView1;

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent() {
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(frmSBEditorTreeViewListViewExt));
            this.listView1 = new RepPP.Toolkit.Window.ListViewRowset();
            this.panelChild.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanel1)).BeginInit();
            this.SuspendLayout();
            // 
            // imageList16x16
            // 
            this.imageList16x16.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList16x16.ImageStream")));
            // 
            // imageList32X32
            // 
            this.imageList32X32.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList32X32.ImageStream")));
            // 
            // listView1
            // 
            this.listView1.AutoMoveAllowed = false;
            this.listView1.AutoRenumField = null;
            this.listView1.AutoRenumIncrement = 10;
            this.listView1.AutoSort = false;
            this.listView1.DownButton = null;
            this.listView1.FullRowSelect = true;
            this.listView1.RowsetDef = null;
            this.listView1.Rowset = null;
            this.listView1.HideSelection = false;
            this.listView1.IsUserFlagFieldShown = true;
            this.listView1.KeySearch = true;
            this.listView1.Location = new System.Drawing.Point(202, 249);
            this.listView1.MaxColumnSize = 40;
            this.listView1.MinColumnSize = 5;
            this.listView1.MultiSelect = false;
            this.listView1.Name = "listView1";
            this.listView1.SelectedItem = null;
            this.listView1.ShadeColor = System.Drawing.Color.WhiteSmoke;
            this.listView1.Size = new System.Drawing.Size(570, 151);
            this.listView1.SizeMethod = RepPP.Toolkit.Window.ListViewBase.SizeMethodE.Auto;
            this.listView1.TabIndex = 1;
            this.listView1.UpButton = null;
            this.listView1.UseCompatibleStateImageBehavior = false;
            this.listView1.View = System.Windows.Forms.View.Details;
            // 
            // frmSBEditorTreeViewListViewExt
            // 
            this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
            this.ClientSize = new System.Drawing.Size(772, 424);
            this.Name = "frmSBEditorTreeViewListViewExt";
            this.Text = "frmSBEditorTreeViewListView";
            this.panelChild.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanel1)).EndInit();
            this.ResumeLayout(false);

        }

        #endregion

    } // End frmSBEditorTreeViewListViewExt class
} // End namespace
