using DevComponents.DotNetBar;
using RepPP.Framework.Window;
using RepPP.Toolkit;
using RepPP.Toolkit.Window;
using RepPP.Toolkit.Window.Ext;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;

namespace RepPP.Framework.Window.Ext {

    //********************************************************************************
    //
    /// <summary>
    /// Implements the base extended interface (using DotNetBar) of an editor holding a selection buffer
    /// </summary>
    //
    //********************************************************************************    
    public class frmSBEditorUIExt : frmSBEditorBase {

        /// <summary>Control displaying error icon on the side of a field</summary>
        private FldErrorProvider            m_errorProvider;
        /// <summary>List that contains GRP.FLD for each direct search field</summary>
        private ArrayList                   m_arrDSFields;
        /// <summary>Macro menu items</summary>
        private ArrayList                   m_arrSearchMethodMenuItems;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                                       Class constructor
        /// </summary>
        //
        //******************************************************************
        public frmSBEditorUIExt() {
            InitializeComponent();
            m_arrDSFields              = new ArrayList();
            m_arrSearchMethodMenuItems = new ArrayList();
        }

        //******************************************************************
        //
        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
        //
        //******************************************************************
        protected override void Dispose(bool disposing) {
            if (disposing && (components != null)) {
                components.Dispose();
            }
            base.Dispose(disposing);
        }

        //******************************************************************
        //
        /// <summary>
        /// ErrorProvider:          The error provider object responsible to display
        ///                         the error icon in the side of the field controls
        /// </summary>
        //
        //******************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public FldErrorProvider ErrorProvider {
            get {
                return(m_errorProvider);
            }
            set {
                m_errorProvider = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// InitEditor:                                 Initializing the editor
        /// </summary>
        /// <param name="app">                          Rep++ application object</param>
        /// <param name="configHelper">                 Object keeping the UI configuration</param>
        /// <param name="strTopTranslatorAtomName">     Name of the atom use by the top translator</param>
        /// <param name="strBaseTranslatorAtomName">    Name of the atom use by the base translator. Null for default one</param>
        //
        //******************************************************************
        protected override void InitEditor(RepPP.Application app, UIConfigurationHelper configHelper, string strTopTranslatorAtomName, string strBaseTranslatorAtomName) {
            base.InitEditor(app,
                            configHelper,
                            strTopTranslatorAtomName,
                            (strBaseTranslatorAtomName != null) ? strBaseTranslatorAtomName : "txtfrmBarBaseSBEditor");
            if (app != null) {
                CommandHolder                              = new SCBarCmdHolder();
                m_errorProvider                            = new FldErrorProvider(ErrorIconAlignment.MiddleLeft, 2);
                Application.FldOnError                    += new RepPP.FldOnErrorEventHandler(OnFldOnError);
                dotNetBarManagerMain.DefinitionLoaded     += new EventHandler(OnDefinitionLoaded);
                dotNetBarManagerMain.ContainerLoadControl += new EventHandler(OnContainerLoad);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// RegisterTriggerToChildEditor:   Register the trigger for the child editor commands
        /// </summary>
        /// <param name="nodeHandler">      Framework RowsetTree manager</param>
        //
        //******************************************************************
        public override void RegisterTriggerToChildEditor(RowsetTreeDefNodeHandlerRootBase nodeHandler) {            
            CmdHolder           cmdHolder;
            Bar                 barMainMenu;
            Bar                 barCard;
            ButtonItem          biItemSave;       
            ButtonItem          biItemNew;        
            ButtonItem          biItemDup;        
            ButtonItem          biItemUndo;       
            ButtonItem          biItemDel;        
            ButtonItem          biFieldChoice;    
            ButtonItem          biFieldHelp;      
            ButtonItem          biFieldErase;     
            ButtonItem          biFieldDefault;   
            ButtonItem          biFieldEraseAll;  
            ButtonItem          biFieldDefaultAll;
            ButtonItem          miFile;
            ButtonItem          miEdit;
            ButtonItem          miSelection;
            ButtonItem          miHelp;
            ButtonItem          miItemSave;
            ButtonItem          miItemUndo;
            ButtonItem          miFieldChoice;
            ButtonItem          miFieldHelp;
            ButtonItem          miFieldErase;
            ButtonItem          miFieldDefault;
            ButtonItem          miFieldEraseAll;
            ButtonItem          miFieldDefaultAll;
            ButtonItem          miItemFirst;
            ButtonItem          miItemPrev;
            ButtonItem          miItemNext;
            ButtonItem          miItemLast;
            ButtonItem          miItemNew;
            ButtonItem          miItemDup;
            ButtonItem          miItemDel;
            ButtonItem          miSetFilter;
            ButtonItem          miResetFilter;
            ButtonItem          miDirectSearch;
            ButtonItem          miMacros;
            
            cmdHolder         = CommandHolder;
            barMainMenu       = dotNetBarManagerMain.Bars["mnuMain"];
            barCard           = dotNetBarManagerMain.Bars["barCard"];
            biItemSave        = (ButtonItem)barCard.Items["ItemSave"];
            biItemNew         = (ButtonItem)barCard.Items["ItemNew"];
            biItemDup         = (ButtonItem)barCard.Items["ItemDup"];
            biItemUndo        = (ButtonItem)barCard.Items["ItemUndo"];
            biItemDel         = (ButtonItem)barCard.Items["ItemDel"];
            biFieldChoice     = (ButtonItem)barCard.Items["FieldChoice"]; 
            biFieldHelp       = (ButtonItem)barCard.Items["FieldHelp"]; 
            biFieldErase      = (ButtonItem)barCard.Items["FieldErase"];
            biFieldDefault    = (ButtonItem)barCard.Items["FieldDefault"];
            biFieldEraseAll   = (ButtonItem)barCard.Items["FieldEraseAll"];
            biFieldDefaultAll = (ButtonItem)barCard.Items["FieldDefaultAll"];
            miFile            = (ButtonItem)barMainMenu.Items["miFile"];
            miEdit            = (ButtonItem)barMainMenu.Items["miEdit"];
            miSelection       = (ButtonItem)barMainMenu.Items["miSelection"];
            miHelp            = (ButtonItem)barMainMenu.Items["miHelp"];
            miItemSave        = (ButtonItem)miFile.SubItems["ItemSave"];
            miItemNew         = (ButtonItem)miFile.SubItems["ItemNew"];
            miItemDup         = (ButtonItem)miFile.SubItems["ItemDup"];
            miItemDel         = (ButtonItem)miFile.SubItems["ItemDel"];
            miItemUndo        = (ButtonItem)miFile.SubItems["ItemUndo"];
            miFieldChoice     = (ButtonItem)miEdit.SubItems["FieldChoice"];
            miFieldErase      = (ButtonItem)miEdit.SubItems["FieldErase"];
            miFieldDefault    = (ButtonItem)miEdit.SubItems["FieldDefault"];
            miFieldEraseAll   = (ButtonItem)miEdit.SubItems["FieldEraseAll"];
            miFieldDefaultAll = (ButtonItem)miEdit.SubItems["FieldDefaultAll"];
            miItemFirst       = (ButtonItem)miSelection.SubItems["ItemFirst"];
            miItemPrev        = (ButtonItem)miSelection.SubItems["ItemPrev"];
            miItemNext        = (ButtonItem)miSelection.SubItems["ItemNext"];
            miItemLast        = (ButtonItem)miSelection.SubItems["ItemLast"];
            miSetFilter       = (ButtonItem)miSelection.SubItems["miSetFilter"];
            miResetFilter     = (ButtonItem)miSelection.SubItems["miResetFilter"];
            miDirectSearch    = (ButtonItem)miSelection.SubItems["miDirectSearch"];
            miMacros          = (ButtonItem)miSelection.SubItems["miMacros"];
            miFieldHelp       = (ButtonItem)miHelp.SubItems["FieldHelp"];
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemSave.ToString(),        new Object[] { biItemSave, miItemSave });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemUndo.ToString(),        new Object[] { biItemUndo, miItemUndo });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldChoice.ToString(),     new Object[] { biFieldChoice, miFieldChoice });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldHelp.ToString(),       new Object[] { biFieldHelp, miFieldHelp });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldErase.ToString(),      new Object[] { biFieldErase, miFieldErase });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldDefault.ToString(),    new Object[] { biFieldDefault, miFieldDefault });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldEraseAll.ToString(),   new Object[] { biFieldEraseAll, miFieldEraseAll });
            cmdHolder.AddTrigger(null, sdHandlerCmd.FieldDefaultAll.ToString(), new Object[] { biFieldDefaultAll, miFieldDefaultAll });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemFirst.ToString(),       new Object[] { miItemFirst });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemPrev.ToString(),        new Object[] { miItemPrev });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemNext.ToString(),        new Object[] { miItemNext });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemLast.ToString(),        new Object[] { miItemLast });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemNew.ToString(),         new Object[] { biItemNew, miItemNew });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemDup.ToString(),         new Object[] { biItemDup, miItemDup });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemDel.ToString(),         new Object[] { biItemDel, miItemDel });
            cmdHolder.AddTrigger(null, "miSetFilter",                           new Object[] { miSetFilter });
            cmdHolder.AddTrigger(null, "miResetFilter",                         new Object[] { miResetFilter });
            cmdHolder.AddTrigger(null, "DirectSearch",                          new Object[] { miDirectSearch });
#if !(SD60 || SD70)
            cmdHolder.AddTrigger(null, "CreateMacro",                           new Object[] { miMacros.SubItems["miCreateMacro"] });
            cmdHolder.AddTrigger(null, "EditMacro",                             new Object[] { miMacros.SubItems["miEditMacro"] });
            cmdHolder.AddTrigger(null, "DeleteMacro",                           new Object[] { miMacros.SubItems["miDeleteMacro"] });
#endif
            if (nodeHandler != null) {
                nodeHandler.ResetFieldError  -= new EventHandler(this.OnResetFieldError);
                nodeHandler.AdjustFieldState -= new EventHandler(this.OnAdjustFieldState);
            }
            nodeHandler.ResetFieldError  += new EventHandler(this.OnResetFieldError);
            nodeHandler.AdjustFieldState += new EventHandler(this.OnAdjustFieldState);
            LoadDirectSearchFields();
            LoadMacros();
            InitMacrosMenu();
        }

        //******************************************************************
        //
        /// <summary>
        /// CreateSBContextMenu:            Creates a context menu
        /// </summary>
        //
        //******************************************************************
        public override ContextMenuStrip CreateSBContextMenu() {
            ContextMenuStrip        ctxMenu;
            ToolStripMenuItem       miSave;
            ToolStripMenuItem       miNew;
            ToolStripMenuItem       miDup;
            ToolStripMenuItem       miDel;
            ToolStripMenuItem       miUndo;
            ToolStripMenuItem       miSetFilter;
            ToolStripMenuItem       miResetFilter;
            ToolStripSeparator      miSep1;
            ToolStripSeparator      miSep2;
            ToolStripSeparator      miSep3;
            ToolStripSeparator      miSep4;
            CmdHolder               cmdHolder;
            ControlTextTranslator   trans;
            
            cmdHolder = CommandHolder;
            trans     = ControlTextTranslator;
            ctxMenu       = new ContextMenuStrip();
            miSave        = new ToolStripMenuItem(trans.GetItemText("ItemSave",      "Text"));
            miNew         = new ToolStripMenuItem(trans.GetItemText("ItemNew",       "Text"));
            miDup         = new ToolStripMenuItem(trans.GetItemText("ItemDup",       "Text"));
            miDel         = new ToolStripMenuItem(trans.GetItemText("ItemDel",       "Text"));
            miUndo        = new ToolStripMenuItem(trans.GetItemText("ItemUndo",      "Text"));
            miSetFilter   = new ToolStripMenuItem(trans.GetItemText("miSetFilter",   "Text"));
            miResetFilter = new ToolStripMenuItem(trans.GetItemText("miResetFilter", "Text"));
            miSep1        = new ToolStripSeparator();
            miSep2        = new ToolStripSeparator();
            miSep3        = new ToolStripSeparator();
            miSep4        = new ToolStripSeparator();
            ctxMenu.Items.AddRange(new ToolStripItem[] { miNew, miSep1, miDup, miUndo, miSep2, miSave, miSep3, miDel, miSep4, miSetFilter, miResetFilter });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemSave.ToString(),        new object[] { miSave });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemNew.ToString(),         new object[] { miNew });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemDup.ToString(),         new object[] { miDup });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemDel.ToString(),         new object[] { miDel });
            cmdHolder.AddTrigger(null, sdHandlerCmd.ItemUndo.ToString(),        new object[] { miUndo });
            cmdHolder.AddTrigger(null, "miSetFilter",                           new object[] { miSetFilter } );
            cmdHolder.AddTrigger(null, "miResetFilter",                         new object[] { miResetFilter } );
            return(ctxMenu);
        }

        //*********************************************************
        //
        /// <summary>
        /// CmdExit:                    Exit the program
        /// </summary>
        //  
        //*********************************************************
        protected virtual void CmdExit() {
            Close();
        }

        //*********************************************************
        //
        /// <summary>
        /// FormRefresh:                Refresh the form
        /// </summary>
        //  
        //*********************************************************
        public virtual void FormRefresh() {
            Refresh(false);
        }

        //******************************************************************
        //
        /// <summary>
        /// RegisterEditorCmd:              Register the editor command
        /// </summary>
        //
        //******************************************************************
        protected virtual void RegisterEditorCmd() {
            CmdHolder   cmdHolder;
            Bar         barMainMenu;
            ButtonItem  biExit;
            ButtonItem  biRefresh;

            cmdHolder           = CommandHolder;
            barMainMenu         = dotNetBarManagerMain.Bars["mnuMain"];
            biExit              = barMainMenu.Items["miFile"].SubItems["miFileExit"] as ButtonItem;
            biRefresh           = barMainMenu.Items["miEdit"].SubItems["FormRefresh"] as ButtonItem;
            cmdHolder.AddCmd(sdHandlerCmd.ItemSaveAll.ToString(), null, CmdHolder.CmdStateE.Hidden);
            cmdHolder.AddCmd(sdHandlerCmd.RowsetTreeSearch.ToString(), null, CmdHolder.CmdStateE.Hidden);
            cmdHolder.AddCmd("Exit",          new CmdHolder.OnCommand(CmdExit), CmdHolder.CmdStateE.Enabled);
            cmdHolder.AddCmd("FormRefresh",   new CmdHolder.OnCommand(FormRefresh), CmdHolder.CmdStateE.Enabled);
            cmdHolder.AddCmd("miSetFilter",   new CmdHolder.OnCommand(SetFilter), CmdHolder.CmdStateE.Enabled);
            cmdHolder.AddCmd("miResetFilter", new CmdHolder.OnCommand(ResetFilter), CmdHolder.CmdStateE.Enabled);
            cmdHolder.AddCmd("DirectSearch",  new CmdHolder.OnCommand(DirectSearch), CmdHolder.CmdStateE.Disabled);
#if !(SD60 || SD70)
            cmdHolder.AddCmd("CreateMacro",   new CmdHolder.OnCommand(CreateMacro), CmdHolder.CmdStateE.Enabled);
            cmdHolder.AddCmd("EditMacro",     new CmdHolder.OnCommand(EditMacro),   CmdHolder.CmdStateE.Enabled);
            cmdHolder.AddCmd("DeleteMacro",   new CmdHolder.OnCommand(DeleteMacro), CmdHolder.CmdStateE.Enabled);
#endif
            cmdHolder.AddTrigger(null, "Exit",        new object[] { biExit });
            cmdHolder.AddTrigger(null, "FormRefresh", new object[] { biRefresh });
        }

        //*********************************************************
        //
        /// <summary>
        /// OnDefinitionLoaded:         Occurs when dotnetbar controls
        ///                             are loaded 
        /// </summary>
        //  
        //*********************************************************
        protected virtual void OnDefinitionLoaded(object sender, EventArgs e) {
            RegisterEditorCmd();
        }

        //*********************************************************
        //
        /// <summary>
        /// OnContainerLoad:            Occurs when a dotnetbar container
        ///                             is loaded 
        /// </summary>
        //  
        //*********************************************************
        protected virtual void OnContainerLoad(object sender, EventArgs e) {
        }

        //*********************************************************
        //
        /// <summary>
        /// OnClosing:                  Check before closing the form
        /// </summary>
        //  
        //*********************************************************
        protected override void OnClosing(CancelEventArgs e) {
            base.OnClosing(e);
            if (!e.Cancel && !AskToSave(false)) {
                e.Cancel = true;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CreateControlTextTranslator:            Creates a control text translator
        /// </summary>
        /// <param name="ctlTxtTranslatorFather">   Parent Translator</param>
        /// <param name="strAtomName">              Atom name of the translator</param>
        /// <returns>
        /// a reference to the created ControlTextTranslator
        /// </returns>
        //
        //******************************************************************
        protected override ControlTextTranslator CreateControlTextTranslator(ControlTextTranslator ctlTxtTranslatorFather, string strAtomName) {
            return(new BarControlTextTranslator(Application, new string[] { strAtomName }, ctlTxtTranslatorFather));
        }

        //******************************************************************
        //
        /// <summary>
        /// OnResetFieldError:      Handle the framework reset field error event
        /// </summary>
        /// <param name="sender">   Event sender</param>
        /// <param name="e">        Event arguments</param>
        // 
        //******************************************************************
        protected virtual void OnResetFieldError(Object sender, EventArgs e) {
            FldErrorProvider        errorProvider;

            errorProvider        = ErrorProvider;
            statusBarPanel1.Text = "";
            errorProvider.Reset();
        }

		//******************************************************************
		//
		/// <summary>
		/// OnFldOnError:           Handle the rep++ field error event
		/// </summary>
		/// <param name="sender">   Event sender</param>
		/// <param name="e">        Event arguments</param>
		// 
		//******************************************************************
		protected virtual void OnFldOnError(Object sender, RepPP.FieldGenEventArgs e) {
			m_errorProvider.Reset();
			m_errorProvider.AddError(this, e.FieldDef, e.ErrorText);
            statusBarPanel1.Text = e.FieldDef.Prompt + ": " + e.ErrorText;
        }

        #region Direct Search and Macros
        //*********************************************************
        //
        /// <summary>
        /// OnExecuteSearchRefresh:       Occurs when Search Execute 
        ///                               has been finish need to be refresh
        /// </summary>
        //  
        //*********************************************************
        protected virtual void OnExecuteSearchRefresh() {
        }

        //******************************************************************
        //
        /// <summary>
        /// OnAdjustFieldState:     Handle the framework adjust field state event
        /// </summary>
        /// <param name="sender">   Event sender</param>
        /// <param name="e">        Event arguments</param>
        // 
        //******************************************************************
        protected virtual void OnAdjustFieldState(Object sender, EventArgs e) {
            bool    bEnable;

            bEnable = IsDirectSearchField(CurrentChildEditor.CurrentFieldDef);
            CommandHolder.SetCmdState("DirectSearch", bEnable ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled);
        }


        //*********************************************************	    
        //
        /// <summary>
        ///	LoadDirectSearchFields:     Loads the list of fields that
        ///                             can be searched directly using
        ///                             the transaction card
        /// </summary>
        //	
        //*********************************************************
        protected virtual void LoadDirectSearchFields() {
            Bar             barDirectSearch;
            ComboBoxItem    cbiDirectSearch;

            barDirectSearch                              = dotNetBarManagerMain.Bars["barDirectSearch"];
            cbiDirectSearch                              = barDirectSearch.Items["miCbxFields"] as ComboBoxItem;
            barDirectSearch.Items["miButSearch"].Enabled = false;
            cbiDirectSearch.Items.Clear();
            m_arrDSFields.Clear();
            LoadDirectSearchFields(CurrentChildEditor.RowsetTreeDef.Nodes);
        }

        //*********************************************************	    
        //
        /// <summary>
        ///	LoadDirectSearchFields:     Loads the list of fields that
        ///                             can be searched directly in a 
        ///                             specific card group
        /// </summary>
	    /// <param name="nodes">        RowsetTreeDefNodes</param>
        //	
        //*********************************************************
        private void LoadDirectSearchFields(RowsetTreeDefNodes nodes) {
            RowsetDef  rowsetDef;

            foreach(RowsetTreeDefNode node in nodes) {
                rowsetDef = node.RowsetDef;
                foreach(FieldDef fldDef in rowsetDef.FieldDefs) {
                    if (IsDirectSearchField(fldDef)) {
                        AddField(fldDef, null);
                    }
                }
                LoadDirectSearchFields(node.Nodes);
            }
        }

        //*********************************************************	    
        //
	    /// <summary>
        /// AddFieldToCombo:            Adds a field name to the list
        ///                             of fields that can be searched
        ///                             to combobox
	    /// </summary>
        /// <param name="strGrpName">   Field Group Name</param>
        /// <param name="strFldName">   Field Name</param>
        /// <param name="strFldDesc">   Current Field Prompt</param>
	    /// <param name="strDefFldDesc">Default Field Prompt</param>
        //	
        //*********************************************************
        protected virtual void AddFieldToCombo(String strGrpName, String strFldName, String strFldDesc, String strDefFldDesc) {
            Bar             barDirectSearch;
            ComboBoxItem    cbiDirectSearch;
            string          strFldPrompt;

            barDirectSearch = dotNetBarManagerMain.Bars["barDirectSearch"];
            cbiDirectSearch = barDirectSearch.Items["miCbxFields"] as ComboBoxItem;
            if (strDefFldDesc != null && strDefFldDesc != string.Empty) {
                strFldPrompt = strDefFldDesc;
            } else if (strFldDesc != string.Empty) {
                strFldPrompt = strFldDesc;
            } else {
                strFldPrompt = "[" + strGrpName + "." + strFldName + "]"; 
            }
            cbiDirectSearch.Items.Add(strFldPrompt);
        }

        //*********************************************************	    
        //
	    /// <summary>
	    /// AddField:                   Adds a field name to the list
        ///                             of fields that can be searched
	    /// </summary>
	    /// <param name="fldDef">       Field Def</param>
	    /// <param name="strFldDesc">   Field Prompt</param>
        //	
        //*********************************************************
        protected virtual void AddField(FieldDef fldDef, string strFldDesc) {
            FieldDef    fldDefAlt;
            string      strRowsetDefName;
            string      strFldDefName;
            string      strFldFullName;

            fldDefAlt = SBFilterBase.GetAlternateFilterField(fldDef);
            if (fldDefAlt != null) {
                fldDef = fldDefAlt;
            }
            strRowsetDefName = fldDef.RowsetDef.Name;
            strFldDefName    = fldDef.Name;
            strFldFullName   = strRowsetDefName + "." + strFldDefName;
            if (!m_arrDSFields.Contains(strFldFullName)) {
                AddFieldToCombo(strRowsetDefName, strFldDefName, RepPP.Toolkit.Util.GetDirectSearchFieldPrompt(fldDef), strFldDesc);
                m_arrDSFields.Add(strFldFullName);
            }

        }

        //*********************************************************	    
        //
	    /// <summary>
        /// dotNetBarManagerMain_GotFocus:   Handle the got focus 
        ///                                  on dotnetbar
        /// </summary>
        /// <param name="sender">            Dotnetbar</param>
	    /// <param name="e">                 Event args</param>
        //	
        //*********************************************************
        private void dotNetBarManagerMain_GotFocus(object sender, EventArgs e) {
            if (CurrentChildEditor != null) {
                CurrentChildEditor.CurrentFieldDef = null;
            }
        }

        //*********************************************************	    
        //
	    /// <summary>
        /// dotNetBarManagerMain_ItemClick:  Handle the click 
        ///                                  on dotnetbar
        /// </summary>
        /// <param name="sender">            Dotnetbar</param>
	    /// <param name="e">                 Event args</param>
        //	
        //*********************************************************
        private void dotNetBarManagerMain_ItemClick(object sender, EventArgs e) {
            Bar             barDirectSearch;
            BaseItem        baseItem;
            bool            bEnable;

            if (sender is BaseItem) {
                baseItem = sender as BaseItem;
                switch(baseItem.Name) {
                case "miCbxFields" :
                    barDirectSearch                              = dotNetBarManagerMain.Bars["barDirectSearch"];
                    bEnable                                      = ((baseItem as ComboBoxItem).SelectedItem != null);
                    barDirectSearch.Items["miButSearch"].Enabled = bEnable;                    
                    break;
                case "miButSearch":
                    OnDirectSearch();
                    break;
                default:
                    //Do Nothing !
                    break;
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// LoadMacros:                  Load Macros for its Editor
        /// </summary>
        //
        //******************************************************************
        private void LoadMacros() {
            ReloadSearchMethods();
#if !(SD60 || SD70)
            RemoveSearchMethodMenuItems();
            AddSearchMethodMenuItems();
#endif
        }

        //******************************************************************
        //
        /// <summary>
        /// InitMacrosMenu:                 Enables/Disables the macro 
        ///                                 manipulation menu items
        /// </summary>
        //
        //******************************************************************
        private void InitMacrosMenu() {
#if !(SD60 || SD70)
            bool        bMacroManipulationAllowed = IsMacroManipulationAllowed();
            string      strSBRowsetTreeDefName    = CurrentChildEditor.RowsetTreeDef.Name;
            Bar         barMainMenu;
            ButtonItem  miSelection;
            ButtonItem  miMacros;
            
            barMainMenu      = dotNetBarManagerMain.Bars["mnuMain"];
            miSelection      = (ButtonItem)dotNetBarManagerMain.Bars["mnuMain"].Items["miSelection"];
            miMacros         = (ButtonItem)miSelection.SubItems["miMacros"];
            miMacros.Visible = bMacroManipulationAllowed;
#endif
        }


#if !(SD60 || SD70)

        //*********************************************************
        //
        /// <summary>
        /// OnExecuteSearch:             executes a search using a macro
        /// </summary>
        //  
        //*********************************************************
        protected override void OnExecuteSearch(object sender, EventArgs e) {
            ISearchMethod   searchMethod;
            int             iNextSearchMethodIndex;
            bool            bContinue = true;

            if (AskToSave(false)) {
                iNextSearchMethodIndex = ((int)((ButtonItem)sender).Tag);
                searchMethod           = SearchMethods[iNextSearchMethodIndex];
                if (searchMethod.HasArguments) {
                    bContinue = GetSearchMethodArgsValues(searchMethod);
                }
                if (bContinue) {
                    NextSearchMethodIndex = iNextSearchMethodIndex;
                    OnExecuteSearchRefresh();
                    Refresh(true);
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ReloadSearchMethods:             Reloads the list of search methods
        /// </summary>
        //
        //******************************************************************
        public override void ReloadSearchMethods() {
            RemoveSearchMethodMenuItems();
            base.ReloadSearchMethods();
            AddSearchMethodMenuItems();
        }

        //******************************************************************
        //
        /// <summary>
        /// SelectionBuffer:                    Selection buffer
        /// </summary>
        //
        //******************************************************************
        public override ISelectionBuffer SelectionBuffer {
            get {
                return base.SelectionBuffer;
            }
            set {
                if (SelectionBuffer != value){
                    RemoveSearchMethodMenuItems();
                    base.SelectionBuffer = value;
                    AddSearchMethodMenuItems();
                }
            }
        }
        //******************************************************************
        //
        /// <summary>
        /// AddSearchMethodMenuItems:   Adds the Search Method menu items to
        ///                             the selection menu
        /// </summary>
        //
        //******************************************************************
        protected virtual void AddSearchMethodMenuItems() {
            Bar                 barMainMenu;
            ButtonItem          mnuSelection;
            ButtonItem          item;
            ISearchMethod       searchMethod;
            string              strMnuText;
            int                 iCount;

            // Do not add the first search method,
            // it is used for direct search
            iCount = SearchMethods.Length;
            if (iCount > 1) {
                barMainMenu  = dotNetBarManagerMain.Bars["mnuMain"];
                if (barMainMenu != null) {
                    mnuSelection = (ButtonItem)barMainMenu.Items["miSelection"];
                    for(int iIndex = 1; iIndex < iCount; iIndex++) {
                        searchMethod = SearchMethods[iIndex];
                        strMnuText   = searchMethod.Name;
                        if (searchMethod.HasArguments) {
                            strMnuText += "...";
                        }
                        item            = new ButtonItem(); 
                        item.Text       = strMnuText;
                        item.Tooltip    = searchMethod.Description ;
                        item.Tag        = iIndex;
                        item.BeginGroup = (iIndex == 1);
                        item.Click     += new EventHandler(OnExecuteSearch);
                        if (searchMethod.AccelerationKey != 0) {
                            try {
                                item.Shortcuts.Add((eShortcut)searchMethod.AccelerationKey);
                            } catch {
                                item.Shortcuts.Clear();
                            }
                        }
                        mnuSelection.SubItems.Add(item);
                        m_arrSearchMethodMenuItems.Add(item);
                    }
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// RemoveSearchMethodMenuItems:    Removes the SearchMethod menu items
        /// </summary>
        //
        //******************************************************************
        protected virtual void RemoveSearchMethodMenuItems() {
            Bar                 barMainMenu;
            ButtonItem          mnuSelection;
            ButtonItem          item;
            int                 iCount;

            iCount = m_arrSearchMethodMenuItems.Count;
            if (iCount > 0) {
                barMainMenu  = dotNetBarManagerMain.Bars["mnuMain"];
                mnuSelection = (ButtonItem)barMainMenu.Items["miSelection"];
                for (int iIndex = 0; iIndex < iCount; iIndex++) {
                    item        = m_arrSearchMethodMenuItems[iIndex] as ButtonItem;
                    item.Click -= new EventHandler(OnExecuteSearch);
                    mnuSelection.SubItems.Remove(item);
                    item.Dispose();
                }
                m_arrSearchMethodMenuItems.Clear();
            }
        }
#endif

        //*********************************************************	    
        //
        /// <summary>
        ///	OnDirectSearch:        Handles the click event of the 
        ///                        Search button
        /// </summary>
        //	
        //*********************************************************	    
        private void OnDirectSearch() {
            Bar                 barDirectSearch;
            ComboBoxItem        miCbxFields;
            TextBoxItem         miTxtFieldValue;
            Application         app;
            FieldDef            fldDef    = null;
            FieldDef            fldDefAlt = null;
            string              strText   = string.Empty;
            string[]            arrVals   = null;

            barDirectSearch = dotNetBarManagerMain.Bars["barDirectSearch"];
            miCbxFields     = barDirectSearch.Items["miCbxFields"] as ComboBoxItem;
            miTxtFieldValue = barDirectSearch.Items["miTxtFieldValue"] as TextBoxItem;
            if (miCbxFields.SelectedItem != null) {
                strText = miTxtFieldValue.ControlText;
                arrVals = ((string)m_arrDSFields[miCbxFields.SelectedIndex]).Split('.');
                if (arrVals != null && arrVals.Length == 2) {
                    app = Application as RepPP.Application;
                    fldDef = app.FieldDef(arrVals[0], arrVals[1]);
                    if (fldDef != null) {
                        fldDefAlt = SBFilterBase.GetAlternateFilterField(fldDef);
                        if (fldDefAlt != null) {
                            fldDef = fldDefAlt;
                        }
                    }
                    ExecuteDirectSearch(fldDef, strText);
                }
            }
        }

        
        #endregion

        #region Windows Form Designer generated code
        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.IContainer components = null;

        /// <summary>ImageList containing All small images</summary>
        protected System.Windows.Forms.ImageList imageList16x16;
        /// <summary>ImageList containing All large images</summary>
        protected System.Windows.Forms.ImageList imageList32X32;
        /// <summary>StatusBarPanel containing the result of editor operation</summary>
        protected System.Windows.Forms.StatusBarPanel statusBarPanel1;
        /// <summary>StatusBar containing the result of editor operation</summary>
        protected System.Windows.Forms.StatusBar mainStatusBar;
        /// <summary>Dotnetbar containing All menus and dockable windows</summary>
        protected DevComponents.DotNetBar.DotNetBarManager dotNetBarManagerMain;
        /// <summary>Dotnetbar Bottom Docking area</summary>
        protected DevComponents.DotNetBar.DockSite barBottomDockSite;
        /// <summary>Dotnetbar Left Docking area</summary>
        protected DevComponents.DotNetBar.DockSite barLeftDockSite;
        /// <summary>Dotnetbar Right Docking area</summary>
        protected DevComponents.DotNetBar.DockSite barRightDockSite;
        /// <summary>Dotnetbar Top Docking area</summary>
        protected DevComponents.DotNetBar.DockSite barTopDockSite;

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent() {
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(frmSBEditorUIExt));
            this.imageList16x16 = new System.Windows.Forms.ImageList(this.components);
            this.imageList32X32 = new System.Windows.Forms.ImageList(this.components);
            this.dotNetBarManagerMain = new DevComponents.DotNetBar.DotNetBarManager(this.components);
            this.barBottomDockSite = new DevComponents.DotNetBar.DockSite();
            this.barLeftDockSite = new DevComponents.DotNetBar.DockSite();
            this.barRightDockSite = new DevComponents.DotNetBar.DockSite();
            this.barTopDockSite = new DevComponents.DotNetBar.DockSite();
            this.mainStatusBar = new System.Windows.Forms.StatusBar();
            this.statusBarPanel1 = new System.Windows.Forms.StatusBarPanel();
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanel1)).BeginInit();
            this.SuspendLayout();
            // 
            // imageList16x16
            // 
            this.imageList16x16.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList16x16.ImageStream")));
            this.imageList16x16.TransparentColor = System.Drawing.Color.Magenta;
            this.imageList16x16.Images.SetKeyName(0, "");
            this.imageList16x16.Images.SetKeyName(1, "");
            this.imageList16x16.Images.SetKeyName(2, "");
            this.imageList16x16.Images.SetKeyName(3, "");
            this.imageList16x16.Images.SetKeyName(4, "");
            this.imageList16x16.Images.SetKeyName(5, "");
            this.imageList16x16.Images.SetKeyName(6, "");
            this.imageList16x16.Images.SetKeyName(7, "");
            this.imageList16x16.Images.SetKeyName(8, "");
            this.imageList16x16.Images.SetKeyName(9, "");
            this.imageList16x16.Images.SetKeyName(10, "");
            this.imageList16x16.Images.SetKeyName(11, "");
            this.imageList16x16.Images.SetKeyName(12, "");
            this.imageList16x16.Images.SetKeyName(13, "");
            this.imageList16x16.Images.SetKeyName(14, "");
            this.imageList16x16.Images.SetKeyName(15, "");
            this.imageList16x16.Images.SetKeyName(16, "");
            this.imageList16x16.Images.SetKeyName(17, "");
            this.imageList16x16.Images.SetKeyName(18, "");
            this.imageList16x16.Images.SetKeyName(19, "");
            this.imageList16x16.Images.SetKeyName(20, "");
            this.imageList16x16.Images.SetKeyName(21, "");
            this.imageList16x16.Images.SetKeyName(22, "");
            this.imageList16x16.Images.SetKeyName(23, "");
            this.imageList16x16.Images.SetKeyName(24, "");
            this.imageList16x16.Images.SetKeyName(25, "");
            this.imageList16x16.Images.SetKeyName(26, "");
            this.imageList16x16.Images.SetKeyName(27, "");
            this.imageList16x16.Images.SetKeyName(28, "");
            this.imageList16x16.Images.SetKeyName(29, "");
            this.imageList16x16.Images.SetKeyName(30, "");
            this.imageList16x16.Images.SetKeyName(31, "");
            this.imageList16x16.Images.SetKeyName(32, "");
            this.imageList16x16.Images.SetKeyName(33, "");
            this.imageList16x16.Images.SetKeyName(34, "");
            this.imageList16x16.Images.SetKeyName(35, "");
            this.imageList16x16.Images.SetKeyName(36, "");
            this.imageList16x16.Images.SetKeyName(37, "");
            this.imageList16x16.Images.SetKeyName(38, "");
            this.imageList16x16.Images.SetKeyName(39, "");
            this.imageList16x16.Images.SetKeyName(40, "");
            this.imageList16x16.Images.SetKeyName(41, "");
            this.imageList16x16.Images.SetKeyName(42, "");
            this.imageList16x16.Images.SetKeyName(43, "");
            this.imageList16x16.Images.SetKeyName(44, "");
            this.imageList16x16.Images.SetKeyName(45, "");
            // 
            // imageList32X32
            // 
            this.imageList32X32.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList32X32.ImageStream")));
            this.imageList32X32.TransparentColor = System.Drawing.Color.Magenta;
            this.imageList32X32.Images.SetKeyName(0, "");
            this.imageList32X32.Images.SetKeyName(1, "");
            this.imageList32X32.Images.SetKeyName(2, "");
            this.imageList32X32.Images.SetKeyName(3, "");
            this.imageList32X32.Images.SetKeyName(4, "");
            this.imageList32X32.Images.SetKeyName(5, "");
            this.imageList32X32.Images.SetKeyName(6, "");
            this.imageList32X32.Images.SetKeyName(7, "");
            this.imageList32X32.Images.SetKeyName(8, "");
            this.imageList32X32.Images.SetKeyName(9, "");
            this.imageList32X32.Images.SetKeyName(10, "");
            this.imageList32X32.Images.SetKeyName(11, "");
            this.imageList32X32.Images.SetKeyName(12, "");
            this.imageList32X32.Images.SetKeyName(13, "");
            this.imageList32X32.Images.SetKeyName(14, "");
            this.imageList32X32.Images.SetKeyName(15, "");
            this.imageList32X32.Images.SetKeyName(16, "");
            this.imageList32X32.Images.SetKeyName(17, "");
            this.imageList32X32.Images.SetKeyName(18, "");
            this.imageList32X32.Images.SetKeyName(19, "");
            this.imageList32X32.Images.SetKeyName(20, "");
            this.imageList32X32.Images.SetKeyName(21, "");
            this.imageList32X32.Images.SetKeyName(22, "");
            this.imageList32X32.Images.SetKeyName(23, "");
            this.imageList32X32.Images.SetKeyName(24, "");
            this.imageList32X32.Images.SetKeyName(25, "");
            this.imageList32X32.Images.SetKeyName(26, "");
            this.imageList32X32.Images.SetKeyName(27, "");
            this.imageList32X32.Images.SetKeyName(28, "");
            this.imageList32X32.Images.SetKeyName(29, "");
            this.imageList32X32.Images.SetKeyName(30, "");
            this.imageList32X32.Images.SetKeyName(31, "");
            this.imageList32X32.Images.SetKeyName(32, "");
            this.imageList32X32.Images.SetKeyName(33, "");
            this.imageList32X32.Images.SetKeyName(34, "");
            this.imageList32X32.Images.SetKeyName(35, "");
            this.imageList32X32.Images.SetKeyName(36, "");
            this.imageList32X32.Images.SetKeyName(37, "");
            this.imageList32X32.Images.SetKeyName(38, "");
            this.imageList32X32.Images.SetKeyName(39, "");
            this.imageList32X32.Images.SetKeyName(40, "");
            this.imageList32X32.Images.SetKeyName(41, "");
            this.imageList32X32.Images.SetKeyName(42, "");
            this.imageList32X32.Images.SetKeyName(43, "");
            this.imageList32X32.Images.SetKeyName(44, "");
            this.imageList32X32.Images.SetKeyName(45, "");
            // 
            // dotNetBarManagerMain
            // 
            this.dotNetBarManagerMain.BottomDockSite = this.barBottomDockSite;
            this.dotNetBarManagerMain.DefinitionName = "frmSBEditorUIExt.dotNetBarManagerMain.xml";
            this.dotNetBarManagerMain.Images = this.imageList32X32;
            this.dotNetBarManagerMain.ImagesLarge = this.imageList32X32;
            this.dotNetBarManagerMain.ImagesMedium = this.imageList16x16;
            this.dotNetBarManagerMain.LeftDockSite = this.barLeftDockSite;
            this.dotNetBarManagerMain.ParentForm = this;
            this.dotNetBarManagerMain.RightDockSite = this.barRightDockSite;
            this.dotNetBarManagerMain.TopDockSite = this.barTopDockSite;
            this.dotNetBarManagerMain.ItemClick += new System.EventHandler(this.dotNetBarManagerMain_ItemClick);
            this.dotNetBarManagerMain.GotFocus += new System.EventHandler(this.dotNetBarManagerMain_GotFocus);
            // 
            // barBottomDockSite
            // 
            this.barBottomDockSite.AccessibleRole = System.Windows.Forms.AccessibleRole.Window;
            this.barBottomDockSite.BackgroundImageAlpha = ((byte)(255));
            this.barBottomDockSite.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.barBottomDockSite.Location = new System.Drawing.Point(202, 232);
            this.barBottomDockSite.Name = "barBottomDockSite";
            this.barBottomDockSite.Size = new System.Drawing.Size(570, 168);
            this.barBottomDockSite.TabIndex = 3;
            this.barBottomDockSite.TabStop = false;
            // 
            // barLeftDockSite
            // 
            this.barLeftDockSite.AccessibleRole = System.Windows.Forms.AccessibleRole.Window;
            this.barLeftDockSite.BackgroundImageAlpha = ((byte)(255));
            this.barLeftDockSite.Dock = System.Windows.Forms.DockStyle.Left;
            this.barLeftDockSite.Location = new System.Drawing.Point(0, 48);
            this.barLeftDockSite.Name = "barLeftDockSite";
            this.barLeftDockSite.Size = new System.Drawing.Size(202, 352);
            this.barLeftDockSite.TabIndex = 0;
            this.barLeftDockSite.TabStop = false;
            // 
            // barRightDockSite
            // 
            this.barRightDockSite.AccessibleRole = System.Windows.Forms.AccessibleRole.Window;
            this.barRightDockSite.BackgroundImageAlpha = ((byte)(255));
            this.barRightDockSite.Dock = System.Windows.Forms.DockStyle.Right;
            this.barRightDockSite.Location = new System.Drawing.Point(772, 48);
            this.barRightDockSite.Name = "barRightDockSite";
            this.barRightDockSite.Size = new System.Drawing.Size(0, 352);
            this.barRightDockSite.TabIndex = 1;
            this.barRightDockSite.TabStop = false;
            // 
            // barTopDockSite
            // 
            this.barTopDockSite.AccessibleRole = System.Windows.Forms.AccessibleRole.Window;
            this.barTopDockSite.BackgroundImageAlpha = ((byte)(255));
            this.barTopDockSite.Dock = System.Windows.Forms.DockStyle.Top;
            this.barTopDockSite.Location = new System.Drawing.Point(0, 0);
            this.barTopDockSite.Name = "barTopDockSite";
            this.barTopDockSite.Size = new System.Drawing.Size(772, 48);
            this.barTopDockSite.TabIndex = 2;
            this.barTopDockSite.TabStop = false;
            // 
            // mainStatusBar
            // 
            this.mainStatusBar.Location = new System.Drawing.Point(0, 400);
            this.mainStatusBar.Name = "mainStatusBar";
            this.mainStatusBar.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
            this.statusBarPanel1});
            this.mainStatusBar.ShowPanels = true;
            this.mainStatusBar.Size = new System.Drawing.Size(772, 24);
            this.mainStatusBar.TabIndex = 14;
            this.mainStatusBar.Text = "statusBar1";
            // 
            // statusBarPanel1
            // 
            this.statusBarPanel1.Name = "statusBarPanel1";
            this.statusBarPanel1.Width = 900;
            // 
            // frmSBEditorUIExt
            // 
            this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
            this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
            this.ClientSize = new System.Drawing.Size(772, 424);
            this.Controls.Add(this.barBottomDockSite);
            this.Controls.Add(this.barLeftDockSite);
            this.Controls.Add(this.barRightDockSite);
            this.Controls.Add(this.barTopDockSite);
            this.Controls.Add(this.mainStatusBar);
            this.Name = "frmSBEditorUIExt";
            this.Text = "frmSBEditorUI";
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanel1)).EndInit();
            this.ResumeLayout(false);

        }

        #endregion

    } // End frmSBEditorUIExt class
} // End namespace