using System;
using System.ComponentModel;
using RepPP;
using RepPP.Toolkit.Web;

namespace RepPP.Framework.Web {

    //********************************************************************************
    //
	/// <summary>
	/// DataAccess:         Implements a data access
	/// </summary>
    //
    //********************************************************************************
	public class DataAccess	: IDataAccess {
        /// <summary>Father page </summary>
        private PageEditor                  m_editorPage;
        /// <summary>Father page </summary>
        private PageWithSelectionBuffer     m_sbPage;

        //********************************************************************************
        //
	    /// <summary>
	    /// ctor:                       Class constructor
	    /// </summary>
	    /// <param name="page">         Page containing this object</param>
	    //
        //********************************************************************************
		public DataAccess(PageWithSelectionBuffer page) {
            m_editorPage = page as PageEditor;
            m_sbPage     = page;
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// Dispose:                    Dispose the object
	    /// </summary>
	    //
        //********************************************************************************
        public void Dispose() {
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// Initialize:                     Performs any required Initialization
	    /// </summary>
	    /// <param name="rowsetTree">            Card instance used to read the data</param>
	    //
        //********************************************************************************
        public void Initialize(RowsetTree rowsetTree) {
            int                     iRetVal;
            RepPP.RowsetTreeDef          rowsetTreeDef;

            rowsetTreeDef = rowsetTree.RowsetTreeDef;
            if (rowsetTreeDef.Nodes[0].SelectSqlCommand == String.Empty) {
                iRetVal = rowsetTreeDef.BuildSqlCommand();
                if (iRetVal != 0) {
                    throw new FWException(FWException.EditorExceptionE.CannotBuildSqlCmds, 
                                            " Faulty Card: " + rowsetTreeDef.Name +
                                            " Return Code: " + iRetVal);
                }
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// LoadSelectionBufferData:        Load the selection buffer data from the database
	    /// </summary>
	    /// <param name="rowsetTree">       Rowset tree used to return the data</param>
	    /// <param name="strWhereClause">   Where Clause</param>
	    /// <returns>
	    /// Error code
	    /// </returns>
	    //
        //********************************************************************************
        public int LoadSelectionBufferData(RowsetTree rowsetTree, string strWhereClause) {
            int                 iRetVal;
            RepPP.RowsetTreeDef rowsetTreeDef;
            CancelEventArgs     eventArgs;

		    eventArgs = new CancelEventArgs(false);
            if (m_sbPage != null) {
                m_sbPage.OnBeforeLoadSelectionBufferFromDB(eventArgs);
            }
		    if (eventArgs.Cancel) {
		        iRetVal = int.MinValue;
		    } else {
                rowsetTreeDef = rowsetTree.RowsetTreeDef;
                if (rowsetTreeDef.Nodes[0].SelectSqlCommand == String.Empty) {
                    iRetVal = rowsetTreeDef.BuildSqlCommand();
                    if (iRetVal != 0) {
                        throw new FWException(FWException.EditorExceptionE.CannotBuildSqlCmds, 
                                              " Faulty Card: " + rowsetTreeDef.Name +
                                              " Return Code: " + iRetVal);
                    }
                }
                rowsetTreeDef.Nodes[0].SetSelectSqlWhere(strWhereClause);
                iRetVal = rowsetTree.ReadFromDb();
		        if (iRetVal == 0 && m_sbPage != null) {
		            m_sbPage.OnAfterLoadSelectionBufferFromDB(eventArgs);
		            if (eventArgs.Cancel) {
		                iRetVal = int.MinValue;
		            }
		        }
            }
            return(iRetVal);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// LoadTransactionData:            Load the transaction data from the database
	    /// </summary>
	    /// <param name="rowsetTree">       Rowset tree used to return the data</param>
	    /// <param name="arrKeyValues">     Array of key values</param>
	    /// <returns>
	    /// Error code
	    /// </returns>
	    //
        //********************************************************************************
        public int LoadTransactionData(RowsetTree rowsetTree, string[] arrKeyValues) {
            int                 iRetVal;
            CancelEventArgs     eventArgs;

		    eventArgs = new CancelEventArgs(false);
            if (m_editorPage != null) {
                m_editorPage.OnBeforeLoadFromDB(eventArgs);
            }
		    if (eventArgs.Cancel) {
		        iRetVal = int.MinValue;
		    } else {
                Initialize(rowsetTree);
                iRetVal = RepPP.Toolkit.Util.ReadItemUsingPK(rowsetTree, arrKeyValues, false);
		        if (iRetVal == 0 && m_editorPage != null) {
                    m_editorPage.OnAfterLoadFromDB(eventArgs);
		            if (eventArgs.Cancel) {
		                iRetVal = int.MinValue;
		            }

		        }
            }
            return(iRetVal);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// SaveTransactionData:            Save data to the database
	    /// </summary>
	    /// <param name="rowsetTree">            Card instance containing the data</param>
	    /// <param name="strDBErrMessage">  The database error message</param>
	    /// <returns>
	    /// Error code
	    /// </returns>
	    //
        //********************************************************************************
        public int SaveTransactionData(RowsetTree rowsetTree, out string strDBErrMessage) {
            int                 iRetVal;
            CancelEventArgs     eventArgs;

            strDBErrMessage = String.Empty;
		    eventArgs = new CancelEventArgs(false);
		    if (m_editorPage != null) {
                m_editorPage.OnBeforeSaveToDB(eventArgs);
            }
		    if (eventArgs.Cancel) {
		        iRetVal = int.MinValue;
		    } else {
                Initialize(rowsetTree);
                iRetVal = rowsetTree.UpdateToDb(false);
		        if (iRetVal == 0) {
                    if (m_editorPage != null) {
		                m_editorPage.OnAfterSaveToDB(eventArgs);
		                if (eventArgs.Cancel) {
		                    iRetVal = int.MinValue;
                        }
                    }
		        } else {
                    strDBErrMessage = rowsetTree.Application.Tool.DBErrorMessage;
                }
                if (iRetVal == 0) {
                    rowsetTree.Application.DataConnection.Commit();
                } else { 
                    rowsetTree.Application.DataConnection.Rollback();
                }
            }
            return(iRetVal);
        }
    } // class DataAccess
} // namespace
