using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using RepPP;

namespace RepPP.Framework.Web {

    //********************************************************************************
    //
	/// <summary>
	/// PageBaseEditor:     Implement base functionnality for pages implementing an editor
	/// </summary>
    //
    //********************************************************************************
	public class FWException : Exception {
	    /// <summary>Category of the exception</summary>
	    public enum EditorExceptionCategoryE {
	        /// <summary>Command Syntax</summary>
	        CmdSyntax                   =   0,
	        /// <summary>Out of Sequence</summary>
	        Sequence                    =   1,
	        /// <summary>Framework error</summary>
	        Framework                   =   2,
	        /// <summary>Database error</summary>
	        Database                    =   3,
	        /// <summary>Unknow error</summary>
            Unknown                     =   4,
	    }
	    
	    /// <summary>Exception code</summary>
	    public enum EditorExceptionE {
	        /// <summary>Specified primary key is not found</summary>
	        KeyNotFound                      =   1,
	        /// <summary>Number of keys doesn't work</summary>
	        KeyMismatch                      =   2,
	        /// <summary>Received command is out of sequence</summary>
	        OutOfSequenceCmd                 =   3,
	        /// <summary>One or more required parameters have invalid values</summary>
	        BadParameter                     =   4,
	        /// <summary>Primary has changed. Was not supposed</summary>
	        PKChanged                        =   5,
	        /// <summary>A Rep++ object was not previously released</summary>
	        RepPPAlreadyInitialized          =   6,
	        /// <summary>Rep++ must be started using the Resource Dispenser. Change the connection setting.</summary>
	        RepPPRequiresResourceDispencer   =   7,
            /// <summary>Cannot get the current transaction entry from the navigation stack</summary>
            CannotFindTransactionEntry       =   8,
            /// <summary>An operation that requires a selection buffer cannot find one</summary>
            CannotFindSB                     =   9,
            /// <summary>Cannot get the transaction rowset tree from the navigation stack</summary>
            CannotFindTransRowsetTree       =   10,
            /// <summary>Invalid command</summary>
            InvalidCommand                   =   11,
            /// <summary>Invalid stack operation</summary>
            InvalidStackOperation            =   12,
            /// <summary>Invalid edit position, valid values are FIRST, LAST, PREVIOUS OR NEXT</summary>
            InvalidPosition                  =   13,
            /// <summary>Cannot navigate through an empty rowset</summary>
            CannotNavigateEmptyRowset =   14,
            /// <summary>The navigation stack is empty</summary>
            NavStackEmpty                    =   15,
            /// <summary>Only root transaction editors can add transactions to the navigation stack</summary>
            NotAllowedToCreateTransaction    =   16,
            /// <summary>We apologize for this error. please report this message to Consyst-SQL's Technical Support</summary>
            DevError                         =   17,
            /// <summary>Rowset tree definition not found</summary>
            RowsetTreeNotFound                     =   18,
            /// <summary>Rowset defniition not found</summary>
            RowsetNotFound                    =   19,
            /// <summary>Invalid usage of the command None</summary>
            InvalidCmd_None                  =   20,
            /// <summary>Invalid usage of the command Edit</summary>
            InvalidCmd_Edit                  =   21,
            /// <summary>Invalid usage of the command New</summary>
            InvalidCmd_New                   =   22,
            /// <summary>Invalid usage of the command Delete</summary>
            InvalidCmd_Delete                =   23,
            /// <summary>Invalid usage of the command Duplicate</summary>
            InvalidCmd_Duplicate             =   24,
            /// <summary>Invalid usage of the command Save</summary>
            InvalidCmd_Save                  =   25,
            /// <summary>Invalid usage of the command DeleteStack</summary>
            InvalidCmd_DeleteStack           =   26,
            /// <summary>One or more required parameters are missing</summary>
            MissingParam                     =   27,
            /// <summary>Cannot load the transaction data. May be caused by a major change in the repository</summary>
            CannotLoadData                   =   28,
            /// <summary>Your session timed out!</summary>
            SessionTimeOut                   =   29,
            /// <summary>There was an attempt to save a sorted selection buffer to the common cache</summary>
            NotAllowedToSaveSortedSB         =   30,
            /// <summary>Cannot build SQL Commands. May be due to a missing PK field in a Group in your Card</summary>
            CannotBuildSqlCmds               =   31,
            /// <summary>Cannot build SQL Commands. May be due to a missing PK field in a Group in your Card</summary>
            WebCodeGenRequired               =   32,
	    };
	    /// <summary>Code describing the exception</summary>
	    public EditorExceptionE             Code;
	    /// <summary>Category of the exception</summary>
        public EditorExceptionCategoryE     Category;

        //********************************************************************************
        //
	    /// <summary>
	    /// ctor:                   Class constructor
	    /// </summary>
	    /// <param name="eCode">    Exception code</param>
	    /// <param name="strMsg">   Message of the exception</param>
	    //
        //********************************************************************************
	    public FWException(EditorExceptionE eCode, string strMsg) : base(strMsg) {
	        Code = eCode;
            Source        = "Rep++ Framework for ASP.NET";
            switch (eCode) {
	            case EditorExceptionE.OutOfSequenceCmd:
                    Category = EditorExceptionCategoryE.Sequence;
                    break;
	            case EditorExceptionE.KeyNotFound:
	            case EditorExceptionE.KeyMismatch:
	            case EditorExceptionE.BadParameter:
	            case EditorExceptionE.PKChanged:
	            case EditorExceptionE.RepPPAlreadyInitialized:
                case EditorExceptionE.RepPPRequiresResourceDispencer:
                case EditorExceptionE.CannotFindTransactionEntry:
                case EditorExceptionE.CannotFindSB:
                case EditorExceptionE.CannotFindTransRowsetTree:
                case EditorExceptionE.InvalidPosition:
                case EditorExceptionE.CannotNavigateEmptyRowset:
                case EditorExceptionE.NavStackEmpty:
                case EditorExceptionE.NotAllowedToCreateTransaction:
                case EditorExceptionE.DevError:
                case EditorExceptionE.RowsetTreeNotFound:
                case EditorExceptionE.RowsetNotFound:
                case EditorExceptionE.MissingParam:
                case EditorExceptionE.CannotLoadData:
                case EditorExceptionE.SessionTimeOut:
                case EditorExceptionE.NotAllowedToSaveSortedSB:
                case EditorExceptionE.CannotBuildSqlCmds:
                case EditorExceptionE.WebCodeGenRequired:
                    Category = EditorExceptionCategoryE.Framework;
                    break;
                case EditorExceptionE.InvalidCommand:
                case EditorExceptionE.InvalidStackOperation:
                case EditorExceptionE.InvalidCmd_None:
                case EditorExceptionE.InvalidCmd_Edit:
                case EditorExceptionE.InvalidCmd_New:
                case EditorExceptionE.InvalidCmd_Delete:
                case EditorExceptionE.InvalidCmd_Duplicate:
                case EditorExceptionE.InvalidCmd_Save:
                case EditorExceptionE.InvalidCmd_DeleteStack:
                    Category = EditorExceptionCategoryE.CmdSyntax;
                    break;
                default:
                    Category = EditorExceptionCategoryE.Unknown;
                    break;
            }
	    }

        //********************************************************************************
        //
	    /// <summary>
	    /// ctor:                   Class constructor
	    /// </summary>
	    /// <param name="eCode">    Exception code</param>
	    //
        //********************************************************************************
	    public FWException(EditorExceptionE eCode) : this(eCode, string.Empty) {
	    }

        //********************************************************************************
        //
	    /// <summary>
	    /// Message:                Message of the exception
	    /// </summary>
	    //
        //********************************************************************************
        public override string Message {
            get {
                string  strRetVal       = null;
                string  strBaseMessage;
                
                switch(Code) {
                case EditorExceptionE.OutOfSequenceCmd:
                    strRetVal = "The received command is out of sequence.";
                    break;
                case EditorExceptionE.PKChanged:
                    strRetVal = "The primary key has been changed.";
                    break;
	            case EditorExceptionE.RepPPAlreadyInitialized:
	                strRetVal = "A Rep++ application object is already initialized for this thread. It means an previous application on this process didn't dispose it correctly. It can also mean someone initialize the Rep++ object before the page.";
	                break;
	            case EditorExceptionE.RepPPRequiresResourceDispencer:
	                strRetVal = "Rep++ must be started using the Resource Dispenser. Change the connection setting.";
	                break;
                case EditorExceptionE.KeyNotFound   :
                    strRetVal = "Specified primary key is not found (Check that the specified key is correct. If you are using a selection buffer, make sure that the groups of the selection buffer and transaction have the same key)";
                    break;
                case EditorExceptionE.KeyMismatch   :
                    strRetVal = "Number of keys doesn't work";
                    break;
                case EditorExceptionE.BadParameter  :
                    strRetVal = "One or more required parameters have invalid values";
                    break;
                case EditorExceptionE.CannotFindTransactionEntry:
                    strRetVal = "Cannot get the current transaction entry from the navigation stack";
                    break;
                case EditorExceptionE.CannotFindSB:
                    strRetVal = "An operation that requires a selection buffer cannot find one";
                    break;
                case EditorExceptionE.CannotFindTransRowsetTree:
                    strRetVal = "Cannot get the transaction rowset tree from the navigation stack";
                    break;
                case EditorExceptionE.InvalidCommand:
                    strRetVal = "Invalid command";
                    break;
                case EditorExceptionE.InvalidStackOperation:
                    strRetVal = "Invalid stack operation";
                    break;
                case EditorExceptionE.InvalidPosition:
                    strRetVal = "Invalid edit position, valid values are FIRST, LAST, PREVIOUS OR NEXT";
                    break;
                case EditorExceptionE.CannotNavigateEmptyRowset:
                    strRetVal = "Cannot navigate through an empty rowset";
                    break;
                case EditorExceptionE.NavStackEmpty:
                    strRetVal = "The navigation stack is empty";
                    break;
                case EditorExceptionE.NotAllowedToCreateTransaction:
                    strRetVal = "Only root transaction editors can add transactions to the navigation stack";
                    break;
                case EditorExceptionE.DevError:
                    strRetVal = "We apologize for this error. please report this message to Consyst-SQL's Technical Support";
                    break;
                case EditorExceptionE.RowsetTreeNotFound:
                    strRetVal = "Rowset tree definition not found";
                    break;
                case EditorExceptionE.RowsetNotFound:
                    strRetVal = "Rowset definition not found";
                    break;
                case EditorExceptionE.InvalidCmd_None:
                    strRetVal = "Invalid usage of the command None. Usage: " +
                                "None";
                    break;
                case EditorExceptionE.InvalidCmd_Edit:
                    strRetVal = "Invalid usage of the command Edit. Usage: " +
                                "Edit,Item,Page,StackOp,Key   or  " +
                                "Edit,Current   or  " +
                                "\nEdit,First   or  " +
                                "\nEdit,Last   or  " +
                                "\nEdit,Previous   or  " +
                                "\nEdit,Next   or  " +
                                "\nEdit,Up   or  " +
                                "\nEdit,UpTransaction   or  " +
                                "\nEdit,Top";
                    break;
                case EditorExceptionE.InvalidCmd_New:
                    strRetVal = "Invalid usage of the command Duplicate. Usage: " +
                                "New,Item,Page,StackOp   or  " +
                                "New,Current";
                    break;
                case EditorExceptionE.InvalidCmd_Delete:
                    strRetVal = "Invalid usage of the command Delete. Usage: " +
                                "Delete,Current";
                    break;
                case EditorExceptionE.InvalidCmd_Duplicate:
                    strRetVal = "Invalid usage of the command Duplicate. Usage: " +
                                "Duplicate,Item,Page,StackOp,Key   or  " +
                                "Duplicate,Current";
                    break;
                case EditorExceptionE.InvalidCmd_Save:
                    strRetVal = "Invalid usage of the command Save. Usage: " +
                                "Save";
                    break;
                case EditorExceptionE.InvalidCmd_DeleteStack:
                    strRetVal = "Invalid usage of the command DeleteStack. Usage: " +
                                "DeleteStack,StackNumber";
                    break;
                case EditorExceptionE.MissingParam:
                    strRetVal = "One or more required parameters are missing";
                    break;
                case EditorExceptionE.CannotLoadData:
                    strRetVal = "Cannot load the transaction data. May be caused by a major change in the repository";
                    break;
                case EditorExceptionE.SessionTimeOut:
                    strRetVal = "Your session timed out!";
                    break;
                case EditorExceptionE.NotAllowedToSaveSortedSB:
                    strRetVal = "There was an attempt to save a sorted selection buffer to the common cache";
                    break;
                case EditorExceptionE.CannotBuildSqlCmds:
                    strRetVal = "Cannot build SQL Commands. May be due to a missing PK field in a Group in your Card";
                    break;
                case EditorExceptionE.WebCodeGenRequired:
                    strRetVal = "The public module WEBCODEGEN is not included in the program. Please add this module to your program before continuing.";
                    break;
                default:
                    break;
                }
                strBaseMessage = base.Message;
                if (strBaseMessage != String.Empty) {
                    if (strRetVal != null) {
                        strRetVal += " - " + strBaseMessage;
                    } else {
                        strRetVal  = strBaseMessage;
                    }
                }
                return(strRetVal);
            }
        }
	} // Class FWException
} // Namespace
