using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using RepPP;

namespace RepPP.Framework.Web {

    //********************************************************************************
    //
	/// <summary>
	/// FWTrace:     Encapsulates access to the ASP.NET Trace context
	/// </summary>
    //
    //********************************************************************************
	public class FWTrace {
	    /// <summary>Tracing category</summary>
        private const string            m_strTraceCat = "RepFW";
        /// <summary>Trace context</summary>
        private TraceContext            m_trace;
        /// <summary>Father page</summary>
        private Page                    m_pageFather;

        //********************************************************************************
        //
	    /// <summary>
	    /// ctor:                       Class constructor
	    /// </summary>
	    /// <param name="pageFather">   Father page</param>
	    //
        //********************************************************************************
        public FWTrace(Page pageFather) {
            m_pageFather = pageFather;
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// InternalTrace:      Returns the Trace object of the first page of a request
	    /// </summary>
	    //
        //********************************************************************************
        private TraceContext InternalTrace {
            get {
                if (m_trace == null) {
                    m_trace = m_pageFather.Trace;
                }
                return(m_trace);
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// WriteTrace:                     Write a trace
	    /// </summary>
	    /// <param name="strMsg">           Message to send to the trace</param>
	    //
        //********************************************************************************
        public virtual void WriteTrace(string strMsg) {
            InternalTrace.Write(m_strTraceCat, strMsg);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// WriteWarning:                   Write a warning to the trace message
	    /// </summary>
	    /// <param name="strMsg">           Message to send to the trace</param>
	    //
        //********************************************************************************
        public virtual void WriteWarning(string strMsg) {
            InternalTrace.Warn(m_strTraceCat, strMsg);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// IsTraceEnabled:                 Returns if the trace is enabled
	    /// </summary>
	    //
        //********************************************************************************
        public virtual bool IsTraceEnabled {
            get {
                return(InternalTrace.IsEnabled);
            }
        }
    } // Class FWTrace
} // Namespace
