using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using RepPP;

namespace RepPP.Framework.Web {

    //********************************************************************************
    //
	/// <summary>
	/// HttpApplicationFW:  Add support to framework events to the HttpApplication class
	/// </summary>
    //
    //********************************************************************************
	public class HttpApplicationFW : System.Web.HttpApplication {

        //********************************************************************************
        //
		/// <summary>
		///  BeforeValidate:                Called before the validation
		/// </summary>
		/// <param name="page">             Page handling the request</param>
		/// <returns>
		/// False to cancel the event, True to continue
		/// </returns>
		//
        //********************************************************************************
		public virtual bool BeforeValidate(PageEditor page) {
            return(true);
        }

        //********************************************************************************
		//
		/// <summary>
		///  AfterValidate:                 Called after the validation
		/// </summary>
		/// <param name="page">             Page handling the request</param>
		/// <returns>
		/// False to cancel the event, True to continue
		/// </returns>
		//
        //********************************************************************************
		public virtual bool AfterValidate(PageEditor page) {
		    return(true);
        }

        //********************************************************************************
		//
		/// <summary>
		///  BeforeDuplicateItem:           Called before an item is duplicated
		/// </summary>
		/// <param name="page">             Page handling the request</param>
		/// <returns>
		/// False to cancel the event, True to continue
		/// </returns>
		//
        //********************************************************************************
		public virtual bool BeforeDuplicateItem(PageEditor page) {
            return(true);
        }

        //********************************************************************************
		//
		/// <summary>
		///  AfterDuplicateItem:            Called after an item is duplicated 
		/// </summary>
		/// <param name="page">             Page handling the request</param>
		//
        //********************************************************************************
		public virtual void AfterDuplicateItem(PageEditor page) {
        }

        //********************************************************************************
		//
		/// <summary>
		///  BeforeNewItem:                 Called before an new item is created
		/// </summary>
		/// <param name="page">             Page handling the request</param>
		/// <returns>
		/// False to cancel the event, True to continue
		/// </returns>
		//
        //********************************************************************************
		public virtual bool BeforeNewItem(PageEditor page) {
            return(true);
        }

        //********************************************************************************
		//
		/// <summary>
		///  AfterNewItem:                  Called after a new item is created
		/// </summary>
		/// <param name="page">             Page handling the request</param>
		//
        //********************************************************************************
		public virtual void AfterNewItem(PageEditor page) {
        }

        //********************************************************************************
		//
		/// <summary>
		///  BeforeDeleteItem:              Called before an item is deleted 
		/// </summary>
		/// <param name="page">             Page handling the request</param>
		/// <returns>
		/// False to cancel the event, True to continue
		/// </returns>
		//
        //********************************************************************************
		public virtual bool BeforeDeleteItem(PageEditor page) {
            return(true);
        }

        //********************************************************************************
		//
		/// <summary>
		///  AfterDeleteItem:               Called after an item is deleted
		/// </summary>
		/// <param name="page">             Page handling the request</param>
		//
        //********************************************************************************
		public virtual void AfterDeleteItem(PageEditor page) {
        }

        //********************************************************************************
		//
		/// <summary>
		///  BeforeLoadFromDB:              Called before the transaction data is read 
		///                                 from the database 
		/// </summary>
		/// <param name="page">             Page handling the request</param>
		/// <returns>
		/// False to cancel the event, True to continue
		/// </returns>
		//
        //********************************************************************************
		public virtual bool BeforeLoadFromDB(PageEditor page) {
            return(true);
        }

        //********************************************************************************
		//
		/// <summary>
		///  AfterLoadFromDB:               Called after the transaction data is read 
		///                                 from the database
		/// </summary>
		/// <param name="page">             Page handling the request</param>
		//
        //********************************************************************************
		public virtual bool AfterLoadFromDB(PageEditor page) {
            return(true);
        }

        //********************************************************************************
		//
		/// <summary>
		///  BeforeSaveToDB:                Called before the transaction data is saved
		///                                 in the database
		/// </summary>
		/// <param name="page">             Page handling the request</param>
		/// <returns>
		/// False to cancel the event, True to continue
		/// </returns>
		//
        //********************************************************************************
		public virtual bool BeforeSaveToDB(PageEditor page) {
            return(true);
        }

        //********************************************************************************
		//
		/// <summary>
		///  AfterSaveToDB:                 Called after the transaction data is saved 
		///                                 in the database 
		/// </summary>
		/// <param name="page">             Page handling the request</param>
		//
        //********************************************************************************
		public virtual bool AfterSaveToDB(PageEditor page) {
            return(true);
        }

        //********************************************************************************
		//
		/// <summary>
		///  BeforeInitializeRepPP:         Called before the initialization of rep++
		/// </summary>
		/// <param name="page">             Page handling the request</param>
		//
        //********************************************************************************
	    public virtual void BeforeInitializeRepPP(Page page) {
	    }

        //********************************************************************************
		//
		/// <summary>
		///  CreateApplication:       Allows the user to create a custom
		///                           application
		/// </summary>
		/// <param name="eLang">      Language of the application</param>
		/// <param name="oldApp">     Old Application</param>
		/// <param name="handler">    ProgInit Event Handler</param>
		/// <returns>
		/// Application or null
		/// </returns>
		//
        //********************************************************************************
		public virtual RepPP.Application CreateApplication(LanguageCode          eLang, 
                                                           RepPP.Application     oldApp, 
                                                           ProgInitEventHandler  handler) {
            return (null);
        }

        //********************************************************************************
		//
		/// <summary>
		///  ProgInit:                      Called during the initialization of rep++
		/// </summary>
		/// <param name="app">             The Rep++ Application being initialized</param>
		//
        //********************************************************************************
	    public virtual void ProgInit(RepPP.Application app) {
	    }

        //********************************************************************************
		//
		/// <summary>
		///  AfterInitializeRepPP:          Called after the initialization of rep++
		/// </summary>
		/// <param name="page">             Page handling the request</param>
		//
        //********************************************************************************
	    public virtual void AfterInitializeRepPP(Page page) {
	    }

        //********************************************************************************
		//
		/// <summary>
		/// SBChange:                       Called when the data of current selection 
		///                                 of a selection buffer changes
		/// </summary>
        /// <param name="sender">           Selection buffer holder</param>
        /// <param name="e">                Event arguments</param>
		//
        //********************************************************************************
	    public virtual void SBChange(SelectionBufferHolder sender, EventArgs e) {
	    }

		//********************************************************************************
		//
		/// <summary>
		/// BeforeLoadSelectionBufferFromDB:    Called before loading the selection buffer
		///                                     data from the database
		/// </summary>
		/// <param name="page">                 Page handling the request</param>
		/// <returns>
		/// returns true if succeed, false to cancel the validation
		/// </returns>
		//
		//********************************************************************************
	    public virtual bool BeforeLoadSelectionBufferFromDB(PageWithSelectionBuffer page) {
	        return(true);
	    }

		//********************************************************************************
		//
		/// <summary>
		/// AfterLoadSelectionBufferFromDB:     Called after loading the selection buffer
		///                                     data from the database
		/// </summary>
		/// <param name="page">                 Page handling the request</param>
		/// <returns>
		/// returns true if succeed, false to cancel the validation
		/// </returns>
		//
		//********************************************************************************
	    public virtual bool AfterLoadSelectionBufferFromDB(PageWithSelectionBuffer page) {
	        return(true);
	    }

        //********************************************************************************
		//
		/// <summary>
		///  SessionTimeout:                Called when the session time out is detected
		/// </summary>
		/// <param name="sender">           The sender that triggered the event</param>
		//
        //********************************************************************************
		public virtual void SessionTimeout(ISessionDataHolder sender) {
            throw new FWException(FWException.EditorExceptionE.SessionTimeOut);
        }

        //********************************************************************************
		//
		/// <summary>
		///  OutOfSequence:                 Called when an out of sequence command is received
		/// </summary>
		/// <param name="page">             Page handling the request</param>
		//
        //********************************************************************************
		public virtual void OutOfSequence(PageWithStack page) {
            throw new FWException(FWException.EditorExceptionE.OutOfSequenceCmd);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// Session_End:                    Handle the session end event
	    /// </summary>
	    /// <param name="sender">           Sender object</param>
	    /// <param name="e">                Event parameter</param>
	    //
        //********************************************************************************
		protected virtual void Session_End(Object sender, EventArgs e) {
            DataHolder  dataHolder;
            NavStack    stack; 
            int         iIndex = 0; 

            dataHolder          = new DataHolder();
            dataHolder.Session  = Session;
            while((stack  = NavStack.CreateFromSession(dataHolder, dataHolder, iIndex)) != null) {
                stack.Destroy();
                iIndex++;
            }
		}
    }
}
