using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using RepPP;

namespace RepPP.Framework.Web {
    /// <summary>Interface implement by the object handling the caching of the application data</summary>
    public interface ICacheDataHolder {

        //********************************************************************************
        //
        /// <summary>
        /// SaveToCache:                Save an object in the cache
        /// </summary>
        /// <param name="strName">      Unique name of the object</param>
        /// <param name="obj">          Object to save</param>
        /// <param name="bPermanent">   true if permanent, false if it can be discarded</param>
        //
        //********************************************************************************
        void    SaveToCache(string strName, object obj, bool bPermanent);

        //********************************************************************************
        //
        /// <summary>
        /// IsExistInCache:             Check if an object exist in the cache
        /// </summary>
        /// <param name="strName">      Unique name of the object</param>
        /// <returns>
        /// true if yes, false if no
        /// </returns>
        //
        //********************************************************************************
        bool    IsExistInCache(string strName);

        //********************************************************************************
        //
        /// <summary>
        /// LoadFromCache:              Load an object from the cache
        /// </summary>
        /// <param name="strName">      Unique name of the object</param>
        /// <returns>
        /// object or null if not in the cache
        /// </returns>
        //
        //********************************************************************************
        object  LoadFromCache(string strName);

        //********************************************************************************
        //
        /// <summary>
        /// DeleteFromCache:            Delete an object from the cache
        /// </summary>
        /// <param name="strName">      Unique name of the object</param>
        //
        //********************************************************************************
        void    DeleteFromCache(string strName);

        //********************************************************************************
        //
        /// <summary>
        /// CacheSyncRoot:				Object use to lock the access to the cache
        /// </summary>
        //
        //********************************************************************************
        object CacheSyncRoot { get; }
    } // Interface ICacheDataHolder
} // Namespace
