using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using RepPP;

namespace RepPP.Framework.Web {
    /// <summary>Interface implement by the object handling the caching of the session data</summary>
    public interface ISessionDataHolder {

        //********************************************************************************
        //
        /// <summary>
        /// SaveToSession:              Save an object in the session
        /// </summary>
        /// <param name="strName">      Unique name of the object</param>
        /// <param name="obj">          Object to save</param>
        //
        //********************************************************************************
        void    SaveToSession(string strName, object obj);

        //********************************************************************************
        //
        /// <summary>
        /// IsExistInSession:           Check if an object exist in the session
        /// </summary>
        /// <param name="strName">      Unique name of the object</param>
        /// <returns>
        /// true if yes, false if no
        /// </returns>
        //
        //********************************************************************************
        bool    IsExistInSession(string strName);

        //********************************************************************************
        //
        /// <summary>
        /// LoadFromSession:            Load an object from the session
        /// </summary>
        /// <param name="strName">      Unique name of the object</param>
        /// <returns>
        /// object or null if not in the session
        /// </returns>
        //
        //********************************************************************************
        object  LoadFromSession(string strName);

        //********************************************************************************
        //
        /// <summary>
        /// DeleteFromSession:          Delete an object from the session
        /// </summary>
        /// <param name="strName">      Unique name of the object</param>
        //
        //********************************************************************************
        void    DeleteFromSession(string strName);

        //********************************************************************************
        //
        /// <summary>
        /// GetUniqueId:                Get a unique id for the session
        /// </summary>
        /// <returns>
        /// Unique id
        /// </returns>
        //
        //********************************************************************************
        int     GetUniqueId();

        //********************************************************************************
        //
        /// <summary>
        /// SessionSyncRoot:			Object use to lock the access to the session
        /// </summary>
        //
        //********************************************************************************
        object SessionSyncRoot { get; }
     } // Interface ISessionDataHolder
} // Namespace