using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Runtime.Serialization;
using RepPP;

namespace RepPP.Framework.Web {
    /// <summary>Defines an entry in the navigation stack</summary>
	[Serializable]
    public class NavEntry {
        /// <summary>Father stack</summary>
        [NonSerialized]
        private NavStack        m_stackFather;
        /// <summary>Unique ID of this entry</summary>
        private int             m_iEntryId;
        /// <summary>Hash containing the list of IDeletableObject attached to the entry</summary>
        private Hashtable       m_hashObj;
        /// <summary>URL attached to this entry</summary>
        private string          m_strUrl;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						        Class Constructor
        /// </summary>
        /// <param name="stackFather">          Father stack</param>
        /// <param name="iEntryId">             Unique entry ID</param>
        /// <param name="strUrl">               Url of the page</param>
        //
        //********************************************************************************
		protected NavEntry(NavStack stackFather, int iEntryId, string strUrl) : base() {
		    m_stackFather   = stackFather;
		    m_iEntryId      = iEntryId;
		    m_hashObj       = new Hashtable();
		    m_strUrl        = strUrl;
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						        Class Constructor
        /// </summary>
        /// <param name="stackFather">          Father stack</param>
        /// <param name="strUrl">               Url of the page</param>
        //
        //********************************************************************************
		public NavEntry(NavStack stackFather, string strUrl) : this(stackFather, stackFather.SessionDataHolder.GetUniqueId(), strUrl) {
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						        Class Constructor. Use for serialization.
        /// </summary>
        //
        //********************************************************************************
		private NavEntry() : this(null, -1, "") {
		}

        //********************************************************************************
        //
        /// <summary>
        /// Destroy:                        Destroy the entry.
        /// </summary>
        //
        //********************************************************************************
		public virtual void Destroy() {
			NavEntryObject entryObjCurrent;

		    foreach (DictionaryEntry obj in m_hashObj) {
		        entryObjCurrent = obj.Value as NavEntryObject;
				entryObjCurrent.Destroy();
		    }
		    m_hashObj.Clear();
            m_hashObj = null;
		}

        //********************************************************************************
        //
        /// <summary>
        /// SetFather:                  Set the father for all include entries
        /// </summary>
        /// <param name="stackFather">  Father</param>
        //
        //********************************************************************************
		internal void SetFather(NavStack stackFather) {
		    m_stackFather = stackFather;
		    foreach (DictionaryEntry obj in m_hashObj) {
		        (obj.Value as NavEntryObject).SetFather(this);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// FatherStack:                        Navigation stack containing this entry
        /// </summary>
        //
        //********************************************************************************
		public virtual NavStack FatherStack {
		    get {
		        return(m_stackFather);
		    }
		    set {
		        m_stackFather = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// EntryId:                            Unique ID of this entry
        /// </summary>
        //
        //********************************************************************************
		public virtual int EntryId {
		    get {
		        return(m_iEntryId);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Url:                            Url attached to the entry
        /// </summary>
        //
        //********************************************************************************
		public virtual string Url {
		    get {
		        return(m_strUrl);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// []:                                 Default array operator
        /// </summary>
        //
        //********************************************************************************
		public virtual NavEntryObject this[string strName] {
		    get {
		        return(m_hashObj[strName] as NavEntryObject);
		    }
		    set {
		        NavEntryObject  objOld;
		        
		        objOld = m_hashObj[strName] as NavEntryObject;
		        if (objOld != value) {
		            if (objOld != null) {
		                objOld.Destroy();
		            }
		            m_hashObj[strName]  = value;
		        }
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// DestroyObj:                         Destroy an object contains in the entry
        /// </summary>
        //
        //********************************************************************************
		public virtual void DestroyObj(string strName) {
		    NavEntryObject  obj;
		    
		    if (m_hashObj.ContainsKey(strName)) {
		        obj = m_hashObj[strName] as NavEntryObject;
		        obj.Destroy();
		        m_hashObj.Remove(strName);
		    }
		}
    } // Class NavEntry
} // Namespace
