using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using RepPP;

namespace RepPP.Framework.Web {
    /// <summary>
    /// Implement an object which can be hold in a navigation entry
    /// </summary>
	[Serializable]
    public abstract class NavEntryObject {
        /// <summary>Name of the object</summary>
        private string          m_strName;
        /// <summary>Entry holding this object</summary>
        [NonSerialized]
        private NavEntry        m_entryFather;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						            Class Constructor
        /// </summary>
        /// <param name="entryFather">              Entry holding this object</param>
        /// <param name="strName">                  Object name</param>
        //
        //********************************************************************************
		protected NavEntryObject(NavEntry  entryFather,
		                         string    strName) {
            m_strName       = strName;
            m_entryFather   = entryFather;
		}

        //********************************************************************************
        //
        /// <summary>
        /// Destroy:                                Destroy the object
        /// </summary>
        //
        //********************************************************************************
        public virtual void Destroy() {
        }

        //********************************************************************************
        //
        /// <summary>
        /// Load:                                   Restores the value of the object from some
        ///                                         persistent storage (e.g. Session or Cache)
        /// </summary>
        //
        //********************************************************************************
        public virtual void Load() {
        }

        //********************************************************************************
        //
        /// <summary>
        /// Save:                                   Saves the value of the object to some
        ///                                         persistent storage (e.g. Session or Cache)
        /// </summary>
        //
        //********************************************************************************
        public virtual void Save() {
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetFather:                  Set the father for all include entries
        /// </summary>
        /// <param name="entryFather">  Father</param>
        //
        //********************************************************************************
		internal void SetFather(NavEntry entryFather) {
		    m_entryFather = entryFather;
		}

        //********************************************************************************
        //
        /// <summary>
        /// FatherEntry:                            Entry holding this object
        /// </summary>
        //
        //********************************************************************************
        public virtual NavEntry FatherEntry {
            get {
                return(m_entryFather);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Name:                                   Gets the object name
        /// </summary>
        //
        //********************************************************************************
        public virtual string Name {
            get {
                return(m_strName);
            }
        }
    } // Class NavEntryObject
} // Namespace