using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using RepPP;

namespace RepPP.Framework.Web {
    /// <summary>
    /// Implement an object attach to a card instance or group instance which can be hold in a navigation entry
    /// </summary>
	[Serializable]
    public class NavEntryObjectInstance : NavEntryObject {
		/// <summary>a string literal to build the name of the session item that will hold the stack</summary>
		private const string        m_strInstanceSessionNamePrefix = "instance_";

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						            Class Constructor
        /// </summary>
        /// <param name="entryFather">              Entry holding this object</param>
        /// <param name="strName">                  Object name</param>
        //
        //********************************************************************************
		public NavEntryObjectInstance(NavEntry   entryFather,
		                              string     strName) : base(entryFather, strName) {
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						            Class Constructor. Use in serialization.
        /// </summary>
        //
        //********************************************************************************
		private NavEntryObjectInstance() : base(null, "") {
		}

        //********************************************************************************
        //
        /// <summary>
        /// DecoratedName:                          Decorated name
        /// </summary>
        //
        //********************************************************************************
        public virtual string DecoratedName {
            get {
                return(m_strInstanceSessionNamePrefix + FatherEntry.EntryId.ToString() + "_" + Name);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Destroy:                                Destroy the object
        /// </summary>
        //
        //********************************************************************************
        public override void Destroy() {
            FatherEntry.FatherStack.SessionDataHolder.DeleteFromSession(DecoratedName);
        }

        //********************************************************************************
        //
        /// <summary>
        /// SaveToSession:                      Save the rowset tree in the session
        /// </summary>
        /// <param name="rowsetTree">           Rowset tree to save</param>
        //
        //********************************************************************************
        public virtual void SaveToSession(RowsetTree rowsetTree) {
            byte[]  arrByte = null;
            
            rowsetTree.SaveToBuf(true,           // Compressed
                                 false,          // Changed only (no)
                                 true,           // Lang neutral
                                 out arrByte);
            FatherEntry.FatherStack.SessionDataHolder.SaveToSession(DecoratedName, arrByte);
        }

        //********************************************************************************
        //
        /// <summary>
        /// SaveToSession:                          Save the rowset in the session
        /// </summary>
        /// <param name="rowset">                   Rowset to save</param>
        /// <param name="bChild">                   true to save the child rowset too</param>
        //
        //********************************************************************************
        public virtual void SaveToSession(Rowset rowset, bool bChild) {
            byte[]  arrByte = null;
            
            rowset.SaveToBuf(true,           // Compressed
                             bChild,
                             false,          // Changed only (no)
                             true,           // Lang neutral
                             out arrByte);
            FatherEntry.FatherStack.SessionDataHolder.SaveToSession(DecoratedName, arrByte);
        }

        //********************************************************************************
        //
        /// <summary>
        /// LoadFromSession:                        Load the rowset tree from the session
        /// </summary>
        /// <param name="rowsetTree">               Rowset tree to load</param>
        /// <returns>
        /// true if succeed, false if not found in cache
        /// </returns>
        //
        //********************************************************************************
        public virtual bool LoadFromSession(RowsetTree rowsetTree) {
            bool    bRetVal;
            byte[]  arrByte = null;
            
            arrByte = FatherEntry.FatherStack.SessionDataHolder.LoadFromSession(DecoratedName) as byte[];
            if (arrByte == null) {
                bRetVal = false;
            } else {
                bRetVal = (0 == rowsetTree.LoadFromBuf(arrByte,
                                                       true,           // Compressed
                                                       true));         // Lang neutral
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// LoadFromSession:                        Load the rowset from the session
        /// </summary>
        /// <param name="rowset">                   Rowset to load</param>
        /// <returns>
        /// true if succeed, false if not found in cache
        /// </returns>
        //
        //********************************************************************************
        public virtual bool LoadFromSession(Rowset rowset) {
            bool    bRetVal;
            byte[]  arrByte = null;
            
            arrByte = FatherEntry.FatherStack.SessionDataHolder.LoadFromSession(DecoratedName) as byte[];
            if (arrByte == null) {
                bRetVal = false;
            } else {
                bRetVal = (0 == rowset.LoadFromBuf(arrByte,
                                                   true,           // Compressed
                                                   true));         // Lang neutral
            }
            return(bRetVal);
        }
    } // Class NavEntryObjectInstance
} // Namespace