using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using RepPP;

namespace RepPP.Framework.Web {
    /// <summary>
    /// Implement an object holding a reference to a SBDescriptor which can be hold in a navigation entry
    /// </summary>
	[Serializable]
    public class NavEntryObjectSBRef : NavEntryObject {
        /// <summary>Name of the rowset tree definition attached to the selection buffer</summary>
        private string                                  m_strSBRowsetTreeName;
        /// <summary>Clause use to filter the content of the selection buffer</summary>
        private string                                  m_strFilterClause;
        /// <summary>Optional parameters used to sort the content of the selection buffer</summary>
        private object[]                                m_arrSortParams;
        /// <summary>Cache mode of the Selection Buffer</summary>
        private PageWithSelectionBuffer.SBCacheModeE    m_eSBCacheMode;
		/// <summary>a string literal to build the name of the cache item that will hold the reference counting</summary>
		private const string                            m_strCountCacheNamePrefix = "SBCount_";
		/// <summary>a string literal to build the name of the cache item that will hold the instance</summary>
		private const string                            m_strInstCacheNamePrefix = "SBInst_";

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						            Class Constructor
        /// </summary>
        /// <param name="entryFather">              Entry holding this object</param>
        /// <param name="strName">                  Object name</param>
        /// <param name="strSBRowsetTreeName">      Name of the rowset tree definition to attached to the selection buffer</param>
        /// <param name="strFilterClause">          Clause use to filter the content of the selection buffer</param>
        /// <param name="arrSortParams">            Parameters used to sort the content of the selection buffer</param>
        /// <param name="eSBCacheMode">             Caching Mode</param>
        //
        //********************************************************************************
		public NavEntryObjectSBRef(NavEntry                             entryFather,
		                           string                               strName,
		                           string                               strSBRowsetTreeName,
		                           string                               strFilterClause,
                                   object[]                             arrSortParams,
                                   PageWithSelectionBuffer.SBCacheModeE eSBCacheMode) : base(entryFather, strName) {
		    m_strSBRowsetTreeName   = strSBRowsetTreeName;
		    m_strFilterClause       = strFilterClause;
            m_arrSortParams         = arrSortParams;
            m_eSBCacheMode          = eSBCacheMode;
		    if (entryFather != null) {
		        IncRefCount();
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						            Class Constructor
        /// </summary>
        /// <param name="entryFather">              Entry holding this object</param>
        /// <param name="strName">                  Object name</param>
        /// <param name="strSBRowsetTreeName">      Name of the rowset tree definition attached to the selection buffer</param>
        /// <param name="strFilterClause">          Clause use to filter the content of the selection buffer</param>
        /// <param name="arrSortParams">            Parameters used to sort the content of the selection buffer</param>
        //
        //********************************************************************************
		public NavEntryObjectSBRef(NavEntry      entryFather,
		                           string        strName,
                                   string        strSBRowsetTreeName,
		                           string        strFilterClause,
                                   object[]      arrSortParams) : this(entryFather, 
                                                                       strName,
                                                                       strSBRowsetTreeName,
                                                                       strFilterClause,
                                                                       arrSortParams,
                                                                       PageWithSelectionBuffer.SBCacheModeE.NonPermanentCache) {
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						            Class Constructor. Use for serialization
        /// </summary>
        //
        //********************************************************************************
		private NavEntryObjectSBRef() : this(null, "", "", "", null) {
		}

        //********************************************************************************
        //
        /// <summary>
        /// BuildCountCacheName:                    Build the name of the cache object which contains the ref count
        /// </summary>
        //
        //********************************************************************************
        private string BuildCountCacheName(string strCardName, string strFilterClause) {
            string  strRetVal;
            
            strRetVal = m_strCountCacheNamePrefix + strCardName + ":" + strFilterClause;
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// BuildInstCacheName:                     Build the name of the cache object which contains the SB instance
        /// </summary>
        //
        //********************************************************************************
        private string BuildInstCacheName(string strCardName, string strFilterClause) {
            string  strRetVal;
            
            strRetVal = m_strInstCacheNamePrefix + strCardName + ":" + strFilterClause;
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IncRefCount:                            Increment reference counting
        /// </summary>
        //
        //********************************************************************************
        private void IncRefCount() {
            int                 iRefCount;
            string              strCacheName;
            object              objRefCount;
            ICacheDataHolder    cacheDataHolder;
            
            cacheDataHolder = FatherEntry.FatherStack.CacheDataHolder;
            strCacheName    = BuildCountCacheName(SBRowsetTreeName, FilterClause);
            lock(cacheDataHolder.CacheSyncRoot) {
                objRefCount = cacheDataHolder.LoadFromCache(strCacheName);
                if (objRefCount == null) {
                    iRefCount = 0;
                } else {
                    iRefCount = (int)objRefCount;
                }
                iRefCount++;
                cacheDataHolder.SaveToCache(strCacheName, iRefCount, true);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DecRefCount:                            Increment reference counting
        /// </summary>
        //
        //********************************************************************************
        private void DecRefCount() {
            int                 iRefCount;
            string              strCacheName;
            object              objRefCount;
            ICacheDataHolder    cacheDataHolder;
            
            cacheDataHolder = FatherEntry.FatherStack.CacheDataHolder;
            strCacheName    = BuildCountCacheName(SBRowsetTreeName, FilterClause);
            lock(cacheDataHolder.CacheSyncRoot) {
                objRefCount = cacheDataHolder.LoadFromCache(strCacheName);
                if (objRefCount != null) {
                    iRefCount = (int)objRefCount;
                    iRefCount--;
                } else {
                    iRefCount = 0;
                }
                if (iRefCount > 0) {
                    cacheDataHolder.SaveToCache(strCacheName, iRefCount, true);
                } else {
                    cacheDataHolder.DeleteFromCache(strCacheName);
                    strCacheName = BuildInstCacheName(SBRowsetTreeName, FilterClause);
                    cacheDataHolder.DeleteFromCache(strCacheName);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SBRowsetTreeName:                   Name of the rowset tree use for the selection buffer
        /// </summary>
        //
        //********************************************************************************
        public virtual string SBRowsetTreeName {
            get {
                return(m_strSBRowsetTreeName);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FilterClause:                           Clause use to filter the content of the selection buffer
        /// </summary>
        //
        //********************************************************************************
        public virtual string FilterClause {
            get {
                return(m_strFilterClause);
            }
             set {
                if (m_strFilterClause != value) {
                    DecRefCount();
                    m_strFilterClause = value;
                    IncRefCount();
                }
             }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SBCacheMode:                    Gets the caching mode of the selection buffer data
        /// </summary>
        //
        //********************************************************************************
        public virtual PageWithSelectionBuffer.SBCacheModeE SBCacheMode {
            get {
                return(m_eSBCacheMode);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SortParams:               Parameters used to sort the content of the selection buffer
        /// </summary>
        //
        //********************************************************************************
        public virtual object[] SortParams {
            get {
                return(m_arrSortParams);
            }
            set {
                m_arrSortParams = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Destroy:                                Destroy the object
        /// </summary>
        //
        //********************************************************************************
        public override void Destroy() {
            DecRefCount();
        }

        //********************************************************************************
        //
        /// <summary>
        /// LoadSBFromCache:                        Load the selection buffer from the cache
        /// </summary>
        /// <param name="rowsetTree">               Rowset tree to fill</param>
        /// <returns>
        /// true if succeed, false if not in cache
        /// </returns>
        //
        //********************************************************************************
        public virtual bool LoadSBFromCache(RowsetTree rowsetTree) {
            bool                bRetVal;
            string              strCacheName;
            ICacheDataHolder    cacheDataHolder;
            byte[]              arrByte;
            
            if (SBCacheMode == PageWithSelectionBuffer.SBCacheModeE.NoCache) {
                throw new FWException(FWException.EditorExceptionE.DevError, 
                                      "NoCache selection buffer must not access the cache");
            }
            cacheDataHolder = FatherEntry.FatherStack.CacheDataHolder;
            strCacheName    = BuildInstCacheName(SBRowsetTreeName, FilterClause);
            lock(cacheDataHolder.CacheSyncRoot) {
                if (cacheDataHolder.IsExistInCache(strCacheName)) {
                    arrByte = cacheDataHolder.LoadFromCache(strCacheName) as byte[];
                    bRetVal = (0 == rowsetTree.LoadFromBuf(arrByte,
                                                           true,     // Compress
                                                           true));   // Lang neutral
                } else {
                    bRetVal = false;
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// SaveSBToCache:                          Save the selection buffer to the cache
        /// </summary>
        /// <param name="rowsetTree">               Rowset tree containing the data</param>
        //
        //********************************************************************************
        public virtual void SaveSBToCache(RowsetTree rowsetTree) {
            string              strCacheName;
            ICacheDataHolder    cacheDataHolder;
            byte[]              arrByte = null;
            
            if (SBCacheMode == PageWithSelectionBuffer.SBCacheModeE.NoCache) {
                throw new FWException(FWException.EditorExceptionE.DevError, 
                                      "NoCache selection buffer must not access the cache");
            }
            cacheDataHolder = FatherEntry.FatherStack.CacheDataHolder;
            strCacheName    = BuildInstCacheName(SBRowsetTreeName, FilterClause);
            lock(cacheDataHolder.CacheSyncRoot) {
                rowsetTree.SaveToBuf(true,   // Compress
                                     false,  // No Change only
                                     true,   // Lang neutral,
                                     out arrByte);
                cacheDataHolder.SaveToCache(strCacheName, arrByte, false);
            }
        }
    } // Class NavEntryObjectSBRef
} // Namespace