using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Runtime.Serialization;
using RepPP;

namespace RepPP.Framework.Web {
    /// <summary>Defines a transaction entry in the navigation stack</summary>
	[Serializable]
    public class NavEntryTransaction : NavEntry {
        /// <summary>Name of the rowset tree of the transaction</summary>
        private string                              m_strTransactionRowsetTreeName;
        /// <summary>Cache mode of the editor</summary>
        private PageEditor.TransactionCacheModeE    m_eTransactionCacheMode;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						                Class Constructor
        /// </summary>
        /// <param name="stackFather">                  Father stack</param>
        /// <param name="iEntryId">                     Unique entry ID</param>
        /// <param name="strUrl">                       Url of the page</param>
        /// <param name="strTransactionRowsetTreeName"> Name of the rowset tree of the transaction</param>
        /// <param name="eTransactionCacheMode">        Caching mode use for the editor</param>
        //
        //********************************************************************************
		protected NavEntryTransaction(NavStack                          stackFather,
		                              int                               iEntryId,
		                              string                            strUrl,
		                              string                            strTransactionRowsetTreeName,
		                              PageEditor.TransactionCacheModeE  eTransactionCacheMode) : base(stackFather, iEntryId, strUrl) {
		    m_strTransactionRowsetTreeName  = strTransactionRowsetTreeName;
		    m_eTransactionCacheMode         = eTransactionCacheMode;
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						                Class Constructor
        /// </summary>
        /// <param name="stackFather">                  Father stack</param>
        /// <param name="strUrl">                       Url of the page</param>
        /// <param name="strTransactionRowsetTreeName"> Name of the rowset tree of the transaction</param>
        /// <param name="eTransactionCacheMode">        Caching mode use for the editor</param>
        //
        //********************************************************************************
		public NavEntryTransaction(NavStack                          stackFather,
		                           string                            strUrl,
		                           string                            strTransactionRowsetTreeName,
		                           PageEditor.TransactionCacheModeE  eTransactionCacheMode) : this(stackFather,
		                                                                                           stackFather.SessionDataHolder.GetUniqueId(),
		                                                                                           strUrl,
                                                                                                   strTransactionRowsetTreeName,
		                                                                                           eTransactionCacheMode) {
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:						        Class Constructor. Use for serialization.
        /// </summary>
        //
        //********************************************************************************
		private NavEntryTransaction() : this(null, -1, "", "", PageEditor.TransactionCacheModeE.CacheAndCommit) {
		}

        //********************************************************************************
        //
        /// <summary>
        /// TransactionRowsetTreeName:              Name of the rowset tree of the transaction
        /// </summary>
        //
        //********************************************************************************
		public virtual string TransactionRowsetTreeName {
		    get {
		        return(m_strTransactionRowsetTreeName);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// TransactionCacheMode:               Cache mode used by the transaction
        /// </summary>
        //
        //********************************************************************************
		public virtual PageEditor.TransactionCacheModeE TransactionCacheMode {
		    get {
		        return(m_eTransactionCacheMode);
		    }
		}
   };
}