using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.IO;
using RepPP;
using RepPP.Toolkit;
using RepPP.Toolkit.Web;

namespace RepPP.Framework.Web {

    //********************************************************************************
    //
	/// <summary>
	/// SBFilter:       Class implementing the non UI filter functionnalities
	/// </summary>
    //
    //********************************************************************************
	public class SBFilter : SBFilterBase {
	    /// <summary>Array of operator</summary>
	    protected string[]          m_arrOp;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                   Class constructor
        /// </summary>
        /// <param name="rowsetDefParam"> RowsetDef use to as template to define the filter</param>
        //
        //********************************************************************************
		public SBFilter(RowsetDef rowsetDefParam) : base(rowsetDefParam) {
		    m_arrOp     = null;
		}

        //********************************************************************************
        //
        /// <summary>
        /// Dispose:                Dispose the filter
        /// </summary>
        //
        //********************************************************************************
        public override void Dispose() {
            base.Dispose();
        }

        //********************************************************************************
        //
		/// <summary>
		/// AddFieldDef:                Add a field to the rowset definition using another field as template
		/// </summary>
		/// <param name="rowsetDef">    Rowset definition</param>
		/// <param name="fldSrc">       Source field definition to use as template</param>
		/// <param name="strFldName">   Name of the field</param>
		/// <param name="bIsVisible">   Is the field visible?</param>
		/// <param name="iLogPos">      Position of the field in the group</param>
        //
        //********************************************************************************
        protected override FieldDef AddFieldDef(RepPP.RowsetDef rowsetDef,
                                                RepPP.FieldDef  fldSrc, 
                                                string          strFldName, 
                                                bool            bIsVisible,
                                                int             iLogPos) {
            return(base.AddFieldDef(rowsetDef, fldSrc, strFldName, true, iLogPos));
        }

	    //******************************************************************
	    //
		/// <summary>
		/// BuildWhereClause:                   Build a WHERE clause using the values in the rowset
		/// </summary>
		/// <param name="eType">                AND or OR</param>
		/// <param name="strFilterClause">      Human readable WHERE clause</param>
		/// <returns>
		/// WHERE clause
		/// </returns>
	    //
	    //******************************************************************
        protected override string BuildWhereClause(RepPP.Toolkit.SBFilterBase.AndOrE eType, out string strFilterClause) {
            string  strRetVal;
            
            strRetVal = BuildWhereClause(eType, m_arrOp, out strFilterClause);
            return(strRetVal);
        }

	    //******************************************************************
	    //
		/// <summary>
		/// BuildOpArray:                       Build the array of operator from the combobox array
		/// </summary>
		/// <param name="arrCombo">             Array of combobox</param>
		/// <returns>
		/// WHERE clause
		/// </returns>
	    //
	    //******************************************************************
        public virtual void BuildOpArray(DropDownList[] arrCombo) {
            int     iCount;
            
            iCount  = arrCombo.Length;
            m_arrOp = new string[iCount];
            for (int iIndex = 0; iIndex < iCount; iIndex++) {
                m_arrOp[iIndex] = (arrCombo[iIndex].SelectedIndex == -1) ? "" : arrCombo[iIndex].SelectedItem.Value;
            }
        }
	} // Class SBFilter
} // Namespace
