using System;
using System.IO;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.Collections;
using System.Collections.Specialized;
using System.Xml;
using System.Globalization;
using RepPP.Toolkit;
using RepPP.Toolkit.Web;

namespace RepPP.Framework.Web {

    //********************************************************************************
    //
	/// <summary>
	/// Implement the standard toolbar for the framework. All button items are injected.
	/// </summary>
    //
    //********************************************************************************
	[ParseChildren(false, ""),
     DefaultEvent("Click"),
	 ToolboxData("<{0}:ToolBarFW runat=server></{0}:ToolBarFW>")]
	public class ToolBarFW : ToolBar {
	    /// <summary>Name of each button in the toolbar</summary>
	    public enum ItemNameE {
	        /// <summary>Save</summary>
	        Save           =   0,
	        /// <summary>New</summary>
	        New            =   1,
	        /// <summary>Duplicate</summary>
	        Duplicate      =   2,
	        /// <summary>Delete</summary>
	        Delete         =   3,
	        /// <summary>The first seperator</summary>
	        Sep1           =   4,
	        /// <summary>Up</summary>
	        Up             =   5,
	        /// <summary>Top</summary>
	        Top            =   6,
	        /// <summary>The second seperator</summary>
	        Sep2           =   7,
	        /// <summary>First</summary>
	        First          =   8,
	        /// <summary>Previous</summary>
	        Previous       =   9,
	        /// <summary>Next</summary>
	        Next           =   10,
	        /// <summary>Last</summary>
	        Last           =   11
	    };
        private const int                   m_iMaximumNumberOfItems	= 12;
	    private RepPP.Toolkit.ControlUtil   m_controlUtil;
        private string                      m_strImagesFolder;

        //********************************************************************************
        //
	    /// <summary>
	    /// ctor:           Class constructor
	    /// </summary>
        //
        //********************************************************************************
		public ToolBarFW() {
		    ImagesFolder  = "Images/";
		    m_controlUtil = new RepPP.Toolkit.ControlUtil(this, new RepPP.Toolkit.Web.WebControlUtilHelper(), true);
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// Dispose:        Dispose the control
	    /// </summary>
        //
        //********************************************************************************
        public override void Dispose() {
            if (m_controlUtil != null) {
                m_controlUtil.Dispose();
                m_controlUtil = null;
            }
            base.Dispose();
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// OnInit:             Called at initialization
	    /// </summary>
	    /// <param name="e">    Event parameter</param>
        //
        //********************************************************************************
        protected override void OnInit(EventArgs e) {
            ArrayList   arrItems;
            
            base.OnInit(e);
            Items.Clear();
            arrItems = BuildStdItems();
		    foreach (ToolBarItemFW item in arrItems) {
		        Items.Add(item);
		    }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// InitItem:                   Initialize an item
	    /// </summary>
        /// <param name="item">         Button item</param>
        /// <param name="strId">        Item id</param>
        /// <param name="strTitle">     Item title</param>
        /// <param name="strTooltip">   Item tooltip</param>
        /// <param name="eType">        Item type</param>
        /// <param name="strImg">       Item image name</param>
        /// <param name="strHeight">    Item height</param>
        /// <param name="strWidth">     Item width</param>
        /// <param name="strServerCmd"> Server side command</param>
        /// <param name="strClientCmd"> Client side command</param>
        //
        //********************************************************************************
        private void InitItem(ToolBarItemFW                     item, 
                              string                            strId,
                              string                            strTitle,
                              string                            strTooltip,
                              ToolBarItemFW.ToolBarItemTypeE    eType,
                              string                            strImg,
                              string                            strHeight,
                              string                            strWidth,
                              string                            strServerCmd,
                              string                            strClientCmd) {
            item.Id                 = strId;
            item.Title              = strTitle;
            item.Tooltip            = strTooltip;
            item.Type               = eType;
            item.OnServerClick      = strServerCmd;
            item.OnClientClick      = strClientCmd;
            item.ImageHeight        = strHeight;
            item.ImageWidth         = strWidth;
            item.ImageUrl           = ImagesFolder + strImg + ".gif";
            item.Enabled            = true;
            item.Visible            = true;
            item.CausesValidation   = true;
            item.CheckIsDirty       = true;
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// InitSep:            Initialize a separator
	    /// </summary>
        //
        //********************************************************************************
        protected void InitSep(ToolBarItemFW item, string strId) {
            InitItem(item, strId, "", "", ToolBarItemFW.ToolBarItemTypeE.separator, "", "", "", "", "");
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// InitButton:                 Initialize a button
	    /// </summary>
	    /// <param name="item">         Button item</param>
	    /// <param name="translator">   Translator</param>
	    /// <param name="strId">        Button id</param>
	    /// <param name="strTitle">     Button default title</param>
	    /// <param name="strImg">       Image name</param>
	    /// <param name="strServerCmd"> Server side command</param>
	    /// <param name="strClientCmd"> Client side command</param>
        //
        //********************************************************************************
        protected void InitButton(ToolBarItemFW             item, 
                                  ControlTextTranslatorBase translator,
                                  string                    strId,
                                  string                    strTitle,
                                  string                    strImg,
                                  string                    strServerCmd,
                                  string                    strClientCmd) {
            string strText;
            string strTooltip;

            strTooltip = strTitle;
            if (translator != null) {
                strText = translator.GetItemText(strId, "Text");
                if (strText != null && strText != String.Empty) {
                    strTitle = strText;
                }
                strTooltip = translator.GetItemText(strId, "Tooltip");
                if (strTooltip == null || strTooltip == String.Empty) {
                    strTooltip = strTitle;
                }
            }
            InitItem(item, strId, strTitle, strTooltip, ToolBarItemFW.ToolBarItemTypeE.button, strImg, "16px", "16px", strServerCmd, strClientCmd);
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// BuildStdItems:  Build the ilst of standard items
	    /// </summary>
        //
        //********************************************************************************
        protected virtual ArrayList BuildStdItems() {
            ToolBarItemFW[]             items;
            RepPP.Application           app;
            ControlTextTranslatorBase   translator              = null;
            
            app         = m_controlUtil.Application;
            if (app != null && app.IsAtomExist("Editor_STD", RepPP.AtomType.sdAtomMsg)) {
                translator = new ControlTextTranslatorBase(app, new string[] {"Editor_STD"});
            }
            items = new ToolBarItemFW[m_iMaximumNumberOfItems];
            for (int iIndex = 0; iIndex < items.Length; iIndex++) {
                items[iIndex]  = new ToolBarItemFW(this);
            }
            InitButton(items[(int)ItemNameE.Save],       translator, "Save",      "Save",         "Save",      PageEditor.REPPP_CMD_FIELD_NAME + "=Save",              "");
            InitButton(items[(int)ItemNameE.New],        translator, "New",       "New",          "New",       PageEditor.REPPP_CMD_FIELD_NAME + "=New,Current",       "");
            InitButton(items[(int)ItemNameE.Duplicate],  translator, "Duplicate", "Duplicate",    "Duplicate", PageEditor.REPPP_CMD_FIELD_NAME + "=Duplicate,Current", "");
            InitButton(items[(int)ItemNameE.Delete],     translator, "Delete",    "Delete",       "Delete",    PageEditor.REPPP_CMD_FIELD_NAME + "=Delete,Current",    "");
            InitSep(items[(int)ItemNameE.Sep1], "sep1");
            InitButton(items[(int)ItemNameE.Up],         translator, "Up",        "Up",           "Up",        PageEditor.REPPP_CMD_FIELD_NAME + "=Edit,Up",           "");
            InitButton(items[(int)ItemNameE.Top],        translator, "Top",       "Top",          "Top",       PageEditor.REPPP_CMD_FIELD_NAME + "=Edit,Top",          "");
            InitSep(items[(int)ItemNameE.Sep2], "sep2");
            InitButton(items[(int)ItemNameE.First],      translator, "First",     "",            "First",      PageEditor.REPPP_CMD_FIELD_NAME + "=Edit,First",        "");
            InitButton(items[(int)ItemNameE.Previous],   translator, "Previous",  "",            "Previous",   PageEditor.REPPP_CMD_FIELD_NAME + "=Edit,Previous",     "");
            InitButton(items[(int)ItemNameE.Next],       translator, "Next",      "",            "Next",       PageEditor.REPPP_CMD_FIELD_NAME + "=Edit,Next",         "");
            InitButton(items[(int)ItemNameE.Last],       translator, "Last",      "",            "Last",       PageEditor.REPPP_CMD_FIELD_NAME + "=Edit,Last",         "");
            return(new ArrayList(items));
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// SetItemState:   Set the state of the a button.
	    /// </summary>
        /// <param name="strItemId">    ID of the button</param>
        /// <param name="bEnable">      true if enable, false if not</param>
        //
        //********************************************************************************
        public virtual void SetItemState(string strItemId, bool bEnable) {
            ToolBarItemFW   item;
            
            item            = Items[strItemId] as ToolBarItemFW;
            item.Enabled    = bEnable;
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// SetItemPostBackState:       Set the postback properties of the item .
	    /// </summary>
        /// <param name="strItemId">    ID of the button</param>
        /// <param name="bIsDirty">     true to check the is dirty, false if not</param>
        /// <param name="bValidate">    true to validate before a postback, false if not</param>
        /// <param name="bConfirm">     true to confirm a delete before a postback, false if not</param>
        //
        //********************************************************************************
        public virtual void SetItemPostBackState(string strItemId, bool bIsDirty, bool bValidate, bool bConfirm) {
            ToolBarItemFW   item;
            
            item                    = Items[strItemId] as ToolBarItemFW;
            item.CheckIsDirty       = bIsDirty;
            item.CausesValidation   = bValidate;
            item.AskForConfirmation = bConfirm;
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// SetItemScript:              Set the client side script of a button.
	    /// </summary>
        /// <param name="iIndex">       Index of the button</param>
        /// <param name="strScript">    The client side script to be executed on the browser
        ///                             when the item is clicked</param>
        //
        //********************************************************************************
        public virtual void SetItemScript(int iIndex, string strScript) {
            ToolBarItemFW     item;
            
            item                = Items[iIndex] as ToolBarItemFW;
            item.OnClientClick  = strScript;
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// Items:          Override the standard property to make it invisible in the designer
	    ///                 and not serializable.
	    /// </summary>
        //
        //********************************************************************************
        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public override RepPP.Toolkit.Web.ToolBar.ToolBarItemCollection Items {
            get {
                return base.Items;
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// ImagesFolder:   The path to the folder containing the toolbar's images
	    /// </summary>
        //
        //********************************************************************************
        [Category("Appearance"),
         NotifyParentProperty(true),
         Description("The path to the folder containing the toolbar's images"),
         Editor(typeof(System.Web.UI.Design.UrlEditor), typeof(System.Drawing.Design.UITypeEditor))]
        public virtual string ImagesFolder {
            get {
                return(m_strImagesFolder);
            }
            set {
                int     iOriSize;
                string  strOriURL;
                string  strURL;
                
                if (m_strImagesFolder != value) {
                    strOriURL         = (m_strImagesFolder == null) ? String.Empty : m_strImagesFolder;
                    iOriSize          = strOriURL.Length;
                    m_strImagesFolder = (value == null) ? String.Empty : value;
                    if (m_strImagesFolder != String.Empty && !m_strImagesFolder.EndsWith("/")) {
                        m_strImagesFolder += "/";
                    }
                    foreach (ToolBarItemFW item in Items) {
                        strURL = item.ImageUrl;
                        if (strURL.Substring(0, iOriSize) == strOriURL) {
                            item.ImageUrl = value + strURL.Substring(iOriSize);
                        }
                    }
                }
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// RegisterAsSaveHandler:    Register the toobar as a save command handler
	    /// </summary>
        //
        //********************************************************************************
        protected virtual void RegisterAsSaveHandler() {
            if (Page != null && Page is PageEditor) {
                (Page as PageEditor).RegisterSaveCmdHandler(UniqueID.Replace(":", "$"));
            }
        }

        //********************************************************************************
        //
	    /// <summary>
	    /// OnLoad:    Creates the menu items
	    /// </summary>
        //
        //********************************************************************************
        protected override void OnLoad(EventArgs e) {
            RegisterAsSaveHandler();
            base.OnLoad(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnPreRender:                    Called before rendering this control to 
        ///                                 register the client script. The script is 
        ///                                 included once per page even in the presence
        ///                                 of multiple ToolBar objects
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        protected override void OnPreRender(EventArgs e) {
            StreamReader        reader;
            const string        strKey = "ToolBarFWScript";
            string              strScript;

            if (!WebUtil.IsClientScriptBlockRegistered(Page, strKey)) {
                reader    = new StreamReader(typeof(ToolBarFW).Assembly.GetManifestResourceStream("RepPP.Framework.Web.ToolBarFWScript.js"));
                strScript = String.Format(reader.ReadToEnd(),
                                          UniqueID.Replace(":", "$"),
                                          "Save");
                WebUtil.RegisterClientScriptBlock(Page, strKey, strScript);                                                
            }
            base.OnPreRender(e);
            SetToolBarState();
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetToolBarState:               Set the state of the toolbar based on the state 
        ///                                of the container page
        /// </summary>
        //
        //********************************************************************************
        protected virtual void SetToolBarState() {
            PageEditor              pageEditor;
            SelectionBufferHolder   selBuf;
            int                     iCurLine;
            int                     iLineCount;
            bool                    bBefore;
            bool                    bAfter;
            bool                    bDuplicate;
            bool                    bIsRoot;
            
            if (Page is PageEditor){
                pageEditor     = Page as PageEditor;
                bIsRoot     = pageEditor.IsRootNode;
                if (bIsRoot) {
                    selBuf = pageEditor.SelectionBuffer;
                    if (selBuf == null) {
                        iCurLine    = 0;
                        iLineCount  = 1;
                    } else {
                        iCurLine    = selBuf.SelectedIndex;
                        iLineCount  = selBuf.ItemCount;
                    }
                } else {
                    iCurLine    = pageEditor.TransactionRowset.CurrentUndelLine;
                    iLineCount  = pageEditor.TransactionRowset.UndelLineCount;
                }
                bBefore     = (iCurLine > 0);
                bAfter      = (iCurLine < (iLineCount - 1));
                bDuplicate  = true; // Why must we disable Duplicate?
                SetItemState(ItemNameE.Duplicate.ToString(),   bDuplicate);
                SetItemState(ItemNameE.Up.ToString(),          !bIsRoot);
                SetItemState(ItemNameE.Top.ToString(),         !bIsRoot);
                SetItemState(ItemNameE.First.ToString(),       bBefore);
                SetItemState(ItemNameE.Previous.ToString(),    bBefore);
                SetItemState(ItemNameE.Next.ToString(),        bAfter);
                SetItemState(ItemNameE.Last.ToString(),        bAfter);
                SetItemPostBackState(ItemNameE.Save.ToString(),       false,      true,       false);
                SetItemPostBackState(ItemNameE.New.ToString(),        bIsRoot,    true,       false);
                SetItemPostBackState(ItemNameE.Duplicate.ToString(),  bIsRoot,    true,       false);
                SetItemPostBackState(ItemNameE.Delete.ToString(),     false,      false,      bIsRoot);
                SetItemPostBackState(ItemNameE.Up.ToString(),         false,      true,       false);
                SetItemPostBackState(ItemNameE.Top.ToString(),        false,      true,       false);
                SetItemPostBackState(ItemNameE.First.ToString(),      bIsRoot,    !bIsRoot,   false);
                SetItemPostBackState(ItemNameE.Previous.ToString(),   bIsRoot,    !bIsRoot,   false);
                SetItemPostBackState(ItemNameE.Next.ToString(),       bIsRoot,    !bIsRoot,   false);
                SetItemPostBackState(ItemNameE.Last.ToString(),       bIsRoot,    !bIsRoot,   false);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnClick:                        Called when a ToolBar item is clicked
        /// </summary>
        /// <param name="e">                Event Argument</param>
        //
        //********************************************************************************
        protected override void OnClick(CommandEventArgs e) {

            if (Page is PageEditor && IsStandardCommand(e.CommandArgument.ToString())) {
                (Page as PageEditor).ExecuteCmds();
            }
            base.OnClick(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// RaisePostBackEvent:             Implements IPostBackEventHandler.RaisePostBackEvent
        ///                                 enables this server control to process an event 
        ///                                 raised when a form is posted to the server
        /// </summary>
        /// <param name="strEventArgument"> Event arguments</param>
        //
        //********************************************************************************
        public override void RaisePostBackEvent(string strEventArgument) {
            string strOnServerClick;

            strOnServerClick = Items[strEventArgument].OnServerClick;
            if (Page is PageEditor && IsStandardCommand(strOnServerClick)) {
                (Page as PageEditor).SetCommands(GetStandardCommand(strOnServerClick));
            }
            base.RaisePostBackEvent(strEventArgument);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsStandardCommand:              Finds out if the item contains a standard command
        /// </summary>
        /// <param name="strCmd">           Command to be verified</param>
        /// <returns>
        /// True if the command is a standard command. False otherwise.
        /// </returns>
        //
        //********************************************************************************
        protected virtual bool IsStandardCommand(string strCmd) {
            bool    bRetVal;

            bRetVal = strCmd.StartsWith(PageEditor.REPPP_CMD_FIELD_NAME);
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetStandardCommand:             Gets the standard command attached to a toolbar item
        /// </summary>
        /// <param name="strCmd">           Containing Command</param>
        /// <returns>
        /// The standard command portion.
        /// </returns>
        //
        //********************************************************************************
        protected virtual string GetStandardCommand(string strCmd) {
            string    strRetVal;
            
            //Remove "_RepPPCmd="
            strRetVal = strCmd.Substring(PageEditor.REPPP_CMD_FIELD_NAME.Length + 1);
            return(strRetVal);
        }

    }// Class ToolBarFW
}// Namespace
