using System;
using System.ComponentModel;
using System.Web.UI;
using RepPP.Toolkit.Web;

namespace RepPP.Framework.Web {
    //********************************************************************************
    //
    /// <summary>
    /// ToolBarItemFW provides a representation and the rendering of ToolBar buttons and
    /// separators of the framework.
    /// </summary>
    //
    //********************************************************************************
    public class ToolBarItemFW : ToolBarItem {

        /// <summary>Whether the item verifies the is dirty before a postback</summary>
        private bool            m_bCheckIsDirty;
        /// <summary>Whether the item asks for confirmation before executing a command</summary>
        private bool            m_bAskForConfirmation;
        /// <summary>Container ToolBar</summary>
        ToolBarFW               m_parent;


        //********************************************************************************
        //
        /// <summary>
        /// ToolBarItemFW                         ToolBarItem default Constructor
        /// </summary>
        //
        //********************************************************************************
        public ToolBarItemFW(ToolBarFW parent) : base() {
            m_parent                = parent;
            m_bCheckIsDirty         = true;
            m_bAskForConfirmation   = false;
        }

        //********************************************************************************
        //
        /// <summary>
        /// CheckIsDirty :                 Get/Sets a value to indicate whether the item 
        ///                                verifies the is dirty before a postback
        /// </summary>
        //
        //********************************************************************************
        [Bindable(true)]
        public virtual bool CheckIsDirty {
            get {
                return(m_bCheckIsDirty);
            }
            set {
                m_bCheckIsDirty = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AskForConfirmation :            Get/Sets a value to indicate whether the item 
        ///                                 asks for confirmation before executing
        /// </summary>
        //
        //********************************************************************************
        [Bindable(true)]
        public virtual bool AskForConfirmation {
            get {
                return(m_bAskForConfirmation);
            }
            set {
                m_bAskForConfirmation = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Render :                                    Writes the HTML representation of the 
        ///                                             ToolBarItem to the page and uses the Unique
        ///                                             Id of the toolbar to generate a unique id
        ///                                             for the ToolBarItem 
        /// </summary>
        /// <param name="output">                       HTML stream to write on a page</param>
        /// <param name="strUniqueID">                  Unique id of the hosting ToolBar</param>
        /// <param name="strValidationPostBackTrigger"> Script to trigger the validation and a postback</param>
        /// <param name="strPostBackTrigger">           Script to trigger a postback</param>
        /// <param name="strEnabledStyle">              Style to use for enabled item</param>
        /// <param name="strDisabledStyle">             Style to use for disabled item</param>
        /// <param name="strSeparatorStyle">            Style to use for separator items</param>
        /// <param name="strEnabledClass">              Class style to use for enabled items</param>
        /// <param name="strDisabledClass">             Class style to use for disabled items</param>
        /// <param name="strSeparatorClass">            Class style to use for separator items</param>
        //
        //********************************************************************************
        public override void Render(HtmlTextWriter output, 
                                    string strUniqueID, 
                                    string strValidationPostBackTrigger, 
                                    string strPostBackTrigger, 
                                    string strEnabledStyle, 
                                    string strDisabledStyle, 
                                    string strSeparatorStyle, 
                                    string strEnabledClass, 
                                    string strDisabledClass, 
                                    string strSeparatorClass) {
            PageEditor  editor;
            string      strPostBack;
            string      strValidationPostBack;
            string      strCheckIsDirtyScript;
            string      strConfirmationScript;

            editor                  = m_parent.Page as PageEditor; 
            strPostBack             = strPostBackTrigger;
            strValidationPostBack   = strValidationPostBackTrigger; 
            if (editor != null) {
                if (CheckIsDirty) {
                    strCheckIsDirtyScript   = editor.CheckIsDirtyScript;
                    strPostBack             = strCheckIsDirtyScript + strPostBack;
                    strValidationPostBack   = strCheckIsDirtyScript + strValidationPostBack; 
                }
                if (AskForConfirmation) {
                    strConfirmationScript   = editor.ConfirmDeleteScript;
                    strPostBack             = strConfirmationScript + strPostBack;
                    strValidationPostBack   = strConfirmationScript + strValidationPostBack; 
                }
            }
            base.Render (output, 
                         strUniqueID, 
                         strValidationPostBack, 
                         strPostBack, 
                         strEnabledStyle, 
                         strDisabledStyle, 
                         strSeparatorStyle, 
                         strEnabledClass, 
                         strDisabledClass, 
                         strSeparatorClass);
        }
    }// class ToolBarItemFW
}// namespace
