using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using RepPP;

namespace RepPP.Framework.Web {

    //********************************************************************************
    //
	/// <summary>
	/// Utility class
	/// </summary>
	//
    //********************************************************************************
	public class Utility {

        //********************************************************************************
        //
        /// <summary>
        /// IsLocal:        Checks if the request is local
        /// </summary>
        //
        //********************************************************************************
        public static bool IsLocal {
            get {
                bool    bRetVal = false;
                string  strServerAdr;
                string  strAdr;
                
                strAdr = HttpContext.Current.Request.UserHostAddress;
                if (strAdr == "127.0.0.1" || strAdr == "::1") {
                    bRetVal = true;
                } else {
                    strServerAdr = HttpContext.Current.Request.ServerVariables["LOCAL_ADDR"];
                    if (strAdr != null && strAdr != String.Empty && strAdr == strServerAdr) {
                        bRetVal = true;
                    }
                }
                return(bRetVal);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetFormName:    Gets the name of the form
        /// </summary>
        /// <returns>
        /// Form name
        /// </returns>
        //
        //********************************************************************************
        public static string GetFormName(Page page) {
            string  strRetVal = "Form1";
            Control ctl;

            ctl = RepPP.Toolkit.Web.WebUtil.FindControlUsingType(page.Controls, typeof(HtmlForm));
            if (ctl != null) {
                strRetVal = ((HtmlForm)ctl).Name;
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// EncodeKey:              Encode a key
        /// </summary>
        /// <param name="arrKey">   Array of keys to encode</param>
        /// <returns>
        /// Encode keys
        /// </returns>
        //
        //********************************************************************************
        public static string EncodeKey(string[] arrKey) {
            return(RepPP.Toolkit.Web.WebUtil.EncodeStringArray(arrKey, new char[] { ':', ',', ';' }));
        }

        //********************************************************************************
        //
        /// <summary>
        /// DecodeKey:              Decode a key
        /// </summary>
        /// <param name="strKeys">  Encoded keys</param>
        /// <returns>
        /// Array of keys
        /// </returns>
        //
        //********************************************************************************
        public static string[] DecodeKey(string strKeys) {
            return(RepPP.Toolkit.Web.WebUtil.DecodeStringArray(strKeys, new char[] { ':', ',', ';' }));
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetTransactionRowsetTreeFromCache       Get the transaction RowsetTree from the cache
        /// </summary>
        /// <param name="rowsetTree">               Transaction RowsetTree </param>
        /// <returns>
        /// True if the transaction RowsetTree was found in the cache. false otherwise
        /// </returns>
        //
        //********************************************************************************
        public static bool GetTransactionRowsetTreeFromCache(RowsetTree rowsetTree) {
            bool                    bRetVal             = true;
            NavEntryTransaction     transEntry;
            NavEntryObjectInstance  objInst;
            PageWithStack           page;

            page = HttpContext.Current.Handler as PageWithStack;
            if (page == null) {
                bRetVal = false;
            } else {
                transEntry = page.NavigationStack.GetCurrentEntry(typeof(NavEntryTransaction)) as NavEntryTransaction;
                if (transEntry == null) {
                    bRetVal = false;
                } else {
                    if (rowsetTree.RowsetTreeDef.Name != transEntry.TransactionRowsetTreeName) {
                        bRetVal = false;
                    } else {
                        objInst         = transEntry[PageEditor.REPPP_TRANSACTION_OBJECT_NAME] as NavEntryObjectInstance;
                        if (objInst == null) {
                            bRetVal = false;
                        } else {
                            if (!objInst.LoadFromSession(rowsetTree)) {
                                throw new FWException(FWException.EditorExceptionE.CannotLoadData);
                            }
                            rowsetTree.RootRowset.SelectLine(0, true, true);
                        }
                    }
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// SaveTransactionRowsetTreeToCache:   Saves the Transaction Rowset tree to the cache
        /// </summary>
        /// <param name="rowsetTree">           Transaction Rowset tree</param>
        /// <returns>
        /// True if successful, false otherwise.
        /// </returns>
        //
        //********************************************************************************
        public static bool SaveTransactionRowsetTreeToCache(RowsetTree rowsetTree) {
            NavEntryTransaction     entryTrans;
            NavEntryObjectInstance  objTrans;
            PageWithStack           page;
            bool                    bRetVal     = true;

            page = HttpContext.Current.Handler as PageWithStack;
            if (page == null) {
                bRetVal = false;
            } else {            
                entryTrans  = page.NavigationStack.GetCurrentEntry(typeof(NavEntryTransaction)) as NavEntryTransaction;
                if (entryTrans == null) {
                    bRetVal = false;
                } else {
                    objTrans = entryTrans[PageEditor.REPPP_TRANSACTION_OBJECT_NAME] as NavEntryObjectInstance;
                    if (objTrans == null) {
                        objTrans = new NavEntryObjectInstance(entryTrans, "_RepPPTransactionRowsetTree");
                        entryTrans["_RepPPTransactionRowsetTree"] = objTrans;
                    }
                    objTrans.SaveToSession(rowsetTree);
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsWebCodeGenIncluded            Check if the WebCodeGen public module is included in the program
        /// </summary>
        /// <param name="app">              Rep++ Application Object</param>
        /// <returns>
        /// True if the public module is included, False Otherwise.
        /// </returns>
        //
        //********************************************************************************
        internal static bool IsWebCodeGenIncluded(Application app) {
            return app.IsAtomExist("Msg_IsDirty", AtomType.sdAtomMsg);
        }
    }// Class Utility
}// Namespace