﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing.Design;
using System.ComponentModel;
using System.Windows.Forms.Design;
using System.Reflection;

namespace RepPP.Toolkit.Designer {

    /// <summary>
    /// Atom class Designer helper 
    /// </summary>
    public class AtmControlDesigner {
        //********************************************************************************
        //
        /// <summary>
        /// Implement an editor for the AtomFullName property
        /// </summary>
        //
        //********************************************************************************
        public class AtmControlEditor : System.Drawing.Design.UITypeEditor {

            //********************************************************************************
            //
            /// <summary>
            /// GetEditStyle:           Gets the style of edition
            /// </summary>
            /// <param name="context">  Context</param>
            /// <returns>
            /// Type of editor (modal)
            /// </returns>
            //
            //********************************************************************************
            public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) {
                UITypeEditorEditStyle eRetVal;

                if (context != null && context.Instance != null) {
                    eRetVal = UITypeEditorEditStyle.Modal;
                } else {
                    eRetVal = base.GetEditStyle(context);
                }
                return (eRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// CreateRealAtmFullName: Create the real object AtmFullName from the proxy object
            /// </summary>
            /// <param name="atmFullName">AtmFullName Proxy object</param>
            /// <param name="typeAtmFullName">Type of the Real AtmFullName</param>
            /// <param name="assemRepPP">Rep++ Assembly used to get some types</param>
            /// <returns>
            /// A Real AtmFullName
            /// </returns>
            //
            //********************************************************************************
            private object CreateRealAtomFullName(AtmFullName atmFullName, Type typeAtmFullName, Assembly assemRepPP) {
                Type[]              arrTypeParams;
                ConstructorInfo     ctorInfos;
                object              objAtmFullnameRetVal;

                arrTypeParams        = new Type[] { assemRepPP.GetType("RepPP.AtomType"), typeof(String) };
                ctorInfos            = typeAtmFullName.GetConstructor(arrTypeParams);
                objAtmFullnameRetVal = ctorInfos.Invoke(new Object[] { atmFullName.AtomType, atmFullName.AtomName });
                return(objAtmFullnameRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// EditValue:              Edit a value
            /// </summary>
            /// <param name="context">  Context</param>
            /// <param name="provider"> Provider</param>
            /// <param name="value">    Value to edit</param>
            /// <returns>
            /// New value
            /// </returns>
            //
            //********************************************************************************
            public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) {
                object                      objRetVal = null;
                FormAtmControlPropPage      frmAtmControlPropPage;
                IAtmControl                 atmControl;
                IWindowsFormsEditorService  edSvc;
                ProxyRepPPTKHelper          proxyTK;
                Type                        typeAtmFullName;
                Assembly                    assemRepPP;                
                Assembly                    assemTK;

                if (context != null && context.Instance != null && provider != null) {
                    edSvc = provider.GetService(typeof(IWindowsFormsEditorService)) as IWindowsFormsEditorService;
                    if (edSvc != null) {
                        typeAtmFullName         = value.GetType();
                        assemTK                 = typeAtmFullName.Assembly;
                        assemRepPP              = DesignerUtil.FindRepPPAssembly(value);
                        proxyTK                 = new ProxyRepPPTKHelper(assemTK);
                        atmControl              = new IAtmControl(proxyTK, context.Instance);
                        frmAtmControlPropPage   = new FormAtmControlPropPage(atmControl, new ProxyRepPPHelper(assemRepPP));
                        try {
                            edSvc.ShowDialog(frmAtmControlPropPage);
                            objRetVal = CreateRealAtomFullName(atmControl.AtomFullName, typeAtmFullName, assemRepPP);
                        } finally {
                            frmAtmControlPropPage.Dispose();
                        }
                    }
                }
                if (objRetVal == null) {
                    objRetVal = base.EditValue(context, provider, value);
                }
                return (objRetVal);
            }
        } // Class AtmControlEditor

        //********************************************************************************
        //
        /// <summary>
        /// Implement an editor for the Atom type property
        /// </summary>
        //
        //********************************************************************************
        public class AtmTypeEditor : System.Drawing.Design.UITypeEditor {

            //********************************************************************************
            //
            /// <summary>
            /// GetEditStyle:           Gets the style of edition
            /// </summary>
            /// <param name="context">  Context</param>
            /// <returns>
            /// Type of editor (modal)
            /// </returns>
            //
            //********************************************************************************
            public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) {
                UITypeEditorEditStyle eRetVal;

                if (context != null && context.Instance != null) {
                    eRetVal = UITypeEditorEditStyle.DropDown;
                } else {
                    eRetVal = base.GetEditStyle(context);
                }
                return (eRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// EditValue:              Edit a value
            /// </summary>
            /// <param name="context">  Context</param>
            /// <param name="provider"> Provider</param>
            /// <param name="value">    Value to edit</param>
            /// <returns>
            /// New value
            /// </returns>
            //
            //********************************************************************************
            public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) {
                object                      objRetVal = null;
                AtmFullName                 atmFullName;
                IWindowsFormsEditorService  edSvc;
                Application                 app;
                PopupListBox                listBox;
                AtomType                    eType;
                ProxyRepPPHelper            proxyRepPP;
                ProxyRepPPTKHelper          proxyTK;

                if (context != null && context.Instance != null && provider != null) {
                    edSvc      = provider.GetService(typeof(IWindowsFormsEditorService)) as IWindowsFormsEditorService;                    
                    proxyRepPP = new ProxyRepPPHelper(DesignerUtil.FindRepPPAssembly(context.Instance));
                    app        = Application.GetApplicationForDesign(proxyRepPP, provider);
                    if (edSvc != null && app != null) {
                        proxyTK             = new ProxyRepPPTKHelper(context.Instance.GetType().Assembly);
                        atmFullName         = new AtmFullName(proxyTK, context.Instance);
                        objRetVal           = atmFullName.AtomType;
                        listBox             = new PopupListBox(edSvc);
                        listBox.Location    = new System.Drawing.Point(0, 0);
                        listBox.Name        = "listBox";
                        listBox.Size        = new System.Drawing.Size(168, 186);
                        listBox.TabIndex    = 1;
                        listBox.BorderStyle = System.Windows.Forms.BorderStyle.None;
                        listBox.Items.Add(AtomType.sdAtomMsg.ToString());
                        listBox.Items.Add(AtomType.sdAtomErr.ToString());
                        edSvc.DropDownControl(listBox);
                        if (listBox.SelectedItem != null) {
                            eType = (AtomType)Enum.Parse(typeof(AtomType), (listBox.SelectedItem as string));
                            objRetVal = eType;
                            if (eType != atmFullName.AtomType) {
                                atmFullName.AtomName = "";
                            }
                        }
                    }
                }
                if (objRetVal == null) {
                    objRetVal = base.EditValue(context, provider, value);
                }
                return (objRetVal);
            }
        } // Class AtomTypeEditor

        //********************************************************************************
        //
        /// <summary>
        /// Implement an editor for the Atom name property
        /// </summary>
        //
        //********************************************************************************
        public class AtmNameEditor : System.Drawing.Design.UITypeEditor {

            //********************************************************************************
            //
            /// <summary>
            /// GetEditStyle:           Gets the style of edition
            /// </summary>
            /// <param name="context">  Context</param>
            /// <returns>
            /// Type of editor (modal)
            /// </returns>
            //
            //********************************************************************************
            public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) {
                UITypeEditorEditStyle eRetVal;

                if (context != null && context.Instance != null) {
                    eRetVal = UITypeEditorEditStyle.DropDown;
                } else {
                    eRetVal = base.GetEditStyle(context);
                }
                return (eRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// EditValue:              Edit a value
            /// </summary>
            /// <param name="context">  Context</param>
            /// <param name="provider"> Provider</param>
            /// <param name="value">    Value to edit</param>
            /// <returns>
            /// New value
            /// </returns>
            //
            //********************************************************************************
            public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) {
                object                      objRetVal = null;
                AtmFullName                 atmFullName;
                IWindowsFormsEditorService  edSvc;
                Application                 app;
                PopupListBox                listBox;
                ProxyRepPPHelper            proxyRepPP;
                ProxyRepPPTKHelper          proxyTK;
                String                      strAtomList;

                if (context != null && context.Instance != null && provider != null) {
                    edSvc      = provider.GetService(typeof(IWindowsFormsEditorService)) as IWindowsFormsEditorService;
                    proxyRepPP = new ProxyRepPPHelper(DesignerUtil.FindRepPPAssembly(context.Instance));
                    app        = Application.GetApplicationForDesign(proxyRepPP, provider);
                    if (edSvc != null && app != null) {
                        proxyTK             = new ProxyRepPPTKHelper(context.Instance.GetType().Assembly);
                        atmFullName         = new AtmFullName(proxyTK, context.Instance);
                        objRetVal           = atmFullName.AtomName;
                        listBox             = new PopupListBox(edSvc);
                        listBox.Location    = new System.Drawing.Point(0, 0);
                        listBox.Name        = "listBox";
                        listBox.Size        = new System.Drawing.Size(168, 186);
                        listBox.TabIndex    = 1;
                        listBox.BorderStyle = System.Windows.Forms.BorderStyle.None;
                        strAtomList         = app.GetAtomList(atmFullName.AtomType);
                        listBox.Items.AddRange(strAtomList.Split(','));
                        edSvc.DropDownControl(listBox);
                        if (listBox.SelectedItem != null) {
                            objRetVal = listBox.SelectedItem as string;
                        }
                    }
                }
                if (objRetVal == null) {
                    objRetVal = base.EditValue(context, provider, value);
                }
                return (objRetVal);
            }
        } // End Class AtmNameEditor   
    } // End Class AtmControlDesigner
} // End namespace

