﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing.Design;
using System.ComponentModel;
using System.Windows.Forms.Design;
using System.Reflection;

namespace RepPP.Toolkit.Designer {

    /// <summary>
    /// Field class Designer helper 
    /// </summary>
    public class FldControlDesigner {

        //********************************************************************************
        //
	    /// <summary>
	    /// Implement an editor for the FieldFullName property
	    /// </summary>
        //
        //********************************************************************************
        public class FldControlEditor : System.Drawing.Design.UITypeEditor {

            //********************************************************************************
            //
            /// <summary>
            /// GetEditStyle:           Gets the style of edition
            /// </summary>
            /// <param name="context">  Context</param>
            /// <returns>
            /// Type of editor (modal)
            /// </returns>
            //
            //********************************************************************************
            public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) {
                UITypeEditorEditStyle eRetVal;
                
                if (context != null && context.Instance != null) {
                    eRetVal = UITypeEditorEditStyle.Modal;
                } else {
                    eRetVal = base.GetEditStyle(context);
                }
                return(eRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// CreateRealFldFullName: Create the real object FldFullName from the proxy object
            /// </summary>
            /// <param name="fldFullName">FldFullName Proxy object</param>
            /// <param name="typeFldFullName">Type of the Real FldFullName</param>
            /// <returns>
            /// A Real FldFullName
            /// </returns>
            //
            //********************************************************************************
            private object CreateRealFldFullName(FldFullName fldFullName, Type typeFldFullName) {
                Type[]              arrTypeParams;
                ConstructorInfo     ctorInfos;
                object              objFldFullnameRetVal;

                arrTypeParams        = new Type[] { typeof(String), typeof(String) };
                ctorInfos            = typeFldFullName.GetConstructor(arrTypeParams);
                objFldFullnameRetVal = ctorInfos.Invoke(new Object[] { fldFullName.RowsetName, fldFullName.FieldName });
                return(objFldFullnameRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// EditValue:              Edit a value
            /// </summary>
            /// <param name="context">  Context</param>
            /// <param name="provider"> Provider</param>
            /// <param name="value">    Value to edit</param>
            /// <returns>
            /// New value
            /// </returns>
            //
            //********************************************************************************
            public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) {
                object                          objRetVal = null;
                FormFldControlPropPage          frmFldControlPropPage;
                IFldControl                     fldControl;
                IWindowsFormsEditorService      edSvc;
                ProxyRepPPTKHelper              proxyTK;
                Type                            typeFldFullName;

                if (context != null && context.Instance != null && provider != null) {
                    edSvc = provider.GetService(typeof(IWindowsFormsEditorService)) as IWindowsFormsEditorService;
                    if (edSvc != null) {
                        typeFldFullName       = value.GetType();
                        proxyTK               = new ProxyRepPPTKHelper(typeFldFullName.Assembly);
                        fldControl            = new IFldControl(proxyTK, context.Instance);
                        frmFldControlPropPage = new FormFldControlPropPage(fldControl, new ProxyRepPPHelper(DesignerUtil.FindRepPPAssembly(value)));
                        try {
                            edSvc.ShowDialog(frmFldControlPropPage);
                            objRetVal = CreateRealFldFullName(fldControl.FieldFullName, typeFldFullName);
                        } finally {
                            frmFldControlPropPage.Dispose();
                        }
                    }
                }
                if (objRetVal == null) {
                    objRetVal = base.EditValue(context, provider, value);
                }
                return (objRetVal);
            }
           
        } // End Class FldControlEditor    
    
        //********************************************************************************
        //
	    /// <summary>
	    /// Implement an editor for the RowsetDef name property
	    /// </summary>
        //
        //********************************************************************************
        public class RowsetNameEditor : System.Drawing.Design.UITypeEditor {

            //********************************************************************************
            //
            /// <summary>
            /// GetEditStyle:           Gets the style of edition
            /// </summary>
            /// <param name="context">  Context</param>
            /// <returns>
            /// Type of editor (modal)
            /// </returns>
            //
            //********************************************************************************
            public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) {
                UITypeEditorEditStyle eRetVal;
                
                if (context != null && context.Instance != null) {
                    eRetVal = UITypeEditorEditStyle.DropDown;
                } else {
                    eRetVal = base.GetEditStyle(context);
                }
                return(eRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// EditValue:              Edit a value
            /// </summary>
            /// <param name="context">  Context</param>
            /// <param name="provider"> Provider</param>
            /// <param name="value">    Value to edit</param>
            /// <returns>
            /// New value
            /// </returns>
            //
            //********************************************************************************
            public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) {
                object                      objRetVal = null;
                FldFullName                 fldFullName;
                IWindowsFormsEditorService  edSvc;
                Application                 app;
                RowsetDef                   rsDef;
                PopupListBox                listBox;
                ProxyRepPPHelper            proxyRepPP;
                ProxyRepPPTKHelper          proxyTK;
                string                      strRowsetName;

                if (context != null && context.Instance != null && provider != null) {
                    edSvc      = provider.GetService(typeof(IWindowsFormsEditorService)) as IWindowsFormsEditorService;
                    proxyRepPP = new ProxyRepPPHelper(DesignerUtil.FindRepPPAssembly(context.Instance));
                    app        = Application.GetApplicationForDesign(proxyRepPP, provider);
                    if (edSvc != null && app != null) {
                        proxyTK             = new ProxyRepPPTKHelper(context.Instance.GetType().Assembly);
                        fldFullName         = new FldFullName(proxyTK, context.Instance);
                        objRetVal           = fldFullName.RowsetName;
                        listBox             = new PopupListBox(edSvc);
                        listBox.Location    = new System.Drawing.Point(0, 0);
                        listBox.Name        = "listBox";
                        listBox.Size        = new System.Drawing.Size(168, 186);
                        listBox.TabIndex    = 1;
                        listBox.BorderStyle = System.Windows.Forms.BorderStyle.None;
                        foreach (Object rowsetDef in app.RowsetDefs) {
                            rsDef = new RowsetDef(proxyRepPP, rowsetDef);
                            listBox.Items.Add(rsDef.Name);
                        }
                        edSvc.DropDownControl(listBox);
                        if (listBox.SelectedItem != null) {
                            strRowsetName = listBox.SelectedItem as string;
                            objRetVal     = strRowsetName;
                            if (strRowsetName != fldFullName.RowsetName) {
                                fldFullName.FieldName = "";
                            }
                        }   
                    }
                }
                if (objRetVal == null) {
                    objRetVal = base.EditValue(context, provider, value);
                }
                return (objRetVal);
            }
        } // End Class RowsetNameEditor

        //********************************************************************************
        //
	    /// <summary>
	    /// Implement an editor for the Field name property
	    /// </summary>
        //
        //********************************************************************************
        public class FieldNameEditor : System.Drawing.Design.UITypeEditor {

            //********************************************************************************
            //
            /// <summary>
            /// GetEditStyle:           Gets the style of edition
            /// </summary>
            /// <param name="context">  Context</param>
            /// <returns>
            /// Type of editor (modal)
            /// </returns>
            //
            //********************************************************************************
            public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) {
                UITypeEditorEditStyle eRetVal;
                
                if (context != null && context.Instance != null) {
                    eRetVal = UITypeEditorEditStyle.DropDown;
                } else {
                    eRetVal = base.GetEditStyle(context);
                }
                return(eRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// EditValue:              Edit a value
            /// </summary>
            /// <param name="context">  Context</param>
            /// <param name="provider"> Provider</param>
            /// <param name="value">    Value to edit</param>
            /// <returns>
            /// New value
            /// </returns>
            //
            //********************************************************************************
            public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) {
                object                          objRetVal = null;
                FldFullName                     fldFullName;
                IWindowsFormsEditorService      edSvc;
                Application                     app;
                RowsetDef                       rowsetDef;
                FieldDef                        fieldDef;
                PopupListBox                    listBox;
                ProxyRepPPHelper                proxyRepPP;
                ProxyRepPPTKHelper              proxyTK;

                if (context != null && context.Instance != null && provider != null) {
                    edSvc      = provider.GetService(typeof(IWindowsFormsEditorService)) as IWindowsFormsEditorService;
                    proxyRepPP = new ProxyRepPPHelper(DesignerUtil.FindRepPPAssembly(context.Instance));
                    app        = Application.GetApplicationForDesign(proxyRepPP, provider);
                    if (edSvc != null && app != null) {
                        proxyTK             = new ProxyRepPPTKHelper(context.Instance.GetType().Assembly);
                        fldFullName         = new FldFullName(proxyTK, context.Instance);
                        objRetVal           = fldFullName.RowsetName;
                        listBox             = new PopupListBox(edSvc);
                        listBox.Location    = new System.Drawing.Point(0, 0);
                        listBox.Name        = "listBox";
                        listBox.Size        = new System.Drawing.Size(168, 186);
                        listBox.TabIndex    = 1;
                        listBox.BorderStyle = System.Windows.Forms.BorderStyle.None;
                        rowsetDef           = (fldFullName.RowsetName != String.Empty) ? app.RowsetDefs[fldFullName.RowsetName] : null;
                        if (rowsetDef != null) {
                            foreach (Object fldDef in rowsetDef.FieldDefs) {
                                fieldDef = new FieldDef(proxyRepPP, fldDef);
                                listBox.Items.Add(fieldDef.Name);
                            }
                        }
                        edSvc.DropDownControl(listBox);
                        if (listBox.SelectedItem != null) {
                            objRetVal = listBox.SelectedItem as string;
                        }   
                    }
                }
                if (objRetVal == null) {
                    objRetVal = base.EditValue(context, provider, value);
                }
                return (objRetVal);
            }
        } // End FieldNameEditor Class

    } //End FldControlDesigner Class 

}// End namespace
