using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Reflection;

namespace RepPP.Toolkit.Designer {
    //********************************************************************************
    //
	/// <summary>
	/// Implement the property page to pickup a Rep++ Field
	/// </summary>
    //
    //********************************************************************************
	public class FormFldControlPropPage : System.Windows.Forms.Form	{
		/// <summary>Required designer variable.</summary>
		private System.ComponentModel.Container     components = null;
		/// <summary>Field control being edited</summary>
		private IFldControl                         m_fldControl;
		/// <summary>true if filling the combobox</summary>
		private bool                                m_bFilling;
        /// <summary>Rep++ Proxy helper</summary>
        private ProxyRepPPHelper                    m_proxyRepPP;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                      Class constructor
        /// </summary>
		/// <param name="fldControl">Field control being edited</param>
        /// <param name="proxyRepPP">Rep++ Proxy helper</param>
        //
        //********************************************************************************
		public FormFldControlPropPage(IFldControl fldControl, ProxyRepPPHelper proxyRepPP) {
			InitializeComponent();
		    m_fldControl = fldControl;
            m_proxyRepPP = proxyRepPP;
		    FillRowsetCombo();
		    SetApplyButtonState();
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //********************************************************************************
		public FormFldControlPropPage()	: this(null, null) {
		}

        //********************************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the object
        /// </summary>
        /// <param name="bDisposing">   true if disposing, false if GC</param>
        //
        //********************************************************************************
		protected override void Dispose(bool bDisposing) {
			if (bDisposing)	{
				if (components != null)	{
					components.Dispose();
				}
			}
			base.Dispose(bDisposing);
		}

        //********************************************************************************
        //
        /// <summary>
        /// FillRowsetCombo:            Fills the rowset combo box with the list of rowset
        /// </summary>
        //
        //********************************************************************************
        private void FillRowsetCombo() {
            FldFullName         fldFullName;
            Application         app;
            RowsetDef           rsDef;
            int                 iIndex;
            
            m_bFilling = true;
            app        = (m_fldControl == null) ? null : new Application(m_proxyRepPP, m_fldControl.Application);
            comboBoxRowsetName.Items.Clear();
            if (app != null) {
                fldFullName = m_fldControl.FieldFullName;
                foreach (Object rowsetDef in app.RowsetDefs) {
                    rsDef = new RowsetDef(m_proxyRepPP, rowsetDef);
                    comboBoxRowsetName.Items.Add(rsDef.Name);
                }
                if (fldFullName.RowsetName != String.Empty) {
                    iIndex = comboBoxRowsetName.Items.IndexOf(fldFullName.RowsetName);
                    comboBoxRowsetName.SelectedIndex = iIndex;
                }
            }
            m_bFilling = false;
            if (comboBoxRowsetName.SelectedIndex == -1 && comboBoxFldName.Items.Count != 0) {
                comboBoxRowsetName.SelectedIndex = 0;
            }
            FillFieldCombo();
        }

        //********************************************************************************
        //
        /// <summary>
        /// FillFieldCombo:         Fills the field combo box with the list of fields
        /// </summary>
        //
        //********************************************************************************
        private void FillFieldCombo() {
            Application         app;
            RowsetDef           rowsetDef;
            FieldDef            fldDef;
            int                 iIndex;
            FldFullName         fldFullName;
            
            m_bFilling = true;
            app        = (m_fldControl == null) ? null : new Application(m_proxyRepPP, m_fldControl.Application);
            comboBoxFldName.Items.Clear();
            if (app != null) {
                fldFullName = m_fldControl.FieldFullName;
                if (comboBoxRowsetName.SelectedIndex != -1) {
                    rowsetDef = app.RowsetDefs[comboBoxRowsetName.SelectedItem as string];
                    if (rowsetDef != null) {
                        foreach(Object fieldDef in rowsetDef.FieldDefs) {
                            fldDef = new FieldDef(m_proxyRepPP, fieldDef);
                            comboBoxFldName.Items.Add(fldDef.Name);
                        }
                        if (fldFullName.FieldName != String.Empty) {
                            iIndex                        = comboBoxFldName.Items.IndexOf(fldFullName.FieldName);
                            comboBoxFldName.SelectedIndex = iIndex;
                        }
                    }
                }
            }
            m_bFilling = false;
            if (comboBoxFldName.SelectedIndex == -1 && comboBoxFldName.Items.Count != 0) {
                comboBoxFldName.SelectedIndex = 0;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetApplyButtonState:    Enable or disable the apply button depending on the context
        /// </summary>
        //
        //********************************************************************************
        private void SetApplyButtonState() {
            bool        bState = false;
            int         iIndex1;
            int         iIndex2;
            FldFullName fldFullName;
            
            if (m_fldControl != null) {                
                fldFullName = m_fldControl.FieldFullName;
                iIndex1     = comboBoxRowsetName.SelectedIndex;
                iIndex2     = comboBoxFldName.SelectedIndex;
                if (iIndex1 != -1 && iIndex2 != -1) {
                    if (comboBoxRowsetName.Items[iIndex1].ToString() != fldFullName.RowsetName ||
                        comboBoxFldName.Items[iIndex2].ToString()    != fldFullName.FieldName) {
                        bState = true;
                    }
                }
                butApply.Enabled = bState;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Apply:          Apply the change
        /// </summary>
        //
        //********************************************************************************
        private void Apply() {
            String          strRowsetName;
            String          strFieldName;
            int             iIndex1;
            int             iIndex2;
            
            if (m_fldControl != null) {
                iIndex1 = comboBoxRowsetName.SelectedIndex;
                iIndex2 = comboBoxFldName.SelectedIndex;
                if (iIndex1 != -1 && iIndex2 != -1) {
                    strRowsetName = comboBoxRowsetName.Items[iIndex1] as String;
                    strFieldName  = comboBoxFldName.Items[iIndex2] as String;
                    if (strRowsetName != m_fldControl.FieldFullName.RowsetName ||
                        strFieldName  != m_fldControl.FieldFullName.FieldName) {
                        m_fldControl.FieldFullName.RowsetName = strRowsetName;
                        m_fldControl.FieldFullName.FieldName  = strFieldName;
                        m_fldControl.TriggerComponentChange();
                    }
                }
            }
            butApply.Enabled = false;
        }

        //********************************************************************************
        //
        /// <summary>
        /// butOk_Click:            Called when the OK button is pressed
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event parameters</param>
        //
        //********************************************************************************
        private void butOk_Click(object sender, System.EventArgs e) {
            Apply();
        }

        //********************************************************************************
        //
        /// <summary>
        /// butApply_Click:         Called when the Apply button is pressed
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event parameters</param>
        //
        //********************************************************************************
        private void butApply_Click(object sender, System.EventArgs e) {
            Apply();
        }

        //********************************************************************************
        //
        /// <summary>
        /// comboBoxRowsetName_SelectedIndexChanged:    Called when the combobox selection is changed
        /// </summary>
        /// <param name="sender">                       Sender object</param>
        /// <param name="e">                            Event parameters</param>
        //
        //********************************************************************************
        private void comboBoxRowsetName_SelectedIndexChanged(object sender, System.EventArgs e) {
            if (!m_bFilling) {
                FillFieldCombo();
                SetApplyButtonState();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// comboBoxFldName_SelectedIndexChanged:   Called when the combobox selection is changed
        /// </summary>
        /// <param name="sender">                   Sender object</param>
        /// <param name="e">                        Event parameters</param>
        //
        //********************************************************************************
        private void comboBoxFldName_SelectedIndexChanged(object sender, System.EventArgs e) {
            if (!m_bFilling) {
                SetApplyButtonState();
            }
        }

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
        private System.Windows.Forms.ComboBox       comboBoxRowsetName;
        private System.Windows.Forms.ComboBox       comboBoxFldName;
        private System.Windows.Forms.Button         butOk;
        private System.Windows.Forms.Button         butCancel;
        private System.Windows.Forms.Button         butApply;
        private System.Windows.Forms.Label          labelRowsetName;
        private System.Windows.Forms.Label          labelFldName;

		private void InitializeComponent()
		{
            this.comboBoxRowsetName = new System.Windows.Forms.ComboBox();
            this.comboBoxFldName = new System.Windows.Forms.ComboBox();
            this.butOk = new System.Windows.Forms.Button();
            this.butCancel = new System.Windows.Forms.Button();
            this.butApply = new System.Windows.Forms.Button();
            this.labelRowsetName = new System.Windows.Forms.Label();
            this.labelFldName = new System.Windows.Forms.Label();
            this.SuspendLayout();
            // 
            // comboBoxRowsetName
            // 
            this.comboBoxRowsetName.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.comboBoxRowsetName.Location = new System.Drawing.Point(152, 32);
            this.comboBoxRowsetName.Name = "comboBoxRowsetName";
            this.comboBoxRowsetName.Size = new System.Drawing.Size(176, 21);
            this.comboBoxRowsetName.TabIndex = 0;
            this.comboBoxRowsetName.SelectedIndexChanged += new System.EventHandler(this.comboBoxRowsetName_SelectedIndexChanged);
            // 
            // comboBoxFldName
            // 
            this.comboBoxFldName.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.comboBoxFldName.Location = new System.Drawing.Point(152, 72);
            this.comboBoxFldName.Name = "comboBoxFldName";
            this.comboBoxFldName.Size = new System.Drawing.Size(176, 21);
            this.comboBoxFldName.TabIndex = 1;
            this.comboBoxFldName.SelectedIndexChanged += new System.EventHandler(this.comboBoxFldName_SelectedIndexChanged);
            // 
            // butOk
            // 
            this.butOk.DialogResult = System.Windows.Forms.DialogResult.OK;
            this.butOk.Location = new System.Drawing.Point(144, 144);
            this.butOk.Name = "butOk";
            this.butOk.TabIndex = 2;
            this.butOk.Text = "OK";
            this.butOk.Click += new System.EventHandler(this.butOk_Click);
            // 
            // butCancel
            // 
            this.butCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            this.butCancel.Location = new System.Drawing.Point(232, 144);
            this.butCancel.Name = "butCancel";
            this.butCancel.TabIndex = 3;
            this.butCancel.Text = "Cancel";
            // 
            // butApply
            // 
            this.butApply.Location = new System.Drawing.Point(320, 144);
            this.butApply.Name = "butApply";
            this.butApply.TabIndex = 4;
            this.butApply.Text = "Apply";
            this.butApply.Click += new System.EventHandler(this.butApply_Click);
            // 
            // labelRowsetName
            // 
            this.labelRowsetName.Location = new System.Drawing.Point(40, 32);
            this.labelRowsetName.Name = "labelRowsetName";
            this.labelRowsetName.Size = new System.Drawing.Size(88, 23);
            this.labelRowsetName.TabIndex = 5;
            this.labelRowsetName.Text = "Rowset";
            // 
            // labelFldName
            // 
            this.labelFldName.Location = new System.Drawing.Point(40, 72);
            this.labelFldName.Name = "labelFldName";
            this.labelFldName.Size = new System.Drawing.Size(96, 23);
            this.labelFldName.TabIndex = 6;
            this.labelFldName.Text = "Field";
            // 
            // FormFldControlPropPage
            // 
            this.AcceptButton = this.butOk;
            this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
            this.CancelButton = this.butCancel;
            this.ClientSize = new System.Drawing.Size(408, 182);
            this.Controls.Add(this.labelFldName);
            this.Controls.Add(this.labelRowsetName);
            this.Controls.Add(this.butApply);
            this.Controls.Add(this.butCancel);
            this.Controls.Add(this.butOk);
            this.Controls.Add(this.comboBoxFldName);
            this.Controls.Add(this.comboBoxRowsetName);
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedToolWindow;
            this.Name = "FormFldControlPropPage";
            this.Text = "Rep++ Field Property Page";
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            this.ResumeLayout(false);

        }
		#endregion

	} // End FormFldControlPropPage Class 
} // End Namespace
