using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace RepPP.Toolkit.Designer {
    //********************************************************************************
    //
	/// <summary>
	/// Implement the property page to pickup a Rep++ RowsetTreeDefNode
	/// </summary>
    //
    //********************************************************************************
	public class FormRowsetTreeDefNodeControlPropPage : System.Windows.Forms.Form	{
		/// <summary>Required designer variable.</summary>
		private System.ComponentModel.Container     components = null;
		/// <summary>Field control being edited</summary>
		private IRowsetTreeDefNodeControl           m_nodeControl;
		/// <summary>true if filling the combobox</summary>
		private bool                                m_bFilling;
        /// <summary>Rep++ Proxy helper</summary>
        private ProxyRepPPHelper                    m_proxyRepPP;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
		/// <param name="nodeControl">RowsetTreeDefNode control being edited</param>
        /// <param name="proxyRepPP">Rep++ Proxy helper</param>
        //
        //********************************************************************************
		public FormRowsetTreeDefNodeControlPropPage(IRowsetTreeDefNodeControl nodeControl, ProxyRepPPHelper proxyRepPP) {
			InitializeComponent();
		    m_nodeControl = nodeControl;
            m_proxyRepPP  = proxyRepPP;
		    FillRowsetTreeDefCombo();
		    SetApplyButtonState();
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        //
        //********************************************************************************
		public FormRowsetTreeDefNodeControlPropPage()	: this(null, null) {
		}

        //********************************************************************************
        //
        /// <summary>
        /// Dispose:                        Dispose the object
        /// </summary>
        /// <param name="bDisposing">       true if disposing, false if GC</param>
        //
        //********************************************************************************
		protected override void Dispose(bool bDisposing) {
			if (bDisposing)	{
				if (components != null)	{
					components.Dispose();
				}
			}
			base.Dispose(bDisposing);
		}

        //********************************************************************************
        //
        /// <summary>
        /// FillRowsetTreeDefCombo:         Fills the card combo box with the list of rowset
        /// </summary>
        //
        //********************************************************************************
        private void FillRowsetTreeDefCombo() {
            Application                 app;
            RowsetTreeDefNodeFullName   nodeFullName;
            RowsetTreeDef               rstDef;
            int                         iIndex;
            
            m_bFilling = true;
            app        = (m_nodeControl == null) ? null : new Application(m_proxyRepPP, m_nodeControl.Application);
            comboBoxRowsetTreeDefName.Items.Clear();
            if (app != null) {
                nodeFullName    = m_nodeControl.RowsetTreeDefNodeFullName;
                foreach (Object rowsetTreeDef in app.RowsetTreeDefs) {
                    rstDef = new RowsetTreeDef(m_proxyRepPP, rowsetTreeDef);
                    if (String.Compare(rstDef.Name, 0, "$HIDE", 0, 5) != 0) {
                        comboBoxRowsetTreeDefName.Items.Add(rstDef.Name);
                    }
                }
                if (nodeFullName.RowsetTreeName != String.Empty) {
                    iIndex = comboBoxRowsetTreeDefName.Items.IndexOf(nodeFullName.RowsetTreeName);
                    comboBoxRowsetTreeDefName.SelectedIndex = iIndex;
                }
            }
            m_bFilling = false;
            if (comboBoxRowsetTreeDefName.SelectedIndex == -1 && comboBoxRowsetTreeDefName.Items.Count != 0) {
                comboBoxRowsetTreeDefName.SelectedIndex = 0;
            }
            FillRowsetDefCombo();
        }

        //********************************************************************************
        //
        /// <summary>
        /// FillRowsetDefCombo:         Fills the group combo box with the list of card groups
        /// </summary>
        //
        //********************************************************************************
        private void FillRowsetDefCombo() {
            Application                 app;
            RowsetTreeDef               rowsetTreeDef;
            int                         iIndex;
            RowsetTreeDefNodeFullName   nodeFullName;
            
            m_bFilling = true;
            app        = (m_nodeControl == null) ? null : new Application(m_proxyRepPP, m_nodeControl.Application);
            comboBoxRowsetName.Items.Clear();
            if (app != null) {
                nodeFullName = m_nodeControl.RowsetTreeDefNodeFullName;
                if (comboBoxRowsetTreeDefName.SelectedIndex != -1) {
                    rowsetTreeDef = app.RowsetTreeDefs[comboBoxRowsetTreeDefName.SelectedItem as string];
                    if (rowsetTreeDef != null) {
                        FillNodeRowsetNames(rowsetTreeDef.Nodes, comboBoxRowsetName);
                        if (nodeFullName.RowsetName != String.Empty) {
                            iIndex                           = comboBoxRowsetName.Items.IndexOf(nodeFullName.RowsetName);
                            comboBoxRowsetName.SelectedIndex = iIndex;
                        }
                    }
                }
            }
            m_bFilling = false;
            if (comboBoxRowsetName.SelectedIndex == -1 && comboBoxRowsetName.Items.Count != 0) {
                comboBoxRowsetName.SelectedIndex = 0;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FillNodeRowsetNames:        Populates a combo box with the names of all ChildRowsetDefs in
        ///                             a rowset tree
        /// </summary>
        /// <param name="nodes">        Nodes of the rowset tree definition</param>
        /// <param name="comboBox">     ComboBox Control</param>
        //
        //********************************************************************************
        internal void FillNodeRowsetNames(RowsetTreeDefNodes nodes, ComboBox comboBox) {
            RowsetTreeDefNode rstDefNode;

            foreach (Object node in nodes) {
                rstDefNode = new RowsetTreeDefNode(m_proxyRepPP, node);
                if (String.Compare(rstDefNode.Name, 0, "$HIDE", 0, 5) != 0) {
                    comboBox.Items.Add(rstDefNode.Name);
                    FillNodeRowsetNames(rstDefNode.Nodes, comboBox);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetApplyButtonState:    Enable or disable the apply button depending on the context
        /// </summary>
        //
        //********************************************************************************
        private void SetApplyButtonState() {
            bool                        bState = false;
            int                         iIndex1;
            int                         iIndex2;
            RowsetTreeDefNodeFullName   nodeFullName;
            
            if (m_nodeControl != null) {
                nodeFullName    = m_nodeControl.RowsetTreeDefNodeFullName;
                iIndex1         = comboBoxRowsetTreeDefName.SelectedIndex;
                iIndex2         = comboBoxRowsetName.SelectedIndex;
                if (iIndex1 != -1 && iIndex2 != -1) {
                    if (comboBoxRowsetTreeDefName.Items[iIndex1].ToString() != nodeFullName.RowsetTreeName ||
                        comboBoxRowsetName.Items[iIndex2].ToString() != nodeFullName.RowsetName) {
                        bState = true;
                    }
                }
                butApply.Enabled = bState;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Apply:          Apply the change
        /// </summary>
        //
        //********************************************************************************
        private void Apply() {
            String                      strRowsetTreeName;
            String                      strRowsetName;
            int                         iIndex1;
            int                         iIndex2;
            
            if (m_nodeControl != null) {
                iIndex1 = comboBoxRowsetTreeDefName.SelectedIndex;
                iIndex2 = comboBoxRowsetName.SelectedIndex;
                if (iIndex1 != -1 && iIndex2 != -1) {
                    strRowsetTreeName = comboBoxRowsetTreeDefName.Items[iIndex1] as String;
                    strRowsetName     = comboBoxRowsetName.Items[iIndex2] as String;
                    if (strRowsetTreeName != m_nodeControl.RowsetTreeDefNodeFullName.RowsetTreeName ||
                        strRowsetName != m_nodeControl.RowsetTreeDefNodeFullName.RowsetName) {
                        m_nodeControl.RowsetTreeDefNodeFullName.RowsetTreeName = strRowsetTreeName;
                        m_nodeControl.RowsetTreeDefNodeFullName.RowsetName     = strRowsetName;
                        m_nodeControl.TriggerComponentChange();
                    }
                }
            }
            butApply.Enabled = false;
        }

        //********************************************************************************
        //
        /// <summary>
        /// butOk_Click:            Called when the OK button is pressed
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event parameters</param>
        //
        //********************************************************************************
        private void butOk_Click(object sender, System.EventArgs e) {
            Apply();
        }

        //********************************************************************************
        //
        /// <summary>
        /// butApply_Click:         Called when the Apply button is pressed
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event parameters</param>
        //
        //********************************************************************************
        private void butApply_Click(object sender, System.EventArgs e) {
            Apply();
        }

        //********************************************************************************
        //
        /// <summary>
        /// comboBoxRowsetTreeDefName_SelectedIndexChanged:   Called when the combobox selection is changed
        /// </summary>
        /// <param name="sender">                   Sender object</param>
        /// <param name="e">                        Event parameters</param>
        //
        //********************************************************************************
        private void comboBoxRowsetTreeDefName_SelectedIndexChanged(object sender, System.EventArgs e) {
            if (!m_bFilling) {
                FillRowsetDefCombo();
                SetApplyButtonState();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// comboBoxRowsetName_SelectedIndexChanged:   Called when the combobox selection is changed
        /// </summary>
        /// <param name="sender">                   Sender object</param>
        /// <param name="e">                        Event parameters</param>
        //
        //********************************************************************************
        private void comboBoxRowsetName_SelectedIndexChanged(object sender, System.EventArgs e) {
            if (!m_bFilling) {
                SetApplyButtonState();
            }
        }

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
        private System.Windows.Forms.Button butOk;
        private System.Windows.Forms.Button butCancel;
        private System.Windows.Forms.ComboBox comboBoxRowsetTreeDefName;
        private System.Windows.Forms.ComboBox comboBoxRowsetName;
        private System.Windows.Forms.Label labelRowsetTreeDefName;
        private System.Windows.Forms.Label labelRowsetName;
        private System.Windows.Forms.Button butApply;

		private void InitializeComponent()
		{
            this.comboBoxRowsetTreeDefName = new System.Windows.Forms.ComboBox();
            this.comboBoxRowsetName = new System.Windows.Forms.ComboBox();
            this.butOk = new System.Windows.Forms.Button();
            this.butCancel = new System.Windows.Forms.Button();
            this.butApply = new System.Windows.Forms.Button();
            this.labelRowsetTreeDefName = new System.Windows.Forms.Label();
            this.labelRowsetName = new System.Windows.Forms.Label();
            this.SuspendLayout();
            // 
            // comboBoxRowsetTreeDefName
            // 
            this.comboBoxRowsetTreeDefName.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.comboBoxRowsetTreeDefName.Location = new System.Drawing.Point(152, 32);
            this.comboBoxRowsetTreeDefName.Name = "comboBoxRowsetTreeDefName";
            this.comboBoxRowsetTreeDefName.Size = new System.Drawing.Size(224, 21);
            this.comboBoxRowsetTreeDefName.TabIndex = 0;
            this.comboBoxRowsetTreeDefName.SelectedIndexChanged += new System.EventHandler(this.comboBoxRowsetTreeDefName_SelectedIndexChanged);
            // 
            // comboBoxRowsetName
            // 
            this.comboBoxRowsetName.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.comboBoxRowsetName.Location = new System.Drawing.Point(152, 69);
            this.comboBoxRowsetName.Name = "comboBoxRowsetName";
            this.comboBoxRowsetName.Size = new System.Drawing.Size(224, 21);
            this.comboBoxRowsetName.TabIndex = 1;
            this.comboBoxRowsetName.SelectedIndexChanged += new System.EventHandler(this.comboBoxRowsetName_SelectedIndexChanged);
            // 
            // butOk
            // 
            this.butOk.DialogResult = System.Windows.Forms.DialogResult.OK;
            this.butOk.Location = new System.Drawing.Point(144, 144);
            this.butOk.Name = "butOk";
            this.butOk.Size = new System.Drawing.Size(75, 23);
            this.butOk.TabIndex = 2;
            this.butOk.Text = "OK";
            this.butOk.Click += new System.EventHandler(this.butOk_Click);
            // 
            // butCancel
            // 
            this.butCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            this.butCancel.Location = new System.Drawing.Point(232, 144);
            this.butCancel.Name = "butCancel";
            this.butCancel.Size = new System.Drawing.Size(75, 23);
            this.butCancel.TabIndex = 3;
            this.butCancel.Text = "Cancel";
            // 
            // butApply
            // 
            this.butApply.Location = new System.Drawing.Point(320, 144);
            this.butApply.Name = "butApply";
            this.butApply.Size = new System.Drawing.Size(75, 23);
            this.butApply.TabIndex = 4;
            this.butApply.Text = "Apply";
            this.butApply.Click += new System.EventHandler(this.butApply_Click);
            // 
            // labelRowsetTreeDefName
            // 
            this.labelRowsetTreeDefName.AutoSize = true;
            this.labelRowsetTreeDefName.Location = new System.Drawing.Point(8, 32);
            this.labelRowsetTreeDefName.Name = "labelRowsetTreeDefName";
            this.labelRowsetTreeDefName.Size = new System.Drawing.Size(82, 13);
            this.labelRowsetTreeDefName.TabIndex = 5;
            this.labelRowsetTreeDefName.Text = "RowsetTreeDef";
            // 
            // labelRowsetName
            // 
            this.labelRowsetName.AutoSize = true;
            this.labelRowsetName.Location = new System.Drawing.Point(8, 72);
            this.labelRowsetName.Name = "labelRowsetName";
            this.labelRowsetName.Size = new System.Drawing.Size(60, 13);
            this.labelRowsetName.TabIndex = 6;
            this.labelRowsetName.Text = "RowsetDef";
            // 
            // FormRowsetTreeDefNodeControlPropPage
            // 
            this.AcceptButton = this.butOk;
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.CancelButton = this.butCancel;
            this.ClientSize = new System.Drawing.Size(408, 182);
            this.Controls.Add(this.labelRowsetName);
            this.Controls.Add(this.labelRowsetTreeDefName);
            this.Controls.Add(this.butApply);
            this.Controls.Add(this.butCancel);
            this.Controls.Add(this.butOk);
            this.Controls.Add(this.comboBoxRowsetName);
            this.Controls.Add(this.comboBoxRowsetTreeDefName);
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedToolWindow;
            this.Name = "FormRowsetTreeDefNodeControlPropPage";
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            this.Text = "Rep++ Child Rowset Definition Property Page";
            this.ResumeLayout(false);
            this.PerformLayout();

        }
		#endregion

    } // End FormRowsetTreeDefNodeControlPropPage Class
} // End Namespace
