﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing.Design;
using System.ComponentModel;
using System.Windows.Forms.Design;

namespace RepPP.Toolkit.Designer  {

    /// <summary>
    /// Rowset class Designer helper 
    /// </summary>
    public class RowsetControlDesigner {
        //********************************************************************************
        //
	    /// <summary>
	    /// Implement an editor for the Rowset name property
	    /// </summary>
        //
        //********************************************************************************
        public class RowsetNameEditor : System.Drawing.Design.UITypeEditor {

            //********************************************************************************
            //
            /// <summary>
            /// GetEditStyle:           Gets the style of edition
            /// </summary>
            /// <param name="context">  Context</param>
            /// <returns>
            /// Type of editor (modal)
            /// </returns>
            //
            //********************************************************************************
            public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) {
                UITypeEditorEditStyle eRetVal;
                
                if (context != null && context.Instance != null) {
                    eRetVal = UITypeEditorEditStyle.DropDown;
                } else {
                    eRetVal = base.GetEditStyle(context);
                }
                return(eRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// EditValue:              Edit a value
            /// </summary>
            /// <param name="context">  Context</param>
            /// <param name="provider"> Provider</param>
            /// <param name="value">    Value to edit</param>
            /// <returns>
            /// New value
            /// </returns>
            //
            //********************************************************************************
            public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) {
                object                      objRetVal = null;
                IRowsetControl              rowsetControl;
                IWindowsFormsEditorService  edSvc;
                Application                 app;
                RowsetDef                   rsDef;
                PopupListBox                listBox;
                ProxyRepPPHelper            proxyRepPP;
                ProxyRepPPTKHelper          proxyTK;
                string                      strRowsetName;

                if (context != null && context.Instance != null && provider != null) {
                    edSvc      = provider.GetService(typeof(IWindowsFormsEditorService)) as IWindowsFormsEditorService;
                    proxyRepPP = new ProxyRepPPHelper(DesignerUtil.FindRepPPAssembly(context.Instance));
                    app        = Application.GetApplicationForDesign(proxyRepPP, provider);
                    if (edSvc != null && app != null) {
                        proxyTK             = new ProxyRepPPTKHelper(context.Instance.GetType().GetInterface("IRowsetControl").Assembly);
                        rowsetControl       = new IRowsetControl(proxyTK, context.Instance);
                        objRetVal           = rowsetControl.RowsetName;
                        listBox             = new PopupListBox(edSvc);
                        listBox.Location    = new System.Drawing.Point(0, 0);
                        listBox.Name        = "listBox";
                        listBox.Size        = new System.Drawing.Size(168, 186);
                        listBox.TabIndex    = 1;
                        listBox.BorderStyle = System.Windows.Forms.BorderStyle.None;
                        listBox.Sorted      = true;
                        foreach (Object rowsetDef in app.RowsetDefs) {
                            rsDef = new RowsetDef(proxyRepPP, rowsetDef);
                            if (String.Compare(rsDef.Name, 0, "$HIDE", 0, 5) != 0) {
                                listBox.Items.Add(rsDef.Name);
                            }
                        }
                        edSvc.DropDownControl(listBox);
                        if (listBox.SelectedItem != null) {
                            strRowsetName = listBox.SelectedItem as string;
                            objRetVal     = strRowsetName;
                        }   
                    }
                }
                if (objRetVal == null) {
                    objRetVal = base.EditValue(context, provider, value);
                }
                return (objRetVal);
            }
        } // End Class RowsetNameEditor
    } // End Class RowsetControlDesigner

}// End namespace
