﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Windows.Forms.Design;
using System.Drawing.Design;
using System.Reflection;

namespace RepPP.Toolkit.Designer {

    /// <summary>
    /// RowsetTreeDefNode class Designer helper 
    /// </summary>
	public class RowsetTreeDefNodeControlDesigner {
          //********************************************************************************
        //
        /// <summary>
        /// Implement an editor for the RowsetTreeDefNodeFullName property
        /// </summary>
        //
        //********************************************************************************
        public class NodeEditor : System.Drawing.Design.UITypeEditor {

            //********************************************************************************
            //
            /// <summary>
            /// GetEditStyle:           Gets the style of edition
            /// </summary>
            /// <param name="context">  Context</param>
            /// <returns>
            /// Type of editor (modal)
            /// </returns>
            //
            //********************************************************************************
            public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) {
                UITypeEditorEditStyle eRetVal;
                
                if (context != null && context.Instance != null) {
                    eRetVal = UITypeEditorEditStyle.Modal;
                } else {
                    eRetVal = base.GetEditStyle(context);
                }
                return(eRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// CreateRealRstDefNodeFullName: Create the real object RowsetTreeDefNodeFullName \
            ///                               from the proxy object
            /// </summary>
            /// <param name="rstDefNodeFullName">RowsetTreeDefNodeFullName Proxy object</param>
            /// <param name="typeRstDefNodeFullName">Type of the Real AtmFullName</param>
            /// <returns>
            /// A Real RowsetTreeDefNodeFullName
            /// </returns>
            //
            //********************************************************************************
            private object CreateRealRstDefNodeFullName(RowsetTreeDefNodeFullName rstDefNodeFullName, Type typeRstDefNodeFullName) {
                Type[]              arrTypeParams;
                ConstructorInfo     ctorInfos;
                object              objRstDegNodeFullnameRetVal;

                arrTypeParams               = new Type[] { typeof(String), typeof(String) };
                ctorInfos                   = typeRstDefNodeFullName.GetConstructor(arrTypeParams);
                objRstDegNodeFullnameRetVal = ctorInfos.Invoke(new Object[] { rstDefNodeFullName.RowsetTreeName, rstDefNodeFullName.RowsetName });
                return(objRstDegNodeFullnameRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// EditValue:              Edit a value
            /// </summary>
            /// <param name="context">  Context</param>
            /// <param name="provider"> Provider</param>
            /// <param name="value">    Value to edit</param>
            /// <returns>
            /// New value
            /// </returns>
            //
            //********************************************************************************
            public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) {
                object                                  objRetVal = null;
                FormRowsetTreeDefNodeControlPropPage    frmNodeControlPropPage;
                IRowsetTreeDefNodeControl               nodeDefControl;
                IWindowsFormsEditorService              edSvc;
                ProxyRepPPTKHelper                      proxyTK;
                Type                                    typeRstDefFullName;

                if (context != null && context.Instance != null && provider != null) {
                    edSvc = provider.GetService(typeof(IWindowsFormsEditorService)) as IWindowsFormsEditorService;
                    if (edSvc != null) {
                        typeRstDefFullName = value.GetType();
                        proxyTK            = new ProxyRepPPTKHelper(typeRstDefFullName.Assembly);
                        nodeDefControl     = new IRowsetTreeDefNodeControl(proxyTK, context.Instance);
                        if (nodeDefControl == null) {
                            nodeDefControl = new IRowsetTreeDefNodeControl(proxyTK, (context.Instance as System.ComponentModel.Design.DesignerActionList).Component);
                        }
                        frmNodeControlPropPage = new FormRowsetTreeDefNodeControlPropPage(nodeDefControl, new ProxyRepPPHelper(DesignerUtil.FindRepPPAssembly(value)));
                        try {
                            edSvc.ShowDialog(frmNodeControlPropPage);
                            objRetVal = CreateRealRstDefNodeFullName(nodeDefControl.RowsetTreeDefNodeFullName, typeRstDefFullName);
                        } finally {
                            frmNodeControlPropPage.Dispose();
                        }
                    }
                }
                if (objRetVal == null) {
                    objRetVal = base.EditValue(context, provider, value);
                }
                return (objRetVal);
            }
        } // Class ChildRowsetDefControlEditor

        //********************************************************************************
        //
        /// <summary>
        /// Implement an editor for the RowsetTreeDef name property
        /// </summary>
        //
        //********************************************************************************
        public class RowsetTreeNameEditor : System.Drawing.Design.UITypeEditor {

            //********************************************************************************
            //
            /// <summary>
            /// GetEditStyle:           Gets the style of edition
            /// </summary>
            /// <param name="context">  Context</param>
            /// <returns>
            /// Type of editor (modal)
            /// </returns>
            //
            //********************************************************************************
            public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) {
                UITypeEditorEditStyle eRetVal;
                
                if (context != null && context.Instance != null) {
                    eRetVal = UITypeEditorEditStyle.DropDown;
                } else {
                    eRetVal = base.GetEditStyle(context);
                }
                return(eRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// EditValue:              Edit a value
            /// </summary>
            /// <param name="context">  Context</param>
            /// <param name="provider"> Provider</param>
            /// <param name="value">    Value to edit</param>
            /// <returns>
            /// New value
            /// </returns>
            //
            //********************************************************************************
            public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) {
                object                          objRetVal = null;
                RowsetTreeDefNodeFullName       nodeDefFullName;
                IWindowsFormsEditorService      edSvc;
                Application                     app;
                RowsetTreeDef                   rstDef;
                PopupListBox                    listBox;
                ProxyRepPPHelper                proxyRepPP;
                ProxyRepPPTKHelper              proxyTK;
                String                          strRowsetTreeName;

                if (context != null && context.Instance != null && provider != null) {
                    edSvc      = provider.GetService(typeof(IWindowsFormsEditorService)) as IWindowsFormsEditorService;
                    proxyRepPP = new ProxyRepPPHelper(DesignerUtil.FindRepPPAssembly(context.Instance));
                    app        = Application.GetApplicationForDesign(proxyRepPP, provider);
                    if (edSvc != null && app != null) {
                        proxyTK             = new ProxyRepPPTKHelper(context.Instance.GetType().Assembly);
                        nodeDefFullName     = new RowsetTreeDefNodeFullName(proxyTK, context.Instance);
                        objRetVal           = nodeDefFullName.RowsetTreeName;
                        listBox             = new PopupListBox(edSvc);
                        listBox.Location    = new System.Drawing.Point(0, 0);
                        listBox.Name        = "listBox";
                        listBox.Size        = new System.Drawing.Size(168, 186);
                        listBox.TabIndex    = 1;
                        listBox.BorderStyle = System.Windows.Forms.BorderStyle.None;
                        listBox.Sorted      = true;
                        foreach (Object rowsetTreeDef in app.RowsetTreeDefs) {
                            rstDef = new RowsetTreeDef(proxyRepPP, rowsetTreeDef);
                            if (String.Compare(rstDef.Name, 0, "$HIDE", 0, 5) != 0) {
                                listBox.Items.Add(rstDef.Name);
                            }
                        }
                        edSvc.DropDownControl(listBox);
                        if (listBox.SelectedItem != null) {
                            strRowsetTreeName = listBox.SelectedItem as string;
                            objRetVal         = strRowsetTreeName;
                            if (strRowsetTreeName != nodeDefFullName.RowsetTreeName) {
                                nodeDefFullName.RowsetName = "";
                            }
                        }
                    }
                }
                if (objRetVal == null) {
                    objRetVal = base.EditValue(context, provider, value);
                }
                return (objRetVal);
            }
        } // Class RowsetTreeDefNameEditor

        //********************************************************************************
        //
        /// <summary>
        /// Implement an editor for the RowsetTreeDefNode name property
        /// </summary>
        //
        //********************************************************************************
        public class RowsetNameEditor : System.Drawing.Design.UITypeEditor {

            //********************************************************************************
            //
            /// <summary>
            /// GetEditStyle:           Gets the style of edition
            /// </summary>
            /// <param name="context">  Context</param>
            /// <returns>
            /// Type of editor (modal)
            /// </returns>
            //
            //********************************************************************************
            public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) {
                UITypeEditorEditStyle eRetVal;
                
                if (context != null && context.Instance != null) {
                    eRetVal = UITypeEditorEditStyle.DropDown;
                } else {
                    eRetVal = base.GetEditStyle(context);
                }
                return(eRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// EditValue:              Edit a value
            /// </summary>
            /// <param name="context">  Context</param>
            /// <param name="provider"> Provider</param>
            /// <param name="value">    Value to edit</param>
            /// <returns>
            /// New value
            /// </returns>
            //
            //********************************************************************************
            public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) {
                object                          objRetVal = null;
                RowsetTreeDefNodeFullName       nodeDefFullName;
                IWindowsFormsEditorService      edSvc;
                Application                     app;
                RowsetTreeDef                   rowsetTreeDef;
                PopupListBox                    listBox;
                ProxyRepPPHelper                proxyRepPP;
                ProxyRepPPTKHelper              proxyTK;
                string                          strRowsetName;

                if (context != null && context.Instance != null && provider != null) {
                    edSvc      = provider.GetService(typeof(IWindowsFormsEditorService)) as IWindowsFormsEditorService;
                    proxyRepPP = new ProxyRepPPHelper(DesignerUtil.FindRepPPAssembly(context.Instance));
                    app        = Application.GetApplicationForDesign(proxyRepPP, provider);
                    if (edSvc != null && app != null) {
                        proxyTK             = new ProxyRepPPTKHelper(context.Instance.GetType().Assembly);
                        nodeDefFullName     = new RowsetTreeDefNodeFullName(proxyTK, context.Instance);
                        objRetVal           = nodeDefFullName.RowsetName;
                        listBox             = new PopupListBox(edSvc);
                        listBox.Location    = new System.Drawing.Point(0, 0);
                        listBox.Name        = "listBox";
                        listBox.Size        = new System.Drawing.Size(168, 186);
                        listBox.TabIndex    = 1;
                        listBox.BorderStyle = System.Windows.Forms.BorderStyle.None;
                        listBox.Sorted      = true;
                        rowsetTreeDef         = (nodeDefFullName.RowsetTreeName == string.Empty) ? null : app.RowsetTreeDefs[nodeDefFullName.RowsetTreeName];
                        if (rowsetTreeDef != null) {
                            FillNodeNames(rowsetTreeDef.Nodes, listBox, proxyRepPP);
                        }
                        edSvc.DropDownControl(listBox);
                        if (listBox.SelectedItem != null) {
                            strRowsetName = listBox.SelectedItem as string;
                            objRetVal     = strRowsetName;
                        }   
                    }
                }
                if (objRetVal == null) {
                    objRetVal = base.EditValue(context, provider, value);
                }
                return (objRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// FillNodeNames:              Populates a list box with the names of all RowsetTreeDefNode in
            ///                             a rowset tree
            /// </summary>
            /// <param name="nodes">        Nodes of a rowset tree definition</param>
            /// <param name="listBox">      ListBox Control</param>
            /// <param name="proxyRepPP">   RepPP Proxy Helper</param>
            //
            //********************************************************************************
            internal void FillNodeNames(RowsetTreeDefNodes nodes, PopupListBox listBox, ProxyRepPPHelper proxyRepPP) {
                RowsetTreeDefNode rstDefNode;

                foreach (Object node in nodes) {
                    rstDefNode = new RowsetTreeDefNode(proxyRepPP, node);
                    if (String.Compare(rstDefNode.Name, 0, "$HIDE", 0, 5) != 0) {
                        listBox.Items.Add(rstDefNode.Name);
                        FillNodeNames(rstDefNode.Nodes, listBox, proxyRepPP);
                    }
                }
            }
        } // End Class RowsetNameEditor
    } // End Class RowsetTreeDefNodeControlDesigner

} // End namespace
