using System;
using System.Collections;
using System.ComponentModel;
using System.ComponentModel.Design;

namespace RepPP.Toolkit {
    //********************************************************************************
    //
	/// <summary>
	/// Implement a full field name (RowsetName.FieldName)
	/// </summary>
    //
    //********************************************************************************
    [TypeConverter(typeof(AtmFullName.AtmFullNameConverter))]
    public class AtmFullName {
        /// <summary>Type of the atom</summary>
        private AtomType    m_eAtomType;
        /// <summary>Name of the Atom</summary>
        private string      m_strAtmName;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        /// <param name="eAtomType">    Atom type</param>
        /// <param name="strAtomName">  Atom name</param>
        //
        //********************************************************************************
        public AtmFullName(AtomType eAtomType, string strAtomName) {
            m_eAtomType  = eAtomType;
            m_strAtmName = strAtomName.ToUpper();
        }

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //********************************************************************************
        public AtmFullName() : this(AtomType.sdAtomMsg, "") {
        }

        //********************************************************************************
        //
        /// <summary>
        /// AtomType:                   Name of the atom
        /// </summary>
        //
        //********************************************************************************
        [Editor("RepPP.Toolkit.Designer.AtmControlDesigner+AtmTypeEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]        
        [Description("Type of the atom")]
        public AtomType AtomType {
            get {
                return(m_eAtomType);
            }
            set {
                m_eAtomType = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AtomName:                  Name of the atom
        /// </summary>
        //
        //********************************************************************************
        [Editor("RepPP.Toolkit.Designer.AtmControlDesigner+AtmNameEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Description("Name of the atom")]
        public string AtomName {
            get {
                return(m_strAtmName);
            }
            set {
                m_strAtmName = (value == null) ? "" : value.ToUpper();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Implementation of a converter for the AtmFullName type
        /// </summary>
        //
        //********************************************************************************
        public class AtmFullNameConverter : TypeConverter {

            //********************************************************************************
            //
            /// <summary>
            /// CanConvertFrom:             Determine if we can convert from a specified type
            /// </summary>
            /// <param name="ctx">          Context</param>
            /// <param name="typeSource">   Source type</param>
            /// <returns>
            /// true if yes, false if not
            /// </returns>
            //
            //********************************************************************************
            public override bool CanConvertFrom(ITypeDescriptorContext ctx, Type typeSource) {
                bool    bRetVal;

                bRetVal = (typeSource == typeof(string)) ? true : base.CanConvertFrom(ctx, typeSource);
                return(bRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// CanConvertTo:                   Determine if we can convert to a specified type
            /// </summary>
            /// <param name="ctx">              Context</param>
            /// <param name="typeDestination">  Destination type</param>
            /// <returns>
            /// true if yes, false if not
            ///</returns>
            //
            //********************************************************************************
            public override bool CanConvertTo(ITypeDescriptorContext ctx, Type typeDestination) {
                bool    bRetVal;

                if (typeDestination == typeof(System.ComponentModel.Design.Serialization.InstanceDescriptor)) {
                    bRetVal = true;
                } else {
                    bRetVal = base.CanConvertTo(ctx, typeDestination);
                }
                return(bRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// ConvertFrom:                    Convert a type to an AtmFullName
            /// </summary>
            /// <param name="ctx">              Context</param>
            /// <param name="cultureInfo">      Culture info</param>
            /// <param name="val">              Object to convert</param>
            /// <returns>
            /// New AtmFullName object or null
            /// </returns>
            //
            //********************************************************************************
            public override object ConvertFrom(ITypeDescriptorContext ctx, System.Globalization.CultureInfo cultureInfo, object val) {
                object                      oRetVal;
                string                      strValue;
                AtomType                    eType;
                int                         iIndex;

                if (val == null) {
                    oRetVal = new AtmFullName();
                } else {
                    if (val.GetType() == typeof(String)) {
                        strValue = val as string;
                        if (strValue != null && strValue.Length != 0) {
                            iIndex = strValue.IndexOf(".");
                            if (iIndex < 1 || iIndex == strValue.Length - 1) {
                                throw new ArgumentException("Invalid object", "value");
                            }
                            eType   = (AtomType)Enum.Parse(typeof(AtomType), strValue.Substring(0, iIndex), true);
                            oRetVal = new AtmFullName(eType, strValue.Substring(iIndex + 1, strValue.Length - iIndex -1));
                        } else {
                            oRetVal = new AtmFullName();
                        }
                    } else {
                        oRetVal = base.ConvertFrom(ctx, cultureInfo, val);
                    }
                }
                return(oRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// ConvertTo:                      Convert a AtmFullName object to a specified type
            /// </summary>
            /// <param name="ctx">              Context</param>
            /// <param name="cultureInfo">      Culture info</param>
            /// <param name="val">              Value to convert</param>
            /// <param name="typeDestination">  Destination type</param>
            /// <returns>
            /// New object or null if error
            /// </returns>
            //
            //********************************************************************************
            public override object ConvertTo(ITypeDescriptorContext ctx, System.Globalization.CultureInfo cultureInfo, object val, Type typeDestination) {
                object                                  oRetVal;
                Type[]                                  arrParamType;
                ArrayList                               arrParam;
                AtmFullName                             atmFullName;
                System.Reflection.ConstructorInfo       cinfo;

                if (typeDestination == null) {
                    throw new ArgumentNullException("destinationType");
                }
                if (val == null) {
                    oRetVal = "";
                } else {
                    if (val.GetType() != typeof(AtmFullName)) {
                        throw new ArgumentException("Invalid object", "value");
                    }
                    if (typeDestination == typeof(string)) {
                        atmFullName = (AtmFullName)val;
                        oRetVal     = atmFullName.AtomType.ToString() + "." + atmFullName.AtomName;
                    } else if (typeDestination == typeof(System.ComponentModel.Design.Serialization.InstanceDescriptor)) {
                        atmFullName      = (AtmFullName)val;
                        arrParamType     = new Type [2];
                        arrParamType[0]  = typeof(AtomType);
                        arrParamType[1]  = typeof(string);
                        cinfo            = atmFullName.GetType().GetConstructor(arrParamType);
                        arrParam         = new ArrayList(2);
                        arrParam.Add(atmFullName.AtomType);
                        arrParam.Add(atmFullName.AtomName);
                        oRetVal = new System.ComponentModel.Design.Serialization.InstanceDescriptor(cinfo, arrParam);
                    } else {
                        oRetVal = base.ConvertTo(ctx, cultureInfo, val, typeDestination);
                    }
                }
                return(oRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// CreateInstance:             Creates an instance of a AtmFullName object
            /// </summary>
            /// <param name="ctx">          Context</param>
            /// <param name="propValues">   Dictionary of the argument values</param>
            /// <returns>
            /// New AtmFullName object or null if error
            /// </returns>
            //
            //********************************************************************************
            public override object CreateInstance(ITypeDescriptorContext ctx, System.Collections.IDictionary propValues) {
                object  objRetVal;

                objRetVal = new AtmFullName((AtomType)propValues["AtomType"],
                                            propValues["AtomName"] as String);
                return(objRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// GetCreateInstanceSupported: Returns true to indicate we support instance creation
            /// </summary>
            /// <param name="context">      Context</param>
            /// <returns>
            /// true
            /// </returns>
            //
            //********************************************************************************
            public override bool GetCreateInstanceSupported(ITypeDescriptorContext context) {
                return(true);
            }

            //********************************************************************************
            //
            /// <summary>
            /// GetProperties:              Gets the list of properties which are implemented by the object
            /// </summary>
            /// <param name="ctx">          Context</param>
            /// <param name="val">          Object</param>
            /// <param name="attributes">   Attributes</param>
            /// <returns>
            /// List of properties
            /// </returns>
            //
            //********************************************************************************
            public override PropertyDescriptorCollection GetProperties(ITypeDescriptorContext ctx, object val, System.Attribute[] attributes) {
                PropertyDescriptorCollection    colRetVal;
                string[]                        arrPropName;

                colRetVal      = TypeDescriptor.GetProperties(typeof(AtmFullName), attributes);
                arrPropName    = new string[2];
                arrPropName[0] = "AtomType";
                arrPropName[1] = "AtomName";
                colRetVal      = colRetVal.Sort(arrPropName);
                return(colRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// GetPropertiesSupported:     Return true to indicate we support properties
            /// </summary>
            /// <param name="context">      Context</param>
            /// <returns>
            /// true
            /// </returns>
            //
            //********************************************************************************
            public override bool GetPropertiesSupported(ITypeDescriptorContext context) {
                return(true);
            }
            
        } // AtmFullNameConverter class
	} // AtmFullName class
} // Namespace
