﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.ComponentModel.Design;

namespace RepPP.Toolkit {

    /// <summary>
    /// Control Utility helper
    /// </summary>
    public class ControlUtilHelperBase : ControlUtil.IControlUtilHelper {

        /// <summary>>Control Util parent</summary>
        protected ControlUtil           m_ctrlUtil;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constructor 
        /// </summary>
        //
        //********************************************************************************
        public ControlUtilHelperBase() {
            m_ctrlUtil = null;
        }

        //********************************************************************************
        //
        /// <summary>Set the Control Util parent used in this class</summary> 
        //
        //********************************************************************************
        public ControlUtil CtrlUtilParent {
            set {
                m_ctrlUtil = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetSite:            Get the component site of the control parent
        /// </summary>
        /// <returns>
        /// Component site of control parent if exist otherwise null
        /// </returns>
        //
        //********************************************************************************
        public virtual System.ComponentModel.ISite GetSite() {
            ISite siteRetVal = null;

            if (m_ctrlUtil != null && m_ctrlUtil.Parent != null && m_ctrlUtil.Parent is System.ComponentModel.IComponent) {
                siteRetVal = ((System.ComponentModel.IComponent)m_ctrlUtil.Parent).Site;
            }
            return (siteRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsDesignMode:   Indicate if the control is in design mode
        /// </summary>
        /// <returns>
        /// True if control is in design mode
        /// </returns>
        //
        //********************************************************************************
        public virtual bool IsDesignMode() {
            bool bRetVal = false;

            bRetVal = (System.Diagnostics.Process.GetCurrentProcess().ProcessName == "devenv");
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// UpdateControl:   Update the control
        /// </summary>
        //
        //********************************************************************************
        public virtual void UpdateControl() {
            if (m_ctrlUtil != null && m_ctrlUtil.Parent != null && m_ctrlUtil.Parent is ControlUtil.IRepControl) {
                ((ControlUtil.IRepControl)m_ctrlUtil.Parent).RepPPReInit();
            }
        }

    } // End ControlUtilHelperBase
} // End namespace
