using System;
using System.Collections;
using System.ComponentModel;
using RepPP;

namespace RepPP.Toolkit {
    //********************************************************************************
    //
	/// <summary>
	/// Implement a row wrapper around a group instance line
	/// </summary>
    //
    //********************************************************************************
	public class DataRowRowset : ICustomTypeDescriptor, IEditableObject, IDataErrorInfo {
	    private class EditInfo {
	        public string[]                 m_arrOriFldVal;     // Array of original field value (before edit)
	        public int[]                    m_arrOriFldState;   // Array of original field state (before edit)
	        public LineState                m_eOriLineState;    // Original state of the line (before edit)
	        public Hashtable                m_hashFldInError;   // Fields in error
	        public string                   m_strLineErr;       // Line error
	    };
	    
	    private DataSourceRowset            m_father;           // Father data source
	    private IntPtr                      m_iBmk;             // Bookmark of the attached line
	    private bool                        m_bBeginEdit;       // true if BeginEdit has been received
	    private EditInfo                    m_editInfo;         // Edit info

        //********************************************************************************
        //
	    /// <summary>
	    /// ctor:                   Class constructor
	    /// </summary>
	    /// <param name="father">   Father data source</param>
	    /// <param name="iBmk">     Line bookmark</param>
		//
        //********************************************************************************
		internal DataRowRowset(DataSourceRowset father, IntPtr iBmk) {
		    m_father        = father;
		    m_iBmk          = iBmk;
		    m_bBeginEdit    = false;
		    m_editInfo      = null;
		}

        //********************************************************************************
        //
		/// <summary>
		/// SelectThisLine:         Be sure this line is selected
		/// </summary>
        //
        //********************************************************************************
		private void SelectThisLine() {
            Rowset  rowset;
            int     iLineNb;
            
            rowset  = Rowset;
            iLineNb = rowset.LineFromBookmark(m_iBmk);
            if (!rowset.IsActive) {
                rowset.Select(false, false);
            }
            if (rowset.CurrentLine != iLineNb) {
                rowset.SelectLine(iLineNb, true, true);
            }
		}

        //********************************************************************************
        //
		/// <summary>
		/// Rowset:          Attached group instance
		/// </summary>
		//
        //********************************************************************************
		public Rowset Rowset {
		    get {
		        return(m_father.Rowset);
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// Bookmark:               Bookmark of the attached line
		/// </summary>
		//
        //********************************************************************************
		public IntPtr Bookmark {
		    get {
		        return(m_iBmk);
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// AllocEditInfo:          Allocates the information to be able to do an UNDO
		/// </summary>
        //
        //********************************************************************************
		private void AllocEditInfo() {
		    Rowset      rowset;
		    int         iFldCount;
		    int         iIndex;
		    FieldDef    fldDef;
		    FieldDef[]  arrFldDef;
		    
		    rowset                      = Rowset;
		    arrFldDef                   = m_father.FieldDefs;
		    iFldCount                   = arrFldDef.Length;
		    m_editInfo                  = new EditInfo();
		    m_editInfo.m_arrOriFldState = new int[iFldCount];
		    m_editInfo.m_arrOriFldVal   = new string[iFldCount];
  		    m_editInfo.m_eOriLineState  = rowset.LineState;
  		    m_editInfo.m_hashFldInError = new Hashtable(iFldCount);
  		    m_editInfo.m_strLineErr     = "";
    		for (iIndex = 0; iIndex < iFldCount; iIndex++) {
    		    fldDef = arrFldDef[iIndex];
    		    m_editInfo.m_arrOriFldState[iIndex] = ((fldDef.IsValid) ? 1 : 0) +
    		                                          ((fldDef.IsChanged) ? 2 : 0);
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// UndoChanged:            Undo the change on a line
		/// </summary>
		//
        //********************************************************************************
		private void UndoChanged() {
		    Rowset      rowset;
		    int         iFldCount;
		    int         iIndex;
		    FieldDef    fldDef;
		    string      strExtVal;
		    int         iState;
		    FieldDef[]  arrFldDef;
		    
		    rowset      = Rowset;
		    arrFldDef   = m_father.FieldDefs;
		    iFldCount   = arrFldDef.Length;
    		for (iIndex = 0; iIndex < iFldCount; iIndex++) {
    		    strExtVal = m_editInfo.m_arrOriFldVal[iIndex];
    		    if (strExtVal != null) {
    		        iState           = m_editInfo.m_arrOriFldState[iIndex];
        		    fldDef           = arrFldDef[iIndex];
        		    fldDef.ExtValue  = strExtVal;
    		        fldDef.IsValid   = ((iState & 1) != 0);
    		        fldDef.IsChanged = ((iState & 2) != 0);
    		    }
    		}
    		rowset.LineState = m_editInfo.m_eOriLineState;
		}

        //********************************************************************************
        //
		/// <summary>
		/// ValidateChar:           Validates the characters of a value
		/// </summary>
		/// <param name="fldDef">   Destination field</param>
		/// <param name="strVal">   Value to validate</param>
		/// <returns>
		/// true Succeed,    false At least one character illegal
		/// </returns>
		//
        //********************************************************************************
        private bool ValidateChar(FieldDef fldDef, string strVal) {
            bool    bRetVal = true;
            
            if (fldDef.OutputMask == null || fldDef.OutputMask == String.Empty) {
                foreach (char cChr in strVal) {
                    if (!fldDef.ValidateChar((int)cChr)) {
                        bRetVal = false;
                        break;
                    }
                }
            }
            return(bRetVal);
        }
		

        //********************************************************************************
        //
        /// <summary>
        ///  Default indexer
        /// </summary>
        //
        //********************************************************************************
        public object this[int iIndex] {
            get {
                string      strRetVal = null;
                string      strDesc;
                int         iChoiceIndex;
                Rowset      rowset;
                FieldDef[]  arrFldDef;
                FieldDef    fldDef;
                int         iLineNb;
                
                arrFldDef   = m_father.FieldDefs;
                fldDef      = arrFldDef[iIndex];
                rowset      = m_father.Rowset;
                iLineNb     = rowset.LineFromBookmark(m_iBmk);
                strRetVal   = rowset.GetFieldValue(fldDef, iLineNb, true);
                if (fldDef.IsChoiceAvailable) {
                     if (m_father.ShowDescription == DataSourceRowset.ShowDescriptionE.Always ||
                        (m_father.ShowDescription == DataSourceRowset.ShowDescriptionE.NotOnEdit && !m_bBeginEdit)) {
                        iChoiceIndex = fldDef.ChoiceList.FindCode(strRetVal);
                        if (iChoiceIndex != -1) {
                            strDesc = fldDef.ChoiceList.GetDesc(iChoiceIndex);
                            if (strDesc != null && strDesc.Length != 0) {
                                strRetVal = strDesc;
                            }
                        }
                    }
                }
                return(strRetVal);
            }
            set {
                Rowset      rowset;
                FieldDef[]  arrFldDef;
                FieldDef    fldDef;
                string      strErr;
                string      strValue;
                int         iChoiceIndex;
                
                SelectThisLine();
                strValue    = value != null ? value.ToString() : String.Empty;
                arrFldDef   = m_father.FieldDefs;
                fldDef      = arrFldDef[iIndex];
                rowset      = m_father.Rowset;
                if (m_bBeginEdit) {
                    if (m_editInfo == null) {
                        AllocEditInfo();
                    }
                    if (m_editInfo.m_hashFldInError.Contains(fldDef.Name)) {
                        m_editInfo.m_hashFldInError.Remove(fldDef.Name);
                    }
                }
                if (fldDef.IsPrimaryKey && rowset.LineState != LineState.sdLineNew) {
                    strErr = fldDef.Application.GetAtomText("$PKC", AtomType.sdAtomErr);
                    if (m_editInfo != null) {
                        m_editInfo.m_strLineErr = strErr;
                    }
                    throw new ArgumentException(strErr);
                }
                if (!ValidateChar(fldDef, strValue)) {
                    if (m_editInfo != null) {
                        m_editInfo.m_hashFldInError.Add(fldDef.Name, fldDef.Application.GetAtomText("$ANSIC", AtomType.sdAtomErr));
                    }
                }
                if (m_editInfo != null) {
                    if (m_editInfo.m_arrOriFldVal[iIndex] == null) {
                        m_editInfo.m_arrOriFldVal[iIndex] = fldDef.ExtValue;
                    }
                }
                if (fldDef.IsChoiceAvailable) {
                     if (m_father.ShowDescription == DataSourceRowset.ShowDescriptionE.Always ||
                        (m_father.ShowDescription == DataSourceRowset.ShowDescriptionE.NotOnEdit && !m_bBeginEdit)) {
                        iChoiceIndex = fldDef.ChoiceList.FindDesc(strValue);
                        if (iChoiceIndex != -1) {
                            strValue = fldDef.ChoiceList.GetCode(iChoiceIndex);
                        }
                    }
                }

                fldDef.ExtValue = strValue;
                m_father.OnListChanged(new ListChangedEventArgs(ListChangedType.ItemChanged, m_father.IndexOf(this)));
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// GetConverter:               Not used in this context
		/// </summary>
		/// <returns>
		/// null
		/// </returns>
        //
        //********************************************************************************
        public TypeConverter GetConverter() { // ICustomTypeDescriptor
            return null;
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetEvents:                  Not used in this context
        /// </summary>
        /// <param name="attributes">   Attribute list</param>
        /// <returns>
        /// null
        /// </returns>
        //
        //********************************************************************************
        public EventDescriptorCollection GetEvents(System.Attribute[] attributes) { // ICustomTypeDescriptor
            return(new System.ComponentModel.EventDescriptorCollection(null));
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetEvents:                  Not used in this context
        /// </summary>
        /// <returns>
        /// null
        /// </returns>
        //
        //********************************************************************************
        EventDescriptorCollection System.ComponentModel.ICustomTypeDescriptor.GetEvents() { // ICustomTypeDescriptor
            return(new System.ComponentModel.EventDescriptorCollection(null));
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetComponentName:           Not used in this context
        /// </summary>
        /// <returns>
        /// null
        /// </returns>
        //
        //********************************************************************************
        public string GetComponentName() { // ICustomTypeDescriptor
            return null;
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetPropertyOwner:           Gets the owner of the property
        /// </summary>
        /// <param name="pd">           Property descriptor</param>
        /// <returns>
        /// this
        /// </returns>
        //
        //********************************************************************************
        public object GetPropertyOwner(PropertyDescriptor pd) { // ICustomTypeDescriptor
            return(this);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetAttributes:              Not used in this context
        /// </summary>
        /// <returns>
        /// Empty collections
        /// </returns>
        //
        //********************************************************************************
        public AttributeCollection GetAttributes() { // ICustomTypeDescriptor
            return(new System.ComponentModel.AttributeCollection(null));
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetProperties:              Not used in this context
        /// </summary>
        /// <param name="attributes">   Attributes</param>
        /// <returns>
        /// Empty collection
        /// </returns>
        //
        //********************************************************************************
        public PropertyDescriptorCollection GetProperties(System.Attribute[] attributes) { // ICustomTypeDescriptor
            return(m_father.GetItemProperties(null));
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetProperties:              Not used in this context
        /// </summary>
        /// <returns>
        /// Empty collection
        /// </returns>
        //
        //********************************************************************************
        PropertyDescriptorCollection System.ComponentModel.ICustomTypeDescriptor.GetProperties() { // ICustomTypeDescriptor
            return(m_father.GetItemProperties(null));
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetEditor:                      Not used in this context
        /// </summary>
        /// <param name="editorBaseType">   Type of the base editor</param>
        /// <returns>
        /// null
        /// </returns>
        //
        //********************************************************************************
        public object GetEditor(Type editorBaseType) { // ICustomTypeDescriptor
            return null;
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetDefaultProperty:             Not used in this context
        /// </summary>
        /// <returns>
        /// null
        /// </returns>
        //
        //********************************************************************************
        public PropertyDescriptor GetDefaultProperty() { // ICustomTypeDescriptor
            return null;
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetDefaultEvent:                Not used in this context
        /// </summary>
        /// <returns>
        /// null
        /// </returns>
        //
        //********************************************************************************
        public EventDescriptor GetDefaultEvent() { // ICustomTypeDescriptor
            return null;
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetClassName:                   Use to indentify the row
        /// </summary>
        /// <returns>
        /// Father list name
        /// </returns>
        //
        //********************************************************************************
        public string GetClassName() { // ICustomTypeDescriptor
            return(m_father.GetListName(null));
        }

        //********************************************************************************
        //
        /// <summary>
        /// BeginEdit:                      Begin the edition of the line
        /// </summary>
        //
        //********************************************************************************
        public void BeginEdit() { // IEditableObject
            if (!m_bBeginEdit) {
                SelectThisLine();
                m_bBeginEdit = true;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// EndEdit:                        End and accept the edition of the line
        /// </summary>
        //
        //********************************************************************************
        public void EndEdit() { // IEditableObject
            Hashtable   hash;
            IEnumerator enu;
            
            if (m_bBeginEdit) {
                SelectThisLine();
                if (m_editInfo != null) {
                    hash = m_editInfo.m_hashFldInError;
                    if (!m_father.IsLineValid(hash)) {
                        enu = hash.Keys.GetEnumerator();
                        if (enu.MoveNext()) {
                            throw new System.ApplicationException(BuildError(enu.Current.ToString(), hash, true));
                        } else {
                            // Must never happens...
                            throw new System.ApplicationException("Field in error!!!");
                        }                        
                    }
                }
                m_bBeginEdit = false;
                m_editInfo   = null;
                if (m_father.m_rowsetRowNewLine == this) {
                    m_father.m_rowsetRowNewLine = null;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// CancelEdit:                     Cancel the edition of the line
        /// </summary>
        //
        //********************************************************************************
        public void CancelEdit() { // IEditableObject
            if (m_bBeginEdit) {
                SelectThisLine();
                m_bBeginEdit = false;
                if (m_father.m_rowsetRowNewLine == this) {
                    m_father.m_rowsetRowNewLine = null;
                    m_father.Remove(this);
                } else if (m_editInfo != null) {
                    UndoChanged();
                }
                m_editInfo = null;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// BuildError:                     Build an error message
        /// </summary>
        /// <param name="strFldName">       Name of the field</param>
        /// <param name="hash">             Hashtable containing the errors</param>
        /// <param name="bIncludeFieldName">true to include field prompt </param>
        /// <returns>
        /// Error message
        /// </returns>
        //
        //********************************************************************************
        private string BuildError(string strFldName, Hashtable hash, bool bIncludeFieldName) {
            string  strRetVal = "";
            FieldDef   fldDef;
            
            fldDef = Rowset.RowsetDef.FieldDefs[strFldName];
            if (fldDef != null) {
                if (bIncludeFieldName) {
                    strRetVal = fldDef.Prompt + " - " + hash[strFldName];
                } else {
                    strRetVal = hash[strFldName].ToString();
                }
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// Indexer:                    Returns the error for a column if any
        /// </summary>
        //
        //********************************************************************************
        string System.ComponentModel.IDataErrorInfo.this[string strColName] {
            get {
                string      strRetVal = "";
                Hashtable   hash;
                
                if (m_editInfo != null) {
                    hash = m_editInfo.m_hashFldInError;
                    if (hash.Contains(strColName)) {
                        strRetVal = BuildError(strColName, hash, false);
                    }
                }
                return(strRetVal);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Error:                      Returns a line error
        /// </summary>
        //
        //********************************************************************************
        public string Error {
            get {
                string      strRetVal = "";
                string      strErr;
                Hashtable   hash;
                
                if (m_editInfo != null) {
                    strRetVal               = m_editInfo.m_strLineErr;
                    hash                    = m_editInfo.m_hashFldInError;
                    foreach (string strFldInErr in hash.Keys) {
                        strErr      = BuildError(strFldInErr, hash, true);
                        if (strErr.Length != 0) {
                            if (strRetVal.Length != 0) {
                                strRetVal = strRetVal + "\r\n";
                            }
                            strRetVal += strErr;
                        }
                    }
                }
                return(strRetVal);
            }
        }
    } // Class
} // Namespace
