using System;
using System.ComponentModel;

namespace RepPP.Toolkit  {
    //********************************************************************************
    //
	/// <summary>
	/// Implement a property descriptor for the rowset row. Each property is a field
	/// in this context.
	/// </summary>
	//
    //********************************************************************************
	public class DataRowRowsetPropertyDescriptor : PropertyDescriptor {
		private int     m_iIndex;           // Index of the column
		private Type    m_type;             // Type of the column
		private bool    m_bIsReadOnly;      // true if column is read-only
		private string  m_strDisplayName;   // Display name of the property

        //********************************************************************************
        //
		/// <summary>
		/// ctor:                           Class constructor
		/// </summary>
		/// <param name="strName">          Name of the property</param>
		/// <param name="strDisplayName">   Display name of the property</param>
		/// <param name="iIndex">           Index of the property</param>
		/// <param name="type">             Type of the property</param>
		/// <param name="bIsReadOnly">      Is property R/O</param>
        //
        //********************************************************************************
		public DataRowRowsetPropertyDescriptor(string    strName,
		                                      string    strDisplayName,
		                                      int       iIndex,
		                                      Type      type,
		                                      bool      bIsReadOnly) : base(strName, null) {
            m_iIndex         = iIndex;
            m_type           = type;
            m_bIsReadOnly    = bIsReadOnly;
            m_strDisplayName = strDisplayName;
		}

        //********************************************************************************
        //
        /// <summary>
        /// DisplayName:            Display name of the property
        /// </summary>
        //
        //********************************************************************************
        public override string DisplayName {
            get {
                return(m_strDisplayName);
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// GetValue:                   Gets a column
		/// </summary>
		/// <param name="component">    Component from which we get the value</param>
		/// <returns>
		/// Column
		/// </returns>
        //
        //********************************************************************************
        public override object GetValue(object component) {
            return((component as DataRowRowset)[m_iIndex]);
        }

        //********************************************************************************
        //
		/// <summary>
		/// ComponentType:              Gets the type of a row
		/// </summary>
        //
        //********************************************************************************
        public override Type ComponentType {
            get {
                return(typeof(DataRowRowset));
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// PropertyType:               Returns the type of the property
        /// </summary>
        //
        //********************************************************************************
        public override Type PropertyType {
            get {
                return(m_type);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ResetValue:                 Reset the value. Not supported
        /// </summary>
        /// <param name="component"></param>
        //
        //********************************************************************************
        public override void ResetValue(object component) {
            // Do nothing
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetValue:                   Set the value of a column
        /// </summary>
        /// <param name="component">    Component (DataRowGInst)</param>
        /// <param name="value">        New value</param>
        //
        //********************************************************************************
        public override void SetValue(object component, object value) {
            (component as DataRowRowset)[m_iIndex] = value;
        }

        //********************************************************************************
        //
        /// <summary>
        /// CanResetValue:              Return false to indicate we don't support ResetValue
        /// </summary>
        /// <param name="component"></param>
        /// <returns>
        /// false
        /// </returns>
        //
        //********************************************************************************
        public override bool CanResetValue(object component) {
            return(false);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ShouldSerializeValue:       False to indicate not to serialize value
        /// </summary>
        /// <param name="component">    Component (DataRowGInst)</param>
        /// <returns>
        /// false
        /// </returns>
        //
        //********************************************************************************
        public override bool ShouldSerializeValue(object component) {
            return(false);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsReadOnly:                 Returns if the column is read only
        /// </summary>
        //
        //********************************************************************************
        public override bool IsReadOnly {
            get {
                return(m_bIsReadOnly);
            }
        }
	}
}
