using System;
using System.Collections;
using System.ComponentModel;
using System.ComponentModel.Design;

namespace RepPP.Toolkit {
    //********************************************************************************
    //
	/// <summary>
	/// Implement a full field name (RowsetName.FieldName)
	/// </summary>
    //
    //********************************************************************************
    [TypeConverter(typeof(FldFullName.FldFullNameConverter))]
    public class FldFullName {
        /// <summary>Event fired every time the fullname of the field changes</summary>
        public event EventHandler   NameChanged;
        /// <summary>Name of the rowset</summary>
        private string              m_strRowsetName;
        /// <summary>Name of the field</summary>
        private string              m_strFldName;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        /// <param name="strRowsetName">Rowset name</param>
        /// <param name="strFldName">   Field name</param>
        //
        //********************************************************************************
        public FldFullName(string strRowsetName, string strFldName) {
            m_strRowsetName = (strRowsetName == null) ? "" : strRowsetName;
            m_strFldName    = (strFldName == null)    ? "" : strFldName;
        }

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //********************************************************************************
        public FldFullName() : this("","") {
        }

        //********************************************************************************
        //
        /// <summary>
        /// RowsetName:                  Name of the group
        /// </summary>
        //
        //********************************************************************************
        [Editor("RepPP.Toolkit.Designer.FldControlDesigner+RowsetNameEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Description("Name of the rowset")]
        public string RowsetName {
            get {
                return(m_strRowsetName);
            }
            set {
                if (m_strRowsetName != value) {
                    m_strRowsetName = (value == null) ? "" : value;
                    OnNameChanged(EventArgs.Empty);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FieldName:                  Name of the field
        /// </summary>
        //
        //********************************************************************************
        [Editor("RepPP.Toolkit.Designer.FldControlDesigner+FieldNameEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Description("Name of the field")]
        public string FieldName {
            get {
                return(m_strFldName);
            }
            set {
                if (m_strFldName != value) {
                    m_strFldName = (value == null) ? "" : value;
                    OnNameChanged(EventArgs.Empty);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnNameChanged:              Fires the NameChanged event
        /// </summary>
        /// <param name="e">            Event arguments</param>
        //
        //********************************************************************************
        protected virtual void OnNameChanged(EventArgs e){
            if (NameChanged != null) {
                NameChanged(this, e);
            }
        }
        //********************************************************************************
        //
        /// <summary>
        /// Implementation of a converter for the FldName type
        /// </summary>
        //
        //********************************************************************************
        public class FldFullNameConverter : TypeConverter {

            //********************************************************************************
            //
            /// <summary>
            /// CanConvertFrom:             Determine if we can convert from a specified type
            /// </summary>
            /// <param name="ctx">          Context</param>
            /// <param name="typeSource">   Source type</param>
            /// <returns>
            /// true if yes, false if not
            /// </returns>
            //
            //********************************************************************************
            public override bool CanConvertFrom(ITypeDescriptorContext ctx, Type typeSource) {
                bool    bRetVal;

                bRetVal = (typeSource == typeof(string)) ? true : base.CanConvertFrom(ctx, typeSource);
                return(bRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// CanConvertTo:                   Determine if we can convert to a specified type
            /// </summary>
            /// <param name="ctx">              Context</param>
            /// <param name="typeDestination">  Destination type</param>
            /// <returns>
            /// true if yes, false if not
            ///</returns>
            //
            //********************************************************************************
            public override bool CanConvertTo(ITypeDescriptorContext ctx, Type typeDestination) {
                bool    bRetVal;

                if (typeDestination == typeof(System.ComponentModel.Design.Serialization.InstanceDescriptor)) {
                    bRetVal = true;
                } else {
                    bRetVal = base.CanConvertTo(ctx, typeDestination);
                }
                return(bRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// ConvertFrom:                    Convert a type to a FldName
            /// </summary>
            /// <param name="ctx">              Context</param>
            /// <param name="cultureInfo">      Culture info</param>
            /// <param name="val">              Object to convert</param>
            /// <returns>
            /// New FldName object or null
            /// </returns>
            //
            //********************************************************************************
            public override object ConvertFrom(ITypeDescriptorContext ctx, System.Globalization.CultureInfo cultureInfo, object val) {
                object                      oRetVal;
                string                      strValue;
                string[]                    arr;

                if (val == null) {
                    oRetVal = new FldFullName();
                } else {
                    if (val.GetType() == typeof(String)) {
                        strValue = val as string;
                        if (strValue != null && strValue.Length != 0) {
                            arr         = strValue.Split('.');
                            if (arr.Length != 2) {
                                throw new ArgumentException("Invalid object", "value");
                            }
                            oRetVal = new FldFullName(arr[0], arr[1]);
                        } else {
                            oRetVal = new FldFullName();
                        }
                    } else {
                        oRetVal = base.ConvertFrom(ctx, cultureInfo, val);
                    }
                }
                return(oRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// ConvertTo:                      Convert a FldFullName object to a specified type
            /// </summary>
            /// <param name="ctx">              Context</param>
            /// <param name="cultureInfo">      Culture info</param>
            /// <param name="val">              Value to convert</param>
            /// <param name="typeDestination">  Destination type</param>
            /// <returns>
            /// New object or null if error
            /// </returns>
            //
            //********************************************************************************
            public override object ConvertTo(ITypeDescriptorContext ctx, System.Globalization.CultureInfo cultureInfo, object val, Type typeDestination) {
                object                                  oRetVal;
                Type[]                                  arrParamType;
                ArrayList                               arrParam;
                FldFullName                             fldFullName;
                System.Reflection.ConstructorInfo       cinfo;

                if (typeDestination == null) {
                    throw new ArgumentNullException("destinationType");
                }
                if (val == null) {
                    oRetVal = "";
                } else {
                    if (val.GetType() != typeof(FldFullName)) {
                        throw new ArgumentException("Invalid object", "value");
                    }
                    if (typeDestination == typeof(string)) {
                        fldFullName = (FldFullName)val;
                        oRetVal     = fldFullName.RowsetName + "." + fldFullName.FieldName;
                    } else if (typeDestination == typeof(System.ComponentModel.Design.Serialization.InstanceDescriptor)) {
                        fldFullName      = (FldFullName)val;
                        arrParamType     = new Type [2];
                        arrParamType[0]  = typeof(string);
                        arrParamType[1]  = typeof(string);
                        cinfo            = fldFullName.GetType().GetConstructor(arrParamType);
                        arrParam         = new ArrayList(2);
                        arrParam.Add(fldFullName.RowsetName);
                        arrParam.Add(fldFullName.FieldName);
                        oRetVal = new System.ComponentModel.Design.Serialization.InstanceDescriptor(cinfo, arrParam);
                    } else {
                        oRetVal = base.ConvertTo(ctx, cultureInfo, val, typeDestination);
                    }
                }
                return(oRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// CreateInstance:             Creates an instance of a FldFullName object
            /// </summary>
            /// <param name="ctx">          Context</param>
            /// <param name="propValues">   Dictionary of the argument values</param>
            /// <returns>
            /// New FldFullName object or null if error
            /// </returns>
            //
            //********************************************************************************
            public override object CreateInstance(ITypeDescriptorContext ctx, System.Collections.IDictionary propValues) {
                object  objRetVal;

                objRetVal = new FldFullName(propValues["RowsetName"] as string,
                                            propValues["FieldName"] as String);
                return(objRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// GetCreateInstanceSupported: Returns true to indicate we support instance creation
            /// </summary>
            /// <param name="context">      Context</param>
            /// <returns>
            /// true
            /// </returns>
            //
            //********************************************************************************
            public override bool GetCreateInstanceSupported(ITypeDescriptorContext context) {
                return(true);
            }

            //********************************************************************************
            //
            /// <summary>
            /// GetProperties:              Gets the list of properties which are implemented by the object
            /// </summary>
            /// <param name="ctx">          Context</param>
            /// <param name="val">          Object</param>
            /// <param name="attributes">   Attributes</param>
            /// <returns>
            /// List of properties
            /// </returns>
            //
            //********************************************************************************
            public override PropertyDescriptorCollection GetProperties(ITypeDescriptorContext ctx, object val, System.Attribute[] attributes) {
                PropertyDescriptorCollection    colRetVal;
                string[]                        arrPropName;

                colRetVal      = TypeDescriptor.GetProperties(typeof(FldFullName), attributes);
                arrPropName    = new string[2];
                arrPropName[0] = "RowsetName";
                arrPropName[1] = "FieldName";
                colRetVal      = colRetVal.Sort(arrPropName);
                return(colRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// GetPropertiesSupported:     Return true to indicate we support properties
            /// </summary>
            /// <param name="context">      Context</param>
            /// <returns>
            /// true
            /// </returns>
            //
            //********************************************************************************
            public override bool GetPropertiesSupported(ITypeDescriptorContext context) {
                return(true);
            }
            
        } // FldFullNameConverter class
	} // FldName class
} // Namespace
