using System;
using System.Collections;
using System.ComponentModel;
using System.ComponentModel.Design;

namespace RepPP.Toolkit {
    //********************************************************************************
    //
	/// <summary>
	/// Implement a full RowsetTreeDefNode name (RowsetTreeName.RowsetName)
	/// </summary>
    //
    //********************************************************************************
    [TypeConverter(typeof(RowsetTreeDefNodeFullName.RowsetTreeDefNodeFullNameConverter))]
    public class RowsetTreeDefNodeFullName {
        /// <summary>Name of the rowset tree</summary>
        private string      m_strRowsetTreeName;
        /// <summary>Name of the rowset</summary>
        private string      m_strRowsetName;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="strRowsetTreeName">    RowsetTreeDef name</param>
        /// <param name="strRowsetName">        Rowset name</param>
        //
        //********************************************************************************
        public RowsetTreeDefNodeFullName(string strRowsetTreeName, string strRowsetName) {
            m_strRowsetTreeName    = (strRowsetTreeName == null) ? "" : strRowsetTreeName.ToUpper();
            m_strRowsetName        = (strRowsetName == null)     ? "" : strRowsetName.ToUpper();
        }

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //********************************************************************************
        public RowsetTreeDefNodeFullName() : this("","") {
        }

        //********************************************************************************
        //
        /// <summary>
        /// RowsetTreeName:                     Name of the rowset tree
        /// </summary>
        //
        //********************************************************************************
        [Editor("RepPP.Toolkit.Designer.RowsetTreeDefNodeControlDesigner+RowsetTreeNameEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Description("Name of the rowset tree")]
        public string RowsetTreeName {
            get {
                return(m_strRowsetTreeName);
            }
            set {
                m_strRowsetTreeName = (value == null) ? "" : value.ToUpper();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// RowsetName:                         Name of the rowset
        /// </summary>
        //
        //********************************************************************************
        [Editor("RepPP.Toolkit.Designer.RowsetTreeDefNodeControlDesigner+RowsetNameEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Description("Name of the rowset")]
        public string RowsetName {
            get {
                return(m_strRowsetName);
            }
            set {
                m_strRowsetName = (value == null) ? "" : value.ToUpper();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Implementation of a converter for the CrdGrpFullName type
        /// </summary>
        //
        //********************************************************************************
        public class RowsetTreeDefNodeFullNameConverter : TypeConverter {

            //********************************************************************************
            //
            /// <summary>
            /// CanConvertFrom:             Determine if we can convert from a specified type
            /// </summary>
            /// <param name="ctx">          Context</param>
            /// <param name="typeSource">   Source type</param>
            /// <returns>
            /// true if yes, false if not
            /// </returns>
            //
            //********************************************************************************
            public override bool CanConvertFrom(ITypeDescriptorContext ctx, Type typeSource) {
                bool    bRetVal;

                bRetVal = (typeSource == typeof(string)) ? true : base.CanConvertFrom(ctx, typeSource);
                return(bRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// CanConvertTo:                   Determine if we can convert to a specified type
            /// </summary>
            /// <param name="ctx">              Context</param>
            /// <param name="typeDestination">  Destination type</param>
            /// <returns>
            /// true if yes, false if not
            ///</returns>
            //
            //********************************************************************************
            public override bool CanConvertTo(ITypeDescriptorContext ctx, Type typeDestination) {
                bool    bRetVal;

                if (typeDestination == typeof(System.ComponentModel.Design.Serialization.InstanceDescriptor)) {
                    bRetVal = true;
                } else {
                    bRetVal = base.CanConvertTo(ctx, typeDestination);
                }
                return(bRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// ConvertFrom:                    Convert a type to a CrdGrpFullName
            /// </summary>
            /// <param name="ctx">              Context</param>
            /// <param name="cultureInfo">      Culture info</param>
            /// <param name="val">              Object to convert</param>
            /// <returns>
            /// New CrdGrpFullName object or null
            /// </returns>
            //
            //********************************************************************************
            public override object ConvertFrom(ITypeDescriptorContext ctx, System.Globalization.CultureInfo cultureInfo, object val) {
                object                      oRetVal;
                string                      strValue;
                string[]                    arr;

                if (val == null) {
                    oRetVal = new RowsetTreeDefNodeFullName();
                } else {
                    if (val.GetType() == typeof(String)) {
                        strValue = val as string;
                        if (strValue != null && strValue.Length != 0) {
                            arr         = strValue.Split('.');
                            if (arr.Length != 2) {
                                throw new ArgumentException("Invalid object", "value");
                            }
                            oRetVal = new RowsetTreeDefNodeFullName(arr[0], arr[1]);
                        } else {
                            oRetVal = new RowsetTreeDefNodeFullName();
                        }
                    } else {
                        oRetVal = base.ConvertFrom(ctx, cultureInfo, val);
                    }
                }
                return(oRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// ConvertTo:                      Convert a RowsetTreeDefNodeFullName object to a specified type
            /// </summary>
            /// <param name="ctx">              Context</param>
            /// <param name="cultureInfo">      Culture info</param>
            /// <param name="val">              Value to convert</param>
            /// <param name="typeDestination">  Destination type</param>
            /// <returns>
            /// New object or null if error
            /// </returns>
            //
            //********************************************************************************
            public override object ConvertTo(ITypeDescriptorContext ctx, System.Globalization.CultureInfo cultureInfo, object val, Type typeDestination) {
                object                                  oRetVal;
                Type[]                                  arrParamType;
                ArrayList                               arrParam;
                RowsetTreeDefNodeFullName               nodeDefFullName;
                System.Reflection.ConstructorInfo       cinfo;

                if (typeDestination == null) {
                    throw new ArgumentNullException("destinationType");
                }
                if (val == null) {
                    oRetVal = "";
                } else {
                    if (val.GetType() != typeof(RowsetTreeDefNodeFullName)) {
                        throw new ArgumentException("Invalid object", "value");
                    }
                    if (typeDestination == typeof(string)) {
                        nodeDefFullName  = (RowsetTreeDefNodeFullName)val;
                        oRetVal          = nodeDefFullName.RowsetTreeName + "." + nodeDefFullName.RowsetName;
                    } else if (typeDestination == typeof(System.ComponentModel.Design.Serialization.InstanceDescriptor)) {
                        nodeDefFullName  = (RowsetTreeDefNodeFullName)val;
                        arrParamType     = new Type [2];
                        arrParamType[0]  = typeof(string);
                        arrParamType[1]  = typeof(string);
                        cinfo            = nodeDefFullName.GetType().GetConstructor(arrParamType);
                        arrParam         = new ArrayList(2);
                        arrParam.Add(nodeDefFullName.RowsetTreeName);
                        arrParam.Add(nodeDefFullName.RowsetName);
                        oRetVal = new System.ComponentModel.Design.Serialization.InstanceDescriptor(cinfo, arrParam);
                    } else {
                        oRetVal = base.ConvertTo(ctx, cultureInfo, val, typeDestination);
                    }
                }
                return(oRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// CreateInstance:             Creates an instance of a CrdGrpFullName object
            /// </summary>
            /// <param name="ctx">          Context</param>
            /// <param name="propValues">   Dictionary of the argument values</param>
            /// <returns>
            /// New CrdGrpFullName object or null if error
            /// </returns>
            //
            //********************************************************************************
            public override object CreateInstance(ITypeDescriptorContext ctx, System.Collections.IDictionary propValues) {
                object  objRetVal;

                objRetVal = new RowsetTreeDefNodeFullName(propValues["RowsetTreeName"] as string,
                                                       propValues["RowsetName"]     as string);
                return(objRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// GetCreateInstanceSupported: Returns true to indicate we support instance creation
            /// </summary>
            /// <param name="context">      Context</param>
            /// <returns>
            /// true
            /// </returns>
            //
            //********************************************************************************
            public override bool GetCreateInstanceSupported(ITypeDescriptorContext context) {
                return(true);
            }

            //********************************************************************************
            //
            /// <summary>
            /// GetProperties:              Gets the list of properties which are implemented by the object
            /// </summary>
            /// <param name="ctx">          Context</param>
            /// <param name="val">          Object</param>
            /// <param name="attributes">   Attributes</param>
            /// <returns>
            /// List of properties
            /// </returns>
            //
            //********************************************************************************
            public override PropertyDescriptorCollection GetProperties(ITypeDescriptorContext ctx, object val, System.Attribute[] attributes) {
                PropertyDescriptorCollection    colRetVal;
                string[]                        arrPropName;

                colRetVal      = TypeDescriptor.GetProperties(typeof(RowsetTreeDefNodeFullName), attributes);
                arrPropName    = new string[2];
                arrPropName[0] = "RowsetTreeName";
                arrPropName[1] = "RowsetName";
                colRetVal      = colRetVal.Sort(arrPropName);
                return(colRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// GetPropertiesSupported:     Return true to indicate we support properties
            /// </summary>
            /// <param name="context">      Context</param>
            /// <returns>
            /// true
            /// </returns>
            //
            //********************************************************************************
            public override bool GetPropertiesSupported(ITypeDescriptorContext context) {
                return(true);
            }
            
        } // RowsetTreeDefNodeFullNameConverter class
	} // RowsetTreeDefNodeFullName class
} // Namespace
