using System;
using System.Text;

namespace RepPP.Toolkit.CodeGen {
	/// <summary>
	/// TypedInstanceGenForCS:  Typed instance generator for C#.
	/// </summary>
	public class TypedInstanceGenForCS : TypedInstanceGenBase {

        //******************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //******************************************************************
		public TypedInstanceGenForCS() : base(new WebServicesGenForCS()) {
		}

        //******************************************************************
        //
        /// <summary>
        /// IntGenerateTypedRowset:                 Generate typed instance for a rowset
        /// </summary>
        /// <param name="node">                     Node of the rowset tree definition</param>
        /// <param name="rowsetDef">                Rowset definition</param>
        /// <param name="strRowsetTreeClassPrefix"> Prefix for generated rowset tree class name</param>
        /// <param name="strRowsetClassPrefix">     Prefix for generated rowset class name</param>
        /// <param name="strRowsetPropertyPrefix">  Prefix for generated rowset property name</param>
        /// <param name="strFldPropertyPrefix">     Prefix for generated field property name</param>
        /// <param name="strNameSpace">             The namespace</param>
        /// <param name="iLevel">                   Right pad level</param>
        /// <param name="strBaseClassName">         Name of the base class</param>
        /// <param name="bGenWS">                   true to generate Web Services</param>
        /// <param name="bUseWSDataContracts">      true to generate Web Service Adapters using DataContract</param>
        /// <returns>
        /// The generated class
        /// </returns>
        //
        //******************************************************************        
		private string IntGenerateTypedRowset(RepPP.RowsetTreeDefNode   node,
		                                      RepPP.RowsetDef           rowsetDef,
                                              string                    strRowsetTreeClassPrefix,
                                              string                    strRowsetClassPrefix,
                                              string                    strRowsetPropertyPrefix,
                                              string                    strFldPropertyPrefix,
                                              string                    strNameSpace,
                                              string                    strBaseClassName,
		                                      int                       iLevel,
		                                      bool                      bGenWS,
                                              bool                      bUseWSDataContracts) {
            StringBuilder           strBuilder;
            WebServicesGenBase      wsGen;
            string                  strFldPropName;
            string                  strRowsetName;
            string                  strRowsetPropName;
            string                  strClassName;
            string                  strChildClassName;
            string                  strContainerClassName;
            string                  strRowsetDefBaseClassName;
            string                  strIndent;
            string                  strFldType;
            int                     iCount;
            int                     iIndex;
            string                  strUserClassName;
            
            wsGen                       = WSGenerator;
            if (bGenWS) {
                wsGen.RowsetTreeClassPrefix = WEB_SERV_CLASS_PREFIX + strRowsetTreeClassPrefix;
                wsGen.RowsetClassPrefix     = WEB_SERV_CLASS_PREFIX + strRowsetClassPrefix;
                wsGen.RowsetPropertyPrefix  = strRowsetPropertyPrefix;
                wsGen.FieldPropertyPrefix   = strFldPropertyPrefix;
            }
            strIndent                   = "".PadRight(iLevel * 4, ' ');
            strBuilder                  = new StringBuilder(null, 4096);            
            iCount                      = rowsetDef.FieldDefs.Count;
            strClassName                = DecorateRowsetClassName(TYPED_ROWSET_PREFIX + strRowsetClassPrefix, (node != null) ? GetNodeRowsetName(node) : GetRowsetName(rowsetDef));
            strRowsetDefBaseClassName   = (strBaseClassName != String.Empty) ? strBaseClassName : "RepPP.TypedRowsetBase";
            strUserClassName            = DecorateRowsetClassName(strRowsetClassPrefix, (node != null) ? GetNodeRowsetName(node) : GetRowsetName(rowsetDef));
            strBuilder.Append(strIndent + "/// <summary>\r\n");
            strBuilder.Append(strIndent + String.Format("/// {0} {1}\r\n", m_cmtResManager.GetClassDesc("TypedRowsetBase"), rowsetDef.Name));
            strBuilder.Append(strIndent + "/// </summary>\r\n");
            if (node == null) {
                strBuilder.Append(strIndent + "[RepPP.TypedInstanceClass(\"" + rowsetDef.Name + "\", 100)]\r\n");
            }
            strBuilder.Append(strIndent + "public abstract class " + strClassName + " : " + strRowsetDefBaseClassName + " {\r\n");
            if (bGenWS) {
                strBuilder.Append(strIndent + "    /// <summary>\r\n");
                strBuilder.Append(strIndent + String.Format("    /// {0}\r\n", m_cmtResManager.GetMemberDesc("TypedRowsetBase", "m_webServ")));
                strBuilder.Append(strIndent + "    /// </summary>\r\n");
                strBuilder.Append(strIndent + "    private WebServices m_webServ;\r\n");
                strBuilder.Append("\r\n");
            }
            strBuilder.Append(strIndent + "    /// <summary>\r\n");
            strBuilder.Append(strIndent + String.Format("    /// {0}\r\n", m_cmtResManager.GetFuncDesc("TypedRowsetBase", "Constructor")));
            strBuilder.Append(strIndent + "    /// </summary>\r\n");
            strBuilder.Append(strIndent + String.Format("    /// <param name=\"pobj\">    {0}</param>\r\n", m_cmtResManager.GetFuncParamDesc("TypedRowsetBase", "Constructor", "pobj")));
            strBuilder.Append(strIndent + "    public " + strClassName + "(IntPtr pobj) : \r\n");
            if (node == null) {
                strBuilder.Append(strIndent + "                             base(pobj,\r\n");
                strBuilder.Append(strIndent + "                                  new string[] {\r\n");
                iIndex = 0;
                foreach (RepPP.FieldDef fld in rowsetDef.FieldDefs) {
                    iIndex++;
                    strBuilder.Append(strIndent + "                                                \"" + fld.Name + "\"" + 
                                      ((iIndex != iCount) ? "," : "") + "\r\n");
                }
                strBuilder.Append(strIndent + "                                               }) {\r\n");
            } else {
                strBuilder.Append(strIndent + "                             base(pobj) {\r\n");
            }
            strBuilder.Append(strIndent + "    }\r\n");
            if (node == null) {            
                strBuilder.Append("\r\n");
                strBuilder.Append(strIndent + "    /// <summary>\r\n");
                strBuilder.Append(strIndent + String.Format("    /// Create:               {0}\r\n", m_cmtResManager.GetFuncDesc("TypedRowsetBase", "Create1")));
                strBuilder.Append(strIndent + "    /// </summary>\r\n");
                strBuilder.Append(strIndent + String.Format("    /// <param name=\"app\">    {0}</param>\r\n", m_cmtResManager.GetFuncParamDesc("TypedRowsetBase", "Create1", "app")));
                strBuilder.Append(strIndent + "    /// <returns>\r\n");
                strBuilder.Append(strIndent + String.Format("    /// {0}\r\n", m_cmtResManager.GetFuncReturnDesc("TypedRowsetBase", "Create1")));
                strBuilder.Append(strIndent + "    /// </returns>\r\n");
                strBuilder.Append(strIndent + "    public static " + strUserClassName + " Create(RepPP.Application app) {\r\n");
                strBuilder.Append(strIndent + "        return(app.RowsetDefs[\"" + rowsetDef.Name + "\"].Rowsets.Create(RepPP.RowsetType.sdHorizontal) as " + strUserClassName + ");\r\n");
                strBuilder.Append(strIndent + "    }\r\n");

                strBuilder.Append("\r\n");
                strBuilder.Append(strIndent + "    /// <summary>\r\n");
                strBuilder.Append(strIndent + String.Format("    /// Create:               {0}\r\n", m_cmtResManager.GetFuncDesc("TypedRowsetBase", "Create2")));
                strBuilder.Append(strIndent + "    /// </summary>\r\n");
                strBuilder.Append(strIndent + String.Format("    /// <param name=\"app\">    {0}</param>\r\n", m_cmtResManager.GetFuncParamDesc("TypedRowsetBase", "Create2", "app")));
                strBuilder.Append(strIndent + String.Format("    /// <param name=\"type\">   {0}</param>\r\n", m_cmtResManager.GetFuncParamDesc("TypedRowsetBase", "Create2", "type")));
                strBuilder.Append(strIndent + "    /// <returns>\r\n");
                strBuilder.Append(strIndent + String.Format("    /// {0}\r\n", m_cmtResManager.GetFuncReturnDesc("TypedRowsetBase", "Create2")));
                strBuilder.Append(strIndent + "    /// </returns>\r\n");
                strBuilder.Append(strIndent + "    protected static " + strUserClassName + " Create(RepPP.Application app, Type type) {\r\n");
                strBuilder.Append(strIndent + "        RepPP.Rowset                rowsetRetVal;\r\n");
                strBuilder.Append(strIndent + "        RepPP.RowsetDef             rowsetDef;\r\n");
                strBuilder.Append(strIndent + "        Type                        typeOld;\r\n");
                strBuilder.Append(strIndent + "        RepPP.TypedInstanceFactory  typedInstFact;\r\n");
                strBuilder.Append("\r\n");
                strBuilder.Append(strIndent + "        rowsetDef       = app.RowsetDefs[\"" + rowsetDef.Name + "\"];\r\n");
                strBuilder.Append(strIndent + "        typedInstFact   = app.GetObjFactory(typeof(RepPP.Rowset)) as RepPP.TypedInstanceFactory;\r\n");
                strBuilder.Append(strIndent + "        typeOld         = typedInstFact.GetTypeAssociateWithRowsetDef(rowsetDef);\r\n");
                strBuilder.Append(strIndent + "        typedInstFact.AddRowsetDef(rowsetDef, type);\r\n");
                strBuilder.Append(strIndent + "        try {\r\n");
                strBuilder.Append(strIndent + "            rowsetRetVal = rowsetDef.Rowsets.Create(RepPP.RowsetType.sdHorizontal);\r\n");
                strBuilder.Append(strIndent + "        } finally {\r\n");
                strBuilder.Append(strIndent + "            typedInstFact.AddRowsetDef(rowsetDef, typeOld);\r\n");
                strBuilder.Append(strIndent + "        }\r\n");
                strBuilder.Append(strIndent + "        return(rowsetRetVal as " + strUserClassName + ");\r\n");
                strBuilder.Append(strIndent + "    }\r\n");
                if (bGenWS) {
                    strBuilder.Append("\r\n");
                    strBuilder.Append(strIndent + "    /// <summary>\r\n");
                    strBuilder.Append(strIndent + String.Format("    /// CreateWebServicesUtil:  {0}\r\n", m_cmtResManager.GetFuncDesc("TypedRowsetBase", "CreateWebServicesUtil")));
                    strBuilder.Append(strIndent + "    /// </summary>\r\n");
                    strBuilder.Append(strIndent + "    /// <returns>\r\n");
                    strBuilder.Append(strIndent + String.Format("    /// {0}\r\n", m_cmtResManager.GetFuncReturnDesc("TypedRowsetBase", "CreateWebServicesUtil")));
                    strBuilder.Append(strIndent + "    /// </returns>\r\n");
                    strBuilder.Append(strIndent + "    protected override RepPP.WebServicesUtil CreateWebServicesUtil() {\r\n");
                    strBuilder.Append(strIndent + "        RepPP.WebServicesUtil   wsUtil;\r\n");
                    strBuilder.Append("\r\n");
                    strBuilder.Append(strIndent + "        wsUtil = base.CreateWebServicesUtil();\r\n");
                    strBuilder.Append(strIndent + "        wsUtil.RowsetPropertyPrefix = \"" + strRowsetPropertyPrefix + "\";\r\n");
                    strBuilder.Append(strIndent + "        wsUtil.FieldPropertyPrefix  = \"" + strFldPropertyPrefix + "\";\r\n");
                    strBuilder.Append(strIndent + "        return(wsUtil);\r\n");
                    strBuilder.Append(strIndent + "    }\r\n");
                }
                foreach (RepPP.FieldDef fld in rowsetDef.FieldDefs) {
                    strFldType      = GetTypeOfFieldFromFieldType(fld.Type, fld.MaxSize);
                    fld.GetType().Name.Replace("FieldDef", "Field");  //TODO: Oops... doesn't work anymore...
                    strFldPropName  = DecorateFieldPropertyName(strFldPropertyPrefix, GetFieldName(fld));
                    strBuilder.Append("\r\n");
                    strBuilder.Append(strIndent + "    /// <summary>\r\n");
                    strBuilder.Append(strIndent + "    /// " + fld.Prompt + "\r\n");
                    strBuilder.Append(strIndent + "    /// </summary>\r\n");
                    strBuilder.Append(strIndent + "    public RepPP." + strFldType + " " + strFldPropName + " {\r\n");
                    strBuilder.Append(strIndent + "        get {\r\n");
                    strBuilder.Append(strIndent + "            return(Fields[\"" + fld.Name + "\"] as RepPP." + strFldType + ");\r\n");
                    strBuilder.Append(strIndent + "        }\r\n");
                    strBuilder.Append(strIndent + "    }\r\n");
                }
                if (bGenWS) {
                    strContainerClassName = (node == null) ? strClassName : DecorateRowsetTreeClassName(strRowsetTreeClassPrefix, GetRowsetTreeName(node.RowsetTreeDef));
                    strBuilder.Append("\r\n");
                    strBuilder.Append(wsGen.GenerateRowsetCode(strContainerClassName, node, rowsetDef, iLevel * 4 + 4, "", strRowsetClassPrefix, bUseWSDataContracts));
                }
            } else {
                foreach (RepPP.RowsetTreeDefNode nodeChild in node.Nodes) {
                    strRowsetName       = nodeChild.RowsetDef.Name;
                    strRowsetPropName   = DecorateRowsetPropertyName(strRowsetPropertyPrefix, GetNodeRowsetName(nodeChild));
                    strChildClassName   = DecorateRowsetClassName(strRowsetClassPrefix, GetNodeRowsetName(nodeChild));
                    strBuilder.Append("\r\n");
                    strBuilder.Append(strIndent + "    /// <summary>\r\n");
                    strBuilder.Append(strIndent + String.Format("    /// {0} {1}\r\n", m_cmtResManager.GetFuncDesc("TypedRowsetBase", "RowsetPropName"), strRowsetName));
                    strBuilder.Append(strIndent + "    /// </summary>\r\n");
                    strBuilder.Append(strIndent + "    public " + strChildClassName + " " + strRowsetPropName + " {\r\n");
                    strBuilder.Append(strIndent + "        get {\r\n");
                    strBuilder.Append(strIndent + "            RepPP.Rowset  rowsetChild;\r\n");
                    strBuilder.Append("\r\n");
                    strBuilder.Append(strIndent + "            rowsetChild = (CurrentLine != -1) ? ChildRowsets[\"" + strRowsetName + "\"] : null;\r\n");
                    strBuilder.Append(strIndent + "            return(rowsetChild as " + strChildClassName + ");\r\n");
                    strBuilder.Append(strIndent + "        }\r\n");
                    strBuilder.Append(strIndent + "    }\r\n");
                }
                foreach (RepPP.RowsetTreeDefNode nodeChild in node.Nodes) {
		            strBuilder.Append(IntGenerateTypedRowset(nodeChild,
		                                                     nodeChild.RowsetDef,
		                                                     strRowsetTreeClassPrefix,
		                                                     strRowsetClassPrefix,
		                                                     strRowsetPropertyPrefix,
		                                                     strFldPropertyPrefix,
                                                             strNameSpace,
                                                             strNameSpace + "." + DecorateRowsetClassName(strRowsetClassPrefix, GetRowsetName(nodeChild.RowsetDef)),
		                                                     iLevel + 1,
		                                                     bGenWS,
                                                             bUseWSDataContracts));
                }
            }
            strBuilder.Append(strIndent + "} // Class " + strClassName + "\r\n");
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateRowsetProperty:                 Generate the rowset properties
        /// </summary>
        /// <param name="node">                     Child rowset definition</param>
        /// <param name="strRowsetClassPrefix">     Prefix for generated rowset class name</param>
        /// <param name="strRowsetPropertyPrefix">  Prefix for generated rowset property name</param>
        /// <returns>
        /// The properties code
        /// </returns>
        //
        //******************************************************************
        private string GenerateRowsetProperty(RepPP.RowsetTreeDefNode   node,
                                              string                    strRowsetClassPrefix,
                                              string                    strRowsetPropertyPrefix) {
            StringBuilder   strBuilder;
            string          strRowsetDefClassName;
            string          strRowsetName;
            string          strRowsetPropName;

            strBuilder              = new StringBuilder(null, 4096);            
            strRowsetName           = node.RowsetDef.Name;
            strRowsetPropName       = DecorateRowsetPropertyName(strRowsetPropertyPrefix, GetNodeRowsetName(node));
            strRowsetDefClassName   = DecorateRowsetClassName(strRowsetClassPrefix, GetRowsetName(node.RowsetDef));
            strBuilder.Append("\r\n");
            strBuilder.Append("        /// <summary>\r\n");
            strBuilder.Append(String.Format("        /// {0} {1}\r\n", m_cmtResManager.GetFuncDesc("TypedRowsetTreeBase", "RowsetPropName"), strRowsetName));
            strBuilder.Append("        /// </summary>\r\n");
            strBuilder.Append("        public " + strRowsetDefClassName + " " + strRowsetPropName + " {\r\n");
            strBuilder.Append("            get {\r\n");
            if (node.Parent == null) {
                strBuilder.Append("                return(RootRowset as " + strRowsetDefClassName + ");\r\n");
            } else {
                strBuilder.Append("                return(GetCurrentRowset(RowsetTreeDef.FindNode(\"" + strRowsetName + "\")) as " + strRowsetDefClassName + ");\r\n");
            }
            strBuilder.Append("            }\r\n");
            strBuilder.Append("        }\r\n");
            foreach (RepPP.RowsetTreeDefNode nodeChild in node.Nodes) {
                strBuilder.Append(GenerateRowsetProperty(nodeChild, strRowsetClassPrefix, strRowsetPropertyPrefix));
            }
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// IntGenerateTypedRowsetTree:                 Generate typed instance for a rowset tree
        /// </summary>
        /// <param name="rowsetTreeDef">                Rowset tree definition</param>
        /// <param name="strRowsetTreeClassPrefix">     Prefix for generated rowset tree class name</param>
        /// <param name="strRowsetClassPrefix">         Prefix for generated rowset class name</param>
        /// <param name="strRowsetPropertyPrefix">      Prefix for generated rowset property name</param>
        /// <param name="strFldPropertyPrefix">         Prefix for generated field property name</param>
        /// <param name="strRowsetTreeBaseClassName">   Name of the base class for the rowset tree</param>
        /// <param name="strRowsetBaseClassName">       Name of the base class for the rowsets</param>
        /// <param name="bGenWS">                       true to generate Web Services</param>
        /// <param name="bUseWSDataContracts">          true to generate Web Service Adapters using DataContract</param>
        /// <returns>
        /// Generated class
        /// </returns>
        //
        //******************************************************************
		private string IntGenerateTypedRowsetTree(RepPP.RowsetTreeDef   rowsetTreeDef,
                                                  string                strRowsetTreeClassPrefix,
                                                  string                strRowsetClassPrefix,
                                                  string                strRowsetPropertyPrefix,
                                                  string                strFldPropertyPrefix,
                                                  string                strRowsetTreeBaseClassName,
                                                  string                strRowsetBaseClassName,
                                                  bool                  bGenWS,
                                                  bool                  bUseWSDataContracts) {
            StringBuilder           strBuilder;
            WebServicesGenBase      wsGen;
            RepPP.RowsetTreeDefNode nodeRoot;
            string                  strRowsetTreeDefClassName;
            string                  strBaseClassName;
            string                  strUserRowsetTreeDefClassName;
            
            wsGen                         = WSGenerator;
            if (bGenWS) {
                wsGen.RowsetTreeClassPrefix   = WEB_SERV_CLASS_PREFIX + strRowsetTreeClassPrefix;
                wsGen.RowsetClassPrefix       = WEB_SERV_CLASS_PREFIX + strRowsetClassPrefix;
                wsGen.RowsetPropertyPrefix    = strRowsetPropertyPrefix;
                wsGen.FieldPropertyPrefix     = strFldPropertyPrefix;
            }
            strBuilder                    = new StringBuilder(null, 4096);            
            strRowsetTreeDefClassName     = DecorateRowsetTreeClassName(TYPED_ROWSET_TREE_PREFIX + strRowsetTreeClassPrefix, GetRowsetTreeName(rowsetTreeDef));
            strBaseClassName              = (strRowsetTreeBaseClassName != String.Empty) ? strRowsetTreeBaseClassName : "RepPP.TypedRowsetTreeBase";
            strUserRowsetTreeDefClassName = DecorateRowsetTreeClassName(strRowsetTreeClassPrefix, GetRowsetTreeName(rowsetTreeDef));
            strBuilder.Append("    /// <summary>\r\n");
            strBuilder.Append(String.Format("    /// {0} {1}\r\n", m_cmtResManager.GetClassDesc("TypedRowsetTreeBase"), rowsetTreeDef.Name));
            strBuilder.Append("    /// </summary>\r\n");
            strBuilder.Append("    [RepPP.TypedInstanceClass(\"" + rowsetTreeDef.Name + "\", 100)]\r\n");
            strBuilder.Append("    public abstract class " + strRowsetTreeDefClassName + " : " + strBaseClassName + " {\r\n");
            strBuilder.Append("\r\n");
            if (bGenWS) {
                strBuilder.Append("        /// <summary>\r\n");
                strBuilder.Append(String.Format("        /// {0}\r\n", m_cmtResManager.GetMemberDesc("TypedRowsetTreeBase", "m_webServ")));
                strBuilder.Append("        /// </summary>\r\n");
                strBuilder.Append("        private WebServices m_webServ;\r\n");
                strBuilder.Append("\r\n");
            }
            strBuilder.Append("        /// <summary>\r\n");
            strBuilder.Append(String.Format("        /// {0}\r\n", m_cmtResManager.GetFuncDesc("TypedRowsetTreeBase", "Constructor")));
            strBuilder.Append("        /// </summary>\r\n");
            strBuilder.Append(String.Format("        /// <param name=\"pobj\">    {0}</param>\r\n", m_cmtResManager.GetFuncParamDesc("TypedRowsetTreeBase", "Constructor", "pobj")));
            strBuilder.Append("        public " + strRowsetTreeDefClassName + "(IntPtr pobj) : base(pobj) {\r\n");
            strBuilder.Append("        }\r\n");
            strBuilder.Append("\r\n");
            strBuilder.Append("        /// <summary>\r\n");
            strBuilder.Append(String.Format("        /// Create:               {0}\r\n", m_cmtResManager.GetFuncDesc("TypedRowsetTreeBase", "Create1")));
            strBuilder.Append("        /// </summary>\r\n");
            strBuilder.Append(String.Format("        /// <param name=\"app\">    {0}</param>\r\n", m_cmtResManager.GetFuncParamDesc("TypedRowsetTreeBase", "Create1", "app")));
            strBuilder.Append("        /// <returns>\r\n");
            strBuilder.Append(String.Format("        /// {0}\r\n", m_cmtResManager.GetFuncReturnDesc("TypedRowsetTreeBase", "Create1")));
            strBuilder.Append("        /// </returns>\r\n");
            strBuilder.Append("        public static " + strUserRowsetTreeDefClassName + " Create(RepPP.Application app) {\r\n");
            strBuilder.Append("            return(app.RowsetTreeDefs[\"" + rowsetTreeDef.Name + "\"].RowsetTrees.Add() as " + strUserRowsetTreeDefClassName + ");\r\n");
            strBuilder.Append("        }\r\n");
            strBuilder.Append("\r\n");
            strBuilder.Append("        /// <summary>\r\n");
            strBuilder.Append(String.Format("        /// Create:               {0}\r\n", m_cmtResManager.GetFuncDesc("TypedRowsetTreeBase", "Create2")));
            strBuilder.Append("        /// </summary>\r\n");
            strBuilder.Append(String.Format("        /// <param name=\"app\">    {0}</param>\r\n", m_cmtResManager.GetFuncParamDesc("TypedRowsetTreeBase", "Create2", "app")));
            strBuilder.Append(String.Format("        /// <param name=\"type\">   {0}>\r\n", m_cmtResManager.GetFuncParamDesc("TypedRowsetTreeBase", "Create2", "type")));
            strBuilder.Append("        /// <returns>\r\n");
            strBuilder.Append(String.Format("        /// {0}\r\n", m_cmtResManager.GetFuncReturnDesc("TypedRowsetTreeBase", "Create2")));
            strBuilder.Append("        /// </returns>\r\n");
            strBuilder.Append("        protected static " + strUserRowsetTreeDefClassName + " Create(RepPP.Application app, Type type) {\r\n");
            strBuilder.Append("            RepPP.RowsetTree            rowsetTreeRetVal;\r\n");
            strBuilder.Append("            RepPP.RowsetTreeDef         rowsetTreeDef;\r\n");
            strBuilder.Append("            Type                        typeOld;\r\n");
            strBuilder.Append("            RepPP.TypedInstanceFactory  typedInstFact;\r\n");
            strBuilder.Append("\r\n");
            strBuilder.Append("            rowsetTreeDef   = app.RowsetTreeDefs[\"" + rowsetTreeDef.Name + "\"];\r\n");
            strBuilder.Append("            typedInstFact   = app.GetObjFactory(typeof(RepPP.RowsetTree)) as RepPP.TypedInstanceFactory;\r\n");
            strBuilder.Append("            typeOld         = typedInstFact.GetTypeAssociateWithRowsetTreeDef(rowsetTreeDef);\r\n");
            strBuilder.Append("            typedInstFact.AddRowsetTreeDef(rowsetTreeDef, type);\r\n");
            strBuilder.Append("            try {\r\n");
            strBuilder.Append("                rowsetTreeRetVal = rowsetTreeDef.RowsetTrees.Add();\r\n");
            strBuilder.Append("            } finally {\r\n");
            strBuilder.Append("                typedInstFact.AddRowsetTreeDef(rowsetTreeDef, typeOld);\r\n");
            strBuilder.Append("            }\r\n");
            strBuilder.Append("            return(rowsetTreeRetVal as " + strUserRowsetTreeDefClassName + ");\r\n");
            strBuilder.Append("        }\r\n");
            if (bGenWS) {
                strBuilder.Append("\r\n");
                strBuilder.Append("        /// <summary>\r\n");
                strBuilder.Append(String.Format("        /// CreateWebServicesUtil:  {0}\r\n", m_cmtResManager.GetFuncDesc("TypedRowsetTreeBase", "CreateWebServicesUtil")));
                strBuilder.Append("        /// </summary>\r\n");
                strBuilder.Append("        /// <returns>\r\n");
                strBuilder.Append(String.Format("        /// {0}\r\n", m_cmtResManager.GetFuncReturnDesc("TypedRowsetTreeBase", "CreateWebServicesUtil")));
                strBuilder.Append("        /// </returns>\r\n");
                strBuilder.Append("        protected override RepPP.WebServicesUtil CreateWebServicesUtil() {\r\n");
                strBuilder.Append("            RepPP.WebServicesUtil   wsUtil;\r\n");
                strBuilder.Append("\r\n");
                strBuilder.Append("            wsUtil = base.CreateWebServicesUtil();\r\n");
                strBuilder.Append("            wsUtil.RowsetPropertyPrefix = \"" + strRowsetPropertyPrefix + "\";\r\n");
                strBuilder.Append("            wsUtil.FieldPropertyPrefix  = \"" + strFldPropertyPrefix + "\";\r\n");
                strBuilder.Append("            return(wsUtil);\r\n");
                strBuilder.Append("        }\r\n");
            }
            nodeRoot = rowsetTreeDef.Nodes[0];
            if (nodeRoot != null) {
                strBuilder.Append(GenerateRowsetProperty(nodeRoot, strRowsetClassPrefix, strRowsetPropertyPrefix));
                if (bGenWS) {
                    strBuilder.Append("\r\n");
                    strBuilder.Append(wsGen.GenerateRowsetTreeCode(strRowsetTreeDefClassName, rowsetTreeDef, 8, "", "", strRowsetClassPrefix, bUseWSDataContracts));
                }
            }
            strBuilder.Append("    } // class " + strRowsetTreeDefClassName + "\r\n");
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateUserTypedRowset:            Generate user typed instance for a rowset
        /// </summary>
        /// <param name="rowsetDef">            Rowset definition</param>
        /// <param name="strRowsetClassPrefix"> Prefix for generated rowset class name</param>
        /// <param name="strNameSpace">         The namespace</param>
        /// <param name="strRootNameSpace">     The root namespace of the project</param>
        /// <returns>
        /// The generated class
        /// </returns>
        //
        //******************************************************************
		public override string GenerateUserTypedRowset(RepPP.RowsetDef  rowsetDef, 
                                                       string           strRowsetClassPrefix,
                                                       string           strNameSpace,
                                                       string           strRootNameSpace) {
            StringBuilder   strBuilder;
            bool            bGenNamesapce = true;
            string          strClassName;
            string          strBaseClassName;
            string          strIndent = "    ";

#if !VC71
            bGenNamesapce = (strNameSpace != String.Empty);
#endif
            strBuilder = new StringBuilder(null, 4096);            
            strBuilder.Append(GenerateUserSignature("//"));
            strBuilder.Append("using System;\r\n");
            strBuilder.Append("\r\n");
            if (bGenNamesapce) {
                strBuilder.Append("namespace " + strNameSpace + " {\r\n");
            }

            strClassName     = DecorateRowsetClassName(strRowsetClassPrefix, GetRowsetName(rowsetDef));
            strBaseClassName = TYPED_INST_NAMESPACE + "." + DecorateRowsetClassName(TYPED_ROWSET_PREFIX + strRowsetClassPrefix, GetRowsetName(rowsetDef));
            strBuilder.Append(strIndent + "/// <summary>\r\n");
            strBuilder.Append(strIndent + String.Format("/// {0} {1}\r\n", m_cmtResManager.GetClassDesc("UserTypedRowset"), rowsetDef.Name));
            strBuilder.Append(strIndent + "/// </summary>\r\n");
            strBuilder.Append(strIndent + "[RepPP.TypedInstanceClass(\"" + rowsetDef.Name + "\", 200)]\r\n");
            strBuilder.Append(strIndent + "public class " + strClassName + " : " + strBaseClassName + " {\r\n");
            strBuilder.Append(strIndent + "    /// <summary>\r\n");
            strBuilder.Append(strIndent + String.Format("    /// {0}\r\n", m_cmtResManager.GetFuncDesc("UserTypedRowset", "Constructor")));
            strBuilder.Append(strIndent + "    /// </summary>\r\n");
            strBuilder.Append(strIndent + String.Format("    /// <param name=\"pobj\">    {0}</param>\r\n", m_cmtResManager.GetFuncParamDesc("UserTypedRowset", "Constructor", "pobj")));
            strBuilder.Append(strIndent + "    public " + strClassName + "(IntPtr pobj) : base(pobj) {\r\n");
            strBuilder.Append(strIndent + "    }\r\n");
            strBuilder.Append("\r\n");
            strBuilder.Append(strIndent + "} // Class " + strClassName + "\r\n");
            if (bGenNamesapce) {
                strBuilder.Append("} // namespace " + strNameSpace + "\r\n");
            }
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateTypedRowset:                    Generates typed instance for a rowset
        /// </summary>
        /// <param name="rowsetDef">                Rowset definition</param>
        /// <param name="strRowsetTreeClassPrefix"> Prefix for generated rowset tree class name</param>
        /// <param name="strRowsetClassPrefix">     Prefix for generated rowset class name</param>
        /// <param name="strRowsetPropertyPrefix">  Prefix for generated rowset property name</param>
        /// <param name="strFldPropertyPrefix">     Prefix for generated field property name</param>
        /// <param name="strNameSpace">             The namespace</param>
        /// <param name="strRootNameSpace">         The root namespace of the project</param>
        /// <param name="strBaseClassName">         Name of the base class</param>
        /// <param name="bGenWS">                   true to generate Web Services</param>
        /// <param name="bUseWSDataContracts">      true to generate Web Service Adapters using DataContract</param>
        /// <returns>
        /// The generated class
        /// </returns>
        //
        //******************************************************************
		public override string GenerateTypedRowset(RepPP.RowsetDef  rowsetDef, 
                                                   string           strRowsetTreeClassPrefix,
                                                   string           strRowsetClassPrefix,
                                                   string           strRowsetPropertyPrefix,
                                                   string           strFldPropertyPrefix,
                                                   string           strNameSpace,
                                                   string           strRootNameSpace,
                                                   string           strBaseClassName,
                                                   bool             bGenWS,
                                                   bool             bUseWSDataContracts) {
            StringBuilder   strBuilder;
            string          strRealNameSpace;

            strRealNameSpace = (strNameSpace != String.Empty) ? strNameSpace + "." + TYPED_INST_NAMESPACE : TYPED_INST_NAMESPACE;
            strBuilder       = new StringBuilder(null, 4096);            
            strBuilder.Append(GenerateSignature("//"));
            strBuilder.Append("using System;\r\n");
            if (bGenWS && bUseWSDataContracts) {
                strBuilder.Append("using System.Runtime.Serialization;\r\n");
                strBuilder.Append("using System.Xml.Serialization;\r\n");
            }
            strBuilder.Append("\r\n");
            strBuilder.Append("namespace " + strRealNameSpace + " {\r\n");
            strBuilder.Append(IntGenerateTypedRowset(null,
                                                     rowsetDef,
                                                     strRowsetTreeClassPrefix,
                                                     strRowsetClassPrefix,
                                                     strRowsetPropertyPrefix,
                                                     strFldPropertyPrefix,
                                                     strNameSpace,
                                                     strBaseClassName,
                                                     1,
                                                     bGenWS,
                                                     bUseWSDataContracts));
            strBuilder.Append("} // namespace " + strRealNameSpace + "\r\n");
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateTypedRowsetTree:                    Generate typed instance for a rowset tree
        /// </summary>
        /// <param name="rowsetTreeDef">                Rowset tree definition</param>
        /// <param name="strRowsetTreeClassPrefix">     Prefix for generated rowset tree class name</param>
        /// <param name="strRowsetClassPrefix">         Prefix for generated rowset class name</param>
        /// <param name="strRowsetPropertyPrefix">      Prefix for generated rowset property name</param>
        /// <param name="strFldPropertyPrefix">         Prefix for generated field property name</param>
        /// <param name="strNameSpace">                 The namespace</param>
        /// <param name="strRootNameSpace">             The root namespace of the project</param>
        /// <param name="strRowsetTreeBaseClassName">   Name of the base class for the rowset tree</param>
        /// <param name="strRowsetBaseClassName">       Name of the base class for the rowset</param>
        /// <param name="bGenWS">                       true to generate Web Services</param>
        /// <param name="bUseWSDataContracts">          true to generate Web Service Adapters using DataContract</param>
        /// <returns>
        /// Generated class
        /// </returns>
        //
        //******************************************************************
		public override string GenerateTypedRowsetTree(RepPP.RowsetTreeDef  rowsetTreeDef, 
                                                       string               strRowsetTreeClassPrefix,
                                                       string               strRowsetClassPrefix,
                                                       string               strRowsetPropertyPrefix,
                                                       string               strFldPropertyPrefix,
                                                       string               strNameSpace,
                                                       string               strRootNameSpace,
                                                       string               strRowsetTreeBaseClassName,
                                                       string               strRowsetBaseClassName,
                                                       bool                 bGenWS,
                                                       bool                 bUseWSDataContracts) {
            StringBuilder           strBuilder;
            RepPP.RowsetTreeDefNode nodeRoot;
            string                  strRealNameSpace;

            strRealNameSpace = (strNameSpace != String.Empty) ? strNameSpace + "." + TYPED_INST_NAMESPACE : TYPED_INST_NAMESPACE;
            strBuilder       = new StringBuilder(null, 4096);            
            if (rowsetTreeDef.Nodes.Count != 0) {
                nodeRoot = rowsetTreeDef.Nodes[0];
                strBuilder.Append(GenerateSignature("//"));
                strBuilder.Append("using System;\r\n");
                if (bGenWS && bUseWSDataContracts) {
                    strBuilder.Append("using System.Runtime.Serialization;\r\n");
                    strBuilder.Append("using System.Xml.Serialization;\r\n");
                }
                strBuilder.Append("\r\n");
                strBuilder.Append("namespace " + strRealNameSpace + " {\r\n");
                strBuilder.Append(IntGenerateTypedRowsetTree(rowsetTreeDef, 
                                                             strRowsetTreeClassPrefix,
                                                             strRowsetClassPrefix,
                                                             strRowsetPropertyPrefix,
                                                             strFldPropertyPrefix,
                                                             strRowsetTreeBaseClassName,
                                                             strRowsetBaseClassName,
                                                             bGenWS,
                                                             bUseWSDataContracts));
                strBuilder.Append("} // namespace " + strRealNameSpace + "\r\n");
            }
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateUserTypedRowsetTree:            Generate user typed instance for a rowset tree
        /// </summary>
        /// <param name="rowsetTreeDef">            Rowset tree definition</param>
        /// <param name="strRowsetTreeClassPrefix"> Prefix for generated rowset tree class name</param>
        /// <param name="strNameSpace">             The namespace</param>
        /// <param name="strRootNameSpace">         The root namespace of the project</param>
        /// <returns>
        /// Generated class
        /// </returns>
        //
        //******************************************************************
		public override string GenerateUserTypedRowsetTree(RepPP.RowsetTreeDef  rowsetTreeDef, 
                                                           string               strRowsetTreeClassPrefix,
                                                           string               strNameSpace,
                                                           string               strRootNameSpace) {
            StringBuilder   strBuilder;
            bool            bGenNamespace = true;
            string          strRowsetTreeDefClassName;
            string          strBaseClassName;

#if !VC71
            bGenNamespace = (strNameSpace != String.Empty);
#endif            
            strBuilder = new StringBuilder(null, 4096);            
            if (rowsetTreeDef.Nodes.Count != 0) {
                strBuilder.Append(GenerateUserSignature("//"));
                strBuilder.Append("using System;\r\n");
                strBuilder.Append("\r\n");
                if (bGenNamespace) {
                    strBuilder.Append("namespace " + strNameSpace + " {\r\n");
                }
                strRowsetTreeDefClassName  = DecorateRowsetTreeClassName(strRowsetTreeClassPrefix, GetRowsetTreeName(rowsetTreeDef));
                strBaseClassName  = TYPED_INST_NAMESPACE + "." + DecorateRowsetTreeClassName(TYPED_ROWSET_TREE_PREFIX + strRowsetTreeClassPrefix, GetRowsetTreeName(rowsetTreeDef));
                strBuilder.Append("    /// <summary>\r\n");
                strBuilder.Append(String.Format("    /// {0} {1}\r\n", m_cmtResManager.GetClassDesc("UserTypedRowsetTree"), rowsetTreeDef.Name));
                strBuilder.Append("    /// </summary>\r\n");
                strBuilder.Append("    [RepPP.TypedInstanceClass(\"" + rowsetTreeDef.Name + "\", 200)]\r\n");
                strBuilder.Append("    public class " + strRowsetTreeDefClassName + " : " + strBaseClassName + " {\r\n");
                strBuilder.Append("\r\n");
                strBuilder.Append("        /// <summary>\r\n");
                strBuilder.Append(String.Format("        /// {0}\r\n", m_cmtResManager.GetFuncDesc("UserTypedRowsetTree", "Constructor")));
                strBuilder.Append("        /// </summary>\r\n");
                strBuilder.Append(String.Format("        /// <param name=\"pobj\">    {0}</param>\r\n", m_cmtResManager.GetFuncParamDesc("UserTypedRowsetTree", "Constructor", "pobj")));
                strBuilder.Append("        public " + strRowsetTreeDefClassName + "(IntPtr pobj) : base(pobj) {\r\n");
                strBuilder.Append("        }\r\n");
                strBuilder.Append("\r\n");
                strBuilder.Append("    } // class " + strRowsetTreeDefClassName + "\r\n");
                if (bGenNamespace) {
                    strBuilder.Append("} // namespace " + strNameSpace + "\r\n");
                }
            }
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
		/// <summary>
		/// RegenerateTypedInstance:            Regenerates all rowset trees and rowsets which need to be regenerated
		/// </summary>
		/// <param name="app">                  Rep++ application object</param>
        /// <param name="log">                  Log object for name clash. Null for no name clash check</param>
		/// <param name="projFolder">           Interface with the project folder</param>
		/// <param name="bFullSync">            true to fully synchronize type. false to only sync the existing one</param>
        /// <param name="bUseWSDataContracts">  true to use DataContract</param>
        //
        //******************************************************************
        public override void RegenerateTypedInstance(RepPP.Application app, ILog log, IProjectFolder projFolder, bool bFullSync, bool bUseWSDataContracts) {
		    RegenerateTypedInstance(app, log, projFolder, ".cs", bFullSync, bUseWSDataContracts);
		}

        //******************************************************************
        //
		/// <summary>
		/// RegenerateTypedInstance:            Regenerates all rowset trees and rowset which need to be regenerated
		/// </summary>
        /// <param name="log">                  Log object for name clash. Null for no name clash check</param>
		/// <param name="projFolder">           Interface with the project folder</param>
		/// <param name="bFullSync">            true to fully synchronize type. false to only sync the existing one</param>
        /// <param name="bUseWSDataContracts">  true to use DataContract</param>
        //
        //******************************************************************
        public override void RegenerateTypedInstance(ILog log, IProjectFolder projFolder, bool bFullSync, bool bUseWSDataContracts) {
		    RegenerateTypedInstance(null, log, projFolder, ".cs", bFullSync, bUseWSDataContracts);
		}
	} // class TypedInstanceGenForCS
} // namespace
