using System;
using System.Text;

namespace RepPP.Toolkit.CodeGen {
	/// <summary>
	/// TypedInstanceGenForVB:  Typed instance generator for visual basic.
	/// </summary>
	public class TypedInstanceGenForVB : TypedInstanceGenBase {
#if VC71
            private const string            m_strRem = "'";
#else
            private const string            m_strRem = "'''";
#endif

        //******************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //******************************************************************
		public TypedInstanceGenForVB() : base(new WebServicesGenForVB()) {
		}

        //******************************************************************
        //
        /// <summary>
        /// IntGenerateTypedRowset:                 Generate typed instance for a rowset
        /// </summary>
        /// <param name="node">                     Child rowset definition</param>
        /// <param name="rowsetDef">                Rowset definition</param>
        /// <param name="strRowsetTreeClassPrefix"> Prefix for generated rowset class name</param>
        /// <param name="strRowsetClassPrefix">     Prefix for generated rowset class name</param>
        /// <param name="strRowsetPropertyPrefix">  Prefix for generated rowset property name</param>
        /// <param name="strFldPropertyPrefix">     Prefix for generated field property name</param>
        /// <param name="strNameSpace">             Namespace</param>
        /// <param name="strBaseClassName">         Name of the base class</param>
        /// <param name="iLevel">                   Right pad level</param>
        /// <param name="bGenWS">                   true to generate Web Services</param>
        /// <param name="bUseWSDataContracts">      true to use DataContract</param>
        /// <returns>
        /// The generated class
        /// </returns>
        //
        //******************************************************************        
		private string IntGenerateTypedRowset(RepPP.RowsetTreeDefNode   node,
		                                      RepPP.RowsetDef           rowsetDef,
                                              string                    strRowsetTreeClassPrefix,
                                              string                    strRowsetClassPrefix,
                                              string                    strRowsetPropertyPrefix,
                                              string                    strFldPropertyPrefix,
                                              string                    strNameSpace,
                                              string                    strBaseClassName,
		                                      int                       iLevel,
		                                      bool                      bGenWS,
                                              bool                      bUseWSDataContracts) {
            StringBuilder           strBuilder;
            WebServicesGenBase      wsGen;
            string                  strRowsetName;
            string                  strRowsetPropName;
            string                  strClassName;
            string                  strFldPropName;
            string                  strChildClassName;
            string                  strContainerClassName;
            string                  strRowsetDefBaseClassName;
            string                  strIndent;
            string                  strFldType;
            int                     iCount;
            int                     iIndex;
            string                  strUserClassName;
            
            wsGen = WSGenerator;
            if (bGenWS) {
                wsGen.RowsetTreeClassPrefix = WEB_SERV_CLASS_PREFIX + strRowsetTreeClassPrefix;
                wsGen.RowsetClassPrefix     = WEB_SERV_CLASS_PREFIX + strRowsetClassPrefix;
                wsGen.RowsetPropertyPrefix  = strRowsetPropertyPrefix;
                wsGen.FieldPropertyPrefix   = strFldPropertyPrefix;
            }
            strIndent                 = "".PadRight(iLevel * 4, ' ');
            strBuilder                = new StringBuilder(null, 4096);        
            iCount                    = rowsetDef.FieldDefs.Count;
            strClassName              = DecorateRowsetClassName(TYPED_ROWSET_PREFIX + strRowsetClassPrefix, ((node != null) ? GetNodeRowsetName(node) : GetRowsetName(rowsetDef)));
            strRowsetDefBaseClassName = (strBaseClassName != String.Empty) ? strBaseClassName : "RepPP.TypedRowsetBase";
            strUserClassName          = DecorateRowsetClassName(strRowsetClassPrefix, (node != null) ? GetNodeRowsetName(node) : GetRowsetName(rowsetDef));
            strBuilder.Append(strIndent + m_strRem + " <summary>\r\n");
            strBuilder.Append(strIndent + m_strRem + String.Format(" {0} {1}\r\n", m_cmtResManager.GetClassDesc("TypedRowsetBase"), rowsetDef.Name));
            strBuilder.Append(strIndent + m_strRem + " </summary>\r\n");
            if (node == null) {
                strBuilder.Append(strIndent + "<RepPP.TypedInstanceClass(\"" + rowsetDef.Name + "\", 100)> _\r\n");
            }
            strBuilder.Append(strIndent + "Public MustInherit Class " + strClassName + "\r\n");
            strBuilder.Append(strIndent + "    Inherits " + strRowsetDefBaseClassName + "\r\n");
            if (bGenWS) {
                strBuilder.Append(strIndent + "    " + m_strRem + " <summary>\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + String.Format(" {0}\r\n", m_cmtResManager.GetMemberDesc("TypedRowsetBase", "m_webServ")));
                strBuilder.Append(strIndent + "    " + m_strRem + " </summary>\r\n");
                strBuilder.Append(strIndent + "    Private m_webServ As WebServices\r\n");
                strBuilder.Append("\r\n");
            }
            strBuilder.Append(strIndent + "    " + m_strRem + " <summary>" + "\r\n");
            strBuilder.Append(strIndent + "    " + m_strRem + String.Format(" {0}\r\n", m_cmtResManager.GetFuncDesc("TypedRowsetBase", "Constructor")));
            strBuilder.Append(strIndent + "    " + m_strRem + " </summary>\r\n");
            strBuilder.Append(strIndent + "    " + m_strRem + String.Format(" <param name=\"pobj\">    {0}</param>\r\n", m_cmtResManager.GetFuncParamDesc("TypedRowsetBase", "Constructor", "pobj")));
            strBuilder.Append(strIndent + "    Public Sub New(pobj As IntPtr)\r\n");
            if (node == null) {
                strBuilder.Append(strIndent + "        MyBase.New(pobj, _\r\n");
                strBuilder.Append(strIndent + "               New string() { _\r\n");
                iIndex = 0;
                foreach (RepPP.FieldDef fld in rowsetDef.FieldDefs) {
                    iIndex = iIndex + 1;
                    strBuilder.Append(strIndent + "                                                \"" + fld.Name + "\"" +
                                        (iIndex != iCount ? ", _" : " _") + "\r\n");
                }
                strBuilder.Append(strIndent + "                                               })\r\n");
            } else {
                strBuilder.Append(strIndent + "        MyBase.New(pobj)\r\n");
            }
            strBuilder.Append(strIndent + "    End Sub\r\n");
            if (node == null) {
                strBuilder.Append("\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + " <summary>\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + String.Format(" Create:               {0}\r\n", m_cmtResManager.GetFuncDesc("TypedRowsetBase", "Create1")));
                strBuilder.Append(strIndent + "    " + m_strRem + " </summary>\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + String.Format(" <param name=\"app\">    {0}</param>\r\n", m_cmtResManager.GetFuncParamDesc("TypedRowsetBase", "Create1", "app")));
                strBuilder.Append(strIndent + "    " + m_strRem + " <returns>\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + String.Format(" {0}\r\n", m_cmtResManager.GetFuncReturnDesc("TypedRowsetBase", "Create1")));
                strBuilder.Append(strIndent + "    " + m_strRem + " </returns>\r\n");
                strBuilder.Append(strIndent + "    Public Shared Function Create(app As RepPP.Application) As " + strUserClassName + "\r\n");
                strBuilder.Append(strIndent + "        Return CType(app.RowsetDefs(\"" + rowsetDef.Name + "\").Rowsets.Create(RepPP.RowsetType.sdHorizontal), " + strUserClassName + ")\r\n");
                strBuilder.Append(strIndent + "    End Function\r\n");
                strBuilder.Append("\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + " <summary>\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + String.Format(" Create:               {0}\r\n", m_cmtResManager.GetFuncDesc("TypedRowsetBase", "Create2")));
                strBuilder.Append(strIndent + "    " + m_strRem + " </summary>\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + String.Format(" <param name=\"app\">    {0}</param>\r\n", m_cmtResManager.GetFuncParamDesc("TypedRowsetBase", "Create2", "app")));
                strBuilder.Append(strIndent + "    " + m_strRem + String.Format(" <param name=\"type\">   {0}</param>\r\n", m_cmtResManager.GetFuncParamDesc("TypedRowsetBase", "Create2", "type")));
                strBuilder.Append(strIndent + "    " + m_strRem + " <returns>\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + String.Format(" {0}\r\n", m_cmtResManager.GetFuncReturnDesc("TypedRowsetBase", "Create2")));
                strBuilder.Append(strIndent + "    " + m_strRem + " </returns>\r\n");
                strBuilder.Append(strIndent + "    Protected Shared Function Create(ByVal app As RepPP.Application, type As Type) As " + strUserClassName + "\r\n");
                strBuilder.Append(strIndent + "        Dim rowsetRetVal    As RepPP.Rowset\r\n");
                strBuilder.Append(strIndent + "        Dim rowsetDef       As RepPP.RowsetDef\r\n");
                strBuilder.Append(strIndent + "        Dim typeOld         As Type\r\n");
                strBuilder.Append(strIndent + "        Dim typedInstFact   As RepPP.TypedInstanceFactory\r\n");
                strBuilder.Append("\r\n");
                strBuilder.Append(strIndent + "        rowsetDef       = app.RowsetDefs(\"" + rowsetDef.Name + "\")\r\n");
                strBuilder.Append(strIndent + "        typedInstFact   = CType(app.GetObjFactory(GetType(RepPP.Rowset)), RepPP.TypedInstanceFactory)\r\n");
                strBuilder.Append(strIndent + "        typeOld         = typedInstFact.GetTypeAssociateWithRowsetDef(rowsetDef)\r\n");
                strBuilder.Append(strIndent + "        typedInstFact.AddRowsetDef(rowsetDef, type)\r\n");
                strBuilder.Append(strIndent + "        Try\r\n");
                strBuilder.Append(strIndent + "            rowsetRetVal = rowsetDef.Rowsets.Create(RepPP.RowsetType.sdHorizontal)\r\n");
                strBuilder.Append(strIndent + "        Finally\r\n");
                strBuilder.Append(strIndent + "            typedInstFact.AddRowsetDef(rowsetDef, typeOld)\r\n");
                strBuilder.Append(strIndent + "        End Try\r\n");
                strBuilder.Append(strIndent + "        Return CType(rowsetRetVal, " + strUserClassName + ")\r\n");
                strBuilder.Append(strIndent + "    End Function\r\n");
                if (bGenWS) {
                    strBuilder.Append("\r\n");
                    strBuilder.Append("        " + m_strRem + " <summary>\r\n");
                    strBuilder.Append("        " + m_strRem + String.Format(" CreateWebServicesUtil:  {0}\r\n", m_cmtResManager.GetFuncDesc("TypedRowsetBase", "CreateWebServicesUtil")));
                    strBuilder.Append("        " + m_strRem + " </summary>\r\n");
                    strBuilder.Append("        " + m_strRem + " <returns>\r\n");
                    strBuilder.Append("        " + m_strRem + String.Format(" {0}\r\n", m_cmtResManager.GetFuncReturnDesc("TypedRowsetBase", "CreateWebServicesUtil")));
                    strBuilder.Append("        " + m_strRem + " </returns>\r\n");
                    strBuilder.Append("        Protected Overrides Function CreateWebServicesUtil() As RepPP.WebServicesUtil\r\n");
                    strBuilder.Append("            Dim wsUtil As RepPP.WebServicesUtil\r\n");
                    strBuilder.Append("\r\n");
                    strBuilder.Append("            wsUtil = MyBase.CreateWebServicesUtil()\r\n");
                    strBuilder.Append("            wsUtil.RowsetPropertyPrefix = \"" + strRowsetPropertyPrefix + "\"\r\n");
                    strBuilder.Append("            wsUtil.FieldPropertyPrefix  = \"" + strFldPropertyPrefix + "\"\r\n");
                    strBuilder.Append("            Return (wsUtil)\r\n");
                    strBuilder.Append("        End Function\r\n");

                }
                foreach (RepPP.FieldDef fld in rowsetDef.FieldDefs) {
                    strFldPropName  = DecorateFieldPropertyName(strFldPropertyPrefix, GetFieldName(fld));
                    strFldType      = fld.GetType().Name.Replace("FieldDef", "Field");  //TODO: Oops... doesn't work anymore...
                    strBuilder.Append("\r\n");
                    strBuilder.Append(strIndent + "    " + m_strRem + " <summary>\r\n");
                    strBuilder.Append(strIndent + "    " + m_strRem + " " + fld.Prompt + "\r\n");
                    strBuilder.Append(strIndent + "    " + m_strRem + " </summary>\r\n");
                    strBuilder.Append(strIndent + "    Public ReadOnly Property " + strFldPropName + " As RepPP." + strFldType + "\r\n");
                    strBuilder.Append(strIndent + "        Get\r\n");
                    strBuilder.Append(strIndent + "            Return(CType(Fields(\"" + fld.Name + "\"), RepPP." + strFldType + "))\r\n");
                    strBuilder.Append(strIndent + "        End Get\r\n");
                    strBuilder.Append(strIndent + "    End Property\r\n");
                }
                if (bGenWS) {
                    strContainerClassName = (node == null) ? strClassName : DecorateRowsetTreeClassName(strRowsetTreeClassPrefix, GetRowsetTreeName(node.RowsetTreeDef));
                    strBuilder.Append("\r\n");
                    strBuilder.Append(wsGen.GenerateRowsetCode(strContainerClassName, node, rowsetDef, iLevel * 4 + 4, "", strRowsetClassPrefix, bUseWSDataContracts));
                }
            } else {
                foreach (RepPP.RowsetTreeDefNode nodeChild in node.Nodes) {
                    strRowsetName       = nodeChild.RowsetDef.Name;
                    strChildClassName   = DecorateRowsetClassName(strRowsetClassPrefix, GetNodeRowsetName(nodeChild));
                    strRowsetPropName   = DecorateRowsetPropertyName(strRowsetPropertyPrefix, GetNodeRowsetName(nodeChild));
                    strBuilder.Append("\r\n");
                    strBuilder.Append(strIndent + "    " + m_strRem + " <summary>\r\n");
                    strBuilder.Append(strIndent + "    " + m_strRem + String.Format(" {0} {1}\r\n", m_cmtResManager.GetFuncDesc("TypedRowsetBase", "RowsetPropName"), strRowsetName));
                    strBuilder.Append(strIndent + "    " + m_strRem + " </summary>" + "\r\n");
                    strBuilder.Append(strIndent + "    Public ReadOnly Property " + strRowsetPropName + " As " + strChildClassName + "\r\n");
                    strBuilder.Append(strIndent + "        Get \r\n");
                    strBuilder.Append(strIndent + "            Dim rowsetChild As RepPP.Rowset\r\n");
                    strBuilder.Append("\r\n");
                    strBuilder.Append(strIndent + "            rowsetChild = CType(IIf(CurrentLine <> -1, ChildRowsets(\"" + strRowsetName + "\"), Nothing), RepPP.Rowset)\r\n");
                    strBuilder.Append(strIndent + "            Return(CType(rowsetChild, " + strChildClassName + "))\r\n");
                    strBuilder.Append(strIndent + "        End Get\r\n");
                    strBuilder.Append(strIndent + "    End Property\r\n");
                }
                foreach (RepPP.RowsetTreeDefNode nodeChild in node.Nodes) {
		            strBuilder.Append(IntGenerateTypedRowset(nodeChild,
		                                                     nodeChild.RowsetDef,
                                                             strRowsetTreeClassPrefix,
                                                             strRowsetClassPrefix,
                                                             strRowsetPropertyPrefix,
                                                             strFldPropertyPrefix,
                                                             strNameSpace,
                                                             strNameSpace + "." + DecorateRowsetClassName(strRowsetClassPrefix, GetRowsetName(nodeChild.RowsetDef)),
		                                                     iLevel + 1,
		                                                     bGenWS,
                                                             bUseWSDataContracts));
                }
            }
            strBuilder.Append(strIndent + "End Class ' class " + strClassName + "\r\n");
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateRowsetProperty:                 Generate the rowset instances properties
        /// </summary>
        /// <param name="node">                     Child rowset definition</param>
        /// <param name="strRowsetClassPrefix">     Prefix for generated rowset class name</param>
        /// <param name="strRowsetPropertyPrefix">  Prefix for generated rowset property name</param>
        /// <returns>
        /// The properties code
        /// </returns>
        //
        //******************************************************************
        private string GenerateRowsetProperty(RepPP.RowsetTreeDefNode   node,
                                              string                    strRowsetClassPrefix,
                                              string                    strRowsetPropertyPrefix) {
            StringBuilder   strBuilder;
            string          strRowsetDefClassName;
            string          strRowsetName;
            string          strRowsetPropName;

            strBuilder              = new StringBuilder(null, 4096);            
            strRowsetName           = node.RowsetDef.Name;
            strRowsetPropName       = DecorateRowsetPropertyName(strRowsetPropertyPrefix, GetNodeRowsetName(node));
            strRowsetDefClassName   = DecorateRowsetClassName(strRowsetClassPrefix, GetRowsetName(node.RowsetDef));
            strBuilder.Append("\r\n");
            strBuilder.Append("        " + m_strRem + " <summary>\r\n");
            strBuilder.Append("        " + m_strRem + " Gets the wrapper for the instance of rowset " + strRowsetName + "\r\n");
            strBuilder.Append("        " + m_strRem + " </summary>\r\n");
            strBuilder.Append("        Public ReadOnly Property " + strRowsetPropName + " As " + strRowsetDefClassName + "\r\n");
            strBuilder.Append("            Get\r\n");
            if (node.Parent == null) {
                strBuilder.Append("                Return(CType(RootRowset, " + strRowsetDefClassName + "))\r\n");
            } else {
                strBuilder.Append("                Return(CType(GetCurrentRowset(RowsetTreeDef.FindNode(\"" + strRowsetName + "\")), " + strRowsetDefClassName + "))\r\n");
            }
            strBuilder.Append("            End Get\r\n");
            strBuilder.Append("        End Property\r\n");
            foreach (RepPP.RowsetTreeDefNode nodeChild in node.Nodes) {
                strBuilder.Append(GenerateRowsetProperty(nodeChild, strRowsetClassPrefix, strRowsetPropertyPrefix));
            }
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// IntGenerateTypedRowsetTree:                 Generate typed instance for a rowset tree
        /// </summary>
        /// <param name="rowsetTreeDef">                Rowset tree definition</param>
        /// <param name="strRowsetTreeClassPrefix">     Prefix for generated rowset class name</param>
        /// <param name="strRowsetClassPrefix">         Prefix for generated rowset class name</param>
        /// <param name="strRowsetPropertyPrefix">      Prefix for generated rowset property name</param>
        /// <param name="strFldPropertyPrefix">         Prefix for generated field property name</param>
        /// <param name="strRowsetTreeBaseClassName">   Name of the base class for the rowset tree</param>
        /// <param name="strRowsetBaseClassName">       Name of the base class for the rowsets</param>
        /// <param name="bGenWS">                       true to generate Web Services</param>
        /// <param name="bUseWSDataContracts">          true to generate Web Service Adapters using DataContract</param>
        /// <returns>
        /// Generated class
        /// </returns>
        //
        //******************************************************************
		private string IntGenerateTypedRowsetTree(RepPP.RowsetTreeDef   rowsetTreeDef, 
                                                  string                strRowsetTreeClassPrefix,
                                                  string                strRowsetClassPrefix,
                                                  string                strRowsetPropertyPrefix,
                                                  string                strFldPropertyPrefix,
                                                  string                strRowsetTreeBaseClassName,
                                                  string                strRowsetBaseClassName,
                                                  bool                  bGenWS,
                                                  bool                  bUseWSDataContracts) {
            StringBuilder           strBuilder;
            WebServicesGenBase      wsGen;
            RepPP.RowsetTreeDefNode nodeRoot;
            string                  strRowsetTreeDefClassName;
            string                  strBaseClassName;
            string                  strUserRowsetTreeDefClassName;
            
            wsGen = WSGenerator;
            if (bGenWS) {
                wsGen.RowsetTreeClassPrefix   = WEB_SERV_CLASS_PREFIX + strRowsetTreeClassPrefix;
                wsGen.RowsetClassPrefix       = WEB_SERV_CLASS_PREFIX + strRowsetClassPrefix;
                wsGen.RowsetPropertyPrefix    = strRowsetPropertyPrefix;
                wsGen.FieldPropertyPrefix     = strFldPropertyPrefix;
            }
            strBuilder                    = new StringBuilder(null, 4096);         
            strRowsetTreeDefClassName     = DecorateRowsetTreeClassName(TYPED_ROWSET_TREE_PREFIX + strRowsetTreeClassPrefix, GetRowsetTreeName(rowsetTreeDef));
            strBaseClassName              = (strRowsetTreeBaseClassName != String.Empty) ? strRowsetTreeBaseClassName : "RepPP.TypedRowsetTreeBase";
            strUserRowsetTreeDefClassName = DecorateRowsetTreeClassName(strRowsetTreeClassPrefix, GetRowsetTreeName(rowsetTreeDef));
            strBuilder.Append("    " + m_strRem + " <summary>\r\n");
            strBuilder.Append("    " + m_strRem + String.Format(" {0} {1}\r\n", m_cmtResManager.GetClassDesc("TypedRowsetTreeBase"), rowsetTreeDef.Name));
            strBuilder.Append("    " + m_strRem + " </summary>\r\n");
            strBuilder.Append("    <RepPP.TypedInstanceClass(\"" + rowsetTreeDef.Name + "\", 100)> _\r\n");
            strBuilder.Append("    Public MustInherit Class " + strRowsetTreeDefClassName + "\r\n");
            strBuilder.Append("        Inherits " + strBaseClassName + "\r\n");
            strBuilder.Append("\r\n");
            if (bGenWS) {
                strBuilder.Append("        " + m_strRem + " <summary>\r\n");
                strBuilder.Append("        " + m_strRem + String.Format(" {0}\r\n", m_cmtResManager.GetMemberDesc("TypedRowsetTreeBase", "m_webServ")));
                strBuilder.Append("        " + m_strRem + " </summary>\r\n");
                strBuilder.Append("        Private m_webServ As WebServices\r\n");
                strBuilder.Append("\r\n");
            }
            strBuilder.Append("        " + m_strRem + " <summary>\r\n");
            strBuilder.Append("        " + m_strRem + String.Format(" {0}\r\n", m_cmtResManager.GetFuncDesc("TypedRowsetTreeBase", "Constructor")));
            strBuilder.Append("        " + m_strRem + " </summary>\r\n");
            strBuilder.Append("        " + m_strRem + String.Format(" <param name=\"pobj\">    {0}</param>\r\n", m_cmtResManager.GetFuncParamDesc("TypedRowsetTreeBase", "Constructor", "pobj")));
            strBuilder.Append("        Public Sub New(pobj As IntPtr)\r\n");
            strBuilder.Append("            MyBase.New(pobj)\r\n");
            strBuilder.Append("        End Sub\r\n");
            strBuilder.Append("\r\n");
            strBuilder.Append("        " + m_strRem + " <summary>\r\n");
            strBuilder.Append("        " + m_strRem + String.Format(" Create:               {0}\r\n", m_cmtResManager.GetFuncDesc("TypedRowsetTreeBase", "Create1")));
            strBuilder.Append("        " + m_strRem + " </summary>\r\n");
            strBuilder.Append("        " + m_strRem + String.Format(" <param name=\"app\">    {0}</param>\r\n", m_cmtResManager.GetFuncParamDesc("TypedRowsetTreeBase", "Create1", "app")));
            strBuilder.Append("        " + m_strRem + " <returns>\r\n");
            strBuilder.Append("        " + m_strRem + String.Format(" {0}\r\n", m_cmtResManager.GetFuncReturnDesc("TypedRowsetTreeBase", "Create1")));
            strBuilder.Append("        " + m_strRem + " </returns>\r\n");
            strBuilder.Append("        Public Shared Function Create(app As RepPP.Application) As " + strUserRowsetTreeDefClassName + "\r\n");
            strBuilder.Append("            Return CType(app.RowsetTreeDefs(\"" + rowsetTreeDef.Name + "\").RowsetTrees.Add(), " + strUserRowsetTreeDefClassName + ")\r\n");
            strBuilder.Append("        End Function\r\n");

            strBuilder.Append("\r\n");
            strBuilder.Append("        " + m_strRem + " <summary>\r\n");
            strBuilder.Append("        " + m_strRem + String.Format(" Create:               {0}\r\n", m_cmtResManager.GetFuncDesc("TypedRowsetTreeBase", "Create2")));
            strBuilder.Append("        " + m_strRem + " </summary>\r\n");
            strBuilder.Append("        " + m_strRem + String.Format(" <param name=\"app\">    {0}</param>\r\n", m_cmtResManager.GetFuncParamDesc("TypedRowsetTreeBase", "Create2", "app")));
            strBuilder.Append("        " + m_strRem + String.Format(" <param name=\"type\">   {0}</param>\r\n", m_cmtResManager.GetFuncParamDesc("TypedRowsetTreeBase", "Create2", "type")));
            strBuilder.Append("        " + m_strRem + " <returns>\r\n");
            strBuilder.Append("        " + m_strRem + String.Format(" {0}\r\n", m_cmtResManager.GetFuncReturnDesc("TypedRowsetTreeBase", "Create2")));
            strBuilder.Append("        " + m_strRem + " </returns>\r\n");
            strBuilder.Append("        Public Shared Function Create(app As RepPP.Application, type As System.Type) As " + strUserRowsetTreeDefClassName + "\r\n");
            strBuilder.Append("            Dim rowsetTreeRetVal  As RepPP.RowsetTree\r\n");
            strBuilder.Append("            Dim rowsetTreeDef     As RepPP.RowsetTreeDef\r\n");
            strBuilder.Append("            Dim typeOld           As System.Type\r\n");
            strBuilder.Append("            Dim typedInstFact     As RepPP.TypedInstanceFactory\r\n");
            strBuilder.Append("\r\n");
            strBuilder.Append("            rowsetTreeDef   = app.RowsetTreeDefs(\"" + rowsetTreeDef.Name + "\")\r\n");
            strBuilder.Append("            typedInstFact   = CType(app.GetObjFactory(GetType(RepPP.RowsetTree)), RepPP.TypedInstanceFactory)\r\n");
            strBuilder.Append("            typeOld         = typedInstFact.GetTypeAssociateWithRowsetTreeDef(rowsetTreeDef)\r\n");
            strBuilder.Append("            typedInstFact.AddRowsetTreeDef(rowsetTreeDef, type)\r\n");
            strBuilder.Append("            Try \r\n");
            strBuilder.Append("                rowsetTreeRetVal = rowsetTreeDef.RowsetTrees.Add()\r\n");
            strBuilder.Append("            Finally\r\n");
            strBuilder.Append("                typedInstFact.AddRowsetTreeDef(rowsetTreeDef, typeOld)\r\n");
            strBuilder.Append("            End Try\r\n");
            strBuilder.Append("            Return CType(rowsetTreeRetVal, " + strUserRowsetTreeDefClassName + ")\r\n");
            strBuilder.Append("        End Function\r\n");
            if (bGenWS) {
                strBuilder.Append("\r\n");
                strBuilder.Append("        " + m_strRem + " <summary>\r\n");
                strBuilder.Append("        " + m_strRem + String.Format(" CreateWebServicesUtil:  {0}\r\n", m_cmtResManager.GetFuncDesc("TypedRowsetTreeBase", "CreateWebServicesUtil")));
                strBuilder.Append("        " + m_strRem + " </summary>\r\n");
                strBuilder.Append("        " + m_strRem + " <returns>\r\n");
                strBuilder.Append("        " + m_strRem + String.Format(" {0}\r\n", m_cmtResManager.GetFuncReturnDesc("TypedRowsetTreeBase", "CreateWebServicesUtil")));
                strBuilder.Append("        " + m_strRem + " </returns>\r\n");
                strBuilder.Append("        Protected Overrides Function CreateWebServicesUtil() As RepPP.WebServicesUtil\r\n");
                strBuilder.Append("            Dim wsUtil As RepPP.WebServicesUtil\r\n");
                strBuilder.Append("\r\n");
                strBuilder.Append("            wsUtil = MyBase.CreateWebServicesUtil()\r\n");
                strBuilder.Append("            wsUtil.RowsetPropertyPrefix = \"" + strRowsetPropertyPrefix + "\"\r\n");
                strBuilder.Append("            wsUtil.FieldPropertyPrefix  = \"" + strFldPropertyPrefix + "\"\r\n");
                strBuilder.Append("            Return (wsUtil)\r\n");
                strBuilder.Append("        End Function\r\n");
            }
            nodeRoot = rowsetTreeDef.Nodes[0];
            if (nodeRoot != null) {
                strBuilder.Append(GenerateRowsetProperty(nodeRoot, strRowsetClassPrefix, strRowsetPropertyPrefix));
                if (bGenWS) {
                    strBuilder.Append("\r\n");
                    strBuilder.Append(wsGen.GenerateRowsetTreeCode(strRowsetTreeDefClassName, rowsetTreeDef, 8, "", "", strRowsetClassPrefix, bUseWSDataContracts));
                }
            }
            strBuilder.Append("    End Class ' class " + strRowsetTreeDefClassName + "\r\n");
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateUserTypedRowset:            Generate user typed instance for a rowset
        /// </summary>
        /// <param name="rowsetDef">            Rowset definition</param>
        /// <param name="strRowsetClassPrefix"> Prefix for generated rowset class name</param>
        /// <param name="strNameSpace">         The namespace</param>
        /// <param name="strRootNameSpace">     The root namespace of the project</param>
        /// <returns>
        /// The generated class
        /// </returns>
        //
        //******************************************************************
		public override string GenerateUserTypedRowset(RepPP.RowsetDef  rowsetDef, 
                                                       string           strRowsetClassPrefix,
                                                       string           strNameSpace,
                                                       string           strRootNameSpace) {
            StringBuilder   strBuilder;
            bool            bGenNamesapce;
            string          strClassName;
            string          strBaseClassName;
            string          strIndent = "    ";

            bGenNamesapce = (strNameSpace != String.Empty);
            strBuilder    = new StringBuilder(null, 4096);            
            strBuilder.Append(GenerateUserSignature("'"));
            strBuilder.Append("Imports System\r\n");
            strBuilder.Append("\r\n");
            if (bGenNamesapce) {
                strBuilder.Append("Namespace " + strNameSpace + "\r\n");
            }
            strClassName     = DecorateRowsetClassName(strRowsetClassPrefix, GetRowsetName(rowsetDef));
            strBaseClassName = TYPED_INST_NAMESPACE + "." + DecorateRowsetClassName(TYPED_ROWSET_PREFIX + strRowsetClassPrefix, GetRowsetName(rowsetDef));
            strBuilder.Append(strIndent + m_strRem + " <summary>\r\n");
            strBuilder.Append(strIndent + m_strRem + String.Format(" {0} {1}\r\n", m_cmtResManager.GetClassDesc("UserTypedRowset"), rowsetDef.Name));
            strBuilder.Append(strIndent + m_strRem + " </summary>\r\n");
            strBuilder.Append(strIndent + "<RepPP.TypedInstanceClass(\"" + rowsetDef.Name + "\", 200)> _\r\n");
            strBuilder.Append(strIndent + "Public Class " + strClassName + "\r\n");
            strBuilder.Append(strIndent + "    Inherits " + strBaseClassName + "\r\n");
            strBuilder.Append(strIndent + "    " + m_strRem + " <summary>" + "\r\n");
            strBuilder.Append(strIndent + "    " + m_strRem + " Class constructor.\r\n");
            strBuilder.Append(strIndent + "    " + m_strRem + " </summary>\r\n");
            strBuilder.Append(strIndent + "    " + m_strRem + String.Format(" <param name=\"pobj\">    {0}</param>\r\n", m_cmtResManager.GetFuncParamDesc("UserTypedRowset", "Constructor", "pobj")));
            strBuilder.Append(strIndent + "    Public Sub New(pobj As IntPtr)\r\n");
            strBuilder.Append(strIndent + "        MyBase.New(pobj)\r\n");
            strBuilder.Append(strIndent + "    End Sub\r\n");
            strBuilder.Append(strIndent + "End Class ' class " + strClassName + "\r\n");
            if (bGenNamesapce) {
                strBuilder.Append("End Namespace ' namespace " + strNameSpace + "\r\n");
            }
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateTypedRowset:                    Generate typed instance for a rowset
        /// </summary>
        /// <param name="rowsetDef">                Rowset definition</param>
        /// <param name="strRowsetTreeClassPrefix"> Prefix for generated rowset class name</param>
        /// <param name="strRowsetClassPrefix">     Prefix for generated rowset class name</param>
        /// <param name="strRowsetPropertyPrefix">  Prefix for generated rowset property name</param>
        /// <param name="strFldPropertyPrefix">     Prefix for generated field property name</param>
        /// <param name="strNameSpace">             The namespace</param>
        /// <param name="strRootNameSpace">         The root namespace of the project</param>
        /// <param name="strBaseClassName">         Name of the base class</param>
        /// <param name="bGenWS">                   true to generate Web Services</param>
        /// <param name="bUseWSDataContracts">      true to generate Web Service Adapters using DataContract</param>
        /// <returns>
        /// The generated class
        /// </returns>
        //
        //******************************************************************
		public override string GenerateTypedRowset(RepPP.RowsetDef  rowsetDef, 
                                                   string           strRowsetTreeClassPrefix,
                                                   string           strRowsetClassPrefix,
                                                   string           strRowsetPropertyPrefix,
                                                   string           strFldPropertyPrefix,
                                                   string           strNameSpace,
                                                   string           strRootNameSpace,
                                                   string           strBaseClassName,
                                                   bool             bGenWS,
                                                   bool             bUseWSDataContracts) {
            StringBuilder   strBuilder;
            string          strRealNamespace;

            strRealNamespace = (strNameSpace != String.Empty) ? strNameSpace + "." + TYPED_INST_NAMESPACE : TYPED_INST_NAMESPACE;
            strBuilder       = new StringBuilder(null, 4096);            
            strBuilder.Append(GenerateSignature("'"));
            strBuilder.Append("Imports System\r\n");
            if (bGenWS && bUseWSDataContracts) {
                strBuilder.Append("Imports System.Runtime.Serialization\r\n");
                strBuilder.Append("Imports System.Xml.Serialization\r\n");
            }
            strBuilder.Append("\r\n");
            strBuilder.Append("Namespace " + strRealNamespace + "\r\n");
            strBuilder.Append(IntGenerateTypedRowset(null,
                                                     rowsetDef,
                                                     strRowsetTreeClassPrefix,
                                                     strRowsetClassPrefix,
                                                     strRowsetPropertyPrefix,
                                                     strFldPropertyPrefix,
                                                     strNameSpace,
                                                     strBaseClassName,
                                                     1, 
                                                     bGenWS,
                                                     bUseWSDataContracts));
            strBuilder.Append("End Namespace ' namespace " + strRealNamespace + "\r\n");
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateUserTypedRowsetTree:            Generate user typed instance for a rowset tree
        /// </summary>
        /// <param name="rowsetTreeDef">            Rowset tree definition</param>
        /// <param name="strRowsetTreeClassPrefix"> Prefix for generated rowset tree class name</param>
        /// <param name="strNameSpace">             The namespace</param>
        /// <param name="strRootNameSpace">         The root namespace of the project</param>
        /// <returns>
        /// Generated class
        /// </returns>
        //
        //******************************************************************
		public override string GenerateUserTypedRowsetTree(RepPP.RowsetTreeDef  rowsetTreeDef, 
                                                           string               strRowsetTreeClassPrefix,
                                                           string               strNameSpace,
                                                           string               strRootNameSpace) {
            StringBuilder           strBuilder;
            RepPP.RowsetTreeDefNode nodeRoot;
            bool                    bGenNamesapce;
            string                  strRowsetTreeDefClassName;
            string                  strBaseClassName;

            bGenNamesapce = (strNameSpace != String.Empty);
            strBuilder    = new StringBuilder(null, 4096);            
            if (rowsetTreeDef.Nodes.Count != 0) {
                nodeRoot = rowsetTreeDef.Nodes[0];
                strBuilder.Append(GenerateUserSignature("'"));
                strBuilder.Append("Imports System" + "\r\n");
                strBuilder.Append("\r\n");
                if (bGenNamesapce) {
                    strBuilder.Append("Namespace " + strNameSpace + "\r\n");
                }
                strRowsetTreeDefClassName  = DecorateRowsetTreeClassName(strRowsetTreeClassPrefix, GetRowsetTreeName(rowsetTreeDef));
                strBaseClassName  = TYPED_INST_NAMESPACE + "." + DecorateRowsetTreeClassName(TYPED_ROWSET_TREE_PREFIX + strRowsetTreeClassPrefix, GetRowsetTreeName(rowsetTreeDef));
                strBuilder.Append("    " + m_strRem + " <summary>\r\n");
                strBuilder.Append("    " + m_strRem + String.Format(" {0} {1}\r\n", m_cmtResManager.GetClassDesc("UserTypedRowsetTree"), rowsetTreeDef.Name));
                strBuilder.Append("    " + m_strRem + " </summary>\r\n");
                strBuilder.Append("    <RepPP.TypedInstanceClass(\"" + rowsetTreeDef.Name + "\", 200)> _\r\n");
                strBuilder.Append("    Public Class " + strRowsetTreeDefClassName + "\r\n");
                strBuilder.Append("        Inherits " + strBaseClassName + "\r\n");
                strBuilder.Append("\r\n");
                strBuilder.Append("        " + m_strRem + " <summary>\r\n");
                strBuilder.Append("        " + m_strRem + String.Format(" {0}\r\n", m_cmtResManager.GetFuncDesc("UserTypedRowsetTree", "Constructor")));
                strBuilder.Append("        " + m_strRem + " </summary>\r\n");
                strBuilder.Append("        " + m_strRem + String.Format(" <param name=\"pobj\">    {0}</param>\r\n", m_cmtResManager.GetFuncParamDesc("UserTypedRowsetTree", "Constructor", "pobj")));
                strBuilder.Append("        Public Sub New(pobj As IntPtr)\r\n");
                strBuilder.Append("            MyBase.New(pobj)\r\n");
                strBuilder.Append("        End Sub\r\n");
                strBuilder.Append("    End Class ' class " + strRowsetTreeDefClassName + "\r\n");
                if (bGenNamesapce) {
                    strBuilder.Append("End Namespace ' namespace " + strNameSpace + "\r\n");
                }
            }
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateTypedRowsetTree:                    Generate typed instance for a rowset tree
        /// </summary>
        /// <param name="rowsetTreeDef">                Rowset tree definition</param>
        /// <param name="strRowsetTreeClassPrefix">     Prefix for generated rowset tree class name</param>
        /// <param name="strRowsetClassPrefix">         Prefix for generated rowset class name</param>
        /// <param name="strRowsetPropertyPrefix">      Prefix for generated rowset property name</param>
        /// <param name="strFldPropertyPrefix">         Prefix for generated field property name</param>
        /// <param name="strNameSpace">                 The namespace</param>
        /// <param name="strRootNameSpace">             The root namespace of the project</param>
        /// <param name="strRowsetTreeBaseClassName">   Name of the base class for the rowset tree</param>
        /// <param name="strRowsetBaseClassName">       Name of the base class for the rowset</param>
        /// <param name="bGenWS">                       true to generate Web Services</param>
        /// <param name="bUseWSDataContracts">          true to generate Web Service Adapters using DataContract</param>
        /// <returns>
        /// Generated class
        /// </returns>
        //
        //******************************************************************
		public override string GenerateTypedRowsetTree(RepPP.RowsetTreeDef  rowsetTreeDef, 
                                                       string               strRowsetTreeClassPrefix,
                                                       string               strRowsetClassPrefix,
                                                       string               strRowsetPropertyPrefix,
                                                       string               strFldPropertyPrefix,
                                                       string               strNameSpace,
                                                       string               strRootNameSpace,
                                                       string               strRowsetTreeBaseClassName,
                                                       string               strRowsetBaseClassName,
                                                       bool                 bGenWS,
                                                       bool                 bUseWSDataContracts) {
            StringBuilder           strBuilder;
            RepPP.RowsetTreeDefNode nodeRoot;
            string                  strRealNamespace;

            strRealNamespace = (strNameSpace != String.Empty) ? strNameSpace + "." + TYPED_INST_NAMESPACE : TYPED_INST_NAMESPACE;
            strBuilder       = new StringBuilder(null, 4096);            
            if (rowsetTreeDef.Nodes.Count != 0) {
                nodeRoot = rowsetTreeDef.Nodes[0];
                strBuilder.Append(GenerateSignature("'"));
                strBuilder.Append("Imports System" + "\r\n");
                if (bGenWS && bUseWSDataContracts) {
                    strBuilder.Append("Imports System.Runtime.Serialization\r\n");
                    strBuilder.Append("Imports System.Xml.Serialization\r\n");
                }
                strBuilder.Append("\r\n");
                strBuilder.Append("Namespace " + strRealNamespace + "\r\n");
                strBuilder.Append(IntGenerateTypedRowsetTree(rowsetTreeDef, 
                                                             strRowsetTreeClassPrefix,
                                                             strRowsetClassPrefix,
                                                             strRowsetPropertyPrefix,
                                                             strFldPropertyPrefix,
                                                             strRowsetTreeBaseClassName,
                                                             strRowsetBaseClassName,
                                                             bGenWS,
                                                             bUseWSDataContracts));
                strBuilder.Append("End Namespace ' namespace " + strRealNamespace + "\r\n");
            }
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
		/// <summary>
		/// RegenerateTypedInstance:            Regenerates all rowset trees and rowsets which need to be regenerated
		/// </summary>
		/// <param name="app">                  Rep++ application object</param>
        /// <param name="log">                  Log object for name clash. Null for no name clash check</param>
		/// <param name="projFolder">           Interface with the project folder</param>
		/// <param name="bFullSync">            true to fully synchronize type. false to only sync the existing one</param>
        /// <param name="bUseWSDataContracts">  true to use DataContract</param>
        /// 
        //
        //******************************************************************
		public override void RegenerateTypedInstance(RepPP.Application app, ILog log, IProjectFolder projFolder, bool bFullSync, bool bUseWSDataContracts) {
            RegenerateTypedInstance(app, log, projFolder, ".vb", bFullSync, bUseWSDataContracts);
		}

        //******************************************************************
        //
		/// <summary>
		/// RegenerateTypedInstance:            Regenerates all rowset trees and rowsets which need to be regenerated
		/// </summary>
        /// <param name="log">                  Log object for name clash. Null for no name clash check</param>
		/// <param name="projFolder">           Interface with the project folder</param>
		/// <param name="bFullSync">            true to fully synchronize type. false to only sync the existing one</param>
        /// <param name="bUseWSDataContracts">  true to use DataContract</param>
        //
        //******************************************************************
		public override void RegenerateTypedInstance(ILog log, IProjectFolder projFolder, bool bFullSync, bool bUseWSDataContracts) {
            RegenerateTypedInstance(null, log, projFolder, ".vb", bFullSync, bUseWSDataContracts);
		}
	} // class TypedInstanceGenForVB
} // namespace
