﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Web.UI.Design;

namespace RepPP.Toolkit.Web {

    /// <summary>
    /// Web Control Utility helper
    /// </summary>
	public class WebControlUtilHelper : ControlUtilHelperBase {

        //********************************************************************************
        //
        /// <summary>
        /// GetSite:            Get the component site of the control parent
        /// </summary>
        /// <returns>
        /// Component site of control parent if exist otherwise null
        /// </returns>
        //
        //********************************************************************************
        public override System.ComponentModel.ISite GetSite() {
            ISite                   siteRetVal = null;
		    System.Web.UI.Control   webCtl;
            Object                  parent;

            if (m_ctrlUtil != null && m_ctrlUtil.Parent != null) {
                parent = m_ctrlUtil.Parent;
                if (parent is System.Web.UI.Control) {
		            webCtl      = (System.Web.UI.Control)parent;
		            siteRetVal  = webCtl.Site;
		            if (siteRetVal == null && webCtl.Page != null) {
		                siteRetVal = webCtl.Page.Site;
		            }
                } else if (parent is System.ComponentModel.IComponent) {
                    siteRetVal = ((System.ComponentModel.IComponent)parent).Site;
		        } 
            }
		    return(siteRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsDesignMode:   Indicate if the control is in design mode
        /// </summary>
        /// <returns>
        /// True if control is in design mode
        /// </returns>
        //
        //********************************************************************************
        public override bool IsDesignMode() {
            Object  parent;
            bool    bRetVal = false;

            if (m_ctrlUtil != null) {
                parent = m_ctrlUtil.Parent;
                if (parent != null && parent is System.Web.UI.Control) {
                    bRetVal = (System.Web.HttpContext.Current == null);
                } else {
                    bRetVal = (System.Diagnostics.Process.GetCurrentProcess().ProcessName == "devenv");
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// UpdateControl:   Update the control
        /// </summary>
        //
        //********************************************************************************
        public override void UpdateControl() {
            Object parent;

            if (m_ctrlUtil != null && m_ctrlUtil.Parent != null) {
                parent = m_ctrlUtil.Parent;
                if (parent is ControlUtil.IRepControl) {
                    ((ControlUtil.IRepControl)parent).RepPPReInit();
                }
                if (parent is System.Web.UI.Control) {
                    UpdateDesignTimeHtml(m_ctrlUtil.Site, parent);
                }
            }
        }
        
        //********************************************************************************
        //
        /// <summary>
        /// UpdateDesignTimeHtml:           Updates the design-time HTML
        /// </summary>
        //
        //********************************************************************************
        protected virtual void UpdateDesignTimeHtml(ISite site, Object parent) {
            IDesignerHost   designerHost;
            ControlDesigner ctlDesigner;
            
            try {
                if (site != null) {
                    designerHost = site.GetService(typeof(IDesignerHost)) as IDesignerHost;
                    if (designerHost != null) {
                        ctlDesigner = designerHost.GetDesigner(parent as System.Web.UI.Control) as ControlDesigner;
                        if (ctlDesigner != null) {
                            ctlDesigner.UpdateDesignTimeHtml();
                        }
                    }
                }
            } catch {
            }
        }

    } // End class WebControlUtilHelper
} //End namespace
