using System;
using System.Reflection;
using System.Collections;
using System.Data;
using System.IO;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.ComponentModel.Design;
using RepPP.Toolkit;
using RepPP;

namespace RepPP.Toolkit.Web {
    //********************************************************************************
    //
	/// <summary>
	/// ASP.Net DataList bound with Rep++ Meta-data
	/// </summary>
	//
    //********************************************************************************
	public class DataListRowset : System.Web.UI.WebControls.DataList {
	    /// <summary>Application control utility class</summary>
	    protected ControlUtil                   m_controlUtil;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //********************************************************************************
		public DataListRowset() {
		    m_controlUtil  = new ControlUtil(this, new WebControlUtilHelper(), true);
		}

        //********************************************************************************
        //
		/// <summary>
		/// Dispose:                    Dispose the object
		/// </summary>
		//
        //********************************************************************************
        public override void Dispose() {
            if (m_controlUtil != null) {
                m_controlUtil.Dispose();
                m_controlUtil = null;
            }
            base.Dispose();
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_controlUtil != null) {
                m_controlUtil.TriggerComponentChange();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Application Application {
		    get {
		        return(m_controlUtil.Application);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// DataKeyField:           Field representing the key. Not supported. The
        ///                         key fields are match to the group one.
        /// </summary>
        //
        //********************************************************************************
        [Browsable(false)]
        public override string DataKeyField {
            get {
                return("");
            }
            set {
                // Do nothing...
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FillKeyList:                Fill the key list with the one from the attached rowset
        /// </summary>
        /// <param name="dataSource">   Source attached to the grid</param>
        //
        //********************************************************************************
        protected virtual void FillKeyList(DataSourceRowset dataSource) {
            DataRowRowset   row;
            ArrayList       arrKeys;
            FieldDef[]      arrPKFld;
            string[]        arrPKVal;
            Rowset          rowset;
            int             iLineIndex;
            int             iCount;
            
            arrKeys = this.DataKeysArray;
            arrKeys.Clear();
            rowset  = dataSource.Rowset;
            if (rowset != null) {
                iCount = 0;
                foreach (FieldDef fldPK in dataSource.RowsetDef.FieldDefs) {
                    if (fldPK.IsPrimaryKey) {
                        iCount++;
                    }
                }
                if (iCount != 0) {
                    arrPKFld = new FieldDef[iCount];
                    arrPKVal = new string[iCount];
                    iCount   = 0;
                    foreach (FieldDef fldPK in dataSource.RowsetDef.FieldDefs) {
                        if (fldPK.IsPrimaryKey) {
                            arrPKFld[iCount++] = fldPK;
                        }
                    }
                    foreach(DataListItem item in Items) {
                        row = dataSource[item.ItemIndex] as DataRowRowset;
                        if (row != null) {
                            iLineIndex = rowset.LineFromBookmark(row.Bookmark);
                            iCount     = 0;
                            foreach (RepPP.FieldDef fldPK in arrPKFld) {
                                arrPKVal[iCount++] = rowset.GetFieldValue(fldPK, iLineIndex, true);
                            }
                            arrKeys.Add(String.Join(",", arrPKVal));
                        }
                    }
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// CreateControlHierarchy:         Creates the grid control hierarchy
        /// </summary>
        /// <param name="bUseDataSource">   true to use the data source</param>
        //
        //********************************************************************************
        protected override void CreateControlHierarchy(bool bUseDataSource) {
            DataSourceRowset dataSource;
            
            base.CreateControlHierarchy(bUseDataSource);
            if (bUseDataSource) {
                dataSource = DataSource as DataSourceRowset;
                if (dataSource != null) {
                    FillKeyList(dataSource);
                }
            }
        }
	} // Class DataListRowset
	
} // Namespace
