//*********************************************************
//
// DynamicFormTable     Implement a table controls which is filled with a dynamic fields
//
// Author:		        Jacques Fournier
//
// (c) 2005 by Consyst SQL Inc.
//
//
//*********************************************************
using System;
using System.Collections;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.ComponentModel.Design;
using RepPP.Toolkit;
using RepPP;

namespace RepPP.Toolkit.Web {
    //*********************************************************
    //
    /// <summary>
    /// DynamicFormTable     Implement a table controls which is filled with a dynamic fields 
    /// </summary>
    //
    //*********************************************************
    [Designer(typeof(RepPP.Toolkit.Web.Design.DynamicFormTableDesign))]
    [ToolboxData("<{0}:DynamicFormTable runat=server></{0}:DynamicFormTable>")]
	public class DynamicFormTable : Table, RowsetControlUtil.IRowsetControl, INamingContainer {
        /// <summary>Generation options</summary>
        public enum OptionE {
            /// <summary>No option</summary>
            OPT_NONE                            = 0,
            /// <summary>Only add field attached to the main table</summary>
            OPT_ONLY_MAIN_TBL_FIELD             = 1,
            /// <summary>Field are always selectable</summary>
            OPT_ALWAYS_SELECTABLE               = 2,
            /// <summary>Replace radio by a combo box</summary>
            OPT_NO_RADIO                        = 4,
            /// <summary>Replace check by listbox</summary>
            OPT_NO_CHECK                        = 8,
            /// <summary>Add a combo box with operators</summary>
            OPT_ADD_OPERATOR_COMBO              = 16,
            /// <summary>Add an additional invisible field for BETWEEN clause</summary>
            OPT_ADD_BETWEEN_FIELD               = 32,
        };
	    /// <summary>Determine the type of the cell being added</summary>
	    protected enum CellTypeE {
	        /// <summary>The cell contains a label control</summary>
	        Label   =   0,
	        /// <summary>The cell contains a combo (dropdown)</summary>
	        Combo   =   1,
	        /// <summary>The cell contains a value control</summary>
	        Value   =   2,
	        /// <summary>The cell contains the second BETWEEN value control</summary>
	        Between =   3,
	        /// <summary>The cell contains something else</summary>
	        Other   =   4
	    };
	    /// <summary>Embedded utility class for control attached to rowset</summary>
	    protected RowsetControlUtil             m_rowsetControlUtil;
	    private OptionE                         m_eOption;
	    private DropDownList[]                  m_arrComboBox;
        private FldControlUtil.IFldControl[]    m_arrLabel;
        private FldControlUtil.IFldControl[]    m_arrFieldCtl;
        private FldControlUtil.IFldControl[]    m_arrBetweenFieldCtl;

        //********************************************************************************
        //
        /// <summary>
        /// ctor
        /// </summary>
        //
        //********************************************************************************
	    public DynamicFormTable() {
		    m_rowsetControlUtil     = new RowsetControlUtil(this, new WebControlUtilHelper(), true);
	        m_arrComboBox           = null;
            m_arrLabel              = null;
            m_arrFieldCtl           = null;
            m_arrBetweenFieldCtl    = null;
		}

        //********************************************************************************
        //
		/// <summary>
		/// Dispose:            Dispose the control
		/// </summary>
		//
        //********************************************************************************
        public override void Dispose() {
            if (m_rowsetControlUtil != null) {
                m_rowsetControlUtil.Dispose();
            }
            base.Dispose();
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReInit:        Called when Rep++ is reinitialized
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReInit() {
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReset:         Called when Rep++ is reset in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReset() {
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_rowsetControlUtil != null) {
                m_rowsetControlUtil.TriggerComponentChange();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Application Application {
		    get {
		        return(m_rowsetControlUtil.Application);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// RowsetDef:              Attached group
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
		[Description("RowsetDef use to fill the table")]
		[DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        public virtual RowsetDef RowsetDef {
            get {
                return(m_rowsetControlUtil.RowsetDef);
            }
            set {
                if (m_rowsetControlUtil.RowsetDef != value) {
                    m_rowsetControlUtil.RowsetDef = value;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// RowsetName:                  Name of the group use to fill the source
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Editor("RepPP.Toolkit.Designer.RowsetControlDesigner+RowsetNameEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Name of the group attached to the source")]
		public virtual string RowsetName {
		    get {
		        return(m_rowsetControlUtil.RowsetName);
		    }
		    set {
		        m_rowsetControlUtil.RowsetName = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Option:         Option to use for field generations
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Behavior")]
        [Description("Option to use for field generations")]
		public virtual OptionE Option {
		    get {
		        return(m_eOption);
		    }
		    set {
		        m_eOption = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// DropDownListArray:      Array of generated dropdown list
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public virtual DropDownList[] DropDownListArray {
		    get {
		        return(m_arrComboBox);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// LabelControlArray:      Array of generated label controls
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public virtual FldControlUtil.IFldControl[] LabelControlArray {
		    get {
		        return(m_arrLabel);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// FieldControlArray:      Array of generated field controls
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public virtual FldControlUtil.IFldControl[] FieldControlArray {
		    get {
		        return(m_arrFieldCtl);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// BetweenFieldControlArray:   Array of generated between field controls
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public virtual FldControlUtil.IFldControl[] BetweenFieldControlArray {
            get {
                return(m_arrBetweenFieldCtl);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Rows:                   Overridden so that the rows are not serialized as
        ///                         inner properties
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public override TableRowCollection Rows {
            get {
                return base.Rows;
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GetControlType:                     Gets the type of control to create
        /// </summary>
	    /// <param name="fldDef">               Field definition for which we create the control</param>
	    /// <param name="eOption">              Generation options</param>
	    /// <returns>
	    /// Control type
	    /// </returns>
        //
        //*********************************************************	    
	    protected virtual CtlType GetControlType(FieldDef fldDef, OptionE eOption) {
	        CtlType     eRetVal = fldDef.ControlType;
	        
            switch(eRetVal) {
            case RepPP.CtlType.sdCtlCheck:
                if ((eOption & OptionE.OPT_NO_CHECK) != 0) {
                    eRetVal = RepPP.CtlType.sdCtlCombo;
                }
                break;
            case RepPP.CtlType.sdCtlRadio:
                if ((eOption & OptionE.OPT_NO_RADIO) != 0) {
                    eRetVal = RepPP.CtlType.sdCtlList;
                }
                break;
            }
            return(eRetVal);
	    }

        //*********************************************************	    
        //
        /// <summary>
        /// CreateControl:              Creates a control attached to the specified field definition
        /// </summary>
        /// <param name="eCtlType">     Type of control to create</param>
	    /// <param name="fldDef">       Field definition for which we create the control</param>
	    /// <param name="strCtlName">   Name of the control (null for using the field name)</param>
	    /// <param name="cell">         Cell which will contain the field control</param>
	    /// <param name="eType">        Cell type</param>
	    /// <returns>
	    /// New control
	    /// </returns>
        //
        //*********************************************************	    
	    protected virtual FldControlUtil.IFldControl CreateControl(CtlType eCtlType, FieldDef fldDef, string strCtlName, TableCell cell, CellTypeE eType) {
	        FldControlUtil.IFldControl  fldCtlRetVal = null;
	        WebControl                  ctl;
	        FldCheckBoxList             ctlCheckListBox;
	        FldDropDownList             ctlDropDownList;
	        FldListBox                  ctlListBox;
	        FldRadioButtonList          ctlRadio;
	        FldLabel                    ctlLabel;
	        FldTextBox                  ctlTextBox;
	        string                      strPrefix;
	        
	        switch(eCtlType) {
	        case CtlType.sdCtlCheck:
	            ctlCheckListBox                     = new FldCheckBoxList();
	            fldCtlRetVal                        = ctlCheckListBox;
	            break;
	        case CtlType.sdCtlCombo:
	            ctlDropDownList                     = new FldDropDownList();
	            fldCtlRetVal                        = ctlDropDownList;
	            break;
	        case CtlType.sdCtlList:
	            ctlListBox                          = new FldListBox();
	            fldCtlRetVal                        = ctlListBox;
	            break;
	        case CtlType.sdCtlRadio:
	            ctlRadio                            = new FldRadioButtonList();
	            fldCtlRetVal                        = ctlRadio;
	            break;
	        case CtlType.sdCtlStatic:
	            ctlLabel                            = new FldLabel();
	            fldCtlRetVal                        = ctlLabel;
	            break;
	        default:
	            ctlTextBox                          = new FldTextBox();
	            fldCtlRetVal                        = ctlTextBox;
	            break;
	        }
	        fldCtlRetVal.FieldFullName.RowsetName   = fldDef.RowsetDef.Name;
	        fldCtlRetVal.FieldFullName.FieldName    = fldDef.Name;
	        strPrefix                               = (eCtlType == CtlType.sdCtlStatic) ? "lbl" : "fld";
	        ctl                                     = fldCtlRetVal as WebControl;
	        ctl.ID                                  = (strCtlName == null) ? strPrefix + fldDef.Name : strCtlName;
	        cell.Controls.Add((Control)fldCtlRetVal);
	        return(fldCtlRetVal);
	    }

        //*********************************************************	    
        //
        /// <summary>
        /// SetRowAttribute:            Set the row attributes
        /// </summary>
        /// <param name="fldDef">       Field definition attached to the row</param>
	    /// <param name="row">          Row</param>
        //
        //*********************************************************	    
	    protected virtual void SetRowAttribute(FieldDef fldDef, TableRow row) {
	    }

        //*********************************************************	    
        //
        /// <summary>
        /// CreateRow:                  Create a new row
        /// </summary>
        /// <param name="fldDef">       Field definition attached to the row</param>
        //
        //*********************************************************	    
	    protected virtual TableRow CreateRow(FieldDef fldDef) {
	        return(new TableRow());
	    }

        //*********************************************************	    
        //
        /// <summary>
        /// SetCellAttribute:           Set the cell attributes
        /// </summary>
        /// <param name="fldDef">       Field definition attached to the cell</param>
	    /// <param name="cell">         Cell</param>
	    /// <param name="eType">        Type of cell</param>
        //
        //*********************************************************	    
	    protected virtual void SetCellAttribute(FieldDef fldDef, TableCell cell, CellTypeE eType) {
	    }

        //*********************************************************	    
        //
        /// <summary>
        /// CreateCell:                 Creates a new cell
        /// </summary>
        /// <param name="fldDef">       Field definition attached to the cell</param>
	    /// <param name="eType">        Type of cell</param>
        //
        //*********************************************************	    
	    protected virtual TableCell CreateCell(FieldDef fldDef, CellTypeE eType) {
	        return(new TableCell());
	    }

        //*********************************************************	    
        //
        /// <summary>
        /// CreateOpCombo:                      Create the dropdownlist which contains the operation
        /// </summary>
        /// <param name="fldDef">               Field definition related to the combo</param>
        /// <param name="cell">                 Father cell</param>
        /// <returns>
        /// New drop down list control
        /// </returns>
        //
        //*********************************************************	    
	    protected virtual DropDownList CreateOpCombo(FieldDef fldDef, TableCell cell) {
	        DropDownList    ctlRetVal;
	        
	        ctlRetVal       = new DropDownList();
	        ctlRetVal.ID    = "com" + fldDef.Name;
	        cell.Controls.Add(ctlRetVal);
	        return(ctlRetVal);
	    }

        //*********************************************************	    
        //
        /// <summary>
        /// CreateRowsetFields:              Create field controls in the specified container.
        /// </summary>
        /// <param name="rowsetDef">            Rowset definition to use as template</param>
        /// <param name="eOption">              Generation options</param>
        /// <param name="arrComboBox">          Array of combo box generated if option specified</param>
        /// <param name="arrLabel">             Array of created static fields</param>
        /// <param name="arrFieldCtl">          Array of created fields</param>
        /// <param name="arrBetweenFieldCtl">   Array of invisible fields created for the BEWTEEN clause</param>
        //
        //*********************************************************	    
        protected virtual void CreateRowsetDefFields(RepPP.RowsetDef                    rowsetDef,
                                                     OptionE                            eOption,
                                                     out DropDownList[]                 arrComboBox,
                                                     out FldControlUtil.IFldControl[]   arrLabel,
                                                     out FldControlUtil.IFldControl[]   arrFieldCtl,
                                                     out FldControlUtil.IFldControl[]   arrBetweenFieldCtl) {
            ArrayList                   listCombo;
            ArrayList                   listLabel;
            ArrayList                   listField;
            ArrayList                   listBetweenField;
            CtlType                     eCtlType;
            bool                        bBetweenField;
            TableRow                    row = null;
            TableRow                    rowInner;
            TableCell                   cellLabel;
            TableCell                   cellCombo;
            TableCell                   cellValue;
            TableCell                   cellBetween;
            TableCell                   cellInner;
            Table                       tableInner;
            FieldDef                    fldDefLastField = null;
            FldControlUtil.IFldControl  fldCtlLabel;
            FldControlUtil.IFldControl  fldCtlValue;
            FldControlUtil.IFldControl  fldCtlBetween;
            DropDownList                ctlCombo;
            
            listCombo           = new ArrayList(rowsetDef.FieldDefs.Count);
            listLabel           = new ArrayList(rowsetDef.FieldDefs.Count);
            listField           = new ArrayList(rowsetDef.FieldDefs.Count);
            listBetweenField    = new ArrayList(rowsetDef.FieldDefs.Count);
            rowInner   = new TableRow();
            cellInner  = new TableCell();
            tableInner = new Table();
            rowInner.Cells.Add(cellInner);
            cellInner.Controls.Add(tableInner);
            Rows.Add(rowInner);
            foreach (RepPP.FieldDef fldDef in rowsetDef.FieldDefs) {
                bBetweenField = (fldDef.Name[0] == '_');
                if (fldDef.IsVisible || ((eOption & OptionE.OPT_ADD_BETWEEN_FIELD) != 0 && bBetweenField)) {
                    if (fldDef.IsDatabaseLinked || (eOption & OptionE.OPT_ONLY_MAIN_TBL_FIELD) == 0) {
                        eCtlType = GetControlType(fldDef, eOption);
                        if (!bBetweenField) {
                            row = CreateRow(fldDef);
                            tableInner.Rows.Add(row);
                            cellLabel      = CreateCell(fldDef, CellTypeE.Label);
                            cellLabel.Wrap = false;
                            row.Cells.Add(cellLabel);
                            SetCellAttribute(fldDef, cellLabel, CellTypeE.Label);
                            fldCtlLabel = CreateControl(CtlType.sdCtlStatic, fldDef, null, cellLabel, CellTypeE.Label);
			                if ((eOption & OptionE.OPT_ADD_OPERATOR_COMBO) != 0) {
                                cellCombo   = CreateCell(fldDef, CellTypeE.Combo);
                                cellCombo.Wrap = false;
			                    row.Cells.Add(cellCombo);
                                SetCellAttribute(fldDef, cellCombo, CellTypeE.Combo);
			                    ctlCombo    = CreateOpCombo(fldDef, cellCombo);
			                    listCombo.Add(ctlCombo);
			                }
                            cellValue   = CreateCell(fldDef, CellTypeE.Value);
                            cellValue.Wrap = false;
                            row.Cells.Add(cellValue);
                            SetCellAttribute(fldDef, cellValue, CellTypeE.Value);
                            fldCtlValue = CreateControl(eCtlType, fldDef, null, cellValue, CellTypeE.Value);
                            listLabel.Add(fldCtlLabel);
                            listLabel.Add(fldCtlValue);
                            fldDefLastField = fldDef;
                        } else {
                            if (row == null) {
                                throw new ArgumentException("The between row must be after the normal field");
                            }
                            cellBetween      = CreateCell(fldDef, CellTypeE.Between);
                            cellBetween.Wrap = false;
                            row.Cells.Add(cellBetween);
                            SetCellAttribute(fldDef, cellBetween, CellTypeE.Between);
                            fldCtlBetween   = CreateControl(eCtlType,
                                                            fldDef,
                                                            (fldDefLastField == null) ? null : "bet" + fldDefLastField.Name,
                                                            cellBetween,
                                                            CellTypeE.Between);
                            listBetweenField.Add(fldCtlBetween);
                        }
			        }
                }
            }
            arrComboBox         = (DropDownList[])listCombo.ToArray(typeof(DropDownList));
            arrLabel            = (FldControlUtil.IFldControl[])listLabel.ToArray(typeof(FldControlUtil.IFldControl));
            arrFieldCtl         = (FldControlUtil.IFldControl[])listField.ToArray(typeof(FldControlUtil.IFldControl));
            arrBetweenFieldCtl  = (FldControlUtil.IFldControl[])listBetweenField.ToArray(typeof(FldControlUtil.IFldControl));
        }

        //*********************************************************	    
        //
        /// <summary>
        /// SourceRowsetDef:            RowsetDef use to define the form
        /// </summary>
        //
        //*********************************************************	    
        protected virtual RowsetDef SourceRowsetDef {
            get {
                return(RowsetDef);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// CreateChildControls:        Fill the table
        /// </summary>
        //
        //*********************************************************	    
        protected override void CreateChildControls() {
            RowsetDef   rowsetDef;
            
            rowsetDef = SourceRowsetDef;
            if (rowsetDef != null) {
                CreateRowsetDefFields(rowsetDef,
                                      Option,
                                      out m_arrComboBox,
                                      out m_arrLabel,
                                      out m_arrFieldCtl,
                                      out m_arrBetweenFieldCtl);
            }
            base.CreateChildControls();
        }

        //********************************************************************************
        //
        /// <summary>
        /// Render :                    Overridden to make sure that the child controls are
        ///                             created before renderring [Mainly for Design Time]
        ///                             
        /// </summary>
        /// <param name="writer">       The html text writer</param>
        //
        //********************************************************************************
        protected override void Render(HtmlTextWriter writer) {
            EnsureChildControls();
            base.Render(writer);
        }

        //********************************************************************************
        //
        /// <summary>
        /// EnsureChildControls :       Overridden to ensure proper refresh and display
        ///                             in design mode
        /// </summary>
        //
        //********************************************************************************
        protected override void EnsureChildControls() {
            base.EnsureChildControls ();
            if (Controls.Count == 0) {
                CreateChildControls();
            }
        }

	} // class DynamicFormTable
} // Namespace
