using System;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.ComponentModel.Design;
using RepPP.Toolkit;
using RepPP;

namespace RepPP.Toolkit.Web {
    //********************************************************************************
    //
    /// <summary>
    /// Implements a CheckBox control which is attached to the Rep++ field meta-data
    /// </summary>
    /// <remarks>
    /// For simple lists, 
    ///     the control is checked when the field's value is the same as the value of the 2nd choice.
    ///     the control is unchecked when the field's value is empty or equals to the value of the first choice.
    /// For mutliple numeric lists,
    ///     the control is checked when the field's value is "1" (i.e. the first (and only) choice is selected)
    ///     the control is unchecked when the field's value is empty or "0"  
    /// For mutliple alphanumeric lists,
    ///     the control is checked when the field's value is the same as the first (and only) choice.
    ///     the control is unchecked when the field's value is empty  
    /// </remarks>
    //
    //********************************************************************************
	public class FldCheckBox : System.Web.UI.WebControls.CheckBox, FldControlUtil.IFldControl {
	    /// <summary>Field control utility class</summary>
	    protected FldControlUtil        m_fldControlUtil;
	    /// <summary>User-Defined Checkbox Text</summary>
	    protected string                m_strForcedText = String.Empty;

        //********************************************************************************
        //
		/// <summary>
		/// ctor:               Class constructor
		/// </summary>
		//
        //********************************************************************************
		public FldCheckBox() {
		    m_fldControlUtil    = new FldControlUtil(this, new WebControlUtilHelper(), true);
		}

        //********************************************************************************
        //
		/// <summary>
		/// Dispose:            Dispose the control
		/// </summary>
		//
        //********************************************************************************
        public override void Dispose() {
            if (m_fldControlUtil != null) {
                m_fldControlUtil.Dispose();
            }
            base.Dispose();
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReInit:        Called when Rep++ is reinitialized in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReInit() {
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReset:         Called when Rep++ is reset in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReset() {
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_fldControlUtil != null) {
                m_fldControlUtil.TriggerComponentChange();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FieldFullName:              Full name of the field attached to the CheckBox
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [PersistenceMode(PersistenceMode.Attribute)]
        [Editor("RepPP.Toolkit.Designer.FldControlDesigner+FldControlEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Full name of the field attached to the CheckBox")]
		public FldFullName FieldFullName {
		    get {
		        return(m_fldControlUtil.FieldFullName);
		    }
		    set {
		        m_fldControlUtil.FieldFullName = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Application Application {
		    get {
		        return(m_fldControlUtil.Application);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// FieldDef:              FieldDef attached to the CheckBox
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public FieldDef FieldDef {
		    get {
		        return(m_fldControlUtil.FieldDef);
		    }
		    set {
		        m_fldControlUtil.FieldDef = value;
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// Field:                  Rep++ Field attached to the control
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Field Field {
		    get {
		        return(m_fldControlUtil.Field);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// AttachedLine:             Attached rowset line
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        [Bindable(false)]
        [DefaultValue(-1)]
        [Description("Attached group instance line index")]
		public virtual int AttachedLine {
            get {
                int    iRetVal;
                object  obj;
                
                obj = ViewState["AttachedLine"];
                if (obj != null) {
                    iRetVal = (int)obj;
                } else {
                    iRetVal = -1;
                }
                return(iRetVal);
            }
            set {
                ViewState["AttachedLine"] = value;
            }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Reverse:               Reverses the way the 'Checked' property is set 
        /// </summary>
        //
        //********************************************************************************
        [Browsable(true)]
        [DefaultValue(false)]
        [Description("Reverses the way the 'Checked' property is set")]
        public virtual bool Reverse {
            get {
                object obj;
                bool   bRetVal = false;

                obj = this.ViewState["Reverse"];
                if (obj != null) {
                    bRetVal = (bool)obj;
                }
                return(bRetVal);
            }
            set {
                this.ViewState["Reverse"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Text:               Text of the CheckBox
        /// </summary>
        //
        //********************************************************************************
        public override string Text {
            get {
                string          strRetVal;
                FldFullName     fldFullName;
                FieldDef        fldDef;
                bool            bDesignMode;

                if (!m_strForcedText.Equals(String.Empty)) {
                    strRetVal = m_strForcedText;
                } else {
                    bDesignMode = m_fldControlUtil.IsDesignMode;
                    fldDef      = m_fldControlUtil.FieldDef;
                    if (fldDef == null) {
                        fldFullName     = m_fldControlUtil.FieldFullName;
                        strRetVal       = fldFullName.RowsetName + "." + fldFullName.FieldName;
                    } else {
                        strRetVal = fldDef.Prompt;
                        if (strRetVal == String.Empty && bDesignMode) {
                            strRetVal = "[" + ID + "]";
                        }
                    }
                }
                return(strRetVal);
            }
            set {
                if (value != null) {
                    m_strForcedText = value; 
                    base.Text = value;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Checked:        Gets or sets a value indicating whether the control is checked
        /// </summary>
        //
        //********************************************************************************        
        public override bool Checked {
            get {
                FieldDef    fldDef;
                bool        bRetVal = false;
                string      strValue;

                fldDef = m_fldControlUtil.FieldDef;
                if (IsFieldValid(fldDef)) {
                    strValue = GetFieldValue(fldDef);
                    switch(fldDef.Type) {
                        case FieldType.sdFieldNumeric:
                        case FieldType.sdFieldBoolean:
                            bRetVal = (strValue.Equals("1"));
                            break;
                        case FieldType.sdFieldSimpleAlphaChoice:
                        case FieldType.sdFieldSimpleNumChoice:
                            bRetVal = (strValue == fldDef.ChoiceList.GetValue(1));
                            break;
                        case FieldType.sdFieldMultiAlphaChoice:
                            bRetVal = (strValue != String.Empty);
                            break;
                        case FieldType.sdFieldMultiNumChoice:
                            bRetVal = ((strValue != String.Empty) && (strValue != "0"));
                            break;
                    }
                }
                if (Reverse) {
                    bRetVal = !bRetVal;
                }
                base.Checked = bRetVal;
                return(bRetVal);
            }
            set {
                FieldDef   fldDef;
                bool    bChecked;

                bChecked     = Reverse ? !value : value;
                if (Checked != bChecked) {
                    base.Checked = bChecked;
                    fldDef       = m_fldControlUtil.FieldDef;
                    if (IsFieldValid(fldDef)) {
                        switch(fldDef.Type) {
                            case FieldType.sdFieldNumeric:
                            case FieldType.sdFieldBoolean:
                                SetFieldValue(fldDef, (bChecked ? "1" : "0"));
                                break;
                            case FieldType.sdFieldSimpleAlphaChoice:
                            case FieldType.sdFieldSimpleNumChoice:
                                if (bChecked) {
                                    SetFieldValue(fldDef, fldDef.ChoiceList.GetValue(1));
                                } else {
                                    SetFieldValue(fldDef, fldDef.ChoiceList.GetValue(0));
                                }
                                break;
                            case FieldType.sdFieldMultiAlphaChoice:
                                if (bChecked) {
                                    SetFieldValue(fldDef, fldDef.ChoiceList.GetValue(0));
                                } else {
                                    SetFieldValue(fldDef, String.Empty);
                                }
                                break;
                            case FieldType.sdFieldMultiNumChoice:
                                if (bChecked) {
                                    SetFieldValue(fldDef, fldDef.ChoiceList.GetValue(0));
                                } else {
                                    SetFieldValue(fldDef, "0");
                                }
                                break;
                        }
                    }
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Enabled:       Determines if the control is enabled
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override bool Enabled {
            get {
                bool        bRetVal;
                FieldDef    fldDef;
                
                fldDef  = m_fldControlUtil.FieldDef;
                bRetVal = (fldDef == null) ? true : fldDef.IsSelectable;
                return(bRetVal);
            }
            set {
                throw new System.NotSupportedException();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Visible:        Determines if the control is visible
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override bool Visible {
            get {
                bool        bRetVal;
                FieldDef    fldDef;
                
                fldDef = m_fldControlUtil.FieldDef;
                bRetVal = (fldDef == null || m_fldControlUtil.IsDesignMode) ? true : fldDef.IsVisible;
                return(bRetVal);
            }
            set {
                throw new System.NotSupportedException();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsFieldValid:           Checks that the field exists and its choice list contains
        ///                         the right number of items
        /// </summary>
        /// <param name="fldDef">   FieldDef to be validated</param>
        //
        //********************************************************************************        
        protected virtual bool IsFieldValid(FieldDef fldDef) {
            bool bRetVal = false;

            if (fldDef != null && fldDef.ChoiceList != null) {
                switch(fldDef.Type) {
                    case FieldType.sdFieldSimpleAlphaChoice:
                    case FieldType.sdFieldSimpleNumChoice:
                    case FieldType.sdFieldBoolean:
                        if (fldDef.ChoiceList.Count != 2) {
                            throw new ApplicationException("ChoiceList must contain exactly two choices to represent checked and unchecked states");
                        }
                        break;
                    case FieldType.sdFieldMultiAlphaChoice:
                    case FieldType.sdFieldMultiNumChoice:
                        if (fldDef.ChoiceList.Count != 1) {
                            throw new ApplicationException("ChoiceList must contain exactly one choice to represent checked state");
                        }
                        break;
                    case FieldType.sdFieldNumeric:
                        break;
                    default:
                        throw new ApplicationException("This control can only be used with boolean or simple/multiple choice fields");
                }
                bRetVal = true;
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetFieldValue:                  Gets the value of the field according to the 
        ///                                 AttachedLine property value
        /// </summary>
        ///
        /// <param name="fldDef">           FieldDef</param> 
        /// <returns>                       The field value</returns>
        //
        //********************************************************************************
        private string GetFieldValue(FieldDef fldDef) {
            string strRetVal;

            if (AttachedLine > -1) {
                strRetVal = fldDef.RowsetDef.ActiveRowset.GetFieldValue(fldDef, AttachedLine, false);
            } else {
                strRetVal = fldDef.Value;
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetFieldValue:                  Sets the value of the field according to the 
        ///                                 AttachedLine property value
        /// </summary>
        ///
        /// <param name="fldDef">           FieldDef</param> 
        /// <param name="strValue">         The field value</param>
        //
        //********************************************************************************
        private void SetFieldValue(FieldDef fldDef, string strValue) {
            if (AttachedLine > -1) {
                fldDef.RowsetDef.ActiveRowset.SetFieldValue(fldDef, 
                                                            AttachedLine,
                                                            false,
                                                            RepPP.SetFieldOpt.sdSetNormal,
                                                            strValue);
            } else {
                fldDef.Value = strValue;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Render:                 Render the control
        /// </summary>
        /// <param name="writer">   HTML writer</param>
        //
        //********************************************************************************
        protected override void Render(HtmlTextWriter writer) {
            if (Visible) {
                base.Render(writer);
            }
        }
    } // Class
} // Namespace
