using System;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.ComponentModel.Design;
using RepPP.Toolkit;
using RepPP;

namespace RepPP.Toolkit.Web {
    //********************************************************************************
    //
    /// <summary>
    /// Implement a CheckBoxList control which is attached to the Rep++ field meta-data
    /// </summary>
    //
    //********************************************************************************
	[ToolboxData("<{0}:FldCheckBoxList runat=server></{0}:FldCheckBoxList>")]
	[DefaultProperty("SelectedIndex")]
    [ValidationProperty("SelectedIndex")]
	public class FldCheckBoxList : System.Web.UI.WebControls.CheckBoxList, IFldEditableControl {
	    /// <summary>Field validator attached to this control</summary>
	    protected FldValidator          m_fldValidator;
	    /// <summary>Field control utility class</summary>
	    protected FldControlUtil        m_fldControlUtil;

        //********************************************************************************
        //
		/// <summary>
		/// ctor:               Class constructor
		/// </summary>
		//
        //********************************************************************************
		public FldCheckBoxList() {
		    m_fldControlUtil    = new FldControlUtil(this, new WebControlUtilHelper(), true);
		    ValidatorPosition   = FldValidator.AutoPos.After;
		}

        //********************************************************************************
        //
		/// <summary>
		/// Dispose:            Dispose the control
		/// </summary>
		//
        //********************************************************************************
        public override void Dispose() {
            if (m_fldValidator != null) {
                m_fldValidator.Dispose();
            }
            if (m_fldControlUtil != null) {
                m_fldControlUtil.Dispose();
            }
            base.Dispose();
        }

        //********************************************************************************
        //
		/// <summary>
		/// Refresh:        Refresh the list of items
		/// </summary>
		//
        //********************************************************************************
        public void Refresh() {
            ListItemCollection items;

            items = base.Items;
            if (items != null) {
                items.Clear();
                FillItemList(items);
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReInit:        Called when Rep++ is reinitialized in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReInit() {
            if (base.Items != null) {
                base.Items.Clear();
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReset:         Called when Rep++ is reset in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReset() {
            if (base.Items != null) {
                base.Items.Clear();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_fldControlUtil != null) {
                m_fldControlUtil.TriggerComponentChange();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FieldFullName:              Full name of the field attached to the CheckBoxList
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [PersistenceMode(PersistenceMode.Attribute)]
        [Editor("RepPP.Toolkit.Designer.FldControlDesigner+FldControlEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Full name of the field attached to the CheckBoxList")]
		public FldFullName FieldFullName {
		    get {
		        return(m_fldControlUtil.FieldFullName);
		    }
		    set {
		        m_fldControlUtil.FieldFullName = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Application Application {
		    get {
		        return(m_fldControlUtil.Application);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// FieldDef:              FieldDef attached to the CheckBoxList
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public FieldDef FieldDef {
		    get {
		        return(m_fldControlUtil.FieldDef);
		    }
		    set {
		        m_fldControlUtil.FieldDef = value;
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// Field:                  Rep++ Field attached to the control
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Field Field {
		    get {
		        return(m_fldControlUtil.Field);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// ID:         ID of the control
        /// </summary>
        //
        //********************************************************************************
        public override string ID {
            get {
                return base.ID;
            }
            set {
                base.ID = value;
                if (m_fldValidator != null) {
                    m_fldValidator.ControlToValidate = value;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ValidatorPosition:      Position of the automatic validator
        /// </summary>
        //
        //********************************************************************************
        [Category("Layout")]
        [Bindable(true)]
        [DefaultValue(FldValidator.AutoPos.After)]
        [Description("Position of the automatic validator")]
        public virtual FldValidator.AutoPos ValidatorPosition {
            get {
                return((m_fldValidator != null) ? m_fldValidator.AutoPosition : FldValidator.AutoPos.None);
            }
            set {
                if (value == FldValidator.AutoPos.None) {
                    if (m_fldValidator != null) {
                        m_fldValidator.Dispose();
                        m_fldValidator = null;
                    }
                } else {
                    if (m_fldValidator == null) {
                        m_fldValidator                   = new FldValidator(this);
                        m_fldValidator.ControlToValidate = ID;
                    }
                    m_fldValidator.AutoPosition = value;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Render:                 Render the control
        /// </summary>
        /// <param name="writer">   HTML writer</param>
        //
        //********************************************************************************
        protected override void Render(HtmlTextWriter writer) {
            if (Visible) {
                base.Render(writer);
                if (m_fldValidator != null) {
                    if (FieldDef != null) {
                        m_fldValidator.InitFromFather(this);
                        m_fldValidator.RenderControl(writer);
                    } else {
                        m_fldValidator.Dispose();
                        m_fldValidator = null;
                    }
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DataMember:         DataMember... We must not support it....
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override string DataMember {
            get {
                return(String.Empty);
            }
            set {
                throw new System.NotSupportedException();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DataSource:         Datasource... We must not support it....
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override object DataSource {
            get {
                return(null);
            }
            set {
                throw new System.NotSupportedException();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DataTextField:      We must not support it....
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override string DataTextField {
            get {
                return(null);
            }
            set {
                throw new System.NotSupportedException();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DataValueField:     We must not support it....
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override string DataValueField {
            get {
                return(null);
            }
            set {
                throw new System.NotSupportedException();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FillItemList:           Fill the item list with the choice list content
        /// </summary>
        /// <param name="items">    List of items</param>
        //
        //********************************************************************************
        private void FillItemList(ListItemCollection items) {
            FieldDef    fldDef;
            int         iIndex;
            ChoiceList  choiceList;
            string      strValue;
            string      strCode;
            string      strDesc;
            bool        bShowDesc;
			ListItem	listItem;

            fldDef = m_fldControlUtil.FieldDef;
            if (fldDef != null && base.Items.Count == 0) {
                choiceList  = fldDef.ChoiceList;
                bShowDesc   = ShowDescription;
                iIndex      = 0;
                do {
                    iIndex = choiceList.ExtractChoice(iIndex, out strValue, out strCode, out strDesc);
                    if (iIndex != -1) {
					    listItem       = new ListItem();
					    listItem.Value = strValue;
                        if (bShowDesc && strDesc != String.Empty) {
                            listItem.Text = strDesc;
                        } else {
                            listItem.Text = strCode;
                        }
                        items.Add(listItem);
                    }
                } while (iIndex != -1);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetSelectedNumItemsFromFieldValue:  Sets the selected flag for each item according to the
        ///                                     Field value
        /// </summary>
        ///
        /// <param name="fldDef">   FieldDef</param> 
        /// <param name="items">    List of items</param>
        //
        //********************************************************************************
        private void SetSelectedNumItemsFromFieldValue(RepPP.FieldDef fldDef, ListItemCollection items) {
            int     iCount;
            int     iFldValue;
            int     iCurrentItemFlag;
            string  strFldValue;
            
            strFldValue = GetFieldValue(fldDef);
            if (!fldDef.IsNull && Util.IsNumeric(strFldValue)) {
                iFldValue   = Convert.ToInt32(strFldValue);
                iCount      = items.Count;
                for (int iIndex = 0; iIndex < iCount ; iIndex++) {
                    iCurrentItemFlag        = Convert.ToInt32(fldDef.ChoiceList.GetValue(iIndex)); 
                    items[iIndex].Selected  = (iFldValue & iCurrentItemFlag) != 0;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetSelectedAlphaItemsFromFieldValue:    Sets the selected flag for each item according to the
        ///                                         Field value
        /// </summary>
        ///
        /// <param name="fldDef">   FieldDef</param> 
        /// <param name="items">    List of items</param>
        //
        //********************************************************************************
        private void SetSelectedAlphaItemsFromFieldValue(RepPP.FieldDef fldDef, ListItemCollection items) {
            int         iCount;
            string[]    arrFldValues;
            ListItem    item;
            string      strFldValue;

            strFldValue     = GetFieldValue(fldDef);
            arrFldValues    = strFldValue.Split(',');
            iCount          = arrFldValues.Length;
            foreach(ListItem itm in items){
                itm.Selected = false;
            }
            for (int iIndex = 0; iIndex < iCount ; iIndex++) {
                item = items.FindByValue(arrFldValues[iIndex]);
                if (item != null) {
                    item.Selected = true;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetFieldValueUsingNumItems:     Sets the field value according to the selected items
        /// </summary>
        ///
        /// <param name="fldDef">           FieldDef</param> 
        /// <param name="items">            List of items</param>
        //
        //********************************************************************************
        private void SetFieldValueUsingNumItems(RepPP.FieldDef fldDef, ListItemCollection items) {
            int iCount;
            int iFldValue           = 0;
            
            iCount = items.Count;
            for (int iIndex = 0; iIndex < iCount ; iIndex++) {
                if (items[iIndex].Selected) {
                    iFldValue += Convert.ToInt32(fldDef.ChoiceList.GetValue(iIndex)); 
                }
            }
            SetFieldValue(fldDef, iFldValue.ToString());
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetFieldValueUsingAlphaItems:   Sets the field value according to the selected items
        /// </summary>
        ///
        /// <param name="fldDef">           FieldDef</param> 
        /// <param name="items">            List of items</param>
        //
        //********************************************************************************
        private void SetFieldValueUsingAlphaItems(RepPP.FieldDef fldDef, ListItemCollection items) {
            string  strFldValue  = String.Empty;

            foreach(ListItem item in items){
                if (item.Selected) {
                    if (strFldValue != String.Empty) {
                        strFldValue += ',';
                    }
                    strFldValue += item.Value;                
                }
            }
            SetFieldValue(fldDef, strFldValue);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetFieldValue:                  Gets the value of the field according to the 
        ///                                 AttachedLine property value
        /// </summary>
        ///
        /// <param name="fldDef">           FieldDef</param> 
        /// <returns>                       The field value</returns>
        //
        //********************************************************************************
        private string GetFieldValue(FieldDef fldDef) {
            string strRetVal;

            if (AttachedLine > -1) {
                strRetVal = fldDef.RowsetDef.ActiveRowset.GetFieldValue(fldDef, AttachedLine, false);
            } else {
                strRetVal = fldDef.Value;
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetFieldValue:                  Sets the value of the field according to the 
        ///                                 AttachedLine property value
        /// </summary>
        ///
        /// <param name="fldDef">           FieldDef</param> 
        /// <param name="strValue">         The field value</param>
        //
        //********************************************************************************
        private void SetFieldValue(FieldDef fldDef, string strValue) {
            if (AttachedLine > -1) {
                fldDef.RowsetDef.ActiveRowset.SetFieldValue(fldDef, 
                                                            AttachedLine,
                                                            false,
                                                            RepPP.SetFieldOpt.sdSetNormal,
                                                            strValue);
            } else {
                fldDef.Value = strValue;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnDataBinding:      Bind the value to the field
        /// </summary>
        /// <param name="e">    Event parameter</param>
        //
        //********************************************************************************
        protected override void OnDataBinding(EventArgs e) {
            RepPP.FieldDef fldDef;

            base.OnDataBinding (e);
            fldDef = m_fldControlUtil.FieldDef;
            if (fldDef != null) {
                if (fldDef.Type == FieldType.sdFieldMultiAlphaChoice) {
                    SetSelectedAlphaItemsFromFieldValue(fldDef, base.Items);
                } else if (fldDef.Type == FieldType.sdFieldMultiNumChoice) {
                    SetSelectedNumItemsFromFieldValue(fldDef, base.Items);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnDataBinding:      Called to bind the value of the control to its associated field
        /// </summary>
        /// <param name="e">    Event parameter</param>
        //
        //********************************************************************************
        protected override void OnSelectedIndexChanged(EventArgs e) {
            RepPP.FieldDef fldDef;

            if (Visible) {
                fldDef = m_fldControlUtil.FieldDef;
                if (fldDef != null) {
                    if (fldDef.Type == FieldType.sdFieldMultiAlphaChoice) {
                        SetFieldValueUsingAlphaItems(fldDef, base.Items);
                    } else if (fldDef.Type == FieldType.sdFieldMultiNumChoice) {
                        SetFieldValueUsingNumItems(fldDef, base.Items);
                    }
                }
                base.OnSelectedIndexChanged (e);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Items:                  List of items in the list
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override ListItemCollection Items {
            get {
                ListItemCollection  itemsRetVal;
                
                itemsRetVal = base.Items;
                if (itemsRetVal != null) {
                    FillItemList(itemsRetVal);
                }
                return(itemsRetVal);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SelectedIndex
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        [Description("Currently selected item")]
        public override int SelectedIndex {
            get {
                return(base.SelectedIndex);
            }
            set {
                base.SelectedIndex = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ShowDescription:    Determine if descriptions or codes are shown in the list
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(true)]
        [Bindable(false)]
        [Description("Determine if descriptions or codes are shown in the list")]
        public virtual bool ShowDescription {
            get {
                bool    bRetVal;
                object  obj;
                
                obj = ViewState["ShowDescription"];
                if (obj != null) {
                    bRetVal = (bool)obj;
                } else {
                    bRetVal = true;
                }
                return(bRetVal);
            }
            set {
                ViewState["ShowDescription"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Enabled:       Determines if the control is enabled
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override bool Enabled {
            get {
                bool        bRetVal;
                FieldDef    fldDef;
                
                fldDef  = m_fldControlUtil.FieldDef;
                bRetVal = (fldDef == null) ? true : fldDef.IsSelectable && fldDef.IsVisible;
                return(bRetVal);
            }
            set {
                throw new System.NotSupportedException();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Visible:        Determines if the control is visible
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override bool Visible {
            get {
                bool        bRetVal;
                FieldDef    fldDef;
                
                fldDef  = m_fldControlUtil.FieldDef;
                bRetVal = (fldDef == null || m_fldControlUtil.IsDesignMode) ? true : fldDef.IsVisible;
                return(bRetVal);
            }
            set {
                throw new System.NotSupportedException();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AttachedLine:             Attached group instance line
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        [DefaultValue(-1)]
        [Description("Attached group instance line index")]
		public virtual int AttachedLine {
            get {
                int     iRetVal;
                object  obj;
                
                obj = ViewState["AttachedLine"];
                if (obj != null) {
                    iRetVal = (int)obj;
                } else {
                    iRetVal = -1;
                }
                return(iRetVal);
            }
            set {
                ViewState["AttachedLine"] = value;
            }
		}
    } // Class
} // Namespace
