using System;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.ComponentModel.Design;
using RepPP.Toolkit;
using RepPP;

namespace RepPP.Toolkit.Web {
    //********************************************************************************
    //
    /// <summary>
    /// Implement a ListBox control which is attached to the Rep++ field meta-data
    /// </summary>
    //
    //********************************************************************************
	[ToolboxData("<{0}:FldListBox runat=server></{0}:FldListBox>")]
	[DefaultProperty("SelectedIndex")]
	public class FldListBox : System.Web.UI.WebControls.ListBox, IFldEditableControl {
	    /// <summary>Field validator attached to this control</summary>
	    protected FldValidator          m_fldValidator;
	    /// <summary>Field control utility class</summary>
	    protected FldControlUtil        m_fldControlUtil;

        //********************************************************************************
        //
		/// <summary>
		/// ctor:               Class constructor
		/// </summary>
		//
        //********************************************************************************
		public FldListBox() {
		    m_fldControlUtil    = new FldControlUtil(this, new WebControlUtilHelper(), true);
		    ValidatorPosition   = FldValidator.AutoPos.After;
		}

        //********************************************************************************
        //
		/// <summary>
		/// Dispose:            Dispose the control
		/// </summary>
		//
        //********************************************************************************
        public override void Dispose() {
            if (m_fldValidator != null) {
                m_fldValidator.Dispose();
            }
            if (m_fldControlUtil != null) {
                m_fldControlUtil.Dispose();
            }
            base.Dispose();
        }

        //********************************************************************************
        //
		/// <summary>
		/// Refresh:        Refresh the list of items
		/// </summary>
		//
        //********************************************************************************
        public void Refresh() {
            ListItemCollection items;

            items = base.Items;
            if (items != null) {
                items.Clear();
                FillItemList(items);
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReset:         Called when Rep++ is reset in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReset() {
            if (base.Items != null) {
                base.Items.Clear();
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReInit:        Called when Rep++ is reinitialized
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReInit() {
            if (base.Items != null) {
                base.Items.Clear();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_fldControlUtil != null) {
                m_fldControlUtil.TriggerComponentChange();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FieldFullName:              Full name of the field attached to the ListBox
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [PersistenceMode(PersistenceMode.Attribute)]
        [Editor("RepPP.Toolkit.Designer.FldControlDesigner+FldControlEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Full name of the field attached to the ListBox")]
		public FldFullName FieldFullName {
		    get {
		        return(m_fldControlUtil.FieldFullName);
		    }
		    set {
		        m_fldControlUtil.FieldFullName = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Application Application {
		    get {
		        return(m_fldControlUtil.Application);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// FieldDef:              FieldDef attached to the ListBox
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public FieldDef FieldDef {
		    get {
		        return(m_fldControlUtil.FieldDef);
		    }
		    set {
		        m_fldControlUtil.FieldDef = value;
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// Field:                  Rep++ Field attached to the control
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Field Field {
		    get {
		        return(m_fldControlUtil.Field);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// ID:         ID of the control
        /// </summary>
        //
        //********************************************************************************
        public override string ID {
            get {
                return base.ID;
            }
            set {
                base.ID                          = value;
                if (m_fldValidator != null) {
                    m_fldValidator.ControlToValidate = value;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ValidatorPosition:      Position of the automatic validator
        /// </summary>
        //
        //********************************************************************************
        [Category("Layout")]
        [Bindable(true)]
        [DefaultValue(FldValidator.AutoPos.After)]
        [Description("Position of the automatic validator")]
        public virtual FldValidator.AutoPos ValidatorPosition {
            get {
                return((m_fldValidator != null) ? m_fldValidator.AutoPosition : FldValidator.AutoPos.None);
            }
            set {
                if (value == FldValidator.AutoPos.None) {
                    if (m_fldValidator != null) {
                        m_fldValidator.Dispose();
                        m_fldValidator = null;
                    }
                } else {
                    if (m_fldValidator == null) {
                        m_fldValidator                   = new FldValidator(this);
                        m_fldValidator.ControlToValidate = ID;
                    }
                    m_fldValidator.AutoPosition = value;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Render:                 Render the control
        /// </summary>
        /// <param name="writer">   HTML writer</param>
        //
        //********************************************************************************
        protected override void Render(HtmlTextWriter writer) {
            int iDummy;

            if (Visible) {
                // Force the reset of the selected index
                iDummy = SelectedIndex;
                base.Render(writer);
                if (m_fldValidator != null) {
                    if (FieldDef != null) {
                        m_fldValidator.InitFromFather(this);
                        m_fldValidator.RenderControl(writer);
                    } else {
                        m_fldValidator.Dispose();
                        m_fldValidator = null;
                    }
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DataMember:         DataMember... We must not support it....
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override string DataMember {
            get {
                return(String.Empty);
            }
            set {
                throw new System.NotSupportedException();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DataSource:         Datasource... We must not support it....
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override object DataSource {
            get {
                return(null);
            }
            set {
                throw new System.NotSupportedException();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DataTextField:      We must not support it....
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override string DataTextField {
            get {
                return(null);
            }
            set {
                throw new System.NotSupportedException();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DataValueField:     We must not support it....
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override string DataValueField {
            get {
                return(null);
            }
            set {
                throw new System.NotSupportedException();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FillItemList:           Fill the item list with the choice list content
        /// </summary>
        /// <param name="items">    List of items</param>
        //
        //********************************************************************************
        private void FillItemList(ListItemCollection items) {
            FieldDef    fldDef;
            int         iIndex;
            ChoiceList  choiceList;
            string      strValue;
            string      strCode;
            string      strDesc;
            bool        bShowDesc;
			ListItem	listItem;

            fldDef = m_fldControlUtil.FieldDef;
            if (fldDef != null && base.Items.Count == 0) {
                choiceList  = fldDef.ChoiceList;
                bShowDesc   = ShowDescription;
                iIndex      = 0;
                do {
                    iIndex = choiceList.ExtractChoice(iIndex, out strValue, out strCode, out strDesc);
                    if (iIndex != -1) {
					    listItem       = new ListItem();
					    listItem.Value = strValue;
                        if (bShowDesc && strDesc != String.Empty) {
                            listItem.Text = strDesc;
                        } else {
                            listItem.Text = strCode;
                        }
                        items.Add(listItem);
                    }
                } while (iIndex != -1);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnDataBinding:      Bind the value to the field
        /// </summary>
        /// <param name="e">    Event parameter</param>
        //
        //********************************************************************************
        protected override void OnDataBinding(EventArgs e) {
            int     iDummy;
            
            base.OnDataBinding (e);
            // Force the reset of the selected index
            iDummy = SelectedIndex;
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnSelectedIndexChanged:      Called when the selected index changes
        /// </summary>
        /// <param name="e">             Event parameter</param>
        //
        //********************************************************************************
        protected override void OnSelectedIndexChanged(EventArgs e) {
            if (Visible) {
                this.SelectedIndex = base.SelectedIndex;
                base.OnSelectedIndexChanged(e);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Items:                  List of items in the list
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override ListItemCollection Items {
            get {
                ListItemCollection  itemsRetVal;
                
                itemsRetVal = base.Items;
                if (itemsRetVal != null) {
                    FillItemList(itemsRetVal);
                }
                return(itemsRetVal);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SelectedIndex
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        [Description("Currently selected item")]
        public override int SelectedIndex {
            get {
                int         iRetVal;
                FieldDef    fldDef;
                string      strValue;
                
                fldDef = m_fldControlUtil.FieldDef;
                if (fldDef == null) {
                    iRetVal = base.SelectedIndex;
                } else {
                    if (AttachedLine > -1) {
                        strValue = fldDef.RowsetDef.ActiveRowset.GetFieldValue(fldDef, AttachedLine, false);
                    } else {
                        strValue = fldDef.Value;
                    }
                    iRetVal = fldDef.ChoiceList.FindValue(strValue);
                    if (iRetVal != base.SelectedIndex) {
                        base.SelectedIndex = iRetVal;
                    }
                }
                return(iRetVal);
            }
            set {
                FieldDef    fldDef;
                string      strValue = String.Empty;
                
                fldDef = m_fldControlUtil.FieldDef;
                if (fldDef != null && fldDef.ChoiceList != null) {
                    if (value >= 0 && value < fldDef.ChoiceList.Count) {
                        strValue = Items[value].Value;
                    }
                    if (AttachedLine > -1) {
                        fldDef.RowsetDef.ActiveRowset.SetFieldValue(fldDef, 
                                                                    AttachedLine,
                                                                    false,
                                                                    RepPP.SetFieldOpt.sdSetNormal,
                                                                    strValue);
                    } else {
                        fldDef.Value = strValue;
                    }
                }
                if (base.SelectedIndex != value) {
                    base.SelectedIndex = value;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SelectionMode:          Gets the list selection mode (single or multiple)
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override ListSelectionMode SelectionMode {
            get {
                ListSelectionMode   eRetVal;
                FieldDef            fldDef;
                
                fldDef = m_fldControlUtil.FieldDef;
                if (fldDef == null) {
                    eRetVal = ListSelectionMode.Single;
                } else {
                    switch(fldDef.Type) {
                    case FieldType.sdFieldMultiAlphaChoice:
                    case FieldType.sdFieldMultiNumChoice:
                        eRetVal = ListSelectionMode.Multiple;
                        break;
                    default:
                        eRetVal = ListSelectionMode.Single;
                        break;
                    }
                }
                return(eRetVal);
            }
            set {
                throw new System.NotSupportedException();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ShowDescription:    Determine if descriptions or codes are shown in the list
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(true)]
        [Bindable(false)]
        [Description("Determine if descriptions or codes are shown in the list")]
        public virtual bool ShowDescription {
            get {
                bool    bRetVal;
                object  obj;
                
                obj = ViewState["ShowDescription"];
                if (obj != null) {
                    bRetVal = (bool)obj;
                } else {
                    bRetVal = true;
                }
                return(bRetVal);
            }
            set {
                ViewState["ShowDescription"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Enabled:       Determines if the control is enabled
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override bool Enabled {
            get {
                bool        bRetVal;
                FieldDef    fldDef;
                
                fldDef  = m_fldControlUtil.FieldDef;
                bRetVal = (fldDef == null) ? true : fldDef.IsSelectable;
                return(bRetVal);
            }
            set {
                throw new System.NotSupportedException();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Visible:        Determines if the control is visible
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override bool Visible {
            get {
                bool        bRetVal;
                FieldDef    fldDef;
                
                fldDef  = m_fldControlUtil.FieldDef;
                bRetVal = (fldDef == null || m_fldControlUtil.IsDesignMode) ? true : fldDef.IsVisible;
                return(bRetVal);
            }
            set {
                throw new System.NotSupportedException();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AttachedLine:             Attached group instance line
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        [DefaultValue(-1)]
        [Description("Attached group instance line index")]
		public virtual int AttachedLine {
            get {
                int     iRetVal;
                object  obj;
                
                obj = ViewState["AttachedLine"];
                if (obj != null) {
                    iRetVal = (int)obj;
                } else {
                    iRetVal = -1;
                }
                return(iRetVal);
            }
            set {
                ViewState["AttachedLine"] = value;
            }
		}
    } // Class
} // Namespace
