using System;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.ComponentModel.Design;
using RepPP.Toolkit;
using RepPP;

namespace RepPP.Toolkit.Web {
    //********************************************************************************
    //
    /// <summary>
    /// Implement a Label control which is attached to the Rep++ field meta-data
    /// </summary>
    //
    //********************************************************************************
	[ToolboxData("<{0}:FldValue runat=server></{0}:FldValue>")]
	[DefaultProperty("Text")]
	public class FldValue : System.Web.UI.WebControls.Label, IFldValueControl {
	    /// <summary>Field control utility class</summary>
	    protected FldControlUtil        m_fldControlUtil;

        //********************************************************************************
        //
		/// <summary>
		/// ctor:               Class constructor
		/// </summary>
		//
        //********************************************************************************
		public FldValue() {
		    m_fldControlUtil = new FldControlUtil(this, new WebControlUtilHelper(), true);
		}

        //********************************************************************************
        //
		/// <summary>
		/// Dispose:            Dispose the control
		/// </summary>
		//
        //********************************************************************************
        public override void Dispose() {
            if (m_fldControlUtil != null) {
                m_fldControlUtil.Dispose();
            }
            base.Dispose();
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReInit:        Called when Rep++ is reinitialized
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReInit() {
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReset:         Called when Rep++ is reset in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReset() {
        }

        //********************************************************************************
        //
        /// <summary>
        /// LoadViewState:              Load the view state
        /// </summary>
        /// <param name="savedState">   Save state</param>
        //
        //********************************************************************************
        protected override void LoadViewState(object savedState) {
            // Nothing to reload
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_fldControlUtil != null) {
                m_fldControlUtil.TriggerComponentChange();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FieldFullName:              Full name of the field attached to the label
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [PersistenceMode(PersistenceMode.Attribute)]
        [Editor("RepPP.Toolkit.Designer.FldControlDesigner+FldControlEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Full name of the field attached to the label")]
		public FldFullName FieldFullName {
		    get {
		        return(m_fldControlUtil.FieldFullName);
		    }
		    set {
		        m_fldControlUtil.FieldFullName = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Application Application {
		    get {
		        return(m_fldControlUtil.Application);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// FieldDef:              FieldDef attached to the label
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public FieldDef FieldDef {
		    get {
		        return(m_fldControlUtil.FieldDef);
		    }
		    set {
		        m_fldControlUtil.FieldDef = value;
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// Field:                  Rep++ Field attached to the control
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Field Field {
		    get {
		        return(m_fldControlUtil.Field);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Text:               Text of the label
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        [Bindable(false)]
        public override string Text {
            get {
                string          strRetVal;
                FldFullName     fldFullName;
                FieldDef        fldDef;
                
                fldDef = m_fldControlUtil.FieldDef;
                if (fldDef == null || m_fldControlUtil.IsDesignMode) {
                    fldFullName     = m_fldControlUtil.FieldFullName;
                    strRetVal       = fldFullName.RowsetName + "." + fldFullName.FieldName;
                } else if (AttachedLine > -1) {
                    strRetVal = fldDef.RowsetDef.ActiveRowset.GetFieldValue(fldDef, AttachedLine, true);
                } else {
                    strRetVal = fldDef.ExtValue;
                }
                return(strRetVal);
            }
            set {
                FieldDef fldDef;
                
				if(m_fldControlUtil != null && m_fldControlUtil.IsDesignMode) {
					fldDef = m_fldControlUtil.FieldDef;
					if (fldDef != null) {
						if (AttachedLine > -1) {
							fldDef.RowsetDef.ActiveRowset.SetFieldValue(fldDef, 
																		AttachedLine,
																		true,
																		RepPP.SetFieldOpt.sdSetNormal,
																		value);
						} else {
							fldDef.ExtValue = value;
						}
					}
				}
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AttachedLine:             Attached group instance line
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        [Bindable(false)]
        [DefaultValue(-1)]
        [Description("Attached group instance line index")]
		public virtual int AttachedLine {
            get {
                int     iRetVal;
                object  obj;
                
                obj = ViewState["AttachedLine"];
                if (obj != null) {
                    iRetVal = (int)obj;
                } else {
                    iRetVal = -1;
                }
                return(iRetVal);
            }
            set {
                ViewState["AttachedLine"] = value;
            }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Visible:        Determines if the control is visible
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override bool Visible {
            get {
                bool        bRetVal;
                FieldDef    fldDef;
                
                fldDef  = m_fldControlUtil.FieldDef;
                bRetVal = (fldDef == null || m_fldControlUtil.IsDesignMode) ? true : fldDef.IsVisible;
                return(bRetVal);
            }
            set {
                throw new System.NotSupportedException();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Render:                 Render the control
        /// </summary>
        /// <param name="writer">   HTML writer</param>
        //
        //********************************************************************************
        protected override void Render(HtmlTextWriter writer) {
            if (Visible) {
                base.Render(writer);
            }
        }
    } // Class
} // Namespace
