﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web;

namespace RepPP.Toolkit.Web.Wizard {
    //********************************************************************************
    //
    /// <summary>
    /// Generates an HTML form from a template
    /// </summary>
    //
    //   BNF
    // Page                    ::= { PageTag | Form }
    // PageTag                 ::= PageGlobalTag | IncludeTag | TestTag
    // PageGlobalTag           ::= '$$REPPP:ATOM$$[' AtomName ']'    
    // IncludeTag              ::= '$$REPPP:INCLUDE$$[' FileName ']'
    // TestTag                 ::= TestAtomTag | TestErrorTag
    // TestAtomTag             ::= '$$REPPP:IF_ATOM_BEGIN$$[' TestAtomExpression ']'
    //                            [ Page ['$$REPPP:ELSE_ATOM$$' ]
    //                            Page '$$REPPP:IF_ATOM_END$$'
    // TestAtomExpression      ::= AtomKey Comparator Value
    // AtomKey                 ::= AtomName [ '.' GroupName ]
    // Comparator              ::= '<' | '>' | '=' | '<>' | '<=' | '>='
    // Occurrence              ::= digits
    // Value                   ::= '"' String '"' | Number
    // Number                  ::= [ '-' ] digits
    // digits                  ::= { 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 }
    // Form                    ::= { FormTag }
    // FormTag                 ::= ForEachChild | ForEachLine | ForEachField | FormGlobalTag
    // FormGlobalTag           ::= '$$REPPP:FORM_NAME$$'
    //                         |   '$$REPPP:ROWSET_NAME$$'
    //                         |   '$$REPPP:VALID_CODE$$'
    //                         |   '$$REPPP:ATOM$$"AtomName"'
    //                         |   PageTag
    // ForEachChild            ::= '$$REPPP:FOR_EACH_CHILD_BEGIN$$'
    //                                { ForEachChildTags }
    //                             '$$REPPP:FOR_EACH_CHILD_END$$'
    // ForEachChildTags        ::= '$$REPPP:CHILD_NAME$$'
    //                         |   ForEachLine
    //                         |   FormGlobalTag
    // ForEachLine             ::= '$$REPPP:FOR_EACH_LINE_BEGIN$$'
    //                                {'$$REPPP:LINE_NUMBER$$' | ForEachField | FormGlobalTag }
    //                             '$$REPPP:FOR_EACH_LINE_END$$'
    // ForEachField            ::= '$$REPPP:FOR_EACH_FIELD_BEGIN$$'
    //                                { FieldTagCond | FormGlobalTag }
    //                             '$$REPPP:FOR_EACH_FIELD_END$$'
    // FieldTagCond            ::= '$$REPPP:IF_FIELD_ATTR_BEGIN$$[' FieldAttributeList ']'
    //                                { FieldTag | FormGlobalTag }
    //                                ['$$REPPP:ELSE_FIELD_ATTR$$'  { FieldTag | FormGlobalTag } ]
    //                             '$$REPPP:IF_FIELD_ATTR_END$$'
    // FieldTag                ::= '$$REPPP:FIELD_LABEL'
    //                         |   '$$REPPP:FIELD_NAME$$'
    //                         |   '$$REPPP:FIELD_CONTROL$$'
    //                         |   '$REPPP:FIELD_VALUE'
    // FieldAttributeList      ::= FieldAttribute {',' FieldAttribute }
    // FieldAttribute          ::= FIRST | NOTFIRST | LAST | NOTLAST | USERDEFINED | NOTUSERDEFINED
    //                         |   REQUIRED | NOTREQUIRED | PRIMARYKEY | NOTPRIMARYKEY | CHANGED
    //                         |   NOTCHANGED | CHOICEAVALAIBLE | CHOICENOTAVALAIBLE | SELECTABLE
    //                         |   NOTSELECTABLE | DATABASEFIELD | NOTDATABASEFIELD | VISIBLE | NOTVISIBLE
    //
    //********************************************************************************
    public class GenHTML : IDisposable {
        /// <summary>Text contains in an atom</summary>
        protected const string TAG_ATOM                    = "$$REPPP:ATOM$$";                    
        /// <summary>Include another template file</summary>
        protected const string TAG_INCLUDE                 = "$$REPPP:INCLUDE$$";
        /// <summary>Begin a conditional block based on an atom expression</summary>
        protected const string TAG_IF_ATOM_BEGIN           = "$$REPPP:IF_ATOM_BEGIN$$"; 
        /// <summary>End a conditional block based on an atom expression</summary>
        protected const string TAG_IF_ATOM_END             = "$$REPPP:IF_ATOM_END$$";
        /// <summary>Else to be used in  a conditional block based on an atom expression</summary>
        protected const string TAG_ELSE_ATOM               = "$$REPPP:ELSE_ATOM$$";
        /// <summary>Begin a security conditional block</summary>
        protected const string TAG_IF_CHOICE_CHECKED_BEGIN = "$$REPPP:IF_CHOICE_CHECKED_BEGIN$$";
        /// <summary>End a security conditional block</summary>
        protected const string TAG_IF_CHOICE_CHECKED_END   = "$$REPPP:IF_CHOICE_CHECKED_END$$"; 
        /// <summary>Else to be used in a security conditional block</summary>
        protected const string TAG_ELSE_CHOICE_CHECKED     = "$$REPPP:ELSE_CHOICE_CHECKED$$";     
        /// <summary>Enumerates each choice (begin)</summary>
        protected const string TAG_CHOICELIST_BEGIN        = "$$REPPP:CHOICELIST_BEGIN$$";
        /// <summary>Enumerates each choice (end)</summary>
        protected const string TAG_CHOICELIST_END          = "$$REPPP:CHOICELIST_END$$";  
        /// <summary>Enumerates each child (begin)</summary>
        protected const string TAG_FOR_EACH_CHILD_BEGIN    = "$$REPPP:FOR_EACH_CHILD_BEGIN$$";    
        /// <summary>Enumerates each child (end)</summary>
        protected const string TAG_FOR_EACH_CHILD_END      = "$$REPPP:FOR_EACH_CHILD_END$$";      
        /// <summary>Enumerates each choice (begin)</summary>
        protected const string TAG_FOR_EACH_CHOICE_BEGIN   = "$$REPPP:FOR_EACH_CHOICE_BEGIN$$";   
        /// <summary>Enumerates each choice (end)</summary>
        protected const string TAG_FOR_EACH_CHOICE_END     = "$$REPPP:FOR_EACH_CHOICE_END$$";     
        /// <summary>Current child name (use in child enum)</summary>
        protected const string TAG_CHILD_NAME              = "$$REPPP:CHILD_NAME$$";              
        /// <summary>Enumerates each line of the current child (begin)</summary>
        protected const string TAG_FOR_EACH_LINE_BEGIN     = "$$REPPP:FOR_EACH_LINE_BEGIN$$";     
        /// <summary>Enumerates each line of the current child (end)</summary>
        protected const string TAG_FOR_EACH_LINE_END       = "$$REPPP:FOR_EACH_LINE_END$$";       
        /// <summary>Current line number</summary>
        protected const string TAG_LINE_NUMBER             = "$$REPPP:LINE_NUMBER$$";             
        /// <summary>Enumerates each field of the current line/child (begin)</summary>
        protected const string TAG_FOR_EACH_FIELD_BEGIN    = "$$REPPP:FOR_EACH_FIELD_BEGIN$$";    
        /// <summary>Enumerates each field of the current line/child (end)</summary>
        protected const string TAG_FOR_EACH_FIELD_END      = "$$REPPP:FOR_EACH_FIELD_END$$";      
        /// <summary>Begin an attribute conditional block</summary>
        protected const string TAG_IF_FIELD_ATTR_BEGIN     = "$$REPPP:IF_FIELD_ATTR_BEGIN$$";   
        /// <summary>End an attribute conditional block</summary>
        protected const string TAG_IF_FIELD_ATTR_END       = "$$REPPP:IF_FIELD_ATTR_END$$";     
        /// <summary>Else to be use in an attribute block</summary>
        protected const string TAG_ELSE_FIELD_ATTR         = "$$REPPP:ELSE_FIELD_ATTR$$";       
        /// <summary>Replaced by the field name</summary>
        protected const string TAG_FIELD_NAME              = "$$REPPP:FIELD_NAME$$";            
        /// <summary>Replaced by the field prompt</summary>
        protected const string TAG_FIELD_LABEL             = "$$REPPP:FIELD_LABEL$$";           
        /// <summary>Replaced by an HTML control representing the field value</summary>
        protected const string TAG_FIELD_CONTROL           = "$$REPPP:FIELD_CONTROL$$";         
        /// <summary>Replaced by the field value</summary>
        protected const string TAG_FIELD_VALUE             = "$$REPPP:FIELD_VALUE$$";           
        /// <summary>Replaced by the internal field value</summary>
        protected const string TAG_FIELD_INTVALUE          = "$$REPPP:FIELD_INTVALUE$$";        
		/// <summary>Replaced by the Choicelist Code front to the field value</summary>
        protected const string TAG_FIELD_CLST_CODE		   = "$$REPPP:FIELD_CLST_CODE$$";		
		/// <summary>Replaced by the Choicelist Description front to the field value</summary>
        protected const string TAG_FIELD_CLST_DESC		   = "$$REPPP:FIELD_CLST_DESC$$";		
        /// <summary>Replaced by the Choicelist Code front to the field value</summary>
        protected const string TAG_CHOICELIST_VALUE		   = "$$REPPP:FIELD_CHOICELIST_VALUE$$";
        /// <summary>Replaced by the Choicelist Code front to the field value</summary>
        protected const string TAG_CHOICELIST_CODE		   = "$$REPPP:FIELD_CHOICELIST_CODE$$"; 
        /// <summary>Replaced by the Choicelist Description front to the field value</summary>
        protected const string TAG_CHOICELIST_DESC		   = "$$REPPP:FIELD_CHOICELIST_DESC$$";	
        /// <summary>Replaced by the current form title (atom FormTitle[:GroupName])</summary>
        protected const string TAG_FORM_NAME               = "$$REPPP:FORM_NAME$$";             
        /// <summary>Replaced by the card name</summary>
        protected const string TAG_CARD_NAME               = "$$REPPP:CARD_NAME$$";				
        /// <summary>Replaced by the rowset tree name</summary>
        protected const string TAG_ROWSET_TREE_NAME        = "$$REPPP:ROWSET_TREE_NAME$$";		
		/// <summary>Replaced by the group name</summary>
        protected const string TAG_GROUP_NAME              = "$$REPPP:GROUP_NAME$$";            
        /// <summary>Replaced by the rowset name </summary>
        protected const string TAG_ROWSET_NAME             = "$$REPPP:ROWSET_NAME$$";                  
        /// <summary>Replaced by the generated validation code</summary>
        protected const string TAG_VALID_CODE              = "$$REPPP:VALID_CODE$$";            
		/// <summary>End Head for Head Implementation</summary>
        protected const string HTML_TAG_ENDHEAD            = "</HEAD>";                         
        /// <summary>End Body for Body Implementation</summary>
        protected const string HTML_TAG_ENDBODY            = "</BODY>";                         

        private enum IFTypeE {
            IT_Begin        = 0,
            IT_Atom         = 0,
            IT_End          = 0
        };

        private enum OperatorE {
            OP_None                 = 0,
            OP_NotEqual             = 1,
            OP_LessOrEqualThan      = 2,
            OP_GreaterOrEqualThan   = 3,
            OP_EqualTo              = 4,
            OP_LessThan             = 5,
            OP_GreaterThan          = 6,
			OP_Contains	            = 7
        };

        private enum FieldAttrE {
            FA_None              = 0,
            FA_First             = 1,
            FA_NotFirst          = 2,
            FA_Last              = 3,
            FA_NotLast           = 4,
            FA_UserDefined       = 5,
            FA_NotUserDefined    = 6,
            FA_Required          = 7,
            FA_NotRequired       = 8,
            FA_PrimaryKey        = 9,
            FA_NotPrimaryKey     = 10,
            FA_Changed           = 11,
            FA_NotChanged        = 12,
            FA_ChoiceAvailable   = 13,
            FA_NoChoiceAvailable = 14, 
            FA_Selectable        = 15,
            FA_NotSelectable     = 16,
            FA_DatabaseField     = 17,
            FA_NotDatabaseField  = 18,
            FA_Visible           = 19,
            FA_NotVisible        = 20,
            FA_OutGroup          = 21,
            FA_InGroup           = 22
        };
          
        /// <summary>Rep++ Application object</summary>
        protected      RepPP.Application                        m_app;
        /// <summary>Rep++ Internet Code Generator</summary>
        protected      RepPP.IntCodeGen                         m_codeGen;
        /// <summary>Generated code</summary>
        protected      string                                   m_strGeneratedCode;
        /// <summary>FieldDef Attributes</summary>
        static private Dictionary<String, FieldAttrE>           m_hashAttr = null;

        //********************************************************************************
        //
        /// <summary>
        /// Static ctor
        /// </summary>
        //
        //********************************************************************************
        static GenHTML() {
            m_hashAttr  = new Dictionary<string,FieldAttrE>(30);
            m_hashAttr.Add("FIRST",               FieldAttrE.FA_First);
            m_hashAttr.Add("NOTFIRST",            FieldAttrE.FA_NotFirst);
            m_hashAttr.Add("LAST",                FieldAttrE.FA_Last);
            m_hashAttr.Add("NOTLAST",             FieldAttrE.FA_NotLast);
            m_hashAttr.Add("USERDEFINED",         FieldAttrE.FA_UserDefined);
            m_hashAttr.Add("NOTUSERDEFINED",      FieldAttrE.FA_NotUserDefined);
            m_hashAttr.Add("REQUIRED",            FieldAttrE.FA_Required);
            m_hashAttr.Add("NOTREQUIRED",         FieldAttrE.FA_NotRequired);
            m_hashAttr.Add("PRIMARYKEY",          FieldAttrE.FA_PrimaryKey);
            m_hashAttr.Add("NOTPRIMARYKEY",       FieldAttrE.FA_NotPrimaryKey);
            m_hashAttr.Add("CHANGED",             FieldAttrE.FA_Changed);
            m_hashAttr.Add("NOTCHANGED",          FieldAttrE.FA_NotChanged);
            m_hashAttr.Add("CHOICEAVAILABLE",     FieldAttrE.FA_ChoiceAvailable);
            m_hashAttr.Add("CHOICENOTAVAILABLE",  FieldAttrE.FA_NoChoiceAvailable);
            m_hashAttr.Add("SELECTABLE",          FieldAttrE.FA_Selectable);
            m_hashAttr.Add("NOTSELECTABLE",       FieldAttrE.FA_NotSelectable);
            m_hashAttr.Add("DATABASEFIELD",       FieldAttrE.FA_DatabaseField);
            m_hashAttr.Add("NOTDATABASEFIELD",    FieldAttrE.FA_NotDatabaseField);
            m_hashAttr.Add("VISIBLE",             FieldAttrE.FA_Visible);
            m_hashAttr.Add("NOTVISIBLE",          FieldAttrE.FA_NotVisible);
            m_hashAttr.Add("OUTGROUP",            FieldAttrE.FA_Visible);
            m_hashAttr.Add("INGROUP",             FieldAttrE.FA_NotVisible);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="app">                  Rep++ Object Application</param>
        //
        //********************************************************************************
        public GenHTML(RepPP.Application app) {
            m_app               = app;
            m_codeGen           = app.CreateExtObj(ExtObject.sdIntCodeGen) as RepPP.IntCodeGen;
            m_strGeneratedCode  = "";
        }

        //********************************************************************************
        //
        /// <summary>
        /// Dispose:                            Class destructor
        /// </summary>
        //
        //********************************************************************************
        public virtual void Dispose() {
            m_app               = null;
            m_strGeneratedCode  = "";
            m_codeGen           = null;
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetAtom:                        Gets the value of an atom
        /// </summary>
        /// <param name="strName">          Atom name</param>
        /// <param name="strRSName">        Rowset name</param>
        /// <param name="eAtomType">        Atom type</param>
        /// <returns>
        /// Atom value
        /// </returns>
        //
        //********************************************************************************
        protected string GetAtom(string strName, string strRSName, RepPP.AtomType eAtomType) {
            string  strRetVal = "";

			if (m_app.IsAtomExist(strName + "." + strRSName, eAtomType)) {
				strRetVal = m_app.GetAtomText(strName + "." + strRSName, eAtomType);
			} else if (m_app.IsAtomExist(strName, eAtomType)) {
			    strRetVal = m_app.GetAtomText(strName, eAtomType);
			}
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetError:                       Gets the value of an error atom
        /// </summary>
        /// <param name="strName">          Atom name</param>
        /// <param name="strRSName">        Rowset name</param>
        /// <returns>
        /// Atom value
        /// </returns>
        //
        //********************************************************************************
        protected string GetError(string strName, string strRSName) {
            return(GetAtom(strName, strRSName, RepPP.AtomType.sdAtomErr));
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetText:                        Gets the value of a text atom
        /// </summary>
        /// <param name="strName">          Atom name</param>
        /// <param name="strRSName">        Rowset name</param>
        /// <returns>
        /// Atom value
        /// </returns>
        //
        //********************************************************************************
        protected string GetText(string strName, string strRSName) {
            return(GetAtom(strName, strRSName, RepPP.AtomType.sdAtomMsg));
        }

		//********************************************************************************
		//
		/// <summary>
		/// DirExists:						Check if the current filename have a Path or not
		/// </summary>
		/// <param name="strFilename">      String which is containing Filename</param>
		/// <returns>
		///	true							if a path is found
		/// </returns>
		//
		//********************************************************************************
		private bool DirExists(string strFilename) {
			return (strFilename.IndexOf('\\') != -1);				
		}

        //********************************************************************************
        //
        /// <summary>
        /// IsBlank:                        Returns true if the character is a blank
        /// </summary>
        /// <param name="cChr">             Character</param>
        /// <returns>
        /// true if yes, no if not
        /// </returns>
        //
        //********************************************************************************
        protected bool IsBlank(char cChr) {
            return(cChr == ' ' || cChr == (char)9 || cChr == (char)12 || cChr == (char)13 || cChr == (char)10);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ExtractString:                  Extract a string between '[' and ']'
        /// </summary>
        /// <param name="str">              String to parse</param>
        /// <param name="iPos">             Position to start with. Returned the new position after the string</param>
        /// <returns>
        /// String without the '[' ']'
        /// </returns>
        //
        //********************************************************************************
        protected string ExtractString(string str, ref int iPos) {
            string  strRetVal = "";
            int     iPos2;
            int     iSize;
            int     iStartPos = iPos;

            iSize = str.Length;
            while (iPos < iSize && IsBlank(str[iPos])) {
                iPos++;
            }
            if (iPos < iSize && str[iPos] == '[') {
                iPos++;
                iPos2 = str.IndexOf(']', iPos);
                if (iPos2 != -1) {
                    strRetVal = str.Substring(iPos, iPos2 - iPos);
                    iPos      = iPos2 + 1;
                } else {
                    throw new ParseException("Missing the closing ']'", str, iStartPos);
                }
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetAtomValue:               Gets the value of an atom. Search in the hierarchy
        /// </summary>
        /// <param name="strAtomName">  Text to process</param>
        /// <param name="strGrpName">   Group name</param>
        /// <returns>
        /// Atom text
        /// </returns>
        //
        //********************************************************************************
        private string GetAtomValue(string strAtomName, string strGrpName) {
            string  strRetVal = "";            
            int     iPos;

            if (strAtomName != null || strAtomName.Length != 0) {
                iPos = strAtomName.IndexOf('.');
                if (iPos != -1) {
                    strGrpName  = strAtomName.Substring(0, iPos);
                    strAtomName = strAtomName.Substring(iPos + 1);
                }
                strRetVal = GetText(strAtomName, strGrpName);
                if (strRetVal.Length == 0) {
                    strRetVal  = GetError(strAtomName, strGrpName);
                }
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// EvalFieldAttr:          Evaluates a field attribute
        /// </summary>
        /// <param name="fldDef">   Field Definition to check</param>
        /// <param name="eAttr">    Attribute to check</param>
        /// <return>
        /// condition value
        /// </return>
        //
        //********************************************************************************
        private bool EvalFieldAttr(RepPP.FieldDef fldDef, FieldAttrE eAttr) {
            bool    bRetVal;
            
            switch(eAttr) {
	        case FieldAttrE.FA_First:
                bRetVal = fldDef.IsFirst;
                break;
	        case FieldAttrE.FA_NotFirst:
	            bRetVal = !fldDef.IsFirst;
                break;
	        case FieldAttrE.FA_Last:
	            bRetVal = fldDef.IsLast;
                break;
	        case FieldAttrE.FA_NotLast:
	            bRetVal = !fldDef.IsLast;
                break;
	        case FieldAttrE.FA_UserDefined:
	            bRetVal = fldDef.IsUserFlagOn;
                break;
	        case FieldAttrE.FA_NotUserDefined:
	            bRetVal = !fldDef.IsUserFlagOn;
                break;
	        case FieldAttrE.FA_Required:
	            bRetVal = fldDef.IsRequired;
                break;
	        case FieldAttrE.FA_NotRequired:
	            bRetVal = !fldDef.IsRequired;
                break;
	        case FieldAttrE.FA_PrimaryKey:
	            bRetVal = fldDef.IsPrimaryKey;
                break;
	        case FieldAttrE.FA_NotPrimaryKey:
                bRetVal = !fldDef.IsPrimaryKey;
                break;
	        case FieldAttrE.FA_Changed:
	            bRetVal = fldDef.IsChanged;
                break;
	        case FieldAttrE.FA_NotChanged:
                bRetVal = !fldDef.IsChanged;
                break;
            case FieldAttrE.FA_ChoiceAvailable:
                bRetVal = fldDef.IsChoiceAvailable;
                break;
	        case FieldAttrE.FA_NoChoiceAvailable:
                bRetVal = !fldDef.IsChoiceAvailable;
                break;
	        case FieldAttrE.FA_Selectable:
	            bRetVal = fldDef.IsSelectable;
                break;
	        case FieldAttrE.FA_NotSelectable:
                bRetVal = !fldDef.IsSelectable;
                break;
	        case FieldAttrE.FA_DatabaseField:
	            bRetVal = fldDef.IsDatabaseLinked;
                break;
	        case FieldAttrE.FA_NotDatabaseField:
                bRetVal = !fldDef.IsDatabaseLinked;
                break;
	        case FieldAttrE.FA_Visible:
	            bRetVal = fldDef.IsVisible;
                break;
	        case FieldAttrE.FA_NotVisible:
                bRetVal = !fldDef.IsVisible;
                break;
	        case FieldAttrE.FA_OutGroup:
	            bRetVal = true;
                break;
	        case FieldAttrE.FA_InGroup:
                bRetVal = false;
                break;
            default:
                throw new ParseException("Unknown attribute code" + eAttr.ToString(), "Internal error", 0);
            };
            return(bRetVal);            
        }

        //********************************************************************************
        //
        /// <summary>
        /// EvalFieldAttr:              Evaluates a field attribute
        /// </summary>
        /// <param name="fldDef">       Field Definition to check</param>
        /// <param name="strAttrList">  Attributes to check</param>
        /// <param name="strText">      Source text for error processing</param>
        /// <param name="iSrcPos">      Source position for error processing</param>
        /// <return>
        /// condition value
        /// </return>
        //
        //********************************************************************************
        private bool EvalFieldAttr(RepPP.FieldDef fldDef, string strAttrList, string strText, int iSrcPos) {
            bool        bRetVal = true;
            int         iPos;
            FieldAttrE  eAttr;
            string      strAttr;
            
            if (strAttrList.Length != 0) {
                strAttrList = strAttrList.ToUpper() + ",";
                iPos = strAttrList.IndexOf(',');
                while (iPos != -1 && bRetVal) {
                    strAttr     = strAttrList.Substring(0, iPos);
                    strAttrList = strAttrList.Substring(iPos + 1);
                    if (m_hashAttr.ContainsKey(strAttr.ToUpper())) {
                        eAttr   = (FieldAttrE)m_hashAttr[strAttr];
                        bRetVal = EvalFieldAttr(fldDef, eAttr);
                    } else {
                        throw new ParseException("Unknown field attribute '" + strAttr + "'", strText, iSrcPos);
                    }
                    iPos = strAttrList.IndexOf(',');
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ReplaceValueTag:            Replace all value-tags with the corresponding value
        /// </summary>
        /// <param name="strText">      Text to process</param>
        /// <param name="strRSName">    Processed rowset name</param>
        /// <param name="strError">     Error text</param>
        /// <returns>
        /// Processed text
        /// </returns>
        //
        //********************************************************************************
        private string ReplaceValueTag(string strText, string strRSName, string strError) {
            int             iStart;
            int             iPos;
            string          strName;

            // atoms
            iStart = strText.IndexOf(TAG_ATOM);
            while (iStart != -1) {
                iPos    = iStart + TAG_ATOM.Length;
                strName = ExtractString(strText, ref iPos);
                if (strName != null && strName.Length != 0) {
                    strText = strText.Substring(0, iStart) +
                              HttpUtility.HtmlEncode(GetAtomValue(strName, strRSName)) +
                              strText.Substring(iPos);                                        
                }
                iStart = strText.IndexOf(TAG_ATOM, iStart + 1);
            }

            return(strText);
        }

        //********************************************************************************
        //
        /// <summary>
        /// CheckCondition:             Check a condition
        /// </summary>
        /// <param name="dOP1">         Operator 1</param>
        /// <param name="eCompareOp">   Compare operator</param>
        /// <param name="dOP2">         Operator 2</param>
        /// <returns>
        /// true if condition is true, false if not
        /// </returns>
        //
        //********************************************************************************
        private bool CheckCondition(double dOP1, OperatorE eCompareOp, double dOP2) {
            bool    bRetVal;
            
            switch(eCompareOp) {
            case OperatorE.OP_EqualTo:
                bRetVal = (dOP1 == dOP2);
                break;
            case OperatorE.OP_GreaterOrEqualThan:
                bRetVal = (dOP1 >= dOP2);
                break;    
            case OperatorE.OP_GreaterThan:
                bRetVal = (dOP1 > dOP2);
                break;    
            case OperatorE.OP_LessOrEqualThan:
                bRetVal = (dOP1 <= dOP2);
                break;    
            case OperatorE.OP_LessThan:
                bRetVal = (dOP1 < dOP2);
                break;    
			case OperatorE.OP_Contains:
				bRetVal = false; // Must implements !!!!
				break;    
            default:
                throw new ParseException("Unknown comparaison op");
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// CheckCondition:             Check a condition
        /// </summary>
        /// <param name="strOP1">       Operator 1</param>
        /// <param name="eCompareOp">   Compare operator</param>
        /// <param name="strOP2">       Operator 2</param>
        /// <returns>
        /// true if condition is true, false if not
        /// </returns>
        //
        //********************************************************************************
        private bool CheckCondition(string strOP1, OperatorE eCompareOp, string strOP2) {
            bool    bRetVal;
            int     iCmp;
            
            iCmp = strOP1.CompareTo(strOP2);
            switch(eCompareOp) {
            case OperatorE.OP_EqualTo:
                bRetVal = (iCmp == 0);
                break;
            case OperatorE.OP_GreaterOrEqualThan:
                bRetVal = (iCmp >= 0);
                break;    
            case OperatorE.OP_GreaterThan:
                bRetVal = (iCmp > 0);
                break;    
            case OperatorE.OP_LessOrEqualThan:
                bRetVal = (iCmp <= 0);
                break;    
            case OperatorE.OP_LessThan:
                bRetVal = (iCmp < 0);
                break;    
            case OperatorE.OP_NotEqual:
                bRetVal = (iCmp != 0);
                break;    
			case OperatorE.OP_Contains:
				bRetVal = (strOP1.IndexOf(strOP2) != -1);
				break;    
            default:
                throw new ParseException("Unknown comparaison op");
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ProcessCondition:           Process a IF condition
        /// </summary>
        /// <param name="eType">        IF type</param>
        /// <param name="strCondition"> Condition to check</param>
        /// <param name="strIFBlock">   Block to return if condition is true</param>
        /// <param name="strELSEBlock"> Block to return if condition is false</param>
        /// <param name="strRSName">    Rowset name for which the condition is processed</param>
        /// <returns>
        /// Processed text
        /// </returns>
        /// <remarks>
        /// BNF
        ///   TestAtomTag             ::= '$$REPPP:IF_ATOM_BEGIN$$[TestAtomExpression]' Page  ['$$REPPP:ELSE_ATOM$$' Page ] '$$REPPP:IF_ATOM_END$$'
        ///   TestSBRootTag           ::= '$$REPPP:IF_SBROOT' Page ['$$REPPP:ELSE_SBROOT$$' Page ] '$$REPPP:IF_SBROOT_END$$'
        ///   TestAtomExpression      ::= AtomKey Comparator Value
        /// </remarks>
        //
        //********************************************************************************
        private string ProcessCondition(IFTypeE   eType,
                                        string    strCondition,
                                        string    strIFBlock,
                                        string    strELSEBlock,
                                        string    strRSName) {
            string          strRetVal           = "";
            string          strValue            = "";
            string          strReference        = "";
            string          strReferenceValue   = "";
            OperatorE       eCompareOp          = OperatorE.OP_None;

            switch (eType) {
            case IFTypeE.IT_Atom:
                strReferenceValue = GetAtomValue(strReference, strRSName);
                break;
            default:
                throw new ParseException("Unknown IF type");
            }
            if (CheckCondition(strReferenceValue, eCompareOp, strValue)) {
                strRetVal = strIFBlock;
            } else {
                strRetVal = strELSEBlock;
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ProcessTests:               Process a IF condition
        /// </summary>
        /// <param name="strText">      Text to process</param>
        /// <param name="eType">        IF type</param>
        /// <param name="strIFTag">     IF TAG</param>
        /// <param name="strELSETag">   ELSE TAG</param>
        /// <param name="strENDTag">    END TAG</param>
        /// <param name="strRSName">    Rowset name for which the test is done</param>
        /// <returns>
        /// Processed text
        /// </returns>
        /// <remarks>
        /// BNF
        ///   TestAtomTag     ::= '$$REPPP:IF_ATOM_BEGIN$$[TestAtomExpression]' Page  ['$$REPPP:ELSE_ATOM$$' Page ] '$$REPPP:IF_ATOM_END$$'
        /// </remarks>
        //
        //********************************************************************************
        private string ProcessTests(string    strText,
	    					        IFTypeE   eType,
	    							string    strIFTag,
	    							string    strELSETag,
	    							string    strENDTag,
	    							string    strRSName) {
            string          strIFBlock;
            string          strELSEBlock;
            string          strCondition;
            string          strBlock;
            int             iIFStartPos;
            int             iIFPos;
            int             iELSEPos;
            int             iENDPos;
            int             iPos;
            int             iPos2;
            int             iMinPos;
            int             iCount;
            int             iTagFound;

            iIFStartPos = strText.IndexOf(strIFTag);
            while (iIFStartPos != -1) {
                iIFPos = iIFStartPos + strIFTag.Length;
                if (iIFPos < strText.Length && strText[iIFPos] == '[') {
                    strCondition = ExtractString(strText, ref iIFPos);
                } else {
                    strCondition = "";
                }
                iCount      = 1;
                iPos        = iIFPos;
                iELSEPos    = -1;
                iENDPos     = -1;
                while (iPos != -1 && iCount != 0) {
                    iTagFound = -1;
                    iMinPos   = strText.IndexOf(strIFTag, iPos);
                    if (iMinPos != -1) {
                        iTagFound = 0;
                    }
                    iPos2 = strText.IndexOf(strELSETag, iPos);
                    if (iPos2 != -1) {
                        if (iMinPos == -1 || iPos2 < iMinPos) {
                            iMinPos   = iPos2;
                            iTagFound = 1;
                        }
                    }
                    iPos2 = strText.IndexOf(strENDTag, iPos);
                    if (iPos2 != -1) {
                        if (iMinPos == -1 || iPos2 < iMinPos) {
                            iMinPos   = iPos2;
                            iTagFound = 2;
                        }
                    }
                    switch(iTagFound) {
                    case 0:
                        iCount++;
                        iPos = iMinPos + 1;
                        break;
                    case 1:
                        if (iCount == 1) {
                            iELSEPos = iMinPos;
                        }
                        iPos = iMinPos + 1;
                        break;
                    case 2:
                        iCount--;
                        if (iCount == 0) {
                            iENDPos = iMinPos;
                        }
                        iPos = iMinPos + 1;
                        break;
                    default:
                        iPos = -1;
                        break;
                    }
                }
                if (iPos == -1) {
                    throw new ParseException("No " + strENDTag + " matching " + strIFTag,
                                                 strText,
                                                 iIFPos);
                }
                if (iELSEPos == -1) {
                    strIFBlock   = strText.Substring(iIFPos, iENDPos - iIFPos);
                    strELSEBlock = "";
                } else {
                    strIFBlock   = strText.Substring(iIFPos, iELSEPos - iIFPos);
                    iELSEPos    += strELSETag.Length;
                    strELSEBlock = strText.Substring(iELSEPos, iENDPos - iELSEPos);
                }
                iPos        = iENDPos + strENDTag.Length;
                strBlock    = ProcessCondition(eType,
                                               strCondition,
                                               strIFBlock,
                                               strELSEBlock,
                                               strRSName);
                strText     = strText.Substring(0, iIFStartPos) +
                              strBlock + 
                              strText.Substring(iPos);
                iIFStartPos = strText.IndexOf(strIFTag);
            }
            return(strText);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ProcessTestBlocks:          Process the IF condition
        /// </summary>
        /// <param name="strText">      Text to process</param>
        /// <param name="strRSName">    Processed rowset name</param>
        /// <returns>
        /// Processed text
        /// </returns>
        /// <remarks>
        /// BNF
        ///   TestAtomTag     ::= '$$REPPP:IF_ATOM_BEGIN$$[TestAtomExpression]' Page  ['$$REPPP:ELSE_ATOM$$' Page ] '$$REPPP:IF_ATOM_END$$'
        /// </remarks>
        //
        //********************************************************************************
        private string ProcessTestBlocks(string strText, string strRSName) {
            string          strIFTag;
            string          strENDTag;
            string          strELSETag;

            for (IFTypeE eType = IFTypeE.IT_Begin; eType <= IFTypeE.IT_End; eType++) {
                switch(eType) {
                case IFTypeE.IT_Atom:
                    strIFTag    = TAG_IF_ATOM_BEGIN;
                    strENDTag   = TAG_IF_ATOM_END;
                    strELSETag  = TAG_ELSE_ATOM;
                    break;
                default:
                    throw new ParseException("internal error. Bad Variable type", "Internal", 0);
                }
                if (strIFTag.Length != 0) {
                    strText = ProcessTests(strText, eType, strIFTag,  strELSETag, strENDTag, strRSName);
                }
            }
            return(strText);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ProcessFieldTag:                    Process the field tags
        /// </summary>
        /// <param name="strText">              Text to process</param>
        /// <param name="rs">                   Rowset</param>
        /// <param name="fldDef">               Field Definition</param>
        /// <returns>
        /// Processed text
        /// </returns>
        /// <remarks>
        /// FieldTagCond    ::= ['$$REPPP:IF_FIELD_ATTR_BEGIN$$''['AttributeList']']'(FieldTag)['$$REPPP:IF_FIELD_ATTR_END$$']
        /// FieldTag        ::= '$$REPPP:FIELD_LABEL' | '$$REPPP:FIELD_NAME$$' | '$$REPPP:FIELD_CONTROL$$' | '$REPPP:FIELD_VALUE' | ACTION_TAG
        /// </remarks>
        //                                                                                             
        //********************************************************************************
        protected virtual string ProcessFieldTag(string         strText,
                                                 RepPP.Rowset   rs,
                                                 RepPP.FieldDef fldDef) {
            string              strHtml;
            RepPP.HTMLCtlType   eCtlType;
			RepPP.ChoiceList	chlFld;
		    string              strCode;
			string              strDesc;
            
			chlFld      = fldDef.ChoiceList;
            strHtml     = (fldDef.IsVisible) ?
                          m_codeGen.GenerateFieldObjCtl(fldDef, fldDef.Name, RepPP.HTMLCtlType.sdHTMLCtlLabel) :
                          "";
            strText  = strText.Replace(TAG_FIELD_LABEL, strHtml);
            eCtlType = (fldDef.IsVisible) ? m_codeGen.DefaultFieldObjCtlType(fldDef) : RepPP.HTMLCtlType.sdHTMLCtlHidden;
            if (eCtlType == RepPP.HTMLCtlType.sdHTMLCtlSelectCombo && !fldDef.IsSelectable) {
                // fix a limit in HTML combo control which would not return valid otherwize
            //    eCtlType = RepPP.HTMLCtlType.sdHTMLCtlText;
            } else if (fldDef.IsPassword) {
                eCtlType = RepPP.HTMLCtlType.sdHTMLCtlPassword;
            }        
            strText = strText.Replace(TAG_FIELD_CONTROL,    m_codeGen.GenerateFieldObjCtl(fldDef, fldDef.Name, eCtlType));
            strText = strText.Replace(TAG_FIELD_VALUE,      HttpUtility.HtmlEncode(fldDef.ExtValue));
			strText = strText.Replace(TAG_FIELD_INTVALUE,   HttpUtility.HtmlEncode(fldDef.Value));
            strText = strText.Replace(TAG_FIELD_NAME,       HttpUtility.HtmlEncode(fldDef.Name));
			if (chlFld != null && chlFld.Count > 0)	{
				strCode = "";
				strDesc = "";
				for (int i = 0; i < chlFld.Count; i++) {
					if (chlFld.GetValue(i) == fldDef.Value) {
						strCode = chlFld.GetCode(i);
						strDesc = chlFld.GetDesc(i);
						break;
					}
				}
				strText = strText.Replace(TAG_FIELD_CLST_CODE, HttpUtility.HtmlEncode(strCode));
				strText = strText.Replace(TAG_FIELD_CLST_DESC, HttpUtility.HtmlEncode(strDesc));
			}
            return(strText);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ProcessIfFieldAttr:                 Process the If attribute block
        /// </summary>
        /// <param name="strText">              Text to process</param>
        /// <param name="rs">                   Rowset</param>
        /// <param name="fldDef">               Field Definition</param>
        /// <returns>
        /// Processed text
        /// </returns>
        /// <remarks>
        /// FieldTagCond    ::= ['$$REPPP:IF_FIELD_ATTR_BEGIN$$''['AttributeList']']'(FieldTag)['$$REPPP:IF_FIELD_ATTR_END$$']
        /// FieldTag        ::= '$$REPPP:FIELD_LABEL' | '$$REPPP:FIELD_NAME$$' | '$$REPPP:FIELD_CONTROL$$' | '$REPPP:FIELD_VALUE' | ACTION_TAG
        /// </remarks>
        //                                                                                             
        //********************************************************************************
        private string ProcessIfFieldAttr(string            strText,
	    								  RepPP.Rowset      rs,
	    								  RepPP.FieldDef 	fldDef) {
	    	string  strRetVal = "";
	    	int     iPos;
	    	int     iStart;
	    	string  strAttr;
	    								
            iStart = 0;            
	        iPos   = strText.IndexOf(TAG_IF_FIELD_ATTR_BEGIN);
	        while (iPos != -1) {
	            strRetVal += ProcessFieldTag(strText.Substring(iStart, iPos - iStart), rs, fldDef);
	            //TODO: The ELSE clause is not processed in any case!                             
	            iPos     += TAG_IF_FIELD_ATTR_BEGIN.Length;
	            strAttr   = (iPos < strText.Length && strText[iPos] == '[') ? ExtractString(strText, ref iPos) : "";
	            iStart    = strText.IndexOf(TAG_IF_FIELD_ATTR_END, iPos);
	            if (iStart == -1) {
                    throw new ParseException("No " + TAG_IF_FIELD_ATTR_END + " matching " + TAG_IF_FIELD_ATTR_BEGIN,
                                                 strText,
                                                 iPos);
	            }
                if (EvalFieldAttr(fldDef, strAttr, strText, iPos)) {
                    strRetVal += ProcessFieldTag(strText.Substring(iPos, iStart - iPos),
                                                 rs,
                                                 fldDef);
                }
                iStart += TAG_IF_FIELD_ATTR_END.Length;
                iPos    = strText.IndexOf(TAG_IF_FIELD_ATTR_BEGIN, iStart);
            }
            strRetVal += ProcessFieldTag(strText.Substring(iStart),
                                         rs,
                                         fldDef);
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ProcessForFieldBlock:               Process the ForEachField block
        /// </summary>
        /// <param name="strText">              Text to process</param>
        /// <param name="rs">                   Rowset</param>
        /// <returns>
        /// Processed text
        /// </returns>
        /// <remarks>
        /// ForEachField    ::= '$$REPPP:FOR_EACH_FIELD_BEGIN$$' {FieldTag} '$$REPPP:FOR_EACH_FIELD_END$$'
        /// FieldTag        ::= '$$REPPP:FIELD_LABEL' | '$$REPPP:FIELD_NAME$$' | '$$REPPP:FIELD_CONTROL$$' | '$REPPP:FIELD_VALUE'
        /// </remarks>
        //                                                                                             
        //********************************************************************************
        protected virtual string ProcessForEachFieldBlock(string        strText,
	    										          RepPP.Rowset  rs) {
            string  strRetVal = "";
            
            foreach (RepPP.FieldDef fld in rs.RowsetDef.FieldDefs) {
                strRetVal += ProcessIfFieldAttr(strText, rs, fld);
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ProcessForField:                    Process the ForEachField tag
        /// </summary>
        /// <param name="strText">              Text to process</param>
        /// <param name="rs">                   Rowset</param>
        /// <returns>
        /// Processed text
        /// </returns>
        /// <remarks>
        /// ForEachField    ::= '$$REPPP:FOR_EACH_FIELD_BEGIN$$' {FieldTag} '$$REPPP:FOR_EACH_FIELD_END$$'
        /// FieldTag        ::= '$$REPPP:FIELD_LABEL' | '$$REPPP:FIELD_NAME$$' | '$$REPPP:FIELD_CONTROL$$' | '$REPPP:FIELD_VALUE'
        /// </remarks>
        //                                                                                             
        //********************************************************************************
        private string ProcessForEachField(string       strText,
                                           RepPP.Rowset rs) {
            string      strRetVal = "";
            int         iPos;
            int 	    iStart = 0;

	        iPos = strText.IndexOf(TAG_FOR_EACH_FIELD_BEGIN);
	        while (iPos != -1) {
	            strRetVal += strText.Substring(iStart, iPos - iStart);
	            iPos      += TAG_FOR_EACH_FIELD_BEGIN.Length;
	            iStart     = strText.IndexOf(TAG_FOR_EACH_FIELD_END, iPos);
	            if (iStart == -1) {
                    throw new ParseException("No " + TAG_FOR_EACH_FIELD_END + " matching " + TAG_FOR_EACH_FIELD_BEGIN,
                                                 strText,
                                                 iPos);
	            }
	            strRetVal += ProcessForEachFieldBlock(strText.Substring(iPos, iStart - iPos), rs);
	            iStart    += TAG_FOR_EACH_FIELD_END.Length;
	            iPos       = strText.IndexOf(TAG_FOR_EACH_FIELD_BEGIN, iStart);
	        }
	        strRetVal += strText.Substring(iStart);
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ProcessForEachLineBlock:            Process the ForEachLine block
        /// </summary>
        /// <param name="strText">              Text to process</param>
        /// <param name="rs">                   Rowset</param>
        /// <returns>
        /// Processed text
        /// </returns>
        /// <remarks>
        /// ForEachLine     ::= '$$REPPP:FOR_EACH_LINE_BEGIN$$' {'$$REPPP:LINE_NUMBER$$' | ForEachField} '$$REPPP:FOR_EACH_LINE_END$$'
        /// ForEachField    ::= '$$REPPP:FOR_EACH_FIELD_BEGIN$$' {FieldTag} '$$REPPP:FOR_EACH_FIELD_END$$'
        /// FieldTag        ::= '$$REPPP:FIELD_LABEL' | '$$REPPP:FIELD_NAME$$' | '$$REPPP:FIELD_CONTROL$$' | '$REPPP:FIELD_VALUE'
        /// </remarks>
        //                                                                                             
        //********************************************************************************
        private string ProcessForEachLineBlock(string       strText,
	    									   RepPP.Rowset rs) {
            string          strRetVal = "";
            int 			iLineIndex;
            int             iStart;
            int             iEnd;

            iStart = 0;
            iEnd   = rs.LineCount;
            for (iLineIndex = iStart; iLineIndex < iEnd; iLineIndex++) {
                rs.SelectLine(iLineIndex, true, false);
                strRetVal += ProcessForEachField(strText, rs).Replace(TAG_LINE_NUMBER, iLineIndex.ToString().Trim());
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ProcessForEachLine:                 Process the ForEachLine clause
        /// </summary>
        /// <param name="strText">              Text to process</param>
        /// <param name="rs">                   Rowset</param>
        /// <returns>
        /// Processed text
        /// </returns>
        /// <remarks>
        /// ForEachLine     ::= '$$REPPP:FOR_EACH_LINE_BEGIN$$' {'$$REPPP:LINE_NUMBER$$' | ForEachField} '$$REPPP:FOR_EACH_LINE_END$$'
        /// ForEachField    ::= '$$REPPP:FOR_EACH_FIELD_BEGIN$$' {FieldTag} '$$REPPP:FOR_EACH_FIELD_END$$'
        /// FieldTag        ::= '$$REPPP:FIELD_LABEL' | '$$REPPP:FIELD_NAME$$' | '$$REPPP:FIELD_CONTROL$$' | '$REPPP:FIELD_VALUE' | ACTION_TAG
        /// </remarks>
        //                                                                                             
        //********************************************************************************
        private string ProcessForEachLine(string        strText,
	    							      RepPP.Rowset 	rs) {
            string      strRetVal = "";
            int 	    iPos;
            int 	    iStart = 0;

	        iPos = strText.IndexOf(TAG_FOR_EACH_LINE_BEGIN);
	        while (iPos != -1) {
	            strRetVal  += ProcessForEachField(strText.Substring(iStart, iPos - iStart), rs);
	            iPos       += TAG_FOR_EACH_LINE_BEGIN.Length;
	            iStart      = strText.IndexOf(TAG_FOR_EACH_LINE_END, iPos);
	            if (iStart == -1) {
                    throw new ParseException("No " + TAG_FOR_EACH_LINE_END + " matching " + TAG_FOR_EACH_LINE_BEGIN,
                                                 strText,
                                                 iPos);
	            }
	            strRetVal  += ProcessForEachLineBlock(strText.Substring(iPos, iStart - iPos), rs);
	            iStart     += TAG_FOR_EACH_LINE_END.Length;
	            iPos        = strText.IndexOf(TAG_FOR_EACH_LINE_BEGIN, iStart);
	        }
	        strRetVal += ProcessForEachField(strText.Substring(iStart), rs);
	        strRetVal = strRetVal.Replace(TAG_CHILD_NAME, HttpUtility.HtmlEncode(rs.RowsetDef.Name));
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ProcessForEachChildBlock:           Process the ForEachChild clause
        /// </summary>
        /// <param name="strText">              Text to process</param>
        /// <param name="rs">                   Rowset</param>
        /// <returns>
        /// Processed text
        /// </returns>
        /// <remarks>
        /// ForEachChild    ::= '$$REPPP:FOR_EACH_CHILD_BEGIN$$' {'$$REPPP:CHILD_NAME$$' | ForEachLine} '$$REPPP:FOR_EACH_CHILD_END$$'
        /// ForEachLine     ::= '$$REPPP:FOR_EACH_LINE_BEGIN$$' {'$$REPPP:LINE_NUMBER$$' | ForEachField} '$$REPPP:FOR_EACH_LINE_END$$'
        /// </remarks>
        //                                                                                             
        //********************************************************************************
        private string ProcessForEachChildBlock(string          strText,
	    										RepPP.Rowset    rs) {
	    	string              strRetVal = "";

            foreach (RepPP.Rowset ginstChild in rs.ChildRowsets) {
                strRetVal += ProcessForEachLine(strText, ginstChild);
            }
		    return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ProcessForEachChild:                Process the ForEachChild clause
        /// </summary>
        /// <param name="strText">              Text to process</param>
        /// <param name="rs">                   Rowset</param>
        /// <returns>
        /// Processed text
        /// </returns>
        /// <remarks>
        /// ForEachChild    ::= '$$REPPP:FOR_EACH_CHILD_BEGIN$$' {'$$REPPP:CHILD_NAME$$' | ForEachLine} '$$REPPP:FOR_EACH_CHILD_END$$'
        /// ForEachLine     ::= '$$REPPP:FOR_EACH_LINE_BEGIN$$' {'$$REPPP:LINE_NUMBER$$' | ForEachField} '$$REPPP:FOR_EACH_LINE_END$$'
        /// ForEachField    ::= '$$REPPP:FOR_EACH_FIELD_BEGIN$$' {FieldTag} '$$REPPP:FOR_EACH_FIELD_END$$'
        /// FieldTag        ::= '$$REPPP:FIELD_LABEL' | '$$REPPP:FIELD_NAME$$' | '$$REPPP:FIELD_CONTROL$$' | '$REPPP:FIELD_VALUE'
        /// </remarks>
        //                                                                                             
        //********************************************************************************
        private string ProcessForEachChild(string       strText,
                                           RepPP.Rowset rs) {
            string          strRetVal = "";
            int 			iPos;
            int 			iStart;

            iPos   = strText.IndexOf(TAG_FOR_EACH_CHILD_BEGIN);
            iStart = 0;
            while (iPos != -1) {
                strRetVal  += ProcessForEachLine(strText.Substring(iStart, iPos - iStart), rs);
                iPos       += TAG_FOR_EACH_CHILD_BEGIN.Length;
                iStart      = strText.IndexOf(TAG_FOR_EACH_CHILD_END, iPos);
                if (iStart == -1) {
                    throw new ParseException("No " + TAG_FOR_EACH_CHILD_END + " matching " + TAG_FOR_EACH_CHILD_BEGIN,
                                                 strText,
                                                 iPos);
                }
                strRetVal  += ProcessForEachChildBlock(strText.Substring(iPos, iStart - iPos), rs);
                iStart     += TAG_FOR_EACH_CHILD_END.Length;
                iPos        = strText.IndexOf(TAG_FOR_EACH_CHILD_BEGIN, iStart);
            }
            strRetVal += ProcessForEachLine(strText.Substring(iStart), rs);
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ProcessFile:                        Process the HTML file
        /// </summary>
        /// <param name="strText">              Text to process</param>
        /// <param name="rs">                   Rowset</param>
        /// <returns>
        /// Processed text
        /// </returns>
        /// <remarks>
        /// Tag             ::= ForEachChild | ForEachLine | ForEachField | GlobalTag | IncludeTag
        /// </remarks>
        //                                                                                             
        //********************************************************************************
        private string ProcessFile(string       strText,
	    						   RepPP.Rowset rs) {
	    	string          strRetVal;
            
	        strRetVal = ProcessForEachChild(strText, rs);
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ProcessIfChoiceChecked:             Process the If the Choice is checked
        /// </summary>
        /// <param name="strText">              Text to process</param>
        /// <param name="strChoiceValue">       Current Choice Value</param>
        /// <param name="fldDef">               Field Definition to check</param>
        /// <returns>
        /// Processed text
        /// </returns>
        //                                                                                             
        //********************************************************************************
        private string ProcessIfChoiceChecked(string            strText,
                                              string            strChoiceValue,
                                              RepPP.FieldDef 	fldDef) {
            string  strIFResult   = "";
            string  strElseResult = "";
            int     iBegin;
            int     iElse;
            int     iEnd;
	    								
            iBegin = strText.IndexOf(TAG_IF_CHOICE_CHECKED_BEGIN);
            while (iBegin != -1) {
                iEnd = strText.IndexOf(TAG_IF_CHOICE_CHECKED_END, iBegin);
                if (iEnd == -1) {
                    throw new ParseException("No " + TAG_IF_CHOICE_CHECKED_END + " matching " + TAG_IF_CHOICE_CHECKED_BEGIN,
                                                 strText,
                                                 iBegin);
                }
                iElse = strText.IndexOf(TAG_ELSE_CHOICE_CHECKED, iBegin);                
                if (iElse != -1) {
                    strIFResult   = strText.Substring(iBegin + TAG_IF_CHOICE_CHECKED_BEGIN.Length, iElse - TAG_ELSE_CHOICE_CHECKED.Length - iBegin - 4);
                    strElseResult = strText.Substring(iElse + TAG_ELSE_CHOICE_CHECKED.Length, iEnd - TAG_IF_CHOICE_CHECKED_END.Length - iElse - 2);
                } else {
                    strIFResult = strText.Substring(iBegin + TAG_IF_CHOICE_CHECKED_BEGIN.Length, iEnd - TAG_IF_CHOICE_CHECKED_END.Length - iBegin - 2);
                }
                strText = strText.Remove(iBegin, (iEnd + TAG_IF_CHOICE_CHECKED_END.Length) - iBegin + 2);
                if (fldDef.Value == strChoiceValue) {
                    strText = strText.Insert(iBegin, strIFResult);
                } else {
                    strText = strText.Insert(iBegin, strElseResult);
                }                
                iBegin = strText.IndexOf(TAG_IF_CHOICE_CHECKED_BEGIN, iBegin);
            }         
            return(strText);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ProcessForEachChoiceBlock:          Process the HTML file
        /// </summary>
        /// <param name="strText">              Text to process</param>
        /// <param name="fldDef">               Field Definition</param>
        /// <returns>
        /// Processed text
        /// </returns>
        //                                                                                             
        //********************************************************************************
        private string ProcessForEachChoiceBlock(string         strText,
                                                 RepPP.FieldDef fldDef) { 
            string  strRetVal = "";
            string  strChoiceList;
            string  strChoiceValue;
            int     iCount;

            iCount = fldDef.ChoiceList.Count;
            for (int i = 0; i < iCount; i++) {
                strChoiceValue = fldDef.ChoiceList.GetValue(i);
                strChoiceList  = ProcessIfChoiceChecked(strText, strChoiceValue, fldDef);
                strChoiceList  = strChoiceList.Replace(TAG_CHOICELIST_CODE,  HttpUtility.HtmlEncode(fldDef.ChoiceList.GetCode(i)));
                strChoiceList  = strChoiceList.Replace(TAG_CHOICELIST_VALUE, HttpUtility.HtmlEncode(strChoiceValue));
                strChoiceList  = strChoiceList.Replace(TAG_CHOICELIST_DESC,  HttpUtility.HtmlEncode(fldDef.ChoiceList.GetDesc(i)));
                strChoiceList  = strChoiceList.Replace(TAG_FIELD_NAME,       HttpUtility.HtmlEncode(fldDef.Name));
                strRetVal     += strChoiceList;
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ProcessForEachChoice:               Process the HTML file
        /// </summary>
        /// <param name="strText">              Text to process</param>
        /// <param name="fldDef">               Field Definition</param>
        /// <returns>
        /// Processed text
        /// </returns>
        //                                                                                             
        //********************************************************************************
        private string ProcessForEachChoice(string          strText,
                                            RepPP.FieldDef  fldDef) { 
            string          strRetVal       = "";         
            int             iPos;
            int 			iStart;

            iStart = 0;
            iPos   = strText.IndexOf(TAG_FOR_EACH_CHOICE_BEGIN);
	        while (iPos != -1) {
	            strRetVal += strText.Substring(iStart, iPos - iStart);
	            iPos      += TAG_FOR_EACH_CHOICE_BEGIN.Length;
	            iStart     = strText.IndexOf(TAG_FOR_EACH_CHOICE_END, iPos);
	            if (iStart == -1) {
                    throw new ParseException("No " + TAG_FOR_EACH_CHOICE_END + " matching " + TAG_FOR_EACH_CHOICE_BEGIN,
                                                 strText,
                                                 iPos);
	            }
	            strRetVal += ProcessForEachChoiceBlock(strText.Substring(iPos, iStart - iPos), fldDef);
	            iStart    += TAG_FOR_EACH_CHOICE_END.Length;
	            iPos       = strText.IndexOf(TAG_FOR_EACH_CHOICE_BEGIN, iStart);
	        }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ProcessChoiceList:                  Process choice list
        /// </summary>
        /// <param name="strText">              Text to process</param>
        /// <param name="rs">                   Rowset</param>
        /// <returns>
        /// Processed text
        /// </returns>
        //                                                                                             
        //********************************************************************************
        private string ProcessChoiceList(string         strText,
                                         RepPP.Rowset   rs) {            
            RepPP.FieldDef  fldDef;
            string          strFieldName;
            int             iPos;
            int             iPosBegin;
            int 			iStart;

            iPos   = strText.IndexOf(TAG_CHOICELIST_BEGIN);
            while (iPos != -1) {
                iPosBegin       = iPos;
                iPos            = iPos + TAG_CHOICELIST_BEGIN.Length;
                strFieldName    = ExtractString(strText, ref iPos);
                fldDef          = rs.RowsetDef.FieldDefs[strFieldName];
                if (fldDef == null) {
                    throw new ParseException("Field '" + strFieldName + "' don't exist", strText, iPos);
                }
                iStart  = strText.IndexOf(TAG_CHOICELIST_END, iPos);
                if (iStart == -1) {
                    throw new ParseException("No " + TAG_CHOICELIST_END + " matching " + TAG_CHOICELIST_BEGIN,
                                                 strText,
                                                 iPos);
                }
                strText = strText.Substring(0, iPosBegin)                                       +
                          ProcessForEachChoice(strText.Substring(iPos , iStart - iPos), fldDef) +
                          strText.Substring(iStart + TAG_CHOICELIST_END.Length);
                iPos    = strText.IndexOf(TAG_CHOICELIST_BEGIN, iStart);
            }
            return(strText);
        }

        //********************************************************************************
        //
        /// <summary>
        /// FormGlobalFilter:                   Replace global tag
        /// </summary>
        /// <param name="strText">              Text to process</param>
        /// <param name="rs">                   Rowset to process</param>
        /// <returns>
        /// Processed text
        /// </returns>
        /// <remarks>
        /// TAG_FORM_NAME         = "$$REPPP:FORM_NAME$$"
        /// TAG_CARD_NAME		  = "$$REPPP:ROWSET_TREE_NAME$$"
        /// TAG_GROUP_NAME        = "$$REPPP:ROWSET_NAME$$"
        /// TAG_VALID_CODE        = "$$REPPP:VALID_CODE$$"
        /// </remarks>
        //
        //********************************************************************************
        private string FormGlobalFilter(string          strText,
	    							    RepPP.Rowset    rs) {
			string  strRSName;
            string  strRSTreeName;
            string  strNodeName;
            string  strTitle;
            
			strRSName      = rs.RowsetDef.Name;
            strRSTreeName  = rs.RowsetTreeDefNode.RowsetTreeDef.Name;
            strNodeName    = strRSTreeName + "." + strRSName;
            strTitle       = GetText("FormTitle", strRSName);
            if (strTitle.Length == 0) {
                strTitle = strRSName;
            }
            strText = strText.Replace(TAG_FORM_NAME,        HttpUtility.HtmlEncode(strTitle));
			strText = strText.Replace(TAG_CARD_NAME,        HttpUtility.HtmlEncode(strRSTreeName));
            strText = strText.Replace(TAG_ROWSET_TREE_NAME, HttpUtility.HtmlEncode(strRSTreeName));
            strText = strText.Replace(TAG_GROUP_NAME,       HttpUtility.HtmlEncode(strRSName));
            strText = strText.Replace(TAG_ROWSET_NAME,      HttpUtility.HtmlEncode(strRSName));
            strText = strText.Replace(TAG_VALID_CODE,       m_strGeneratedCode);
            strText = ProcessTestBlocks(strText, strRSName);
            return(strText);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ReadInclude:                    Reads the include file
        /// </summary>
        /// <param name="strFileName">      Original file name</param>
        /// <param name="strDir">           Base directory</param>
        /// <param name="strText">          Input text to scan for input text</param>
        /// <returns>
        /// New text with included file
        /// </returns>
        //
        //********************************************************************************
        private string ReadInclude(string strFileName, string strDir, string strText) {
            int                     iTagLen = TAG_INCLUDE.Length;
            int                     iStart;
            int                     iPos;
            string                  strIncFileName;
            string                  strIncludeText;

            if (strDir.Length == 0 || strDir[strDir.Length-1] != '\\') {
                strDir += "\\";
            }
            iStart = strText.IndexOf(TAG_INCLUDE);
            while (iStart != -1) {
                iPos           = iStart + iTagLen;
                strIncFileName = ExtractString(strText, ref iPos);
                if (strIncFileName.Length != 0) {
                    strIncFileName = strDir + strIncFileName;
                    strIncludeText = "";
                    try {
                        strIncludeText = ReadFile(strDir, strIncFileName, false);
                    } finally {
                        strText = strText.Substring(0, iStart) +
                                  strIncludeText +
                                  strText.Substring(iPos);
                    }
                }
                iStart = strText.IndexOf(TAG_INCLUDE);
            }
            return(strText);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ReadFile:                       Reads the content of the file
        /// </summary>
        /// <param name="strDir">           Base directory</param>
        /// <param name="strFileName">      File to read</param>
        /// <param name="bIgnoreNotFound">  Ignore the file not find error</param>
        /// <returns>
        /// File content
        /// </returns>
        //
        //********************************************************************************
        private string ReadFile(string    strDir,              
                                string    strFileName,
                                bool      bIgnoreNotFound) {

            string                  strRetVal = "";
            System.IO.StreamReader  stream;            

            stream = null;
            try {
				if (DirExists(strFileName)) {
					stream = System.IO.File.OpenText(strFileName);
				} else {
					stream = System.IO.File.OpenText(strDir + strFileName);
                }
                using(stream) {
                    strRetVal = stream.ReadToEnd();
                    try {
                        strRetVal = ReadInclude(strFileName, strDir, strRetVal);
                    } catch {
                    }
                }
            } catch (System.Exception e) {
                if (!bIgnoreNotFound) {
                    throw new ParseException("Unable to read the file " + strFileName + " - " + e.Message);
                }
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ParseTemplate:                      Parse a template
        /// </summary>
        /// <param name="rs">                   Rowset to process</param>
        /// <param name="strTemplateText">      Template text to use</param>
        /// <returns>
        /// Result
        /// </returns>
        //
        //********************************************************************************
        public string ParseTemplate(RepPP.Rowset    rs,
                                    string          strTemplateText) {
            string                      strRetVal;
            RepPP.RowsetDef             rsDef;

            rsDef   = rs.RowsetDef;
            foreach (RepPP.FieldDef fld in rsDef.FieldDefs) {
                if (fld.IsVisible) {
                    m_codeGen.AddFieldObjToCodeGeneration(fld, rsDef.Name, fld.Name);
                }
            }
            m_strGeneratedCode  = m_codeGen.GenerateCode(RepPP.HTMLScriptType.sdHTMLJScript,
                                                         RepPP.HTMLGenOption.sdHTMLGenFormValidation);
            strRetVal           = FormGlobalFilter(strTemplateText, rs);
            strRetVal           = ProcessChoiceList(strRetVal, rs);
            strRetVal           = ProcessFile(strRetVal, rs);
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// Parser exception class
        /// </summary>
        //
        //********************************************************************************
        public class ParseException : Exception {
            private string                  m_strText;          // Text in fault
            private int                     m_iPos;             // Error position in the text
            private int                     m_iPosInLine;       // Error position in the line
            private string                  m_strLineInError;   // Line in fault

            //********************************************************************************
            //
            /// <summary>
            /// ctor:                               Class constructor
            /// </summary>
            /// <param name="str">                  Error message</param>
            /// <param name="strText">              Text in error</param>
            /// <param name="iPos">                 Position of the error in the text</param>
            //
            //********************************************************************************
            public ParseException(string str, string strText, int iPos) : base(str) {
                int     iEndLinePos;
                int     iLastLinePos;

                m_strText   = strText;    
                if (strText.Length == 0) {
                    m_iPosInLine        = 0;
                    m_iPos              = 0;
                    m_strLineInError    = "";
                } else {
                    if (iPos >= m_strText.Length) {
                        iPos = m_strText.Length - 1;
                    }
                    m_iPos       = iPos;
                    iLastLinePos = strText.LastIndexOf('\r',  m_iPos);
                    if (iLastLinePos == -1) {
                        iLastLinePos = 0;
                    } else {
                        if (iLastLinePos + 1 < strText.Length) {
                            iLastLinePos++;
                        }
                        if (iLastLinePos + 1 < strText.Length && strText[iLastLinePos] == '\n') {
                            iLastLinePos++;
                        }
                    }
                    iEndLinePos = strText.IndexOf('\r', iLastLinePos);
                    if (iEndLinePos == -1) {
                        iEndLinePos = strText.Length;
                    }
                    m_strLineInError = strText.Substring(iLastLinePos, iEndLinePos - iLastLinePos);
                    m_iPosInLine     = m_iPos - iLastLinePos;                    
                }
            }

            //********************************************************************************
            //
            /// <summary>
            /// ctor:                               Class constructor
            /// </summary>
            /// <param name="str">                  Error message</param>
            //
            //********************************************************************************
            public ParseException(string str) : this(str, "Internal", 0) {
            }

            //********************************************************************************
            //
            /// <summary>
            /// ToString:                           Convert the error in a message
            /// </summary>
            /// <returns>
            /// Error message in HTML
            /// </returns>
            //
            //********************************************************************************
            public override string ToString() {
                string  strRetVal;

                strRetVal = System.Web.HttpUtility.HtmlEncode(Message) + "<br>" +
                            System.Web.HttpUtility.HtmlEncode(m_strLineInError) + "<br>" + "".PadRight(m_iPos, ' ') + "^<br>";
                return(strRetVal);                        
            }
        } // End Class SCFWParseException

    } // End Class GenHTML

}// End Namespace
