﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using System.Collections;
using System.ComponentModel;
using System.Web;
using System.Security.Permissions;
using System.Web.UI.Design;
using System.ComponentModel.Design;

namespace RepPP.Toolkit.Web {

    //********************************************************************************
    //
    /// <summary>
    /// Rep++ DataSource Control
    /// </summary>
    //
    //********************************************************************************
    [DisplayName("Rep++ Rowset"), Description("Provides access to data contained in a Rep++ Rowset object"),System.Drawing.ToolboxBitmap(typeof(RowsetDataSourceCtl)), Designer(typeof(RowsetDataSourceCtlDesigner)), AspNetHostingPermission(SecurityAction.Demand, Level = AspNetHostingPermissionLevel.Minimal)]
    public class RowsetDataSourceCtl : DataSourceControl, RowsetTreeDefNodeControlUtil.IRowsetTreeDefNodeControl {
        /// <summary>Rep++ Default and Only DataSourceView</summary>
        private RowsetDataSourceCtlView   m_view = null;

        //********************************************************************************
        //
        /// <summary>
        /// Class Constructor
        /// </summary>
        //
        //********************************************************************************
        public RowsetDataSourceCtl(): base() {
        }

        //********************************************************************************
        //
        /// <summary>
        /// Dispose
        /// </summary>
        //
        //********************************************************************************
        public override void Dispose() {
            if (m_view != null) {
                m_view.Dispose();
                m_view = null;
            }
            base.Dispose();
        }

        //********************************************************************************
        //
        /// <summary>
        /// Default DataSourceView object
        /// </summary>
        //
        //********************************************************************************
        private RowsetDataSourceCtlView View {
            get {
                return ((RowsetDataSourceCtlView)this.GetView(String.Empty));
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// The embedded DataSource object associated with the DataSource Control
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public DataSourceRowset DataSource {
            get {
                return (View.DataSource);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// WHERE clause to use to autofill the datasource
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("WHERE clause use to auto-fill the data source")]
        [DefaultValue("")]
        public virtual string WhereClause {
            get {
                return (View.WhereClause);
            }
            set {
                if (View.WhereClause != value) {
                    View.WhereClause = value;
                    RaiseDataSourceChangedEvent(EventArgs.Empty);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Use to auto-fill the data source
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Auto fill the data source")]
        [DefaultValue(false)]
        public virtual bool AutoFill {
            get {
                return (View.AutoFill);
            }
            set {
                if (View.AutoFill != value) {
                    View.AutoFill = value;
                    RaiseDataSourceChangedEvent(EventArgs.Empty);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Show description of choice if true, code if false
        /// </summary>
        //
        //********************************************************************************
        [Browsable(true)]
        [Category("Behavior")]
        [Description("How to display a choice value")]
        public RepPP.Toolkit.DataSourceRowset.ShowDescriptionE ShowDescription {
            get {
                return (View.ShowDescription);
            }
            set {
                if (View.ShowDescription != value) {
                    View.ShowDescription = value;
                    RaiseDataSourceChangedEvent(EventArgs.Empty);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Returns the colums use for sort (or "" if none)
        /// </summary>
        //
        //********************************************************************************
        [Browsable(false)]
        [DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        public string Sort {
            get {
                return (View.Sort);
            }
            set {
                if (View.Sort != value) {
                    View.Sort = value;
                    RaiseDataSourceChangedEvent(EventArgs.Empty);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Determines if the source allow insertion of rows
        /// </summary>
        //
        //********************************************************************************
        [Browsable(true)]
        [Category("Behavior")]
        [Description("Allow the insertion of new rows")]
        public bool AllowNew {
            get {
                return (View.AllowNew);
            }
            set {
                if (View.AllowNew != value) {
                    View.AllowNew = value;
                    RaiseDataSourceChangedEvent(EventArgs.Empty);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Determines if the sort will be performed locally or using the internal list
        /// </summary>
        //
        //********************************************************************************
        [Browsable(true)]
        [Category("Behavior")]
        [Description("Sort the Rowset instead of the internal list")]
        public bool SortRowset {
            get {
                return (View.SortRowset);
            }
            set {
                if (View.SortRowset != value) {
                    View.SortRowset = value;
                    RaiseDataSourceChangedEvent(EventArgs.Empty);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Determines if the source allow deletion of rows
        /// </summary>
        //
        //********************************************************************************
        [Browsable(true)]
        [Category("Behavior")]
        [Description("Allow removal of rows")]
        public bool AllowRemove {
            get {
                return (View.AllowRemove);
            }
            set {
                if (View.AllowRemove != value) {
                    View.AllowRemove = value;
                    RaiseDataSourceChangedEvent(EventArgs.Empty);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Determines if the source allow modification of items
        /// </summary>
        //
        //********************************************************************************
        [Browsable(true)]
        [Category("Behavior")]
        [Description("Allow edition of a row")]
        public bool AllowEdit {
            get {
                return (View.AllowEdit);
            }
            set {
                if (View.AllowEdit != value) {
                    View.AllowEdit = value;
                    RaiseDataSourceChangedEvent(EventArgs.Empty);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Return a strongly typed view for the current data source control.
        /// </summary>
        /// <param name="viewName">     View Name</param>
        /// <returns>
        /// Strongly-Typed View
        /// </returns>
        //
        //********************************************************************************
        protected override DataSourceView GetView(string viewName) {
            if (m_view == null) {
                m_view = new RowsetDataSourceCtlView(this, DesignMode);
            }
            return (m_view);
        }

        //********************************************************************************
        //
        /// <summary>
        /// The ListSourceHelper class calls GetList, which
        /// calls the DataSourceControl.GetViewNames method.
        /// Override the original implementation to return
        /// a collection of one element, the default view name.
        /// </summary>
        /// <returns>
        /// List of names of the Views supported by a DataSourceControl
        /// </returns>
        //
        //********************************************************************************
        protected override ICollection GetViewNames() {
            return (new string[] { RowsetDataSourceCtlView.DefaultViewName });
        }

        #region IRowsetTreeDefNodeControl Members

        //********************************************************************************
        //
        /// <summary>
        /// Gets the Rep++ Application
        /// </summary>
        //
        //********************************************************************************
        [Browsable(false)]
        [DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        public Application Application {
            get {
                return (DataSource.Application);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public void TriggerComponentChange() {
            DataSource.TriggerComponentChange();
        }

        //********************************************************************************
        //
        /// <summary>
        /// Name of the RowsetTreeDefNode attached to the source
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [PersistenceMode(PersistenceMode.Attribute)]        
        [Editor("RepPP.Toolkit.Designer.RowsetTreeDefNodeControlDesigner+NodeEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]    
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Name of the ChildRowsetDef attached to the source")]
        public virtual RowsetTreeDefNodeFullName RowsetTreeDefNodeFullName {
            get {
                return (View.RowsetTreeDefNodeFullName);
            }
            set {
                if (View.RowsetTreeDefNodeFullName != value) {
                    View.RowsetTreeDefNodeFullName = value;
                    RaiseDataSourceChangedEvent(EventArgs.Empty);
                }
            }
        }
        #endregion

        #region IRepControl Members

        //********************************************************************************
        //
        /// <summary>
        /// Called when Rep++ is reinitialized in design mode
        /// </summary>
        //
        //********************************************************************************
        public void RepPPReInit() {
            DataSource.RepPPReInit();
        }

        //********************************************************************************
        //
        /// <summary>
        /// Called when Rep++ is reset in design mode
        /// </summary>
        //
        //********************************************************************************
        public void RepPPReset() {
            DataSource.RepPPReset();
        }
        #endregion
    } // Class RowsetDataSourceCtl

    //********************************************************************************
    //
    /// <summary>
    /// The RowsetDataSourceCtlView class encapsulates the
    /// capabilities of the RowsetDataSourceCtl data source control.
    /// </summary>
    //
    //********************************************************************************
    public class RowsetDataSourceCtlView : DataSourceView, IDisposable {
        /// <summary>RowsetDataSourceCtl supports one view only</summary>
        internal static string            DefaultViewName = "RepView";
        /// <summary>The embedded DataSource object</summary>
        private DataSourceRowset          m_dsRowset      = null;

        //********************************************************************************
        //
        /// <summary>
        /// Class Constructor
        /// </summary>
        /// <param name="owner">        Owner IDataSource</param>
        /// <param name="bDesignMode">  True to indicate DesignMode</param>
        //
        //********************************************************************************
        public RowsetDataSourceCtlView(IDataSource owner, bool bDesignMode) : base(owner, DefaultViewName) {
            m_dsRowset                           = new DataSourceRowset(owner, false);
            m_dsRowset.AllowEdit                 = true;
            m_dsRowset.AllowNew                  = true;
            m_dsRowset.AllowRemove               = true;
            m_dsRowset.AutoFill                  = false;
            m_dsRowset.RowsetTreeDefNodeFullName = new RepPP.Toolkit.RowsetTreeDefNodeFullName("", "");
            m_dsRowset.ShowDescription           = RepPP.Toolkit.DataSourceRowset.ShowDescriptionE.Always;
            m_dsRowset.SortRowset                = true;
            m_dsRowset.WhereClause               = String.Empty;
        }

        //********************************************************************************
        //
        /// <summary>
        /// Deletes the embedded DataSource
        /// </summary>
        //
        //********************************************************************************
        public void Dispose() {
            m_dsRowset.Dispose();
            m_dsRowset = null;
        }

        //********************************************************************************
        //
        /// <summary>
        /// The embedded DataSource object
        /// </summary>
        //
        //********************************************************************************
        public virtual DataSourceRowset DataSource {
            get {
                return(m_dsRowset);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Gets a list of data from the underlying data storage.
        /// </summary>
        /// <param name="selectArgs">   Used to request operations on the data beyond basic data retrieval.</param>
        /// <returns>
        /// An IEnumerable list of data from the underlying data storage.
        /// </returns>
        //
        //********************************************************************************
        protected override IEnumerable ExecuteSelect(DataSourceSelectArguments selectArgs) {
            return((IEnumerable)m_dsRowset);
        }

        //********************************************************************************
        //
        /// <summary>
        /// Gets a value indicating whether the DataSourceView associated with the current DataSourceControl
        /// supports the ExecuteDelete operation.
        /// </summary>
        //
        //********************************************************************************
        public override bool CanDelete {
            get {
                return(m_dsRowset.AllowRemove);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Performs a delete operation on the list of data that the DataSourceView object represents.
        /// </summary>
        /// <param name="keys">     An IDictionary of object or row keys to be deleted by the <see cref="M:System.Web.UI.DataSourceView.ExecuteDelete(System.Collections.IDictionary,System.Collections.IDictionary)"></see> operation.</param>
        /// <param name="values">   An IDictionary of name/value pairs that represent data elements and their original values.</param>
        /// <returns>
        /// The number of items that were deleted from the underlying data storage.
        /// </returns>
        //
        //********************************************************************************
        protected override int ExecuteDelete(IDictionary keys, IDictionary values) {
            throw new NotSupportedException();
        }

        //********************************************************************************
        //
        /// <summary>
        /// Gets a value indicating whether the DataSourceView associated with the current DataSourceControl
        /// supports the ExecuteInsert operation.
        /// </summary>
        //
        //********************************************************************************
        public override bool CanInsert {
            get {
                return(m_dsRowset.AllowNew);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Performs an insert operation on the list of data that the DataSourceView object represents.
        /// </summary>
        /// <param name="values">   An IDictionary of name/value pairs used during an insert operation.</param>
        /// <returns>
        /// The number of items that were inserted into the underlying data storage.
        /// </returns>
        //
        //********************************************************************************
        protected override int ExecuteInsert(IDictionary values) {
            throw new NotSupportedException();
        }

        //********************************************************************************
        //
        /// <summary>
        /// Gets a value indicating whether the DataSourceView associated with the current DataSourceControl
        /// supports the ExecuteUpdate operation.
        /// </summary>
        //
        //********************************************************************************
        public override bool CanUpdate {
            get {
                return(m_dsRowset.AllowEdit);
            }
        }

    
        //********************************************************************************
        //
        /// <summary>
        /// Performs an update operation on the list of data that the DataSourceView object represents.
        /// </summary>
        /// <param name="keys">     An IDictionary of object or row keys to be updated by the update operation.</param>
        /// <param name="values">   An IDictionary of name/value pairs that represent data elements and their new values.</param>
        /// <param name="oldValues">An IDictionary of name/value pairs that represent data elements and their original values.</param>
        /// <returns>
        /// The number of items that were updated in the underlying data storage.
        /// </returns>
        //
        //********************************************************************************
        protected override int ExecuteUpdate(IDictionary keys, IDictionary values, IDictionary oldValues) {
            throw new NotSupportedException();
        }

        //********************************************************************************
        //
        /// <summary>
        /// Name of the RowsetTreeDefNode attached to the source
        /// </summary>
        //
        //********************************************************************************
        public virtual RowsetTreeDefNodeFullName RowsetTreeDefNodeFullName {
            get {
                return (m_dsRowset.RowsetTreeDefNodeFullName);
            }
            set {
                m_dsRowset.RowsetTreeDefNodeFullName = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// WHERE clause to use to autofill the datasource
        /// </summary>
        //
        //********************************************************************************
        public virtual string WhereClause {
            get {
                return (m_dsRowset.WhereClause);
            }
            set {
                m_dsRowset.WhereClause = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Use to auto-fill the data source
        /// </summary>
        //
        //********************************************************************************
        public virtual bool AutoFill {
            get {
                return (m_dsRowset.AutoFill);
            }
            set {
                m_dsRowset.AutoFill = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Show description of choice if true, code if false
        /// </summary>
        //
        //********************************************************************************
        public RepPP.Toolkit.DataSourceRowset.ShowDescriptionE ShowDescription {
            get {
                return (m_dsRowset.ShowDescription);
            }
            set {
                m_dsRowset.ShowDescription = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Returns the colums use for sort (or "" if none)
        /// </summary>
        //
        //********************************************************************************
        public string Sort {
            get {
                return (m_dsRowset.Sort);
            }
            set {
                m_dsRowset.Sort = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Determines if the source allow insertion of rows
        /// </summary>
        //
        //********************************************************************************
        public bool AllowNew {
            get {
                return (m_dsRowset.AllowNew);
            }
            set {
                m_dsRowset.AllowNew = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Determines if the sort will be performed locally or using the internal list
        /// </summary>
        //
        //********************************************************************************
        public bool SortRowset {
            get {
                return (m_dsRowset.SortRowset);
            }
            set {
                m_dsRowset.SortRowset = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AllowRemove:        Determines if the source allow deletion of rows
        /// </summary>
        //
        //********************************************************************************
        public bool AllowRemove {
            get {
                return (m_dsRowset.AllowRemove);
            }
            set {
                m_dsRowset.AllowRemove = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AllowEdit:          Determines if the source allow modification of items
        /// </summary>
        //
        //********************************************************************************
        public bool AllowEdit {
            get {
                return (m_dsRowset.AllowEdit);
            }
            set {
                m_dsRowset.AllowEdit = value;
            }
        }
    } // Class RowsetDataSourceCtlView

    //********************************************************************************
    //
    /// <summary>
    /// RowsetDataSourceCtl Designer Class
    /// </summary>
    //
    //********************************************************************************
    public class RowsetDataSourceCtlDesigner : DataSourceDesigner {

        //********************************************************************************
        //
        /// <summary>
        /// Class Constructor
        /// </summary>
        //
        //********************************************************************************
        public RowsetDataSourceCtlDesigner() : base(){
        }

        //********************************************************************************
        //
        /// <summary>
        /// Called to notify the designer that the component has changed. so that we update
        /// the design-time Html but most importantly to notify the controls attached to
        /// the datasource to update their design time html
        /// </summary>
        /// <param name="sender">   The RowsetDataSourceCtl Control</param>
        /// <param name="ce">       Event Arguments</param>
        //
        //********************************************************************************
        public override void OnComponentChanged(object sender, System.ComponentModel.Design.ComponentChangedEventArgs ce) {
            base.OnComponentChanged(sender, ce);
            this.UpdateDesignTimeHtml();
            this.OnDataSourceChanged(EventArgs.Empty);
       }
    } // Class RowsetDataSourceCtlDesigner
} // Namespace RepPP.ToolKit.Web
