using System;
using RepPP.Toolkit.Window;

namespace RepPP.Toolkit.Window.Ext {
	/// <summary>
	/// Implement the BarCmd and BarCmdHolder classes. These classes derived
	/// from the corresponding base class in ToolkitExt (Cmd and CmdHolder).
	/// They add support for DotNetBar menu.
	/// </summary>
	public class BarCmd : Cmd {

        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        /// <param name="strCmdName">   Unique command name</param>
        /// <param name="del">          Delegate to execute the command</param>
        /// <param name="eState">       Original state of the command</param>
        //
        //*********************************************************
        public BarCmd(string strCmdName, CmdHolder.OnCommand del, CmdHolder.CmdStateE eState) : 
                base(strCmdName, del, eState) {
        }

        //*********************************************************
        //
        /// <summary>
        /// AddHandlerv:                Add event handler for the trigger
        /// </summary>
        /// <param name="obj">          Trigger object</param>
        //
        //*********************************************************
        protected override void AddHandler(object obj) {
            if (obj is DevComponents.DotNetBar.ButtonItem) {
                ((DevComponents.DotNetBar.ButtonItem)obj).Click += m_delOnSelect;
            } else {
                base.AddHandler(obj);
            } 
        }

        //*********************************************************
        //
        /// <summary>
        /// RemoveTrigger:              Remove a trigger to the command
        /// </summary>
        /// <param name="obj">          Trigger to remove of the command</param>
        //
        //*********************************************************
        public override void RemoveTrigger(object obj) {
            if (obj is DevComponents.DotNetBar.ButtonItem) {
                ((DevComponents.DotNetBar.ButtonItem)obj).Click -= m_delOnSelect;
            }
            base.RemoveTrigger(obj);
        }
    }

    //*********************************************************
    //
    /// <summary>
	/// Superclass SCCmdHolder to support event generated by the DotNetBar
	/// </summary>
	//
    //*********************************************************
    public class SCBarCmdHolder : CmdHolder
	{

        //*********************************************************
        //
        /// <summary>
        /// AddCmd:                         Add a new command.
        /// </summary>
        /// <param name="strCmdName">       Command name</param>
        /// <param name="del">              Delegate to execute the command</param>
        /// <param name="eState">           Original state</param>
        //
        //*********************************************************
        public override void AddCmd(string strCmdName, CmdHolder.OnCommand del, CmdHolder.CmdStateE eState) {
            if (FindCommand(strCmdName) != null) {
                throw new System.ApplicationException("The command '" + strCmdName + "' already defined");
            }
            CheckState(strCmdName, ref eState);
            m_arrCmd.Add(new BarCmd(strCmdName, del, eState));
        }
    }
}
