using System;
using RepPP.Toolkit.Window;
using DevComponents.DotNetBar;

namespace RepPP.Toolkit.Window.Ext {
    //******************************************************************        
    //
    /// <summary>
    /// This base class implements controls text translator of a form which
    /// contains .Net Bar components.
    /// </summary>
    //
    //******************************************************************        
    public class BarControlTextTranslator : ControlTextTranslator {

        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        /// <param name="app">          Rep++ application object</param>
        /// <param name="arrAtomName">  Array of atoms</param>
        //
        //*********************************************************        
        public BarControlTextTranslator(RepPP.IApplication app, string[] arrAtomName) : base(app, arrAtomName) {
        }

        //*********************************************************
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        /// <param name="app">              Rep++ application object</param>
        /// <param name="arrAtomName">      Array of atoms</param>
        /// <param name="fatherTranslator"> Father translator</param>
        //
        //*********************************************************        
        public BarControlTextTranslator(RepPP.IApplication app,
                                        string[] arrAtomName,
                                        ControlTextTranslator fatherTranslator) : base(app, arrAtomName, fatherTranslator) {
        }

        //*********************************************************
        //
        /// <summary>
        /// SetComponentInfo:          Set component info (Text, Tooltip)
        /// </summary>
        /// <param name="obj">         Object</param>
        /// <param name="strObjName">  Object name</param>
        /// <param name="toolTip">     Tooltip component</param>
        //
        //*********************************************************
        protected override void SetComponentInfo(object obj, string strObjName, System.Windows.Forms.ToolTip toolTip) {
            DotNetBarManager                    barManager;
            Bar                                 barObj;
            DevComponents.DotNetBar.TabStrip    tabsItemComponent;

            if (obj is DevComponents.DotNetBar.TabStrip) {
                tabsItemComponent = obj as DevComponents.DotNetBar.TabStrip;
                foreach (TabItem tab in tabsItemComponent.Tabs) {
                    base.SetComponentInfo(tab as object, tab.Name, toolTip);
                }
            }
            if (obj is DotNetBarManager) {
                barManager = obj as DotNetBarManager;
                foreach (Bar bar in barManager.Bars) {
                    for (int i = 0; i < bar.Items.Count; i++) {
                        SetItemText(bar.Items[i], toolTip);
                    }
                }
            } else if (obj is Bar) {
                barObj = obj as Bar;
                for (int i = 0; i < barObj.Items.Count; i++) {
                    SetItemText(barObj.Items[i], toolTip);
                }
                base.SetComponentInfo(obj, strObjName, toolTip);
            } else {
                base.SetComponentInfo(obj, strObjName, toolTip);
            }
        }

        //**************************************************************************
        //
        /// <summary>
        /// SetItemText:         Set the text of the baseItem and its childs
        /// </summary>
        /// <param name="bi">       Button item</param>
        /// <param name="toolTip">  ToolTip Component</param>
        //
        //**************************************************************************
        private void SetItemText(BaseItem bi, System.Windows.Forms.ToolTip toolTip) {
            SetComponentInfo(bi, bi.Name, toolTip);
            foreach (BaseItem biChild in bi.SubItems) {
                SetItemText(biChild, toolTip);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetText:                    Set the text property of an object
        /// </summary>
        /// <param name="obj">          Object</param>
        /// <param name="strItemText">  Text</param>
        //
        //*********************************************************
        protected override void SetText(Object obj, string strItemText) {
            if (obj is BaseItem) {
                ((BaseItem)obj).Text = strItemText;
            } else if (obj is TabItem) {
                ((TabItem)obj).Text = strItemText;
            } else if (obj is Bar) {
                ((Bar)obj).Text = strItemText;
            } else {
                base.SetText(obj, strItemText);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetTooltipText:             Set the tooltip property of an object
        /// </summary>
        /// <param name="obj">          Object</param>
        /// <param name="strTooltipText">   Tooltip text</param>
        /// <param name="toolTip">          Tooltip component</param>
        //
        //*********************************************************                        
        protected override void SetTooltipText(Object obj, string strTooltipText, System.Windows.Forms.ToolTip toolTip) {
            if (obj is BaseItem) {
                ((BaseItem)obj).Tooltip = strTooltipText;
            } else {
                base.SetTooltipText(obj, strTooltipText, toolTip);
            }
        }
    }
}

