using System;
using System.Xml;
using System.Windows.Forms;
using System.Drawing;
using DevComponents.DotNetBar;
using RepPP.Toolkit.Window;

namespace RepPP.Toolkit.Window.Ext {

    //*********************************************************
    //
    /// <summary>
    /// Superclass the UIConfigurationHelper to implement serialization of 
    /// DotNetBar manager (manager of docking windows)
    /// </summary>
    //
    //*********************************************************
    public class BarUIConfigurationHelper : UIConfigurationHelper {

        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        /// <param name="strFileName">  XML file name containing the configuration</param>
        //
        //*********************************************************
        public BarUIConfigurationHelper(string strFileName) : base(strFileName) {
        }

        //*********************************************************
        //
        /// <summary>
        /// LoadBarState:               Load the state of the docking windows
        /// </summary>
        /// <param name="node">         Node from which to load the information</param>
        /// <param name="bar">          Bar to restore</param>
        //
        //*********************************************************
        private void LoadBarState(XmlNode node, Bar bar) {
            string[]	        strValue;
            XmlNode		        nodeBar;
            XmlNode		        nodeDockContainer;
            Point		        location;
            Size		        size;
            string		        strVersion;
            Form                frmParent;
            Point               pointLocation;
            Size                sizeSize;
            DockContainerItem   dockContainerItem = null;
            bool                bVisible;

            strValue	= new string[4];
            nodeBar		= FindNode(node, "BAR" + bar.Name);
            try {
                foreach (BaseItem item in bar.Items) {
                    if (item is DockContainerItem) {
                        dockContainerItem = (DockContainerItem)item;
                        break;
                    }
                }
                strValue[0] = GetNodeAttribute(nodeBar, "AUTOHIDE");
                if (strValue[0] != null) {
                    bar.AutoHide = (strValue[0] != "0");
                }
                strValue[0]  = GetNodeAttribute(nodeBar, "AUTOHIDEVISIBLE");
                if (strValue[0] != null) {
                    try {
                        bar.AutoHideVisible = (strValue[0] != "0");
                    } 
                    catch {
                    }
                }
                strValue[0] = GetNodeAttribute(nodeBar, "DOCKLINE");
                if (strValue[0] != null) {
                    bar.DockLine = Int32.Parse(strValue[0]);
                }
                strValue[0] = GetNodeAttribute(nodeBar, "DOCKSIDE");
                if (strValue[0] != null) {
                    bVisible     = bar.Visible;
                    bar.DockSide = (eDockSide)Int32.Parse(strValue[0]);
                    // Patch : If the bar is invisible during docking then the bar should be visible.
                    if (!bVisible) {
                        bar.Visible = false;
                    }
                }
                strValue[0] = GetNodeAttribute(nodeBar, "LOCKDOCKPOSITION");
                if (strValue[0] != null) {
                    bar.LockDockPosition = (strValue[0] != "0");
                }
                strValue[0] = GetNodeAttribute(nodeBar, "DOCKOFFSET");
                if (strValue[0] != null) {
                    bar.DockOffset = Int32.Parse(strValue[0]);
                }
                strValue[0] = GetNodeAttribute(nodeBar, "DOCKORIENTATION");
                if (strValue[0] != null) {
                    bar.DockOrientation = (eOrientation)Int32.Parse(strValue[0]);
                }
                strValue[0] = GetNodeAttribute(nodeBar, "PADDINGBOTTOM");
                if (strValue[0] != null) {
                    bar.PaddingBottom = Int32.Parse(strValue[0]);
                }
                strValue[0] = GetNodeAttribute(nodeBar, "PADDINGLEFT");
                if (strValue[0] != null) {
                    bar.PaddingLeft = Int32.Parse(strValue[0]);
                }
                strValue[0] = GetNodeAttribute(nodeBar, "PADDINGRIGHT");
                if (strValue[0] != null) {
                    bar.PaddingRight = Int32.Parse(strValue[0]);
                }
                strValue[0] = GetNodeAttribute(nodeBar, "PADDINGTOP");
                if (strValue[0] != null) {
                    bar.PaddingTop = Int32.Parse(strValue[0]);
                }
                bar.SizeChanged += new EventHandler(bar_SizeChanged);
                nodeDockContainer	= FindNode(nodeBar, "DOCKCONTAINER");            
                strVersion			= GetNodeAttribute(nodeDockContainer, "VERSION");
                if (strVersion != null && strVersion == "1" && dockContainerItem != null) {
                    LoadDockContainerState(nodeDockContainer, dockContainerItem, bar.DockOrientation);
                }
                if (bar.DockSide == eDockSide.None) {
                    frmParent = bar.FindForm();
                    LoadFormPosState(frmParent, nodeBar, out pointLocation, out sizeSize);
                }
                strValue[0]	= GetNodeAttribute(nodeBar, "CLIENTSIZEHEIGHT");    
                strValue[1]	= GetNodeAttribute(nodeBar, "CLIENTSIZEWIDTH");
                strValue[2] = GetNodeAttribute(nodeBar, "LOCATIONX");
                strValue[3]	= GetNodeAttribute(nodeBar, "LOCATIONY");
                if (strValue[0] != null && strValue[1] != null && strValue[2] != null && strValue[3] != null) {
                    size		= new System.Drawing.Size(Int32.Parse(strValue[1]), Int32.Parse(strValue[0]));
                    location	= new System.Drawing.Point(Int32.Parse(strValue[2]), Int32.Parse(strValue[3]));
                    bar.Bounds	= new System.Drawing.Rectangle(location, size);
                    bar.RecalcSize();
                }
                foreach (Control ctl in bar.Controls) {
                    LoadControlState(nodeBar, ctl);
                }
            } catch(System.Exception) {
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// LoadDockContainerState:         Load DockContainer state
        /// </summary>
        /// <param name="node">             Node in which to save the information</param>
        /// <param name="dockContainer">    DockContainer</param>
        /// <param name="dockOrientation">  Dock orientation</param>
        //
        //*********************************************************
        private void LoadDockContainerState(XmlNode node, DockContainerItem dockContainer, eOrientation dockOrientation) 
        {
            string	    strValue;
            XmlNode		nodeDockContainer;

            nodeDockContainer	= FindNode(node, "DOCKCONTAINER" + dockContainer.Name);
            try {
                if (dockOrientation == eOrientation.Horizontal) {
                    strValue = GetNodeAttribute(nodeDockContainer, "SIZEHEIGHT");
                    if (strValue != null && strValue != String.Empty) {
                        dockContainer.Height = Int32.Parse(strValue);
                    }
                } else if (dockOrientation == eOrientation.Vertical) {
                    strValue = GetNodeAttribute(nodeDockContainer, "SIZEWIDTH");
                    if (strValue != null && strValue != String.Empty) {
                        dockContainer.Width	 = Int32.Parse(strValue);
                    }
                }
            } 
            catch(System.Exception) {
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SaveBarState:		    Saves Bar state 
        /// </summary>
        /// <param name="node">     Node in which to save the information</param>
        /// <param name="bar">      Bar</param>
        //
        //*********************************************************
        private void SaveBarState(XmlNode node, Bar bar) 
        {
            XmlNode nodeBar;
            XmlNode nodeDockContainer;
            Form    frmParent;

            nodeBar     = FindNode(node, "BAR" + bar.Name);
            if (bar.DockSide == eDockSide.None) {
                frmParent   = bar.FindForm();
                SaveFormPosState(frmParent, nodeBar, frmParent.Location, frmParent.Size);
            }
            SetNodeAttribute(nodeBar, "CLIENTSIZEHEIGHT",	bar.Bounds.Height.ToString());
            SetNodeAttribute(nodeBar, "CLIENTSIZEWIDTH",	bar.Bounds.Width.ToString());
            SetNodeAttribute(nodeBar, "LOCATIONX",			bar.Bounds.X.ToString());
            SetNodeAttribute(nodeBar, "LOCATIONY",			bar.Bounds.Y.ToString());
            SetNodeAttribute(nodeBar, "AUTOHIDEVISIBLE",	bar.AutoHideVisible ? "1" : "0");
            SetNodeAttribute(nodeBar, "AUTOHIDE",	        bar.AutoHide ? "1" : "0");
            SetNodeAttribute(nodeBar, "DOCKLINE",			bar.DockLine.ToString());
            SetNodeAttribute(nodeBar, "DOCKSIDE",			((int)bar.DockSide).ToString());
            SetNodeAttribute(nodeBar, "LOCKDOCKPOSITION",	bar.LockDockPosition ? "1" : "0");
            SetNodeAttribute(nodeBar, "DOCKOFFSET",			bar.DockOffset.ToString());
            SetNodeAttribute(nodeBar, "DOCKORIENTATION",	((int)bar.DockOrientation).ToString());
            SetNodeAttribute(nodeBar, "PADDINGBOTTOM",		bar.PaddingBottom.ToString());
            SetNodeAttribute(nodeBar, "PADDINGLEFT",		bar.PaddingLeft.ToString());
            SetNodeAttribute(nodeBar, "PADDINGRIGHT",		bar.PaddingRight.ToString());
            SetNodeAttribute(nodeBar, "PADDINGTOP",			bar.PaddingTop.ToString());

            nodeDockContainer = FindNode(nodeBar, "DOCKCONTAINER");
            nodeDockContainer.RemoveAll();
            SetNodeAttribute(nodeDockContainer, "VERSION", "1");
            foreach (BaseItem item in bar.Items) {
                if (item is DockContainerItem) {
                    SaveDockContainerState(nodeDockContainer, (DockContainerItem)item);
                }
            }
            foreach (Control ctl in bar.Controls) {
                SaveControlState(nodeBar, ctl);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SaveDockContainerState:		    Saves DockContainer state 
        /// </summary>
        /// <param name="node">             Node in which to save the information</param>
        /// <param name="dockContainer">    DockContainer</param>
        //
        //*********************************************************
        private void SaveDockContainerState(XmlNode node, DockContainerItem dockContainer) 
        {
            XmlNode nodeDockContainer;

            nodeDockContainer = FindNode(node, "DOCKCONTAINER" + dockContainer.Name);
            SetNodeAttribute(nodeDockContainer, "SIZEHEIGHT",	dockContainer.Height.ToString());
            SetNodeAttribute(nodeDockContainer, "SIZEWIDTH",	dockContainer.Width.ToString());
        }

        //*********************************************************
        //
        /// <summary>
        /// SaveFormDockWindowState:    Save the specified dockable window state 
        /// </summary>
        /// <param name="node">         Form node into which to save the information</param>
        /// <param name="barManager">   Array of Bar (dockable window)</param>
        //
        //*********************************************************
        private void SaveBarState(XmlNode node, DotNetBarManager barManager) 
        {
            XmlNode     nodeBar;
            
            nodeBar = FindNode(node, "DOCKABLEWINDOWS");
            nodeBar.RemoveAll();            
            SetNodeAttribute(nodeBar, "VERSION", "1");
            foreach (Bar bar in barManager.Bars) {
                SaveBarState(nodeBar, bar);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// LoadBarState:               Load the state of a bar manager
        /// </summary>
        /// <param name="node">         Node containing the data</param>
        /// <param name="barManager">   Bar manager</param>
        //
        //*********************************************************
        private void LoadBarState(XmlNode node, DotNetBarManager barManager) 
        {
            XmlNode     nodeBar;
            string      strVersion;
            
            nodeBar    = FindNode(node, "DOCKABLEWINDOWS");            
            strVersion = GetNodeAttribute(nodeBar, "VERSION");
            if (strVersion != null && strVersion == "1") {
                foreach (Bar bar in barManager.Bars) {
                    LoadBarState(nodeBar, bar);
                }
            }
        }

        //*********************************************************
        //
	    /// <summary>
	    /// LoadControlState:           Load the state of a control
	    /// </summary>
	    /// <param name="node">         Father node</param>
	    /// <param name="ctl">          Form to load the state</param>
	    //
        //*********************************************************
        public override void LoadControlState(XmlNode node, System.Windows.Forms.Control ctl) 
        {
            DockSite    dockSite;

            if (ctl is DockSite) {
                dockSite = (DockSite)ctl;
                if (dockSite.Dock == DockStyle.Top) {
                    LoadBarState(node, dockSite.Owner);
                }
            } else {
                base.LoadControlState(node, ctl);
            }
        }

        //*********************************************************
        //
	    /// <summary>
	    /// SaveControlState:           Load the state of a control
	    /// </summary>
	    /// <param name="node">         Father node</param>
	    /// <param name="ctl">          Form to load the state</param>
	    //
        //*********************************************************
        public override void SaveControlState(XmlNode node, System.Windows.Forms.Control ctl) 
        {
            DockSite    dockSite;

            if (ctl is DockSite) {
                dockSite = (DockSite)ctl;
                if (dockSite.Dock == DockStyle.Top) {
                    SaveBarState(node, dockSite.Owner);
                }
            } else {
                base.SaveControlState(node, ctl);
            }
        }


        private void bar_SizeChanged(object sender, EventArgs e) {

        }
    } /*SCBarUIConfigurationHelper*/
} // Namespace
