using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;

namespace RepPP.Toolkit.Window.Ext {
    /// <summary>User interface helper class for delta</summary>
    public class DeltaObjectUIUtil {
        /// <summary>Delta object</summary>
        private DeltaObject m_deltaObject;

        //*********************************************************     
        //
        /// <summary>
        /// ctor
        /// </summary>
        /// <param name="deltaObject"> Delta object</param>
        //  
        //*********************************************************     
        public DeltaObjectUIUtil(DeltaObject deltaObject) {
            m_deltaObject = deltaObject;
        }

        //*********************************************************     
        //
        /// <summary>
        /// Attached delta object
        /// </summary>
        //  
        //*********************************************************     
        public DeltaObject DeltaObject {
            get {
                return(m_deltaObject);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Attach the field control to a new RowsetDef. Used because the second source of controls are attached to a
        /// dynamicly created RowsetDef.
        /// </summary>
        /// <param name="ctls">     Collection of controls</param>
        /// <param name="rsdOld">   RowsetDef to which the controls are currently attached</param>
        /// <param name="rsdNew">   RowsetDef to which to attach the controls</param>
        //  
        //*********************************************************     
        private void AttachControlToRowsetDef(Control.ControlCollection ctls, RepPP.RowsetDef rsdOld, RepPP.RowsetDef rsdNew) {
            RepPP.Toolkit.FldControlUtil.IFldControl    fldCtl;
            RepPP.RepPPField                            repppFieldCtl;
            
            foreach (Control ctl in ctls) {
                fldCtl = ctl as RepPP.Toolkit.FldControlUtil.IFldControl;
                if (fldCtl != null) {
                    if (String.Compare(fldCtl.FieldFullName.RowsetName, rsdOld.Name, true) == 0) {
                        fldCtl.FieldFullName.RowsetName = rsdNew.Name;
                    }
                } else {
                    repppFieldCtl = ctl as RepPP.RepPPField;
                    if (repppFieldCtl != null && String.Compare(repppFieldCtl.RowsetName, rsdOld.Name, true) == 0) {
                        repppFieldCtl.RowsetName = rsdNew.Name;
                    }
                }
                AttachControlToRowsetDef(ctl.Controls, rsdOld, rsdNew);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Attach the field control to a new RowsetDef. Used because the second source of controls are attached to a
        /// dynamicly created RowsetDef.
        /// </summary>
        /// <param name="ctls">         Collection of controls</param>
        /// <param name="nodeLowest">   Lowest node to which the from is concerned</param>
        //  
        //*********************************************************     
        public void AttachControlToRowsetDef(Control.ControlCollection ctls, RepPP.RowsetTreeDefNode nodeLowest) {
            RepPP.RowsetTreeDefNode nodeSrc2;

            nodeSrc2 = m_deltaObject.FindSource2Node(nodeLowest);
            while (nodeSrc2 != null) {
                AttachControlToRowsetDef(ctls, nodeLowest.RowsetDef, nodeSrc2.RowsetDef);
                nodeLowest = nodeLowest.Parent as RepPP.RowsetTreeDefNode;
                nodeSrc2   = nodeSrc2.Parent   as RepPP.RowsetTreeDefNode;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// Replace the Rowset references in a SQL command for the dynamic one for all RowsetDefs in the hierarchy.
        /// </summary>
        /// <param name="strSqlCmd">    SQL command to modify</param>
        /// <param name="nodeLowest">   Lowest node to which the from is concerned</param>
        //  
        //*********************************************************     
        public string ReplaceRowsetNameInSqlCmd(string strSqlCmd, RepPP.RowsetTreeDefNode nodeLowest) {
            RepPP.RowsetTreeDefNode     node2;

            node2 = m_deltaObject.FindSource2Node(nodeLowest);
            while (node2 != null) {
                strSqlCmd   = strSqlCmd.Replace(":" + nodeLowest.RowsetDef.Name.ToUpper(), ":" + node2.RowsetDef.Name);
                nodeLowest  = nodeLowest.Parent as RepPP.RowsetTreeDefNode;
                node2       = node2.Parent      as RepPP.RowsetTreeDefNode;
            }
            return(strSqlCmd);
        }
    } // Class DeltaObjectUIUtil
} // Namespace
