//*********************************************************
//
// Edit:		Overload the rep++ unicode editor to interact with toolbar
//              status bar context menu, find and replace form
//
// Author:		Maher Najah
//
// (c) 2002 by Consyst SQL Inc.
//
//
//*********************************************************
using System;
using System.Collections;
using RepPP;
using RepPP.Toolkit.Window;

namespace RepPP.Toolkit.Window.Ext {
    public enum sdEditCmd {
        sdCmdCopy       = 0,       // Copy
        sdCmdCut        = 1,       // Cut
        sdCmdPaste      = 2,       // Paste
        sdCmdUndo       = 3,       // Undo
        sdCmdRedo       = 4,       // Redo
        sdCmdDelLine    = 5,       // Delete the current line
        sdCmdDelEol     = 6,       // Delete util the end of the line
        sdCmdJoinLine   = 7,       // Join line
        sdCmdDuplicLine = 8,       // Duplicate line
        sdCmdIndentStr  = 9,       // Indent with string
        sdCmdFind       = 10,      // Find
        sdCmdInsMode    = 11,      // Insertion mode
        sdCmdBlockMode  = 12,      // Block selection mode
        sdCmdIndentTab  = 13,      // Indent Tab
        sdCmdOutdentTab = 14,      // Outdent Tab
        sdCmdSelectAll  = 15,      // Select all
        sdCmdReplace    = 16       // Replace
    };

	/// <summary>
	/// Summary description for Edit.
	/// </summary>
	public class Edit : RepPP.RepPPEdit {
        private   System.Windows.Forms.ContextMenu       mnuCtxMenu;
        protected System.Windows.Forms.MenuItem          mnuInsMode;
        protected System.Windows.Forms.MenuItem          mnuBlockSel;
        private   System.Windows.Forms.MenuItem          mnuEditSep1;
        private   System.Windows.Forms.MenuItem          mnuDelLine;
        private   System.Windows.Forms.MenuItem          mnuDelEOL;
        private   System.Windows.Forms.MenuItem          mnuJoinLine;
        private   System.Windows.Forms.MenuItem          mnuDuplicLine;
        private   System.Windows.Forms.MenuItem          mnuEditSep2;
        private   System.Windows.Forms.MenuItem          mnuCut;
        private   System.Windows.Forms.MenuItem          mnuCopy;
        private   System.Windows.Forms.MenuItem          mnuPaste;
        private   System.Windows.Forms.MenuItem          mnuEditSep3;
        private   System.Windows.Forms.MenuItem          mnuUndo;
        private   System.Windows.Forms.MenuItem          mnuRedo;
        private   System.Windows.Forms.MenuItem          mnuEditSep5;
        private   System.Windows.Forms.MenuItem          mnuSelectAll;
        private   System.Windows.Forms.MenuItem          mnuEditSep4;
        private   System.Windows.Forms.MenuItem          mnuFind;
        private   System.Windows.Forms.MenuItem          mnuReplace;
        private   System.Windows.Forms.MenuItem          mnuEditSep6;
        private   System.Windows.Forms.MenuItem          mnuIndentTab;
        private   System.Windows.Forms.MenuItem          mnuOutdentTab;
        private   System.Windows.Forms.MenuItem          mnuIndentStr;

        private   System.Windows.Forms.StatusBarPanel    m_PosPanel;          // The attched position statusbar panel
        private   System.Windows.Forms.StatusBarPanel    m_InsPanel;          // The attched insertion mode statusbar panel
        private   CmdHolder                              m_cmdHolder;         // Attached command holder
        private   IApplication                           m_app;               // The attached rep++ application
        private   EditFind                               m_findForm;          // The attached find form
        private   EditReplace                            m_replaceForm;       // The attached replace form
        private   ArrayList                              m_findListItems;     // The find list items
        private   ArrayList                              m_replaceListItems;  // The replace list items
        private   ControlTextTranslator                  m_txtTranslator;     // control text translator object
        //*********************************************************     
        //
        /// <summary>
        /// ctor:               Class constructor
        /// </summary>
        /// 
        //  
        //*********************************************************     
		public Edit() {
            InitializeComponent();
            mnuInsMode.Checked = true;
            this.ContextMenu   = mnuCtxMenu; 
            m_PosPanel         = null;
            m_InsPanel         = null;
            m_findForm         = null;
            m_replaceForm      = null;
            m_findListItems    = new ArrayList();
            m_replaceListItems = new ArrayList();
            m_txtTranslator    = null;
        }

        //*********************************************************	    
        //
        /// <summary>
        // Dispose:                     Dispose the editor
        /// </summary>
        /// <param name="bDisposing">   true if dispose, false if GC </param>
        //
        //*********************************************************	    
        protected override void Dispose(bool bDisposing ) {
            if (bDisposing) {
                if (m_findForm != null) {
                    m_findForm.Dispose();
                    m_findForm = null;
                }
                if (m_replaceForm != null) {
                    m_replaceForm.Dispose();
                    m_replaceForm = null;
                }
                m_findListItems    = null;
                m_replaceListItems = null;
            }
            base.Dispose(bDisposing);
        }

        #region Component Designer generated code
        /// <summary> 
        /// Required method for Designer support - do not modify 
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent() {
            this.mnuCtxMenu = new System.Windows.Forms.ContextMenu();
            this.mnuInsMode = new System.Windows.Forms.MenuItem();
            this.mnuBlockSel = new System.Windows.Forms.MenuItem();
            this.mnuEditSep1 = new System.Windows.Forms.MenuItem();
            this.mnuDelLine = new System.Windows.Forms.MenuItem();
            this.mnuDelEOL = new System.Windows.Forms.MenuItem();
            this.mnuJoinLine = new System.Windows.Forms.MenuItem();
            this.mnuDuplicLine = new System.Windows.Forms.MenuItem();
            this.mnuEditSep2 = new System.Windows.Forms.MenuItem();
            this.mnuCut = new System.Windows.Forms.MenuItem();
            this.mnuCopy = new System.Windows.Forms.MenuItem();
            this.mnuPaste = new System.Windows.Forms.MenuItem();
            this.mnuEditSep3 = new System.Windows.Forms.MenuItem();
            this.mnuUndo = new System.Windows.Forms.MenuItem();
            this.mnuRedo = new System.Windows.Forms.MenuItem();
            this.mnuEditSep5 = new System.Windows.Forms.MenuItem();
            this.mnuSelectAll = new System.Windows.Forms.MenuItem();
            this.mnuEditSep6 = new System.Windows.Forms.MenuItem();
            this.mnuFind = new System.Windows.Forms.MenuItem();
            this.mnuReplace = new System.Windows.Forms.MenuItem();
            this.mnuEditSep4 = new System.Windows.Forms.MenuItem();
            this.mnuIndentStr = new System.Windows.Forms.MenuItem();
            this.mnuIndentTab = new System.Windows.Forms.MenuItem();
            this.mnuOutdentTab = new System.Windows.Forms.MenuItem();

            // 
            // mnuCtxMenu
            // 
            this.mnuCtxMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
                                                                                       this.mnuInsMode,
                                                                                       this.mnuBlockSel,
                                                                                       this.mnuEditSep1,
                                                                                       this.mnuDelLine,
                                                                                       this.mnuDelEOL,
                                                                                       this.mnuJoinLine,
                                                                                       this.mnuDuplicLine,
                                                                                       this.mnuEditSep2,
                                                                                       this.mnuCut,
                                                                                       this.mnuCopy,
                                                                                       this.mnuPaste,
                                                                                       this.mnuEditSep3,
                                                                                       this.mnuUndo,
                                                                                       this.mnuRedo,
                                                                                       this.mnuEditSep5,
                                                                                       this.mnuSelectAll,
                                                                                       this.mnuEditSep4,
                                                                                       this.mnuFind,
                                                                                       this.mnuReplace,
                                                                                       this.mnuEditSep6,
                                                                                       this.mnuIndentTab,
                                                                                       this.mnuOutdentTab,
                                                                                       this.mnuIndentStr});
            // 
            // mnuInsMode
            // 
            this.mnuInsMode.Checked = true;
            this.mnuInsMode.Index = 0;
            this.mnuInsMode.Shortcut = System.Windows.Forms.Shortcut.Ins;
            this.mnuInsMode.Text = "Insert Mode";
            // 
            // mnuBlockSel
            // 
            this.mnuBlockSel.Checked = false;
            this.mnuBlockSel.Index = 1;
            this.mnuBlockSel.Text = "Block Selection";
            // 
            // mnuEditSep1
            // 
            this.mnuEditSep1.Index = 2;
            this.mnuEditSep1.Text = "-";
            // 
            // mnuDelLine
            // 
            this.mnuDelLine.Index = 3;
            this.mnuDelLine.Text = "Delete Line";
            // 
            // mnuDelEOL
            // 
            this.mnuDelEOL.Index = 4;
            this.mnuDelEOL.Shortcut = System.Windows.Forms.Shortcut.CtrlE;
            this.mnuDelEOL.Text = "Delete to End of Line";
            // 
            // mnuJoinLine
            // 
            this.mnuJoinLine.Index = 5;
            this.mnuJoinLine.Shortcut = System.Windows.Forms.Shortcut.CtrlJ;
            this.mnuJoinLine.Text = "Join Line";
            // 
            // mnuDuplicLine
            // 
            this.mnuDuplicLine.Index = 6;
            this.mnuDuplicLine.Shortcut = System.Windows.Forms.Shortcut.CtrlD;
            this.mnuDuplicLine.Text = "Duplicate Line";
            // 
            // mnuEditSep2
            // 
            this.mnuEditSep2.Index = 7;
            this.mnuEditSep2.Text = "-";
            // 
            // mnuCut
            // 
            this.mnuCut.Index = 8;
            this.mnuCut.Shortcut = System.Windows.Forms.Shortcut.CtrlX;
            this.mnuCut.Text = "Cut";
            // 
            // mnuCopy
            // 
            this.mnuCopy.Index = 9;
            this.mnuCopy.Shortcut = System.Windows.Forms.Shortcut.CtrlC;
            this.mnuCopy.Text = "Copy";
            // 
            // mnuPaste
            // 
            this.mnuPaste.Index = 10;
            this.mnuPaste.Shortcut = System.Windows.Forms.Shortcut.CtrlV;
            this.mnuPaste.Text = "Paste";
            // 
            // mnuEditSep3
            // 
            this.mnuEditSep3.Index = 11;
            this.mnuEditSep3.Text = "-";
            // 
            // mnuUndo
            // 
            this.mnuUndo.Index = 12;
            this.mnuUndo.Shortcut = System.Windows.Forms.Shortcut.CtrlZ;
            this.mnuUndo.Text = "Undo";
            // 
            // mnuRedo
            // 
            this.mnuRedo.Index = 13;
            this.mnuRedo.Shortcut = System.Windows.Forms.Shortcut.CtrlY;
            this.mnuRedo.Text = "Redo";
            // 
            // mnuEditSep5
            // 
            this.mnuEditSep5.Index = 14;
            this.mnuEditSep5.Text = "-";
            // 
            // mnuSelectAll
            // 
            this.mnuSelectAll.Index = 15;
            this.mnuSelectAll.Shortcut = System.Windows.Forms.Shortcut.CtrlA;
            this.mnuSelectAll.Text = "Select All";
            // 
            // mnuEditSep4
            // 
            this.mnuEditSep4.Index = 16;
            this.mnuEditSep4.Text = "-";
            // 
            // mnuFind
            // 
            this.mnuFind.Index = 17;
            this.mnuFind.Shortcut = System.Windows.Forms.Shortcut.CtrlF;
            this.mnuFind.Text = "Find...";
            // 
            // mnuReplace
            // 
            this.mnuReplace.Index = 18;
            this.mnuReplace.Shortcut = System.Windows.Forms.Shortcut.CtrlH;
            this.mnuReplace.Text = "Replace...";
            // 
            // mnuEditSep6
            // 
            this.mnuEditSep6.Index = 19;
            this.mnuEditSep6.Text = "-";
            // 
            // mnuIndentTab
            // 
            this.mnuIndentTab.Index = 20;
            this.mnuIndentTab.Text = "Indent Selection";
            // 
            // mnuOutdentTab
            // 
            this.mnuOutdentTab.Index = 21;
            this.mnuOutdentTab.Text = "Outdent Selection";
            // 
            // mnuIndentStr
            // 
            this.mnuIndentStr.Index = 22;
            this.mnuIndentStr.Text = "Indent With String...";

        }
        #endregion

        //*********************************************************
        //
        /// <summary>
        /// Application:       Gets and sets attached rep++ application
        /// </summary>
        // 
        //*********************************************************
        public IApplication Application {
            get {
                return m_app;
            }
            set {
                if (value != null && m_app != value) {
                    m_app           = value;
                    m_txtTranslator = new ControlTextTranslator(m_app, new string[] {"txtSCEdit"});
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// PositionPanel:       Gets and sets attached position panel
        /// </summary>
        // 
        //*********************************************************
        public System.Windows.Forms.StatusBarPanel PositionPanel {
            get {
                return m_PosPanel;
            }
            set {
                m_PosPanel = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// InsModePanel:       Gets and sets attached insert mode panel
        /// </summary>
        // 
        //*********************************************************
        public System.Windows.Forms.StatusBarPanel InsModePanel {
            get {
                return m_InsPanel;
            }
            set {
                m_InsPanel = value;
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        ///	RegisterCommand:      Register the command to the command holder
        /// </summary>
        //	
        //*********************************************************	    
        protected virtual void RegisterCommand() {   
            CommandHolder.AddCmd("Copy",       new CmdHolder.OnCommand(CmdCopy),    CmdHolder.CmdStateE.Enabled);
            CommandHolder.AddCmd("Cut",        new CmdHolder.OnCommand(CmdCut),     CmdHolder.CmdStateE.Enabled);
            CommandHolder.AddCmd("Paste",      new CmdHolder.OnCommand(CmdPaste),   CmdHolder.CmdStateE.Enabled);
            CommandHolder.AddCmd("Undo",       new CmdHolder.OnCommand(CmdUndo),    CmdHolder.CmdStateE.Enabled);
            CommandHolder.AddCmd("Redo",       new CmdHolder.OnCommand(CmdRedo),    CmdHolder.CmdStateE.Enabled);
            CommandHolder.AddCmd("SelectAll",  new CmdHolder.OnCommand(CmdSelectAll),CmdHolder.CmdStateE.Enabled);
            CommandHolder.AddCmd("DelLine",    new CmdHolder.OnCommand(CmdDelLine), CmdHolder.CmdStateE.Enabled);
            CommandHolder.AddCmd("DelEol",     new CmdHolder.OnCommand(CmdDelEol),  CmdHolder.CmdStateE.Enabled);
            CommandHolder.AddCmd("JoinLine",   new CmdHolder.OnCommand(CmdJoinLine),CmdHolder.CmdStateE.Enabled);
            CommandHolder.AddCmd("DuplicLine", new CmdHolder.OnCommand(CmdDuplicLine), CmdHolder.CmdStateE.Enabled);
            CommandHolder.AddCmd("IndentStr",  new CmdHolder.OnCommand(CmdIndentStr), CmdHolder.CmdStateE.Enabled);
            CommandHolder.AddCmd("Find",       new CmdHolder.OnCommand(CmdFind),    CmdHolder.CmdStateE.Enabled);
            CommandHolder.AddCmd("Replace",    new CmdHolder.OnCommand(CmdReplace), CmdHolder.CmdStateE.Enabled);
            CommandHolder.AddCmd("InsMode",    new CmdHolder.OnCommand(CmdInsMode), CmdHolder.CmdStateE.Enabled);
            CommandHolder.AddCmd("BlockMode",  new CmdHolder.OnCommand(CmdBlockMode), CmdHolder.CmdStateE.Enabled);
            CommandHolder.AddCmd("IndentTab",  new CmdHolder.OnCommand(CmdIndentTab), CmdHolder.CmdStateE.Enabled);
            CommandHolder.AddCmd("OutdentTab", new CmdHolder.OnCommand(CmdOutdentTab),CmdHolder.CmdStateE.Enabled);
        }

        //*********************************************************	    
        //
        /// <summary>
        ///	RegisterTrigger:        Register the button to the command holder
        /// </summary>
        //	
        //*********************************************************	    
        private void RegisterTrigger() {            
            CommandHolder.AddTrigger(null, "Copy",       new object[] {mnuCopy});
            CommandHolder.AddTrigger(null, "Cut",        new object[] {mnuCut});
            CommandHolder.AddTrigger(null, "Paste",      new object[] {mnuPaste});
            CommandHolder.AddTrigger(null, "Undo",       new object[] {mnuUndo});
            CommandHolder.AddTrigger(null, "Redo",       new object[] {mnuRedo});
            CommandHolder.AddTrigger(null, "SelectAll",  new object[] {mnuSelectAll});
            CommandHolder.AddTrigger(null, "Find",       new object[] {mnuFind});
            CommandHolder.AddTrigger(null, "Replace",    new object[] {mnuReplace});
            CommandHolder.AddTrigger(null, "DelLine",    new object[] {mnuDelLine});
            CommandHolder.AddTrigger(null, "DelEol",     new object[] {mnuDelEOL});
            CommandHolder.AddTrigger(null, "JoinLine",   new object[] {mnuJoinLine});
            CommandHolder.AddTrigger(null, "DuplicLine", new object[] {mnuDuplicLine});
            CommandHolder.AddTrigger(null, "IndentTab",  new object[] {mnuIndentTab});
            CommandHolder.AddTrigger(null, "OutdentTab", new object[] {mnuOutdentTab});
            CommandHolder.AddTrigger(null, "IndentStr",  new object[] {mnuIndentStr});
            CommandHolder.AddTrigger(null, "InsMode",    new object[] {mnuInsMode});
            CommandHolder.AddTrigger(null, "BlockMode",  new object[] {mnuBlockSel});
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnPosChg:        Handle the position changed event
        /// </summary>
        /// 
        /// <param name="e"> The vent argument </param>
        //  
        //*********************************************************     
        protected override void OnPosChg(PosChgEventArgs e) {
            if (m_PosPanel != null) {
                m_PosPanel.Text = "Ln " + e.PosY + "      Col " + e.PosX;
            }
            base.OnPosChg(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnInsModeChg:    Handle the insertion mode changed event
        /// </summary>
        /// 
        /// <param name="e"> The vent argument </param>
        //  
        //*********************************************************     
        protected override void OnInsModeChg(ModeChgEventArgs e) {
            if (m_InsPanel != null) {
                if (e.Mode) {
                    m_InsPanel.Text = "INS";
                } else {
                    m_InsPanel.Text = "OVR";
                }
            }
            base.OnInsModeChg(e);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CommandHolder:     Get the command holder
        /// </summary>
        //  
        //*********************************************************     
        public virtual CmdHolder CommandHolder {
            get {
                return(m_cmdHolder);
            }
            set {
                if (m_cmdHolder == null && value != null) {
                    m_cmdHolder = value;
                    RegisterCommand();
                    RegisterTrigger();
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdCopy:            Command 'Copy'
        /// </summary>
        //  
        //*********************************************************     
        protected void CmdCopy() {
            HandleCommand(sdEditCmd.sdCmdCopy);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdCut:            Command 'Cut'
        /// </summary>
        //  
        //*********************************************************     
        protected void CmdCut() {
            HandleCommand(sdEditCmd.sdCmdCut);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdPaste:            Command 'Paste'
        /// </summary>
        //  
        //*********************************************************     
        protected void CmdPaste() {
            HandleCommand(sdEditCmd.sdCmdPaste);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdUndo:            Command 'Undo'
        /// </summary>
        //  
        //*********************************************************     
        protected void CmdUndo() {
            HandleCommand(sdEditCmd.sdCmdUndo);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdRedo:            Command 'Redo'
        /// </summary>
        //  
        //*********************************************************     
        protected void CmdRedo() {
            HandleCommand(sdEditCmd.sdCmdRedo);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdSelectAll:            Command 'Select All'
        /// </summary>
        //  
        //*********************************************************     
        protected void CmdSelectAll() {
            HandleCommand(sdEditCmd.sdCmdSelectAll);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdDelLine:            Command 'DelLine'
        /// </summary>
        //  
        //*********************************************************     
        protected void CmdDelLine() {
            HandleCommand(sdEditCmd.sdCmdDelLine);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdDelEol:            Command 'DelEol'
        /// </summary>
        //  
        //*********************************************************     
        protected void CmdDelEol() {
            HandleCommand(sdEditCmd.sdCmdDelEol);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdJoinLine:            Command 'JoinLine'
        /// </summary>
        //  
        //*********************************************************     
        protected void CmdJoinLine() {
            HandleCommand(sdEditCmd.sdCmdJoinLine);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdDuplicLine:            Command 'DuplicLine'
        /// </summary>
        //  
        //*********************************************************     
        protected void CmdDuplicLine() {
            HandleCommand(sdEditCmd.sdCmdDuplicLine);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdIndentStr:            Command 'IndentStr'
        /// </summary>
        //  
        //*********************************************************     
        protected void CmdIndentStr() {
            HandleCommand(sdEditCmd.sdCmdIndentStr);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdIndentTab:            Command 'IndentTab'
        /// </summary>
        //  
        //*********************************************************     
        protected void CmdIndentTab() {
            HandleCommand(sdEditCmd.sdCmdIndentTab);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdOutdentTab:            Command 'OutdentTab'
        /// </summary>
        //  
        //*********************************************************     
        protected void CmdOutdentTab() {
            HandleCommand(sdEditCmd.sdCmdOutdentTab);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdFind:            Command 'Find'
        /// </summary>
        //  
        //*********************************************************     
        protected void CmdFind() {
            HandleCommand(sdEditCmd.sdCmdFind);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdReplace:            Command 'Replace'
        /// </summary>
        //  
        //*********************************************************     
        protected void CmdReplace() {
            HandleCommand(sdEditCmd.sdCmdReplace);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdInsMode:            Command 'InsMode'
        /// </summary>
        //  
        //*********************************************************     
        protected void CmdInsMode() {
            HandleCommand(sdEditCmd.sdCmdInsMode);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CmdBlockMode:            Command 'BlockMode'
        /// </summary>
        //  
        //*********************************************************     
        protected void CmdBlockMode() {
            HandleCommand(sdEditCmd.sdCmdBlockMode);
        }

        //*********************************************************     
        //
        /// <summary>
        /// HandleCommand:          Handle a command coming from a menu or a toolbar
        /// </summary>
        /// <param name="eCmd">     Command Id </param>
        //  
        //*********************************************************     
        public virtual void HandleCommand(sdEditCmd eCmd) {
            switch(eCmd) {
                case sdEditCmd.sdCmdCopy:
                    Copy();
                    break;
                case sdEditCmd.sdCmdCut:
                    Cut();
                    break;
                case sdEditCmd.sdCmdPaste:
                    Paste();
                    break;
                case sdEditCmd.sdCmdUndo:
                    Undo();
                    break;
                case sdEditCmd.sdCmdRedo:
                    Redo();
                    break;
                case sdEditCmd.sdCmdSelectAll:
                    SelectText(0, -1);
                    break;
                case sdEditCmd.sdCmdDelLine:
                    DeleteLine();
                    break;
                case sdEditCmd.sdCmdDelEol:
                    DeleteEndOfLine();
                    break;
                case sdEditCmd.sdCmdDuplicLine:
                    DuplicateLine();
                    break;
                case sdEditCmd.sdCmdJoinLine:
                    JoinLines();
                    break;
                case sdEditCmd.sdCmdIndentStr:
                    IndentWithStr();
                    break;
                case sdEditCmd.sdCmdIndentTab:
                    IndentTab();
                    break;
                case sdEditCmd.sdCmdOutdentTab:
                    UnindentTab();
                    break;
                case sdEditCmd.sdCmdInsMode:
                    mnuInsMode.Checked = !mnuInsMode.Checked;
                    SetInsMode(mnuInsMode.Checked);
                    break;
                case sdEditCmd.sdCmdBlockMode:
                    mnuBlockSel.Checked = !mnuBlockSel.Checked;
                    SetSelMode(mnuBlockSel.Checked ? EditSelectionMode.sdBlock : EditSelectionMode.sdChar);
                    break;
                case sdEditCmd.sdCmdFind:
                    if (m_replaceForm != null && m_replaceForm.IsHandleCreated) {
                        m_replaceForm.Close();
                        m_replaceForm.Dispose();
                        m_replaceForm = null;
                    }
                    SetShowSelOnNoFocus(true);
                    if (m_findForm != null && m_findForm.IsHandleCreated) {
                        m_findForm.FindText = GetCurWord();
                        m_findForm.Select();
                    } else {
                        if (m_findForm != null) {
                            m_findForm.Dispose();
                            m_findForm = null;
                        }
                        m_findForm                = new EditFind(m_app);
                        m_findForm.Owner          = this.ParentForm;
                        m_findForm.FindDataSource = m_findListItems;
                        m_findForm.Edit           = this;
                        m_txtTranslator.TranslateControl(m_findForm);
                        m_findForm.Show();
                    }
                    break;
                case sdEditCmd.sdCmdReplace:
                    if (m_findForm != null && m_findForm.IsHandleCreated) {
                        m_findForm.Close();
                        m_findForm.Dispose();
                        m_findForm = null;
                    }
                    SetShowSelOnNoFocus(true);
                    if (m_replaceForm != null && m_replaceForm.IsHandleCreated) {
                        m_replaceForm.FindText = GetCurWord();
                        m_replaceForm.Select();
                    } else {
                        if (m_replaceForm != null) {
                            m_replaceForm.Dispose();
                            m_replaceForm = null;
                        }
                        m_replaceForm                   = new EditReplace(m_app);
                        m_replaceForm.Owner             = this.ParentForm;
                        m_replaceForm.FindDataSource    = m_findListItems;
                        m_replaceForm.ReplaceDataSource = m_replaceListItems;
                        m_replaceForm.Edit              = this;
                        m_txtTranslator.TranslateControl(m_replaceForm);
                        m_replaceForm.Show();
                    }
                    break;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// MergeMenu:               Merge the edit menu in a main menu
        /// </summary>
        /// <param name="mainMenu">  The main menu </param>
        /// <param name="iIndex">    position index in the main menu </param>
        //  
        //*********************************************************     
        public void MergeMenu(System.Windows.Forms.MainMenu mainMenu, int iIndex) {
            System.Windows.Forms.MenuItem   mnuEdit;
            System.Windows.Forms.MenuItem   mnuItem;
            
            // 
            // mnuEdit
            // 
            mnuEdit       = new System.Windows.Forms.MenuItem();
            mnuEdit.Index = 0;
            mnuEdit.Text  = (m_txtTranslator != null) ? m_txtTranslator.GetItemText("mnuEdit", "Text") : "Edit";
            mainMenu.MenuItems.Add(iIndex, mnuEdit);
            // 
            // mnuDelLine
            //
            mnuItem       = new System.Windows.Forms.MenuItem();
            mnuItem.Index = 3;
            mnuItem.Text  = mnuDelLine.Text;
            mnuEdit.MenuItems.Add(mnuItem);
            if (CommandHolder != null) {
                CommandHolder.AddTrigger(null, "DelLine", new object[] {mnuItem});
            }
            // 
            // mnuDelEOL
            // 
            mnuItem          = new System.Windows.Forms.MenuItem();
            mnuItem.Index    = 4;
            mnuItem.Shortcut = System.Windows.Forms.Shortcut.CtrlE;
            mnuItem.Text     = mnuDelEOL.Text;
            mnuEdit.MenuItems.Add(mnuItem);
            if (CommandHolder != null) {
                CommandHolder.AddTrigger(null, "DelEol", new object[] {mnuItem});
            }
            // 
            // mnuJoinLine
            // 
            mnuItem          = new System.Windows.Forms.MenuItem();
            mnuItem.Index    = 5;
            mnuItem.Shortcut = System.Windows.Forms.Shortcut.CtrlJ;
            mnuItem.Text     = mnuJoinLine.Text;
            mnuEdit.MenuItems.Add(mnuItem);
            if (CommandHolder != null) {
                CommandHolder.AddTrigger(null, "JoinLine", new object[] {mnuItem});
            }
            // 
            // mnuDuplicLine
            // 
            mnuItem          = new System.Windows.Forms.MenuItem();
            mnuItem.Index    = 6;
            mnuItem.Shortcut = System.Windows.Forms.Shortcut.CtrlD;
            mnuItem.Text     = mnuDuplicLine.Text;
            mnuEdit.MenuItems.Add(mnuItem);
            if (CommandHolder != null) {
                CommandHolder.AddTrigger(null, "DuplicLine", new object[] {mnuItem});
            }
            // 
            // mnuEditSep2
            //
            mnuItem       = new System.Windows.Forms.MenuItem();
            mnuItem.Index = 7;
            mnuItem.Text  = "-";
            mnuEdit.MenuItems.Add(mnuItem);
            // 
            // mnuCut
            //
            mnuItem          = new System.Windows.Forms.MenuItem();
            mnuItem.Index    = 8;
            mnuItem.Shortcut = System.Windows.Forms.Shortcut.CtrlX;
            mnuItem.Text     = mnuCut.Text;
            mnuEdit.MenuItems.Add(mnuItem);
            if (CommandHolder != null) {
                CommandHolder.AddTrigger(null, "Cut", new object[] {mnuItem});
            }
            // 
            // mnuCopy
            // 
            mnuItem          = new System.Windows.Forms.MenuItem();
            mnuItem.Index    = 9;
            mnuItem.Shortcut = System.Windows.Forms.Shortcut.CtrlC;
            mnuItem.Text     = mnuCopy.Text;
            mnuEdit.MenuItems.Add(mnuItem);
            if (CommandHolder != null) {
                CommandHolder.AddTrigger(null, "Copy", new object[] {mnuItem});
            }
            // 
            // mnuPaste
            //
            mnuItem          = new System.Windows.Forms.MenuItem();
            mnuItem.Index    = 10;
            mnuItem.Shortcut = System.Windows.Forms.Shortcut.CtrlV;
            mnuItem.Text     = mnuPaste.Text;
            mnuEdit.MenuItems.Add(mnuItem);
            if (CommandHolder != null) {
                CommandHolder.AddTrigger(null, "Paste", new object[] {mnuItem});
            }
            // 
            // mnuEditSep3
            //
            mnuItem       = new System.Windows.Forms.MenuItem();
            mnuItem.Index = 11;
            mnuItem.Text  = "-";
            mnuEdit.MenuItems.Add(mnuItem);
            // 
            // mnuUndo
            //
            mnuItem          = new System.Windows.Forms.MenuItem();
            mnuItem.Index    = 12;
            mnuItem.Shortcut = System.Windows.Forms.Shortcut.CtrlZ;
            mnuItem.Text     = mnuUndo.Text;
            mnuEdit.MenuItems.Add(mnuItem);
            if (CommandHolder != null) {
                CommandHolder.AddTrigger(null, "Undo", new object[] {mnuItem});
            }
            // 
            // mnuRedo
            //
            mnuItem          = new System.Windows.Forms.MenuItem();
            mnuItem.Index    = 13;
            mnuItem.Shortcut = System.Windows.Forms.Shortcut.CtrlY;
            mnuItem.Text     = mnuRedo.Text;
            mnuEdit.MenuItems.Add(mnuItem);
            if (CommandHolder != null) {
                CommandHolder.AddTrigger(null, "Redo", new object[] {mnuItem});
            }
            // 
            // mnuEditSep3
            //
            mnuItem       = new System.Windows.Forms.MenuItem();
            mnuItem.Index = 14;
            mnuItem.Text  = "-";
            mnuEdit.MenuItems.Add(mnuItem);
            // 
            // mnuFind
            //
            mnuItem          = new System.Windows.Forms.MenuItem();
            mnuItem.Index    = 15;
            mnuItem.Shortcut = System.Windows.Forms.Shortcut.CtrlF;
            mnuItem.Text     = mnuFind.Text;
            mnuEdit.MenuItems.Add(mnuItem);
            if (CommandHolder != null) {
                CommandHolder.AddTrigger(null, "Find", new object[] {mnuItem});
            }
        }
    }       
}           
