using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.ComponentModel;
using DevComponents.DotNetBar;

namespace RepPP.Toolkit.Window.Ext {

    //**************************************************************************
    //
    /// <summary>
    /// TabControlSB:			        SDStudio Multi Selection Buffer Tab Control
    /// </summary>
    //
    //**************************************************************************
    public class TabControlSB : TabControlExt {

        //******************************************************************
        //
        /// <summary>
        /// ctor:                            Class constructor
        /// </summary>
        //
        //******************************************************************
        public TabControlSB(): base() {
            m_ctxMenu.Popup += new EventHandler(m_ctxMenu_Popup);
            this.TabMoved   += new DevComponents.DotNetBar.TabStrip.TabMovedEventHandler(TabControlSB_TabMoved);
        }

        //******************************************************************
        //
		/// <summary>
		/// Dispose:                    Clean up any resources being used.
		/// </summary>
		/// <param name="bDisposing">   true if disposing, false if finalizing</param>
        //
        //******************************************************************
        protected override void Dispose(bool bDisposing) {
            if (bDisposing) {
                if (m_ctxMenu != null) {
                    m_ctxMenu.Popup -= new EventHandler(m_ctxMenu_Popup);
                }
                this.TabMoved -= new DevComponents.DotNetBar.TabStrip.TabMovedEventHandler(TabControlSB_TabMoved);
            }
            base.Dispose(bDisposing);
        }

        //******************************************************************        
        //
        /// <summary>
        /// ActiveRowset:        Gets the Active Selection Buffer 
        ///                             Rowset
        /// </summary>
        //
        //******************************************************************  
        public RepPP.Rowset ActiveRowset {
            get {
                RepPP.Rowset rsRetVal = null;

                if (SelectedTab != null && SelectedTab is TabItemSB) {
                    rsRetVal = ((TabItemSB)SelectedTab).Rowset;
                }
                return(rsRetVal);
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// IsSBTabRoot:                 Indicate if current tab selected
        ///                              is the root Selection Buffer (first tab)
        /// </summary>
        //
        //******************************************************************   
        public bool IsSBTabRoot {
            get {
                return (SelectedTab.Equals(Tabs[0]));
            }
        }

        //******************************************************************
        //
		/// <summary>
        /// Gets the tab Root
		/// </summary>
        //
        //******************************************************************
        public TabItem TabRoot {
            get {
                return(Tabs[0]);
            }
        }

        //******************************************************************
        //
		/// <summary>
        /// CreateTabFactory:           Tab factory
		/// </summary>
        //
        //******************************************************************
        protected override TabItemExt CreateTabFactory() {
            return(new TabItemSB(this));
        }

        //******************************************************************        
        //
        /// <summary>
        /// SelectSBTabRoot:             Select tab root 
        ///                              Selection Buffer (first tab)
        /// </summary>
        //
        //******************************************************************   
        public void SelectSBTabRoot() {
            SelectedTab = TabRoot; 
        }

        //******************************************************************
        //
		/// <summary>
        /// m_ctxMenu_Popup:            Occurs when before the context menu
        ///                             is show
		/// </summary>
        /// <param name="sender">       Sender</param>
        /// <param name="e">            Event arguments</param>
        //
        //******************************************************************
        private void m_ctxMenu_Popup(object sender, EventArgs e) {
            m_ctxMenu.MenuItems["miClose"].Enabled = !IsSBTabRoot;         
        }

        //******************************************************************
        //
		/// <summary>
        /// TabControlSB_TabMoved:      Occurs when tab item is moved
		/// </summary>
        /// <param name="sender">       Sender</param>
        /// <param name="e">            Tab Move event arguments</param>
        //
        //******************************************************************
        private void TabControlSB_TabMoved(object sender, DevComponents.DotNetBar.TabStripTabMovedEventArgs e) {
            if (e.NewIndex == 0 || e.OldIndex == 0) {
                e.Cancel = true;
            }
        }  

    } // End class TabControlSB

    //**************************************************************************
    //
    /// <summary>
    /// TabItemSB:			        SDStudio Selection Buffer Tab Item
    /// </summary>                  
    //
    //**************************************************************************
    public class TabItemSB : TabItemExt {

        /// <summary>Selection Buffer</summary>
        private RepPP.Toolkit.Window.ListViewRowset           m_listviewSB;
        /// <summary>Selection Buffer RowsetTree</summary>
        private RepPP.RowsetTree                              m_rst;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                            Class constructor
        /// </summary>
        /// <param name="tabControlParent">  Tab Control parent</param>
        //
        //******************************************************************
        public TabItemSB(DevComponents.DotNetBar.TabControl tabControlParent) : base(tabControlParent) {
            CreateListview();
        }

        //******************************************************************
        //
        /// <summary>
        /// ctor:                            Class constructor
        /// </summary>
        /// <param name="tabControlParent">  Tab Control parent</param>
        /// <param name="rstSB">             Selection Buffer RowsetTree</param>
        //
        //******************************************************************
        public TabItemSB(DevComponents.DotNetBar.TabControl tabControlParent, RepPP.RowsetTree rstSB) : base(tabControlParent) {
            m_rst = rstSB;
            CreateListview();
            Fill(rstSB);
        }

        //******************************************************************
        //
		/// <summary>
		/// Dispose:                    Clean up any resources being used.
		/// </summary>
		/// <param name="bDisposing">   true if disposing, false if finalizing</param>
        //
        //******************************************************************
        protected override void Dispose(bool bDisposing) {
            if (bDisposing) {
                if (m_listviewSB != null) {
                    m_listviewSB.Dispose();
                    m_listviewSB = null;
                }
            }
           base.Dispose(bDisposing);
        }

        //******************************************************************        
        //
        /// <summary>
        /// Listview:                    Gets the Selection Buffer listview
        /// </summary>
        //
        //******************************************************************    
        public RepPP.Toolkit.Window.ListViewRowset Listview {
            get {
                return(m_listviewSB);
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// RowsetTree:                Gets/Sets the Selection Buffer 
        ///                              Cardinstance containing root data
        /// </summary>
        //
        //******************************************************************    
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public RepPP.RowsetTree RowsetTree {
            get {
                return(m_rst);
            }
            set {
                m_rst = value;
                if (value != null) {
                    Fill(value);
                }
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// Rowset:               Gets the Selection Buffer Groupinstance
        /// </summary>
        //
        //******************************************************************    
        public RepPP.Rowset Rowset {
            get {
                RepPP.Rowset rsRetVal = null;

                if (m_listviewSB != null) {
                    rsRetVal = m_listviewSB.Rowset;
                }
                return(rsRetVal);
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// Fill:                        Fill the Selection Buffer 
        /// </summary>
        /// <param name="ginstSB">       Root Selection Buffer RowsetTree
        ///                              containing Rowset data</param>
        //
        //******************************************************************    
        public void Fill(RepPP.RowsetTree rstSB) {
            if (m_listviewSB == null) {
                throw new ApplicationException("Listview is not initialized !");
            }
            m_listviewSB.Fill(rstSB.RootRowset, false);
        }

        //******************************************************************        
        //
        /// <summary>
        /// CreateListview:              Create Listview containing 
        ///                              Selection Buffer 
        /// </summary>
        //
        //******************************************************************    
        protected virtual void CreateListview() {
            m_listviewSB                      = new RepPP.Toolkit.Window.ListViewRowset();
            m_listviewSB.AllowColumnReorder   = true;
            m_listviewSB.AutoSort             = true;
            m_listviewSB.Dock                 = DockStyle.Fill;
            m_listviewSB.FullRowSelect        = true;
            m_listviewSB.HideSelection        = false;
            m_listviewSB.KeySearch            = true;
            m_listviewSB.TabStop              = false;
            m_listviewSB.IsUserFlagFieldShown = false;
            m_listviewSB.LabelEdit            = false;
            m_listviewSB.View                 = View.Details;
            m_listviewSB.Name                 = "lvSB";
            AttachedControl.Controls.Add(m_listviewSB);
        }
    } // End class TabItemSB
} // End namespace
