using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using RepPP.Toolkit.Window;


namespace RepPP.Toolkit.Window.Ext {
    //******************************************************************        
    //
	/// <summary>
	/// This base class implement common functionalities of all top
	/// editor. Top editor are managed form which are child of the main
	/// form. All top form (except the main form) are deriving from this
	/// class.
	/// </summary>
	//
    //******************************************************************        
	public class frmEditor : frmManagedForm	{
		private   System.ComponentModel.IContainer  components       = null;
		protected frmBaseMain                       m_frmFather      = null;    // Main form

        //******************************************************************        
        //
        /// <summary>
        /// ctor:                   Class constructor
        /// </summary>
        //
        //******************************************************************        
        public frmEditor() 
        {
            InitializeComponent();
        }

        //******************************************************************        
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        /// <param name="frmFather">        Father form</param>
        //
        //******************************************************************        
        public frmEditor(frmBaseMain frmFather) : base(frmFather) 
        {
            InitializeComponent();
            m_frmFather = frmFather;
            if (frmFather != null) {
                m_configHelper = frmFather.ConfigHelper;
            }
        }

        //******************************************************************        
        //
        /// <summary>
        /// Dispose:                    Dispose the form
        /// </summary>
        /// <param name="bDisposing">   true if disposing, false if finalizing</param>
        //
        //******************************************************************        
        protected override void Dispose(bool bDisposing) 
        {
            if (bDisposing) {
                if (components != null)	{
                    components.Dispose();
                }
            }
            base.Dispose(bDisposing);
        }

        //******************************************************************
        //
        /// <summary>
        /// MainForm:    Return the attached main form
        /// </summary>
        //
        //******************************************************************
        public frmBaseMain MainForm 
        {
            get {
                    return (m_frmFather);
                }
        }

        //******************************************************************
        //
        /// <summary>
        /// SetFormTextV:                   Set the text of the form using an atom
        /// </summary>
        /// <param name="strTitle">         Form title</param>
        //
        //******************************************************************
        public virtual void SetFormTextV(string strTitle) 
        {
            Text = m_app.Tool.FormatMessage(strTitle + " - %1 - %2",
                                            new object[] {m_app.DataConnection.Name, m_app.UserName});
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnFileSaveConfigClick:				Save UI Layout
        /// </summary>
        //
        //********************************************************************************
        protected override void OnFileSaveConfigClick() 
        {
            string strConfigSaved;
            string strTitleConfig;

            try {
                SaveConfigurationToFile();
                if (m_app.IsAtomExist("CONFIGURATION_SAVED", RepPP.AtomType.sdAtomMsg)) {
                    strConfigSaved = m_app.GetAtomText("CONFIGURATION_SAVED",RepPP.AtomType.sdAtomMsg);
                } else {
                    strConfigSaved = "The configuration has been properly saved";
                }
                if (m_app.IsAtomExist("TITLE_CONFIGURATION_SAVED", RepPP.AtomType.sdAtomMsg)) {
                    strTitleConfig = m_app.GetAtomText("TITLE_CONFIGURATION_SAVED", RepPP.AtomType.sdAtomMsg);
                } else {
                    strTitleConfig = "Configuration";
                }
                MessageBox.Show(strConfigSaved,
                                strTitleConfig,
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Information);
            } catch (Exception ex) {
                if (m_app.IsAtomExist("TITLE_CONFIGURATION_NOT_SAVED", RepPP.AtomType.sdAtomMsg)) {
                    strTitleConfig = m_app.GetAtomText("TITLE_CONFIGURATION_NOT_SAVED", RepPP.AtomType.sdAtomMsg);
                } else {
                    strTitleConfig = "Configuration not saved";
                }
                MessageBox.Show(ex.Message + ex.StackTrace,
                                strTitleConfig,
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnManagedFormClosed:    Remove the form from the list when its closing
        /// </summary>
        /// <param name="sender">   Event sender</param>
        /// <param name="e">        Event arguments</param>
        //
        //******************************************************************
        protected override void OnManagedFormClosed(object sender, System.EventArgs e) 
        {
            base.OnManagedFormClosed(sender, e);
            if (m_frmFather != null) {
                if (m_frmFather.Equals(Owner)) {
                    Owner = null; // Patch to avoid the infinity loop during the closing of its father 
                }
                m_frmFather.ChildFormClosing(this);
            }
        }

		#region Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            // 
            // frmEditor
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.BackColor = System.Drawing.SystemColors.Control;
            this.ClientSize = new System.Drawing.Size(292, 273);
            this.Name = "frmEditor";

        }
		#endregion
	}
}

