using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using RepPP;
using System.Xml;
using DevComponents.DotNetBar;
using RepPP.Toolkit.Window;

namespace RepPP.Toolkit.Window.Ext {

    //*********************************************************
    //
    /// <summary>
    /// Base class for all managed form. Managed form are top form which
    /// are managed by the main form. They can be minimized, activated, arranged
    /// etc. This class handle all the function of the Windows menu.
    /// </summary>
    //
    //*********************************************************
    public class frmManagedForm : frmSerializabledForm, IManagedForm 
    {
        public enum ArrangeStyleE {
            Vertical    = 1,                            // Tiled verticaly
            Horizontal  = 2,                            // Tiled horizontaly
            Cascade     = 3                             // Tiled in cascade
        };

        private		System.ComponentModel.IContainer	components			= null;
        public      SCBarCmdHolder                      m_cmdHolder;                    // Handle command/trigger
        protected	Application						    m_app;							// Pointer to Rep++ application object
        private		const int                           m_iBISepPos			=  6;		// Button Item Last Seperator position in main menu
        private		ArrayList                           m_arrTopForm;					// Stores all opened forms
        protected	ButtonItem							m_mnuCmdWindows		= null;		// Menu Item Windows
        private		Form								m_frmActiveForm		= null;		// Active Form			

        //******************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //******************************************************************
        public frmManagedForm() 
        {
            InitializeComponent();
        }

        //******************************************************************        
        //
        /// <summary>
        /// ctor:				        Class constructor
        /// </summary>
        /// <param name="frmFather">    Father form</param>
        //
        //******************************************************************        
        public frmManagedForm(frmManagedForm frmFather)
        {
            InitializeComponent();
            InitV(frmFather);
        }

        //******************************************************************        
        //
        /// <summary>
        /// ctor:				            Class constructor
        /// </summary>
        /// <param name="app">              Rep++ Application object</param>
        /// <param name="arrTopForm">       Array of top form</param>
        /// <param name="frmActiveForm">    Active form</param>
        //
        //******************************************************************        
        public frmManagedForm(RepPP.Application app, ArrayList arrTopForm, Form frmActiveForm) {
            InitializeComponent();
            InitV(app, arrTopForm, frmActiveForm);
        }

        //******************************************************************        
        //
        /// <summary>
        /// InitV:				            Initialize the class
        /// </summary>
        /// <param name="app">              Rep++ Application object</param>
        /// <param name="arrTopForm">       Array of top form</param>
        /// <param name="frmActiveForm">    Active form</param>
        //
        //******************************************************************        
        protected virtual void InitV(RepPP.Application app, ArrayList arrTopForm, Form frmActiveForm) {
            m_cmdHolder      = new SCBarCmdHolder();
            m_app			 = app;
            m_arrTopForm	 = arrTopForm;
            m_frmActiveForm	 = frmActiveForm;
        }

        //******************************************************************        
        //
        /// <summary>
        /// InitV:				        Initialize the class
        /// </summary>
        /// <param name="frmFather">    Father manage form</param>
        //
        //******************************************************************        
        protected virtual void InitV(frmManagedForm frmFather)
        {
            m_cmdHolder      = new SCBarCmdHolder();
            if (frmFather != null) {
                m_app			 = frmFather.Application;
                m_arrTopForm	 = frmFather.ManagedFormListArray;
                m_arrTopForm.Add(this);
            }
            this.Closed     += new System.EventHandler(OnManagedFormClosed);
            m_frmActiveForm	 = this;
        }


        //******************************************************************
        //
        /// <summary>
        /// Dispose:                        Dispose the form
        /// </summary>
        /// <param name="bDisposing">       true to dispose, false for finalize</param>
        //
        //******************************************************************
        protected override void Dispose(bool bDisposing) 
        {
            if (bDisposing) {
                if (components != null) {
                    components.Dispose();
                }
            }
            base.Dispose(bDisposing);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnFormLoad:                     Load the form state when form is loading
        /// </summary>
        /// <param name="sender">           Event sender</param>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        protected override void OnLoad(System.EventArgs e) {
            InitMenu();
            base.OnLoad(e);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// InitMenu:                       Initialize the menu
        /// </summary>
        //
        //*********************************************************	    
        protected virtual void InitMenu() {
            if (m_app != null) {
                InitWindowsMenuV();
                m_cmdHolder.AddCmd("FileSaveConfig",
                                   new CmdHolder.OnCommand(OnFileSaveConfigClick),    CmdHolder.CmdStateE.Enabled);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ButtonItemWindows:         Button Item
        /// </summary>
        //
        //*********************************************************	    
        public ButtonItem ButtonItemWindows {
            get {
                return(m_mnuCmdWindows);
            }
            set {
                m_mnuCmdWindows = value;
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Application:                Rep++ Application object
        /// </summary>
        //
        //*********************************************************	    
        public RepPP.Application Application {
            get {
                return(m_app);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// InitWindowsMenuV:           Initialize the windows menu
        /// </summary>
        //
        //******************************************************************
        public void InitWindowsMenuV() {
            ButtonItem  miCmdMinimizeAll;
            ButtonItem	miCmdMaximizeAll;
            ButtonItem	miCmdRestoreAll;
            ButtonItem	miCmdArrangeVertically;
            ButtonItem	miCmdArrangeHorizontally;
            ButtonItem	miCmdArrangeInCascade;
            ButtonItem	miCmdCloseAll;
			
            if (m_mnuCmdWindows == null) {
                m_mnuCmdWindows = new ButtonItem();
            }
            m_mnuCmdWindows.SubItems.Clear();            
            miCmdMinimizeAll					= new ButtonItem("miWindowsMinimizeAll", "M&inimize All");
            miCmdRestoreAll					    = new ButtonItem("miWindowsRestoreAll",  "&Restore All");
            miCmdMaximizeAll					= new ButtonItem("miWindowsMaximizeAll", "Ma&ximize All");
            miCmdArrangeVertically				= new ButtonItem("miWindowsArrangeVertically", "Arrange &Vertically");
            miCmdArrangeHorizontally			= new ButtonItem("miWindowsArrangeHorizontally", "Arrange &Horizontally");
            miCmdArrangeInCascade				= new ButtonItem("miWindowsArrangeCascade", "&Cascade");
            miCmdCloseAll						= new ButtonItem("miWindowsCloseAll", "Close &All");
            miCmdArrangeVertically.BeginGroup	= true;
            miCmdCloseAll.BeginGroup            = true;

			m_mnuCmdWindows.SubItems.Add(miCmdMinimizeAll);
			m_mnuCmdWindows.SubItems.Add(miCmdMaximizeAll);
			m_mnuCmdWindows.SubItems.Add(miCmdRestoreAll);
			m_mnuCmdWindows.SubItems.Add(miCmdArrangeVertically);
			m_mnuCmdWindows.SubItems.Add(miCmdArrangeHorizontally);
			m_mnuCmdWindows.SubItems.Add(miCmdArrangeInCascade);
			m_mnuCmdWindows.SubItems.Add(miCmdCloseAll);

//            m_mnuCmdWindows.SubItems.AddRange(new ButtonItem[] {   miCmdMinimizeAll,
//                                                                   miCmdMaximizeAll,
//                                                                   miCmdRestoreAll,
//                                                                   miCmdArrangeVertically,
//                                                                   miCmdArrangeHorizontally,
//                                                                   miCmdArrangeInCascade,
//                                                                   miCmdCloseAll});
            m_cmdHolder.AddCmd("WindowsMinimizeAll",
                               new CmdHolder.OnCommand(OnWindowsMinimizeAllClick),        CmdHolder.CmdStateE.Disabled);
            m_cmdHolder.AddCmd("WindowsMaximizeAll",
                               new CmdHolder.OnCommand(OnWindowsMaximizeAllClick),        CmdHolder.CmdStateE.Disabled);
            m_cmdHolder.AddCmd("WindowsRestoreAll",
                               new CmdHolder.OnCommand(OnWindowsRestoreAllClick),         CmdHolder.CmdStateE.Disabled);
            m_cmdHolder.AddCmd("WindowsArrangeVertically",
                               new CmdHolder.OnCommand(OnWindowsArrangeVerticallyClick),  CmdHolder.CmdStateE.Disabled);
            m_cmdHolder.AddCmd("WindowsArrangeHorizontally",
                               new CmdHolder.OnCommand(OnWindowsArrangeHorizontallyClick),CmdHolder.CmdStateE.Disabled);
            m_cmdHolder.AddCmd("WindowsArrangeInCascade",
                               new CmdHolder.OnCommand(OnWindowsArrangeInCascadeClick),   CmdHolder.CmdStateE.Disabled);
            m_cmdHolder.AddCmd("WindowsCloseAll",
                               new CmdHolder.OnCommand(OnWindowsCloseAllClick),           CmdHolder.CmdStateE.Disabled);
            m_cmdHolder.AddTrigger(null, "WindowsMinimizeAll",         new object[] { miCmdMinimizeAll });
            m_cmdHolder.AddTrigger(null, "WindowsMaximizeAll",         new object[] { miCmdMaximizeAll });
            m_cmdHolder.AddTrigger(null, "WindowsRestoreAll",          new object[] { miCmdRestoreAll });
            m_cmdHolder.AddTrigger(null, "WindowsArrangeVertically",   new object[] { miCmdArrangeVertically });
            m_cmdHolder.AddTrigger(null, "WindowsArrangeHorizontally", new object[] { miCmdArrangeHorizontally });
            m_cmdHolder.AddTrigger(null, "WindowsArrangeInCascade",    new object[] { miCmdArrangeInCascade });
            m_cmdHolder.AddTrigger(null, "WindowsCloseAll",            new object[] { miCmdCloseAll });
            m_mnuCmdWindows.ExpandChange += new System.EventHandler(OnWindowsMenuPopup);
        }

        //******************************************************************
        //
        /// <summary>
        /// RefreshWindowsMenuV:		Adjust Windows menu
        /// </summary>
        /// <param name="bi">           Menu item to adjust</param>
        /// <param name="frmThis">      Currently active form</param>
        //
        //******************************************************************
        public void RefreshWindowsMenuV(ButtonItem bi, Form frmThis) 
        {
            int         iCount = 1;
            int         iChildCount;
            string		strText;
            ButtonItem  buttonItem;

            if (m_arrTopForm != null) {
                iChildCount            = m_arrTopForm.Count;
                m_cmdHolder.SetCmdState("WindowsMinimizeAll",
                                        (iChildCount != 0) ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled);
                m_cmdHolder.SetCmdState("WindowsMaximizeAll",
                                        (iChildCount != 0) ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled);
                m_cmdHolder.SetCmdState("WindowsRestoreAll",
                                        (iChildCount != 0) ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled);
                m_cmdHolder.SetCmdState("WindowsArrangeVertically",
                                        (IsArrangeAvailableB(ArrangeStyleE.Vertical)) ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled);
                m_cmdHolder.SetCmdState("WindowsArrangeHorizontally",
                                        (IsArrangeAvailableB(ArrangeStyleE.Horizontal)) ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled);
                m_cmdHolder.SetCmdState("WindowsArrangeInCascade",
                                        (IsArrangeAvailableB(ArrangeStyleE.Cascade)) ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled);
                m_cmdHolder.SetCmdState("WindowsCloseAll",
                                        (iChildCount != 0) ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled);
                while (bi.SubItems.Count > m_iBISepPos) {
                    bi.SubItems.RemoveAt(m_iBISepPos);
                }
                if (iChildCount != 0) {
                    foreach (Form frmChild in m_arrTopForm) {
                        strText				    = iCount.ToString() + " " + frmChild.Text;
                        buttonItem				= new ButtonItem("mi" + iCount.ToString(), "&" + strText) ;
                        buttonItem.Tag		    = iCount;				
                        buttonItem.Click	   += new System.EventHandler(OnManagedFormSelect);
                        buttonItem.BeginGroup   = (iCount == 1);
                        if (frmChild == frmThis) {
                            buttonItem.Checked = true;
                        }
                        bi.SubItems.Add(buttonItem);
                        iCount++;
                    }
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnWindowsMenuPopup:		Refresh the windows menu on popup
        /// </summary>
        /// <param name="sender">   Event sender</param>
        /// <param name="e">        Event arguments</param>
        //
        //******************************************************************
        private void OnWindowsMenuPopup(object sender, EventArgs e) 
        {
            RefreshWindowsMenuV(m_mnuCmdWindows, m_frmActiveForm);
        }

        //******************************************************************
        //
        /// <summary>
        /// OnManagedFormSelect:    Activate the top form when menu is triggered
        /// </summary>
        /// <param name="sender">   Event sender</param>
        /// <param name="e">        Event arguments</param>
        //
        //******************************************************************
        public virtual void OnManagedFormSelect(object sender, EventArgs e) 
        {
            ButtonItem		bi;
            int				iIndex;
            Form			frmChild;
		    
            if (m_arrTopForm != null) {
                bi			= (ButtonItem)sender;
                iIndex		= ((int)bi.Tag) - 1;
                if (iIndex >= 0 && iIndex < m_arrTopForm.Count) {
                    frmChild = (Form)m_arrTopForm[iIndex];
                    frmChild.Activate();
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnManagedFormClosed:    Remove the form from the list when its closing
        /// </summary>
        /// <param name="sender">   Event sender</param>
        /// <param name="e">        Event arguments</param>
        //
        //******************************************************************
        protected virtual void OnManagedFormClosed(object sender, System.EventArgs e) 
        {
            if (m_arrTopForm != null) {
                m_arrTopForm.Remove(sender);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// FindManagedForm:        Find a managed form in the array using its type
        /// </summary>
        /// <param name="type">     Form type to search for</param>
        /// <returns>
        /// Top form or null if not found
        /// </returns>
        //
        //******************************************************************
        public Form FindManagedForm(System.Type type) 
        {
            Form    frmRetVal = null;
		    
            if (m_arrTopForm != null) {
                foreach(Form frm in m_arrTopForm) {
                    if (frm.GetType() == type) {
                        frmRetVal = frm;
                        break;
                    }
                }
            }
            return(frmRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// ManagedFormListArray:   Gets the top form array
        /// </summary>
        //
        //******************************************************************
        public ArrayList ManagedFormListArray 
        {
            get {
                return(m_arrTopForm);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CloseAllManagedFormB:       Try to close all managed form
        /// </summary>
        /// <returns>
        /// true if succeed, false if not
        /// </returns>
        //
        //******************************************************************
        public bool CloseAllManagedFormB() 
        {
            bool    bRetVal = true;
		    
            if (m_arrTopForm != null) {
                foreach (Form childForm in m_arrTopForm) {
                    if (((IManagedForm)childForm).IsDirtyB()) {
                        if (!((IManagedForm)childForm).SaveB()) {
                            bRetVal = false;
                            break;
                        }
                    }
                }
                if (bRetVal) {
                    while (m_arrTopForm.Count != 0) {
                        ((Form)m_arrTopForm[0]).Close();
                    }
                }
            }
            return(bRetVal);		    
        }

        //******************************************************************
        //
        /// <summary>
        /// IsArrangeAvailableB:        Returns true if the top form can be arranged
        ///                             using the specified method
        /// </summary>
        /// <param name="eArrangeStyle">Method to use to arrange the top form</param>
        /// <returns>
        /// false if no, true if yes
        /// </returns>
        //******************************************************************
        public bool IsArrangeAvailableB(ArrangeStyleE eArrangeStyle) 
        {
            bool    bRetVal;

            if (m_arrTopForm != null) {
                if (m_arrTopForm.Count < 2) {
                    bRetVal = false;
                } else {
                    switch(eArrangeStyle) {
                    case ArrangeStyleE.Cascade:
                        bRetVal = true;
                        break;
                    case ArrangeStyleE.Horizontal:
                        bRetVal = (m_arrTopForm.Count == 2);
                        break;
                    default:
                        bRetVal = (m_arrTopForm.Count <= 8);
                        break;
                    }
                }
            } else {
                bRetVal = false;
            }
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// ArrangeAllManagedFormV:     Arrange top form using the specified method
        /// </summary>
        /// <param name="eArrangeStyle">Method to use</param>
        //
        //******************************************************************
        public void ArrangeAllManagedFormV(ArrangeStyleE eArrangeStyle) 
        {
            int         iCount;
            int         iDeltaX;
            int         iDeltaY;
            int         iTop;
            int         iLeft;
            int         iWidth;
            int         iHeight;
            Rectangle   rectFormPos;

            if (IsArrangeAvailableB(eArrangeStyle)) {
                rectFormPos = SystemInformation.WorkingArea;
                iTop        = rectFormPos.Top;
                iLeft       = rectFormPos.Left;
                iWidth      = rectFormPos.Width;
                iHeight     = rectFormPos.Height;
                iCount      = m_arrTopForm.Count;
                switch(eArrangeStyle) {
                case ArrangeStyleE.Horizontal:
                    iWidth      /= iCount;
                    iDeltaX      = iWidth;
                    iDeltaY      = 0;
                    break;                
                case ArrangeStyleE.Vertical:
                    iHeight     /= iCount;
                    iDeltaX      = 0;
                    iDeltaY      = iHeight;
                    break;
                default:
                    iDeltaX      = SystemInformation.HorizontalScrollBarThumbWidth;
                    iDeltaY      = SystemInformation.CaptionHeight;
                    iWidth      -= (iCount - 1) * iDeltaX;
                    iHeight     -= (iCount - 1) * iDeltaY;
                    break;
                }
                foreach (Form frm in m_arrTopForm) {
                    if (frm.WindowState != FormWindowState.Normal) {
                        frm.WindowState = FormWindowState.Normal;
                    }
                    frm.SetDesktopBounds(iLeft, iTop, iWidth, iHeight);
                    frm.BringToFront();
                    iLeft += iDeltaX;
                    iTop  += iDeltaY;
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// MinimizeAllManagedFormV:        Mimimize all top form
        /// </summary>
        //
        //******************************************************************
        public void MinimizeAllManagedFormV() 
        {
            if (m_arrTopForm != null) {
                foreach (Form childForm in m_arrTopForm) {
                    if (childForm.WindowState != FormWindowState.Minimized) {
                        childForm.WindowState = FormWindowState.Minimized;
                    }
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// RestoreAllManagedFormV:         Restore all top form
        /// </summary>
        //
        //******************************************************************
        public void RestoreAllManagedFormV() 
        {
            if (m_arrTopForm != null) {
                foreach (Form childForm in m_arrTopForm) {
                    if (childForm.WindowState != FormWindowState.Normal) {
                        childForm.WindowState = FormWindowState.Normal;
                    }
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// MaximizeAllManagedFormV:        Maximize all top form
        /// </summary>
        //
        //******************************************************************
        public void MaximizeAllManagedFormV() {
            if (m_arrTopForm != null) {
                foreach (Form childForm in m_arrTopForm) {
                    if (childForm.WindowState != FormWindowState.Maximized) {
                        childForm.WindowState = FormWindowState.Maximized;
                    }
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnWindowsMinimizeAllClick:          Minimize command
        /// </summary>
        //
        //******************************************************************
        private void OnWindowsMinimizeAllClick() 
        {
            MinimizeAllManagedFormV();
        }

        //******************************************************************
        //
        /// <summary>
        /// OnWindowsRestoreAllClick:           Restore command
        /// </summary>
        //
        //******************************************************************
        private void OnWindowsRestoreAllClick() 
        {
            RestoreAllManagedFormV();
        }

        //******************************************************************
        //
        /// <summary>
        /// OnWindowsMaximizeAllClick:          Maximize command
        /// </summary>
        //
        //******************************************************************
        private void OnWindowsMaximizeAllClick() 
        {
            MaximizeAllManagedFormV();
        }

        //******************************************************************
        //
        /// <summary>
        /// OnWindowsArrangeVerticallyClick:   Arrange vertically command
        /// </summary>
        //
        //******************************************************************
        private void OnWindowsArrangeVerticallyClick() 
        {
            ArrangeAllManagedFormV(ArrangeStyleE.Vertical);
        }

        //******************************************************************
        //
        /// <summary>
        /// OnWindowsArrangeHorizontallyClick:  Arrange horizontally command
        /// </summary>
        //
        //******************************************************************
        private void OnWindowsArrangeHorizontallyClick() 
        {
            ArrangeAllManagedFormV(ArrangeStyleE.Horizontal);
        }

        //******************************************************************
        //
        /// <summary>
        /// OnWindowsArrangeInCascadeClick:     Cascade command
        /// </summary>
        //
        //******************************************************************
        private void OnWindowsArrangeInCascadeClick() 
        {
            ArrangeAllManagedFormV(ArrangeStyleE.Cascade);
        }

        //******************************************************************
        //
        /// <summary>
        /// OnWindowsCloseAllClick:             Close all command
        /// </summary>
        //
        //******************************************************************
        private void OnWindowsCloseAllClick() 
        {
            CloseAllManagedFormB();
        }

        //******************************************************************
        //
        /// <summary>
        /// IsDirtyB:			        Check if managed form is dirty
        /// </summary>
        /// <returns>
        /// true if yes, false if not.
        /// </returns>
        //
        //******************************************************************
        public virtual bool IsDirtyB() 
        {
            bool bRetVal;

            bRetVal = false;
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// SaveB:                      Save the content of the form
        /// </summary>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //
        //******************************************************************
        public virtual bool SaveB() 
        {
            bool bRetVal;

            bRetVal = true;
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// BarManager:                 Manager responsible for windows docking
        /// </summary>
        //
        //********************************************************************************
        protected virtual DotNetBarManager BarManager 
        {
            get {
                return(null);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// LoadState:                      Load the form state.
        ///                                 Implementing RepPPToolkit.IUIConfiguration
        /// </summary>
        /// <param name="configHelper">     Configuration helper object</param>
        /// <param name="node">             Node containing the data</param>
        //
        //********************************************************************************
//        public override void LoadState(RepPPToolkit.SCUIConfigurationHelper configHelper, XmlNode node) 
//        {
//            string                      strVersion;
//            SCBarUIConfigurationHelper  helper;
//            DotNetBarManager            barManager;
//            
//            base.LoadState(configHelper, node);
//            strVersion = configHelper.GetNodeAttribute(node, "VERSION");
//            if (strVersion != null && strVersion == "1") {
//                barManager = BarManager;
//                if (configHelper is SCBarUIConfigurationHelper && barManager != null) {
//                    helper = (SCBarUIConfigurationHelper)configHelper;
//                    helper.LoadBarState(node, barManager);
//                }
//            }
//        }

        //********************************************************************************
        //
        /// <summary>
        /// SaveState:                      Save the form state
        ///                                 Implementing RepPPToolkit.IUIConfiguration
        /// </summary>
        /// <param name="configHelper">     Configuration helper object</param>
        /// <param name="node">             Node containing the data</param>
        //
        //********************************************************************************
//        public override void SaveState(RepPPToolkit.SCUIConfigurationHelper configHelper, XmlNode node) 
//        {
//            SCBarUIConfigurationHelper  helper;
//            DotNetBarManager            barManager;
//
//            base.SaveState(configHelper, node);
//            barManager = BarManager;
//            if (configHelper is SCBarUIConfigurationHelper && barManager != null) {
//                helper = (SCBarUIConfigurationHelper)configHelper;
//                helper.SaveBarState(node, barManager);
//            }
//        }

        #region Windows Form Designer generated code

        private void InitializeComponent()
        {
            // 
            // frmManageForm
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.BackColor = System.Drawing.Color.FromArgb(((System.Byte)(224)), ((System.Byte)(224)), ((System.Byte)(224)));
            this.ClientSize = new System.Drawing.Size(292, 273);
            this.Name = "frmManageForm";
            this.Text = "frmManageForm";

        }
        #endregion

    } /*frmManageForm*/
} // Namespace
