//
// Rep++ Framework for Windows Application
// Copyright (C) 2005 Consyst SQL Inc.
// All Rights Reserved
//
// This source code is only intended as a supplement to the
// Rep++ Studio products.
//
using System;
using System.Collections;
using RepPP;
using RepPP.Toolkit.Window;

namespace RepPP.Toolkit.Window {

    //*********************************************************
    //
    /// <summary>
    /// LineSelectedInListEventArgs:  Arguments of the event when a new line is selected
    /// </summary>
    //
    //*********************************************************
    public class LineSelectedInListEventArgs : System.EventArgs {
        /// <summary>Line which was selected before the event</summary>
        public  int         OldLine;
        /// <summary>New selected line</summary>
        public  int         NewLine;

        //*********************************************************
        //
        /// <summary>
        /// ctor:                   Class constructor
        /// </summary>
        /// <param name="OldLine">  Line previously selected</param>
        /// <param name="NewLine">  Line newly selected</param>
        //
        //*********************************************************
        public LineSelectedInListEventArgs(int OldLine, int NewLine) : base() { this.OldLine = OldLine; this.NewLine = NewLine; }

        //*********************************************************
        //
        /// <summary>
        /// ctor:                   Class constructor
        /// </summary>
        /// <param name="NewLine">  Line newly selected</param>
        //
        //*********************************************************
        public LineSelectedInListEventArgs(int NewLine) : this(-1, NewLine) {}
    }; // Class LineSelectedEventArgs

    /// <summary>Delegate for the event</summary>
    public delegate void LineSelectedInListEventHandler(object sender, LineSelectedInListEventArgs e);

    /// <summary>
    /// ILineListHandler:       Interface for object handling a list of lines
    /// </summary>
	public interface ILineListHandler {
        /// <summary> Event triggered when the line list handler changes the selection</summary>
        event LineSelectedInListEventHandler LineSelectedInList;
        //*********************************************************     
        //
        /// <summary>
        /// SelectLine:                     Select a line
        /// </summary>
        /// <param name="iPos">             Line position</param>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //  
        //*********************************************************     
		bool SelectLine(int iPos);
        //*********************************************************     
        //
        /// <summary>
        /// InsertLine:                     Insert a line
        /// </summary>
        /// <param name="bAtEnd">           true to insert at the end of the list, false at the current position</param>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //  
        //*********************************************************     
		bool InsertLine(bool bAtEnd);
        //*********************************************************     
        //
        /// <summary>
        /// DuplicateLine:                  Duplicates a line
        /// </summary>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //  
        //*********************************************************     
		bool DuplicateLine();
        //*********************************************************     
        //
        /// <summary>
        /// DeleteCurrentLine:              Delete the current line
        /// </summary>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //  
        //*********************************************************     
		bool DeleteCurrentLine();
        //*********************************************************     
        //
        /// <summary>
        /// CommitEdit:                     Commit the edit
        /// </summary>
        //  
        //*********************************************************     
		void CommitEdit();
        //*********************************************************     
        //
        /// <summary>
        /// Refresh:                        Refresh the list from the rowset
        /// </summary>
        //  
        //*********************************************************     
		void Refresh();
        //*********************************************************     
        //
        /// <summary>
        /// AssociateRowset:                Associate a rowset
        /// </summary>
        /// <param name="rowset">           Rowset to associate</param>
        //  
        //*********************************************************     
		void AssociateRowset(Rowset rowset);
        //*********************************************************     
        //
        /// <summary>
        /// OnLineSelectedInList:         Triggers the event
        ///                               LineSelectedInList
        /// </summary>
        /// <param name="e">              Event Arguments</param>
        //  
        //*********************************************************     
        void OnLineSelectedInList(LineSelectedInListEventArgs e);
    }
}
