//*********************************************************
//
// DataGridComboBoxColumn:	    Implement a choice list editor for the Datagrid
//
// Author:						JFO
//
// (c) 2002 by Consyst SQL Inc.
//
//*********************************************************
using System;
using System.Data;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;

namespace RepPP.Toolkit.Window {

    //*********************************************************
    //
    /// <summary>
    /// Combo box implementation for the datagrid
    /// </summary>
    //
    //*********************************************************
    public class DataGridComboBoxColumn : IDataGridColumn {

        private class SCComboBox : ComboBox {
            //*********************************************************
            //
            // Combo box super-classing to intercept the OnSelectionChangeCommited
            //
            //*********************************************************
            public  bool                        m_bStartEditing;    // true if edition is started
            public  DataGridComboBoxColumn      m_father;           // Column editor owning the combo box

            //*********************************************************
            //
            // OnSelectionChangeCommited:   Called when a new selection is done.
            //                              Intercepted to indicate to the grid when column is edited
            //
            //*********************************************************
            protected override void OnSelectionChangeCommitted(EventArgs e) {
                GridDataTable               dtbl;
                CurrencyManager             bnd;

                if (!m_bStartEditing) {
                    m_bStartEditing = true;
                    if (m_father != null) {
                        m_father.m_gridColStyle.ColumnStartedEditing(this);
                    }
                }
                base.OnSelectionChangeCommitted(e);
                dtbl = m_father.m_gridDataTable;
                bnd = (CurrencyManager)dtbl.AttachedDataGrid.BindingContext[dtbl];
                if (bnd != null) {
                    m_father.Commit(bnd, bnd.Position);
                }
                dtbl.ClearRowError();
            }

            //*********************************************************
            //
            // IsInputKey:          Determine if the key is processed by the control
            //                      Intercepted to accept Alt-DownArrow for poping up selection.
            // In:
            //  keyData:            Key data
            // Out:
            //  true:               Input key
            //  false:              Process by the message loop
            //
            //*********************************************************
            protected override bool IsInputKey(Keys keyData) {
                bool    bRetVal;
                
                switch(keyData) {
                case Keys.Down | Keys.Alt:
                case Keys.Up | Keys.Alt:
                    bRetVal = true;
                    break;
                default:
                    bRetVal = base.IsInputKey(keyData);
                    break;
                }
                return(bRetVal);
            }

            //*********************************************************
            //
            // ProcessKeyMessage:   Process a key message. 
            //                      Override to correct what seems to be a bug in .NET
            //                      When the control is selected via m_combo.Select() in Edit() and
            //                      the edit is called because of a TAB key, the KEYUP message cause
            //                      the control to be unselected.
            // In:
            //  msg:				Message
            // Out:
            //  true:               Message processed by the control
            //  false:              Not processed by the control
            //
            //*********************************************************
            protected override bool ProcessKeyMessage(ref Message m) {
                bool    bRetVal  = false;
                bool    bCallBase = true;

                switch(m.Msg) {
                case 256:   // WM_KEYDOWN
                case 257:   // WM_KEYUP
                    if (m.Msg == 257 && (int)m.WParam == 9) {
                        // When the control is selected via m_combo.Select() in Edit() and
                        // the edit is called because of a TAB key, the KEYUP message cause
                        // the control to be unselected.
                        bRetVal   = true;
                        bCallBase = false;
                    } else {
                        switch((Keys)(int)m.WParam) {
                        case Keys.Down:
                        case Keys.Up:
                            if (DroppedDown) {
                                bCallBase = false;
                            }
                            break;
                        default:
                            break;
                        }
                    }
                    break;
                default:        
                    break;
                }
                if (bCallBase) {
                    bRetVal = base.ProcessKeyMessage(ref m);
                }
                return(bRetVal);
            }
        } // SCComboBox

        private DataGridColumn                  m_gridColStyle; // Grid column style
        private GridDataTable                   m_gridDataTable;// Grid data table
        private SCComboBox                      m_combo;        // Combo box control
        private RepPP.FieldDef                  m_fldDef;          // Attached field
        private bool                            m_bActive;      // true if combo has been activated (must be commited)
        private string                          m_strCurCode;   // Current code displayed
        private string                          m_strCurDesc;   // Current description displayed
        private bool                            m_bListDesc;    // Use the description in the list if true
        private bool                            m_bShowDesc;    // Show description in the paint if true

        //*********************************************************
        //
        /// <summary>
        /// Datagrid combo box control ctor
        /// </summary>
        /// <param name="fldDef">              Attached field</param>
        /// <param name="gridDataTable">    Attached grid data table</param>
        /// <param name="gridColStyle">     Datagrid column style</param>
        //
        //*********************************************************
        public DataGridComboBoxColumn(RepPP.FieldDef fldDef, GridDataTable gridDataTable, DataGridColumn gridColStyle) {
            m_gridDataTable         = gridDataTable;
            m_gridColStyle          = gridColStyle;
            m_bActive               = false;
            m_fldDef                = fldDef;
            m_strCurCode            = "";
            m_strCurDesc            = "";
            m_bListDesc             = true;
            m_bShowDesc             = true;
            m_combo                 = new SCComboBox();
            m_combo.m_father        = this;
            m_combo.m_bStartEditing = true;
            m_combo.DropDownStyle   = System.Windows.Forms.ComboBoxStyle.DropDownList;
            m_combo.Location        = new System.Drawing.Point(100, 100);
            m_combo.Name            = "comboBox1";
            m_combo.Size            = new System.Drawing.Size(168, 21);
            m_combo.TabIndex        = 1;
            m_combo.TabStop         = true;
            m_combo.Visible         = false;
        }

        //*********************************************************
        //
        /// <summary>
        /// Dispose the control
        /// </summary>
        //
        //*********************************************************
        public void Dispose() {
            m_combo.Dispose();
            m_combo = null;
        }

        //*********************************************************
        //
        /// <summary>
        /// Fills the control with the list of values
        /// </summary>
        //
        //*********************************************************
        private void FillCtlv() {
            RepPP.ChoiceList    choiceList;
            string              strChoiceValue;
            string              strChoiceCode;
            string              strChoiceDesc;
            string              strFldValue;
            int                 iChoiceIndex;
            int                 iChoiceCount;
            int                 iSelectedIndex = -1;
            
            m_combo.m_bStartEditing = true;
            m_combo.Items.Clear();
            choiceList  = m_fldDef.ChoiceList;
            strFldValue = m_fldDef.Value.ToString();
            if (choiceList != null) {
                iChoiceCount = choiceList.Count;
                for (iChoiceIndex = 0; iChoiceIndex < iChoiceCount; iChoiceIndex++) {
                    strChoiceValue  = choiceList.GetValue(iChoiceIndex);
                    strChoiceCode   = choiceList.GetCode(iChoiceIndex);
                    strChoiceDesc   = choiceList.GetDesc(iChoiceIndex);
                    if (strChoiceDesc.Length == 0) {
                        strChoiceDesc = strChoiceCode;
                    }
                    if (m_bListDesc) {
                        m_combo.Items.Add(strChoiceDesc);
                    } else {
                        m_combo.Items.Add(strChoiceCode);
                    }
                    if (strChoiceValue == strFldValue) {
                        iSelectedIndex = iChoiceIndex;
                    }
                }
            }
            if (iSelectedIndex != -1) {
                m_gridDataTable.ClearRowError();
                m_combo.SelectedIndex = iSelectedIndex;
            } else if (m_fldDef.IsNull == false) {
                m_gridDataTable.SetRowError(String.Format("The value '{0}' doesn't exist in the choice list of the field '{1}.{2}", strFldValue, m_fldDef.RowsetDef.Name, m_fldDef.Name));
            }
            m_combo.m_bStartEditing = false;
        }

        //*********************************************************
        //
        /// <summary>
        /// Gets if the combobox list show description or code
        /// </summary>
        /// <returns>
        /// true to display the choice description in list.
        /// false to display the code in list
        /// </returns>
        //
        //*********************************************************
        public bool GetListDescription() {
            return(m_bListDesc);
        }

        //*********************************************************
        //
        /// <summary>
        /// Determine if the combobox list show description or code
        /// </summary>
        /// <param name="value">    true to display the choice description in list.
        ///                         false to display the code in list</param>
        //
        //*********************************************************
        public void SetListDescription(bool value) {
            m_bListDesc = value;
        }

        //*********************************************************
        //
        /// <summary>
        /// Returns if the choice description is shown in the grid
        /// </summary>
        /// <returns>
        /// true to display the choice description in list.
        /// false to display the code in list
        /// </returns>
        //
        //*********************************************************
        public bool GetShowDescription() {
            return(m_bShowDesc);
        }

        //*********************************************************
        //
        /// <summary>
        /// Determine if the choice description is shown in the grid
        /// </summary>
        //
        //*********************************************************
        public void SetShowDescription(bool value) {
            if (value != m_bShowDesc) {
                m_bShowDesc = value;
                m_gridDataTable.Refresh();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Gets the description maximum size
        /// </summary>
        /// <param name="gr">       Graphic handle</param>
        /// <param name="font">     Font</param>
        /// <param name="fldDef">   Field definition</param>
        /// <returns>
        /// Maximum description size
        /// </returns>
        //
        //*********************************************************
        public static int GetDescMaxSizeI(Graphics gr, Font font, RepPP.FieldDef fldDef) {
            int                 iRetVal = 0;
            int                 iSize;
            RepPP.ChoiceList    choiceList;
            int                 iChoiceIndex;
            int                 iChoiceCount;
            string              strChoiceDesc;
            
            choiceList  = fldDef.ChoiceList;
            if (choiceList != null) {
                iChoiceCount = choiceList.Count;
                for (iChoiceIndex = 0; iChoiceIndex < iChoiceCount; iChoiceIndex++) {
                    strChoiceDesc   = choiceList.GetDesc(iChoiceIndex);
                    if (strChoiceDesc.Length == 0) {
                        strChoiceDesc = choiceList.GetCode(iChoiceIndex);
                    }
                    iSize = (int)gr.MeasureString(strChoiceDesc, font).Width;
                    if (iSize > iRetVal) {
                        iRetVal = iSize;
                    }
                }
            }
            return(iRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// Adds the control to the datagrid list
        /// </summary>
        /// <param name="dataGrid"> Datagrid in which the style is added</param>
        //
        //*********************************************************
        public virtual void SetDataGridInColumn(DataGrid dataGrid) {
            if (dataGrid.Controls.GetChildIndex(m_combo, false) == -1) {
                dataGrid.Controls.Add(m_combo);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Removes the control to the datagrid list
        /// </summary>
        /// <param name="dataGrid"> Datagrid in which the style is removed</param>
        //
        //*********************************************************
        public virtual void UnsetDataGridInColumn(DataGrid dataGrid) {
            if (dataGrid.Controls.GetChildIndex(m_combo, false) != -1) {
                dataGrid.Controls.Remove(m_combo);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Start the column edition
        /// </summary>
        /// <param name="dataSource">       Current data source</param>
        /// <param name="iRowNum">          Row number</param>
        /// <param name="bounds">           Position of the cell</param>
        /// <param name="bReadOnly">        true for read only column</param>
        /// <param name="strInstantText">   ???</param>
        /// <param name="bCellIsVisible">   true if cell is visible, false if not.</param>
        //
        //*********************************************************
        public virtual void Edit(CurrencyManager    dataSource, 
								 int                iRowNum, 
								 Rectangle          bounds,
								 bool               bReadOnly, 
								 string             strInstantText, 
								 bool               bCellIsVisible) {
			DataTableRowset.Trace(DataTableRowset.TraceTypeE.TT_ENTER, DataTableRowset.TraceCatE.TC_COMBOEDIT, m_fldDef.RowsetDef, null, "SCDataGridComboBox::Edit");
            if (!bReadOnly && m_fldDef.IsSelectable) {
                if (!m_bActive) {
                    FillCtlv();
                    m_bActive = true;
                }
				if (!m_combo.Visible) {
					m_combo.Bounds  = bounds;
					m_combo.Show();
                    m_combo.Select();
				}
            }
            DataTableRowset.Trace(DataTableRowset.TraceTypeE.TT_EXIT, DataTableRowset.TraceCatE.TC_COMBOEDIT, m_fldDef.RowsetDef, null, "SCDataGridComboBox::Edit");
        }

        //*********************************************************
        //
        /// <summary>
        /// Abort the edit
        /// </summary>
        /// <param name="iRowNum">  Row number</param>
        //
        //*********************************************************
        public virtual void Abort(int iRowNum) {
            DataTableRowset.Trace(DataTableRowset.TraceTypeE.TT_ENTER, DataTableRowset.TraceCatE.TC_COMBOEDIT, m_fldDef.RowsetDef, null, "SCDataGridComboBox::Abort");
            if (m_bActive) {
                if (m_combo.Visible) {
                    m_gridDataTable.AttachedDataGrid.Select();
                    m_combo.Visible = false;
                }
                m_bActive = false;
            }
            DataTableRowset.Trace(DataTableRowset.TraceTypeE.TT_EXIT, DataTableRowset.TraceCatE.TC_COMBOEDIT, m_fldDef.RowsetDef, null, "SCDataGridComboBox::Abort");
        }

        //*********************************************************
        //
        /// <summary>
        /// Commit the edit
        /// </summary>
        /// <param name="dataSource">   Current data source</param>
        /// <param name="iRowNum">      Row number</param>
        /// <returns>
        /// true
        /// </returns>
        //
        //*********************************************************
        public virtual bool Commit(CurrencyManager dataSource, int iRowNum) {
            bool                bRetVal;
            int                 iSelectedIndex;
            string              strValue = "";
            bool                bValueChanged = false;
            BindingManagerBase  bnd;

            DataTableRowset.Trace(DataTableRowset.TraceTypeE.TT_ENTER, DataTableRowset.TraceCatE.TC_COMBOEDIT, m_fldDef.RowsetDef, null, "SCDataGridComboBox::Commit");
            if (m_bActive) {
                iSelectedIndex = m_combo.SelectedIndex;
                if (iSelectedIndex == -1) {
                    if (!m_fldDef.IsNull) {
                        bValueChanged = true;
                    }
                } else {
                    strValue = m_fldDef.ChoiceList.GetCode(iSelectedIndex);
                    if (strValue != m_fldDef.ExtValue) {
                        bValueChanged = true;
                    }
                }

                //
                //  Bug from Microsoft???
                //  Sometimes, the current position of the source is different from iRowNum
                //  Cause a problem... so just skip it...
                //
                if (bValueChanged) {
                    bnd = m_gridDataTable.AttachedDataGrid.BindingContext[m_gridDataTable];
                    if (bnd == null || bnd.Position == iRowNum) {
                        m_gridColStyle.SetColumnValueAtRow(dataSource, iRowNum, strValue);
                    } else {
                        DataTableRowset.Trace(DataTableRowset.TraceTypeE.TT_NONE, DataTableRowset.TraceCatE.TC_COMBOEDIT, m_fldDef.RowsetDef, null, "SCDataGridComboBox::Commit - Skip COMMIT!");
                    }
                }
                m_gridDataTable.AttachedDataGrid.Select();
                m_combo.Visible = false;
                m_bActive       = false;
                bRetVal         = true;
            } else {
                bRetVal = true;
            }
            DataTableRowset.Trace(DataTableRowset.TraceTypeE.TT_EXIT, DataTableRowset.TraceCatE.TC_COMBOEDIT, m_fldDef.RowsetDef, null, "SCDataGridComboBox::Commit");
            return(bRetVal);
        }            

        //*********************************************************
        //
        /// <summary>
        /// Close the combo
        /// </summary>
        //
        //*********************************************************
        public virtual void ConcedeFocus() {
            // JFO 2002-11-04: The following line cause a strange problem.
            // Cannot select other field if the control is read-only
            // m_gridDataTable.AttachedDataGrid.Select();
            m_combo.Visible = false;
        }

        //*********************************************************
        //
        /// <summary>
        /// Gets the minimum height 
        /// </summary>
        /// <returns>
        /// Object size
        /// </returns>
        //
        //*********************************************************
        public virtual int GetMinimumHeight() {
            return(m_combo.Height);
        }                                                                                                      

        //*********************************************************
        //
        /// <summary>
        /// Gets the preferred height
        /// </summary>
        /// <param name="g">        Graphic object</param>
        /// <param name="value">    ???</param>
        /// <returns>
        /// Object height
        /// </returns>
        //
        //*********************************************************
        public virtual int GetPreferredHeight(Graphics g,
                                              object   value) {

            return(m_combo.Height);
        }

        //*********************************************************
        //
        /// <summary>
        /// Gets the preferred size of the combo
        /// </summary>
        /// <param name="g">        Graphic object</param>
        /// <param name="value">    ???</param>
        /// <returns>
        /// Object size
        /// </returns>
        //  
        //*********************************************************
        public virtual Size GetPreferredSize(Graphics g,
                                             object   value) {
            Size    size;
            int     iWidth;

            iWidth = GetDescMaxSizeI(g, m_combo.Font, m_fldDef);
            size = new Size(iWidth, 0);
            return(size);
        }                                                                                                      

        //*********************************************************
        //
        /// <summary>
        /// Paint the value of the cell
        /// </summary>
        /// <param name="gr">           Graphic object</param>
        /// <param name="bounds">       Bound where to draw the value</param>
        /// <param name="source">       Source</param>
        /// <param name="iRowNum">      Row num</param>
        /// <param name="backBrush">    Background brush</param>
        /// <param name="foreBrush">    Foreground brush</param>
        /// <param name="bAlignToRight">Right alignment</param>
        //
        //*********************************************************
        public virtual bool Paint(Graphics          gr,
                                  Rectangle         bounds,
                                  CurrencyManager   source,
                                  int               iRowNum,
                                  Brush             backBrush,
                                  Brush             foreBrush,
                                  bool              bAlignToRight) {
            RepPP.ChoiceList  choiceList;
            DataGrid          dataGrid;
            string            str;
            int               iIndex;

            dataGrid = m_gridColStyle.DataGridTableStyle.DataGrid;                
            if (backBrush == null) {
                backBrush = new System.Drawing.SolidBrush(dataGrid.BackColor);
            }
            str = m_gridColStyle.GetColumnValueAtRow(source, iRowNum).ToString();
            if (str != m_strCurCode) {
                m_strCurCode = str;
                m_strCurDesc = "";
                if (m_bShowDesc) {
                    choiceList   = m_fldDef.ChoiceList;
                    if (choiceList != null) {
                        iIndex = choiceList.FindCode(str);
                        if (iIndex >= 0) {
                            m_strCurDesc = choiceList.GetDesc(iIndex);
                        }
                    }
                    if (m_strCurDesc.Length == 0) {
                        m_strCurDesc = m_strCurCode;
                    }
                } else {
                    m_strCurDesc = m_strCurCode;
                }
            }
            if (foreBrush == null) {
                using (foreBrush = new System.Drawing.SolidBrush(dataGrid.ForeColor)) {
                    gr.FillRectangle(backBrush, bounds);
                }
                foreBrush = null;
            } else {
                gr.FillRectangle(backBrush, bounds);
            }            
            TextRenderer.DrawText(gr, m_strCurDesc, dataGrid.Font, new Point(bounds.X, bounds.Y), dataGrid.ForeColor);
            return(true);
        }

        //*********************************************************
        //
        /// <summary>
        /// Paint the value of the cell
        /// </summary>
        /// <param name="gr">           Graphic object</param>
        /// <param name="bounds">       Bound where to draw the value</param>
        /// <param name="source">       Source</param>
        /// <param name="iRowNum">      Row num</param>
        /// <param name="bAlignToRight">Right alignment</param>
        //
        //*********************************************************
        public virtual bool Paint(Graphics         gr,
                                  Rectangle        bounds,
                                  CurrencyManager  source,
                                  int              iRowNum,
                                  bool             bAlignToRight) {
            Paint(gr, bounds, source, iRowNum, null, null, bAlignToRight);
            return(true);
        }

        //*********************************************************
        //
        /// <summary>
        /// Paint the value of the cell
        /// </summary>
        /// <param name="gr">           Graphic object</param>
        /// <param name="bounds">       Bound where to draw the value</param>
        /// <param name="source">       Source</param>
        /// <param name="iRowNum">      Row num</param>
        //
        //*********************************************************
        public virtual bool Paint(Graphics          gr,
                                  Rectangle         bounds,
                                  CurrencyManager   source,
                                  int               iRowNum) {
            Paint(gr, bounds, source, iRowNum, null, null, false);
            return(true);
        }

    } // DataGridComboBoxColumn
} // Namespace
