using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using RepPP;
using RepPP.Toolkit;
using System.Xml;

namespace RepPP.Toolkit.Window {
    //*********************************************************
    //
	/// <summary>
	/// DataGridViewRowset:  Defines a DataGridView which is attached to a rowset
	/// </summary>
	//
    //*********************************************************
    public class DataGridViewRowset : DataGridViewRowsetBase, RowsetTreeDefNodeControlUtil.IRowsetTreeDefNodeControl, ILineListHandler {
        /// <summary>Inner datasource of the grid</summary>
        private DataSourceRowset                    m_dataSourceRowset;
        /// <summary>Event argument when the row change</summary>
        private LineSelectedInListEventArgs         m_evtArgLastRowChange = null;
        /// <summary>Field selector</summary>
        private FieldSelector                       m_fldSelector;
        /// <summary> Called when a line is selected in the LineListHandler</summary>
        public event LineSelectedInListEventHandler LineSelectedInList;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                       Build the class from a group. The source will be empty
        /// </summary>
        //
        //********************************************************************************
        public DataGridViewRowset() : base() {
            m_fldSelector       = new FieldSelector(this);
            m_dataSourceRowset  = new DataSourceRowset(this, false);
            base.DataSource     = m_dataSourceRowset;
        }

        //********************************************************************************
        //
        /// <summary>
        /// Dispose:                    Releases the resources
        /// </summary>
        //
        //********************************************************************************
        protected override void Dispose(bool disposing)
        {
            if (disposing){
                if (m_dataSourceRowset != null) {
                    m_dataSourceRowset.Dispose();
                    m_dataSourceRowset = null;
                }
                if (m_fldSelector != null) {
                    m_fldSelector.Dispose();
                    m_fldSelector = null;
                }
            }
            base.Dispose(disposing);
        }
        //********************************************************************************
        //
        /// <summary>
        /// DataSourceRowset:           Returns the data source attached to a rowset
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
		[Description("Data Source attached to a rowset")]
		[DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        protected virtual DataSourceRowset DataSourceRowset {
            get {
                return(m_dataSourceRowset);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DataSource:                     Ignore the setting of the datasource
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
		[DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        public new object DataSource {
            get {
                return(base.DataSource);
            }
            set {
                base.DataSource = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// WhereClause:            WHERE clause to use to autofill the datasource
        /// </summary>
        //
        //********************************************************************************
        public override Boolean AutoFill {
            get {
                return(base.AutoFill);
            }
            set {
                base.AutoFill = value;
                if (m_dataSourceRowset != null) {
                    m_dataSourceRowset.AutoFill = value;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// WhereClause:            WHERE clause to use to autofill the datasource
        /// </summary>
        //
        //********************************************************************************
        public override string WhereClause {
            get {
                return(base.WhereClause);
            }
            set {
                base.WhereClause = value;
                if (m_dataSourceRowset != null) {
                    m_dataSourceRowset.WhereClause = value;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnDataSourceChanged:                     Ignore the setting of the datasource
        /// </summary>
        //
        //********************************************************************************
        protected override void OnDataSourceChanged(EventArgs e) {
            if (base.DataSource != null && base.DataSource != m_dataSourceRowset) {
                base.DataSource = m_dataSourceRowset;
                throw new InvalidOperationException("We can't set a DataSource in context of DataGridViewRowset");
            }
            base.OnDataSourceChanged(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        RepPP.Application RowsetTreeDefNodeControlUtil.IRowsetTreeDefNodeControl.Application {
            get {
                return(DataSourceRowset.Application);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// RowsetTreeDefNodeFullName:  Name of the node of the rowset tree definition attached to the source
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Editor("RepPP.Toolkit.Designer.RowsetTreeDefNodeControlDesigner+NodeEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Name of the node of the rowset tree definition attached to the source")]
        public RowsetTreeDefNodeFullName RowsetTreeDefNodeFullName {
            get {
                return(DataSourceRowset.RowsetTreeDefNodeFullName);
            }
            set {
                DataSourceRowset.RowsetTreeDefNodeFullName = value;
                if (DesignMode) {
                    base.DataSource = null;
                    base.DataSource = m_dataSourceRowset;
                }
                SetColumnsInfo();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        void RowsetTreeDefNodeControlUtil.IRowsetTreeDefNodeControl.TriggerComponentChange() {
            DataSourceRowset.TriggerComponentChange();
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReInit:        Called when Rep++ is reinitialized
		/// </summary>
		//
        //********************************************************************************
        void ControlUtil.IRepControl.RepPPReInit() {
            DataSourceRowset.RepPPReInit();
            SetColumnsInfo();
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReset:         Called when Rep++ is resetted
		/// </summary>
		//
        //********************************************************************************
        void ControlUtil.IRepControl.RepPPReset() {
            DataSourceRowset.RepPPReset();
        }

        //********************************************************************************
        //
        /// <summary>
        /// RowsetDef:              Attached group
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
		[Description("RowsetDef which will be represented by this Data Source")]
		[DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        public virtual RowsetDef RowsetDef {
            get {
                return(DataSourceRowset.RowsetDef);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Rowset:      Attached rowset
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
		[Description("Rowset represented by this Data Source")]
        [DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        public override Rowset Rowset {
            get {
                return(DataSourceRowset.Rowset);
            }
            set {
                DataSourceRowset.Rowset = value;
                base.DataSource         = DataSourceRowset;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ShowDescription:        Show description of choice if true, code if false
        /// </summary>
        //
        //********************************************************************************
		[Browsable(true)]
		[Category("Behavior")]
		[Description("How to display a choice value")]
        public bool ShowDescription {
            get {
                return(DataSourceRowset.ShowDescription == DataSourceRowset.ShowDescriptionE.Always);
            }
            set {
                DataSourceRowset.ShowDescription = (value) ? DataSourceRowset.ShowDescriptionE.Always : DataSourceRowset.ShowDescriptionE.Never;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetColumnsInfo:                 Set the information of columns attached to a field
        /// </summary>
        //
        //********************************************************************************
        protected virtual void SetColumnsInfo() {
            RepPP.RowsetDef                 rowsetDef;
            RepPP.FieldDef                  fldDef;
            string                          strPropName;
            bool                            bShowDescription;
            
            bShowDescription = ShowDescription;
            rowsetDef        = RowsetDef;
            if (rowsetDef != null) {
                foreach (DataGridViewColumn col in Columns) {
                    strPropName = col.DataPropertyName;
                    fldDef = (strPropName == null) ? null : rowsetDef.FieldDefs[col.DataPropertyName];
                    if (fldDef != null) {
                        SetColumnInfo(col, fldDef, bShowDescription);
                    }   
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnCreateControl:        Called when the control is created
        /// </summary>
        //
        //********************************************************************************
        protected override void  OnCreateControl() {
 	        base.OnCreateControl();
 	        SetColumnsInfo();
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetRowsetLineFromRowIndex:       Finds the index of a line in the rowset
        ///                                 using the index of a row in the grid
        /// </summary>
        /// <param name="iRowIndex">        the index of the row</param>
        /// <returns>
        /// Index of the line or -1 if not found.
        /// </returns>
        //
        //********************************************************************************
        protected override int GetRowsetLineFromRowIndex(int iRowIndex){
            Rowset              rsDataGrid;
            DataSourceRowset    dataSourceRowset;
            DataRowRowset       rowsetRowset;
            int                 iRetVal             = -1;

            rsDataGrid = InnerRowset;
            if (rsDataGrid != null) {
                dataSourceRowset = DataSourceRowset;
                if (iRowIndex >= 0 && iRowIndex < dataSourceRowset.Count) {
                    rowsetRowset = (dataSourceRowset == null) ? null : dataSourceRowset[iRowIndex] as DataRowRowset;
                    if (rsDataGrid != null && rowsetRowset != null) {
                        iRetVal = rsDataGrid.LineFromBookmark(rowsetRowset.Bookmark);
                    }
                }
            }
            return(iRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetRowState:                    Enables the ReadOnly state of the PK cells in the 
        ///                                 current Row if it represents a new line
        /// </summary>
        /// <param name="iLineIndex">       the index of the line in the Rowset</param>
        /// <param name="iRowIndex">        the index of the row</param>
        //
        //********************************************************************************
        private void SetRowState(int iLineIndex, int iRowIndex){
            RepPP.Rowset     rsDataGrid;
            DataSourceRowset dataSourceRowset;
            int              iColumnIndex;

            rsDataGrid = InnerRowset;
            if (rsDataGrid != null) {
                if (rsDataGrid.ThisLineState(iLineIndex) == LineState.sdLineNew){
                    dataSourceRowset = DataSourceRowset;
                    foreach (FieldDef fldDef in dataSourceRowset.FieldDefs) {
                        if (fldDef.IsPrimaryKey) {
                            iColumnIndex = FindColumnIndex(fldDef);
                            if (iColumnIndex != -1) {
                                Rows[iRowIndex].Cells[iColumnIndex].ReadOnly = false;                        
                            }
                        }
                    }
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnRowValidating:                Called when validation of a row occurs
        /// </summary>
        /// <param name="e">                Event parameter</param>
        //
        //********************************************************************************
        protected override void OnRowValidating(DataGridViewCellCancelEventArgs e) {
            RepPP.Rowset rsGrid;
            DataGridViewRow row;
            String strError;
            String strFldError;
            int iLinePos;
            int iColIndex;

            rsGrid = InnerRowset;
            base.OnRowValidating(e);
            if (!e.Cancel && !Rows[e.RowIndex].IsNewRow && rsGrid != null && rsGrid.LineCount != 0) {
                iLinePos = GetRowsetLineFromRowIndex(e.RowIndex);
                if (iLinePos != -1 && iLinePos < rsGrid.LineCount) {
                    if (rsGrid.ThisLineState(iLinePos) != LineState.sdLineDeleted) {
                        rsGrid.RowsetDef.SaveActiveRowset();
                        rsGrid.SaveLinePos();
                        try {
                            rsGrid.Select(false, false);
                            rsGrid.SelectLine(iLinePos, true, false);
                            row = Rows[e.RowIndex];
                            strError = String.Empty;
                            ClearError(row);
                            foreach (Field fld in rsGrid.Fields) {
                                if (fld.FieldDef.Validate(out strFldError) != 0) {
                                    iColIndex = FindColumnIndex(fld.FieldDef);
                                    if (iColIndex != -1) {
                                        row.Cells[iColIndex].ErrorText = strFldError;
                                    }
                                    if (strError != String.Empty) {
                                        strError += "\r\n";
                                    }
                                    strError += fld.FieldDef.Prompt + ": " + strFldError;
                                    e.Cancel = true;
                                }
                            }
                            if (e.Cancel) {
                                row.ErrorText = strError;
                            } else if (rsGrid.ValidActiveLine(true) != 0) {
                                e.Cancel = true;
                            }
                        } finally {
                            rsGrid.RestoreLinePos(true);
                            rsGrid.RowsetDef.RestoreActiveRowset(true);
                        }
                    }
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnCellEnter:                        Called before a cell got focus
        /// </summary>
        /// <param name="e">                    Event Argument</param>
        //
        //********************************************************************************
        protected override void OnCellEnter(DataGridViewCellEventArgs e) {
            string          strPropName;
            RepPP.RowsetDef rowsetDef;
            RepPP.FieldDef  fldDef;

            rowsetDef = RowsetDef;
            if (m_fldSelector != null && rowsetDef != null) {
                strPropName = Columns[e.ColumnIndex].DataPropertyName;
                fldDef      = (strPropName == null) ? null : rowsetDef.FieldDefs[strPropName];
                if (fldDef != null) {
                    m_fldSelector.Select(fldDef, true);
                }
            }
            base.OnCellEnter(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnCellLeave:                        Called before a cell loose focus
        /// </summary>
        /// <param name="e">                    Event Argument</param>
        //
        //********************************************************************************
        protected override void OnCellLeave(DataGridViewCellEventArgs e) {
            string          strPropName;
            RepPP.RowsetDef rowsetDef;
            RepPP.FieldDef  fldDef;

            rowsetDef = RowsetDef;
            if (m_fldSelector != null && rowsetDef != null) {
                strPropName = Columns[e.ColumnIndex].DataPropertyName;
                fldDef      = (strPropName == null) ? null : rowsetDef.FieldDefs[strPropName];
                if (fldDef != null) {
                    m_fldSelector.Select(fldDef, false);
                }
            }
            base.OnCellLeave(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnRowEnter:                         Called before the line selection changes
        /// </summary>
        /// <param name="e">                    Event Argument</param>
        //
        //********************************************************************************
        protected override void OnRowEnter(DataGridViewCellEventArgs e) {
            int iOldLine = -1;
            int iNewLine = -1;

            base.OnRowEnter(e);
            if (CurrentRow != null) {
                iOldLine = GetRowsetLineFromRowIndex(CurrentRow.Index);
            }
            iNewLine              = GetRowsetLineFromRowIndex(e.RowIndex);
            m_evtArgLastRowChange = new LineSelectedInListEventArgs(iOldLine, iNewLine);
            SetRowState(iNewLine, e.RowIndex);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnSelectionChanged:                 Called after the line selection changes
        /// </summary>
        /// <param name="e">                    Event Argument</param>
        //
        //********************************************************************************
        protected override void OnSelectionChanged(EventArgs e) {
            base.OnSelectionChanged(e);
            if (m_evtArgLastRowChange != null) {
                OnLineSelectedInList(m_evtArgLastRowChange);
            }
            m_evtArgLastRowChange = null;
        }

        //********************************************************************************
        //
        /// <summary>
        /// SelectLine:                    Select a row
        /// </summary>
        /// <param name="iPos">            Row position</param>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //
        //********************************************************************************
        public bool SelectLine(int iPos) {
            DataGridViewRow row;
            bool            bRetVal;
            
            if (InnerRowset == null) {
                bRetVal = false;
            } else {
                ClearSelection();
                row = Rows[iPos];
                if (row.Cells.Count > 0) {
                    CurrentCell = row.Cells[0]; 
                    row.Selected = true;
                }
                bRetVal = true;
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// InsertLine:                     Insert a row
        /// </summary>
        /// <param name="bAtEnd">           true to insert at the end of the list</param>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //
        //********************************************************************************
        public bool InsertLine(bool bAtEnd) {
            DataGridViewRow row;
            bool            bRetVal;

            if (InnerRowset == null) {
                bRetVal = false;
            } else {
                if (!AllowUserToAddRows) {
                    DataSourceRowset.AddNew();
                    row = Rows[Rows.Count - 1];
                    if (row.Cells.Count > 0) {
                        CurrentCell = row.Cells[0];
                        CurrentCell.Selected = true;
                    }
                }
                bRetVal = true;
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// DuplicateLine:                  Duplicate a row
        /// </summary>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //
        //********************************************************************************
        public bool DuplicateLine() {
            bool            bRetVal;
            DataGridViewRow row;

            if (InnerRowset == null) {
                bRetVal = false;
            } else {
                if (CurrentRow == null) {
                    bRetVal = false;
                } else {
                    DataSourceRowset.AddDuplicate();
                    row = Rows[Rows.Count - 1];
                    if (row.Cells.Count > 0) {
                        CurrentCell = row.Cells[0];
                        CurrentCell.Selected = true;
                    }
                    bRetVal             = true;
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// DeleteCurrentLine:              Delete the current row
        /// </summary>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //
        //********************************************************************************
        public bool DeleteCurrentLine() {
            bool    bRetVal;

            if (InnerRowset == null) {
                bRetVal = false;
            } else {
                if (CurrentRow != null) {
                    Rows.Remove(CurrentRow);
                    bRetVal = true;
                } else {
                    bRetVal = false;
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// CommitEdit:                     Commit the edit
        /// </summary>
        //
        //********************************************************************************
        public void CommitEdit() {
            CommitEdit(DataGridViewDataErrorContexts.Commit);
        }

        //********************************************************************************
        //
        /// <summary>
        /// Refresh:                        Refresh the content of the grid
        /// </summary>
        //
        //********************************************************************************
        void ILineListHandler.Refresh() {
            RepPP.Rowset   rsBinded;

            rsBinded = InnerRowset;
            Rowset   = null;
            Rowset   = rsBinded;
        }

        //********************************************************************************
        //
        /// <summary>
        /// AssociateRowset:         Associate a rowset with the grid
        /// </summary>
        /// <param name="rowset">            Rowset to associate</param>
        //
        //********************************************************************************
        public void AssociateRowset(Rowset rowset) {
            Rowset = rowset;
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnLineSelectedInList:           Triggers the LineSelectedInList event
        /// </summary>
        /// <param name="e">                Event Argument</param>
        //
        //********************************************************************************
        public void OnLineSelectedInList(LineSelectedInListEventArgs e) {
            if (LineSelectedInList != null) {
                LineSelectedInList(this, e);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// BeginInit:                      Begins the initialization
        /// </summary>
        /// <remarks>
        /// Used to avoid an exception in design mode 
        /// </remarks>
        //
        //********************************************************************************
        public void BeginInit() {     //ISupportInitialize
        }

        //********************************************************************************
        //
        /// <summary>
        /// EndInit:                      Ends the initialization
        /// </summary>
        /// <remarks>
        /// Used to avoid an exception in design mode 
        /// </remarks>
        //
        //********************************************************************************
        public void EndInit() {     //ISupportInitialize
        }

    } // Class DataGridViewRowset
} // Namespace
