//*********************************************************
//
// SCFldCheckBox:		Overload the microsoft .net combobox to implement 
//                      rep++ checkbox field control
//
// Author:		        Jacques Fournier
//
// (c) 2003 by Consyst SQL Inc.
//
//  Attached field type         Behavior
//  -------------------------------------------------------
//  Simple choice (2 choices)   Check box 2 states. 
//                              First choice=off, second choice=on
//  Simple choice (3 choices)   Check box 3 states
//                              First choice=off, second choice=on, third choice=Unknown
//  Multiple choices            N check boxes (N=# of choices).
//                              Each check box must have a choice index (0..N-1)
//
//*********************************************************
using System;
using System.Collections;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using RepPP;
using RepPP.Toolkit;

namespace RepPP.Toolkit.Window {
    //*********************************************************
    //
	/// <summary>
	/// FldCheckBox:        Create a checkbox around a rep++ field
	/// </summary>
	//
    //*********************************************************
	public class FldCheckBox : System.Windows.Forms.CheckBox, FldControlUtil.IFldControl {
        /// <summary>Choice index to use</summary>
        private int                                 m_iChoiceIndex;
        /// <summary>Field control utility class</summary>
        protected WndFldControlUtil                 m_fldControlUtil;
        /// <summary>True if filling the control</summary>
        private bool                                m_bFilling;
		/// <summary> Required designer variable</summary>
		private System.ComponentModel.Container     components = null;

        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //*********************************************************
		public FldCheckBox() {
			InitializeComponent();
            m_iChoiceIndex           = -1;
            m_bFilling               = false;
            m_fldControlUtil         = new WndFldControlUtil(this, new WndControlUtilHelper(), false);
		}

        //*********************************************************	    
        //
        /// <summary>
        /// Dispose:                     Dispose the editor
        /// </summary>
        /// <param name="bDisposing">   true if dispose, false if GC </param>
        //
        //*********************************************************	    
        protected override void Dispose(bool bDisposing) {
            RepPP.Application   app;

			if (bDisposing) {
                if(components != null) {
                    components.Dispose();
                }
                if (m_fldControlUtil != null) {
                    app = Application;
                    if (app != null) {
                        app.ProgInit   -= new RepPP.ProgInitEventHandler(this.OnProgInit);
                        app.ProgEnd    -= new RepPP.ProgEndEventHandler(this.OnProgEnd);
                        app.FldChange  -= new RepPP.FldChangeEventHandler(this.OnFldChange);
                    }
                    m_fldControlUtil.Dispose();
                    m_fldControlUtil = null;
                }
			}
			base.Dispose(bDisposing);
		}

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReInit:        Called when Rep++ is reinitialized in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReInit() {
            InitializeCtlv();
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReset:         Called when Rep++ is reset in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReset() {
            Text = "[" + FieldFullName.RowsetName + "." + FieldFullName.FieldName + "]";
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_fldControlUtil != null) {
                m_fldControlUtil.TriggerComponentChange();
                InitializeCtlv();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FieldFullName:              Full name of the field attached to the label
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Editor("RepPP.Toolkit.Designer.FldControlDesigner+FldControlEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Full name of the field attached to the label")]
		public FldFullName FieldFullName {
		    get {
		        return(m_fldControlUtil.FieldFullName);
		    }
		    set {
		        m_fldControlUtil.FieldFullName = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public RepPP.Application Application {
		    get {
		        return(m_fldControlUtil.Application);
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// FieldName:      Get and set the field name
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public string FieldName {
            get {
                return(FieldFullName.FieldName);
            }
            set {
                FieldFullName.FieldName = value;
                Invalidate();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// RowsetName:      Get and set the group name
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public string RowsetName {
            get {
                return(FieldFullName.RowsetName);
            }
            set {
                FieldFullName.RowsetName = value;
                Invalidate();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// FieldDef:              Get the attached field
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public FieldDef FieldDef {
		    get {
		        return(m_fldControlUtil.FieldDef);
		    }
		    set {
		        m_fldControlUtil.FieldDef = value;
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// Field:                  Rep++ Field attached to the control
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Field Field {
		    get {
		        return(m_fldControlUtil.Field);
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// ChoiceIndex:    Get and set the choice index
        /// </summary>
        //
        //*********************************************************
        public int ChoiceIndex {
            get {
                return(m_iChoiceIndex);
            }
            set {
                if (m_iChoiceIndex != value) {
                    m_iChoiceIndex = value;
                    InitializeCtlv();
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// BindVisible:            Indicates whether or not the visible property of the control
        ///                         is bound to the field property
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [Category("Behavior")]
        [Description("Indicates whether or not the Visible property of the control is bound to the field property")]
        public bool BindVisible {
            get {
                return(m_fldControlUtil.BindVisible);
            }
            set {
                m_fldControlUtil.BindVisible = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// BindEnabled:          Indicates whether or not the Enabled  property 
        ///                       of the control is bound to the field property
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [Category("Behavior")]
        [Description("Indicates whether or not the Enabled property of the control is bound to the field property")]
        public bool BindEnabled {
            get {
                return(m_fldControlUtil.BindSelectable);
            }
            set {
                m_fldControlUtil.BindSelectable = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetStatev:      Sets the control state
        /// </summary>
        //
        //*********************************************************
        private void SetStatev() {
            RepPP.FieldDef fldDef;

            fldDef = m_fldControlUtil.FieldDef;
            if (fldDef != null) {
                if (BindEnabled) {
                    Enabled = fldDef.IsSelectable;
                }
                if (BindVisible) {
                    Visible = fldDef.IsVisible;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// InitializeCtlv:     Initialize the control
        /// </summary>
        //
        //*********************************************************
        private void InitializeCtlv() {
            int             iChoiceCount;
            string          strChoice;
            RepPP.FieldDef  fldDef;

            fldDef = m_fldControlUtil.FieldDef;
            if (fldDef == null) {
                Text = "[" + FieldFullName.RowsetName + "." + FieldFullName.FieldName + "]";
            } else {
                if (!DesignMode) {
                    SetStatev();
                }
                switch(fldDef.Type) {
                case RepPP.FieldType.sdFieldSimpleAlphaChoice:
                case RepPP.FieldType.sdFieldSimpleNumChoice:
                    iChoiceCount = fldDef.ChoiceList.Count;
                    Text = fldDef.Prompt;                            
                    switch(iChoiceCount) {
                    case 2:
                        ThreeState = false;
                        break;
                    case 3:
                        ThreeState = true;
                        break;
                    default:
                        throw new System.Exception("The field " + fldDef.RowsetDef.Name + "." + fldDef.Name +
                                                    " must have 2 or 3 choices!");
                    }
                    break;
                case RepPP.FieldType.sdFieldBoolean:
                    Text       = fldDef.Prompt;                            
                    ThreeState = false;
                    break;
                case RepPP.FieldType.sdFieldMultiAlphaChoice:
                case RepPP.FieldType.sdFieldMultiNumChoice:
                    if (m_iChoiceIndex != -1) {
                        ThreeState   = false;
                        iChoiceCount = fldDef.ChoiceList.Count;
                        if (m_iChoiceIndex < 0 || m_iChoiceIndex >= iChoiceCount) {
                            throw new System.Exception("The index of the field " + fldDef.RowsetDef.Name + "." + fldDef.Name +
                                                    " is out of range!");
                        }
                        strChoice = fldDef.ChoiceList.GetDesc(m_iChoiceIndex);
                        if (strChoice == null || strChoice.Length == 0) {
                            strChoice = fldDef.ChoiceList.GetCode(m_iChoiceIndex);
                        }
                        Text = strChoice;
                    }
                    break;
                default:
                    throw new System.Exception("The type of the field " + fldDef.RowsetDef.Name + "." + fldDef.Name +
                                                " must be a choice type");
                }
                if (!DesignMode) {
                    GetFieldValuev();
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetFieldValuev:     Set the field value
        /// </summary>
        //
        //*********************************************************
        private void SetFieldValuev() {
            CheckState          eState;
            string              strValue;
            string              strChoiceValue;
            int                 iChoiceValue;
            int                 iValue;
            int                 iIndex;
            RepPP.ChoiceList    choiceList;
            RepPP.FieldDef      fldDef;

            fldDef = m_fldControlUtil.FieldDef;                        
            if (fldDef != null) {
                eState = CheckState;
                switch(fldDef.Type) {
                case RepPP.FieldType.sdFieldMultiAlphaChoice:
                    strValue        = fldDef.Value;
                    choiceList      = fldDef.Application.CreateChoiceList(RepPP.ChoiceType.sdSimpleAlphaChoice);
                    choiceList.List = strValue;
                    strChoiceValue  = fldDef.ChoiceList.GetValue(m_iChoiceIndex);
                    if (eState == CheckState.Checked) {
                        if (choiceList.FindValue(strChoiceValue) == -1) {
                            iIndex = choiceList.AddChoice("", strChoiceValue, "", -1);
                        }
                    } else {
                        iIndex = choiceList.FindValue(strChoiceValue);
                        if (iIndex != -1) {
                            choiceList.RemoveChoice(iIndex);
                        }
                    }
                    strValue = choiceList.List;
                    break;
                case RepPP.FieldType.sdFieldMultiNumChoice:
                    iValue          = fldDef.Value == string.Empty ? 0 : Int32.Parse(fldDef.Value);
                    iChoiceValue    = Int32.Parse(fldDef.ChoiceList.GetValue(m_iChoiceIndex));
                    if (eState == CheckState.Checked) {
                        iValue |= iChoiceValue;
                    } else {
                        iValue &= ~iChoiceValue;
                    }
                    strValue = iValue.ToString();
                    break;
                case RepPP.FieldType.sdFieldSimpleAlphaChoice:
                case RepPP.FieldType.sdFieldSimpleNumChoice:
                    switch(eState) {
                    case CheckState.Unchecked:
                        strValue = fldDef.ChoiceList.GetValue(0);
                        break;
                    case CheckState.Checked:
                        strValue = fldDef.ChoiceList.GetValue(1);
                        break;
                    default:
                        strValue = fldDef.ChoiceList.GetValue(2);
                        break;
                    }
                    break;
                case RepPP.FieldType.sdFieldBoolean:
                    strValue = (eState == CheckState.Checked) ? "1" : "0";
                    break;
                default:
                    throw new SystemException("Bad field type. " + fldDef.RowsetDef.Name + "." + fldDef.Name);
                }
                if (fldDef.Value != strValue) {
                    m_bFilling     = true;
                    fldDef.Value   = strValue;
                    m_bFilling     = false;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// GetFieldValuev:     Get the field value
        /// </summary>
        //
        //*********************************************************
        private void GetFieldValuev() {
            CheckState          eState;
            string              strValue;
            string              strChoiceValue;
            int                 iChoiceValue;
            int                 iValue;
            RepPP.ChoiceList    choiceList;
            RepPP.FieldDef      fldDef;

            fldDef = m_fldControlUtil.FieldDef;
            if (fldDef != null) {
                if (fldDef.IsNull) {
                    eState = CheckState.Unchecked;
                } else {
                    switch(fldDef.Type) {
                    case RepPP.FieldType.sdFieldMultiAlphaChoice:
                        strValue        = fldDef.Value;
                        choiceList      = fldDef.Application.CreateChoiceList(RepPP.ChoiceType.sdSimpleAlphaChoice);
                        choiceList.List = strValue;
                        strChoiceValue  = fldDef.ChoiceList.GetValue(m_iChoiceIndex);
                        eState          = (choiceList.FindValue(strChoiceValue) == -1) ? CheckState.Unchecked : CheckState.Checked;
                        break;
                    case RepPP.FieldType.sdFieldMultiNumChoice:
                        iValue          = fldDef.Value == string.Empty ? 0 : Int32.Parse(fldDef.Value);
                        iChoiceValue    = Int32.Parse(fldDef.ChoiceList.GetValue(m_iChoiceIndex));
                        eState          = ((iValue & iChoiceValue) == 0) ? CheckState.Unchecked : CheckState.Checked;
                        break;
                    case RepPP.FieldType.sdFieldSimpleAlphaChoice:
                    case RepPP.FieldType.sdFieldSimpleNumChoice:
                        switch(fldDef.ChoiceList.FindValue(fldDef.Value)) {
                        case 0:
                            eState = CheckState.Unchecked;
                            break;
                        case 1:
                            eState = CheckState.Checked;
                            break;
                        default:
                            eState = CheckState.Indeterminate;
                            break;
                        }
                        break;
                    case RepPP.FieldType.sdFieldBoolean:
                        eState = (fldDef.Value == "1") ? CheckState.Checked : CheckState.Unchecked;
                        break;
                    default:
                        throw new SystemException("Bad field type. " + fldDef.RowsetDef.Name + "." + fldDef.Name);
                    }                        
                }
                if (CheckState != eState) {
                    m_bFilling  = true;
                    CheckState  = eState;
                    m_bFilling  = false;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnProgInit:           Handle the application initialized event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnProgInit(object sender, RepPP.ApplicationEventArgs e) {
            InitializeCtlv();
        }

        //*********************************************************
        //
        /// <summary>
        /// OnProgEnd:            Handle the application end event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnProgEnd(object sender, RepPP.ApplicationEventArgs e) {
            FieldDef = null;
        }

        //*********************************************************
        //
        /// <summary>
        /// OnFldChange:          Handle the field change event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnFldChange(object sender, RepPP.FieldGenEventArgs e) {
            RepPP.FieldDef fldDef;

            fldDef = m_fldControlUtil.FieldDef;
            if (fldDef != null && fldDef == e.FieldDef && !m_bFilling) {
                if (fldDef.Type == RepPP.FieldType.sdFieldSimpleAlphaChoice ||
                    fldDef.Type == RepPP.FieldType.sdFieldSimpleNumChoice   ||
                    fldDef.Type == RepPP.FieldType.sdFieldBoolean) {
                    Text = fldDef.Prompt;
                }
                switch(e.FldEvent) {
                case RepPP.FldEvent.sdFldValueChg:
                    if (!m_bFilling) {
                        GetFieldValuev();
                    }
                    break;
                case RepPP.FldEvent.sdFldChoiceListChg:
                    InitializeCtlv();
                    break;
                case RepPP.FldEvent.sdFldStateChg:
                    SetStatev();
                    break;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnCheckedChange:      Handle the checked change event
        /// </summary>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        protected override void OnCheckStateChanged(System.EventArgs e) {
            if (FieldDef != null && !m_bFilling) {
                SetFieldValuev();
            }
            base.OnCheckStateChanged(e);
        }

        //*********************************************************
        //
        /// <summary>
        /// OnCreateControl:    Handle create control event
        /// </summary>
        //
        //*********************************************************
        protected override void OnCreateControl() {
            RepPP.Application   app;

            app = Application;
            if (app != null) {
                InitializeCtlv();
                if (!DesignMode) {
                    app.ProgInit    += new RepPP.ProgInitEventHandler(this.OnProgInit);
                    app.ProgEnd     += new RepPP.ProgEndEventHandler(this.OnProgEnd);
                    app.FldChange   += new RepPP.FldChangeEventHandler(this.OnFldChange);
                }
            }
            base.OnCreateControl();
        }
		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion
	} // class FldCheckBox
} // namesapce
