//*********************************************************
//
// FldComboBox:		    Overload the microsoft .net combobox to implement 
//                      rep++ combobox field control
//
// Author:		        Maher Najah
//
// (c) 2003 by Consyst SQL Inc.
//
//
//*********************************************************
using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.ComponentModel.Design;
using RepPP.Toolkit;
using RepPP;

namespace RepPP.Toolkit.Window {
	/// <summary>
	/// SCFldComboBox: Create a combobox around a rep++ field
	/// </summary>
	public class FldComboBox : System.Windows.Forms.ComboBox, FldControlUtil.IFldControl
	{
        /// <summary>Field control utility class</summary>
        protected WndFldControlUtil                 m_fldControlUtil;
        /// <summary>True if filling the control</summary>
        protected bool                              m_bFilling;
        /// <summary>The maximum code length in the choice list</summary>
        protected int                               m_iMaxCodeLength;
		/// <summary> Required designer variable</summary>
		private System.ComponentModel.Container     components = null;
        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //*********************************************************
		public FldComboBox() {
			InitializeComponent();
            m_bFilling               = false;
            m_iMaxCodeLength         = 0;
            m_fldControlUtil         = new WndFldControlUtil(this, new WndControlUtilHelper(), false);
		}

        //*********************************************************	    
        //
        /// <summary>
        /// Dispose:                    Dispose the editor
        /// </summary>
        /// <param name="bDisposing">   true if dispose, false if GC </param>
        //
        //*********************************************************	    
        protected override void Dispose(bool bDisposing) {
            RepPP.Application   app;

			if (bDisposing) {
                if (components != null) {
                    components.Dispose();
                }
                if (m_fldControlUtil != null) {
                    app = Application;
                    if (app != null) {
                        app.ProgInit   -= new RepPP.ProgInitEventHandler(this.OnProgInit);
                        app.ProgEnd    -= new RepPP.ProgEndEventHandler(this.OnProgEnd);
                        app.FldChange  -= new RepPP.FldChangeEventHandler(this.OnFldChange);
                    }
                    m_fldControlUtil.Dispose();
                    m_fldControlUtil = null;
                }
			}
			base.Dispose(bDisposing);
		}

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReInit:        Called when Rep++ is reinitialized in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReInit() {
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReset:         Called when Rep++ is reset in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReset() {
            Items.Clear();
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_fldControlUtil != null) {
                m_fldControlUtil.TriggerComponentChange();
                InitializeCtlv();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FieldFullName:              Full name of the field attached to the label
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Editor("RepPP.Toolkit.Designer.FldControlDesigner+FldControlEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Full name of the field attached to the label")]
		public FldFullName FieldFullName {
		    get {
		        return(m_fldControlUtil.FieldFullName);
		    }
		    set {
		        m_fldControlUtil.FieldFullName = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public RepPP.Application Application {
		    get {
		        return(m_fldControlUtil.Application);
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// FieldName:      Get and set the field name
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public string FieldName {
            get {
                return(FieldFullName.FieldName);
            }
            set {
                FieldFullName.FieldName = value;
                Invalidate();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// RowsetName:      Get and set the group name
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public string RowsetName {
            get {
                return(FieldFullName.RowsetName);
            }
            set {
                FieldFullName.RowsetName = value;
                Invalidate();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// FieldDef:              Get the attached field
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public FieldDef FieldDef {
		    get {
		        return(m_fldControlUtil.FieldDef);
		    }
		    set {
		        m_fldControlUtil.FieldDef = value;
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// Field:                  Rep++ Field attached to the control
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Field Field {
		    get {
		        return(m_fldControlUtil.Field);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// BindVisible:            Indicates whether or not the visible property of the control
        ///                         is bound to the field property
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [Category("Behavior")]
        [Description("Indicates whether or not the Visible property of the control is bound to the field property")]
        public bool BindVisible {
            get {
                return(m_fldControlUtil.BindVisible);
            }
            set {
                m_fldControlUtil.BindVisible = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// BindEnabled:          Indicates whether or not the Enabled  property 
        ///                       of the control is bound to the field property
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [Category("Behavior")]
        [Description("Indicates whether or not the Enabled property of the control is bound to the field property")]
        public bool BindEnabled {
            get {
                return(m_fldControlUtil.BindSelectable);
            }
            set {
                m_fldControlUtil.BindSelectable = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// FindItem:               Find the item in the list of items using the value
        /// </summary>
        /// <param name="strValue"> Value of the item</param>
        /// <returns>
        /// Index of the item if found
        /// -1 if not found
        /// </returns>
        //
        //*********************************************************
        private int FindItem(string strValue) {
            int         iRetVal = -1;
            int         iIndex;
            ListItem    listItem;
            
            for (iIndex = 0; iIndex < Items.Count && iRetVal == -1; iIndex++) {
                listItem = Items[iIndex] as ListItem;
                if (listItem.Value == strValue) {
                    iRetVal = iIndex;
                }
            }
            return(iRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// InitializeCtlv:     Initialize the control using the field properties
        /// </summary>
        //
        //*********************************************************
        private void InitializeCtlv() {
            FieldDef   fldDef;
            bool        bDesignMode;

            fldDef      = m_fldControlUtil.FieldDef;
            bDesignMode = this.DesignMode;
            if (bDesignMode == false && fldDef != null) {
                m_bFilling = true;
                if (BindEnabled) {
                    this.Enabled = fldDef.IsSelectable;
                }
                if (BindVisible) {
                    this.Visible = fldDef.IsVisible;
                }
                this.MaxLength = fldDef.MaxSize;
                if (fldDef.IsChoiceAvailable == true) {
                    BuildListItems(false);
                    this.SelectedIndex = FindItem(fldDef.Value);
                } else {
                    this.Text = fldDef.Value;
                }
                m_bFilling = false;
            } else if (bDesignMode == true) {
                m_bFilling  = true;
                this.Text   = m_fldControlUtil.FieldFullName.RowsetName + "." +
                              m_fldControlUtil.FieldFullName.FieldName;
                m_bFilling  = false;
            }
        }
        //********************************************************************************
        //
        /// <summary>
        /// Text:               Text of the TextBox
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        [Bindable(false)]
        public override string Text {
            get {
                return base.Text;
            }
            set {
                base.Text = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// GetMaxCodeWidth:        Get the max width of the list codes
        ///                         measured using the current font
        /// </summary>
        /// <param name="lst">      Choice list</param>
        /// <returns>
        /// Max code width
        /// </returns>
        //
        //*********************************************************
        protected virtual int GetMaxCodeWidth(ChoiceList lst) {
            int         iRetVal  = 0;
            int         iIndex   = 0;
            string      strCode;
            string      strValue;
            string      strDesc;
            int         iCodeWidth;
            Graphics    gr;

            gr = CreateGraphics();
            do {
                iIndex = lst.ExtractChoice(iIndex, out strValue, out strCode, out strDesc);
                if (iIndex != -1) {
                    iCodeWidth  = (int)gr.MeasureString(strCode, this.Font).Width;
                    if (iCodeWidth > iRetVal) {
                        iRetVal = iCodeWidth;
                        m_iMaxCodeLength = strCode.Length;
                    }
                }
            } while (iIndex != -1);
            return(iRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// ReloadItems:        Reload the combobox list of items
        /// </summary>
        //
        //*********************************************************
        public void ReloadItems() {
            FieldDef    fldDef;

            BuildListItems(true);
            fldDef = m_fldControlUtil.FieldDef;
            if (!this.DesignMode && fldDef != null && !fldDef.IsNull) {
                m_bFilling = true;
                if (fldDef.IsChoiceAvailable == true) {
                    this.SelectedIndex = FindItem(fldDef.Value);
                } else {
                    this.Text = fldDef.Value;
                }
                m_bFilling = false;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// BuildListItemsv:        Build the combobox list items
        /// </summary>
        /// <param name="bForce">   Force the building of the list items</param>
        //
        //*********************************************************
        protected virtual void BuildListItems(bool bForce) {
            int         iIndex;
            int         iMaxCodeWidth;
            int         iSpaceWidth;
            int         iCodeWidth;
            string      strCode;
            string      strDesc;
            string      strValue;
            string      strChoiceAns;
            FieldDef    fldDef;
            Graphics    gr;
            ChoiceList  lst;
            ListItem    listItem;
             
            if (bForce || this.Items.Count == 0) {
                fldDef = m_fldControlUtil.FieldDef;
                this.Items.Clear();
                if (fldDef != null && fldDef.IsChoiceAvailable == true) {
                    lst             = fldDef.ChoiceList;
                    gr              = CreateGraphics();
                    iSpaceWidth     = (int)gr.MeasureString(" ", Font).Width;
                    iMaxCodeWidth   = GetMaxCodeWidth(lst) + iSpaceWidth * 4;
                    iIndex          = 0;
                    do {
                        iIndex = lst.ExtractChoice(iIndex, out strValue, out strCode, out strDesc);
                        if (iIndex != -1) {
                            listItem = new ListItem();
                            if (strValue.Equals(String.Empty)) {
    					        listItem.Value = strCode;
                            } else {
	    				        listItem.Value = strValue;
                            }
                            if (strDesc != String.Empty) {
                                iCodeWidth   = (int)gr.MeasureString(strCode, Font).Width; 
                                strChoiceAns = strCode.PadRight(strCode.Length + (iMaxCodeWidth - iCodeWidth)/iSpaceWidth);
                                strChoiceAns = strChoiceAns + strDesc;
                            } else {
                                strChoiceAns = strCode;
                            }
                            listItem.Text = strChoiceAns;
                            this.Items.Add(listItem);
                        }
                    } while (iIndex != -1);
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetFieldValuev:     Set the field value
        /// </summary>
        //
        //*********************************************************
        protected virtual void SetFieldValuev() {
            string      strValue;
            FieldDef    fldDef;

            fldDef = m_fldControlUtil.FieldDef;            
            if (fldDef != null) {
                if (fldDef.IsChoiceAvailable && this.SelectedItem != null) {
                    strValue = (this.SelectedItem as ListItem).Value;
                } else {
                    strValue = this.Text;
                }
                if (fldDef.Value != strValue) {
                    m_bFilling      = true;
                    fldDef.Value    = strValue;
                    m_bFilling      = false;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnProgInit:           Handle the application initialized event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnProgInit(object sender, RepPP.ApplicationEventArgs e) {
            this.Items.Clear();
            InitializeCtlv();
        }

        //*********************************************************
        //
        /// <summary>
        /// OnProgEnd:            Handle the application end event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnProgEnd(object sender, RepPP.ApplicationEventArgs e) {
            FieldDef = null;
            this.Items.Clear();
        }

        //*********************************************************
        //
        /// <summary>
        /// OnFldChange:          Handle the field change event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnFldChange(object sender, RepPP.FieldGenEventArgs e) {
            FieldDef   fldDef;

            if (m_fldControlUtil != null) {
                fldDef = m_fldControlUtil.FieldDef;
                if (fldDef != null && fldDef == e.FieldDef && !m_bFilling) {
                    if (e.FldEvent == FldEvent.sdFldChoiceListChg) {
                        ReloadItems();
                    } else {
                        InitializeCtlv();
                    }
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnSelectedIndexChanged:     Raise the selected index changed event
        /// </summary>
        /// <param name="e">            Event arguments </param>
        //
        //*********************************************************
        protected override void OnSelectedIndexChanged(System.EventArgs e) {
            if (!m_bFilling) {
                SetFieldValuev();
            }
            base.OnSelectedIndexChanged(e);
        }

        //*********************************************************
        //
        /// <summary>
        /// OnTextChanged:      Raise the text changed event
        /// </summary>
        /// <param name="e">    Event arguments </param>
        //
        //*********************************************************
        protected override void OnTextChanged(System.EventArgs e) {
            if (!m_bFilling) {
                SetFieldValuev();
            }
            base.OnTextChanged(e);
        }

        //*********************************************************
        //
        /// <summary>
        /// IsInputChar:              Determines if a character is recognized by the control
        /// </summary>
        /// <param name="CharCode">   The character to test </param>
        //
        //*********************************************************
        protected override bool IsInputChar(char CharCode) {
            bool        bRetVal;
            FieldDef    fldDef;

            fldDef = m_fldControlUtil.FieldDef;
            if (CharCode >= 32 && CharCode <= 255) {
                bRetVal = this.Enabled == false && fldDef != null && fldDef.ValidateChar(CharCode);
            } else {
                bRetVal = base.IsInputChar(CharCode);
            }
            return(bRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// OnCreateControl:    Handle create control event
        /// </summary>
        //
        //*********************************************************
        protected override void OnCreateControl() {
            RepPP.Application   app;

            app = Application;
            if (app != null) {
                InitializeCtlv();
                if (!DesignMode) {
                    app.ProgInit    += new RepPP.ProgInitEventHandler(this.OnProgInit);
                    app.ProgEnd     += new RepPP.ProgEndEventHandler(this.OnProgEnd);
                    app.FldChange   += new RepPP.FldChangeEventHandler(this.OnFldChange);
                }
            }
            base.OnCreateControl();
        }

        //*********************************************************
        //
        /// <summary>
        /// OnKeyDown:          Handle Key down control event
        /// </summary>
        //
        //*********************************************************
        protected override void OnKeyDown(KeyEventArgs e) {
            if (this.DropDownStyle == ComboBoxStyle.DropDownList && e.KeyCode == Keys.Delete && !this.FieldDef.IsNull) {
                if (this.DroppedDown) {
                    this.DroppedDown = false;
                }
                this.FieldDef.Value = string.Empty;
            }
            base.OnKeyDown(e);
        }

        //*********************************************************
        /// <summary>
        /// Class defining an item of the combobox items list
        /// </summary>
        //
        //*********************************************************
        public class ListItem {
            /// <summary>Value of the item. Used to identify the item</summary>
            string m_strValue;
            /// <summary>Text of the item. Used for the display</summary>
            string m_strText;

            //*********************************************************
            /// <summary>
            /// Constructor
            /// </summary>
            /// <param name="strValue"> Value</param>
            /// <param name="strText">  Text</param>
            //
            //*********************************************************
            public ListItem(string strValue, string strText) {
                m_strText   = strText;
                m_strValue  = strValue;
            }

            //*********************************************************
            /// <summary>
            /// Constructor
            /// </summary>
            //
            //*********************************************************
            public ListItem() : this(String.Empty, String.Empty) {
            }

            //*********************************************************
            /// <summary>
            /// ToString:   Serialize the object using string format
            /// </summary>
            /// <returns>
            /// Serialization string
            /// </returns>
            //
            //*********************************************************
            public override string ToString() {
                return(m_strText);
            }

            //*********************************************************
            /// <summary>
            /// Text:       Text property
            /// </summary>
            //
            //*********************************************************
            public string Text {
                get {
                    return(m_strText);
                }
                set {
                    m_strText = value;
                }
            }

            //*********************************************************
            /// <summary>
            /// Value:       Value property
            /// </summary>
            //
            //*********************************************************
            public string Value {
                get {
                    return(m_strValue);
                }
                set {
                    m_strValue = value;
                }
            }
        } // Class ListItem

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion
	} // SCFldComboBox
} // namespace
