//*********************************************************
//
// FldDateTimePicker:		Overload the nullable DateTimePicker to implement 
//                          rep++ DateTimePicker field control
//
// Author:		            Maher Najah
//
// (c) 2003 by Consyst SQL Inc.
//
//
//*********************************************************
using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.ComponentModel.Design;
using RepPP.Toolkit;
using RepPP;

namespace RepPP.Toolkit.Window {
	/// <summary>
	/// SCFldDateTimePicker: Create a DateTimePicker around a rep++ field
	/// </summary>
	public class FldDateTimePicker : NullableDateTimePicker, FldControlUtil.IFldControl {
        /// <summary>Field control utility class</summary>
        protected WndFldControlUtil                 m_fldControlUtil;
        /// <summary>True if filling the control</summary>
        private bool                                m_bFilling;
        /// <summary>True if the control is created</summary>
        private bool                                m_bCreated;
		/// <summary> Required designer variable</summary>
		private System.ComponentModel.Container     components = null;
        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //*********************************************************
		public FldDateTimePicker() {
			InitializeComponent();
            m_bFilling               = false;
            m_bCreated               = false;
            m_fldControlUtil         = new WndFldControlUtil(this, new WndControlUtilHelper(), false);
		}

        //*********************************************************	    
        //
        /// <summary>
        /// Dispose:                    Dispose the editor
        /// </summary>
        /// <param name="bDisposing">   true if dispose, false if GC </param>
        //
        //*********************************************************	    
        protected override void Dispose(bool bDisposing) {
            RepPP.Application   app;

			if (bDisposing) {
                if(components != null) {
                    components.Dispose();
                }
                if (m_fldControlUtil != null) {
                    app = Application;
                    if (app != null) {
                        app.ProgInit   -= new RepPP.ProgInitEventHandler(this.OnProgInit);
                        app.ProgEnd    -= new RepPP.ProgEndEventHandler(this.OnProgEnd);
                        app.FldChange  -= new RepPP.FldChangeEventHandler(this.OnFldChange);
                    }
                    m_fldControlUtil.Dispose();
                    m_fldControlUtil = null;
                }
			}
			base.Dispose(bDisposing);
		}

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReInit:        Called when Rep++ is reinitialized in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReInit() {
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReset:         Called when Rep++ is reset in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReset() {
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_fldControlUtil != null) {
                m_fldControlUtil.TriggerComponentChange();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FieldFullName:              Full name of the field attached to the label
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Editor("RepPP.Toolkit.Designer.FldControlDesigner+FldControlEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Full name of the field attached to the label")]
		public FldFullName FieldFullName {
		    get {
		        return(m_fldControlUtil.FieldFullName);
		    }
		    set {
		        m_fldControlUtil.FieldFullName = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public RepPP.Application Application {
		    get {
		        return(m_fldControlUtil.Application);
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// FieldName:      Get and set the field name
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public string FieldName {
            get {
                return(FieldFullName.FieldName);
            }
            set {
                FieldFullName.FieldName = value;
                Invalidate();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// RowsetName:      Get and set the group name
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public string RowsetName {
            get {
                return(FieldFullName.RowsetName);
            }
            set {
                FieldFullName.RowsetName = value;
                Invalidate();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// FieldDef:              Get the attached field
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public FieldDef FieldDef {
		    get {
		        return(m_fldControlUtil.FieldDef);
		    }
		    set {
		        m_fldControlUtil.FieldDef = value;
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// Field:                  Rep++ Field attached to the control
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Field Field {
		    get {
		        return(m_fldControlUtil.Field);
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// IsNull:              Indicates if the control contains a null date
        /// </summary>
        //
        //*********************************************************
		public override bool IsNull {
			get {
                return(base.IsNull);
			}
			set {
                RepPP.FieldDef fldDef;

                if (IsNull != value) {
                    base.IsNull = value;
                    if (!m_bFilling) {
                        fldDef = m_fldControlUtil.FieldDef;
                        if (value == true && fldDef != null && 
                            fldDef.RowsetDef.ActiveRowset != null && 
                            fldDef.RowsetDef.ActiveRowset.LineCount > 0 ) {
                            fldDef.Value = String.Empty;
                        }
                    }
				}
			}
		}

        //********************************************************************************
        //
        /// <summary>
        /// BindVisible:            Indicates whether or not the visible property of the control
        ///                         is bound to the field property
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [Category("Behavior")]
        [Description("Indicates whether or not the Visible property of the control is bound to the field property")]
        public bool BindVisible {
            get {
                return(m_fldControlUtil.BindVisible);
            }
            set {
                m_fldControlUtil.BindVisible = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// BindEnabled:          Indicates whether or not the Enabled  property 
        ///                       of the control is bound to the field property
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [Category("Behavior")]
        [Description("Indicates whether or not the Enabled property of the control is bound to the field property")]
        public bool BindEnabled {
            get {
                return(m_fldControlUtil.BindSelectable);
            }
            set {
                m_fldControlUtil.BindSelectable = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// InitializeCtlv:     Initialize the control
        /// </summary>
        //
        //*********************************************************
        private void InitializeCtlv() {
            FieldDef   fldDef;

            fldDef = m_fldControlUtil.FieldDef;
            if (this.DesignMode == false && fldDef != null) {
                if (fldDef.Type == RepPP.FieldType.sdFieldDate || 
                    fldDef.Type == RepPP.FieldType.sdFieldDateTime ||
                    fldDef.Type == RepPP.FieldType.sdFieldTime) {
                    SetStatev();
                    m_bFilling = true;
                    if (!fldDef.IsNull) {
                        IsNull = false;
                        Value  = DateTime.Parse(fldDef.Value);
                    } else {
                        IsNull = true;
                    }
                    m_bFilling = false;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetStatev:      Sets the control state
        /// </summary>
        //
        //*********************************************************
        private void SetStatev() {
            RepPP.FieldDef fldDef;

            fldDef = m_fldControlUtil.FieldDef;
            if (fldDef != null) {
                if (BindEnabled) {
                    Enabled = fldDef.IsSelectable;
                }
                if (BindVisible) {
                    Visible = fldDef.IsVisible;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetFieldValuev:     Set the field value
        /// </summary>
        //
        //*********************************************************
        private void SetFieldValuev() {
            FieldDef   fldDef;

            fldDef = m_fldControlUtil.FieldDef;            
            if (fldDef != null && 
                (fldDef.Type == RepPP.FieldType.sdFieldDate ||
                 fldDef.Type == RepPP.FieldType.sdFieldDateTime ||
                 fldDef.Type == RepPP.FieldType.sdFieldTime)) {
                m_bFilling = true;
                if (!IsNull) {
                    if (fldDef.Type == RepPP.FieldType.sdFieldDate && fldDef.Value != Value.ToString("yyyy/MM/dd")) {
                        fldDef.Value = Value.ToString("yyyy/MM/dd");
                    } else if (fldDef.Value != Value.ToString("yyyy/MM/dd HH:mm:ss")) {
                        fldDef.Value = Value.ToString("yyyy/MM/dd HH:mm:ss");
                    }
                } else {
                    fldDef.Value = String.Empty;
                }
                m_bFilling = false;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnProgInit:           Handle the application initialized event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnProgInit(object sender, RepPP.ApplicationEventArgs e) {
            InitializeCtlv();
        }

        //*********************************************************
        //
        /// <summary>
        /// OnProgEnd:            Handle the application end event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnProgEnd(object sender, RepPP.ApplicationEventArgs e) {
            FieldDef = null;
        }

        //*********************************************************
        //
        /// <summary>
        /// OnFldChange:          Handle the field change event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnFldChange(object sender, RepPP.FieldGenEventArgs e) {
            FieldDef   fldDef;

            fldDef = (m_fldControlUtil != null) ? m_fldControlUtil.FieldDef : null;
            if (fldDef != null && fldDef == e.FieldDef && !m_bFilling) {
                switch(e.FldEvent) {
                case RepPP.FldEvent.sdFldStateChg:
                    SetStatev();
                    break;
                default:
                    InitializeCtlv();
                    break;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnValueChanged:    Handle the value changed event
        /// </summary>
        /// <param name="e">   Event arguments </param>
        //
        //*********************************************************
        protected override void OnValueChanged(System.EventArgs e) {
            if (!m_bFilling && LicenseManager.UsageMode != LicenseUsageMode.Designtime) {
                SetFieldValuev();
            }
            base.OnValueChanged(e);
        }

        //*********************************************************
        //
        /// <summary>
        /// OnCreateControl:    Handle create control event
        /// </summary>
        //
        //*********************************************************
        protected override void OnCreateControl() {
            RepPP.Application   app;

            app = Application;
            if (app != null && !m_bCreated) {
                m_bCreated = true;
                InitializeCtlv();
                if (!DesignMode) {
                    app.ProgInit    += new RepPP.ProgInitEventHandler(this.OnProgInit);
                    app.ProgEnd     += new RepPP.ProgEndEventHandler(this.OnProgEnd);
                    app.FldChange   += new RepPP.FldChangeEventHandler(this.OnFldChange);
                }
            }
            base.OnCreateControl();
        }

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion
	} // SCFldDateTimePicker
} // namespace
