using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Drawing2D;

namespace RepPP.Toolkit.Window {
    /// <summary>
    /// 
    /// </summary>
    public class FldLabelComboBox : Control, FldControlUtil.IFldControl {

        /// <summary>Width of the arrow glyph</summary>
        private const int                                   ARROW_WIDTH  = 7;
        /// <summary>Height of the arrow glyph</summary>
        private const int                                   ARROW_HEIGHT = 4;
        /// <summary>Delegate for the ItemSelected event</summary>
        public delegate void ItemSelectedHandler(object sender, EventArgs e);
        /// <summary>Occurs when an item in list has been selected in combobox</summary>
        [Description("Occurs when an item in list has been selected."), Category("Behavior")]
        public event ItemSelectedHandler                    ItemSelected;
        /// <summary>Field control utility class</summary>
        protected    WndFldControlUtil                      m_fldControlUtil;
        /// <summary>Context menu used like a combobox list</summary>
        private      ContextMenuStrip                       m_ctx;
        /// <summary>Flag indicate if the combobox is opened</summary>
        private      Boolean                                m_bCtxShown;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                   Class constructor
        /// </summary>
        //
        //********************************************************************************
        public FldLabelComboBox() {
            Cursor             = Cursors.Hand;            
            m_fldControlUtil   = new WndFldControlUtil(this, new WndControlUtilHelper(), false);
            m_bCtxShown        = false;
            m_ctx              = new ContextMenuStrip();
            m_ctx.Opened      += new EventHandler(OnComboOpened);
            m_ctx.Closed      += new ToolStripDropDownClosedEventHandler(OnComboClosed);
            m_ctx.ItemClicked += new ToolStripItemClickedEventHandler(OnComboItemClicked);
        }

        //********************************************************************************
        //
		/// <summary>
		/// Dispose:                Clean up any resources being used.
		/// </summary>
        //
        //********************************************************************************
        protected override void Dispose(bool bDisposing) {
            RepPP.Application   app;

            if (bDisposing) {
                if (m_fldControlUtil != null) {
                    app = Application;
                    if (app != null) {
                        app.FldChange -= new RepPP.FldChangeEventHandler(this.OnFldChange);
                    }
                    m_fldControlUtil.Dispose();
                    m_fldControlUtil = null;
                }
            }
            base.Dispose(bDisposing);
        }

        //********************************************************************************
        //
        /// <summary>
        /// Gets the Rep++ application from the control
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public RepPP.Application Application {
            get {
                RepPP.Application       appRetVal = null;

                if (m_fldControlUtil != null) {
                    appRetVal = m_fldControlUtil.Application;
                }
                return(appRetVal);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FieldFullName:          Full name of the field attached to the label
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Editor("RepPP.Toolkit.Designer.FldControlDesigner+FldControlEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Full name of the field attached to the label")]
		public FldFullName FieldFullName {
		    get {
		        return(m_fldControlUtil.FieldFullName);
		    }
		    set {
		        m_fldControlUtil.FieldFullName = value;
		    }
		} 

        //*********************************************************
        //
        /// <summary>
        /// FieldName:      Get and set the field name
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public string FieldName {
            get {
                return(FieldFullName.FieldName);
            }
            set {
                FieldFullName.FieldName = value;
                Invalidate();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// RowsetName:      Get and set the group name
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public string RowsetName {
            get {
                return(FieldFullName.RowsetName);
            }
            set {
                FieldFullName.RowsetName = value;
                Invalidate();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// FieldDef:              Get the attached field
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public FieldDef FieldDef {
		    get {
		        return(m_fldControlUtil.FieldDef);
		    }
		    set {
		        m_fldControlUtil.FieldDef = value;
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// Field:                  Rep++ Field attached to the control
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Field Field {
		    get {
		        return(m_fldControlUtil.Field);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Gets or sets the selected index of the combobox
        /// </summary>
        //
        //********************************************************************************
        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Description("Gets or sets the index specifying the currently selected item.")]
        public int SelectedIndex {
            get {
                RepPP.FieldDef  fldDef;
                int             iRetVal = -1;

                fldDef = m_fldControlUtil.FieldDef;
                if (fldDef != null) {
                    iRetVal = fldDef.ChoiceList.FindValue(fldDef.Value);
                }
                return(iRetVal);
            }
            set {
                RepPP.FieldDef      fldDef;
                RepPP.ChoiceList    choiceList;

                fldDef = m_fldControlUtil.FieldDef;
                if (fldDef == null) {
                    throw new System.ApplicationException("No Field bound to the control !");
                }
                choiceList = fldDef.ChoiceList;
                if (value < 0 || value > choiceList.Count) {
                    throw new ArgumentException("The index is out of range !");
                }
                fldDef.Value = choiceList.GetValue(value);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Gets or sets the cursor that is displayed when the mouse pointer is over
        ///  the control.
        /// </summary>
        //
        //********************************************************************************
        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public override Cursor Cursor {
            get {
                return(base.Cursor);
            }
            set {
                base.Cursor = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Gets the Text of Field Label combobox
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        [Bindable(false)]
        public override string Text {
            get {
                return(base.Text);
            }
            set {
                // Do nothing
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Gets the item count in combobox
        /// </summary>
        //
        //********************************************************************************
        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Description("Gets or sets the index specifying the currently selected item.")]
        public int ItemCount {
            get {
                RepPP.FieldDef  fldDef;
                int             iRetVal = -1;

                fldDef = m_fldControlUtil.FieldDef;
                if (fldDef != null) {
                    iRetVal = fldDef.ChoiceList.Count;
                }
                return(iRetVal);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Gets or sets the autosize mode for the combobox label
        /// </summary>
        //
        //********************************************************************************
        [Browsable(true)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Category("Layout")]
        [DefaultValue(false)]
        [Description("Enables automatic resizing based on font size. Note that this is only valid for label controls that do not wrap text.")]
        [EditorBrowsable(EditorBrowsableState.Always)]
        public override bool AutoSize {
            get {
                return(base.AutoSize);
            }
            set {
                base.AutoSize = value;
                Invalidate();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// BindVisible:            Indicates whether or not the visible property of the control
        ///                         is bound to the field property
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [DefaultValue(true)]
        [Category("Behavior")]
        [Description("Indicates whether or not the Visible property of the control is bound to the field property")]
        public bool BindVisible {
            get {
                return(m_fldControlUtil.BindVisible);
            }
            set {
                m_fldControlUtil.BindVisible = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// BindEnabled:          Indicates whether or not the Enabled  property 
        ///                       of the control is bound to the field property
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [DefaultValue(true)]
        [Category("Behavior")]
        [Description("Indicates whether or not the Enabled property of the control is bound to the field property")]
        public bool BindEnabled {
            get {
                return(m_fldControlUtil.BindSelectable);
            }
            set {
                m_fldControlUtil.BindSelectable = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// BuildItemList:          Build the item list of the combobox
        /// </summary>
        //
        //********************************************************************************
        private void BuildItemList() {
            RepPP.FieldDef      fldDef;
            RepPP.ChoiceList    choiceList;
            ToolStripItem[]     arrToolStripItem;
            ToolStripMenuItem   tsItem;
            String              strChoiceValue;
            String              strChoiceCode;
            String              strChoiceDesc;
            String              strFieldValue;
            int                 iChoiceCount;            
            int                 iIndex          = 0;
            int                 iPos            = 0;

            m_ctx.Items.Clear();
            fldDef = m_fldControlUtil.FieldDef;            
            if (fldDef != null) {
                choiceList   = fldDef.ChoiceList;
                iChoiceCount = choiceList.Count;
                if (iChoiceCount != 0) {
                    strFieldValue    = fldDef.Value;
                    arrToolStripItem = new ToolStripItem[iChoiceCount];
                    do {
                        iIndex = choiceList.ExtractChoice(iIndex, out strChoiceValue, out strChoiceCode, out strChoiceDesc);
                        if (iIndex != -1) {
                            tsItem      = new ToolStripMenuItem();
                            tsItem.Tag  = (strChoiceValue == String.Empty) ? strChoiceCode : strChoiceValue;
                            tsItem.Text = (strChoiceDesc == String.Empty) ? strChoiceCode : strChoiceDesc;
                            arrToolStripItem[iPos] = tsItem;
                            iPos++;
                        }
                    } while(iIndex != -1);
                    m_ctx.Items.AddRange(arrToolStripItem);            
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetFldValueText: Gets the field value text from its Choice list
        /// </summary>
        /// <returns>
        /// Field value text
        /// </returns>
        //
        //********************************************************************************
        private String GetFldValueText() {
            RepPP.FieldDef      fldDef;
            RepPP.ChoiceList    choiceLst;
            String              strChoiceDesc;
            String              strRetVal       = String.Empty;
            int                 iChoiceIndex;

            fldDef = m_fldControlUtil.FieldDef;
            if (fldDef != null) {
                choiceLst    = fldDef.ChoiceList;
                iChoiceIndex = choiceLst.FindValue(fldDef.Value);
                if (iChoiceIndex != -1) {
                    strChoiceDesc = choiceLst.GetDesc(iChoiceIndex);
                    strRetVal     = (strChoiceDesc == String.Empty) ? choiceLst.GetCode(iChoiceIndex) : strChoiceDesc;
                }
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// InitializeCtl:          Initialize the control 
        /// </summary>
        //
        //********************************************************************************
        private void InitializeCtl() {
            RepPP.FieldDef          fldDef;
            String                  strRowsetName;
            String                  strFieldName;

            fldDef = m_fldControlUtil.FieldDef;
            if (DesignMode) {
                if (fldDef != null) {
                    strRowsetName = m_fldControlUtil.FieldFullName.RowsetName;
                    strFieldName  = m_fldControlUtil.FieldFullName.FieldName;
                } else {
                    strRowsetName = "?";
                    strFieldName  = "?";
                }
                base.Text = String.Format ("[{0}.{1}]", strRowsetName, strFieldName);
            } else if (fldDef != null) {
                if (BindEnabled) {
                    this.Enabled = fldDef.IsSelectable;
                }
                if (BindVisible) {
                    this.Visible = fldDef.IsVisible;
                }
                base.Text = GetFldValueText();
                BuildItemList();
            }
            Invalidate();
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnCreateControl:        Occurs when the control is creating
        /// </summary>
        //
        //********************************************************************************
        protected override void OnCreateControl() {
            RepPP.Application app;

            app = Application;
            if (app != null) {
                if (!DesignMode) {
                    app.FldChange += new RepPP.FldChangeEventHandler(this.OnFldChange);
                }
            }
            InitializeCtl();
            base.OnCreateControl();
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetInnerRectangle:      Gets the rectangle without its border (inner area)
        /// </summary>
        /// <returns>
        /// Inner rectangle
        /// </returns>
        //
        //********************************************************************************
        private Rectangle GetInnerRectangle() {
            Rectangle rectRetVal;

            rectRetVal = new Rectangle(1, 1, Width - 2, Height - 2);
            return(rectRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// DrawHighLight:          Draw the highlight when the mouse is over the control
        /// </summary>
        /// <param name="gr">Graphics layer to draw the highlight</param>
        //
        //********************************************************************************
        private void DrawHighLight(Graphics gr) {
            Rectangle           rect;
            Pen                 pencil;
            LinearGradientBrush brush;

            rect  = GetInnerRectangle();
            brush = new LinearGradientBrush(rect, Color.White, SystemColors.GradientInactiveCaption, 90);
            using (pencil = new Pen(SystemColors.GradientInactiveCaption)) {
                gr.DrawRectangle(pencil, 0, 0, Width - 1, Height - 1);
            } 
            gr.FillRectangle(brush, rect);                
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsMouseHover:           Indicate if the mouse cursor is over the control
        /// </summary>
        /// <returns>
        /// True if cursor is over control otherwise false
        /// </returns>
        //
        //********************************************************************************
        private Boolean IsMouseHover() {
            Point       pt;
            Rectangle   rectMouse;
            Boolean     bRetVal;                         

            pt        = PointToClient(Control.MousePosition);
            rectMouse = new Rectangle(pt, new Size(1, 1));
            bRetVal   = (this.ClientRectangle.IntersectsWith(rectMouse));
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetDisplayedTextSize:   Get the size of the displayed text in combobox 
        /// </summary>
        /// <param name="gr">Graphics layer to draw the text</param>
        /// <returns>
        /// Size of the displayed text in combobox
        /// </returns>
        //
        //********************************************************************************
        private Size GetDisplayedTextSize(Graphics gr) {
            SizeF   size;
            String  strText;

            strText = String.IsNullOrEmpty(Text) ? " " : Text;
            size    = gr.MeasureString(strText, this.Font);
            return(size.ToSize());
        }

        //********************************************************************************
        //
        /// <summary>
        /// DrawDownArrowGlyph:     Draw a arrow glyph with down direction like orientation
        /// </summary>
        /// <param name="gr">Graphics object to render the glyph</param>
        /// <param name="color">Color to draw the glyph</param>
        /// <param name="iX">Left position to start drawing the glyph</param>
        /// <param name="iY">Top position to start drawing the glyph</param>
        /// <param name="iWidth">Width of the glyph</param>
        /// <param name="iHeight">Heigth of the glyph</param>
        //
        //********************************************************************************
        private void DrawDownArrowGlyph(Graphics gr, Color color, int iX, int iY, int iWidth, int iHeight) {
            Point[] pt      = new Point[3];
            Brush   brush;

            using (brush = new SolidBrush(color)) {
                pt[0].X = iX;
                pt[0].Y = iY;
                pt[1].X = iX + (iWidth / 2);
                pt[1].Y = iY + iHeight;
                pt[2].X = iX + iWidth;
                pt[2].Y = iY;         
                gr.FillPolygon(brush, pt);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnPaint:                Occurs when the control needs be painted
        /// </summary>
        /// <param name="e">Event args</param>
        //
        //********************************************************************************
        protected override void OnPaint(PaintEventArgs e) {
            Graphics        gr;
            SolidBrush      brush;
            Pen             pencil;
            Rectangle       rect;
            Size            size;
            Color           cDrawColor;
            Color           cArrowColor;
            int             iHeight;
            int             iTextArea;
            int             iImageArea;

            gr                 = e.Graphics;
            cDrawColor         = Color.Black;
            using (brush = new SolidBrush(cDrawColor)) {            
                size           = GetDisplayedTextSize(gr);
                Height         = ((int)size.Height) + 6;
                iImageArea     = ARROW_WIDTH + 4;
                iTextArea      = Width - iImageArea;
                if (AutoSize) {
                    Width = size.Width + ARROW_WIDTH + 32;
                } 
                if (!DesignMode) {
                    if (m_bCtxShown || IsMouseHover()) {
                        DrawHighLight(gr);
                    }
                    if (Focused) {
                        using (pencil = new Pen(brush)) {
                            pencil.DashStyle = System.Drawing.Drawing2D.DashStyle.Dot;
                            gr.DrawRectangle(pencil, 1, 1, Width - 3, Height - 3);                            
                        }
                    }             
                }
                iHeight = (Height - size.Height) / 2;
                rect    = new Rectangle(0, iHeight, iTextArea, size.Height);
                if (this.Enabled) {
                    cArrowColor = cDrawColor;
                    TextRenderer.DrawText(gr, Text, Font, rect, this.ForeColor, Color.Transparent, TextFormatFlags.EndEllipsis);
                } else {
                    cArrowColor = SystemColors.ControlDark;
                    ControlPaint.DrawStringDisabled(gr, Text, Font, SystemColors.Control, rect, TextFormatFlags.EndEllipsis);
                }
                DrawDownArrowGlyph(gr, cArrowColor, iTextArea, (Height - ARROW_HEIGHT) / 2, ARROW_WIDTH, ARROW_HEIGHT);              
            }            
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnLostFocus:            Occurs when the control lost its focus
        /// </summary>
        /// <param name="e">Event arguments</param>
        //
        //********************************************************************************
        protected override void OnLostFocus(EventArgs e) {
            base.OnLostFocus(e);
            this.Invalidate();
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnGotFocus:             Occurs when the control get the focus
        /// </summary>
        /// <param name="e">Event arguments</param>
        //
        //********************************************************************************
        protected override void OnGotFocus(EventArgs e) {
            base.OnGotFocus(e);
            this.Invalidate();
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnClick:                Occurs when the control is clicked
        /// </summary>
        /// <param name="e">Event arguments</param>
        //
        //********************************************************************************
        protected override void OnClick(EventArgs e) {
            if (!Focused) {
                Focus();
            }
            if (m_ctx.Items.Count > 0) {
                m_ctx.Show(this, new Point(0, Height + 2));
            }
            base.OnClick(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnMouseHover:           Occurs when the mouse is over the control
        /// </summary>
        /// <param name="e">Event arguments</param>
        //
        //********************************************************************************
        protected override void OnMouseHover(EventArgs e) {
            base.OnMouseHover(e);
            this.Invalidate();
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnMouseHover:           Occurs when the mouse leave the control
        /// </summary>
        /// <param name="e">Event arguments</param>
        //
        //********************************************************************************
        protected override void OnMouseLeave(EventArgs e) {
            base.OnMouseLeave(e);
            this.Invalidate();  
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component 
        ///                             property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_fldControlUtil != null) {
                m_fldControlUtil.TriggerComponentChange();
                InitializeCtl();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// RepPPReInit:        Called when Rep++ is reinitialized in design mode
        /// </summary>
        //
        //********************************************************************************
        public void RepPPReInit() {
            Invalidate();
        } 

        //********************************************************************************
        //
        /// <summary>
        /// RepPPReset:         Called when Rep++ is reset in design mode
        /// </summary>
        //
        //********************************************************************************
        public void RepPPReset() {
            Invalidate();
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnFldChange:          Occurs when the field value is changed
        /// </summary>
        /// <param name="sender">Sender object   </param>
        /// <param name="e">Event arguments </param>
        //
        //********************************************************************************
        private void OnFldChange(object sender, RepPP.FieldGenEventArgs e) {
            RepPP.FieldDef      fldDef;

            fldDef = m_fldControlUtil.FieldDef;
            if (fldDef != null && fldDef == e.FieldDef) {
                switch (e.FldEvent) {
                case FldEvent.sdFldChoiceListChg:
                    BuildItemList();
                    break;
                case FldEvent.sdFldValueChg:
                    base.Text = GetFldValueText();
                    break;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnComboItemClicked:     Occurs when an item has been clicked in combobox
        /// </summary>
        /// <param name="sender">Sender Object</param>
        /// <param name="e">Event arguments</param>
        //
        //********************************************************************************
        private void OnComboItemClicked(object sender, ToolStripItemClickedEventArgs e) {
            ToolStripItem       tsItem;
            
            tsItem                          = e.ClickedItem;
            Text                            = tsItem.Text;
            m_fldControlUtil.FieldDef.Value = tsItem.Tag as String;
            Invalidate();
            if (ItemSelected != null) {
                ItemSelected(this, EventArgs.Empty);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnComboClosed:          Occurs when the combobox list is closed
        /// </summary>
        /// <param name="sender">Sender Object</param>
        /// <param name="e">Event arguments</param>
        //
        //********************************************************************************
        private void OnComboClosed(object sender, ToolStripDropDownClosedEventArgs e) {
            m_bCtxShown = false;
            if (e.CloseReason == ToolStripDropDownCloseReason.Keyboard) {
                Invalidate();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnComboOpened:          Occurs when the combobox list is opened
        /// </summary>
        /// <param name="sender">Sender Object</param>
        /// <param name="e">Event arguments</param>
        //
        //********************************************************************************
        private void OnComboOpened(object sender, EventArgs e) {
            m_bCtxShown = true;
        }

    }//End class FldLabelComboBox    
}//End namespace
