//*********************************************************
//
// FldLinkLabel:		Overload the microsoft .net LinkLabel to implement 
//                      rep++ Link Label field control
//
// Author:		        Maher Najah
//
// (c) 2003 by Consyst SQL Inc.
//
//*********************************************************
using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.ComponentModel.Design;
using RepPP.Toolkit;
using RepPP;

namespace RepPP.Toolkit.Window {
	/// <summary>
	/// FldLinkLabel: Create a Label around a rep++ field
	/// </summary>
	public class FldLinkLabel : System.Windows.Forms.LinkLabel, FldControlUtil.IFldControl {
        /// <summary>Field control utility class</summary>
        protected FldControlUtil                    m_fldControlUtil;
        /// <summary>Text of the label</summary>
		private string                              m_strText;
        /// <summary> Required designer variable</summary>
		private System.ComponentModel.Container     components = null;
        /// <summary>Indicates if we should display a star when the Bound field is required</summary>
        private FldLabel.RequiredModeE              m_eRequiredMode;    

        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //*********************************************************
		public FldLinkLabel()	{
			InitializeComponent();
            m_fldControlUtil = new FldControlUtil(this, new WndControlUtilHelper(), false);
            m_eRequiredMode  = FldLabel.RequiredModeE.None;
            m_strText        = null;
		}

        //*********************************************************	    
        //
        /// <summary>
        /// Dispose:                    Dispose the control
        /// </summary>
        /// <param name="bDisposing">   true if dispose, false if GC </param>
        //
        //*********************************************************	    
        protected override void Dispose(bool bDisposing) {
            RepPP.Application   app;

			if (bDisposing) {
                if(components != null) {
                    components.Dispose();
                }
                if (m_fldControlUtil != null) {
                    app = Application;
                    if (app != null) {
                        app.ProgInit   -= new RepPP.ProgInitEventHandler(this.OnProgInit);
                        app.ProgEnd    -= new RepPP.ProgEndEventHandler(this.OnProgEnd);
                        app.FldChange  -= new RepPP.FldChangeEventHandler(this.OnFldChange);
                    }
                    m_fldControlUtil.Dispose();
                    m_fldControlUtil = null;
                }
			}
			base.Dispose(bDisposing);
		}

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReInit:        Called when Rep++ is reinitialized in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReInit() {
            RefreshCtl();
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReset:         Called when Rep++ is reset in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReset() {
            m_strText = "[" + FieldFullName.RowsetName + "." + FieldFullName.FieldName + "]";
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_fldControlUtil != null) {
                m_fldControlUtil.TriggerComponentChange();
                RefreshCtl();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FieldFullName:              Full name of the field attached to the label
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Editor("RepPP.Toolkit.Designer.FldControlDesigner+FldControlEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Full name of the field attached to the label")]
		public FldFullName FieldFullName {
		    get {
		        return(m_fldControlUtil.FieldFullName);
		    }
		    set {
		        m_fldControlUtil.FieldFullName = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public RepPP.Application Application {
		    get {
		        return(m_fldControlUtil.Application);
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// FieldName:      Get and set the field name
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public string FieldName {
            get {
                return(FieldFullName.FieldName);
            }
            set {
                FieldFullName.FieldName = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// RowsetName:      Get and set the group name
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public string RowsetName {
            get {
                return(FieldFullName.RowsetName);
            }
            set {
                FieldFullName.RowsetName = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// FieldDef:              Get the attached field
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public FieldDef FieldDef {
		    get {
		        return(m_fldControlUtil.FieldDef);
		    }
		    set {
		        m_fldControlUtil.FieldDef = value;
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// Field:                  Rep++ Field attached to the control
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Field Field {
		    get {
		        return(m_fldControlUtil.Field);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// BindVisible:            Indicates whether or not the visible property of the control
        ///                         is bound to the field property
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [Category("Behavior")]
        [Description("Indicates whether or not the Visible property of the control is bound to the field property")]
        public bool BindVisible {
            get {
                return(m_fldControlUtil.BindVisible);
            }
            set {
                m_fldControlUtil.BindVisible = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// RequiredMode:          Indicates if we should display a star when 
        ///                        the Bound field is required
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [Category("Behavior")]
        [DefaultValue(FldLabel.RequiredModeE.None)]
        [Description("Indicates if a Star will be displayed when the bound Field is required")]
        public FldLabel.RequiredModeE RequiredMode {
		    get {
		        return(m_eRequiredMode);
		    }
		    set {
                if (m_eRequiredMode != value) {
                    m_eRequiredMode = value;
                    Invalidate();
                }
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// RefreshCtl:         Refresh the control
        /// </summary>
        //
        //*********************************************************
        private void RefreshCtl() {
            FieldDef   fldDef;

            if (BindVisible) {
                fldDef  = (m_fldControlUtil != null) ? m_fldControlUtil.FieldDef : null;
                Visible = (fldDef == null) ? true : fldDef.IsVisible;
            }
            m_strText   = null;
            Invalidate();
        }

        //*********************************************************
        //
        /// <summary>
        /// OnProgInit:           Handle the application initialized event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnProgInit(object sender, RepPP.ApplicationEventArgs e) {
            RefreshCtl();
        }

        //*********************************************************
        //
        /// <summary>
        /// OnProgEnd:            Handle the application end event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnProgEnd(object sender, RepPP.ApplicationEventArgs e) {
            FieldDef = null;
        }

        //*********************************************************
        //
        /// <summary>
        /// OnFldChange:          Handle the field change event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnFldChange(object sender, RepPP.FieldGenEventArgs e) {
            FieldDef   fldDef;

            fldDef = m_fldControlUtil.FieldDef;
            if (fldDef != null && fldDef == e.FieldDef) {
                switch(e.FldEvent) {
                case FldEvent.sdFldPromptChg:
                case FldEvent.sdFldStateChg:
                    RefreshCtl();
                    break;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnCreateControl:    Called when the control is created
        /// </summary>
        //
        //*********************************************************
        protected override void OnCreateControl() {
            RepPP.Application   app;

            app = Application;
            if (app != null) {
                RefreshCtl();
                if (!DesignMode) {
                    app.ProgInit    += new RepPP.ProgInitEventHandler(this.OnProgInit);
                    app.ProgEnd     += new RepPP.ProgEndEventHandler(this.OnProgEnd);
                    app.FldChange   += new RepPP.FldChangeEventHandler(this.OnFldChange);
                }
            }
            base.OnCreateControl();
        }

        //*********************************************************
        //
        /// <summary>
        /// Text:               Text of the control
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public override String Text {
            get {
                String          strRetVal;
                String          strRowsetName       = String.Empty;
                String          strFieldName        = String.Empty;
                RepPP.FieldDef  fldDef              = null;

                if (m_fldControlUtil != null) {
                    strRowsetName = m_fldControlUtil.FieldFullName.RowsetName;
                    strFieldName  = m_fldControlUtil.FieldFullName.FieldName;
                    fldDef        = m_fldControlUtil.FieldDef;
                }
                if (m_strText == null) {                    
                    if (fldDef == null) {
                        m_strText = "[? " + strRowsetName + "." + strFieldName + " ?]";
                    } else {
                        m_strText = fldDef.Prompt;
                    }
                }
                strRetVal = m_strText;
                if ( m_eRequiredMode == FldLabel.RequiredModeE.ForceStar ||
                    (m_eRequiredMode == FldLabel.RequiredModeE.AddStarIfRequired && fldDef != null && (fldDef.IsRequired || fldDef.IsPrimaryKey))) {
                    strRetVal += "*";
                }
                return(strRetVal);
            }
            set {
                // Do nothing
            }
        }

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion
	} // class SCFldLinkLable
} // namespace
