//*********************************************************
//
// FldRadioButton:		    Overload the microsoft .net radiobutton to implement 
//                          rep++ radiobutton field control
//
// Author:		            Maher Najah
//
// (c) 2003 by Consyst SQL Inc.
//
//*********************************************************
using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.ComponentModel.Design;
using RepPP.Toolkit;
using RepPP;

namespace RepPP.Toolkit.Window {
    //*********************************************************
    //
	/// <summary>
	/// FldRadioButton: Create a radiobutton around a rep++ field
	/// </summary>
	//
    //*********************************************************
	public class FldRadioButton : System.Windows.Forms.RadioButton, FldControlUtil.IFldControl {
        /// <summary>Choice index to use</summary>
        private int                                 m_iChoiceIndex;
        /// <summary>Field control utility class</summary>
        protected WndFldControlUtil                 m_fldControlUtil;
        /// <summary>True if filling the control</summary>
        private bool                                m_bFilling;
		/// <summary> Required designer variable</summary>
		private System.ComponentModel.Container     components = null;

        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //*********************************************************
		public FldRadioButton()	{
			InitializeComponent();
            m_iChoiceIndex           = -1;
            m_bFilling               = false;
            m_fldControlUtil         = new WndFldControlUtil(this, new WndControlUtilHelper(), false);
		}

        //*********************************************************	    
        //
        /// <summary>
        /// Dispose:                    Dispose the editor
        /// </summary>
        /// <param name="bDisposing">   true if dispose, false if GC </param>
        //
        //*********************************************************	    
        protected override void Dispose(bool bDisposing) {
            RepPP.Application   app;

			if (bDisposing) {
                if(components != null) {
                    components.Dispose();
                }
                if (m_fldControlUtil != null) {
                    app = Application;
                    if (app != null) {
                        app.ProgInit   -= new RepPP.ProgInitEventHandler(this.OnProgInit);
                        app.ProgEnd    -= new RepPP.ProgEndEventHandler(this.OnProgEnd);
                        app.FldChange  -= new RepPP.FldChangeEventHandler(this.OnFldChange);
                    }
                    m_fldControlUtil.Dispose();
                    m_fldControlUtil = null;
                }
			}
			base.Dispose(bDisposing);
		}

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReInit:        Called when Rep++ is reinitialized in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReInit() {
            InitializeCtlv();
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReset:         Called when Rep++ is reset in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReset() {
            Text = "[" + FieldFullName.RowsetName + "." + FieldFullName.FieldName + "]";
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_fldControlUtil != null) {
                m_fldControlUtil.TriggerComponentChange();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FieldFullName:              Full name of the field attached to the label
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Editor("RepPP.Toolkit.Designer.FldControlDesigner+FldControlEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Full name of the field attached to the label")]
		public FldFullName FieldFullName {
		    get {
		        return(m_fldControlUtil.FieldFullName);
		    }
		    set {
		        m_fldControlUtil.FieldFullName = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public RepPP.Application Application {
		    get {
		        return(m_fldControlUtil.Application);
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// FieldName:      Get and set the field name
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public string FieldName {
            get {
                return(FieldFullName.FieldName);
            }
            set {
                FieldFullName.FieldName = value;
                Invalidate();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// RowsetName:      Get and set the group name
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public string RowsetName {
            get {
                return(FieldFullName.RowsetName);
            }
            set {
                FieldFullName.RowsetName = value;
                Invalidate();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// FieldDef:              Get the attached field
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public FieldDef FieldDef {
		    get {
		        return(m_fldControlUtil.FieldDef);
		    }
		    set {
		        m_fldControlUtil.FieldDef = value;
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// Field:                  Rep++ Field attached to the control
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Field Field {
		    get {
		        return(m_fldControlUtil.Field);
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// ChoiceIndex:    Get and set the choice index
        /// </summary>
        //
        //*********************************************************
        public int ChoiceIndex {
            get {
                return(m_iChoiceIndex);
            }
            set {
                if (m_iChoiceIndex != value) {
                    m_iChoiceIndex = value;
                    InitializeCtlv();
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// BindVisible:            Indicates whether or not the visible property of the control
        ///                         is bound to the field property
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [Category("Behavior")]
        [Description("Indicates whether or not the Visible property of the control is bound to the field property")]
        public bool BindVisible {
            get {
                return(m_fldControlUtil.BindVisible);
            }
            set {
                m_fldControlUtil.BindVisible = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// BindEnabled:          Indicates whether or not the Enabled  property 
        ///                       of the control is bound to the field property
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [Category("Behavior")]
        [Description("Indicates whether or not the Enabled property of the control is bound to the field property")]
        public bool BindEnabled {
            get {
                return(m_fldControlUtil.BindSelectable);
            }
            set {
                m_fldControlUtil.BindSelectable = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetStatev:      Sets the control state
        /// </summary>
        //
        //*********************************************************
        private void SetStatev() {
            RepPP.FieldDef fldDef;

            fldDef = m_fldControlUtil.FieldDef;
            if (fldDef != null) {
                if (BindEnabled) {
                    Enabled = fldDef.IsSelectable;
                }
                if (BindVisible) {
                    Visible = fldDef.IsVisible;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// InitializeCtlv:     Initialize the control
        /// </summary>
        //
        //*********************************************************
        private void InitializeCtlv() {
            int             iChoiceCount;
            string          strChoice;
            RepPP.FieldDef  fldDef;
            bool            bDesignMode;

            bDesignMode = DesignMode;
            fldDef      = m_fldControlUtil.FieldDef;            
            if (fldDef != null) {
                if (!bDesignMode) {
                    SetStatev();
                }
                switch(fldDef.Type) {
                case RepPP.FieldType.sdFieldSimpleAlphaChoice:
                case RepPP.FieldType.sdFieldSimpleNumChoice:
                case RepPP.FieldType.sdFieldBoolean:
                    if (m_iChoiceIndex != -1) {
                        iChoiceCount = fldDef.ChoiceList.Count;
                        if (m_iChoiceIndex < 0 || m_iChoiceIndex >= iChoiceCount) {
                            throw new System.Exception("The index of the field " + fldDef.RowsetDef.Name + "." + fldDef.Name +
                                                    " is out of range!");
                        }
                        strChoice = fldDef.ChoiceList.GetDesc(m_iChoiceIndex);
                        if (strChoice == null || strChoice.Length == 0) {
                            strChoice = fldDef.ChoiceList.GetCode(m_iChoiceIndex);
                        }
                        Text = strChoice;
                    }
                    break;
                default:
                    throw new System.Exception("The type of the field " + fldDef.RowsetDef.Name + "." + fldDef.Name +
                                                " must be a choice type or a boolean");
                }
                if (!bDesignMode) {
                    GetFieldValuev();
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetFieldValuev:     Set the field value
        /// </summary>
        //
        //*********************************************************
        private void SetFieldValuev() {
            string          strValue;
            RepPP.FieldDef  fldDef;

            fldDef = m_fldControlUtil.FieldDef;
            if (fldDef != null) {
                switch(fldDef.Type) {
                case RepPP.FieldType.sdFieldSimpleAlphaChoice:
                case RepPP.FieldType.sdFieldSimpleNumChoice:
                case RepPP.FieldType.sdFieldBoolean:
                    if (Checked) {
                        strValue = fldDef.ChoiceList.GetValue(m_iChoiceIndex);
                        if (fldDef.Value != strValue) {
                            m_bFilling  = true;
                            fldDef.Value   = strValue;
                            m_bFilling  = false;
                        }
                    }
                    break;
                default:
                    throw new SystemException("Bad field type. " + fldDef.RowsetDef.Name + "." + fldDef.Name);
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// GetFieldValuev:     Get the field value
        /// </summary>
        //
        //*********************************************************
        private void GetFieldValuev() {
            bool            bChecked;
            int             iChoiceIndex;
            RepPP.FieldDef  fldDef;

            fldDef = m_fldControlUtil.FieldDef;
            if (fldDef != null) {
                switch(fldDef.Type) {
                case RepPP.FieldType.sdFieldSimpleAlphaChoice:
                case RepPP.FieldType.sdFieldSimpleNumChoice:
                case RepPP.FieldType.sdFieldBoolean:
                    iChoiceIndex = fldDef.ChoiceList.FindValue(fldDef.Value);
                    bChecked     = (m_iChoiceIndex == iChoiceIndex);
                    break;
                default:
                    throw new SystemException("Bad field type. " + fldDef.RowsetDef.Name + "." + fldDef.Name);
                }                        
                if (Checked != bChecked) {
                    m_bFilling  = true;
                    Checked     = bChecked;
                    m_bFilling  = false;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnProgInit:           Handle the application initialized event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnProgInit(object sender, RepPP.ApplicationEventArgs e) {
            InitializeCtlv();
        }

        //*********************************************************
        //
        /// <summary>
        /// OnProgEnd:            Handle the application end event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnProgEnd(object sender, RepPP.ApplicationEventArgs e) {
            FieldDef = null;
        }

        //*********************************************************
        //
        /// <summary>
        /// OnFldChange:          Handle the field change event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnFldChange(object sender, RepPP.FieldGenEventArgs e) {
            RepPP.FieldDef fldDef;

            fldDef = (m_fldControlUtil != null) ? m_fldControlUtil.FieldDef : null;
            if (fldDef != null && fldDef == e.FieldDef && !m_bFilling) {
                switch(e.FldEvent) {
                case RepPP.FldEvent.sdFldValueChg:
                    if (!m_bFilling) {
                        GetFieldValuev();
                    }
                    break;
                case RepPP.FldEvent.sdFldChoiceListChg:
                    InitializeCtlv();
                    break;
                case RepPP.FldEvent.sdFldStateChg:
                    SetStatev();
                    break;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnCheckedChange:        Handle the checked change event
        /// </summary>
        /// <param name="e">        Event arguments</param>
        //
        //*********************************************************
        protected override void OnCheckedChanged(System.EventArgs e) {
            if (FieldDef != null && !m_bFilling) {
                SetFieldValuev();
            }
            base.OnCheckedChanged(e);
        }

        //*********************************************************
        //
        /// <summary>
        /// OnCreateControl:    Handle create control event
        /// </summary>
        //
        //*********************************************************
        protected override void OnCreateControl() {
            RepPP.Application   app;

            app = Application;
            if (app != null) {
                InitializeCtlv();
                if (!DesignMode) {
                    app.ProgInit    += new RepPP.ProgInitEventHandler(this.OnProgInit);
                    app.ProgEnd     += new RepPP.ProgEndEventHandler(this.OnProgEnd);
                    app.FldChange   += new RepPP.FldChangeEventHandler(this.OnFldChange);
                }
            }
            base.OnCreateControl();
        }
		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion
	} // class FldRadioButton
} // namespace
