//*********************************************************
//
// FldTextBox:		    Overload the microsoft .net Textbox to implement 
//                      rep++ textbox field control
//
// Author:		        Maher Najah
//
// (c) 2003 by Consyst SQL Inc.
//
//
//*********************************************************
using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.ComponentModel.Design;
using RepPP.Toolkit;
using RepPP;

namespace RepPP.Toolkit.Window {
	/// <summary>
	/// FldTextBox: Create a textbox around a rep++ field
	/// </summary>
	public class FldTextBox : System.Windows.Forms.TextBox, FldControlUtil.IFldControl {
        /// <summary>Field control utility class</summary>
        protected WndFldControlUtil                 m_fldControlUtil;
        /// <summary>True if filling the control</summary>
        private bool                                m_bFilling;
        /// <summary> Required designer variable</summary>
		private System.ComponentModel.Container     components = null;
        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //*********************************************************
		public FldTextBox() {
			InitializeComponent();
            m_bFilling               = false;
            m_fldControlUtil         = new WndFldControlUtil(this, new WndControlUtilHelper(), false);
		}

        //*********************************************************	    
        //
        /// <summary>
        /// Dispose:                    Dispose the editor
        /// </summary>
        /// <param name="bDisposing">   true if dispose, false if GC </param>
        //
        //*********************************************************	    
        protected override void Dispose(bool bDisposing) {
            RepPP.Application   app;

			if (bDisposing) {
                if(components != null) {
                    components.Dispose();
                }
                if (m_fldControlUtil != null) {
                    app = Application;
                    if (app != null) {
                        app.FldChange -= new RepPP.FldChangeEventHandler(this.OnFldChange);
                    }
                    m_fldControlUtil.Dispose();
                    m_fldControlUtil = null;
                }
			}
			base.Dispose(bDisposing);
		}

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReInit:        Called when Rep++ is reinitialized in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReInit() {
            InitializeCtlv();
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReset:         Called when Rep++ is reset in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReset() {
            Text = "[" + FieldFullName.RowsetName + "." +FieldFullName.FieldName + "]";
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_fldControlUtil != null) {
                m_fldControlUtil.TriggerComponentChange();
                InitializeCtlv();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FieldFullName:              Full name of the field attached to the label
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Editor("RepPP.Toolkit.Designer.FldControlDesigner+FldControlEditor, RepPP.Toolkit.Designer.V1B", typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Full name of the field attached to the label")]
		public FldFullName FieldFullName {
		    get {
		        return(m_fldControlUtil.FieldFullName);
		    }
		    set {
		        m_fldControlUtil.FieldFullName = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public RepPP.Application Application {
		    get {
		        return(m_fldControlUtil.Application);
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// FieldName:      Get and set the field name
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public string FieldName {
            get {
                return(FieldFullName.FieldName);
            }
            set {
                FieldFullName.FieldName = value;
                Invalidate();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// RowsetName:      Get and set the group name
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public string RowsetName {
            get {
                return(FieldFullName.RowsetName);
            }
            set {
                FieldFullName.RowsetName = value;
                Invalidate();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// FieldDef:              Get the attached field
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public FieldDef FieldDef {
		    get {
		        return(m_fldControlUtil.FieldDef);
		    }
		    set {
		        m_fldControlUtil.FieldDef = value;
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// Field:                  Rep++ Field attached to the control
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Field Field {
		    get {
		        return(m_fldControlUtil.Field);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// BindVisible:            Indicates whether or not the visible property of the control
        ///                         is bound to the field property
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [Category("Behavior")]
        [Description("Indicates whether or not the Visible property of the control is bound to the field property")]
        public bool BindVisible {
            get {
                return(m_fldControlUtil.BindVisible);
            }
            set {
                m_fldControlUtil.BindVisible = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// BindReadOnly:         Indicates whether or not the ReadOnly  property 
        ///                       of the control is bound to the field property
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [Category("Behavior")]
        [Description("Indicates whether or not the ReadOnly property of the control is bound to the field property")]
        public bool BindReadOnly {
            get {
                return(m_fldControlUtil.BindSelectable);
            }
            set {
                m_fldControlUtil.BindSelectable = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// InitializeCtlv:     Initialize the control
        /// </summary>
        //
        //*********************************************************
        protected virtual void InitializeCtlv() {
            FieldDef    fldDef;
            bool        bDesignMode;

            fldDef          = m_fldControlUtil.FieldDef;
            bDesignMode     = this.DesignMode;
            if (bDesignMode == false && fldDef != null) {
                if (BindReadOnly) {
                    this.ReadOnly = !fldDef.IsSelectable;
                }
                if (BindVisible) {
                    this.Visible = fldDef.IsVisible;
                }
                this.MaxLength        = fldDef.MaxSize;
                this.PasswordChar     = fldDef.IsPassword ? '*' : '\0';
                this.Multiline        = (fldDef.Type == FieldType.sdFieldMemo);
                this.CharacterCasing  = (fldDef.Type == FieldType.sdFieldUpperString) ? CharacterCasing.Upper : CharacterCasing.Normal;
                m_bFilling            = true;
                this.Text             = fldDef.ExtValue;
                m_bFilling            = false;
            } else if (bDesignMode == true) {
                m_bFilling            = true;
                if (fldDef == null) {
                    this.Text             = "[? " + m_fldControlUtil.FieldFullName.RowsetName + "." +
                                            m_fldControlUtil.FieldFullName.FieldName + " ?]";
                } else {
                    this.Multiline        = (fldDef.Type == FieldType.sdFieldMemo);
                    this.Text             = "[" + m_fldControlUtil.FieldFullName.RowsetName + "." +
                                            m_fldControlUtil.FieldFullName.FieldName + "]";
                }
                m_bFilling            = false;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Text:               Text of the TextBox
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        [Bindable(false)]
        public override string Text {
            get {
                return base.Text;
            }
            set {
                base.Text = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// MultiLine:          Multiline
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        [Bindable(false)]
        public new bool Multiline {
            get {
                return base.Multiline;
            }
            set {
                base.Multiline = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetFieldValuev:     Set the field value
        /// </summary>
        //
        //*********************************************************
        private void SetFieldValuev() {
            FieldDef   fldDef;

            fldDef = m_fldControlUtil.FieldDef;
            if (fldDef != null) {
                if (fldDef.ExtValue != this.Text) {
                    m_bFilling   = true;
                    fldDef.ExtValue = this.Text;
                    m_bFilling   = false;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnFldChange:          Handle the field change event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnFldChange(object sender, RepPP.FieldGenEventArgs e) {
            FieldDef   fldDef;

            fldDef = (m_fldControlUtil != null) ? m_fldControlUtil.FieldDef : null;
            if (fldDef != null && fldDef == e.FieldDef && !m_bFilling) {
                InitializeCtlv();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnTextChanged:          Raise the text changed event
        /// </summary>
        /// <param name="e">        Event arguments </param>
        //
        //*********************************************************
        protected override void OnTextChanged(System.EventArgs e) {
            if (!DesignMode && !m_bFilling) {
                SetFieldValuev();
            }
            base.OnTextChanged(e);
        }

        //*********************************************************
        //
        /// <summary>
        /// IsInputChar:              Determines if a character is recognized by the control
        /// </summary>
        /// <param name="CharCode">   The character to test </param>
        //
        //*********************************************************
        protected override bool IsInputChar(char CharCode) {
            bool    bRetVal;
            FieldDef   fldDef;

            fldDef = m_fldControlUtil.FieldDef;
            if (CharCode >= 32 && CharCode <= 255) {
                bRetVal = this.ReadOnly == false && fldDef != null && fldDef.ValidateChar(CharCode);
            } else {
                bRetVal = base.IsInputChar(CharCode);
            }
            return(bRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// OnCreateControl:    Handle create control event
        /// </summary>
        //
        //*********************************************************
        protected override void OnCreateControl() {
            RepPP.Application   app;

            app = Application;
            if (app != null) {
                InitializeCtlv();
                if (!DesignMode) {
                    app.FldChange += new RepPP.FldChangeEventHandler(this.OnFldChange);
                }
            }
            base.OnCreateControl();
        }
		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion
	} // class FldTextBox
} // namesapce
