//*********************************************************
//
// FldValue:		    Overload the Rep++ Field Label to implement 
//                      Rep++ Label Value field control
//
// Author:		        Anthony Thierry
//
// (c) 2006 by Consyst SQL Inc.
//
//
//*********************************************************
using System;
using System.ComponentModel;

namespace RepPP.Toolkit.Window {
    /// <summary>
    /// FldValue: Create a Label Value around a rep++ field
    /// </summary>
    public class FldValue : RepPP.Toolkit.Window.FldLabel {

        /// <summary>Flag Indicates whether or not we should displayed the external value</summary>
        private bool            m_bDispExternal;

        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //*********************************************************
        public FldValue() : base() {
            m_bDispExternal = true;
        }

        //********************************************************************************
        //
        /// <summary>
        /// DisplayExternalValue:   Indicates whether or not we should displayed the external value
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Browsable(true)]
        [DefaultValue(true)]
        [Category("Behavior")]
        [Description("Indicates whether or not we should displayed the external value")]
        public bool DisplayExternalValue {
            get {
                return(m_bDispExternal);
            }
            set {
                m_bDispExternal = value;
            }
        }


        //*********************************************************
        //
        /// <summary>
        /// RequiredMode:          Indicates if we should display a star when 
        ///                        the Bound field is required
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public override FldLabel.RequiredModeE RequiredMode {
            get {
                return(base.RequiredMode);
            }
            set {
                throw new NotImplementedException();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// GetTextDisplayed:   Get Control Text to display 
        ///                     from field
        /// </summary>
        //
        //*********************************************************
        protected override string GetTextDisplayedFromField(RepPP.FieldDef fldDef) {
            String strRetVal;
            
            if (DesignMode) {
                strRetVal = (fldDef == null) ? String.Format("[? {0}.{1} ?]", m_fldControlUtil.FieldFullName.RowsetName, m_fldControlUtil.FieldFullName.FieldName) :
                                            String.Format("[{0}.{1}]", m_fldControlUtil.FieldFullName.RowsetName, m_fldControlUtil.FieldFullName.FieldName);
            } else {
                strRetVal = (m_bDispExternal) ? fldDef.ExtValue : fldDef.Value;
            }
            return(strRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// OnFldChange:          Handle the field change event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        protected override void OnFldChange(object sender, FieldGenEventArgs e) {
            FieldDef   fldDef;

            fldDef = (m_fldControlUtil != null) ? m_fldControlUtil.FieldDef : null;
            if (e.FldEvent == FldEvent.sdFldValueChg && fldDef != null && fldDef == e.FieldDef) {
                m_strText = GetTextDisplayedFromField(fldDef);
                DoAutoSize();
                Invalidate();                
            }
            base.OnFldChange(sender, e);
        }

    } //End class FldValue
} //End namespace



