//*********************************************************
//
// GridDataTable:				Overload the .NET DataTable to interact with group instance and
//								DataGrid
//
// Author:						JFO
//
// (c) 2002 by Consyst SQL Inc.
//
//
//*********************************************************
using System;
using System.Data;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;
using System.Collections;

namespace RepPP.Toolkit.Window {

    //*********************************************************
    //
    /// <summary>
    /// Arguments of the event when a new line is selected
    /// </summary>
    //
    //*********************************************************
    public class BeforeSelectingLineEventArgs : System.EventArgs {
        /// <summary>Line being selected</summary>
        public	int			iLine;
        /// <summary>true to cancel the selection</summary>
        public  bool        bCancel;
        //*********************************************************
        //
        /// <summary>
        /// ctor:               Class constructor
        /// </summary>
        /// <param name="i">    Line being selected</param>
        //
        //*********************************************************
        public				BeforeSelectingLineEventArgs(int i) : base() { iLine = i; bCancel = false; }
    };

    //*********************************************************
    //
    /// <summary>
    /// Arguments of the event when a new line is selected
    /// </summary>
    //
    //*********************************************************
	public class LineSelectedEventArgs : System.EventArgs {
        /// <summary>Line being selected</summary>
		public	int			iLine;
        //*********************************************************
        //
        /// <summary>
        /// ctor:               Class constructor
        /// </summary>
        /// <param name="i">    Line being selected</param>
        //
        //*********************************************************
		public				LineSelectedEventArgs(int i) : base() { iLine = i; }
	};

    //*********************************************************
    //
    /// <summary>
    /// Arguments of the event when a field is selected
    /// </summary>
    //
    //*********************************************************
	public class FieldSelectedEventArgs : System.EventArgs {
		/// <summary>Field being selected</summary>
		public	RepPP.FieldDef	fldDef;
        //*********************************************************
        //
        /// <summary>
        /// ctor:               Class constructor
        /// </summary>
        /// <param name="f">    FieldDef being selected</param>
        //
        //*********************************************************
		public				FieldSelectedEventArgs(RepPP.FieldDef f) : base() { fldDef = f; }
    };


    //*********************************************************
    //
    /// <summary>
    /// Arguments of the event when a row in Error
    /// </summary>
    //
    //*********************************************************
	public class DataRowErrorEventArgs : System.EventArgs {
		/// <summary>Row in error</summary>
		public	DataRow  RowError;
        //*********************************************************
        //
        /// <summary>
        /// ctor:               Class constructor
        /// </summary>
        /// <param name="row">  Row in error</param>
        //
        //*********************************************************
        public DataRowErrorEventArgs(DataRow row) : base() { RowError = row; }
    };

	/// <summary>Delegate for LineSelected event</summary>
	public delegate void LineSelectedEventHandler(object sender, LineSelectedEventArgs e);
	/// <summary>Delegate for FieldSelected event</summary>
	public delegate void FieldSelectedEventHandler(object sender, FieldSelectedEventArgs e);
	/// <summary>Delegate for BeforeSelectintLine event</summary>
	public delegate void BeforeSelectingLineEventHandler(object sender, BeforeSelectingLineEventArgs e);
	/// <summary>Delegate for LineDeleted event</summary>
	public delegate void LineDeletedEventHandler(object sender, System.EventArgs e);
	/// <summary>Delegate for line inserted event</summary>
	public delegate void LineInsertedEventHandler(object sender, System.EventArgs e);
    /// <summary>Delegate for row in error event</summary>
    public delegate void DataRowErrorEventHandler(object sender, DataRowErrorEventArgs e);


	//*********************************************************
	//
    /// <summary>
    ///  This class extend the DataTable class. 
    ///  First, the constructor build the base SCDataTable, build a datagrid table styles and attached it to the datagrid.
    ///  Second, when a cell is entered, this class must ensure that the corresponging group instance line is selected.
    ///  Third, this class validated a line before leaving it. If invalid, the line cannot be unselected.
    ///
    ///  The datagrid work with cell composed of column and row index. These indexes can be used to find the related DataViewRow.
    ///  The column and row index express the index and the column of a DataViewRow, not of a DataTableRow. If a sort is applied,
    ///  the index doesn't matched. For this reason, a fourth position system is defined: a grid position. Grid never display deleted
    ///  lines.
	/// </summary>
    //
    //  ******************
    //  Debug info: 
    //
    //      The DebugToolTip property can be used to display information about the current
    //      selected line of the attached group instance. This way, we can check if the current line
    //      is the one corresponding with the current selected datagrid line.
    //  ******************
    //
	//*********************************************************
	public class GridDataTable : DataTableRowset {
		/// <summary>Type of field protections</summary>
		public enum FieldProtectionE {
		    /// <summary>No field protected</summary>
            FP_NONE     = 0,
            /// <summary>Protect primary key fields</summary>
            FP_PK       = 1,
            /// <summary>Protect all fields</summary>
            FP_ALL      = 2
        };

        /// <summary>Event when a line is selected</summary>
        public event LineSelectedEventHandler	        LineSelected;
        /// <summary>Event when a field is selected</summary>
		public event FieldSelectedEventHandler	        FieldSelected;
		/// <summary>Event before selecting a line</summary>
		public event BeforeSelectingLineEventHandler    BeforeSelectingLine;
		/// <summary>Event when a line is deleted</summary>
		public event LineDeletedEventHandler            LineDeleted;
		/// <summary>Event when a line is inserted</summary>
		public event LineInsertedEventHandler           LineInserted;
        /// <summary>Event when a row is in error</summary>
        public event DataRowErrorEventHandler           DataRowError;

        private DataGrid                m_dataGrid;         // Attached datagrid
        private System.EventHandler     m_eventHandler1;    // Event handler for grid cell changed event
		private System.EventHandler     m_eventHandler2;    // Event handler for grid cell changed event
		private int                     m_iCurLine;         // Current selected line
        private ToolTip                 m_toolTipDebug;     // Debug tooltip info if <> null
        private FieldProtectionE        m_eFieldProtection; // Type of field protections
        private bool                    m_bInitialGridRO;   // Original R/O mode of the grid
		private RepPP.FieldDef			m_fldDefCurrent;		// Current field definition
		/// <summary>Button to move up item</summary>
        protected Button                m_butUp;
        /// <summary>Button to move down the item</summary>
        protected Button                m_butDown;
        /// <summary>Event handler for button UP click</summary>
        protected EventHandler          m_evButUpClick;
        /// <summary>FieldDef which must be autorenum after a move</summary>
        protected RepPP.FieldDef        m_fldAutoRenum;
        /// <summary>Increment for the auto-renum</summary>
        protected int                   m_iAutoInc;
        /// <summary>Event handler for button Down click</summary>
        protected EventHandler          m_evButDownClick;
        /// <summary>Show field with user flag if true</summary>
        protected bool                  m_bShowUserFlagFld;
        /// <summary>Is the value of AllowNew permanent</summary>
        protected bool                  m_bIsAllowNewPermanent;

        //*********************************************************
        //
        /// <summary>
        /// ctor:                           Constructor for the attached dataset
        /// </summary>
        /// <param name="rowsetDef">        Rowset definition</param>
        /// <param name="dataGrid">         Datagrid to attached to the dataset</param>
        /// <param name="bUseCombo">        Use combo box for choice list if true</param>
        /// <param name="bUseDatePicker">   Use date picker for date, time or date time if true</param>
        /// <param name="iMinColSize">      Minimum character size use to display column</param>
        /// <param name="iMaxColSize">      Maximum character size use to display column</param>
        //
        //*********************************************************
        public GridDataTable(RepPP.RowsetDef rowsetDef, DataGrid dataGrid, bool bUseCombo, bool bUseDatePicker, int iMinColSize, int iMaxColSize) : base(rowsetDef) {
            DataGridTableStyle  style;

            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDCTOR, rowsetDef, null, "SCGridDataTable::SCGridDataTable");
            m_bShowUserFlagFld = true;
            m_eventHandler1    = null;
			m_eventHandler2    = null;
			if (dataGrid.TableStyles[TableName] == null) {
                style = BuildDataGridTableStyleO(dataGrid, bUseCombo, bUseDatePicker, iMinColSize, iMaxColSize);
                dataGrid.TableStyles.Add(style);
            }
            dataGrid.SetDataBinding(this, "");
            AttachedDataGrid        = dataGrid;
            m_toolTipDebug          = null;
            m_eFieldProtection      = FieldProtectionE.FP_NONE;
			m_fldDefCurrent		    = null;
            m_butUp                 = null;
            m_butDown               = null;
            m_evButUpClick          = null;
            m_evButDownClick        = null;
            m_fldAutoRenum          = null;
            m_iAutoInc              = 10;
            m_bIsAllowNewPermanent  = false;
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDCTOR, rowsetDef, null, "SCGridDataTable::SCGridDataTable");
        }

        //*********************************************************
        //
		/// <summary>
		/// ctor:                           Constructor for the attached dataset
		/// </summary>
		/// <param name="rowset">           Rowset</param>
		/// <param name="dataGrid">         Datagrid to attached to the dataset</param>
		/// <param name="bUseCombo">        Use combo box for choice list if true</param>
		/// <param name="bUseDatePicker">   Use date picker for date, time or date time if true</param>
		/// <param name="iMinColSize">      Minimum character size use to display column</param>
		/// <param name="iMaxColSize">      Maximum character size use to display column</param>
        //
        //*********************************************************
		public GridDataTable(RepPP.Rowset rowset, DataGrid dataGrid, bool bUseCombo, bool bUseDatePicker, int iMinColSize, int iMaxColSize) : base(rowset) {
            DataGridTableStyle  style;

            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDCTOR, rowset.RowsetDef, rowset, "SCGridDataTable::SCGridDataTable");
            m_bShowUserFlagFld = true;
            m_eventHandler1    = null;
			m_eventHandler2    = null;
			if (dataGrid.TableStyles[TableName] == null) {
                style = BuildDataGridTableStyleO(dataGrid, bUseCombo, bUseDatePicker, iMinColSize, iMaxColSize);
                dataGrid.TableStyles.Add(style);
            }
            dataGrid.SetDataBinding(this, "");
            AttachedDataGrid       = dataGrid;
            m_toolTipDebug         = null;
            m_eFieldProtection     = FieldProtectionE.FP_NONE;
			Rowset                 = rowset;
            m_butUp                = null;
            m_butDown              = null;
            m_evButUpClick         = null;
            m_evButDownClick       = null;
            m_fldAutoRenum         = null;
            m_iAutoInc             = 10;
            m_bIsAllowNewPermanent = false;
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDCTOR, rowset.RowsetDef, rowset, "SCGridDataTable::SCGridDataTable");
        }

        //*********************************************************
        //
        /// <summary>
        /// UpButton:                   Button to move UP the selected item
        /// </summary>
        //
        //*********************************************************
        public Button UpButton {
            get {
                return(m_butUp);
            }
            set {
                Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDMETHOD, "SCGridDataTable::UpButton");
                if (m_butUp != value) {
                    if (m_evButUpClick != null && m_butUp != null) {
                        m_butUp.Click   -= m_evButUpClick;
                        m_evButUpClick   = null;
                    }
                    m_butUp = value;
                    if (m_butUp != null) {
                        m_evButUpClick   = new EventHandler(OnButUpClick);
                        m_butUp.Click   += m_evButUpClick;
                        SetUpDownButtonStatev();
                    }
                }
                Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDMETHOD, "SCGridDataTable::UpButton");
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// DownButton:                 Button to move DOWN the selected item
        /// </summary>
        //
        //*********************************************************
        public Button DownButton {
            get {
                return(m_butDown);
            }
            set {
                Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDMETHOD, "SCGridDataTable::DownButton");
                if (m_butDown != value) {
                    if (m_evButDownClick != null && m_butDown != null) {
                        m_butDown.Click -= m_evButDownClick;
                        m_evButDownClick = null;
                    }
                    m_butDown = value;
                    if (m_butDown != null) {
                        m_evButDownClick   = new EventHandler(OnButDownClick);
                        m_butDown.Click   += m_evButDownClick;
                        SetUpDownButtonStatev();
                    }
                }
                Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDMETHOD, "SCGridDataTable::DownButton");
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// AutoRenumField:             Field use to auto-renum the line after a move
        /// </summary>
        //
        //*********************************************************
        public RepPP.FieldDef AutoRenumField {
            get {
                return(m_fldAutoRenum);
            }
            set {
                m_fldAutoRenum = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// AutoRenumIncrement:         Value use to auto-renum the lines after a move
        /// </summary>
        //
        //*********************************************************
        public int AutoRenumIncrement {
            get {
                return(m_iAutoInc);
            }
            set {
                m_iAutoInc = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// IsAllowNewPermanent:         Value used to indicate that the current
        ///                              AllowNew must not change
        /// </summary>
        //
        //*********************************************************
        public bool IsAllowNewPermanent {
            get {
                return(m_bIsAllowNewPermanent);
            }
            set {
                m_bIsAllowNewPermanent = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetUpDownButtonStatev:  Set the state of the up and down buttons
        /// </summary>
        //
        //*********************************************************
        private void SetUpDownButtonStatev()
        {
            int iRowIndex;

            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_DATAMETHOD, "SCGridDataTable::SetUpDownButtonStatev");
            if (m_dataGrid != null) {
                iRowIndex = m_dataGrid.CurrentRowIndex;
                if (m_butUp != null) {
                    m_butUp.Enabled = (iRowIndex > 0);
                }
                if (m_butDown != null) {
                    m_butDown.Enabled = (iRowIndex >= 0 && iRowIndex < DefaultView.Count - 1);
                }
            }
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_DATAMETHOD, "SCGridDataTable::SetUpDownButtonStatev");
        }

        //******************************************************************
        //
        /// <summary>
        /// AutoRenum:                  Autorenum the lines after a move
        /// </summary>
        //
        //******************************************************************
        protected virtual void AutoRenumV()
        {
            IntPtr                  iCurBmk;
            int                     iCurLine;
            int                     iVal;
            string                  strVal;
            RepPP.LineState         eState;

            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_DATAMETHOD, "SCGridDataTable::AutoRenumV");
            if (m_fldAutoRenum != null && m_rowset != null && m_rowset.LineCount != 0) {
                iCurBmk = m_rowset.LineBookmark(m_rowset.CurrentLine);
                iVal    = m_iAutoInc;
                // Be careful, line count can change because of changes in PK
                for (int iIndex = 0; iIndex < m_rowset.LineCount; iIndex++) {
                    if (m_rowset.ThisLineState(iIndex) != RepPP.LineState.sdLineDeleted) {
                        m_rowset.SelectLine(iIndex, false, false);
                        strVal = iVal.ToString();
                        if (m_fldAutoRenum.Value != strVal) {
                            m_fldAutoRenum.Value = strVal;
                            eState               = m_rowset.ThisLineState(iIndex);
                        }
                        iVal += m_iAutoInc;
                    }
                }
                iCurLine = m_rowset.LineFromBookmark(iCurBmk);
                if (iCurLine != -1 && iCurLine != m_rowset.CurrentLine) {
                    m_rowset.SelectLine(iCurLine, false, false);
                }
            }
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_DATAMETHOD, "SCGridDataTable::AutoRenumV");
        }

        //******************************************************************
        //
        /// <summary>
        /// MoveLinev:                          Move the current line after the specified one
        /// </summary>
        /// <param name="iAfterLineIndex">      Line after which the current line will be moved to </param>
        //
        //******************************************************************
        private void MoveLinev(int iAfterLineIndex)
        {
            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_DATAMETHOD, "SCGridDataTable::MoveLinev");
            m_rowset.MoveLineTo(iAfterLineIndex);
            AutoRenumV();
            Refresh();
            OnLineSelected(new LineSelectedEventArgs(m_rowset.CurrentLine));
            SetUpDownButtonStatev();
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_DATAMETHOD, "SCGridDataTable::MoveLinev");
        }

        //******************************************************************
        //
        /// <summary>
        /// OnButUpClick:           Called when the button up is clicked
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event parameter</param>
        //
        //******************************************************************
        private void OnButUpClick(object sender, EventArgs e) {
            int                     iIndex;
            int                     iCurLine;
            bool                    bFound;

            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDSINK, "SCGridDataTable::OnButUpClick");
            m_rowset.Select(false, false);
            iCurLine  = m_rowset.CurrentLine;
            if (iCurLine > 0 && m_rowset.LineState != RepPP.LineState.sdLineDeleted) {
                bFound = false;
                iIndex = iCurLine - 1;
                while (iIndex >= 0 && bFound == false) {
                    if (m_rowset.ThisLineState(iIndex) != RepPP.LineState.sdLineDeleted) {
                        MoveLinev(iIndex - 1);
                        bFound = true;
                    }
                    iIndex--;
                }
            }
            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDSINK, "SCGridDataTable::OnButUpClick");
        }

        //******************************************************************
        //
        /// <summary>
        /// OnButDownClick:         Called when the button down is clicked
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event parameter</param>
        //
        //******************************************************************
        private void OnButDownClick(object sender, EventArgs e) 
        {
            int     iIndex;
            int     iCurLine;
            bool    bFound;

            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDSINK, "SCGridDataTable::OnButDownClick");
            m_rowset.Select(false, false);
            iCurLine  = m_rowset.CurrentLine;
            if (iCurLine != -1 && iCurLine < m_rowset.LineCount - 1 && m_rowset.LineState != RepPP.LineState.sdLineDeleted) {
                bFound = false;
                iIndex = iCurLine + 1;
                while (iIndex < m_rowset.LineCount && bFound == false) {
                    if (m_rowset.ThisLineState(iIndex) != RepPP.LineState.sdLineDeleted) {
                        MoveLinev(iIndex);
                        bFound = true;
                    }
                    iIndex++;
                }
            }
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDSINK, "SCGridDataTable::OnButDownClick");
        } 

        //*********************************************************
        //
        /// <summary>
        /// OnBeforeSelectingLine:	Trigger before selecting a line event
        /// </summary>
        /// <param name="e">        Event arguments</param>
        //
        //*********************************************************
        protected virtual void OnBeforeSelectingLine(BeforeSelectingLineEventArgs e) {
            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDEVENT, "SCGridDataTable::OnBeforeSelectingLine");
            if (BeforeSelectingLine != null) {
                BeforeSelectingLine(this, e);
            }
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDEVENT, "SCGridDataTable::OnBeforeSelectingLine");
        }

		//*********************************************************
		//
		/// <summary>
		/// OnLineSelected:		Trigger the LineSelected event
		/// </summary>
		/// <param name="e">    Event arguments</param>
		//
		//*********************************************************
		protected virtual void OnLineSelected(LineSelectedEventArgs e) {
            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDEVENT, "SCGridDataTable::OnLineSelected");
            if (LineSelected != null) {
				LineSelected(this, e);
			}
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDEVENT, "SCGridDataTable::OnLineSelected");
        }

        //*********************************************************
        //
        /// <summary>
        ///  OnLineDeleted:		Triggered when a line is deleted via the datagrid
        /// </summary>
        /// <param name="e">    Event argument</param>
        //
        //*********************************************************
        protected virtual void OnLineDeleted(System.EventArgs e) {
            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDEVENT, "SCGridDataTable::LineDeleted");
            if (LineDeleted != null) {
                LineDeleted(this, e);
            }
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDEVENT, "SCGridDataTable::LineDeleted");
        }

        //*********************************************************
        //
        /// <summary>
        /// OnLineInserted:		Triggered when a line is inserted via the datagrid
        /// </summary>
        /// <param name="e">    Event arguments</param>
        //
        //*********************************************************
        protected virtual void OnLineInserted(System.EventArgs e) {
            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDEVENT, "SCGridDataTable::LineInserted");
            if (LineInserted != null) {
                LineInserted(this, e);
            }
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDEVENT, "SCGridDataTable::LineInserted");
        }

		//*********************************************************
		//
		/// <summary>
		/// OnFieldSelected     Trigger the FieldSelected event
		/// </summary>
		/// <param name="e">    Event arguments</param>
		//
		//*********************************************************
		protected virtual void OnFieldSelected(FieldSelectedEventArgs e) {
            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDEVENT, "SCGridDataTable::FieldSelected");
            if (FieldSelected != null) {
				FieldSelected(this, e);
			}
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDEVENT, "SCGridDataTable::FieldSelected");
        }

        //*********************************************************	    
        //
        /// <summary>
        /// EndEditV:               End the edition
        /// </summary>
        /// <param name="bCancel">  true to cancel</param>
        //
        //*********************************************************	    
        public void EndEditV(bool bCancel) {
            int                 iRowIndex;
            int                 iColIndex;
            DataRowView         vrow;
            DataGridColumnStyle colStyle;

            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDSINK, "SCGridDataTable::EndEdit Cancel = " + bCancel.ToString());
            try {
                if (!m_bInitializing && m_rowset != null && m_dataGrid != null) {
                    iRowIndex = m_dataGrid.CurrentCell.RowNumber;
                    iColIndex = m_dataGrid.CurrentCell.ColumnNumber;
                    if (iRowIndex < DefaultView.Count) {
                        vrow   = DefaultView[iRowIndex];
                        if (vrow != null) {
                            if (bCancel) {
                                vrow.CancelEdit();
                            } else {
                                vrow.EndEdit();
                            }
                        }
                    }
                    if (m_dataGrid.TableStyles.Count > 0 &&
                        m_dataGrid.TableStyles[0].GridColumnStyles.Count > iColIndex) {
                        colStyle = m_dataGrid.TableStyles[0].GridColumnStyles[iColIndex];
                        if (colStyle != null) {
                            m_dataGrid.EndEdit(colStyle, iRowIndex, bCancel);
                        }
                    }
                }
            } catch(System.Exception eArg) {
                MessageBox.Show(eArg.ToString());
            }                
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDSINK, "SCGridDataTable::EndEdit Cancel = " + bCancel.ToString());
        }

        //*********************************************************	    
        //
        /// <summary>
        /// OnColumnChanged:    Called when a datatable column changed
        /// </summary>
        /// <param name="e">    Event arguments</param>
        //
        //*********************************************************	    
        protected override void OnColumnChanged(DataColumnChangeEventArgs e) {
            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDSINK, "SCGridDataTable::OnColumnChanged");
            base.OnColumnChanged(e);
            EndEditV(false);
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDSINK, "SCGridDataTable::OnColumnChanged");
        }

        //*********************************************************
        //
        /// <summary>
        /// FieldProtection:        Gets / sets the field protection level
        /// </summary>
        //
        //*********************************************************
        public FieldProtectionE FieldProtection {
            get {
                return(m_eFieldProtection);
            }
            set {
                Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDMETHOD, "SCGridDataTable::FieldProtection = " + value.ToString());
                if (value != m_eFieldProtection) {                    
                    DataGridTableStyle  style;
                    RepPP.FieldDef         fldDef;

                    style = m_dataGrid.TableStyles[TableName];
                    if (style != null) {
                        if (value == FieldProtectionE.FP_ALL) {
                            style.ReadOnly = true;
                        } else {
                            style.ReadOnly = false;
                            foreach (DataGridColumnStyle cs in style.GridColumnStyles) {
                                fldDef = m_rowsetDef.FieldDefs[cs.MappingName];
                                if (fldDef != null) {
                                    if (fldDef.IsPrimaryKey) {
                                        cs.ReadOnly = (value == FieldProtectionE.FP_PK) || (fldDef.IsSelectable == false);
                                    }
                                }
                            }
                        }
                    }                    
                    m_eFieldProtection = value;
                }
                Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDMETHOD, "SCGridDataTable::FieldProtection = " + value.ToString());
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// FindFirstNonDeletedLinei:   Find the closer non deleting line
        /// </summary>
        /// <param name="iLine">        Line to start with</param>
        /// <returns>
        /// Line number
        /// </returns>
        //
        //*********************************************************	    
        private int FindFirstNonDeletedLinei(int iLine) {
            int     iLineCount;
            
            iLineCount = m_rowset.LineCount;
            while ((iLine+1) < iLineCount && m_rowset.ThisLineState(iLine) == RepPP.LineState.sdLineDeleted) {
                iLine++;
            }
            while (iLine >= 0 && m_rowset.ThisLineState(iLine) == RepPP.LineState.sdLineDeleted) {
                iLine--;
            }
            return(iLine);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Refresh:    Refresh the content of the dataset with the group instance
        /// </summary>
        //
        //*********************************************************	    
        public override void Refresh() {
            int     iRowIndex;

            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDMETHOD, "SCGridDataTable::Refresh");
            EndEditV(true);
            base.Refresh();
            m_bInitializing = true;
            if (m_rowset == null) {
                m_dataGrid.ReadOnly = true;
            } else {
                m_dataGrid.ReadOnly = m_bInitialGridRO;
                m_iCurLine = m_rowset.CurrentLine;
                if (m_iCurLine != -1 && m_rowset.ThisLineState(m_iCurLine) == RepPP.LineState.sdLineDeleted) {
                    m_iCurLine = FindFirstNonDeletedLinei(m_iCurLine);
                    if (m_iCurLine != -1) {
                        m_rowset.SelectLine(m_iCurLine, true, true);
                    }                    
                }
                if (m_iCurLine != -1) {
                    iRowIndex = CnvLineToNeutralPosI(m_iCurLine);
                    m_dataGrid.CurrentRowIndex = iRowIndex;
                }
            }
            SetDebugInfov();
            SetUpDownButtonStatev();
            m_bInitializing = false;
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDMETHOD, "SCGridDataTable::Refresh");
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Rowset:      Get or set the attached group instance
        /// </summary>
        //
        //*********************************************************	    
        public override RepPP.Rowset Rowset {
            get {
                return(base.Rowset);
            }
            set {
                Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDMETHOD, m_rowsetDef, value, "SCGridDataTable::Rowset");
                EndEditV(true);
                base.Rowset = value;
                Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDMETHOD, m_rowsetDef, value, "<Exit>  SCGridDataTable::Rowset");
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// DebugToolTip:       Set or get the debug tooltip control
        /// </summary>
        //
        //*********************************************************
        public ToolTip DebugTooltip {
            get {
                return(m_toolTipDebug);
            }
            set {
                m_toolTipDebug = value;
                SetDebugInfov();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetDebugInfov:      Set a tooltip with debug info
        /// </summary>
        //
        //*********************************************************
        private void SetDebugInfov() {
            string  str;

            if (m_rowset != null) {
                if (m_toolTipDebug != null && m_dataGrid != null) {
                    if (m_rowset.CurrentLine == -1) {
                        str = "<No line>";
                    } else if (m_iCurLine == -1) {
                        str = "<new line>";
                    } else {
                        str = "C=" + m_rowset.LineCount + "/" + m_rowset.DeletedLineCount + ",L=" + m_rowset.CurrentLine + ",S=" + m_rowset.LineState + "] " + BuildPrimKeyValStr();
                    }
                    m_toolTipDebug.SetToolTip(m_dataGrid, str);
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// AttachedDataGrid:   Datagrid to which to attach
        /// </summary>
        //
        //*********************************************************
        public DataGrid AttachedDataGrid {
            get {
                return(m_dataGrid);
            }
            set {
                Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDMETHOD, "SCGridDataTable::AttachedDataGrid");
                if (m_dataGrid != null && m_eventHandler1 != null) {
					m_dataGrid.BindingContext[this].CurrentChanged -= m_eventHandler1;
					m_eventHandler1									= null;
				}
				if (m_dataGrid != null && m_eventHandler2 != null) {
					m_dataGrid.CurrentCellChanged -= m_eventHandler2;
					m_eventHandler2				   = null;
				}
                m_dataGrid									    = value;
                this.m_bInitialGridRO                           = value.ReadOnly;
                m_eventHandler1								    = new System.EventHandler(this.CurrentChangedSink);
				m_eventHandler2								    = new System.EventHandler(this.CurrentCellChangedSink);
				m_dataGrid.BindingContext[this].CurrentChanged += m_eventHandler1;
				m_dataGrid.CurrentCellChanged				   += m_eventHandler2;
				m_fldDefCurrent									= null;
				if (m_rowset == null) {
				    m_iCurLine = -1;
				} else {
                    m_iCurLine = CnvNeutralPosToLineI(GetCurRowPosI());
                    if (m_iCurLine != -1) {
                        if (m_iCurLine < m_rowset.LineCount) {
                            m_rowset.SelectLine(m_iCurLine, true, true);
                        } else {
                            MessageBox.Show("Too many lines???");
                        }
                    }
                }
                SetDebugInfov();
                Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDMETHOD, "SCGridDataTable::AttachedDataGrid");
            }
        }

        //*********************************************************
		//
		/// <summary>
		/// GetCurRowPosI:			Gets the current row position.
		/// </summary>
		/// <returns>
		/// Row position or -1
		/// </returns>
		//
        //*********************************************************
		private int GetCurRowPosI() {
			int			        iRetVal = -1;
			DataRowView	        rowView;
			BindingManagerBase  mbase;

            mbase = m_dataGrid.BindingContext[this];
            if (mbase != null) {
                if (mbase.Count != 0 && mbase.Position != -1) {
        			rowView   = (DataRowView)mbase.Current;
	        		if (rowView != null) {
		        		iRetVal = GetRowNeutralPosI(rowView.Row);
		            }
		        }
			}
			return(iRetVal);
		}

        //*********************************************************
		//
		/// <summary>
		/// FindFldColPosI:			Find the position of a field
		/// </summary>
		/// <param name="fldDef">   FieldDef</param>
		/// <returns>
		/// Column position or -1 if not found
		/// </returns>
        //
        //*********************************************************
		private int FindColPosFromNameI(RepPP.FieldDef fldDef) {
			int					iRetVal = -1;
			DataGridTableStyle	tableStyle;

			tableStyle = m_dataGrid.TableStyles[fldDef.RowsetDef.Name];
			if (tableStyle != null) {
				iRetVal = tableStyle.GridColumnStyles.IndexOf(tableStyle.GridColumnStyles[fldDef.Name]);
			}
			return(iRetVal);
		}

		//*********************************************************
		//
		/// <summary>
		/// FindFldByColPosO:		Find the field from a column position
		/// </summary>
		/// <param name="iColPos">  Column position</param>
		/// <returns>
		/// FieldDef or null if none
		/// </returns>
		//
		//*********************************************************
		private RepPP.FieldDef FindFldByColPosO(int iColPos) {
			RepPP.FieldDef	    fldRetVal = null;
			DataGridTableStyle	tableStyle;
			DataGridColumnStyle	colStyle;

			tableStyle = m_dataGrid.TableStyles[m_rowsetDef.Name];
			if (tableStyle != null) {
				colStyle = tableStyle.GridColumnStyles[iColPos];
				if (colStyle != null) {
					fldRetVal = m_rowsetDef.FieldDefs[colStyle.MappingName];
				}
			}
			return(fldRetVal);
		}

        //*********************************************************
		//
		//	ChangeCurrentLinev:		Change the current line
		//
        //*********************************************************
		private void ChangeCurrentLinev() {
			RepPP.FieldDef  fldDef;
			int			    iRowIndex;
			int             iCurLine;
			int			    iFldPos = -1;
			bool            bCancel = false;

            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDSINK, "SCGridDataTable::ChangeCurrentLinev");
            try {
                if (!m_bInitializing && m_rowset != null && !m_rowset.IsDetached) {            
			        iRowIndex = GetCurRowPosI();
			        iCurLine  = CnvNeutralPosToLineI(iRowIndex);
			        if (iCurLine != m_iCurLine) {
				        if (m_iCurLine != -1) {
					        if (m_rowset.ThisLineState(m_iCurLine) != RepPP.LineState.sdLineDeleted) {
					            if (m_rowset.RowsetDef.ValidAns(out fldDef) != 0) {
						            iFldPos	  = FindColPosFromNameI(fldDef);
						            bCancel   = true;
						        } else {
						            BeforeSelectingLineEventArgs    earg = new BeforeSelectingLineEventArgs(iCurLine);
    						        
						            OnBeforeSelectingLine(earg);
						            bCancel = earg.bCancel;
						        }
						        if (bCancel) {
                                    iRowIndex = CnvNeutralPosToViewPosI(CnvLineToNeutralPosI(m_iCurLine));
                                    if (iRowIndex == -1) {
                                        MessageBox.Show("Unable to find the row????");
                                    } else {
                                        if (iFldPos != -1) {
                                            m_dataGrid.CurrentCell = new DataGridCell(iRowIndex, iFldPos);
                                        } else {
                                            m_dataGrid.CurrentRowIndex = iRowIndex;
                                        }
                                        iCurLine = m_iCurLine;
                                    }
                                }
                            }
				        }
				        if (iCurLine != m_iCurLine) {
					        m_iCurLine = iCurLine;
					        if (iCurLine != -1) {
					            m_rowset.SelectLine(iCurLine, true, true);
					        }
					        if (!bCancel) {
						        OnLineSelected(new LineSelectedEventArgs(iCurLine));
						    }
					    }
			        }
			    }
                SetUpDownButtonStatev();
                SetDebugInfov();
            } catch(System.Exception eArg) {
                MessageBox.Show(eArg.ToString());
            }
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDSINK, "SCGridDataTable::ChangeCurrentLinev");
        }

        //*********************************************************
		//
		//	CurrentChangedSink:		Called when a new row is selected.
		//
        //*********************************************************
		private void CurrentChangedSink(Object sender, EventArgs e) {
            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDSINK, "SCGridDataTable::CurrentChangedSink");
            ChangeCurrentLinev();
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDSINK, "SCGridDataTable::CurrentChangedSink");
        }

		//*********************************************************
		//
		//	CurrentCellChangedSink:	    Called when a new cell is selected.
		//
		//*********************************************************
		private void CurrentCellChangedSink(Object sender, EventArgs e) {
			RepPP.FieldDef fldDef = null;

            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDSINK, "SCGridDataTable::CurrentCellChangedSink");
            try {
			    fldDef = FindFldByColPosO(m_dataGrid.CurrentCell.ColumnNumber);
			    if (fldDef != m_fldDefCurrent) {
				    m_fldDefCurrent = fldDef;
				    OnFieldSelected(new FieldSelectedEventArgs(fldDef));
			    }
                ChangeCurrentLinev();
            } catch(System.Exception eArg) {
                MessageBox.Show(eArg.ToString());
            }
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDSINK, "SCGridDataTable::CurrentCellChangedSink");
        }

		//*********************************************************
		//
        /// <summary>
        /// ClearError:     Clear the error for the current row
        /// </summary>
        /// <param name="row">row to clear error</param>
        //
        //*********************************************************
        public void ClearRowError(DataRow row) {
            if (row != null && row.RowError != String.Empty) {
                row.RowError = String.Empty;
            }
        }

		//*********************************************************
		//
        /// <summary>
        /// ClearError:     Clear the error for the row index
        /// </summary>
        /// <param name="iRowIndex">Row index</param>
        //
        //*********************************************************
        public void ClearRowError(int iRowIndex) {
            ClearRowError(Rows[iRowIndex]);
        }

		//*********************************************************
		//
        /// <summary>
        /// ClearError:     Clear the error at the current row position
        /// </summary>
        //
        //*********************************************************
        public void ClearRowError() {
            int iCurrentIndex;

            iCurrentIndex = AttachedDataGrid.CurrentRowIndex;
            if (iCurrentIndex != -1) {
                ClearRowError(iCurrentIndex);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// ClearErrors:     Clear all Errors
        /// </summary>
        //
        //*********************************************************
        public void ClearRowErrors() {
            foreach (DataRow row in Rows) {
                ClearRowError(row);
            }            
        }

        //*********************************************************
        //
        /// <summary>
        /// SetRowError:        Set an error for that row
        /// </summary>
        /// <param name="row">  Row to set an error</param>
        /// <param name="strMessage">Error message</param>
        //
        //*********************************************************
        public void SetRowError(DataRow row, string strMessage) {
            if (row != null) {
                row.RowError = strMessage;
                if (DataRowError != null) {
                    DataRowError(this, new DataRowErrorEventArgs(row));
                }                
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetRowError:        Set an error for the row index
        /// </summary>
        /// <param name="iRowIndex">  Row Index</param>
        /// <param name="strMessage">Error message</param>
        //
        //*********************************************************
        public void SetRowError(int iRowIndex, string strMessage) {
            SetRowError(Rows[iRowIndex], strMessage);
        }

        //*********************************************************
        //
        /// <summary>
        /// SetRowError:        Set an error for the current row
        /// </summary>
        /// <param name="strMessage">Error message</param>
        //
        //*********************************************************
        public void SetRowError(string strMessage) {
            int iCurrentIndex;

            iCurrentIndex = AttachedDataGrid.CurrentRowIndex;
            if (iCurrentIndex != -1) {
                SetRowError(iCurrentIndex, strMessage);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// ListChangedSink:        Called when a dataview list changes
        /// </summary>
        /// <param name="sender">   Event sender</param>
        /// <param name="e">        Event argument</param>
        //
        //*********************************************************
        protected override void ListChangedSink(object sender, ListChangedEventArgs e) {
            int     iViewPos;
            int     iLine;
            int     iLineCount = 0;
            
            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDSINK, "SCGridDataTable::ListChangedSink = " + e.ListChangedType.ToString());
            try {
                if (!m_bInitializing && m_rowset != null && !m_rowset.IsDetached) {
                    switch(e.ListChangedType) {
                    case ListChangedType.ItemAdded:
                        iLineCount = m_rowset.LineCount;
                        break;
                    case ListChangedType.ItemChanged:
                        break;
                    case ListChangedType.ItemDeleted:
                        // If the line deleted is the current line, we must select a new
                        // line. So, set the current line to 0 so the new line will be selected
                        // without validation.
                        iViewPos = e.NewIndex;
                        iLine    = CnvNeutralPosToLineI(CnvViewPosToNeutralPosI(iViewPos));
                        if (iLine == m_rowset.CurrentLine) {
                            m_iCurLine = -1;
                            Trace(TraceTypeE.TT_NONE, TraceCatE.TC_GRIDSINK, "SCGridDataTable::ListChangedSink Select line -1!");
                        }
                        break;
                    case ListChangedType.ItemMoved:
                        break;
                    case ListChangedType.Reset:
                        break;
                    default:
                        break;
                    }
                }
                base.ListChangedSink(sender, e);            
                if (!m_bInitializing && m_rowset != null && !m_rowset.IsDetached) {
                    if (e.ListChangedType == ListChangedType.ItemDeleted) {
                        OnLineDeleted(System.EventArgs.Empty);
                    } else if (e.ListChangedType == ListChangedType.ItemAdded) {
                        if (iLineCount != m_rowset.LineCount) {
                            OnLineInserted(System.EventArgs.Empty);
                        }                    
                    }
                }
                SetDebugInfov();
            } catch(System.Exception eArg) {
                MessageBox.Show(eArg.ToString());
            }
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDSINK, "SCGridDataTable::ListChangedSink = " + e.ListChangedType.ToString());
        }

        //*********************************************************
        //
        /// <summary>
        /// SetColumnControlType:       Set the control type of a column
        /// </summary>
        /// <param name="fldDef">       FieldDef</param>
        /// <param name="eType">        Control type</param>
        /// <param name="bShowDesc">    true to show description, false to show code</param>
        /// <param name="bListDesc">    true to show description in list, false to show code</param>
        //
        //*********************************************************
        public void SetColumnControlType(RepPP.FieldDef fldDef, RepPP.CtlType eType, bool bShowDesc, bool bListDesc) {
            DataGridColumn      colStyle;
            DataGridColumnStyle colStyleOld;
            DataGridTableStyle  tblStyle;

            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDMETHOD, "SCGridDataTable::SetColumnControlType Field = " + fldDef.Name + " Type = " + eType.ToString());
            tblStyle = m_dataGrid.TableStyles[TableName];
            if (tblStyle != null) {
                colStyleOld = tblStyle.GridColumnStyles[fldDef.Name];
                if (colStyleOld is DataGridColumn) {
                    colStyle = (DataGridColumn)colStyleOld;
                    switch(eType) {
                    case RepPP.CtlType.sdCtlCombo:
                        colStyle.ColStyle        = DataGridColumn.ColStyleE.CS_COMBO;
                        colStyle.ShowDescription = bShowDesc;
                        colStyle.ListDescription = bListDesc;
                        break;
                    case RepPP.CtlType.sdCtlFmtDate:
                        colStyle.ColStyle = DataGridColumn.ColStyleE.CS_DATE;
                        break;
                    default:
                        colStyle.ColStyle = DataGridColumn.ColStyleE.CS_STD;
                        break;
                    }
                }
            }
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDMETHOD, "SCGridDataTable::SetColumnControlType Field = " + fldDef.Name + " Type = " + eType.ToString());
        }

        //*********************************************************
        //
        /// <summary>
        /// SetColumnControlType:       Set the control type of a column
        /// </summary>
        /// <param name="fldDef">          FieldDef</param>
        /// <param name="eType">        Control type</param>
        //
        //*********************************************************
        public void SetColumnControlType(RepPP.FieldDef fldDef, RepPP.CtlType eType) {
            SetColumnControlType(fldDef, eType, true, true);
        }

        //*********************************************************
        //
        /// <summary>
        /// IsUserFlagFieldShown:       Determine if the field with user flag on must
        ///                             be included in the list.
        /// </summary>
        //
        //*********************************************************
        public bool IsUserFlagFieldShown {
            get {
                return(m_bShowUserFlagFld);
            }
            set {
                m_bShowUserFlagFld = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// IsFieldShown:           
        /// </summary>
        //
        //*********************************************************
        protected bool IsFieldShown(RepPP.FieldDef fldDef) {
            bool    bRetVal;
            
            bRetVal = fldDef.IsVisible && (!fldDef.IsUserFlagOn || m_bShowUserFlagFld);
            return(bRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// BuildDataGridTableStyleO:       Build style for datagrid using the dataset as template.
        /// </summary>
        /// <param name="dataGrid">         Datagrid for which to build the style</param>
        /// <param name="bUseCombo">        Use combo box for choice list if true</param>
        /// <param name="bUseDatePicker">   Use date picker for date, time or date time if true</param>
        /// <param name="iMinColSize">      Minimum character size use to display column</param>
        /// <param name="iMaxColSize">      Maximum character size use to display column</param>
        /// <returns>
        /// DataGridTableStyle for the grid
        /// </returns>
        //
        //*********************************************************
        public DataGridTableStyle BuildDataGridTableStyleO(DataGrid dataGrid, bool bUseCombo, bool bUseDatePicker, int iMinColSize, int iMaxColSize) {
            DataGridTableStyle              oRetVal;
            DataGridColumnStyle             colStyle;
            PropertyDescriptorCollection    props;
            PropertyDescriptor              prop;
            Graphics                        gr;
            int                             iHeaderSize;
            int                             iTextSize;
            int                             iSize;
            int                             iDescSize = 0;
            string                          str;
            string                          str2;

            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDMETHOD, "SCGridDataTable::BuildDataGridTableStyle");
            str2 = new String('0', iMaxColSize);
            gr = Graphics.FromHwnd(dataGrid.Handle);            
            oRetVal             = new DataGridTableStyle();
            oRetVal.MappingName = TableName;
            props               = TypeDescriptor.GetProperties(this);
            foreach(RepPP.FieldDef fldDef in m_rowsetDef.FieldDefs) {
                if (IsFieldShown(fldDef)) {
                    prop                 = props.Find(fldDef.Name, true);
                    colStyle             = null;
                    switch(fldDef.Type) {
                    case RepPP.FieldType.sdFieldSimpleAlphaChoice:
                    case RepPP.FieldType.sdFieldSimpleNumChoice:
                        if (bUseCombo) {
                            colStyle  = new DataGridColumn(prop, fldDef, this, DataGridColumn.ColStyleE.CS_COMBO);
                            iDescSize = DataGridComboBoxColumn.GetDescMaxSizeI(gr, dataGrid.Font, fldDef) +
                                        (int)gr.MeasureString("WW", dataGrid.Font).Width;    // Size of the scroll
                        }
                        break;
                    case RepPP.FieldType.sdFieldDate:
                    case RepPP.FieldType.sdFieldDateTime:
                    case RepPP.FieldType.sdFieldTime:
                        if (bUseDatePicker) {
                            colStyle  = new DataGridColumn(prop, fldDef, this, DataGridColumn.ColStyleE.CS_DATE);
                            iDescSize = 0;
                        }
                        break;
                    default:
                        break;
                    }
                    if (colStyle == null) {
                        colStyle  = new DataGridColumn(prop, fldDef, this, DataGridColumn.ColStyleE.CS_STD);
                        iDescSize = 0;
                    }
                    colStyle.MappingName = fldDef.Name;
                    colStyle.HeaderText  = fldDef.Prompt;   
                    colStyle.ReadOnly    = fldDef.IsSelectable ? false : true;            

                    str                  = fldDef.Prompt;
                    if (str.Length > iMaxColSize) {
                        str = str.Substring(0, iMaxColSize);
                    } else {
                        str += "i";     // A little bit larger please
                    }
                    iHeaderSize          = (int)gr.MeasureString(str, dataGrid.HeaderFont).Width;
                    iSize                = fldDef.MaxSize;
                    if (iSize < iMinColSize) {
                        iSize = iMinColSize;
                    } else if (iSize > iMaxColSize) {
                        iSize = iMaxColSize;
                    }
                    str                  = str2.Substring(0, iSize);
                    iTextSize            = (int)gr.MeasureString(str, dataGrid.Font).Width;
                    colStyle.Width       = (iHeaderSize > iTextSize) ? iHeaderSize : iTextSize;
                    if (iDescSize > colStyle.Width) {
                        colStyle.Width = iDescSize;
                    }
                    colStyle.NullText    = "";
                    System.Collections.IList list = (IList)oRetVal.GridColumnStyles;
                    oRetVal.GridColumnStyles.Add(colStyle);
                }
            }
            gr.Dispose();
            m_eFieldProtection = FieldProtectionE.FP_NONE;
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDMETHOD, "SCGridDataTable::BuildDataGridTableStyle");
            return(oRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// GetSelectedRow:	        Gets the current selected row by the datagrid
        /// </summary>
        /// <returns>
        /// Selected row
        /// </returns>
        //
        //*********************************************************
        public DataRow GetSelectedRow() {
            DataRow     rowRetVal = null;
            DataRowView	rowView;

            rowView   = (DataRowView)m_dataGrid.BindingContext[this].Current;
            if (rowView != null) {
                rowRetVal = rowView.Row;
            }
            return(rowRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// ResetLinePosV:      Reset the line position to correspond to the grid
        /// </summary>
        //
        //*********************************************************
        public void ResetLinePosV() {
            int     iRowIndex;
            int     iLine;
            int     iCount;

            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDMETHOD, "ResetLinePosV");
            if (m_rowset != null) {
                iCount = m_rowset.LineCount;
                if (iCount != 0) {
                    iRowIndex = GetCurRowPosI();
                    if (iRowIndex != -1) {
                        iLine = CnvNeutralPosToLineI(iRowIndex);
                        if (iLine != m_rowset.CurrentLine && iLine >= 0 && iLine < iCount) {
                            m_rowset.SelectLine(iLine, true, true);
                        }
                    }
                }
                SetDebugInfov();
            }
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDMETHOD, "<Exit>  ResetLinePosV");
        }

        //*********************************************************
        //
        /// <summary>
        /// InsertLine:         Insert a new line
        /// </summary>
        //
        //*********************************************************
        public override void InsertLine() {
            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDMETHOD, "InsertLineV");
            base.InsertLine();
            OnLineSelected(new LineSelectedEventArgs(m_rowset.CurrentLine));
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDMETHOD, "InsertLineV");
        }

        //*********************************************************
        //
        /// <summary>
        /// DuplicateLine:      Duplicate a line
        /// </summary>
        //
        //*********************************************************
        public override void DuplicateCurrentLine() {
            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDMETHOD, "DuplicateLineV");
            base.DuplicateCurrentLine();
            OnLineSelected(new LineSelectedEventArgs(m_rowset.CurrentLine));
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDMETHOD, "DuplicateLineV");
        }

        //*********************************************************
        //
        /// <summary>
        /// DeleteCurrentLine:  Deletes the current line
        /// </summary>
        /// <returns>
        /// true if line is deleted
        /// </returns>
        //
        //*********************************************************
        public override bool DeleteCurrentLine() {
            bool    bRetVal;

            Trace(TraceTypeE.TT_ENTER, TraceCatE.TC_GRIDMETHOD, "DeleteCurrentLineB");
            EndEditV(false);            
            bRetVal = base.DeleteCurrentLine();
            if (bRetVal) {
                OnLineSelected(new LineSelectedEventArgs(m_rowset.CurrentLine));
            }
            Trace(TraceTypeE.TT_EXIT, TraceCatE.TC_GRIDMETHOD, "DeleteCurrentLineB");
            return(bRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// AllowNew:       Determine if new line is allowed
        /// </summary>
        //
        //*********************************************************
        public virtual bool AllowNew {
            get {
                CurrencyManager cur = (CurrencyManager)this.m_dataGrid.BindingContext[this];
                DataView        dview = (DataView)cur.List;
                return(dview.AllowNew);
            }
            set {
                if (!m_bIsAllowNewPermanent) {
                    CurrencyManager cur = (CurrencyManager)this.m_dataGrid.BindingContext[this];
                    DataView        dview = (DataView)cur.List;
                    if (dview.AllowNew != value) {
                        dview.AllowNew = value;
                    }
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// AllowEdit:      Determine if line edition is allowed
        /// </summary>
        //
        //*********************************************************
        public virtual bool AllowEdit {
            get {
                CurrencyManager cur = (CurrencyManager)this.m_dataGrid.BindingContext[this];
                DataView        dview = (DataView)cur.List;
                return(dview.AllowEdit);
            }
            set {
                CurrencyManager cur = (CurrencyManager)this.m_dataGrid.BindingContext[this];
                DataView        dview = (DataView)cur.List;
                if (dview.AllowEdit != value) {
                    dview.AllowEdit = value;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// AllowDelete:    Determine if line deletion is allowed
        /// </summary>
        //
        //*********************************************************
        public virtual bool AllowDelete {
            get {
                CurrencyManager cur = (CurrencyManager)this.m_dataGrid.BindingContext[this];
                DataView        dview = (DataView)cur.List;
                return(dview.AllowDelete);
            }
            set {
                CurrencyManager cur = (CurrencyManager)this.m_dataGrid.BindingContext[this];
                DataView        dview = (DataView)cur.List;
                if (dview.AllowDelete != value) {
                    dview.AllowDelete = value;
                }
            }
        }
    }
}
