using System;
using System.Collections;
using System.Windows.Forms;
using RepPP;
//
// Rep++ Selection Buffer Implementation
// Copyright (C) 2003 Consyst SQL Inc.
// All Rights Reserved
//
// This source code is only intended as a supplement to the
// Rep++ Studio products.
//
// Author:     JFO
//
//
namespace RepPP.Toolkit.Window {
    /// <summary>
    /// The ISelectionBuffer is the interface called to refresh the selection
    /// buffer user interface.
    /// </summary>
    public interface ISelectionBufferUI {
        /// <summary>
        /// Add an item to the selection buffer
        /// </summary>
        /// <param name="rowsetSB">Rowset of the selection buffer</param>
        /// <param name="bDuplicate">true if duplicate the item</param>
        void        AddItemToSelectionBuffer(RepPP.Rowset rowsetSB, bool bDuplicate);
        /// <summary>
        /// Refresh the item in the selection buffer
        /// </summary>
        /// <param name="rowset">Rowset where to take the updated value</param>
        void        RefreshItemInSelectionBuffer(RepPP.Rowset rowset);
        /// <summary>
        /// Delete the item from the selection buffer
        /// </summary>
        /// <param name="bReselect">true to reselect a new item after deletion</param>
        void        DeleteItemFromSelectionBuffer(bool bReselect);
        /// <summary>
        /// Get the number of items if the current kind
        /// </summary>
        /// <returns>Number of items</returns>
        int         GetItemCount();
        /// <summary>
        /// Get the current item index for the current kind
        /// </summary>
        /// <returns>Item index (0..n)</returns>
        int         GetCurrentItem();
        /// <summary>
        /// Select an item in the selection buffer
        /// </summary>
        /// <param name="iPos">Position of the item to select (0..n)</param>
        /// <returns>true if succeed, false if not</returns>
        bool        SelectItem(int iPos);
    }
}