//*********************************************************
//
// NullableDateTimePicker:		    Overload the microsoft .net DateTimePicker to implement 
//                                  datetime picker control which accepts null value
//
// Author:		                    Maher Najah
//
// (c) 2004 by Consyst SQL Inc.
//
//
//*********************************************************
using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace RepPP.Toolkit.Window {
	/// <summary>
	/// SCNullableDateTimePicker: Create a nullable DateTimePicker
	/// </summary>
	public class NullableDateTimePicker : System.Windows.Forms.DateTimePicker {
		private System.ComponentModel.Container components = null;
        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //*********************************************************
		public NullableDateTimePicker()	{
			InitializeComponent();
            ShowCheckBox = true;
            Checked      = false;
		}

        //*********************************************************	    
        //
        /// <summary>
        /// Dispose:                     Dispose the editor
        /// </summary>
        /// <param name="bDisposing">   true if dispose, false if GC </param>
        //
        //*********************************************************	    
        protected override void Dispose(bool bDisposing)
		{
			if(bDisposing) {
                if(components != null) {
                    components.Dispose();
                }
			}
			base.Dispose(bDisposing);
		}

        //*********************************************************
        //
        /// <summary>
        /// IsNull:              Indicates if the control contains a null date
        /// </summary>
        //
        //*********************************************************
		public virtual bool IsNull {
			get {
                return(!Checked);
			}
			set {
                Checked = !value;
			}
		}

        //*********************************************************
        //
        /// <summary>
        /// OnKeyDown:         Occurs when a key is down
        /// </summary>
        /// <param name="e">   Event arguments </param>
        //
        //*********************************************************
		protected override void OnKeyDown(KeyEventArgs e) {
			base.OnKeyDown(e);
			if (e.KeyCode == Keys.Delete)
				IsNull = true; 
		}

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion
	}
}
