using System;
using System.Collections;
using System.Windows.Forms;

namespace RepPP.Toolkit.Window {
    //*********************************************************	    
    //
    /// <summary>
    /// Implement the base class of a tree node attached to a TreeViewRowsetTree 
    /// </summary>
    // 
    //*********************************************************	    
    public class TreeNodeText : TreeNodeBase {
        /// <summary>Array of child group instances attached to this node.</summary>
        private ArrayList                               m_arrChildRowsets;
        /// <summary>Node primary key if user want to specify one</summary>
        private string                                  m_strNodePK = null;

        //*********************************************************	    
        //
        /// <summary>
        /// ctor:                                   Class constructor
        /// </summary>
        /// <param name="treeFather">               Father treeview</param>
        /// <param name="strText">                  Text to associate to the node</param>
        /// <param name="iNodeType">                Type to associate to the node</param>
        /// <param name="iImageIndex">              Index of the image for the unselected node</param>
        /// <param name="iSelectedImageIndex">      Index of the image for the selected node</param>
        /// <param name="arrChildRowsets">          Array of child rowsets</param>
        /// <param name="childNodeFactory">         Node factory used to create child. Can be NULL</param>
        //
        //*********************************************************	    
        public TreeNodeText(TreeViewRowsetTree                      treeFather,
                            string                                  strText,
                            int                                     iNodeType,
                            int                                     iImageIndex,
                            int                                     iSelectedImageIndex,
                            RepPP.Rowset[]                          arrChildRowsets,
                            TreeNodeRowset.IRowsetChildNodeFactory  childNodeFactory) : base(strText, iNodeType, iImageIndex, iSelectedImageIndex, childNodeFactory)
        {
            Text                 = strText;
            m_arrChildRowsets    = new ArrayList();
            if (arrChildRowsets != null) {
                foreach (RepPP.Rowset rowset in arrChildRowsets) {
                    m_arrChildRowsets.Add(rowset);
                }
            }
            Init(treeFather, (arrChildRowsets != null && arrChildRowsets.Length != 0));
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="treeFather">           Father treeview</param>
        /// <param name="strText">              Text of the node or null to get the group instance values</param>
        /// <param name="arrChildRowsets">      Array of child rowsets</param>
        /// <param name="childNodeFactory">     Node factory used to create child. Can be NULL</param>
        //
        //*********************************************************	    
        public TreeNodeText(TreeViewRowsetTree                      treeFather,
                            string                                  strText,
                            RepPP.Rowset[]                          arrChildRowsets,
                            TreeNodeRowset.IRowsetChildNodeFactory  childNodeFactory) : this(treeFather, strText, 0, -1, -1, arrChildRowsets, childNodeFactory)
        {
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Dispose:                Free the node
        /// </summary>
        //
        //*********************************************************	    
        public override void Dispose()
        {
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Clone:                      Clone the node and all its descendant
        /// </summary>
        /// <returns>
        /// Node key
        ///</returns>
        //
        //*********************************************************	    
        public override object Clone() {
            TreeNodeText    nodeRetVal;

            nodeRetVal                   = base.Clone() as TreeNodeText;
            nodeRetVal.m_strNodePK       = m_strNodePK;
            nodeRetVal.m_arrChildRowsets = new ArrayList();
            return(nodeRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// NodePK:                 Current node primary key if any
        /// </summary>
        //
        //*********************************************************	    
        public virtual string NodePK
        {
            get {
                return(m_strNodePK);
            }
            set {
                m_strNodePK = value;
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GetNodeKey:                 Gets the key of the node
        /// </summary>
        /// <returns>
        /// Node key
        ///</returns>
        //
        //*********************************************************	    
        public override string GetNodeKey()
        {
            return((m_strNodePK == null) ? "BASE:" + Text : m_strNodePK);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ChildRowsets:    Array of child rowsets
        /// </summary>
        //
        //*********************************************************	    
        public ArrayList ChildRowsets {
            get {
                return(m_arrChildRowsets);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// FindChildRowset:            Find a child rowset if any
        /// </summary>
        /// <param name="rowsetDef">    Rowset definition</param>
        /// <returns>
        /// null or the specified rowset
        /// </returns>
        //
        //*********************************************************	    
        public override RepPP.Rowset FindChildRowset(RepPP.RowsetDef rowsetDef)
        {
            RepPP.Rowset rowsetRetVal = null;
            
            foreach (RepPP.Rowset rowset in m_arrChildRowsets) {
                if (rowset.RowsetDef == rowsetDef) {
                    rowsetRetVal = rowset;
                }
            }
            return(rowsetRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ReadChildren:           Read node's children
        /// </summary>
        //
        //*********************************************************	    
        public override void ReadChildren()
        {
            TreeViewRowsetTree.BeforeChildReadEventArgs  beforeArg;
            TreeViewRowsetTree.AfterChildReadEventArgs   afterArg;
            
            if (!IsChildrenRead) {
                IsChildrenRead  = true;
                // Remove the 'Dummy' node created by the base class so the '+' appear
                Nodes.Clear();
                beforeArg       = new TreeViewRowsetTree.BeforeChildReadEventArgs(this);
                OnBeforeChildRead(beforeArg);
                if (!beforeArg.Cancel) {
                    foreach (RepPP.Rowset rowset in m_arrChildRowsets) {
                        TreeNodeRowset.FillNodes(TreeView as TreeViewRowsetTree, Nodes, rowset, ChildNodeFactory);
                    }
                    afterArg = new TreeViewRowsetTree.AfterChildReadEventArgs(this);
                    OnAfterChildRead(afterArg);
                }
            }
        }
    }
}
