using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Security.Permissions;
using System.Runtime.InteropServices;
using System.Security;
using RepPP.Toolkit.Window;

namespace RepPP.Toolkit.Window {

    //******************************************************************
    //
    /// <summary>
    /// A Html Reporting preview with priting fonctions
    /// </summary>
    //
    //******************************************************************
    public partial class frmHtmlPreview : Form {

        /// <summary>Html Report URL</summary>
        private String              m_strHtmlReportUrl;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                            Class constructor
        /// </summary>
        //
        //******************************************************************
        public frmHtmlPreview() {
            m_strHtmlReportUrl = String.Empty;
            InitializeComponent();
        }

        //******************************************************************
        //
        /// <summary>
        /// ctor:                            Class constructor
        /// </summary>
        /// <param name="app">Rep++ Object Application</param>
        /// <param name="strHtmlReportUrl">Html Report URL</param>
        //
        //******************************************************************
        public frmHtmlPreview(RepPP.Application app, String strHtmlReportUrl) {
            m_strHtmlReportUrl = strHtmlReportUrl;
            InitializeComponent();
        }

        //******************************************************************
        //
        /// <summary>
        /// btnPrint_Click:         Occurs when button is clicked
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Arguments</param>
        //
        //******************************************************************
        private void btnPrint_Click(object sender, EventArgs e) {
            wbReport.ShowPrintDialog();
        }

        //******************************************************************
        //
        /// <summary>
        /// btnSetup_Click:         Occurs when button is clicked
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Arguments</param>
        //
        //******************************************************************
        private void btnSetup_Click(object sender, EventArgs e) {            
            wbReport.ShowPageSetupDialog();
        } 

        //******************************************************************
        //
        /// <summary>
        /// frmHtmlPreview_Shown:         
        ///                         Occurs when the form is first shown
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Arguments</param>
        //
        //******************************************************************
        private void frmHtmlPreview_Shown(object sender, EventArgs e) {
            System.Windows.Forms.Application.DoEvents();
            if (!DesignMode && !String.IsNullOrEmpty(m_strHtmlReportUrl)) {
                wbReport.Url = new System.Uri(m_strHtmlReportUrl, System.UriKind.Absolute);   
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// wbReport_DocumentCompleted:         
        ///                         Occurs when the document is fully loaded
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Arguments</param>
        //
        //******************************************************************
        private void wbReport_DocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e) {            
            if (e.Url.IsFile) {
                pnlReport.Enabled = true;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// An extended version of the WebBrowser.net control.
        /// </summary>
        //
        //******************************************************************
        protected class WebBrowserExt : System.Windows.Forms.WebBrowser {

            /// <summary>Native Web Browser object (axWebBrowser ActiveX)</summary>
            private IWebBrowser2            m_axIWebBrowser2;

            //******************************************************************
            //
            /// <summary>
            /// Gets the native Web Browser object (axWebBrowser ActiveX)
            /// </summary>
            ///                                  
            //
            //******************************************************************
            public IWebBrowser2 NativeWebBrowser {
                get {
                    return(m_axIWebBrowser2);
                }
            }

            //******************************************************************
            //
            /// <summary>
            /// AttachInterfaces:           This method supports the .NET Framework 
            ///                             infrastructure and is not intended to be 
            ///                             used directly from your code. 
            ///                             Called by the control when the underlying 
            ///                             ActiveX control is created. 
            /// </summary>
            /// <param name="nativeActiveXObject">native ActiveX create 
            ///                                   by the WebBrowser object</param>
            //
            //******************************************************************
            [PermissionSet(SecurityAction.LinkDemand, Name = "FullTrust")]
            protected override void AttachInterfaces(object nativeActiveXObject) {
                m_axIWebBrowser2 = (IWebBrowser2)nativeActiveXObject;
                base.AttachInterfaces(nativeActiveXObject);
            }

            //******************************************************************
            //
            /// <summary>
            /// DetachInterfaces:           This method supports the .NET Framework 
            ///                             infrastructure and is not intended to be 
            ///                             used directly from your code. 
            ///                             Called by the control when the underlying 
            ///                             ActiveX control is discarded. 
            /// </summary>
            //
            //****************************************************************** 
            [PermissionSet(SecurityAction.LinkDemand, Name = "FullTrust")]
            protected override void DetachInterfaces() {
                m_axIWebBrowser2 = null;
                base.DetachInterfaces();
            }

            //******************************************************************
            //
            /// <summary>
            /// ExecWB:                     Executes a command and returns the status of 
            ///                             the command execution using 
            ///                             the IOleCommandTarget interface.
            /// </summary>
            /// <param name="eCmdID">       Command to execute</param>
            /// <param name="eCmdExecOpt">  Command Option</param>
            /// <param name="pvaIn">        Command Parameters</param>
            /// <param name="pvaOut">       Command Result</param>
            //
            //****************************************************************** 
            public void ExecWB(NativeMethods.OLECmdID eCmdID, NativeMethods.OLECmdExecOpt eCmdExecOpt, ref Object pvaIn, IntPtr pvaOut) {
                m_axIWebBrowser2.ExecWB(eCmdID, eCmdExecOpt, ref pvaIn, pvaOut);
            }

            //******************************************************************
            //
            /// <summary>
            /// QueryStatusWB:              Queries the object for the status of 
            ///                             commands using the 
            ///                             IOleCommandTarget interface.
            /// </summary>
            /// <param name="eCmdID">       Command to execute</param>
            /// <returns>
            /// The status of commands
            /// </returns>
            //
            //****************************************************************** 
            public NativeMethods.OLECmdF QueryStatusWB(NativeMethods.OLECmdID eCmdID) {
                return(m_axIWebBrowser2.QueryStatusWB(eCmdID));
            }

            //******************************************************************
            //
            /// <summary>
            /// Static class containing the enums to be use with 
            /// the native WebBrowser (axWebBrowser)
            /// </summary>
            //
            //****************************************************************** 
            public static class NativeMethods {
                /// <summary>Status of the executed command</summary>
                public enum OLECmdF {
                    /// <summary></summary>
                    OLECMDF_SUPPORTED           = 1,
                    /// <summary></summary>
                    OLECMDF_ENABLED             = 2,
                    /// <summary></summary>
                    OLECMDF_LATCHED             = 4,
                    /// <summary></summary>
                    OLECMDF_NINCHED             = 8,
                    /// <summary></summary>
                    OLECMDF_INVISIBLE           = 0x10,
                    /// <summary></summary>
                    OLECMDF_DEFHIDEONCTXTMENU   = 0x20
                }
                /// <summary>Command to be executed by the Web Browser</summary>
                public enum OLECmdID {
                    /// <summary>SaveAs Command</summary>
                    OLECMDID_SAVEAS              = 4,
                    /// <summary>Print Command</summary>
                    OLECMDID_PRINT               = 6,
                    /// <summary>Print Preview Command</summary>
                    OLECMDID_PRINTPREVIEW        = 7,
                    /// <summary>Page Setup Command</summary>
                    OLECMDID_PAGESETUP           = 8,
                    /// <summary>Properties Command</summary>
                    OLECMDID_PROPERTIES          = 10,
                    /// <summary>Show Script Error Command</summary>
                    OLECMDID_SHOWSCRIPTERROR     = 40
                }
                /// <summary>Option of the Command to be executed</summary>
                public enum OLECmdExecOpt {
                    /// <summary></summary>
                    OLECMDEXECOPT_DODEFAULT      = 0,
                    /// <summary></summary>
                    OLECMDEXECOPT_PROMPTUSER     = 1,
                    /// <summary></summary>
                    OLECMDEXECOPT_DONTPROMPTUSER = 2,
                    /// <summary></summary>
                    OLECMDEXECOPT_SHOWHELP       = 3
                }
            }//End class NativeMethods 

            //******************************************************************
            //
            /// <summary>
            /// Interface of axWebBrowser 
            /// </summary>
            //
            //****************************************************************** 
            [ComImport, SuppressUnmanagedCodeSecurity, TypeLibType(TypeLibTypeFlags.FOleAutomation | (TypeLibTypeFlags.FDual | TypeLibTypeFlags.FHidden)), Guid("D30C1661-CDAF-11d0-8A3E-00C04FC9E26E")]
            public interface IWebBrowser2 {
                /// <summary>Back Action</summary>
                [DispId(100)]               
                void GoBack();
                /// <summary>Forward Action</summary>
                [DispId(0x65)]
                void GoForward();
                /// <summary>Home Action</summary>
                [DispId(0x66)]
                void GoHome();
                /// <summary>Search Action</summary>
                [DispId(0x67)]
                void GoSearch();
                /// <summary>Navigate Action</summary>
                [DispId(0x68)]
                void Navigate([In] string Url, [In] ref object flags, [In] ref object targetFrameName, [In] ref object postData, [In] ref object headers);
                /// <summary>Refresh Action</summary>
                [DispId(-550)]
                void Refresh();
                /// <summary>Refresh Action</summary>
                [DispId(0x69)]
                void Refresh2([In] ref object level);
                /// <summary>Stop navigation Action</summary>
                [DispId(0x6a)]
                void Stop();
                /// <summary>Application Browser object</summary>
                [DispId(200)]
                object Application { [return: MarshalAs(UnmanagedType.IDispatch)] get; }
                /// <summary>Parent containing the browser object</summary>
                [DispId(0xc9)]
                object Parent { [return: MarshalAs(UnmanagedType.IDispatch)] get; }
                /// <summary>Container object</summary>
                [DispId(0xca)]
                object Container { [return: MarshalAs(UnmanagedType.IDispatch)] get; }
                /// <summary>Document (DOM) object</summary>
                [DispId(0xcb)]
                object Document { [return: MarshalAs(UnmanagedType.IDispatch)] get; }
                /// <summary>Top Container object</summary>
                [DispId(0xcc)]
                bool TopLevelContainer { get; }
                /// <summary>Type ?</summary>
                [DispId(0xcd)]
                string Type { get; }
                /// <summary>Left position</summary>
                [DispId(0xce)]
                int Left { get; set; }
                /// <summary>Top position</summary>
                [DispId(0xcf)]
                int Top { get; set; }
                /// <summary>Width position</summary>
                [DispId(0xd0)]
                int Width { get; set; }
                /// <summary>Height position</summary>
                [DispId(0xd1)]
                int Height { get; set; }
                /// <summary>Location title</summary>                
                [DispId(210)]
                string LocationName { get; }
                /// <summary>Location URL</summary>                
                [DispId(0xd3)]
                string LocationURL { get; }
                /// <summary>Indicates if the browser is busy</summary>  
                [DispId(0xd4)]                
                bool Busy { get; }
                /// <summary>Quit the brower</summary>  
                [DispId(300)]                
                void Quit();
                /// <summary>Client to window</summary>  
                [DispId(0x12d)]
                void ClientToWindow(out int pcx, out int pcy);
                /// <summary>Put user property in browser</summary>  
                [DispId(0x12e)]                
                void PutProperty([In] string property, [In] object vtValue);
                /// <summary>Get user property in browser</summary>  
                [DispId(0x12f)]
                object GetProperty([In] string property);
                /// <summary>Browser name object</summary>
                [DispId(0)]
                string Name { get; }
                /// <summary>Handle of the browser object</summary>
                [DispId(-515)]
                int HWND { get; }
                /// <summary>Fullname ?</summary>
                [DispId(400)]
                string FullName { get; }
                /// <summary>Path</summary>
                [DispId(0x191)]                
                string Path { get; }
                /// <summary>Indicates if the browser is visible</summary>  
                [DispId(0x192)]
                bool Visible { get; set; }
                /// <summary>Indicates if the browser status bar is visible</summary>  
                [DispId(0x193)]
                bool StatusBar { get; set; }
                /// <summary>Gets/set the status text</summary>  
                [DispId(0x194)]
                string StatusText { get; set; }
                /// <summary>Toolbar</summary>  
                [DispId(0x195)]
                int ToolBar { get; set; }
                /// <summary>Indicates if the menu bar is visible</summary>  
                [DispId(0x196)]
                bool MenuBar { get; set; }
                /// <summary>Gets/Sets the browser in fullscreen mode</summary>  
                [DispId(0x197)]
                bool FullScreen { get; set; }
                /// <summary>Navigate to URL</summary>  
                [DispId(500)]
                void Navigate2([In] ref object URL, [In] ref object flags, [In] ref object targetFrameName, [In] ref object postData, [In] ref object headers);
                /// <summary>Get the status from the last command executed</summary>  
                [DispId(0x1f5)]                  
                NativeMethods.OLECmdF QueryStatusWB([In] NativeMethods.OLECmdID cmdID);
                /// <summary>Execute a special command</summary>  
                [DispId(0x1f6)]
                void ExecWB([In] NativeMethods.OLECmdID cmdID, [In] NativeMethods.OLECmdExecOpt cmdexecopt, ref object pvaIn, IntPtr pvaOut);
                /// <summary>Show the progress bar in browser</summary>  
                [DispId(0x1f7)]                
                void ShowBrowserBar([In] ref object pvaClsid, [In] ref object pvarShow, [In] ref object pvarSize);
                /// <summary>ndicates if the browser is ready</summary>  
                [DispId(-525)]
                WebBrowserReadyState ReadyState { get; }
                /// <summary>Gets/set the offline mode</summary>  
                [DispId(550)]
                bool Offline { get; set; }
                /// <summary>Gets/set the silent mode</summary>  
                [DispId(0x227)]
                bool Silent { get; set; }
                /// <summary>Gets/set the registration as browser</summary>  
                [DispId(0x228)]
                bool RegisterAsBrowser { get; set; }
                /// <summary>Gets/set the registration as drop target</summary>  
                [DispId(0x229)]
                bool RegisterAsDropTarget { get; set; }
                /// <summary>Gets/set the theater mode</summary>  
                [DispId(0x22a)]
                bool TheaterMode { get; set; }
                /// <summary>Gets/Sets the address bar visibility</summary>  
                [DispId(0x22b)]
                bool AddressBar { get; set; }
                /// <summary>Gets/Sets the browser resizable</summary>  
                [DispId(0x22c)]
                bool Resizable { get; set; }

            }//End Inteface IWebBrowser2 

        }//End class WebBrowserExt         
    }//End class frmHtmlPreview
}//End namespace